#========================================================================================
# package      DE_EPAGES::Address::API::Attributes::Address_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for User.
#========================================================================================
package DE_EPAGES::Address::API::Attributes::Address_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::PerlTools qw (trim);
use DE_EPAGES::Object::API::Table::Country qw (GetInfoCountry);

sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        if( $Name eq 'Country' ) {
            my $CountryID = $Object->get('CountryID');
            $Values{$Name} = defined $CountryID ? GetInfoCountry($CountryID) : undef;
        }
        elsif( $Name eq 'FullName' ) {
            my $FirstName = $Object->get('FirstName');
            my $LastName = $Object->get('LastName');
            my $FullName = '';
            if (defined $FirstName && defined $LastName) {
                $FullName = $FirstName.' '.$LastName;
            }
            elsif (defined $FirstName) {
                $FullName = $FirstName;
            }
            elsif (defined $LastName) {
                $FullName = $LastName;
            }
            $FullName =~ s/^\s+//g;
            $FullName =~ s/\s+$//g;
            $Values{$Name} = $FullName;
        }
        elsif( $Name eq 'StreetOnly' ) {
            $Values{$Name} = (_ExtractHouseNumber( $Object->get('Street') ))[0];
        }
        elsif( $Name eq 'HouseNumber' ) {
            $Values{$Name} = (_ExtractHouseNumber( $Object->get('Street') ))[1];
        }
    }

    return \%Values;
}

#========================================================================================
# function     _ExtractHouseNumber
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($StreetOnly, $HouseNumber) = _ExtractHouseNumber( $Street );
#----------------------------------------------------------------------------------------
# description  Splits the street address into street name and house number using some
#               magic heuristics.
#----------------------------------------------------------------------------------------
# input        $Street | street address | string
# return       $StreetOnly | street name | string
# return       $HouseNumber | house number | string
#========================================================================================
sub _ExtractHouseNumber {
    my( $Street ) = @_;

    $Street //= '';
    return '', '' unless $Street =~ /\S/;

    # remove leading and trailing space and normalize whitespace
    ($Street) = trim( $Street );
    $Street =~ s/\s+/ /g;

    # What does a housenumber look like?
    # 3 ; 3a ; 3 a ; 3-a ; 77 ; 77f ; 77 f ; 77-f ; 111 ; 111b ; 111 b ; 111-b

    # So we start with a number, at a word boundary. We match the number, i.e. a
    # sequence of digits. Just for kicks, also match negative housenumbers. :)
    # And also positive ones, of course.
    # Then, there can be a letter (just one; okay, let's make it two; but no more
    # else we risk matching words like "Weg") directly attached to the number, or
    # separated by space, or punctuation.
    # And we completely forgot housenumber like Herdentorsteinweg 44-45!

    # perldoc: perlre, perluniprops: \p{P} = \p{General_Category: Punctuation}
    my $sp = qr/[\s\p{P}]+/; # whitespace and punctuation

    my $range_indicator = qr/[-+_\/\x{2012}\x{2013}\x{2014}]/x; # range characters

    my $num = qr/
            [+-]?\d+                     # number preceded by +- or boundary
            (?:
              (?:$sp)?
              [a-qs-z]{1,2}              # optionally followed by a letter (but not R which is "rue" in French, hence [a-qs-z])
            )?
            /xi;
    my $houseno = qr/
            $num
            (?:
              \s*
              $range_indicator           # optionally followed by a range (44-45)
              \s*
              $num
            )?
        /xi;

    my $houseno_de = qr/$houseno(?:$sp)?/ix;
    my $houseno_fr = qr/$houseno/ix;

    my( $StreetOnly, $HouseNumber );
    if ( $Street =~ m/^\d/ ) {
        # 23a, boulevard de la misre
        # treat punctuation and space as separators
        ( $HouseNumber, $StreetOnly ) = $Street =~ qr/^($houseno_fr)$sp(.*?)$/x;
    }
    else {
        # Schloallee 23a!
        # treat punctuation and space as part of the number
        ( $StreetOnly, $HouseNumber ) = $Street =~ qr/^(.*?)[\s,;:]+($houseno_de)?\s*$/x;
    }

    # If we didn't extract a street and housenumber it was probably
    # a failure so we go safe and return the input data.
    return $Street, '' unless $StreetOnly and $HouseNumber;

    # Else we trim and return the extracted street and housenumber.
    return trim($StreetOnly), trim($HouseNumber);
}

1;
