#========================================================================================
# package      DE_EPAGES::Address::API::GeoData
# state        public
#----------------------------------------------------------------------------------------
# description  Wrapper for Google Geocoding API.
#               <br><br>
#               Use function <function GetGeoData> to query Google using this function
#               expect an hash with address data and an output filter as arguments. Two
#               output filter are part of this API : <function FilterAll> and <function FilterLocation>
#               <br><br>
#               Example Request constructed by function GetGeoData:
#               http://maps.googleapis.com/maps/api/geocode/json?address=Leutragraben+1+07749+Jena+Germany&sensor=false
#               <br><br>
#               For convenience function <function GetGeoLocation> is provided, which returns Longitude and latitude for given address.
#               It uses internaly output filter <function FilterLocation> to achieve this.
#               <br>
#               Word of caution the number of calls are limited 2500 per 24h. Repeatedly exceeding the limit
#               may lead to temporary blocking or banning your IP.
#               <br>
#               <ul>
#                 <li>Limitations: https://developers.google.com/maps/documentation/geocoding/?hl=en#Limits</li>
#                 <li> Dev-Doc:    https://developers.google.com/maps/documentation/geocoding/?hl=en</li>
#               </ul>
#----------------------------------------------------------------------------------------
# example
#               # return latitude and longitude in hash
#               my $hGeoLocation = GetGeoLocation($hAddress)
#----------------------------------------------------------------------------------------
# example
#
#               # return custom data from Google's response
#               sub FilterLocation ($) {
#                   my ($Response) = @_;
#                   TestObject( 'Response', $Response, 'HTTP::Response' );
#                   my $hResult   = from_json( $Response->decoded_content );
#                   my $Status    = $hResult->{status};
#                   my $hLocation = undef;
#
#                   if ( $Status eq 'OK' || scalar $hResult->{results} == 1 ) {
#                   ($hLocation) = map { $_->{geometry}{location} } @{ $hResult->{results} };
#                   }
#                   return $hLocation;
#               }
#
#               my $hGeoLocation = GetGeoData($hAddress, \&FilterLocation);
#
#========================================================================================
package DE_EPAGES::Address::API::GeoData;
use parent qw ( Exporter );
use strict;

use DE_EPAGES::Core::API::Error         qw ( Error ExistsError GetError);
use DE_EPAGES::Core::API::Log           qw ( GetLog );
use DE_EPAGES::Core::API::Url           qw ( UrlEncodeValue );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestObject );

use JSON qw ( from_json );
use HTTP::Request::Common qw ( GET );
use HTTP::Status;
use LWP::UserAgent;

use constant TIMEOUT_IN_SEC    => 10;
use constant HAS_GPS_SENSOR    => 'sensor=false';
use constant RESPONSE_FORMAT   => 'json';
use constant GEOCODE_REQUEST   => 'http://maps.googleapis.com/maps/api/geocode/' . RESPONSE_FORMAT;
use constant GOOGLE_DOC_STATUS => 'https://developers.google.com/maps/documentation/geocoding/?hl=en#StatusCode';

our @EXPORT_OK = qw (
    BuildAddress
    GetGeoData
    GetGeoLocation
    FilterAll
    FilterLocation
);

sub _GetUserAgent {
    my $UA = LWP::UserAgent->new();
    $UA->timeout(TIMEOUT_IN_SEC);
    return $UA;
}

#========================================================================================
# function     FilterLocation
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hLocation = FilterLocation( $Response );
#----------------------------------------------------------------------------------------
# description  Output filter for <function GetGeoData>. Extract location information from
#               HTTP::Response of Google Geocoding API. This function assumes only one
#               location is in the result.
#----------------------------------------------------------------------------------------
# input        $Response | google response | HTTP::Response
# return       $hLocation | contain key lat, lng | hash.ref. or undef
#========================================================================================
sub FilterLocation ($) {
    my ($Response) = @_;
    TestObject( 'Response', $Response, 'HTTP::Response' );
    my $hResult   = from_json( $Response->decoded_content );
    my $Status    = $hResult->{status};
    my $hLocation = undef;

    if ( $Status eq 'OK' && scalar @{$hResult->{results}} == 1 ) {
        ($hLocation) = map { $_->{geometry}{location} } @{ $hResult->{results} };
    }
    return $hLocation;
}

#========================================================================================
# function     FilterAll
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hLocation = FilterAll( $Response );
#----------------------------------------------------------------------------------------
# description  Output filter for <function GetGeoData>. Return data as hash. ref. from
#               HTTP::Response of Google Geocoding API.<br>
#----------------------------------------------------------------------------------------
# input        $Response | goole response | HTTP::Response
# return       $hLocation | contain key lat, lng | hash.ref. or undef
#========================================================================================
sub FilterAll ($) {
    my ($Response) = @_;
    TestObject( 'Response', $Response, 'HTTP::Response' );

    return from_json( $Response->decoded_content );
}

#========================================================================================
# function     GetGeoData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCustomData = GetGeoData( $hAddress, $cOutputFilter );
#----------------------------------------------------------------------------------------
# description  Query Google Geo Coding API with $hAddress as address parameters.
#
#               Google's response will be validated by checking decoded body, the status key
#               must be 'OK' or 'ZERO_RESULTS' indicating that the request was correctly processed,
#               otherwise an error will be raised. As final step $cOutputFilter is executed to
#               to extract the components of interest. <br>
#               <br>
#               Example request:
#               http://maps.googleapis.com/maps/api/geocode/json?address=Leutragraben+1+07743+Jena+Germany&sensor=false
#               <br>
#               <br>
#               List of errors:
#               <ul>
#                   <li>OVER_QUERY_LIMIT - Google 24h query limit is exceeded</li>
#                   <li>REQUEST_DENIED -  Google is blocking the request.</li>
#                   <li>INVALID_REQUEST - ePages request is not valid</li>
#                   <li>OTHER - Google response contains undocumented status X </li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        $hAddress | Following address contains following keys:
#                       <ul>
#                           <li>Street - Leutragraben 1 </li>
#                           <li>Zipcode - 07743</li>
#                           <li>City - Jena</li>
#                           <li>Country -  Germany (english country name)</li>
#                       </ul>
#               | hash.ref.
#
# input        $cOutputFilter | sub to handle Google's response; input is HTTP::Response
#                                with JSON structure as body, output must be a scalar value | sub
# return       $hLocation | output depend on $cCustomFilter | any
#========================================================================================
sub GetGeoData ($$) {
    my ( $hAddress, $cOutputFilter ) = @_;
    TestHash( 'hAddress', $hAddress );

    # http://maps.googleapis.com/maps/api/geocode/json?address=Leutragraben+1+07749+Jena&sensor=false
    my $Get = GEOCODE_REQUEST . '?' . BuildAddress($hAddress) . '&' . HAS_GPS_SENSOR;
    my $Request  = GET( $Get );
    my $Response = _GetUserAgent()->request($Request);

    GetLog->info(sprintf ('GET %s %s %i', $Get, $Response->code, length $Response->decoded_content) );
    GetLog(__PACKAGE__ . '::GetGeoData::Response')->debug($Response->as_string);

    my $FilterResult = undef;
    $FilterResult = _HandleResponse( $Response, $cOutputFilter );

    return $FilterResult;
}

#========================================================================================
# function     GetGeoLocation
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetGeoLocation( $hAddress );
#----------------------------------------------------------------------------------------
# description  Convenience  function wrapping <function GetGeoData>.
#               Return longitude (lng) and latitude (lat) for given address.
#               Uses output filter <function FilterLocation>
#----------------------------------------------------------------------------------------
# input        $hAddress | address like <function GetGeoData> | ref.hash.
# return       $ReturnVariable | hash with keys lng & lat | ref.hash.
#========================================================================================
sub GetGeoLocation($) {
    my ($hAddress) = @_;
    return GetGeoData( $hAddress, \&FilterLocation );
}

#========================================================================================
# function     BuildAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       BuildAddress( $hAddr );
#----------------------------------------------------------------------------------------
# description  Build up google geo location request param string.
#----------------------------------------------------------------------------------------
# input        $hAddr | address like <function GetGeoData> | ref.hash.
# return       $ddress | address string | string
#========================================================================================
sub BuildAddress($) {
    my ($hAddr) = @_;

    my $Address = 'address=';
    eval {
        # address=Leutragraben+1+07749+Jena+Germany
        $Address .=  UrlEncodeValue( join ' ', map { $hAddr->{$_} } qw (Street Zipcode City Country) );
    };
    if(ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ( $Error->code eq 'STRING_EMPTY' );
    }

    return $Address;
}

#========================================================================================
# function     _HandleResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _HandleResponse( $Response, $cFilter );
#----------------------------------------------------------------------------------------
# description  Check status from Google Geocoding API response and extract specific data using $cfilter<br>
#               See https://developers.google.com/maps/documentation/geocoding/?hl=en#StatusCodes
#----------------------------------------------------------------------------------------
# input        $Response | HTTP::Response object containing answer from Google Geocoding API | HTTP::Response
# input        $cFilter  | output filter like <function FilterLocation> | HTTP::Response
# return       $ReturnVariable | result depend on $cFilter; default filter is <function _FilterLocation> | any
#========================================================================================
sub _HandleResponse($$) {
    my ( $Response, $cFilter ) = @_;
    Error( "HTTP Request failed\n" . $Response->as_string ) unless $Response->is_success;

    my $hResult = from_json( $Response->decoded_content );
    my $Status = $hResult->{status};
    GetLog->debug( "Status: $Status" );

    if ( $Status eq 'ZERO_RESULTS' ) { return $cFilter->($Response) }
    elsif ( $Status eq 'OK' )        { return $cFilter->($Response) }

    my $Error;
    if ( $Status eq 'OVER_QUERY_LIMIT' ) {
        $Error = "Google response status '%s'. You exceeded 24h query limit\nURI: %s\nResponse\n%s";
    }
    elsif ( $Status eq 'REQUEST_DENIED' ) {
        $Error = "Google response status '%s'. Google is blocking you. You may have exceeded 24h query limit\nURI: %s\nResponse:\n%s";
    }
    elsif ( $Status eq 'INVALID_REQUEST' ) {
        $Error = "Google response status '%s'. ePages request syntax contains an error\n%sURI: %s";
    }
    else {
        $Error = "Google response contains undocumented status '%s'. Did Google Geo Coding API change ?\nURI %s\nResponse:\n%s";
    }
    Error(sprintf($Error, $Status, $Response->base->as_string, $Response->as_string));
}

1;
