#========================================================================================
# package      -
# state        public
#----------------------------------------------------------------------------------------
# description  Script to interactivly create new cartridges
#========================================================================================

use strict;
use Config;
use Getopt::Long;
use File::Spec;
use Term::ReadKey;
#use DE_EPAGES::Cartridge::API::Cartridge;
use DE_EPAGES::Cartridge::API::CartridgeCreator;
use DE_EPAGES::Core::API::Constants;
use DE_EPAGES::Core::API::File                   qw ( WriteFile GetFileContent );
use DE_EPAGES::Core::API::PerlTools              qw ( ElementOf );
use DE_EPAGES::Core::API::Script                 qw ( RunScript );
use DE_EPAGES::TLE::API::TLE                     qw ( GetByteCodeByText ReplaceTLEVars );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Cartridge;
use Data::Dumper;

##no critic qw(ProhibitCascadingIfElse)

sub Main {
    local $| = 1;
    my ($Help, $Interactive, $Template, $Verbose, $Debug, $ConfigFile, $BasePackage) =
        (0, 0, 'Basic', 0, 0, undef, 'DE_EPAGES::Cartridge');
    GetOptions(
        'help'              => \$Help,
        'i'                 => \$Interactive,
        'verbose'           => \$Verbose,
        'debug'             => \$Debug,
        'template=s'        => \$Template,
        'basepackage=s'     => \$BasePackage,
        'configfile=s'      => \$ConfigFile,
    );
    if ("\U$Template" eq 'BASIC')   { $Template = 'Basic';   }
    if ("\U$Template" eq 'EXAMPLE') { $Template = 'Example'; }
    usage() if $Help;
    if( (! (scalar @ARGV)) && (!$Interactive) ) {
        print STDERR "\nNo package name specified.\n\n"; usage();
    }
    my $Package = (scalar @ARGV) ? $ARGV[0] : 'Default::Example';
    my $hOptions = setCartridgeDefaults ( $Package, $Verbose, $Debug, $ConfigFile,$BasePackage );
    my $aFilesWritten = $Interactive ? UserMenu($hOptions) : SimpleTemplate($Template,$BasePackage,$hOptions);
    if (defined $aFilesWritten) {
        print headline('Creating Cartridge '.$hOptions->{'CartridgePackage'});
        print underline('Copying files '.($Template? "(Template: $Template)" :''))."\n" unless $hOptions->{'_NOOUTPUT'};
        if (not $hOptions->{'_NOOUTPUT'}) {
            foreach my $FileName ( @{$aFilesWritten} ) {
                print File::Spec->canonpath($FileName)." written.\n";
            }
        }
        print underline('To install the cartridge do the following steps:');
        if ($Config{osname} ne 'MSWin32') {
            my $EpagesProject = $ENV{EPAGES_PROJECT};
            $EpagesProject = 'epages5' unless $EpagesProject =~ /\S/;
            my $EpagesDefault = "/etc/default/$EpagesProject";
            print "\ncd ",
                File::Spec->catfile(
                    $ENV{EPAGES_CARTRIDGES},
                    $hOptions->{'Company'},
                    $hOptions->{'Cartridge'}
                ),
                "\n";
            print ". $EpagesDefault";
            print "\$PERL Makefile.PL\n";
            print "make install STORE=Store\n";
        } else {
            print "\ncd /D ",
                File::Spec->catfile(
                    $ENV{EPAGES_CARTRIDGES},
                    $hOptions->{'Company'},
                    $hOptions->{'Cartridge'}
                ),
                "\n";
            print "perl Makefile.PL\n";
            print "nmake install STORE=Store\n";
        }
    } else {
        print "\n\nHave a nice day!\n";
    }
}
#========================================================================================
# Helper functions                                                                      #
#========================================================================================
sub getInput {
    my ($Head, $Default, $ShowDefault) = @_;

    if (not defined $ShowDefault) {
        $ShowDefault = 1;
    }
    ReadMode 1;
    if (defined $Default && $ShowDefault) {
        print "$Head. ($Default): ";
    }
    else {
        print "$Head. : ";
    }
    my $Line = ReadLine(0);
    ReadMode 0;
    if (ord($Line) == 10) {  ## no critic qw(ProhibitMagicNumbers)
        return $Default;
    }
    else {
        # Remove NewLine character at end
        return substr ( $Line,0, length($Line)-1 );
    }
}

sub isAliasUsed {
# checks if: in an rArray of rHash lists already exists a list that has the Alias
# of $AliasValue in Element {$HashKey}
  my ( $AliasValue, $Hashkey, @List ) = @_;
    my $AliasExists = 0;
    foreach my $hElement ( @List ) {
        if ($hElement->{$Hashkey} eq $AliasValue) { $AliasExists=1; }
    }
    if ($AliasExists) { print "\n$AliasValue already exists!\n\n"; }
    return $AliasExists;
}

sub createKeyword {
    my ( $Name ) = @_;
    $Name =~ s/\W//g;
    return $Name;
}

sub getKeyYesNo {
    my ($PrintKey) = @_;
    return getMappedKey( { 'y' => 'Yes', 'n' => 'No' }, $PrintKey );
}

sub getMappedKey {
    my ($KeyOutputMap, $hOptions) = @_;
    $hOptions->{'PrintKey'} = 1 unless (defined $hOptions->{'PrintKey'});
    $hOptions->{'AllowAll'} = 0 unless (defined $hOptions->{'AllowAll'});
    my @AllowedKeys = keys %{$KeyOutputMap};
    my $key = '';
    if ( scalar @AllowedKeys > 0 ) {
        do  {
            $key = getKey();
            # if all keys are allowed return even if key is not in keymap
            if( ($hOptions->{'AllowAll'}) && (!ElementOf($key, @AllowedKeys)) ) { return 'bad key'; }
        } until ( ElementOf($key, @AllowedKeys) );
    } else {

        $key = getKey();
    }
    if (defined $KeyOutputMap->{$key} ) {
       $key = $KeyOutputMap->{$key};
    }
    if ($hOptions->{'PrintKey'}) {
        print $key;
    }
    return $key
}

sub getKey {
    ReadMode 4;
    my $key = '';
    $key = ReadKey(0);
    ReadMode 0;
    return $key;
}

sub ReplaceTLEs {
    my ($sText, $hVars) = @_;
    my $rByteCode =  GetByteCodeByText( $sText );
    return ReplaceTLEVars( $rByteCode, $hVars );
}

sub headline {
    my ($header, $symbol) = @_;
    if (not defined $symbol) { $symbol = '*'; }
    my $len = length($header);
    my $line = '';
    foreach(1 .. $len + 4) {
        $line .= $symbol;
    }
    return "\n\n $line\n $symbol $header $symbol\n $line\n";
}

sub underline {
    my ($header, $symbol) = @_;
    if (not defined $symbol) { $symbol = '-'; }
    my $len = length($header);
    my $line = '';
    foreach(1 .. $len + 2) {
         $line .= $symbol;
    }
    return "\n $header \n$line\n";
}

sub backslashes2slashes {
    my ($text) = @_;
    $text =~ s/\\/\//g;
    return $text;
}
#========================================================================================
# Init functions                                                                        #
#========================================================================================
sub setCartridgeDefaults {
    my ($Package, $Verbose, $Debug, $ConfigFile, $BasePackage) = @_;
    my ($Company, $Cartridge) = split /::/, $Package, 2;
    if (not defined $Cartridge) { print "\nInvalid package name specified!\n\n"; usage(); }

    my $Version = DE_EPAGES::Core::API::Cartridge->new()->version();

    my $hInfo = {
        'Company'              => $Company,
        'Cartridge'            => $Cartridge,
        'CartridgeDirectory'   => $Company.'/'.$Cartridge,
        'OSCartridgeDirectory' => File::Spec->catfile($ENV{EPAGES_CARTRIDGES},$Company,$Cartridge),
        'CartridgePackage'   => $Package,
        'CartridgeVersion'   => $Version,
        'AllDependencies'    => [
            { 'Package' => 'DE_EPAGES::Cartridge', 'Comment' => 'default dependency' },
            { 'Package' => 'DE_EPAGES::Design',    'Comment' => 'default dependency' }
        ],
        'Purpose'            => 'Give a short summary of what your cartridge is capable of.',
        'CreateTranslationFiles' => 1,
        'Concepts'           => 'It follows a listing and explanation of key functions and concepts of your cartridge.',
        'Model'              => 'Models/'.$Cartridge.'.pdm',
        '_DEBUG'             => $Debug,
        '_NOOUTPUT'          => !($Verbose),
        'Hash'               => '#'
    };

    # add defaults in config.ini
    my ($BaseCompany, $BaseCartridge) = split /::/, $BasePackage, 2;
    my $DefaultConfigFile = File::Spec->catfile(
        $ENV{EPAGES_CARTRIDGES},
        $BaseCompany,
        $BaseCartridge,
        'CartridgeTemplates',
        'config.ini'
    );
    _AddIniSettings($DefaultConfigFile, $hInfo) if (-r $DefaultConfigFile);

    # add defaults in customer ini
    if (defined $ConfigFile ) {
        if (not -e $ConfigFile) {
            print STDERR 'ERROR: No such file: '.$ConfigFile. "\n";
            exit 1;
        }
        if (not  -r $ConfigFile) {
            print STDERR 'ERROR: File not readable: '.$ConfigFile."\n";
            exit 1;
        }
        _AddIniSettings($ConfigFile, $hInfo);
    }

    return $hInfo;
}

sub setMenuDefaults {
    return {
        'MainMenu' => {
            'View' => \(
                headline('CartridgeCreator').
                "\n n - Update cartridge name & version".
                "\n d - Add or remove cartridge dependencies".
                "\n a - Add an user-defined attribute".
                "\n h - Alter cartridge documentation".
                "\n p - Add and remove page types".
                "\n v - View cartridge information".
                "\n w - Create New Cartridge And Write All Template Files To Disk".
                "\n\n x - Exit\n\nYour Choice: "
            ),
            'KeyMap' => {
                'x' => 'exit',
                'd' => 'dependency',
                'n' => 'name',
                'v' => 'view',
                'w' => 'write',
                'h' => 'doc',
                'p' => 'pagetype',
                'a' => 'attribute'
            },
        },
        'BasicInfo' => {
            'View' => \(
                "\n".
                underline('Basic cartridge information').
                "\nCompany            : #Company".
                "\nCartridge          : #Cartridge".
                "\nPackage            : #CartridgePackage".
                "\nCartridge directory: #OSCartridgeDirectory".
                "\n\n"
            ),
            'KeyMap' => undef,
        },
        'DependencyInfo' => {
            'View' => \(
                underline('Cartridge dependencies').
                "\n#LOOP(#AllDependencies)".
                "#IF(#Package)#Package#ENDIF#IF(#Comment) (#Comment)#ENDIF\n".
                "#ENDLOOP\n"
            ),
            'KeyMap' => undef,
        },
        'DocInfo' => {
            'View' => \(
                underline('Cartridge documentation').
                "\nPurpose:\n\n".
                '#Purpose'.
                "\n\nConcepts:\n\n".
                "#Concepts\n\n"
            ),
            'KeyMap' => undef,
        },
        'UpdateDoc' => {
            'View' => \(
                headline('Cartridge documentation').
                "\nPurpose:\n\n".
                '#Purpose'.
                "\n\nConcepts:\n\n".
                "#Concepts\n\n".
                "\n p - Alter cartridge purpose".
                "\n c - Alter cartridge concepts".
                "\n x - Return to main menu".
                "\n\nYour Choice: "
            ),
            'KeyMap' => {
                'p' => 'purpose',
                'c' => 'concepts',
                'x' => 'exit'
            },
        },
        'UpdateDependency' => {
            'View' => \(
                headline('Cartridge dependencies').
                "\n#LOOP(#AllDependencies)".
                "#IF(#Package)#Package#ENDIF#IF(#Comment) (#Comment)#ENDIF\n".
                "#ENDLOOP\n".
                "\n a - Add a new cartridge dependency".
                "\n r - Remove a cartridge dependency".
                "\n x - Return to main menu".
                "\n\nYour Choice: "
            ),
            'KeyMap' => {
                'a' => 'add', 'r' => 'remove', 'x' => 'exit', 'q' => 'quit',
            },
        },
        'CreatePageType' => {
            'View' => \(
                headline('Page-types').
                "\n s - Add a new storefront page-type".
                "\n m - Add a new merchant backoffice page-type".
                "\n r - Remove a page-type".
                "\n v - View all your page-types".
                "\n\n x - Return to Main Menu".
                "\n\nYour Choice: "
            ),
            'KeyMap' => {
                's' => 'sf',
                'm' => 'mbo',
                'r' => 'remove',
                'v' => 'view',
                'x' => 'exit'
            },
        },
        'AddMBOPageChooseSubmenu' => {
            'View' => \(
                'In which context should your new page type appear?'.
                "\n o - Orders".
                "\n c - Customer".
                "\n p - Products".
                "\n t - Categories".
                "\n d - Design/Styles".
                "\n m - Marketing".
                "\n s - Settings".
                "\nYour Choice: "
            ),
            'KeyMap' => {
                'o' => 'MBO-OrderManager',
                'c' => 'MBO-CustomerManager',
                'p' => 'MBO-ProductManager',
                't' => 'MBO-ContentManager',
                'd' => 'MBO-Styles',
                'm' => 'MBO-Marketing',
                's' => 'MBO-Setup'
            },
        },
        'viewPageTypes' => {
            'View' => \(
                "\n".underline('Storefront page-types:').
                '#IF(#COUNT(#AllSFPages))'.
                '#LOOP(#AllSFPages)'.
                "\n#PageTypeAlias (Base: #BasePageType)".
                '#ENDLOOP'.
                "#ELSE\nnone defined.#ENDIF".
                "\n\n".underline('Merchant backoffice page-types:').
                '#IF(#COUNT(#AllMBOPages))'.
                '#LOOP(#AllMBOPages)'.
                "\n#PageTypeAlias (Base: #PageTpeContext)".
                '#ENDLOOP'.
                "#ELSE\nnone defined.#ENDIF\n"
            ),
            'KeyMap' => {
                'o' => 'Orders',
                'c' => 'Customers',
                'p' => 'Products',
                'c' => 'Categories',
                'd' => 'Design',
                'm' => 'Marketing'
            },
        },
        'CreateAttribute' => {
            'View' => \(
                headline('Attributes').
                "\n a - Add Attribute".
                "\n r - Remove Attribute".
                "\n v - View Attributes".
                "\n\n x - Return to Main Menu".
                "\n\nYour Choice: "
            ),
            'KeyMap' => {
                'a' => 'add',
                'r' => 'remove',
                'v' => 'view',
                'x' => 'exit',
            },
        },
         'ViewAttributes' => {
            'View' => \(
                "\n".underline('Attribute list').
                '#IF(#COUNT(#AllAttributes))'.
                '#LOOP(#AllAttributes)'.
                "\n  Alias       : #AttributeAlias\n".
                "  Name        : #AttributeName\n".
                "  Description : #AttributeDescription\n".
                "  Type        : #AttributeType\n".
                "  Package     : #AttributePackage\n".
                "#ENDLOOP#ELSE\nnone defined.\n#ENDIF"
            ),
         },
    };
}
#========================================================================================
# Basic Cartridge Info                                                                  #
#========================================================================================
sub Menu_ViewInfo {
    my ($hOptions, $Menu) = @_;
    print ReplaceTLEs($Menu->{'BasicInfo'}->{'View'}, $hOptions );
    print ReplaceTLEs($Menu->{'DependencyInfo'}->{'View'}, $hOptions );
    print ReplaceTLEs($Menu->{'DocInfo'}->{'View'}, $hOptions );
    return '';
}

sub Menu_UpdateBasic {
    my ($hOptions, $MenuItem) = @_;
    print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
    $hOptions->{'Company'} = getInput('Company',$hOptions->{'Company'});
    $hOptions->{'Cartridge'} = getInput('Cartridge',$hOptions->{'Cartridge'});
    # $hOptions->{'CartridgeVersion'} = getInput('CartridgeVersion',$hOptions->{'CartridgeVersion'});
    # $hOptions->{'Model'} = backslashes2slashes(getInput('Database model',$hOptions->{'Model'}));
    $hOptions->{'CartridgePackage'} = $hOptions->{'Company'}.'::'.$hOptions->{'Cartridge'};
    $hOptions->{'CartridgeDirectory'} = $hOptions->{'Company'}.'/'.$hOptions->{'Cartridge'};
    $hOptions->{'OSCartridgeDirectory'} = File::Spec->catfile(
        $ENV{EPAGES_CARTRIDGES},
        $hOptions->{'Company'},
        $hOptions->{'Cartridge'}
    );
    return '';
}

sub Menu_UpdateDoc {
    my ($hOptions, $MenuItem) = @_;
    while (1) {
        print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
        my $key = getMappedKey($MenuItem->{'KeyMap'}, { 'AllowAll' => 1 });
        print "\n\n";
        if ($key eq 'purpose') { $hOptions->{'Purpose'} = getInput('Purpose',$hOptions->{'Purpose'}, 0); }
        elsif ($key eq 'concepts') { $hOptions->{'Concepts'} = getInput('Concepts',$hOptions->{'Concepts'}, 0); }
        elsif ($key eq 'exit') { return; }
        else {print "\n\nBad input. Try again. ('x' - exit)\n" };
    }
    return '';
}

sub Menu_UpdateDependency {
    my ($hOptions, $MenuItem) = @_;
    while (1) {
        print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
        my $key = getMappedKey($MenuItem->{'KeyMap'}, { 'AllowAll' => 1 } ) ;
        if ($key eq 'add') { addDependency($hOptions); }
        elsif ($key eq 'remove') { removeDependency($hOptions); }
        elsif ($key eq 'exit') { return; }
        else {print "\n\nBad input. Try again. ('x' - exit)\n" };
    }
}

sub addDependency {
    my ($hOptions) = @_;
    print "\n\n";
    my $Package = getInput('Package name','DE_EPAGES::Cartridge');
    my $Comment = getInput('Comment');
    # check if it is already in the dependencies
    my $countDependencies = scalar @{ $hOptions->{'AllDependencies'} };
    my $foundDependency = 0;
    # if Entry exists, just alter the comment
    if ($countDependencies >0) {
        for (my $i = 0; $i < $countDependencies; $i++) {
            if ($hOptions->{'AllDependencies'}[$i]{'Package'} eq $Package) {
                $hOptions->{'AllDependencies'}->[$i]->{'Comment'} = $Comment;
                $foundDependency = 1;
            }
        }
    }
    # else add it to the array
    if (!$foundDependency) {
        push @{$hOptions->{'AllDependencies'}}, {
            'Package' => $Package,
            'Comment' => $Comment
        };
    }
}

sub removeDependency {
    my ($hOptions) = @_;
    print "\n\n";
    my $count = defined $hOptions->{'AllDependencies'} ? scalar @{ $hOptions->{'AllDependencies'} } : 0;
    my $default = $count ? $hOptions->{'AllDependencies'}->[$count-1]->{'Package'} : '';
    my $Package = getInput('Remove which Package? ',$default);
    my $found = 0;
    if ($count >0) {
        for (my $i = 0; $i < $count; $i++) {
            if ($hOptions->{'AllDependencies'}[$i]{'Package'} eq $Package) {
                splice @{$hOptions->{'AllDependencies'}},$i,1;
                $found = 1;
                last;
            }
        }
    }
    if (!$found) {
        print "\nDependency not found.\n";
    }
}
#========================================================================================
# Page types                                                                            #
#========================================================================================
sub addSFPageType {
    my ($hOptions) = @_;
    print "\n".underline('Add storefront page-type')."\n";
    my $InputValues = {};
    $InputValues->{'BasePageType'} = getInput('BasePageType','SF');
    my @AllPages = ();
    if (defined $hOptions->{'AllMBOPages'}) { push @AllPages, @{$hOptions->{'AllMBOPages'}}; }
    if (defined $hOptions->{'AllSFPages'}) { push @AllPages, @{$hOptions->{'AllSFPages'}}; }
    do {
            $InputValues->{'PageTypeAlias'} = getInput('Alias for the new PageType', 'SF-Example');
    } while ( isAliasUsed($InputValues->{'PageTypeAlias'}, 'PageTypeAlias', @AllPages ) );
    $InputValues->{'PageTypeAliasKeyword'} = createKeyword( $InputValues->{'PageTypeAlias'} );
    ## no critic qw(ProhibitLongLines);
    $InputValues->{'PageTypeViewAction'}  = getInput('Alias of the view action  for this PageType', 'View'.$InputValues->{'PageTypeAlias'});
    $InputValues->{'ContentFileName'}     = backslashes2slashes(getInput('Filename for content template', File::Spec->catfile('SF',$InputValues->{'PageTypeAlias'}.'.Content.html')));
    $InputValues->{'NavElementAlias'}     = getInput('Alias of the corresponding navigation element', $InputValues->{'PageTypeAlias'}.'-Link');
    $InputValues->{'NavElementAliasKeyword'} = createKeyword( $InputValues->{'NavElementAlias'} );
    $InputValues->{'NavElementDesc'}      = getInput('Short description of this navigation element', "Link to $InputValues->{'BasePageType'}");
    $InputValues->{'NavElementText'}      = getInput('Text on this clickable Navigation Element', "View $InputValues->{'BasePageType'}");
    $InputValues->{'NavElementFileName'}  = backslashes2slashes(getInput('Filename of navigation element template',
        File::Spec->catfile($InputValues->{'BasePageType'},'NavElements',$InputValues->{'BasePageType'}.'.'.$InputValues->{'NavElementAlias'}.'.html')));
    ## use critic
    print 'Do you really want to add this page-type? (y/n) ';
    if ( getKeyYesNo() eq 'Yes') {
        push @{$hOptions->{'AllSFPages'}}, $InputValues;
        print "\nPage type added.\n";
    }
}

sub addMBOPagetype {
    my ($hOptions, $Menu) = @_;
    print "\n".underline('Add merchant backoffice page-type')."\n";
    my $InputValues = {};
    print ReplaceTLEs($Menu->{'AddMBOPageChooseSubmenu'}->{'View'});
    $InputValues->{'PageTypeContext'} = getMappedKey($Menu->{'AddMBOPageChooseSubmenu'}->{'KeyMap'});
    print "\n";
    my @AllPages = ();
    if (defined $hOptions->{'AllMBOPages'}) { push @AllPages, @{$hOptions->{'AllMBOPages'}}; }
    if (defined $hOptions->{'AllSFPages'}) { push @AllPages, @{$hOptions->{'AllSFPages'}}; }
    do {
            $InputValues->{'PageTypeAlias'} = getInput('Alias for the new PageType', 'MBO-Example');
    } while ( isAliasUsed($InputValues->{'PageTypeAlias'}, 'PageTypeAlias', @AllPages ) );
    $InputValues->{'PageTypeAliasKeyword'} = createKeyword( $InputValues->{'PageTypeAlias'} );
    ## no critic qw(ProhibitLongLines ProhibitMagicNumbers);
    $InputValues->{'PageTypeViewAction'}  = getInput('Alias of the view action  for this page-type', 'View'.$InputValues->{'PageTypeAlias'});
    $InputValues->{'ContentFileName'} = backslashes2slashes(getInput('Filename for content template', File::Spec->catfile('MBO',$InputValues->{'PageTypeAlias'}.'.Content.html')));
    $InputValues->{'TabPageFileName'} = backslashes2slashes(getInput('Filename for tab-page template', File::Spec->catfile('MBO',$InputValues->{'PageTypeAlias'}.'.TabPage.html')));
    $InputValues->{'TabPageSaveActionAlias'} = getInput('Alias of the save action for this tab-page', 'Save');
    $InputValues->{'MBOContextMenuTemplateFileName'}  = backslashes2slashes(getInput('Filename for the context menu',File::Spec->catfile('MBO','MBO.Link-'.$InputValues->{'PageTypeAlias'}.'.html')));
    $InputValues->{'MBOContextActiveMenuTemplateFileName'}  = substr($InputValues->{'MBOContextMenuTemplateFileName'},0,(length $InputValues->{'MBOMarketingMenuTemplateFileName'})-4).'Active.html';
    if ($InputValues->{'PageTypeContext'} eq 'MBO-Marketing') {
        $InputValues->{'MBOMarketingMenuTemplateFileName'}  = backslashes2slashes(getInput('Filename for the marketing menu template',
            File::Spec->catfile('MBO',$InputValues->{'PageTypeContext'}.'.MarketingMenu-'.$InputValues->{'PageTypeAlias'}.'.html')));
        $InputValues->{'MBOMarketingMenuText'} = getInput('What text should appear in the Marketing Menu',  'A short hint about what the user can do here');
        $InputValues->{'MBOMarketingMenuKeyword'} = createKeyword ( $InputValues->{'PageTypeAlias'}.'Info');
        $InputValues->{'MBOMarketingMenuTemplateFileName_en'} = substr($InputValues->{'MBOMarketingMenuTemplateFileName'},0,(length $InputValues->{'MBOMarketingMenuTemplateFileName'})-4).'en.xml';
        $InputValues->{'MBOMarketingMenuTemplateFileName_de'} = substr($InputValues->{'MBOMarketingMenuTemplateFileName'},0,(length $InputValues->{'MBOMarketingMenuTemplateFileName'})-4).'de.xml';
    }
    ## use critic
    print 'Do you really want to add this page-type? (y/n) ';
    if ( getKeyYesNo() eq 'Yes') {
        push @{$hOptions->{'AllMBOPages'}}, $InputValues;
        print "\nPage type added.\n";
    }
}

sub viewPagetypes {
    my ($hOptions, $MenuItem) = @_;
    print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
}

sub removePagetype {
    my ($hOptions) = @_;
    print "\n\n";
    my @AllPages = ();
    if (defined $hOptions->{'AllSFPages'}) { push @AllPages, @{$hOptions->{'AllSFPages'}}; }
    if (defined $hOptions->{'AllMBOPages'}) { push @AllPages, @{$hOptions->{'AllMBOPages'}}; }
    my $countSF  = defined $hOptions->{'AllSFPages'} ? scalar @{$hOptions->{'AllSFPages'}} : 0;
    my $countMBO = defined $hOptions->{'AllMBOPages'} ? scalar @{$hOptions->{'AllMBOPages'}} : 0;
    my $count    = $countSF + $countMBO;
    my $default  = $count ? $AllPages[$count-1]->{'PageTypeAlias'} : '';
    my $Alias    = getInput('Remove which Pagetype? ',$default);
    my $found = 0;
    if ($countSF) {
        for (my $i = 0; $i < $countSF; $i++) {
            if ( $hOptions->{'AllSFPages'}[$i]{'PageTypeAlias'} eq $Alias) {
                $found = 1;
                splice @{$hOptions->{'AllSFPages'}},$i,1;
                last;
            }
        }
    }
    if ($countMBO) {
        for (my $i = 0; $i < $countMBO; $i++) {
            if ( $hOptions->{'AllMBOPages'}[$i]{'PageTypeAlias'} eq $Alias) {
                $found = 1;
                splice @{$hOptions->{'AllMBOPages'}},$i,1;
                last;
            }
        }
    }
    if (!$found) {
        print "\nPagetype not found.\n";
    }
}

sub Menu_CreatePageType {
    my ($hOptions, $Menu) = @_;
    my $MenuItem=$Menu->{'CreatePageType'};
    while (1) {
        print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
        my $key = getMappedKey($MenuItem->{'KeyMap'}, { 'AllowAll' => 1 } );
        if ($key eq 'sf')        { addSFPageType($hOptions); }
        elsif ($key eq 'mbo')    { addMBOPagetype($hOptions, $Menu); }
        elsif ($key eq 'remove') { removePagetype($hOptions); }
        elsif ($key eq 'view')   { viewPagetypes($hOptions, $Menu->{'viewPageTypes'}); }
        elsif ($key eq 'exit')   { return; }
        else {print "\n\nBad input. Try again. ('x' - exit)\n" };
    }
}
#========================================================================================
# Attributes                                                                            #
#========================================================================================
sub addAttribute {
   my ($hOptions, $Menu) = @_;
    print "\n".underline('Add user defined attribute')."\n";
    my $InputValues = {};
    $InputValues->{'ObjectPath'} = getInput('Objectpath','/Classes/Shop');
    do {
        $InputValues->{'AttributeAlias'} = getInput('Attribute Alias','MyAttribute');
    } while ( isAliasUsed($InputValues->{'AttributeAlias'}, 'AttributeAlias', @{$hOptions->{'AllAttributes'}}) );
    $InputValues->{'AttributeName'} = getInput('Attribute name','My user-defined Attribute');
    $InputValues->{'AttributeDescription'} = getInput('Attribute description','Short info about this attribute.');
    $InputValues->{'AttributeType'} = getInput('Attribute Type','String');
    $InputValues->{'AttributePackage'} = getInput('Package','DE_EPAGES::Object::API::Attributes::DefaultAttribute');
    print 'Do you really want to add this attribute? (y/n) ';
    if ( getKeyYesNo() eq 'Yes') {
        push @{$hOptions->{'AllAttributes'}}, $InputValues;
        print "\nAttribute added.\n";
    }
}

sub removeAttribute {
    my ($hOptions) = @_;
    print "\n\n";
    my $count = defined $hOptions->{'AllAttributes'} ? scalar @{ $hOptions->{'AllAttributes'} } : 0;
    my $default = $count ? $hOptions->{'AllAttributes'}->[$count-1]->{'AttributeAlias'} : '';
    my $Alias = getInput('Remove which Attribute? ',$default);
    # check if it is already in the dependencies
    my $found = 0;
    # check if dependency exists
    if ($count >0) {
        for (my $i = 0; $i < $count; $i++) {
            if ($hOptions->{'AllAttributes'}[$i]{'AttributeAlias'} eq $Alias) {
                $found = 1;
                splice @{$hOptions->{'AllAttributes'}},$i,1;
                last;
            }
        }
    }
    if (!$found) {
        print "\nAttribute not found.\n";
    }
}

sub viewAttributes {
    my ($hOptions, $MenuItem) = @_;
    print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
}

sub Menu_CreateAttribute {
    my ($hOptions, $Menu) = @_;
    my $MenuItem=$Menu->{'CreateAttribute'};
    while (1) {
        print ReplaceTLEs($MenuItem->{'View'}, $hOptions );
        my $key = getMappedKey($MenuItem->{'KeyMap'}, { 'AllowAll' => 1 } );
        if ($key eq 'add')       { addAttribute($hOptions, $Menu); }
        elsif ($key eq 'remove') { removeAttribute($hOptions); }
        elsif ($key eq 'view')   { viewAttributes($hOptions, $Menu->{'ViewAttributes'}); }
        elsif ($key eq 'exit')   { return; }
        else {print "\n\nBad input. Try again. ('x' - exit)\n" };
    }
}
#========================================================================================
# Menu                                                                                  #
#========================================================================================
sub UserMenu {
    my ($hOptions) = @_;
    my $Menu = setMenuDefaults();
    while ( 1 ) {
        if ($hOptions->{'_DEBUG'}) { print "hOptions:\n"; print Dumper($hOptions); }
        print ReplaceTLEs($Menu->{'MainMenu'}->{'View'}, $hOptions );
        my $key = getMappedKey( $Menu->{'MainMenu'}->{'KeyMap'}, { 'AllowAll' => 1 } ) ;

        if ($key eq 'view') {
            $key = Menu_ViewInfo($hOptions, $Menu);
        }
        elsif($key eq 'name') {
            $key = Menu_UpdateBasic($hOptions, $Menu->{'BasicInfo'});
        }
        elsif($key eq 'doc') {
            $key = Menu_UpdateDoc($hOptions, $Menu->{'UpdateDoc'});
        }
        elsif($key eq 'dependency') {
            $key = Menu_UpdateDependency($hOptions, $Menu->{'UpdateDependency'});
        }
        elsif($key eq 'pagetype') {
            $key = Menu_CreatePageType($hOptions, $Menu);
        }
        elsif($key eq 'attribute') {
            $key = Menu_CreateAttribute($hOptions, $Menu);
        }
        elsif($key eq 'exit') {
            print "\n\nReally exit without writing cartridge files to disk? (y/n) ";
            return undef if (getKeyYesNo() eq 'Yes');
        }
        elsif($key eq 'write') {
            print "\n\nWrite cartridge files to disk and exit? (y/n) ";
            return SaveUserTemplate($hOptions) if (getKeyYesNo() eq 'Yes');
        }
        else {
            print "\n\nBad input. Try again. ('x' - exit)\n"
        }
    }
}

#========================================================================================
# Template creation functions                                                           #
#========================================================================================
sub SavePredefinedTemplates {
    my ($SourcePath, $hOptions) = @_;
    my $Creator = DE_EPAGES::Cartridge::API::CartridgeCreator->new(
        TemplateDir => $SourcePath,
        Vars => $hOptions
    );
    return $Creator->create;
}

sub getFlags {
   my $hFlags = {};
    if( $Config{'osname'} ne 'MSWin32' ) {
        my $ApplUser         = 'ep_appl';
        $hFlags->{'Mode'}    = GROUP_R;
        $hFlags->{'UserID'}  = (getpwnam $ApplUser)[2];
        $hFlags->{'GroupID'} = (getpwuid $hFlags->{'UserID'})[3];
    }
    $hFlags->{'create_directory'} = 1;
    return $hFlags;
}

sub RenameAndSaveTemplate {
    my ( $SourceFile, $DestinationFile, $hVars ) = @_;
    my $sContent = GetFileContent($SourceFile);
    my $NewContent = ReplaceTLEs ($sContent, $hVars);
    WriteFile($DestinationFile, \$NewContent,  getFlags() );
    return $DestinationFile;
}

sub SaveSpecialMBOPages {
    my ($FileTemplateBaseDir, $hOptions) = @_;
    my $aFilesWritten = [];
    foreach my $MBOPage (@{$hOptions->{'AllMBOPages'}}) {
        my $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'MBO.Content.tmpl');
        my $FileName = File::Spec->catfile( $hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'ContentFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

        $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'MBO.TabPage.tmpl');
        $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'TabPageFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

        $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'ContextMenu.LinkActive.tmpl');
        $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'MBOContextActiveMenuTemplateFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

        $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'ContextMenu.LinkInactive.tmpl');
        $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'MBOContextMenuTemplateFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

        if ($MBOPage->{'PageTypeContext'} eq 'MBO-Marketing') {
            $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'MarketingMenu.tmpl');
            $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'MBOMarketingMenuTemplateFileName'});
            push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

            $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'MarketingMenu.de.tmpl');
            $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'MBOMarketingMenuTemplateFileName_de'});
            push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );

            $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'MarketingMenu.en.tmpl');
            $FileName = File::Spec->catfile($hOptions->{'OSCartridgeDirectory'},'Templates',$MBOPage->{'MBOMarketingMenuTemplateFileName_en'});
            push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $MBOPage );
        }
    }
    return $aFilesWritten;
}

sub SaveSpecialSFPages {
    my ($FileTemplateBaseDir, $hOptions) = @_;
    my $aFilesWritten = [];
    foreach my $SFPage (@{$hOptions->{'AllSFPages'}}) {
        my $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'SF.Content.tmpl');
        my $FileName = File::Spec->catfile( $hOptions->{'OSCartridgeDirectory'},'Templates',$SFPage->{'ContentFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $SFPage );

        $SourceFile = File::Spec->catfile($FileTemplateBaseDir,'SF.NavElement.tmpl');
        $FileName = File::Spec->catfile( $hOptions->{'OSCartridgeDirectory'},'Templates',$SFPage->{'NavElementFileName'});
        push @{$aFilesWritten}, RenameAndSaveTemplate( $SourceFile, $FileName, $SFPage );
    }
    return $aFilesWritten;
}

sub SaveUserTemplate {
    my ($hOptions) = @_;
    my $TemplateBaseDir = File::Spec->catfile($ENV{EPAGES_CARTRIDGES},'DE_EPAGES','Cartridge','CartridgeTemplates','User');
    my $aFilesWritten = [];
    # Copying default templates
    push @{$aFilesWritten}, @{SavePredefinedTemplates(File::Spec->catfile($TemplateBaseDir,'Default'),$hOptions)};

    my $HasAllSFPages  = defined $hOptions->{'AllSFPages' } && scalar @{$hOptions->{'AllSFPages' }};
    my $HasAllMBOPages = defined $hOptions->{'AllMBOPages'} && scalar @{$hOptions->{'AllMBOPages'}};
    # Copying templates for all page types
    if ( $HasAllSFPages || $HasAllMBOPages ) {
        push @{$aFilesWritten}, @{SavePredefinedTemplates(File::Spec->catfile($TemplateBaseDir,'AllPageTypes'),$hOptions)};
    }
    # Storefront general templates
    if ($HasAllSFPages) {
        push @{$aFilesWritten}, @{SavePredefinedTemplates(File::Spec->catfile($TemplateBaseDir,'SFPageTypes'),$hOptions)};
        push @{$aFilesWritten}, @{SaveSpecialSFPages(File::Spec->catfile($TemplateBaseDir,'FileTemplates'),$hOptions)};
    }
    # MBO general templates
    if ($HasAllMBOPages) {
        push @{$aFilesWritten}, @{SavePredefinedTemplates(File::Spec->catfile($TemplateBaseDir,'MBOPageTypes'),$hOptions)};
        push @{$aFilesWritten}, @{SaveSpecialMBOPages(File::Spec->catfile($TemplateBaseDir,'FileTemplates'),$hOptions)};
    }
    return $aFilesWritten;
}

sub SimpleTemplate {
    my ($Template,$BasePackage,$hOptions) = @_;

    my ($Company, $Package) = split /::/, $BasePackage, 2;

    my $TemplateDir = File::Spec->catfile(
        $ENV{EPAGES_CARTRIDGES},
        $Company,
        $Package,
        'CartridgeTemplates',
        $Template
    );

    if (not -d $TemplateDir) {
        print STDERR 'Invalid base package: '.$BasePackage. "\n";
        exit 2;
    }

    my $Creator = DE_EPAGES::Cartridge::API::CartridgeCreator->new(
        TemplateDir => $TemplateDir,
        Vars => $hOptions
    );
    return $Creator->create;
}


#========================================================================================
# function     _AddIniSettings
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->_AddIniSettings( $ConfigFile, $hInfo);
# example      $Object->_AddIniSettings( 'c:/config/my.ini', $hInfo );
#----------------------------------------------------------------------------------------
# description  adds ini settings to hInfo hash.
#               Sample ini:
#               [MAIN]
#               ; this generates top-level TLE #CartridgeVersion
#               CartridgeVersion=6.9.3
#               ; This generates a LOOPable hash list
#               ; #LOOP(#AllDependencies)<Dependency Package="#Package" Comment="#Comment" />#ENDLOOP
#               AllDependencies=<<EOT
#               Package=DE_EPAGES::Cartridge, Comment=Test
#               Package=DE_EPAGES::Design
#               Package=DE_EPAGES::ThirdPartyConfig
#               EOT
#               [Portal]
#               ; This generates TLE-Hash #Portal.UseFTP
#               UseFTP=1
#----------------------------------------------------------------------------------------
# input        $ConfigFile | path to ini file | string
# input        $hInfo      | hInfo TLE-Varibale hash | ref.hash
#========================================================================================
sub _AddIniSettings {
    my ($ConfigFile, $hInfo) = @_;
    my $IniConfig = DE_EPAGES::Core::API::IniConfig->new(
        FileName => $ConfigFile,
        CaseSensitive => 1
    );

    foreach my $Section ( @{$IniConfig->sectionNames} ) {
        my %Entries = %{$IniConfig->section($Section)};
        while(my ($Key, $Value) = each(%Entries)) {
            my @Values;
            # multi-values? creating hashref list
            if ($Value =~ /\n/ or $Value =~ /=/) {
                my @StringLines = split /\n/, $Value;

                foreach my $Line (@StringLines){
                    my %Hash;
                    my @KeyValuesPairs = split /,\s+/, $Line;

                    foreach my $Pair (@KeyValuesPairs){
                        my ($PairKey, $PairValue) = ($Pair =~ /([^=]+)=(.*)/x);
                        $Hash{$PairKey} = $PairValue;
                    }
                    push @Values, \%Hash;
                }
                $Value = \@Values;
            }

            if ($Section eq 'MAIN') {
                # top-level TLE
                $hInfo->{$Key} = $Value;
            } else {
                # sub-level TLE
                $hInfo->{$Section}->{$Key} = $Value;
            }
        }
    }
}


sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags] packagename

options:
    -template Basic            Creates basic folders and files (default)
    -template Example          Creates an extended version
    -configfile (filename)     Read configfile with additional settings
    -basepackage (DE_EPAGES::Cartridge) use custom base package.
                               This is only possible if cartridge contains
                               a CartridgeTemplates directory.
                               Possible values are:
                               DE_EPAGES::Cartridge (default)
                               DE_EPAGES::ProductPortal
flags:
    -help                      Show the command line options
    -i                         Enters interactive mode
    -verbose                   Outputs additional status information

Example:
    perl $0 -configfile ../MyCartridgeTemplates/template.ini -template Basic DE_MyCompany::MyCartridge
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
