#========================================================================================
# package      DE_EPAGES::Content::API::Attributes::ContentObject_Ext
# base         DE_EPAGES::Object::API::BaseAttribute
# state        private
# description  Implements the attribute ContentObject.ContentAttributeValues
#========================================================================================
package DE_EPAGES::Content::API::Attributes::ContentObject_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::Html qw ( RemoveHtmlTags );
use DE_EPAGES::Core::API::String qw ( ConvertNewlines );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObject);
use DE_EPAGES::Permission::API::Permission qw ( ActionByName RoleByName HaveSameSite Everyone Registered IsActionAllowed SwitchInherit );
use DE_EPAGES::Permission::API::Table::Permission   qw ( DeletePermission InsertPermission ExistsPermission );
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);
use HTML::Entities qw( encode_entities );
use URI::Escape qw( uri_escape_utf8 );

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     getAttributes
# state        private
# syntax       $Value = $Object->get($AttributeName);
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my $hInfo = {};
    foreach my $Name ( @$aNames ) {
        if( $Name eq 'VisibleChildObjects' ) {
            $hInfo->{$Name} = _VisibleChildObjects( $Object );
        }
        elsif( $Name eq 'VisibleContentObjects' ) {
            $hInfo->{$Name} = _VisibleContentObjects( $Object );
        }
        elsif( $Name eq 'ContentObjectsInNavigation' ) {
            $hInfo->{$Name} = _ContentObjectsInNavigation( $Object );
        }
        elsif( $Name eq 'NavigationString' ) {
            my $String = $Object->get('NavigationCaption', $LanguageID);
            $String = $Object->get('Name', $LanguageID) if not defined $String or $String !~ /\S/;
            $String = $Object->alias if not defined $String or $String !~ /\S/;
            $hInfo->{$Name} = $String;
        }
        elsif( $Name eq 'TagList' ) {
            $hInfo->{$Name} = _TagList($Object->id);
        }
        elsif( $Name eq 'TagNameList' ) {
            $hInfo->{$Name} = [map { $_->get('Tag') } @{_TagList($Object->id)}];
        }
        elsif( $Name eq 'ReadPermissionFor' ) {
            $hInfo->{$Name} = _ReadPermissionFor($Object);
        }
        elsif( $Name eq 'WritePermissionFor' ) {
            $hInfo->{$Name} = _WritePermissionFor($Object);
        }
        elsif( $Name eq 'IsVisibleContentView' ) {
            $hInfo->{$Name} = $Object->instanceOf('StartPage') ? !$Object->getSite()->get('ClosedByMerchant') : $Object->get('IsVisible');
        }
        elsif( $Name eq 'CanonicalURL' ) {
            my $Uri;
            my $Shop = $Object->getSite();
            my $HasSSLCertificate = $Shop->get('HasSSLCertificate');
            if (defined $Shop->get('DomainName')) {
                if ($Object->instanceOf('StartPage')) {
                    if ($Shop->get('LanguageID') != $LanguageID) {
                        $Uri = $Shop->localeByLanguage($LanguageID);
                    } else {
                        $Uri = '';
                    }
                } else {
                    $Uri = $Object->get('URI', $LanguageID);
                }
            }

            my $CanonicalURL;
            if (defined $Uri) {
                $CanonicalURL = ($HasSSLCertificate) ? $Shop->get('ProtocolAndServerSSL') : $Shop->get('ProtocolAndServer');
                $CanonicalURL = $CanonicalURL . '/'; # EPG-18932 : ending slash for shop domain
                $CanonicalURL = $CanonicalURL . $Uri if $Uri ne '';
            } else {
                my $Value = uri_escape_utf8( $Object->get('Path') );
                $Value =~ s/%2F/\//ig;
                $CanonicalURL = ($HasSSLCertificate) ? $Shop->get('WebServerScriptUrlSSL') : $Shop->get('WebServerScriptUrl');
                $CanonicalURL = $CanonicalURL . '/' . $Shop->localeByLanguage($LanguageID) . '/?ObjectPath=' . $Value;
            }
            $hInfo->{$Name} = $CanonicalURL;
        }
        elsif( $Name eq 'DisplayTitle' ) {
            $hInfo->{$Name} = _DisplayTitle($Object, $LanguageID);
        }
        elsif( $Name eq 'DisplayDescription' ) {
            #   Description[nohtml,nobreak,html]
            my $Description = $Object->get('Description', $LanguageID);
            if($Description) {
                # == [nohtml,nobreak,html] formatters
                $hInfo->{$Name} = encode_entities( ConvertNewlines( RemoveHtmlTags($Description) ), '<>"' );
            }
        }
    }
    return $hInfo;
}

# groups that include other groups should be tested first
sub _ShopGroups {
    my ($Shop) = @_;

    my $MerchantGroup;
    my @UserGroups;
    foreach my $Group ( @{$Shop->child('Groups')->children} ) {
        if( $Group->alias eq 'Merchants' ) {
            $MerchantGroup = $Group;
        }
        else {
            push @UserGroups, $Group;
        }
    }

    return [
        Everyone(),
        Registered(),
        @UserGroups,
        $MerchantGroup,
    ];
}

sub _ReadPermissionFor {
    my ($Object) = @_;

    my $Action = ActionByName( $Object->class, 'View' );
    foreach my $Group ( @{ _ShopGroups( $Object->getSite() ) } ) {
        return $Group->id if IsActionAllowed( $Object, $Group, $Action );
    }

    return undef;
}

sub _WritePermissionFor {
    my ($Object) = @_;

    my $RoleWriter = RoleByName(LoadClassByAlias('Shop'), 'Writer');
    foreach my $Group ( @{ _ShopGroups( $Object->getSite() ) } ) {
        return $Group->id if IsActionAllowed( $Object, $Group, $RoleWriter );
    }

    return undef;
}

sub _TagList {
    my ($ObjectID) = @_;
    return [sort { $a->get('Tag') cmp $b->get('Tag') } @{LoadClassByAlias('ContentTag')->getObjects('ByContentObjectID',[$ObjectID])}];
}

#========================================================================================
# function     cloneAttributes
# state        private
# description  clone assigned attributes of object
#========================================================================================
sub cloneAttributes {
    shift;
    my ($Object, $Clone) = @_;

    # clone layout
    my $PageType = LoadPageTypeByViewAction( ActionByName( $Object->class, 'View' ) );
    my $aTemplateTypeFolders = $PageType->templateTypeFolders;
    foreach my $TemplateName ( map {$_->alias} @$aTemplateTypeFolders) {
        if ($PageType->existsDirectTemplate( $TemplateName, $Object )) {
            my $hTemplate = $PageType->template( $TemplateName, $Object );
            $PageType->insertTemplate( {
                ObjectID    => $Clone->id,
                Name        => $TemplateName,
                FileName    => $hTemplate->{'FileName'},
                CartridgeID => $hTemplate->{'CartridgeID'},
            }) unless $PageType->existsDirectTemplate( $TemplateName, $Clone);
        }
    }
}

sub _VisibleChildObjects {
    my ($Object) = @_;

    return [
        grep { $_->class->existsAttribute('IsVisible') && $_->get('IsVisible') }
        @{ $Object->children }
    ];
}

sub _VisibleContentObjects {
    my ($Object) = @_;
    return [
        sort { $a->get('Position') <=> $b->get('Position') }
        @{LoadClassByAlias('ContentObject')->getObjects('ByParentIDIsVisible', [$Object->id, 1])}
    ];
}

sub _ContentObjectsInNavigation {
    my ($Object) = @_;
    return [
        sort { $a->get('Position') <=> $b->get('Position') }
        grep { $_->get('VisibleInNavigation') }
        @{ LoadClassByAlias('ContentObject')->getObjects('ByParentIDIsVisible', [$Object->id, 1]) }
    ];
}

#========================================================================================
# function     _DisplayTitle
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DisplayTitle = _DisplayTitle($Object, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Returns the page title of the storefront page.
#----------------------------------------------------------------------------------------
# input        $Object | product object | DE_EPAGES::Content::API::Object::ContentObject
# input        $LanguageID | language id | int
# return       $DisplayTitle | display title | string
#========================================================================================
sub _DisplayTitle {
    my ($Object, $LanguageID) = @_;

    my $Title = $Object->get('Title', $LanguageID);
    return $Title if defined $Title;

    my $ShopName = $Object->getSite->get('Name', $LanguageID);
    my $Name = $Object->get('NameOrAlias', $LanguageID);

    return $Name unless defined $ShopName;
    return $Name if $ShopName eq $Name;
    return "$Name - $ShopName";
}

#========================================================================================
# function     setAttributes
# state        private
# syntax       $Value = $Object->set($AttributeName, $Value);
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hAttributes, $LanguageID) = @_;

    foreach my $Name (keys %$hAttributes) {
        my $value = $hAttributes->{$Name};
        if ($Name eq 'ReadPermissionFor') {
            next unless defined $value;
            my $RoleReaderID = RoleByName(LoadClassByAlias('Shop'), 'Customer')->id;
            if ($value == Everyone()->id) {
                SwitchInherit($Object) unless $Object->get('Inherit');
            } else {
                SwitchInherit($Object) if $Object->get('Inherit');
            }
            _setPermission($Object, $value, $RoleReaderID);
            # insert customer permission for admins
            my $Permission = {
                ObjectID => $Object->id,
                TrusteeID => $Object->getSite()->child('Groups')->child('Merchants')->id,
                MetaActionID => $RoleReaderID,
                Allow => 1,
            };
            if ($Object->get('Inherit')) {
                DeletePermission($Permission) if ExistsPermission($Permission);
            } else {
                InsertPermission($Permission) unless ExistsPermission($Permission);
            }
        } elsif ($Name eq 'WritePermissionFor') {
            next unless defined $value;
            my $RoleWriterID = RoleByName(LoadClassByAlias('Shop'), 'Writer')->id;
            _setPermission($Object, $value, $RoleWriterID);
        } elsif ($Name eq 'IsVisibleContentView') {
            if ($Object->instanceOf('StartPage')) {
                $Object->getSite()->set({'ClosedByMerchant'=> $hAttributes->{$Name} ? 0 : 1});
            } else {
                $Object->set({'IsVisible'=> $hAttributes->{$Name} ? 1 : 0});
            }
        }
    }
}

sub _setPermission {
    my ($Object, $TrusteeID, $MetaActionID) = @_;

    my $ObjectID = $Object->id;
    if (not HaveSameSite(LoadObject($TrusteeID), $Object)) {
        die("Foreign Trustee ($TrusteeID) used for object ($ObjectID)");
    }

    # delete old direct permissions of object
    DeletePermission($_) foreach grep {$_->{'MetaActionID'} == $MetaActionID } @{$Object->get('DirectPermissions')};
    # insert new permission, if not assigned yet
    InsertPermission({
        ObjectID => $ObjectID,
        TrusteeID => $TrusteeID,
        MetaActionID => $MetaActionID,
        Allow => 1,
    }) unless grep {$_->{'MetaActionID'} == $MetaActionID && $_->{'TrusteeID'} == $TrusteeID } @{$Object->get('Permissions')};
}

1;
