#========================================================================================
# package      DE_EPAGES::Core::API::CSV::BaseImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Base class for csv import drivers
#========================================================================================
package DE_EPAGES::Core::API::CSV::BaseImportHandler;

use strict;
use FileHandle;

use DE_EPAGES::Core::API::Constants qw( BOM DEFAULT_ENCODING );
use DE_EPAGES::Core::API::CSV;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError HandleError );
use DE_EPAGES::Core::API::File qw ( DetectFileEncoding );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );

my %FILE_FORMAT = (
    'eol'           => "\n",
    'encoding'      => DEFAULT_ENCODING,
    'quote_char'    => '"',
    'escape_char'   => '"',
    'sep_char'      => ',',
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'FILE_OPEN_FAILED'  => 'CSV file (#FileName): can\'t open file (#Code)',
        'FILE_CLOSE_FAILED' => 'CSV file (#FileName): can\'t close file (#Code)',
        'FILE_SEEK_FAILED' => 'CSV file (#FileName): can\'t see to position #Position (#Code)',
        'READ_LINE_FAILED' => 'CSV file (#FileName), line #Line: can\'t read line (#Code)',
        'FILE_NOT_TEXT' => 'The file #FileName is not a text file.'
    };
}
#ERRORDEFINITION

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::Core::API::CSV::BaseImportHandler->new(%options);
#----------------------------------------------------------------------------------------
# description  create a new csv import handler
#----------------------------------------------------------------------------------------
# input        %options |  importer options | hash
# return       $Handler |  importer | object
#========================================================================================
sub new {
    my $class = shift;
    my %params = @_;
    $params{$_} = $FILE_FORMAT{$_} foreach grep {not exists $params{$_}} keys %FILE_FORMAT;
    $params{'Converter'} = DE_EPAGES::Core::API::CSV->new(\%params);

    return bless \%params, $class;
}

#========================================================================================
# function     importCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->importCSV($FileName, $hFormat);
#----------------------------------------------------------------------------------------
# description  imports data (2 dim. array) from a file.
#               opens the document, calls function setLinesCount, calls function parse
#               and closes the document
#----------------------------------------------------------------------------------------
# input        $FileName |  export file | string
# input        $hOptions |  export file format
#                   <ul>
#                     <li>eol - end of line (optional, default: "\n")</li>
#                     <li>encoding - character encoding (optional, default: "iso-8859-1")</li>
#                     <li>quote_char - field delimiter (optional, default: '"')</li>
#                     <li>escape_char - escape character (optional, default: '"')</li>
#                     <li>sep_char - field separator (optional, default: ",")</li>
#                     <li>... other options</li>
#                   </ul> | ref.hash
# return       $Importer |  importer | object
#========================================================================================
sub importCSV {
    my $class = shift;
    my ($FileName, $hOptions) = @_;
    TestString( 'FileName', $FileName );

    Error( 'FileNotFound', { FileName => $FileName } ) unless -f $FileName;
    Error( 'FILE_NOT_TEXT', { FileName => $FileName } ) unless -T $FileName;

    my $self = $class->new(defined $hOptions? %$hOptions : ());

    my $DetectedEncoding = DetectFileEncoding( $FileName );
    if( defined $DetectedEncoding ) {
        $self->{'encoding'} = $DetectedEncoding;
        $self->{'UseBOM'} = 1;
    }

    $self->{'FileName'} = $FileName;
    $self->{'LineNumber'} = 0;
    $self->{'FileHandle'} = FileHandle->new;

    my $fh = $self->fileHandle;
    my $OpenMode = '<:encoding('. $self->encodeFrom . ')';

    $fh->open( $FileName, $OpenMode ) or Error('FILE_OPEN_FAILED', {
        'Mode'=>$OpenMode,
        'FileName'=>$FileName,
        'Code'=>$!
    });

    # count the records
    $self->setLinesCount;

    # revert to start of file
    $fh->seek( 0, 0 ) or Error('FILE_SEEK_FAILED', {
        'FileName'=>$FileName,
        'Position' => 0,
        'Code'=>$!
    });

    # remove the Unicode byte order mark
    if( $self->useBOM ) {
        my $BOM;
        $fh->read( $BOM, 1 );
        die 'Invalid BOM' if $BOM ne BOM;
    }

    # import data
    eval {
        $self->parse();
    };
    HandleError(
        OnError => sub {
            my ($Error) = @_;
            $Error->addVars({ 'FileName'=>$FileName });
            $fh->close;
        },
    );

    $fh->close or Error('FILE_CLOSE_FAILED', {
        'Mode'=>$OpenMode,
        'FileName'=>$FileName,
        'Code'=>$!
    });

    return $self;
}

#========================================================================================
# function     setLinesCount
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setLinesCount;
#----------------------------------------------------------------------------------------
# description  counts the lines of current fileHandle by reading the file
#               save the result to get it via function linesCount
#========================================================================================
sub setLinesCount {
    my $self = shift;

    my $fh = $self->fileHandle;
    $fh->seek( 0, 0 ); # revert to start of file
    if( $self->useBOM ) {
        # read BOM
        my $BOM;
        $fh->read( $BOM, 1 );
        die 'Invalid BOM' if $BOM ne chr(0xFEFF); ## no critic qw(ProhibitMagicNumbers)
    }

    # ignore encoding warnings
    local $SIG{__WARN__} = sub { ; };

    # try to read the header line
    my $converter = $self->converter;
    my $aFields = eval { $converter->getline($fh) };
    if( ExistsError() ) {
        my $Error = GetError();
        Error( 'FILE_NOT_TEXT', { FileName => $self->fileName }, $Error ) if $Error->code eq 'GET_LINE_FAILED';
        Error( $Error );
    }

    if( !defined $aFields ||                               # file is empty
        ( 1 == scalar @$aFields && $aFields->[0] !~ /\S/ ) # header is empty
    ) {
        Error( 'FILE_NOT_TEXT', {
            FileName => $self->fileName,
            $self->converter->formatSpec
        } );
    }

    # count the data lines
    my $LinesCount = 0;
    while ($aFields) {
        $aFields = eval {
            $converter->getline($fh)
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $ErrorCode = $Error->code;

            if ($ErrorCode eq 'GET_LINE_FAILED' and $LinesCount == 0) {
                Error('FILE_NOT_TEXT', {'FileName' => $self->fileName});
            }
            elsif ($ErrorCode eq 'GET_LINE_FAILED') {
                Error('READ_LINE_FAILED', {
                    'FileName' => $self->fileName,
                        'Code' => $Error->vars->{'Message'},
                        'Line' => $LinesCount + 2
                      });
            }
            elsif ($ErrorCode eq 'NON_PRINTABLE_CHARS') {
                Error('READ_LINE_FAILED', {
                    'FileName' => $self->fileName,
                        'Code' => 'contains characters ' . $Error->vars->{'Chars'},
                        'Line' => $LinesCount + 2
                      });
            }
            else {
                Error($Error);
            }
        }

        last unless defined $aFields;
        $LinesCount++;
    }

    $self->{'LinesCount'} = $LinesCount;
    return;
}

#========================================================================================
# function     linesCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->linesCount;
#----------------------------------------------------------------------------------------
# description  return the count of lines of current fileHandle
#----------------------------------------------------------------------------------------
# return       $LinesCount | set by setLinesCount | integer
#========================================================================================
sub linesCount { return shift->{'LinesCount'}; }

#========================================================================================
# function     fileHandle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileHandle;
#----------------------------------------------------------------------------------------
# description  return the current fileHandle
#----------------------------------------------------------------------------------------
# return       $FileHandle | setted by exportCSV | filehandle
#========================================================================================
sub fileHandle { return shift->{'FileHandle'}; }

#========================================================================================
# function     header
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->header;
#----------------------------------------------------------------------------------------
# description  return the header columns
#----------------------------------------------------------------------------------------
# return       $Header | header columns | ref.array.string
#========================================================================================
sub header { return shift->{'Header'}; }


#========================================================================================
# function     fileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileName;
#----------------------------------------------------------------------------------------
# description  return the current fileName
#----------------------------------------------------------------------------------------
# return       $FileName | export file name | string
#========================================================================================
sub fileName { return shift->{'FileName'}; }

#========================================================================================
# function     converter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $converter = $self->converter;
#----------------------------------------------------------------------------------------
# description  return the current csv converter
#----------------------------------------------------------------------------------------
# return       $converter
#========================================================================================
sub converter { return shift->{'Converter'}; }

#========================================================================================
# function     encodeFrom
# state        public
#----------------------------------------------------------------------------------------
# syntax       encodeFrom();
# example      $Driver->encodeFrom();
#----------------------------------------------------------------------------------------
# description  returns the target encoding (was set in <function exportCSV>)
#----------------------------------------------------------------------------------------
# return       $Encoding | target encoding (utf8,iso-8859-1,...) | string
#========================================================================================
sub encodeFrom { return shift->{'encoding'}; }

#========================================================================================
# function     useBOM
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UseBPM = $Driver->useBOM;
#----------------------------------------------------------------------------------------
# description  returns true if the file has a Unicode Byte Order Mark
#----------------------------------------------------------------------------------------
# return       $UseBPM | true if the file has a Unicode Byte Order Mark | boolean
#========================================================================================
sub useBOM { return shift->{'UseBOM'}; }

#========================================================================================
# function     incLineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->incLineNumber;
#----------------------------------------------------------------------------------------
# description  increments the line number and returns the new number
#----------------------------------------------------------------------------------------
# return       $linenumber | current line number | integer
#========================================================================================
sub incLineNumber { return ++shift->{'LineNumber'}; }

#========================================================================================
# function     lineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->lineNumber;
#----------------------------------------------------------------------------------------
# description  return the current linenumber
#----------------------------------------------------------------------------------------
# return       $linenumber | current line number | integer
#========================================================================================
sub lineNumber { return shift->{'LineNumber'}; }

#========================================================================================
# function     readLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aLineElements = $self->readLine;
#----------------------------------------------------------------------------------------
# description  imports a line ( reads and converts from the filehandle )
#----------------------------------------------------------------------------------------
# return       $aLineElements | list of values or undef on end of file | ref.array.string
#========================================================================================
sub readLine {
    my $self = shift;

    local $SIG{__WARN__} = sub {
        my ($Message) = @_;
        $Message =~ s/ at \S+ line \d+\b.*//; # remove code line
        Error('READ_LINE_FAILED',{
            'FileName'=> $self->fileName,
            'Code'=> $Message,
            'Line' => $self->lineNumber,
        });
    };

    my $aLineElements = eval { $self->converter->getline( $self->fileHandle ) };
    if( ExistsError() ) {
        my $Error = GetError();
        if( $Error->code eq 'GET_LINE_FAILED' ) {
            Error('READ_LINE_FAILED',{
                'FileName' => $self->fileName,
                'Code' => $Error->vars->{'Message'},
                'Line' => $self->lineNumber
            })
        };
        Error( $Error );
    }

    return undef unless defined $aLineElements;

    $self->incLineNumber;
    return $aLineElements;
}

#========================================================================================
# function     readHeader
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->readHeader();
#----------------------------------------------------------------------------------------
# description  read the header
#========================================================================================
sub readHeader {
    my $self = shift;

    my $aHeader = $self->readLine;
    # convert header to usefull structure
    $self->{'Header'} = $aHeader;
    return;
}

#========================================================================================
# function     importLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importLine();
#----------------------------------------------------------------------------------------
# description  imports one data line, get the line element from function topID('Item')
#               this is only a dummy funtion here.
#========================================================================================
sub importLine {
    my $self = shift;

    # add item to database
    return;
}

#========================================================================================
# function     readLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->readLines();
#----------------------------------------------------------------------------------------
# description  reads the content lines via function readLine, calls for each line function importLine
#========================================================================================
sub readLines {
    my $self = shift;
    while (my $aLine = $self->readLine) {
        $self->pushID('Item' , $aLine);
        $self->importLine();
        $self->popID('Item');
    }
    return;
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  read the data content lines, uses function readLines
#========================================================================================
sub drive {
    my $self = shift;

    $self->readLines();
    return;
}

#========================================================================================
# function     parse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->parse();
#----------------------------------------------------------------------------------------
# description  reads the header via function readHeader, calls function drive
#========================================================================================
sub parse {
    my $self = shift;

    $self->readHeader();
    $self->drive();
    return;
}

#========================================================================================
# function     pushID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->pushID( $Key, $Value );
# example      $self->pushID( 'CompanyID', $CompanyID );
#----------------------------------------------------------------------------------------
# description  adds a parameter to the parameter stack. Any previous parameter with the
#               same name will be hidden until $self->popID( $Key ) is called.
#               Use $self->topID( $Key ) to get the parameter value.
#               Use $self->popID( $Key ) to remove the parameter from the stack.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# input        $Value | parameter value
#========================================================================================
sub pushID {
    my $self = shift;
    my( $Key, $Value ) = @_;

    $self->{'IDs'}->{$Key} = [] unless exists $self->{'IDs'}->{$Key};
    push @{$self->{'IDs'}->{$Key}}, $Value;
}

#========================================================================================
# function     popID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->popID( $Key );
# example      $CompanyID = $self->popID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  removes a parameter from the parameter stack and returns its value.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub popID {
    my $self = shift;
    my( $Key ) = @_;

    return pop @{$self->{'IDs'}->{$Key}};
}

#========================================================================================
# function     topID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->topID( $Key );
# example      $CompanyID = $self->topID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  returns the current value of a parameter
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub topID {
    my $self = shift;
    my( $Key ) = @_;

    return () unless exists $self->{'IDs'}->{$Key};
    return $self->{'IDs'}->{$Key}->[-1];
}

1;
