#==============================================================================
# package      DE_EPAGES::Core::API::FileInstaller
# base         DE_EPAGES::Core::API::BaseInstaller
# state        public
# description  install/patch/uninstall files from /Data/Private and /Data/Public
#==============================================================================
package DE_EPAGES::Core::API::FileInstaller;
use base DE_EPAGES::Core::API::BaseInstaller;

use strict;
use Config;
use Config::IniFiles;
use File::Basename  qw ( dirname );
use Sort::Versions qw(versioncmp);

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw ( CompareMD5File GetFileMD5Digest BackupFile CopyFile
                                    DeleteFile MoveFile ExistsFile WriteFile SetAccessRights
                                    MergeIniFiles CopyDirectory DeleteDirectory
                                    RenameDirectory GetInfoDirectory ExistsDirectory );
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Core::API::PerlTools qw (GetArchName);
use DE_EPAGES::Core::API::TestParameter qw (TestString);

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'FILE_OPEN_FAILED'      => 'couldn\'t open file (#FileName) because #Code.',
        'FILE_WRITE_FAILED'     => 'couldn\'t write file (#FileName) because #Code.',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Call functions <function copyPrivateFiles>, <function copyBinaryFiles>,
#               <function copySchedulerFiles>, <function copySharedConfigFiles>,
#               <function copyWebRootFiles>.
#========================================================================================
sub install ($$) {
    my $self = shift;
    $self->SUPER::install;

    my $Msg = sprintf('install files of %s on store %s', $self->cartridgeDirectory, $self->storeName);
    GetLog->debug($Msg);

    $self->copyPrivateFiles();
    $self->copyBinaryFiles();
    $self->copySchedulerFiles() unless $self->noScheduler;
    $self->copySharedConfigFiles() unless $self->noSharedConfig;
    $self->copyWebRootFiles();

    return;
}

#========================================================================================
# function     installStoreType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->installStoreType;
#----------------------------------------------------------------------------------------
# description  Call functions <function copyPublicFiles>, <function copyJavascriptFiles>.
#----------------------------------------------------------------------------------------
# input        %Options | option
#               <ul>
#                   <li>update - installStoreType even for cartridges that exist in
#                       StoreTypes.d/$Version/$StoreType.conf but not up-to-date.</li>
#               </ul>
#               | hash
#========================================================================================
sub installStoreType($$) {
    my $self = shift;
    my (%Options) = @_;

    my $StoreType = $self->storeType;
    my $Package   = $self->cartridgePackage;
    my $Version   = $self->storeTypeVersion;

    my $ConfigFile = "$ENV{EPAGES_CONFIG}/StoreTypes.d/$Version/$StoreType.conf";
    my $Content = "[Cartridges]\n";
    WriteFile($ConfigFile, \$Content, { create_directory=>1} ) unless ExistsFile($ConfigFile);

    tie my %StoreTypeConfig, 'Config::IniFiles', ( -file => $ConfigFile ) or
        Error('FILE_OPEN_FAILED',{'FileName' => $ConfigFile, 'Code' => $!});

    my $DoInstall = not exists $StoreTypeConfig{Cartridges}{$Package};
    if (not $DoInstall and $Options{update}) {
        # install if version in config file is less than cartridge version
        $DoInstall = versioncmp( $StoreTypeConfig{Cartridges}{$Package}, $self->version ) < 0;
    }
    return unless $DoInstall;

    $self->copyPublicFiles();
    $self->copyJavascriptFiles();

    $StoreTypeConfig{'Cartridges'}{$Package} = $self->version;
    tied( %StoreTypeConfig )->RewriteConfig() or
        Error('FILE_WRITE_FAILED',{ 'FileName' => $ConfigFile, 'Code' => $!});
    SetAccessRights($ConfigFile);

    return;
}

#========================================================================================
# function     uninstallStoreType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstallStoreType;
#----------------------------------------------------------------------------------------
# description  Rebuilds directories EPAGES_STORES/STORE_NAME and EPAGES_WEBROOT/STORE_NAME
#               from backup directories created by <function install>. Directories will be
#               rebuilt with <function DE_EPAGES::Core::API::File::RebuildDirectory>.
#========================================================================================
sub uninstallStoreType($$) {
    my $self = shift;

    my $StoreType = $self->storeType;
    my $Package   = $self->cartridgePackage;
    my $Version   = $self->storeTypeVersion;

    my $ConfigFile = "$ENV{'EPAGES_CONFIG'}/StoreTypes.d/$Version/$StoreType.conf";
    my $Content = "[Cartridges]\n";
    WriteFile($ConfigFile, \$Content, { create_directory=>1} ) unless ExistsFile($ConfigFile);

    tie my %StoreTypeConfig, 'Config::IniFiles', ( -file => $ConfigFile )
        or Error('FILE_OPEN_FAILED',{'FileName' => $ConfigFile, 'Code' => $!});

    return unless exists $StoreTypeConfig{'Cartridges'}{$Package};

    delete $StoreTypeConfig{'Cartridges'}{$Package};

    $self->deletePublicFiles;
    $self->deleteJavascriptFiles;

    tied( %StoreTypeConfig )->RewriteConfig() or
        Error('FILE_WRITE_FAILED',{'FileName' => $ConfigFile, 'Code' => $!});
    SetAccessRights($ConfigFile);

    return;
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Rebuilds directories EPAGES_STORES/STORE_NAME and EPAGES_WEBROOT/STORE_NAME
#               from backup directories created by <function install>. Directories will be
#               rebuilt with <function DE_EPAGES::Core::API::File::RebuildDirectory>.
#========================================================================================
sub uninstall ($$) {
    my $self = shift;

    my $Msg = sprintf('uninstall files of %s on store %s', $self->cartridgeDirectory, $self->storeName);
    GetLog->debug($Msg);

    # delete installed files
    $self->deletePrivateFiles;

    # don't delete binary files, WebRoot files and scheduler entries, because they may
    # still be used by other databases

    $self->SUPER::uninstall;
}

#========================================================================================
# function     deletePrivateFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deletePrivateFiles;
#----------------------------------------------------------------------------------------
# description  Deletes files that were installed by the cartridge to the directory
#               EPAGES_STORES/$Store.
#========================================================================================
sub deletePrivateFiles {
    my $self = shift;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Private');
    my $TargetDir = join ('/', $ENV{EPAGES_STORES}, $self->storeName);
    my $BackupSubDir = 'Shared/Stores/' . $self->storeName;
    $self->_deleteFiles( $SourceDir, $TargetDir, $BackupSubDir );
    return;
}

#========================================================================================
# function     deletePublicFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deletePublicFiles;
#----------------------------------------------------------------------------------------
# description  Deletes files that were installed by the cartridge to the directory
#               WebRoot/$Store.
#========================================================================================
sub deletePublicFiles {
    my $self = shift;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public');
    my $TargetDir = join ('/', $ENV{EPAGES_WEBROOT}, $self->storeTypeDir);
    my $BackupSubDir = 'Shared/WebRoot/' . $self->storeTypeDir;
    $self->_deleteFiles( $SourceDir, $TargetDir, $BackupSubDir );
    return;
}

#========================================================================================
# function     deleteJavascriptFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deleteJavascriptFiles;
#----------------------------------------------------------------------------------------
# description  Deletes files that were installed by the cartridge to the directory
#               WebRoot/StoreTypes/$EpagesVersion/$StoreType/javascript/cartridges/<cartridgeDirectory>.
#========================================================================================
sub deleteJavascriptFiles {
    my $self = shift;

    my $javascript_dir = join ('/', $self->storeTypeDir, 'javascript/epages/cartridges', lc ($self->cartridgeDirectory));
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/javascript');
    my $TargetDir = "$ENV{EPAGES_WEBROOT}/$javascript_dir";
    my $BackupDir = 'Shared/WebRoot/' . $javascript_dir;
    $self->_deleteFiles( $SourceDir, $TargetDir, $BackupDir );
    return;
}

#========================================================================================
# function     _deleteFiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->_deleteFiles($Source, $Target, $BackupSubDir)
# example      my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public');
#               my $TargetDir = join ('/', $ENV{EPAGES_WEBROOT}, $self->storeName);
#               my $BackupSubDir = 'Shared/WebRoot/' . $self->storeName;
#               $self->_deleteFiles( $SourceDir, $TargetDir, $BackupSubDir );
#----------------------------------------------------------------------------------------
# description  Deletes files that were installed by the cartridge.
#               Writes backup copies to
#----------------------------------------------------------------------------------------
# input        $Source | source dir | string
# input        $Target | target dir | string
# input        $BackupSubDir | (optional) backup dir relative to $ENV{EPAGES_BACKUP} | string
#========================================================================================
sub _deleteFiles {
    my $self = shift;
    my ($Source, $Target, $BackupSubDir) = @_;
    GetLog->debug("_deleteFiles($Source, $Target, $BackupSubDir)");

    return unless ExistsDirectory( $Source);

    my $Backup = $ENV{EPAGES_BACKUP_DIR} && $BackupSubDir ? "$ENV{EPAGES_BACKUP_DIR}/$BackupSubDir" : undef;

    # delete/backup files
    my $hDir = GetInfoDirectory( $Source, { recursive => 1 } );
    foreach my $SourceFile ( @{$hDir->{'Files'}} ) {
        my $TargetFile = $SourceFile;
        $TargetFile =~ s/^\Q$Source\E/$Target/;
        next unless ExistsFile($TargetFile);

        if( $Backup ) {
            my $BackupFile = $SourceFile;
            $BackupFile =~ s/^\Q$Source\E/$Backup/;
            MoveFile( $TargetFile, $BackupFile, { create_directory => 1, stat => 'preserve' } );
        }
        else {
            DeleteFile( $TargetFile );
        }
    }

    # delete empty directores
    my @SourceDirectories = ( $Source, @{$hDir->{'Directories'}} );
    foreach my $SourceDir ( reverse @SourceDirectories ) {
        my $TargetDir = $SourceDir;
        $TargetDir =~ s/^\Q$Source\E/$Target/;
        DeleteDirectory($TargetDir) if $self->_isDirectoryEmpty($TargetDir);
    }

    return;
}

#========================================================================================
# function     _isDirectoryEmpty
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->_isDirectoryEmpty( $DirPath );
#----------------------------------------------------------------------------------------
# description  Returns true if the directory exists and is empty.
#----------------------------------------------------------------------------------------
# input        $DirPath | directory name | string
# return       $DirIsEmpty | directory exists and is empty | boolean
#========================================================================================
sub _isDirectoryEmpty {
    my $self = shift;
    my ($DirPath) = @_;

    opendir my $DIR, $DirPath or return 0;
    my @SubItems = readdir $DIR;
    closedir $DIR or return 0;

    # empty directories contain entries: '.' and '..'
    return scalar @SubItems == 2 ? 1 : 0;
}

#========================================================================================
# function     copyPrivateFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyPrivateFiles();
#----------------------------------------------------------------------------------------
# example      $Installer->copyPrivateFiles();
#----------------------------------------------------------------------------------------
# description  Copies files from /Data/Private to EPAGES_STORES/STORE_NAME with backup
#               in EPAGES_BACKUP_DIR/Shared/Stores/STORE_NAME.
#               Directories will be copied with <function DE_EPAGES::Core::API::File::CopyDirectory>.
#               Called by function <function install>. If CopyDirectory() creates a file
#               "$BackupDir.inexistent", add it to EPAGES_BACKUP_DIR/Shared/Stores.inexistent.
#========================================================================================
sub copyPrivateFiles {
    my $self = shift;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Private');
    my $TargetDir = join ('/', $ENV{EPAGES_STORES}, $self->storeName);
    $self->copyDirectory($SourceDir, $TargetDir, 'Shared/Stores', $self->storeName, 1);
    return;
}

#========================================================================================
# function     copyPublicFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyPublicFiles();
#----------------------------------------------------------------------------------------
# description  Copies files from /Data/Public to EPAGES_WEBROOT/StoreTypes/EPAGES_VERSION/STORE_TYPE.
#               Directories will be copied with <function DE_EPAGES::Core::API::File::CopyDirectory>.
#               Called by function <function install>.
#========================================================================================
sub copyPublicFiles {
    my $self    = shift;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public');
    my $TargetDir = join ('/', $ENV{EPAGES_WEBROOT}, $self->storeTypeDir);

    #Do not backup because files are copied to a versioned directory.
    $self->copyDirectory($SourceDir, $TargetDir);
    return;
}

#========================================================================================
# function     copyPublicFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyPublicFile( $File );
# example      $Installer->copyPublicFile( 'Data/Public/SF/PrintScreen.css' );
#----------------------------------------------------------------------------------------
# description  Copies a file from /Data/Public to
#               EPAGES_WEBROOT/StoreTypes/EPAGES_VERSION/STORE_TYPE.
#----------------------------------------------------------------------------------------
# input        $File | file name | string
#========================================================================================
sub copyPublicFile {
    my $self = shift;
    my ($File) = @_;
    TestString( 'File', $File );

    my $FileName = $File;
    $FileName =~ s/^Data\/Public\///;

    my $SourceFile = join '/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public', $FileName;
    my $TargetFile = join '/', $ENV{EPAGES_WEBROOT}, $self->storeTypeDir, $FileName;
    CopyFile( $SourceFile, $TargetFile, { 'create_directory' => 1 } );
    return;
}

#========================================================================================
# function     copyBinaryFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyBinaryFiles();
#----------------------------------------------------------------------------------------
# example      $Installer->copyBinaryFiles();
#----------------------------------------------------------------------------------------
# description  Copies files from /bin/$ArchDir to EPAGES_PERL/bin with backup
#               in EPAGES_BACKUP_DIR/Perl/bin.
#               Directories will be copied with <function DE_EPAGES::Core::API::File::CopyDirectory>.
#               Called by function <function install>.
#========================================================================================
sub copyBinaryFiles {
    my $self = shift;

    my ($ArchDir, $ArchDir32) = GetArchName();
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'bin/');
    $ArchDir = $ArchDir32 unless -d "$SourceDir$ArchDir";
    $ArchDir = $Config{osname} unless -d "$SourceDir$ArchDir";
    $SourceDir = "$SourceDir$ArchDir";

    my $TargetDir = dirname($ENV{PERL});
    $self->copyDirectory($SourceDir, $TargetDir, 'Perl/bin');
    return;
}

#========================================================================================
# function     copySchedulerFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copySchedulerFiles();
#----------------------------------------------------------------------------------------
# example      $Installer->copySchedulerFiles();
#----------------------------------------------------------------------------------------
# description  Reads sections from /Data/Scheduler/Scheduler.conf and writes them to
#               EPAGES_CONFIG/Scheduler.conf (if section not exists)
#               Called by function <function install>
#========================================================================================
sub copySchedulerFiles {
    my $self = shift;

    # extend $ENV{EPAGES_CONFIG}/Scheduler.conf
    my $TargetIni = join ('/', $ENV{EPAGES_CONFIG}, 'Scheduler.conf');
    my $SourceIni = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Scheduler/Scheduler.conf');

    writeNewSections($SourceIni, $TargetIni) if ExistsFile($SourceIni);
    return;
}

#========================================================================================
# function     copySharedConfigFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copySharedConfigFiles();
#----------------------------------------------------------------------------------------
# example      $Installer->copySharedConfigFiles();
#----------------------------------------------------------------------------------------
# description  Reads sections from /Data/SharedConfig/*.conf and writes them to
#               EPAGES_CONFIG/*.conf (if section not exists)
#               Called by function <function install>
#========================================================================================
sub copySharedConfigFiles {
    my $self = shift;

    my $TargetDir = $ENV{EPAGES_CONFIG};
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/SharedConfig');

    return unless ExistsDirectory($SourceDir);
    my $hDir = GetInfoDirectory($SourceDir, {recursive => 1, relative => 1});

    foreach my $SourceFile ( @{$hDir->{'Files'}} ) {
        my $TargetFile = "$TargetDir/$SourceFile";
        if (not ExistsFile($TargetFile)) {
            CopyFile("$SourceDir/$SourceFile", $TargetFile, {'create_directory' => 1});
        }
        elsif (isIniFile("$SourceDir/$SourceFile")) {
            MergeIniFiles($TargetFile, "$SourceDir/$SourceFile", undef,
                          { FirstSections => [qw(LOOPS EXAMPLE)] });
        }
    }
    return;
}

#========================================================================================
# function     isIniFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       isIniFile($FileName)
#----------------------------------------------------------------------------------------
# example      isIniFile("$ENV{EPAGES_CARTRIDGES}/Core/Data/Scheduler/Scheduler.conf");
#               -> true
# example      isIniFile("$ENV{EPAGES_CONFIG}/log4perl.conf");
#               -> false
#----------------------------------------------------------------------------------------
# description  Checks if $FileName is ini file:
#               - must end in .conf
#               - Config::IniFiles->new() must be successfully
#----------------------------------------------------------------------------------------
# input        $FileName | file to check | string
# return       $IsIniFile | is ini file (true) or not (false) | boolean
#========================================================================================
sub isIniFile {
    my ($FileName) = @_;

    return 0 unless $FileName =~ /\.conf$/;

    if (tie my %FileHash, 'Config::IniFiles', ( -file => $FileName )) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     copyWebRootFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyWebRootFiles();
#----------------------------------------------------------------------------------------
# example      $Installer->copyWebRootFiles();
#----------------------------------------------------------------------------------------
# description  Copies files from /Data/WebRoot to EPAGES_WEBROOT with backup
#               in EPAGES_BACKUP_DIR/Shared/WebRoot.
#               Directories will be copied with <function DE_EPAGES::Core::API::File::CopyDirectory>.
#               Called by function <function install>.
#========================================================================================
sub copyWebRootFiles {
    my $self = shift;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/WebRoot');
    my $TargetDir = $ENV{EPAGES_WEBROOT};
    $self->copyDirectory($SourceDir, $TargetDir, 'Shared/WebRoot');
    return;
}

#========================================================================================
# function     onPatchDeletePrivateFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeletePrivateFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeletePrivateFile(
#                        'Data/Private/Templates/DE_EPAGES/Shop/Templates/MBO/MBO.UserBox.html',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeletePrivateFile
#               If <function DE_EPAGES::Core::API::File::CompareMD5File>($Target,$hParams->{'OldMD5Digest'}) returns true:
#               Deletes Private file at store directory, Moves BackupFile to TargetFile if BackupFile exists.
#               Else file will be ignored and GetLog->info(
#               "File '$Target' was not deleted because the original target file has been modified."
#               ) is called.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeletePrivateFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = $File;
    $FileName =~ s/^Data\/Private\///;

    my $Target = join ('/', $ENV{EPAGES_STORES}, $self->storeName, $FileName);
    my $Backup = join ('/', $ENV{EPAGES_BACKUP_DIR}, 'Shared/Stores', $self->storeName, $FileName);
    $self->deleteFileIfOriginal($Target, $Backup, $hParams->{'OldMD5Digest'});
    return;
}

#========================================================================================
# function     onPatchDeletePrivateDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeletePrivateDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeletePrivateDirectory(
#                        'Data/Private/Templates/DE_EPAGES/Shop/Templates/MBO/MBO.UserBox.html',
#                         {'CVS' => 'Removed',
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeletePrivateDirectory
#               Deletes Private Directory at store directory.
#----------------------------------------------------------------------------------------
# input        $Directory | cartridgeDirectoryname without cartridge directory | string
# input        $hParams | patch Directory parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeletePrivateDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    my $DirectoryName = $Directory;
    $DirectoryName =~ s/^Data\/Private\///;

    my $Target = join ('/', $ENV{EPAGES_STORES}, $self->storeName, $DirectoryName);
    my $Backup = join ('/', $ENV{EPAGES_BACKUP_DIR}, 'Shared/Stores', $self->storeName, $DirectoryName);
    $self->moveOrDeleteDirectory($Target, $Backup);
    return;
}

#========================================================================================
# function     onPatchDeletePublicFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeletePublicFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeletePublicFile(
#                        'Data/Public/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeletePublicFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchDeletePublicDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeletePublicDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeletePublicDirectory(
#                        'Data/Public/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Removed',
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $Directory | cartridgeDirectoryname without cartridge directory | string
# input        $hParams | patch Directory parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeletePublicDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchDeleteSchedulerFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteSchedulerFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteSchedulerFile(
#                        'Data/Scheduler/DoScheduler.sh',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeleteSchedulerFile
#               If <function DE_EPAGES::Core::API::File::CompareMD5File>($Target,$hParams->{'OldMD5Digest'}) returns true:
#               Deletes Scheduler file at store directory, Moves BackupFile to TargetFile if BackupFile exists.
#               Else file will be ignored and GetLog->info(
#               "File '$Target' was not deleted because the original target file has been modified."
#               ) is called.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteSchedulerFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    # do nothing
    return;
}

#========================================================================================
# function     onPatchDeleteSharedConfigFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteSharedConfigFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteSharedConfigFile('Data/SharedConfig/WebInterface.conf');
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeleteSharedConfigFile
#               Deletes Shared/Config file
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
#========================================================================================
sub onPatchDeleteSharedConfigFile {
    my $self = shift;
    my ( $File ) = @_;

    # do nothing
    return;
}

#========================================================================================
# function     onPatchCopyPrivateFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopyPrivateFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopyPrivateFile(
#                        'Data/Private/Templates/DE_EPAGES/Shop/Templates/MBO/MBO.UserBox.html',
#                         {'CVS' => 'Changed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by Method onPatchCopyPrivateFile.
#               If <function DE_EPAGES::Core::API::File::CompareMD5File>($Target,$hParams->{'OldMD5Digest'}) returns true:
#               Copies private file to store directory.
#               Else file will be ignored and GetLog->info(
#               "'$Source' not copied. File '$Target' was not overwritten  because the original target file has been modified."
#               ) is called.
#               If the file is new or not defined $hParams->{'OldMD5Digest'}:
#               Backups file <function DE_EPAGES::Core::API::File::BackupFile>( $Source, $Target, $Backup )
#               Copies file <function DE_EPAGES::Core::API::File::CopyFile>( $Source, $Target, { create_directory => 1 } )
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                      (only for files with CVS= 'Changed')
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchCopyPrivateFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = $File;
    $FileName =~ s/^Data\/Private\///;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Private');
    my $TargetDir = join ('/', $ENV{EPAGES_STORES}, $self->storeName);
    my $BackupDir = join ('/', $ENV{EPAGES_BACKUP_DIR}, 'Shared/Stores', $self->storeName);
    $self->copyFileIfOriginal($FileName, $SourceDir, $TargetDir, $BackupDir, $hParams->{'OldMD5Digest'});
    return;
}

#========================================================================================
# function     onPatchCopyPublicFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopyPublicFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopyPublicFile(
#                         'Data/Public/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Changed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                      (only for files with CVS= 'Changed')
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchCopyPublicFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchCopyPublicDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopyPublicDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopyPublicDirectory(
#                         'Data/Public/BO',
#                         {'CVS' => 'Changed'
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $Directory | relative cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchCopyPublicDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchCopySchedulerFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopySchedulerFile($File);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopySchedulerFile('Data/Scheduler/Scheduler.conf');
#----------------------------------------------------------------------------------------
# description  Reads sections from /Data/Scheduler/Scheduler.conf and writes them to
#               EPAGES_CONFIG/Scheduler.conf (if section not exists)
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
#========================================================================================
sub onPatchCopySchedulerFile {
    my $self = shift;
    my ( $File ) = @_;

    return if $self->noScheduler;

    # for backward compatibility
    $File = 'Data/Scheduler/Scheduler.conf' unless defined $File;

    # extend $ENV{EPAGES_CONFIG}/Scheduler.conf
    my $TargetIni = join ('/', $ENV{EPAGES_CONFIG}, 'Scheduler.conf');
    my $SourceIni = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, $File);

    writeNewSections($SourceIni, $TargetIni) if ExistsFile($SourceIni);

    return;
}

#========================================================================================
# function     onPatchCopySharedConfigFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopySharedConfigFile($File);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopySharedConfigFile('Data/SharedConfig/WebInterface.conf');
#----------------------------------------------------------------------------------------
# description  Reads sections from /Data/SharedConfig/SharedConfig.conf and writes them to
#               EPAGES_CONFIG/SharedConfig.conf (if section not exists)
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
#========================================================================================
sub onPatchCopySharedConfigFile {
    my $self = shift;
    my ( $File ) = @_;

    return if $self->noSharedConfig;

    my $TargetFile = $File;
    $TargetFile =~ s,Data/SharedConfig/,,;
    $TargetFile = join ('/', $ENV{EPAGES_CONFIG}, $TargetFile);
    my $SourceFile = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, $File);

    return unless ExistsFile($SourceFile);

    if (not ExistsFile($TargetFile)) {
        CopyFile($SourceFile, $TargetFile, {'create_directory' => 1});
    }
    elsif ($SourceFile =~ /\.conf$/) {
        writeNewSections($SourceFile, $TargetFile);
    }
    return;
}

#========================================================================================
# function     onPatchDeleteCartridgeFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteCartridgeFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteCartridgeFile(
#                         'Data/Public/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by Method onPatchDeleteCartridgeFile.
#               Deletes the cartridge file $File.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteCartridgeFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, $File);
    DeleteFile($FileName);
    return;
}

#========================================================================================
# function     onPatchDeleteCartridgeDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteCartridgeDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteCartridgeDirectory(
#                         'Data/Public/BO/icons',
#                         {'CVS' => 'Removed',
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by Method onPatchDeleteCartridgeDirectory.
#               Deletes the cartridge directory $Directory.
#----------------------------------------------------------------------------------------
# input        $Directory | cartridgeDirectoryName without cartridge directory | string
# input        $hParams | patch Directory parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteCartridgeDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    my $DirectoryName = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory,  $Directory);
    DeleteDirectory($DirectoryName, { recursive=>1 });
    return;
}

#========================================================================================
# function     onPatchCopyWebRootFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopyWebRootFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopyWebRootFile(
#                         'Data/WebRoot/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Changed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by Method onPatchCopyWebRootFile
#               If <function DE_EPAGES::Core::API::File::CompareMD5File>($Target,$hParams->{'OldMD5Digest'}) returns true:
#               Copies WebRoot file to $ENV{'EPAGES_WEBROOT'} directory.
#               Else file will be ignored and GetLog->info(
#               "'$Source' not copied. File '$Target' was not overwritten  because the original target file has been modified."
#               ) is called.
#               If the file is new or not defined $hParams->{'OldMD5Digest'}:
#               Backups file <function DE_EPAGES::Core::API::File::BackupFile>( $Source, $Target, $Backup ).
#               Copies file <function DE_EPAGES::Core::API::File::CopyFile>( $Source, $Target, { create_directory => 1 } ).
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                      (only for files with CVS= 'Changed')
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchCopyWebRootFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = $File;
    $FileName =~ s/^Data\/WebRoot\///;

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/WebRoot');
    my $TargetDir = $ENV{EPAGES_WEBROOT};
    my $BackupDir = join ('/', $ENV{EPAGES_BACKUP_DIR}, 'Shared/WebRoot');
    $self->copyFileIfOriginal($FileName, $SourceDir, $TargetDir, $BackupDir, $hParams->{'OldMD5Digest'});
    return;
}

#========================================================================================
# function     onPatchDeleteWebRootFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteWebRootFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteWebRootFile(
#                        'Data/WebRoot/BO/icons/context_ico_s_helpbook.gif',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  Called by PatchImportHandler by method onPatchDeleteWebRootFile.
#               If <function DE_EPAGES::Core::API::File::CompareMD5File>($Target,$hParams->{'OldMD5Digest'}) returns true:
#               Deletes WebRoot file at $ENV{EPAGES_WEBROOT} directory, Moves BackupFile to TargetFile if BackupFile exists.
#               Else file will be ignored and GetLog->info(
#               "File '$Target' was not deleted because the original target file has been modified."
#               ) is called.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteWebRootFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    my $FileName = $File;
    $FileName =~ s/^Data\/WebRoot\///;

    my $Target = join ('/', $ENV{EPAGES_WEBROOT}, $FileName);
    my $Backup = join ('/', $ENV{EPAGES_BACKUP_DIR}, 'Shared/WebRoot', $FileName);
    $self->deleteFileIfOriginal($Target, $Backup, $hParams->{'OldMD5Digest'});
    return;
}

#========================================================================================
# function     onPatchDeleteWebRootDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteWebRootDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteWebRootDirectory(
#                         'Data/WebRoot/Doc/Help/de/ClickAndBuy',
#                         {'CVS' => 'Removed',
#                         });
#----------------------------------------------------------------------------------------
# description  Deletes the directory $Directory in EPAGES_WEBROOT.
#----------------------------------------------------------------------------------------
# input        $Directory | cartridgeDirectoryName without cartridge directory | string
# input        $hParams | patch Directory parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteWebRootDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    my $DirectoryName = $Directory;
    $DirectoryName =~ s/^Data\/WebRoot\///;

    my $Target = "$ENV{EPAGES_WEBROOT}/$DirectoryName";
    my $Backup = "$ENV{EPAGES_BACKUP_DIR}/Shared/WebRoot/$DirectoryName";
    $self->moveOrDeleteDirectory($Target, $Backup);
    return;
}

#========================================================================================
# function     copyJavascriptFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyJavascriptFiles;
#----------------------------------------------------------------------------------------
# description  Copies files from /Data/javascript to
#               EPAGES_WEBROOT/StoreTypes/EPAGES_VERSION/STORE_TYPE/javascript/cartridges/<cartridgeDirectory>
#               with backup in EPAGES_WEBROOT/EPAGES_VERSION/STORE_TYPE/javascript/cartridges/<cartridgeDirectory>.
#               Directories will be copied with <function DE_EPAGES::Core::API::File::CopyDirectory>.
#               Called by function <function install>. If CopyDirectory() creates a file
#               "$BackupDir.inexistent", add it to EPAGES_BACKUP_DIR/Shared/WebRoot.inexistent.
#========================================================================================
sub copyJavascriptFiles {
    my $self    = shift;

    # copy javascript files
    my $javascript_dir = join ('/', $self->storeTypeDir, 'javascript/epages/cartridges', lc ($self->cartridgeDirectory));

    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/javascript');
    my $TargetDir = "$ENV{EPAGES_WEBROOT}/$javascript_dir";
    $self->copyDirectory($SourceDir, $TargetDir, 'Shared/WebRoot', $javascript_dir, 1);
    return;
}

#========================================================================================
# function     copyJavascriptFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyJavascriptFile( $File );
# example      $Installer->copyJavascriptFile( 'Data/javascript/dragdrop.js' );
#----------------------------------------------------------------------------------------
# description  Copies a file from /Data/javascript to
#               EPAGES_WEBROOT/StoreTypes/EPAGES_VERSION/STORE_TYPE/javascript/cartridges/<cartridgeDirectory>
#----------------------------------------------------------------------------------------
# input        $File | file name | string
#========================================================================================
sub copyJavascriptFile {
    my $self = shift;
    my ($File) = @_;
    TestString( 'File', $File );

    my $FileName = $File;
    $FileName =~ s/^Data\/javascript\///;

    my $StoreTypeDir = $ENV{EPAGES_WEBROOT} .'/'. $self->storeTypeDir,
    my $SourceFile = join '/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/javascript', $FileName;
    my $TargetFile = "$StoreTypeDir/javascript/epages/cartridges/" . lc($self->cartridgeDirectory) .'/'. $FileName;
    CopyFile( $SourceFile, $TargetFile, { 'create_directory' => 1 } );

    # copy to 'build' directory
    $TargetFile = "$StoreTypeDir/javascript/build/epages/cartridges/" . lc($self->cartridgeDirectory) .'/'. $FileName;
    CopyFile( $SourceFile, $TargetFile, { 'create_directory' => 1 } );
    return;
}

#========================================================================================
# function     onPatchCopyJavascriptFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchCopyJavascriptFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchCopyJavascriptFile(
#                         'Data/javascript/widget/Mywidget.js',
#                         {'CVS' => 'Changed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could be New or Changed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                      (only for files with CVS= 'Changed')
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchCopyJavascriptFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchDeleteJavascriptFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteJavascriptFile($File, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteJavascriptFile(
#                        'Data/javascript/widget/Mywidget.js',
#                         {'CVS' => 'Removed',
#                          'OldMD5Digest' => '4c832c34a88de899dab3d00b7ccca632'
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $File | cartridgefilename without cartridge directory | string
# input        $hParams | patch file parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#                   <li>OldMD5Digest - md5 hex digest of the old release of the file
#                                    - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteJavascriptFile {
    my $self = shift;
    my ( $File, $hParams ) = @_;

    return;
}

#========================================================================================
# function     onPatchDeleteJavascriptDirectory
# state        Javascript
#----------------------------------------------------------------------------------------
# syntax       $Installer->onPatchDeleteJavascriptDirectory($Directory, $hParams);
#----------------------------------------------------------------------------------------
# example      $Installer->onPatchDeleteJavascriptDirectory(
#                        'Data/javascript/widget',
#                         {'CVS' => 'Removed',
#                         });
#----------------------------------------------------------------------------------------
# description  obsolete. does nothing.
#----------------------------------------------------------------------------------------
# input        $Directory | cartridgeDirectoryname without cartridge directory | string
# input        $hParams | patch Directory parameters, keys are
#               <ul>
#                   <li>CVS          - could only be Removed - string
#               </ul> | ref.hash.*
#========================================================================================
sub onPatchDeleteJavascriptDirectory {
    my $self = shift;
    my ( $Directory, $hParams ) = @_;

    return;
}

#========================================================================================
# function     writeNewSections
# state        private
#----------------------------------------------------------------------------------------
# syntax       writeNewSections($NewIniConfig, $ExistingIniConfig);
#----------------------------------------------------------------------------------------
# example      writeNewSections("$ENV{EPAGES_CARTRIDGES}/Core/Data/Scheduler/Scheduler.conf",
#                                "$ENV{EPAGES_CONFIG}/Scheduler.conf");
#----------------------------------------------------------------------------------------
# description  Reads sections from $NewIniConfig and writes them to $ExistingIniConfig
#               (if section not exists)
#----------------------------------------------------------------------------------------
# input        $NewIniFile | new configuration file | string
# input        $ExistingIniFile | existing configuration file | string
#========================================================================================
sub writeNewSections {
    my ($NewIniFile, $ExistingIniFile) = @_;

    MergeIniFiles($ExistingIniFile, $NewIniFile, [ 'LOOPS' ],
                  { delcomments => '#', FirstSections => [qw(LOOPS EXAMPLE)] });
    return;
}

#========================================================================================
# function     copyFileIfOriginal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyFileIfOriginal($File, $Source, $Target, $Backup, $OldMD5Digest)
#----------------------------------------------------------------------------------------
# description  Copies file $File from $Source to $Target unless target was modified,
#               i.e. does not match the specified MD5 digest.
#               When the file is copied, a backup file is stored in directory $Backup.
#----------------------------------------------------------------------------------------
# input        $File  | source file | string
# input        $Source | source dir | string
# input        $Target | target dir | string
# input        $Backup | backup dir | string
# input        $OldMD5Digest | (optional) MD5 of file | string
# return       $DoCopyFile | copy file or not | boolean
#========================================================================================
sub copyFileIfOriginal {
    my $self = shift;
    my ($File, $Source, $Target, $Backup, $OldMD5Digest) = @_;
    GetLog->debug("copyFileIfOriginal($File, $Source, $Target, $Backup)");

    my $FileExists = ExistsFile("$Target/$File") ? 1 : 0;

    if (defined $OldMD5Digest and $FileExists) {
        if (not CompareMD5File("$Target/$File", $OldMD5Digest) and
            not CompareMD5File("$Target/$File", GetFileMD5Digest("$Source/$File"))) {
            GetLog->warn("'$Source/$File' not copied. File '$Target/$File' was not overwritten because the original target file has been modified.");
            return 0;
        }
    }

    if ($ENV{EPAGES_BACKUP_DIR}) {
        GetLog->debug("BackupFile($Source/$File, $Target/$File, $Backup/$File)");
        BackupFile("$Source/$File", "$Target/$File", "$Backup/$File", { stat => 'preserve' });
        if (not $FileExists) {
            my $Content = "$File\n";
            WriteFile("$Backup.inexistent", \$Content, { append=>1, create_directory=>1 } );
        }
    }

    GetLog->debug("CopyFile( $Source/$File, $Target/$File)");
    CopyFile( "$Source/$File", "$Target/$File", { create_directory => 1 } );
    return 1;
}

#========================================================================================
# function     deleteFileIfOriginal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->deleteFileIfOriginal($Target, $Backup, $OldMD5Digest)
#----------------------------------------------------------------------------------------
# description  Deletes file $Target unless target was modified,
#               i.e. does not match the specified MD5 digest.
#               When the file is copied, a backup file is stored in $Backup.
#----------------------------------------------------------------------------------------
# input        $Target | file to delete | string
# input        $Backup | backup file | string
# input        $OldMD5Digest | OldMD5Digest - md5 hex digest of the old release of the file (optional) | string
#========================================================================================
sub deleteFileIfOriginal {
    my $self = shift;
    my ($Target, $Backup, $OldMD5Digest) = @_;
    return unless ExistsFile($Target);
    GetLog->debug("deleteFileIfOriginal($Target, $Backup)");

    if (defined $OldMD5Digest and not CompareMD5File($Target, $OldMD5Digest)) {
        GetLog->warn("File '$Target' was not deleted because the original target file has been modified.");
        return;
    }

    if ($ENV{EPAGES_BACKUP_DIR}) {
        GetLog->debug("MoveFile($Target, $Backup)");
        MoveFile($Target, $Backup, { create_directory => 1, stat => 'preserve' });
    } else {
        GetLog->debug("DeleteFile($Target)");
        DeleteFile($Target);
    }
    return;
}

#========================================================================================
# function     copyDirectory
# state        pubilc
#----------------------------------------------------------------------------------------
# syntax       $Installer->copyDirectory($Source, $Target, $Backup, $Backup2, $Inexistent)
# example      my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $self->cartridgeDirectory, 'Data/Public');
#               my $TargetDir = join ('/', $ENV{EPAGES_WEBROOT}, $self->storeName);
#               $self->copyDirectory($SourceDir, $TargetDir, "Shared/WebRoot", $self->storeName, 1);
#----------------------------------------------------------------------------------------
# description  Runs CopyDirectory($Source, $Target, $Backup).
#               If $Inexistent, run additionally WriteFile($Inexistent, "$Backup.inexistent")
#----------------------------------------------------------------------------------------
# input        $Source | source dir | string
# input        $Target | target dir | string
# input        $Backup | backup dir | string
# input        $Backup2 | directory inside backup dir | string
# input        $Inexistent | .inexistent file | string
#========================================================================================
sub copyDirectory {
    my $self = shift;
    my ($Source, $Target, $Backup1, $Backup2, $Inexistent) = @_;
    GetLog->debug("copyDirectory($Source, $Target, $Backup1, $Backup2, $Inexistent)");

    my $Backup = $ENV{EPAGES_BACKUP_DIR} && $Backup1 ? "$ENV{EPAGES_BACKUP_DIR}/$Backup1" : undef;
    $Backup .= "/$Backup2" if $Backup && $Backup2;

    if (-d $Source) {
        GetLog->debug("CopyDirectory($Source, $Target, $Backup)");
        CopyDirectory($Source, $Target, $Backup, { bakstat => 'preserve' } );
    }

    if ($Inexistent && $Backup && -s "$Backup.inexistent") {
        my $Content = "$Backup2.inexistent\n";
        WriteFile("$ENV{EPAGES_BACKUP_DIR}/$Backup1.inexistent", \$Content, { append=>1, create_directory=>1 } );
    }
    return;
}

#========================================================================================
# function     moveOrDeleteDirectory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->moveOrDeleteDirectory($Target, $Backup)
#----------------------------------------------------------------------------------------
# description  If $Backup, moves $Target to $Backup, else removes $Target.
#----------------------------------------------------------------------------------------
# input        $Target | file to delete | string
# input        $Backup | backup file | string
#========================================================================================
sub moveOrDeleteDirectory {
    my $self = shift;
    my ($Target, $Backup) = @_;
    return unless ExistsDirectory($Target);
    GetLog->debug("moveOrDeleteDirectory($Target, $Backup)");

    if ($ENV{EPAGES_BACKUP_DIR}) {
        GetLog->debug("RenameDirectory($Target, $Backup)");
        if (-d $Backup) {
            CopyDirectory($Target, $Backup, undef, { stat => 'preserve' } );
            DeleteDirectory($Target, { recursive=>1 } );
        } else {
            RenameDirectory($Target, $Backup, { create_directory => 1, stat => 'preserve' });
        }
    } else {
        GetLog->debug("DeleteDirectory($Target)");
        DeleteDirectory($Target, { recursive=>1 } );
    }
    return;
}

1;
