#========================================================================================
# package      DE_EPAGES::Core::API::IniConfig
#----------------------------------------------------------------------------------------
# description  Module for working with Windows-style .ini files.
# example      my $IniConfig = DE_EPAGES::Core::API::IniConfig->new(
#                   FileName => 'c:/test/test.ini',
#                   CaseSensitive => 1,
#               );
#               my $Value = $IniConfig->get( 'Section 1', 'Key 1' );
#               $IniConfig->set( 'Section1', 'key2', 'value2' );
#               $IniConfig->rewrite();
#
#               An ini file has the following structure:
#               [Section 1]
#               Key 1=value1
#               key2=value2
#
#               private attributes ( for use inside this module only ):
#               {FileName}  - name of the config file
#               {CaseSensitive}    - flag set if keys and sections are case insensitive
#               {Values}    - hash of sections
#               $self->{Values}->{$Section} is a hash of key-value pairs
#               $self->{Values}->{$Section}->{$key} is a single value
#========================================================================================
package DE_EPAGES::Core::API::IniConfig;

use strict;
use Config::IniFiles;
use File::Basename  qw ( dirname );
use Sort::Naturally qw(nsort);
use DE_EPAGES::Core::API::Error qw( Error VarsOnError );
use DE_EPAGES::Core::API::File qw( CreateDirectory ExistsFile SetAccessRights OpenFile LockFile );

# Hack to allow empty values, because otherwise Ini::Config doesn't write them to the file
*Config::IniFiles::_org_OutputParam = \&Config::IniFiles::_OutputParam; ## no critic qw(ProtectPrivateVars)
no warnings 'redefine'; ## no critic qw(ProhibitNoWarnings)
*Config::IniFiles::_OutputParam = sub { ## no critic qw(ProtectPrivateVars)
    my ($self, $sect, $parm, $val, $ors, $end_comment, $output_cb) = @_;
    $val = [ '' ] if 0 == scalar @$val;
    return Config::IniFiles::_org_OutputParam($self, $sect, $parm, $val, $ors, $end_comment, $output_cb); ## no critic qw(ProtectPrivateSubs)
};
use warnings 'redefine';

## no critic qw(ProhibitInterpolationOfLiterals)
sub ErrorDefinition {
    return {
        NoParameter => 'The parameter #Parameter is missing.',
        NoFile      => 'The file #FileName does not exist.',
        OpenRead    => "Can't read file #FileName: #OSError.",
        OpenWrite   => "Can't write file #FileName: #OSError.",
    };
}
## use critic

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig = DE_EPAGES::Core::API::IniConfig->new(
#                   FileName => $FileName,
#                   CaseSensitive => $CaseSensitive,
#                   Create => $Create,
#               );
# example      $IniConfig = DE_EPAGES::Core::API::IniConfig->new(
#                   FileName => 'win.ini'
#                   CaseSensitive => 1,
#               );
#----------------------------------------------------------------------------------------
# description  Constructs a new config file object.
#               By default, key and section names are case insensitive.
#----------------------------------------------------------------------------------------
# input    %options | parameters for DE_EPAGES::Core::API::IniConfig->new.
#                      The following parameters can be used:
#                       <ul>
#                           <li>FileName - config file name</li>
#                           <li>CaseSensitive - if true, key and section are case sensitive</li>
#                           <li>Create - if true, returns no error if file doesn't exist</li>
#                           <li>FirstSections - list of section names to write before other sections - ref.array
#                       </ul>
#                    | hash ref
# return   $IniConfig | an IniConfig object | object
#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;
    $options{'Create'} //= 0;
    $options{'CaseSensitive'} //= 0;
    $options{'FirstSections'} //= [];

    my $fileName = delete $options{FileName};

    Error( NoParameter => { 'Parameter' => 'FileName' } ) unless  $fileName;
    Error( NoFile => { FileName => $fileName } ) unless $options{'Create'} or -e $fileName;

    my $self = bless { %options }, $class;
    $self->{'ini'} = Config::IniFiles->new( -nocase => $options{'CaseSensitive'} ? 0 : 1,
                                            -allowempty => 1 );
    $self->fileName($fileName) or die "Can't set file name: $fileName";
    $self->{'FirstSections'} = $options{'FirstSections'};

    if( ExistsFile( $fileName) ) {
        $self->read( $fileName );
    }

    return $self;
}

#========================================================================================
# function     lock
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig = DE_EPAGES::Core::API::IniConfig->lock(
#                   FileName => $FileName,
#                   CaseSensitive => $CaseSensitive,
#                   Create => $Create,
#                   Access => $Access,
#                   Sub => $Sub,
#               );
# example      DE_EPAGES::Core::API::IniConfig->lock(
#                   FileName => 'win.ini',
#                   CaseSensitive => 1,
#                   Sub => sub {
#                       my ($IniConfig) = @_;
#                       print $IniConfig->get('Mail', 'MAPI');
#                   },
#               );
#----------------------------------------------------------------------------------------
# description  Constructs a new config file object, locks the file and executes a
#               function before the lock is released.
#               By default, key and section names are case insensitive.
#----------------------------------------------------------------------------------------
# input    %options | parameters for DE_EPAGES::Core::API::IniConfig->new.
#                      The following parameters can be used:
#                       <ul>
#                           <li>FileName - config file name</li>
#                           <li>CaseSensitive - if true, key and section are case sensitive</li>
#                           <li>Create - if true, returns no error if file doesn't exist</li>
#                           <li>Sub - function to execute during the lock. The function
#                               gets an IniFile object as first parameter - code ref</li>
#                           <li>Access - requested access ('r'=read, 'w'=write) - string</li>
#                           <li>FirstSections - list of section names to write before other sections - ref.array
#                       </ul>
#                    | hash ref
#========================================================================================
sub lock {
    my $class = shift;
    my %options = @_;
    $options{'Create'} //= 0;
    $options{'CaseSensitive'} //= 0;
    $options{'FirstSections'} //= [];

    my $Access = delete $options{'Access'} or Error( NoParameter => { 'Parameter' => 'Access' } );
    Error( InvalidAccess => { Access => $Access } ) unless $Access =~ /^r|w$/;
    my $Sub = delete $options{'Sub'} or Error( NoParameter => { 'Parameter' => 'Sub' } );
    my $FileName = delete $options{FileName} or Error( NoParameter => { 'Parameter' => 'FileName' } );
    my $Exists = ExistsFile( $FileName );

    Error( NoFile => { FileName => $FileName } ) unless ($Exists || $options{'Create'});

    my $self = bless { %options, IsLocked => 1 }, $class;

    CreateDirectory( dirname( $FileName ), {'recursive' => 1 } ) unless $Exists;

    my $Mode = $Access eq 'r' ? '<' : $Exists ? '+<' : '>';
    OpenFile( $Mode, $FileName, sub {
        my ($FileHandle) = @_;
        eval {
            LockFile( $FileHandle, $Access, sub {
                $self->{'ini'} = Config::IniFiles->new(
                    -nocase => $options{'CaseSensitive'} ? 0 : 1,
                    -file => $FileHandle,
                    -allowempty => 1
                ) or die join("\n", @Config::IniFiles::errors); ## no critic qw(ProhibitPackageVars)

                $self->fileName($FileName) or die "Can't set file name: $FileName";

                $Sub->( $self );

                if ($self->{'Rewrite'}) {
                     truncate $FileHandle, 0 or die "Can't truncate $FileName: $!";
                     $self->{'FirstSections'} = $options{'FirstSections'};
                     print $FileHandle $self->sorted_content;
                }
            });
        };
        VarsOnError({ 'IniFileName' => $FileName });
    });

    SetAccessRights( $FileName, {} ) if $self->{'Rewrite'};

    return;
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig = DE_EPAGES::Core::API::IniConfig->create(
#                   FileName => $FileName,
#                   CaseSensitive => $CaseSensitive,
#               );
# example      $IniConfig = DE_EPAGES::Core::API::IniConfig->create(
#                   FileName => 'win.ini'
#               );
#----------------------------------------------------------------------------------------
# description  Creates a new config file object. The only difference
#               between this function and "new" is that "create" does not
#               require that the configuration file already exists.
#               If a file with the same name exists, it will be overwritten.
#----------------------------------------------------------------------------------------
# input        @Params | same parameters as for new | array
# return       $IniConfig | an IniConfig object | object
#========================================================================================
sub create {
    my $class = shift;
    my @Params = @_;

    return $class->new( @Params, Create => 1 );
}

#========================================================================================
# function     fileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->fileName( $FileName );
# syntax       $FileName = $IniConfig->fileName;
#----------------------------------------------------------------------------------------
# description  Sets or returns the config file name.
#----------------------------------------------------------------------------------------
# input        $FileName | config file name | string
# return       $FileName | config file name | string
#========================================================================================
sub fileName {
    my $self = shift;
    my ($fileName) = @_;

    if( $self->{'IsLocked'} ) {
        if( $fileName ) {
            return $self->{'FileName'} = $fileName;
        }
        else {
            return $self->{'FileName'};
        }
    }
    else {
        if( $fileName ) {
            return $self->{'ini'}->SetFileName($fileName);
        }
        else {
            return $self->{'ini'}->GetFileName();
        }
    }
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->get( $Section, $Key );
# syntax       $IniConfig->get( $Section, $Key, $Default );
# example      $IniConfig->get( 'Database', 'User', 'sa' );
#----------------------------------------------------------------------------------------
# description  Returns a value from the config file or $Default if the given
#               key was not found.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Key | key name | string
# input        $Default | (optional) default value, used if the key does not exist | string
# return       $Value | configuration value | string
#========================================================================================
sub get {
    my $self = shift;
    my( $Section, $Key, $Default ) = @_;

    return $self->{'ini'}->val( $Section, $Key, $Default ) // undef;
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->set( $Section, $Key, $Value );
# example      $IniConfig->set( 'Database', 'User', 'sa' );
#----------------------------------------------------------------------------------------
# description  Sets a value in the config file. Call $INI->rewrite to make the
#               changes persistent.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Key | key name | string
# input        $Value | configuration value | string
#========================================================================================
sub set {
    my $self = shift;
    my( $Section, $Key, $Value ) = @_;
    $Value //= '';

    $self->{'ini'}->newval($Section, $Key, $Value)
        or die "Error adding value '$Key=$Value' in section '$Section'";
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->delete( $Section, $Key );
# example      $IniConfig->delete( 'Database', 'User' );
#----------------------------------------------------------------------------------------
# description  Removes the value form the section in the config file. Call
#               $IniConfig->rewrite to make the changes persistent.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Key | key name | string
#========================================================================================
sub delete {
    my $self = shift;
    my( $Section, $Key ) = @_;

    $self->{'ini'}->delval( $Section, $Key );
    return;
}

#========================================================================================
# function     existsSection
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $IniConfig->existsSection($Section);
#----------------------------------------------------------------------------------------
# description  Returns true if the section exists in the ini file.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# return       $SectionExists | 1 if $Section, else 0 | boolean
#========================================================================================
sub existsSection {
    my $self = shift;
    my ($Section) = @_;

    return $self->{'ini'}->SectionExists($Section);
}

#========================================================================================
# function     addSection
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->addSection($Section);
#----------------------------------------------------------------------------------------
# description  Adds a section to the ini file if the section not exists.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# return       $Section | section name | string
#========================================================================================
sub addSection {
    my $self = shift;
    my ($Section) = @_;

    $self->{'ini'}->AddSection($Section) unless $self->existsSection($Section);
    return $Section;
}

#========================================================================================
# function     sectionNames
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSections = $IniConfig->sectionNames
#----------------------------------------------------------------------------------------
# description  Returns all section names found in the config file.
#----------------------------------------------------------------------------------------
# return       $aSections | section names | array ref
#========================================================================================
sub sectionNames {
    my $self = shift;

    my @sections = $self->{'ini'}->Sections();
    return \@sections;
}

#========================================================================================
# function     section
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSection = $IniConfig->section( $Section );
# syntax       $hSection = $IniConfig->section( $Section, $hSection );
# example      $hDatabase = $IniConfig->section( 'Database' );
#----------------------------------------------------------------------------------------
# description  Returns the specified section with key-value pairs as a hash reference
#               or creates a section with the new hash.
#               Returns an empty hash reference if the section does not exist.
#                If 'CaseSensitive' was set to 1, all keys are returned in lowercase
#                letters!
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $hSection | key-value pairs (optional) | hash ref
# return       $hSection | key-value pairs | hash ref
#========================================================================================
sub section {
    my $self = shift;
    my( $Section, $hSection ) = @_;

    if( defined $hSection ) {
        # delete all old values
        my $hOldSection = $self->section( $Section );
        $self->delete($Section, $_) foreach grep { !exists $hSection->{$_} } keys %$hOldSection;

        # set a section, make a copy of the hash
        $self->set($Section, $_, $hSection->{$_} ) foreach keys %$hSection;
    }

    # get a section
    my @parameters = $self->{'ini'}->Parameters($Section)  ;
    my %paramValues;
    foreach my $param(@parameters) {
        # If user wants case insensitive, we lowercase all keys.
        my $_param = $self->{'CaseSensitive'} ? $param : lc $param;
        $paramValues{$_param} = $self->{'ini'}->val($Section, $param);
    }

    return \%paramValues;
}

#========================================================================================
# function     valueComment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Comment = $IniConfig->sectionComment( $Section );
# syntax       $IniConfig->sectionComment( $Section, $Comment );
#----------------------------------------------------------------------------------------
# description  Sets or gets the comment of a section.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Comment | value comment | string
# return       $Comment | value comment | string
#========================================================================================
sub sectionComment { ## no critic qw(RequireArgUnpacking)
    my $self = shift;
    my( $Section, $Comment ) = @_;

    if( scalar @_ >= 2 ) {
        $self->{'ini'}->SetSectionComment($Section, $Comment);
    }
    else {
        return $self->{'ini'}->GetSectionComment($Section);
    }
}

#========================================================================================
# function     deleteSectionComment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->deleteSectionComment( $Section );
#----------------------------------------------------------------------------------------
# description  Removes the comment from a section.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
#========================================================================================
sub deleteSectionComment {
    my $self = shift;
    my( $Section ) = @_;

    $self->{'ini'}->DeleteSectionComment($Section);
    return;
}

#========================================================================================
# function     valueComment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Comment = $IniConfig->valueComment( $Section, $Key );
# syntax       $IniConfig->valueComment( $Section, $Key, $Comment );
#----------------------------------------------------------------------------------------
# description  Sets or gets the comment of a value in the section.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Key | key name | string
# input        $Comment | value comment | string
# return       $Comment | value comment | string
#========================================================================================
sub valueComment { ## no critic qw(RequireArgUnpacking)
    my $self = shift;
    my( $Section, $Key, $Comment ) = @_;

    if( scalar @_ >= 3 ) {
        $self->{'ini'}->SetParameterComment($Section, $Key, $Comment);
    }
    else {
        return $self->{'ini'}->GetParameterComment($Section, $Key);
    }
}

#========================================================================================
# function     deleteValueComment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->deleteValueComment( $Section, $Key );
#----------------------------------------------------------------------------------------
# description  Removes the comment from the value in the section.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
# input        $Key | key name | string
#========================================================================================
sub deleteValueComment {
    my $self = shift;
    my( $Section, $Key ) = @_;

    $self->{'ini'}->DeleteParameterComment($Section, $Key);
}

#========================================================================================
# function     deleteSection
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->deleteSection( $Section );
# example      $IniConfig->deleteSection( 'Database' );
#----------------------------------------------------------------------------------------
# description  Removes the section from the config file. Call $IniConfig->rewrite to
#               make the changes persistent.
#----------------------------------------------------------------------------------------
# input        $Section | section name | string
#========================================================================================
sub deleteSection {
    my $self = shift;
    my( $Section ) = @_;

    foreach my $Sec ($self->{'ini'}->Sections()) {
        $self->{'ini'}->DeleteSection($Sec) if (lc $Sec eq lc $Section);
    }

    return;
}

#========================================================================================
# function     read
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->read( $FileName );
# example      $IniConfig->read( 'database.ini' );
#----------------------------------------------------------------------------------------
# description  Reads the configuration file in the windows ini file format.
#----------------------------------------------------------------------------------------
# input        $FileName | config file name | string
#========================================================================================
sub read {
    my $self = shift;
    my( $FileName ) = @_;

    $self->fileName( $FileName );
    $self->{'ini'}->ReadConfig() or Error('OpenRead', {
        FileName => $FileName,
        OSError => join( '; ', @Config::IniFiles::errors ) ##no critic (ProhibitPackageVars)
    });

    return;
}

#========================================================================================
# function     write
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->write( $FileName );
# example      $IniConfig->write( 'database.ini' );
#----------------------------------------------------------------------------------------
# description  Writes the content of the config file object to the specified
#               file in the windows ini file format.
#----------------------------------------------------------------------------------------
# input        $FileName | config file name | string
#========================================================================================
sub write {
    my $self = shift;
    my( $FileName ) = @_;

    my $DirName = dirname( $FileName );
    CreateDirectory( $DirName, {'recursive' => 1 } );

    # write config file
    OpenFile( '>', $FileName, sub {
        my ($FileHandle) = @_;
        print $FileHandle $self->sorted_content;
    });

    SetAccessRights( $FileName, {} );
    return;
}

#========================================================================================
# function     rewrite
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->rewrite
#----------------------------------------------------------------------------------------
# description  Writes the configuration file back to disk using the file name that was
#               passed to <function new> or <function create>.
#               Note: when the file was opened as case insensitive, then all section
#               names and keys are converted to lower case.
#========================================================================================
sub rewrite {
    my $self = shift;

    if( $self->{'IsLocked'} ) {
        $self->{'Rewrite'} = 1;
    }
    else {
        $self->write( $self->fileName );
    }

    return;
}

#========================================================================================
# function     print
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->print;
#----------------------------------------------------------------------------------------
# description  Prints the content of the config file object in the windows
#               ini file format to STDOUT.
# return       $Content | prints content of $self->fileName to STDOUT | string
#========================================================================================
sub print {
    my $self = shift;

    return $self->{'ini'}->OutputConfig();
}

#========================================================================================
# function     content
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->content;
#----------------------------------------------------------------------------------------
# description  Returns the current content of the ini file.
# return       $Content | content of $self->fileName | string
#========================================================================================
sub content {
    my $self = shift;
    return $self->sorted_content;
}

#========================================================================================
# function     sorted_content
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->sorted_content;
#----------------------------------------------------------------------------------------
# description  Returns the sorted content of the ini file.
# return       $Content | sorted content of $self->fileName | string
#========================================================================================
sub sorted_content {
    my $self = shift;
    my $FirstSections = $self->{'FirstSections'};
    $FirstSections //= [];
    my $Ini = $self->{'ini'};

    my ($Comments, $Content, @SectionsDone);
    # create sorted content of ini file
    my @AllSections = nsort($Ini->Sections());
    foreach my $Section ((@$FirstSections, @AllSections)) {
      next unless scalar grep { $_ eq $Section } @AllSections;
      next if scalar grep { $_ eq $Section } @SectionsDone;
      $Comments = join("\n", $Ini->GetSectionComment($Section));
      $Comments .= "\n" if $Comments;
      $Content .= "${Comments}[$Section]\n";
      foreach my $Key (nsort($Ini->Parameters($Section))) {
        $Comments = join("\n", $Ini->GetParameterComment($Section, $Key));
        $Comments .= "\n" if $Comments;
        $Content .= "$Comments$Key=". $Ini->val($Section, $Key). "\n";
        $Comments = join("\n", $Ini->GetParameterTrailingComment($Section, $Key));
        $Content .= "$Comments\n" if $Comments;
      }
      $Content .= "\n";
      push(@SectionsDone, $Section);
    }
    $Content =~ s,[\n\s]+$,,;
    return "$Content\n";
}

#========================================================================================
# function     merge
#----------------------------------------------------------------------------------------
# syntax       $IniConfig->merge($IniSource, $aEditSections, $hFlags);
#----------------------------------------------------------------------------------------
# example      my $IniSource = DE_EPAGES::Core::API::IniConfig->new(
#                   FileName => 'c:/test/source.ini'
#               );
#               my $IniTarget = DE_EPAGES::Core::API::IniConfig->new(
#                   FileName => 'c:/test/target.ini'
#               );
#               $IniTarget->merge( $IniSource, [ 'GLOBAL' ], { 'replace' => 'parameters' } );
#               $IniTarget->rewrite;
#----------------------------------------------------------------------------------------
# description  Adds sections from $IniSource if not yet exist.
#               For all $aEditSections, do not simply add/replace section by $Source
#               section but add/replace only the parameters that exist in $Source.
#               Removes duplicate comments.
#----------------------------------------------------------------------------------------
# input        $IniSource | source of changes | DE_EPAGES::Core::API::IniConfig
# input        $aEditSections | (optional) list of section names | ref.array
# input        $hFlags | (optional) flags to modify the behaviour of MergeIniFiles.
#                       <ul>
#                           <li>replace
#                           if set to 'sections' - overwrites target for all but $aEditSections
#                           if set to 'parameters' - overwrites target only for $aEditSections
#                           if otherwise set - overwrites target for all
#                           if unset - existing sections/parameters in target are not changed</li>
#                           <li>delcomments
#                           if set to ';' - deletes comments starting with ';'
#                           if set to '#' - deletes comments starting with '#'
#                           if otherwise set - deletes all comments
#                           if unset - comments remain
#                           <li>delete
#                           if set delete sections on the fly
#                           <li>FirstSections
#                           list of section names to write before other sections (ref.array)
#                       </ul>
#                       | hash ref
#========================================================================================
sub merge {
    my $self = shift;
    my ($IniSource, $aEditSections, $hFlags) = @_;
    $aEditSections //= [];
    $hFlags //= {};

    my ($ReplaceSections, $ReplaceParameters) = (0, 0);
    if (defined $hFlags->{replace}) {
        $ReplaceSections = 1 unless $hFlags->{replace} eq 'parameters';
        $ReplaceParameters = 1 unless $hFlags->{replace} eq 'sections';
    }

    # add or replace complete sections

    foreach my $Section ( @{ $IniSource->sectionNames } ) {

        next if $Section ~~ $aEditSections;

        if( $self->existsSection( $Section ) ) {
            next unless $ReplaceSections;
            $self->deleteSectionComment( $Section );
        }

        my $hSourceSection = $IniSource->section($Section);
        $self->section( $Section, $hSourceSection );
        if( my $Comment = $IniSource->sectionComment($Section) ) {
            $self->sectionComment($Section, $Comment);
        }
        foreach my $Parameter (keys %$hSourceSection) {
            if( my $Comment = $IniSource->valueComment($Section, $Parameter) ) {
                $self->valueComment($Section, $Parameter, $Comment);
            }
        }
    }

    # update individual values in @$aEditSections

    foreach my $Section (@$aEditSections) {
        if( my $Comment = $IniSource->sectionComment($Section) ) {
            $self->sectionComment($Section, $Comment)
        }

        my $hSourceSection = $IniSource->section($Section);
        foreach my $Parameter (keys %$hSourceSection) {
            next if defined $self->get($Section, $Parameter) and not $ReplaceParameters;
            $self->set($Section, $Parameter, $hSourceSection->{$Parameter});
            if( my $Comment = $IniSource->valueComment($Section, $Parameter) ) {
                $self->valueComment($Section, $Parameter, $Comment);
            }
        }
    }

    # delete comment lines

    $self->_deleteComments($hFlags->{delcomments}) if defined $hFlags->{delcomments};

    # delete obsolete sections

    if( $hFlags->{delete} ) {
        $self->deleteSection($_) foreach split(/\s+/, $hFlags->{delete});
    }

    return;
}

sub _deleteComments {
    my $self = shift;
    my ($CommentChars) = @_;
    $CommentChars = ';#' unless $CommentChars =~ /^[;#]+$/;

    foreach my $Section ( @{ $self->sectionNames } ) {
        if( my $Comment = $self->sectionComment($Section) ) {
            # $ChangedComment contains lines not comment and not empty/space-only
            my $ChangedComment = join("\n", grep { /^\s*[^\s$CommentChars]/ } split(/\n/, $Comment));
            if (not $ChangedComment) {
                $self->deleteSectionComment($Section);
            }
            elsif ($ChangedComment ne $Comment) {
                $self->setSectionComment($Section, $ChangedComment);
            }
        }

        foreach my $Parameter (keys %{ $self->section($Section) }) {
            if( my $Comment = $self->valueComment($Section, $Parameter) ) {
                my $ChangedComment = join("\n", grep { /^\s*[^\s$CommentChars]/ } split(/\n/, $Comment));
                if (not $ChangedComment) {
                    $self->deleteValueComment($Section, $Parameter);
                } elsif ($ChangedComment ne $Comment) {
                    $self->valueComment($Section, $Parameter, $ChangedComment);
                }
            }
        }
    }
}

#========================================================================================
# function     MergeIniFiles
#----------------------------------------------------------------------------------------
# syntax       MergeIniFiles($Target, $Source, $aEditSections, $hFlags);
#----------------------------------------------------------------------------------------
# example      MergeIniFiles("$ENV{EPAGES_CONFIG}/Scheduler.conf",
#                 "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Database/Data/Scheduler/Scheduler.conf",
#                 [ 'LOOPS' ], { 'replace' => 'parameters' } );
#----------------------------------------------------------------------------------------
# description  Adds sections from $Source to $Target if not yet exist in $Target.
#               For all $aEditSections, do not simply add/replace $Target section by $Source
#               section but add/replace only the $Target parameters that exist in $Source.
#               Removes duplicate comments in $Target.
#----------------------------------------------------------------------------------------
# input        $Target | name of file | string
# input        $Source | name of file | string
# input        $aEditSections | (optional) list of section names | ref.array
# input        $hFlags | (optional) flags to modify the behaviour of MergeIniFiles.
#                       <ul>
#                           <li>replace
#                           if set to 'sections' - overwrites target for all but $aEditSections
#                           if set to 'parameters' - overwrites target only for $aEditSections
#                           if otherwise set - overwrites target for all
#                           if unset - existing sections/parameters in target are not changed</li>
#                           <li>delcomments
#                           if set to ';' - deletes comments starting with ';'
#                           if set to '#' - deletes comments starting with '#'
#                           if otherwise set - deletes all comments
#                           if unset - comments remain
#                           <li>delete
#                           if set delete sections on the fly
#                           <li>FirstSections
#                           list of section names to write before other sections (ref.array)
#                       </ul>
#                       | hash ref
#========================================================================================
sub MergeIniFiles ($$;$$) {
    my ($Target, $Source, $aEditSections, $hFlags) = @_;

    __PACKAGE__->lock( FirstSections => $hFlags->{'FirstSections'},
                       FileName => $Target, CaseSensitive => 1, Create => 1, Access => 'w', Sub => sub {
        my ($self) = @_;
        my $IniSource = __PACKAGE__->new( FileName => $Source, CaseSensitive => 1 );

        $self->{'FirstSections'} = $hFlags->{'FirstSections'};
        $self->merge( $IniSource, $aEditSections, $hFlags );
        $self->rewrite;
    });

    return;
}

1;
