package DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports;
use base DE_EPAGES::Core::API::PPI::Transform::CriticTransform;

use strict;
use Params::Util  qw{_INSTANCE };
use DE_EPAGES::Core::API::PPI::Imports;
use DE_EPAGES::Core::API::PPI::Tools qw( LocationIsMatch );

sub applies_to {
    return 'RequireCorrectImports';
}

sub name {
    'OrganizeImports';
}

sub description {
    'Tries to resolve problems reported by RequireCorrectImports. (1) Adds missing imports if they can be identified (2) removes imports if unneeded (3) organizes imports alphabetical list ';
}

sub document {
    my $self = shift;
    my ($doc) = @_;

    unless ( defined _INSTANCE($doc, 'PPI::Document') ) {
        PPI::Exception->throw('Did not provide document.');
    }

    my $changes = 0;
    my $Imports = DE_EPAGES::Core::API::PPI::Imports->new('doc' => $doc);
    # do nothing on multiple package definitions. we don't cover this yet!
    my $aPackageDecls = $doc->find('PPI::Statement::Package');
    return 0 if ($aPackageDecls && scalar @{$aPackageDecls} > 1);

    my $FirstElement = $Imports->getFirstImport() // $aPackageDecls->[0]->snext_sibling();
    return 0 unless $FirstElement;

    foreach my $violation (@{$self->violations()}) {
        next unless (my $DocElem = $self->find_matching_element($violation, $doc));
        my $ViolationMsg = $violation->description(). ' ('.$violation->explanation().')';

        if ($ViolationMsg =~ /^Package .* is included several times/) {
            $changes += $self->_removeDuplicatePackage($violation, $DocElem, $Imports);
        }
        elsif ($ViolationMsg =~ /^Method "[^"]*" is included several times/) {
            # this is done by rewriteImports because of  unique.
            $changes += $self->_removeDuplicateImport($violation, $DocElem, $Imports);
        }
        elsif ($ViolationMsg =~ /^Imported but not used "([^"]+)"/) {
            # that's a bug in RegularExpressions::ProhibitCaptureWithoutTest
            $changes += $self->_removeUnneededImport($violation, $DocElem, $Imports, $1); ## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
        }
        # This only resolves import suggestions without ambiguity!
        elsif ($ViolationMsg =~ /^Used but not imported "[^"]+".*\(Add "\s*use (\S+) qw \((\S+)\);\s*"\)/) { ## no critic (DE_EPAGES::Style::RequireExtendedFormatting) "
            # that's a bug in RegularExpressions::ProhibitCaptureWithoutTest
            $changes += $Imports->addMissingImport($1, $2); ## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
        }
    }
    $changes += $Imports->rewriteImports($FirstElement);

    return $changes;
}

# remove one import line, but combine imports
sub _removeDuplicatePackage {
    my $self = shift;
    my ($violation, $DocElem, $Imports) = @_;

    my $changes = 0;
    my $hImports = $Imports->getImports();

    my $DuplImport;
    my $removeI;
    for ( my $i=0; $i < scalar @{$hImports->{'inc'}}; ++$i ) {
        my $Import = $hImports->{'inc'}[$i];
        if (LocationIsMatch($Import->{'Location'}, $DocElem->location())) {
            $DuplImport = $Import;
            $removeI = $i;
            last;
        }
    }
    return $changes unless $DuplImport;

    my $CorrectImport;
    for ( my $i=0; $i < scalar @{$hImports->{'inc'}}; ++$i ) {
        $CorrectImport = $hImports->{'inc'}[$i];
        # check package string
        next if ($CorrectImport->{'Package'} ne $DuplImport->{'Package'});
        # location must not be equal
        next if (LocationIsMatch($CorrectImport->{'PPI'}->location(), $DocElem->location()));
        # ok, this is the matching line where we can add our imports.
        if (scalar @{$DuplImport->{'Words'}}) {
            push @{$CorrectImport->{'Words'}},
                grep { ! ($_ ~~ @{$CorrectImport->{'Words'}} ) }
                @{$DuplImport->{'Words'}};
        }
        push @{$hImports->{'del'}}, $DuplImport->{'PPI'};
        splice @{$hImports->{'inc'}},$removeI,1;
        ++$changes;
        last;
    }

    return $changes;

}

sub _removeDuplicateImport {
    my $self = shift;
    my ($violation, $DocElem, $Imports) = @_;
    return 1;
}

sub _removeUnneededImport {
    my $self = shift;
    my ($violation, $DocElem, $Imports, $RemoveMe) = @_;
    # remove import from list

    my $hImports = $Imports->getImports();

    my $UnneededImport;
    foreach my $Import (@{$hImports->{'inc'}}){
        if (LocationIsMatch($Import->{'Location'}, $DocElem->location())) {
            $UnneededImport = $Import;
            last;
        }
    }
    return 0 unless $UnneededImport;
    return 0 unless ($RemoveMe ~~ @{$UnneededImport->{'Words'}});

    @{$UnneededImport->{'Words'}} = grep { $_ ne $RemoveMe } @{$UnneededImport->{'Words'}};
    return 1;

}

1;
