#!/srv/epages/eproot/Perl/bin/perl

use strict;

use Config;
use Getopt::Long;
use Text::CSV_XS;
use Sys::Hostname;

use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw( GetFileContentLines );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::IniConfig;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'FILE_OPEN_FAILED'  => 'CSV file (#FileName): can\'t open file (#Code)',
        'FILE_CLOSE_FAILED'  => 'CSV file (#FileName): can\'t close file (#Code)',
    };
}
#ERRORDEFINITION

my $CONFIG_FILENAME = "$ENV{'EPAGES_CONFIG'}/Scheduler.conf";
my $WRAP_SH = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/bin/wrapScheduler.sh";
my $WRAP_CMD = $ENV{'EPAGES'}. '\bin\wrapScheduler.cmd';

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 add {At|Schtasks|Cron} [User] > LOGFILE.bat
    perl $0 del {At|Schtasks} LOGFILE.at > LOGFILE.bat

arguments:
    first          add or remove scheduled tasks (add, del)
    second         scheduler type (At, Schtasks, Cron)
    third (w/ del) file that contains current scheduler listing

flags:
    -help          show the command line options

Example:
    perl $0 del Schtasks epagesScheduler.at > epagesScheduler.bat
    epagesScheduler.bat
    perl $0 add Schtasks > epagesScheduler.bat
    epagesScheduler.bat

    perl $0 add Cron > crontab.txt
    crontab crontab.txt
END_USAGE
    exit 2;
}

sub Main {

    local $| = 1;

    my $Help;

    GetOptions( 'help' => \$Help );

    usage() if $Help;
    usage() if scalar @ARGV < 2;

    my $Task = $ARGV[0];
    my $Scheduler = $ARGV[1];

    if ($Task eq 'add') {
        if (scalar @ARGV > 3 or $Scheduler !~ /^(At|Schtasks|Cron)$/) {
            usage();
        } else {
            AddTasks($Scheduler, $ARGV[2]);
        }
    } elsif ($Task eq 'del') {
        if (scalar @ARGV != 3 or $Scheduler !~ /^(At|Schtasks)$/) {
            usage();
        } elsif ($Scheduler eq 'At') {
            DelAtTasks($ARGV[2]);
        } elsif ($Scheduler eq 'Schtasks') {
            DelSchTasks($ARGV[2]);
        }
    } else {
        usage();
    }
}

#========================================================================================
# function     AddTasks
#========================================================================================
sub AddTasks {
    my ($SchedulerType, $User) = @_;

    $User = 'ep_appl' unless $User;

    my $TaskNotFound = 1;
    my $HostName = hostname();
    $HostName =~ s/\..*//;

    my ($Options, $Command)  = ('', '');

    my $IniConfig = DE_EPAGES::Core::API::IniConfig->new( FileName => $CONFIG_FILENAME, CaseSensitive => 1 );

    foreach my $SectionName (@{$IniConfig->sectionNames}) {

        next if $SectionName eq 'LOOPS';
        my $hSection = $IniConfig->section( $SectionName );
        next if $hSection->{'IsActive'} ne '1';
        next unless $hSection->{$SchedulerType} =~ /\S/;
        my $Machine = ",$hSection->{'Machine'},";
        next if $Machine !~ /,localhost,/x and $Machine !~ /,\Q$HostName\E,/x;

        next if !$hSection->{'User'} && $User ne 'ep_appl';
        my $SectionUser = _ReplaceENV( $hSection->{'User'} );

        if ( $Config{'osname'} eq 'MSWin32' ) {
            next if $SectionUser and $User ne $SectionUser;
        } else {
            next if $SectionUser and (getpwnam $User)[2] ne (getpwnam $SectionUser)[2];
        }

        my $SchedulerCmd = "$SchedulerType ";
        if ($SchedulerType eq 'Schtasks') {
            $Command = "/tn \"epages6-$SectionName\" /tr \"$WRAP_CMD $SectionName\"";
        } elsif ($SchedulerType eq 'At') {
            $Command = "\"$WRAP_CMD $SectionName\"";
        } elsif ($hSection->{'MaxDuration'} =~ /\S/) {
            $SchedulerCmd = '';
            $Command = "$WRAP_SH -d $hSection->{'MaxDuration'} $SectionName $ENV{EPAGES_PROJECT}";
        } else {
            $SchedulerCmd = '';
            $Command = "$WRAP_SH $SectionName $ENV{EPAGES_PROJECT}";
        }

        print "\@echo off\n" if $SchedulerType =~ /^(At|Schtasks)$/ and $TaskNotFound;
        $Options = $SchedulerType eq 'Schtasks' ? '/create /ru System ' : '';
        print "$SchedulerCmd$hSection->{$SchedulerType} $Options$Command\n";
        $TaskNotFound = 0;
    }
    exit $TaskNotFound;
}

# replace '$ENV{Varname}' in string by the actual values of environment variables
sub _ReplaceENV {
    my ($String) = @_;

    $String =~ s/\$ENV\{(.*?)\}/$ENV{$1}/ge;
    return $String;
}

#========================================================================================
# function     DelAtTasks
#========================================================================================
sub DelAtTasks {
    my ($SchedulerListing) = @_;

    my $FileLines = GetFileContentLines($SchedulerListing);

    # remove header
    shift @$FileLines; shift @$FileLines;

    # syntax of at output:
    # Status ID   Day                     Time          Command Line
    # -------------------------------------------------------------------------------
    #         1   Each 1                  04:00 AM      eBayResetAuctionsPerMonth.bat

    my $TaskNotFound = 1;
    foreach my $Line (@$FileLines) {

        if( $Line =~ /^.{6}\s*([0-9]+)\s*.{38}(.*)/x ) {
            my ($ID, $Command) = ($1,$2);

            if (index($Command, $WRAP_CMD) == 0) {
                print "\@echo off\n" if $TaskNotFound;
                print "at $ID /delete\n";
                $TaskNotFound = 0;
            }
        }
    }
    exit $TaskNotFound;
}

#========================================================================================
# function     DelSchTasks
#========================================================================================
sub DelSchTasks {
    my ($SchedulerListing) = @_;

    my $CsvObject = Text::CSV_XS->new();
    open my $CSV_FILE, '<', $SchedulerListing ## no critic qw( RequireBriefOpen )
        or Error('FILE_OPEN_FAILED', {'Mode' => '<', 'FileName' => $SchedulerListing, 'Code' => $!});

    my $TaskNotFound = 1;
    while (<$CSV_FILE>) {
        s/[\200-\377]//g; # Text::CSV_XS doesn't like 8-bit chars
        $CsvObject->parse($_);
        my @Line = $CsvObject->fields();
        last unless scalar @Line;
        my $TaskName = $Line[1];
        # column number differs between windows versions -> try any
        foreach my $Column (@Line) {
            if (index($Column, $WRAP_CMD) == 0) {
                print "\@echo off\n" if $TaskNotFound;
                print "schtasks /delete /tn $TaskName /f\n";
                $TaskNotFound = 0;
                last;
            }
        }
    }

    close $CSV_FILE or Error('FILE_CLOSE_FAILED', {'FileName' => $SchedulerListing, 'Code' => $!});

    exit $TaskNotFound;
}

RunScript(
    'Sub' => \&Main
);

1;
