use strict;
use Test::More tests => 20;
use DateTime;
use DateTime::Locale;
use DE_EPAGES::Core::API::Constants qw( PERL538 );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

## no critic qw(ProhibitLongLines)

sub cldr_to_strftime {
    my ($loc, $function) = @_;

    my $strf;
    if ($] < PERL538) {
        my $locale = DateTime::Locale->load($loc);
        my $cldr = $locale->$function();
        $strf = $locale->_cldr_to_strftime($cldr); ## no critic qw(ProhibitPrivateSubroutineUse)
        $strf =~ s/%\{hour\}/%H/g;
        $strf =~ s/%\{time_zone_long_name\}/%z/g;
        $strf =~ s/%y|%\{ce_year\}/%Y/g;
        $strf =~ s/%\{month\}/%m/g;
        $strf =~ s/%\{day\}/%e/g;
        $strf =~ s/%\{hour_12\}/%l/g;
    } else {
        my @format = split(/_/, $function);
        $strf = DE_EPAGES::Core::API::Object::DateTimeFormatter::_cldr_to_strftime($loc, $format[0], $format[2]); ## no critic qw(ProtectPrivateSubs)
    }
    return $strf;
}

sub cldrStrf {
    my ($loc, $function) = @_;
    return "[$function]" if $function =~ /%/;

    my $locale = DateTime::Locale->load($loc);
    my $cldr = $locale->$function();
    my $strf = cldr_to_strftime($loc, $function);
    return "[$function: $cldr > $strf]";
}

sub Main {

    ok( 1, 'init');

    my $dmY   = '%d.%m.%Y';     my $Apr12 = '12.04.2004';
    my $Ymd   = '%Y-%m-%d';     my $Apr12ISO8601 = '2004-04-12';
    my $dmYHM = $dmY. ' %H:%M'; my $Apr12Time = $Apr12. ' 11:24';
    my $YmdHM = $Ymd. ' %H:%M'; my $Apr12TimeISO8601 = $Apr12ISO8601. ' 11:24';

    my $dBYHMS = '%d %B %Y %H.%M.%S'; my $Apr12SE = '12 april 2004 11.24.00';

    my $Noon = 12; ##no critic qw(ProhibitMagicNumbers)
    my $Midnight = '00:00';
    my $Apr12UK = '12 Apr 2004';
    my $Apr12TimeUK = $Apr12UK. ' 11:24:00';

    my ($Locale, $Formatter, $Found, $cldrStrf);

    # date time
    $Locale = 'sv_SE';
    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, 'datetime_format_long');
    my $Parse = $] < PERL538 ? "$Apr12SE +0200" : $Apr12SE;
    $Found = $Formatter->parse_datetime($Parse, 'datetime', 'long')->strftime($dBYHMS);
    ok($Apr12SE eq $Found, "$Locale parse: $Apr12SE ~ $Found $cldrStrf");

    $Locale = 'de_DE';
    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, 'time_format_full');
    $Parse = $Midnight;
    if ($] < PERL538) {
        my $dt = DateTime->now(time_zone => 'Europe/Berlin');
        $Parse = $dt->is_dst ? "$Midnight:00 +0200" : "$Midnight:00 +0100";
    }
    $Found = $Formatter->parse_datetime($Parse, 'time', 'full')->strftime('%H:%M');
    ok($Midnight eq $Found, "$Locale parse: $Midnight ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, $dmY);
    $Found = $Formatter->parse_datetime($Apr12,'date','medium')->strftime($dmY);
    ok($Apr12 eq $Found, "$Locale parse: $Apr12 ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, 'date_format_medium');
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12,'date','medium'),'date','medium');
    ok($Apr12 eq $Found, "$Locale format: $Apr12 ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, 'datetime_format_medium');
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Noon,'time', '%H'),'datetime','medium');
    # old perl had some issues which are fixed with 5.38.0
    my $EpochNoon = $] < PERL538 ? '01.01.1 12:00:00' : '01.01.1970 12:00:00';
    ok($EpochNoon eq $Found, "$Locale format: $EpochNoon ~ $Found $cldrStrf");

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, $dmYHM);
    $Found = $Formatter->parse_datetime($Apr12Time,'datetime', $dmYHM)->strftime($dmYHM);
    ok($Apr12Time eq $Found, "$Locale parse: $Apr12Time ~ $Found $cldrStrf");
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12Time,'datetime', $dmYHM), 'datetime', $dmYHM);
    ok($Apr12Time eq $Found, "$Locale format: $Apr12Time ~ $Found $cldrStrf");

    $Locale = 'en_GB';
    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, $dmY);
    $Found = $Formatter->parse_datetime($Apr12UK,'date', 'medium')->strftime($dmY);
    ok($Apr12 eq $Found, "$Locale parse: $Apr12 ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, 'date_format_medium');
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12UK, 'date', 'medium'), 'date', 'medium');
    ok($Apr12UK eq $Found, "$Locale format: $Apr12UK ~ $Found $cldrStrf");

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, $dmYHM);
    $Found = $Formatter->parse_datetime($Apr12TimeUK, 'datetime', 'medium' )->strftime($dmYHM);
    ok($Apr12Time eq $Found, "$Locale parse: $Apr12Time ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, 'datetime_format_medium');
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12TimeUK, 'datetime', 'medium' ), 'datetime', 'medium' );
    $Apr12TimeUK = $Apr12UK. ' 11:24:00';
    ok($Apr12TimeUK eq $Found, "$Locale format: $Apr12TimeUK ~ $Found $cldrStrf");

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    $cldrStrf = cldrStrf($Locale, 'datetime_format_medium');
    $Found = $Formatter->format_datetime(DateTime->now, 'datetime', 'medium');
    ok($Found, "$Locale format: DateTime->now $cldrStrf");
    $Found = $Formatter->parse_datetime($Formatter->format_datetime(DateTime->now, 'datetime', 'medium'), 'datetime', 'medium');
    ok($Found, "$Locale parse: DateTime->now $cldrStrf");

    # custom date time format
    $Locale = 'de_DE';
    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, $dmY);
    $Found = $Formatter->parse_datetime($Apr12ISO8601,'date',$Ymd)->strftime($dmY);
    ok($Apr12 eq $Found, "ISO-8601 parse: $Apr12 ~ $Found $cldrStrf");
    $cldrStrf = cldrStrf($Locale, $Ymd);
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12,'date','medium'),'date',$Ymd);
    ok($Apr12ISO8601 eq $Found, "ISO-8601 format: $Apr12ISO8601 ~ $Found $cldrStrf");

    $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>$Locale);
    $cldrStrf = cldrStrf($Locale, $dmYHM);
    $Found = $Formatter->parse_datetime($Apr12TimeISO8601,'datetime', $YmdHM)->strftime($dmYHM);
    ok($Apr12Time eq $Found, "ISO-8601 parse: $Apr12Time ~ $Found $cldrStrf");
    $Found = $Formatter->format_datetime($Formatter->parse_datetime($Apr12TimeISO8601,'datetime', $YmdHM), 'datetime', $dmYHM);
    ok($Apr12Time eq $Found, "ISO-8601 format: $Apr12Time ~ $Found $cldrStrf");

    eval { $Formatter->format_datetime( DateTime->now, 'date_tz') };
    ok(ExistsError(), 'Unknown format');
    is( GetError()->code, 'FORMAT_UNKNOWN', 'Error code for unknown format' ) or Test::More::diag GetError()->longMessage;
}

RunScript(Sub=>\&Main, Debug=>1);
