use Test::More tests => 63;
use strict;
use Config;
use File::Compare qw ( compare );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw( ShellCommand );

RunScript( Sub => sub {

    my $missingFile = 'Missing.conf';
    my $invalidFile = './Ini/Invalid.conf';
    my $newFile = './Ini/New.conf';
    my $tmpFile1 = './Ini/test1.conf';
    my $tmpFile2 = './Ini/test2.conf';

    my $ePagesConf = './Ini/epages.conf';
    my $DatabaseConf = './Ini/Database.conf';

    DeleteFile($tmpFile1);
    DeleteFile($tmpFile2);

    # Initalization
    eval {
        DE_EPAGES::Core::API::IniConfig->new( );
    };
    ok( ExistsError(), 'Filename or create missing in constructor' );
    is( GetError->code, 'NoParameter', 'error code: NoParameter' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( Create => 1);
    };
    ok( ExistsError(), 'Non-existing filename in constructor' );
    is( GetError->code, 'NoParameter', 'error code: NoParameter' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( FileName => $missingFile );
    };
    ok( ExistsError(), 'Filename missing in constructor' );
    is( GetError->code, 'NoFile', 'error code: NoFile' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( FileName => $invalidFile );
    };
    ok( ExistsError(), 'Non-existing filename in constructor' );
    is( GetError->code, 'OpenRead', 'error code: OpenRead' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( FileName => $newFile, Create => 1 );
    };
    ok (! ExistsError(), 'Creating new config file');

    my $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $ePagesConf );
    ok($ini, 'Given filename should be used as config file');

    my $ini2 = DE_EPAGES::Core::API::IniConfig->new( FileName => $DatabaseConf );
    ok($ini2->existsSection('Store.Default'), 'Reinit successfull');
    ok($ini->existsSection('DE_EPAGES::Mail'), 'Reinit successfull');

    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $ePagesConf );

    # get
    is( $ini->get('DE_EPAGES::Store', 'DefaultWebServicePassword'), 'wsadmin', 'Getting value from ini file');
    is( $ini->get('de_epages::Store', 'DefaultWebServicePassword'), 'wsadmin', 'Getting value from ini file - section in lowercase');
    is( $ini->get('DE_EPAGES::Store', 'defaultwebservicepassword'), 'wsadmin', 'Getting value from ini file - parameter in lowercase');
    is( $ini->get('Foo:bar', 'FUBAR', 'SNAFU'), 'SNAFU', 'Getting default value from ini file');
    is( $ini->get('Foo:bar', 'fubar', 'SNAFU'), 'SNAFU', 'Getting default value from ini file - in lowercase');
    ok( ! $ini->get('Foo:bar', 'FUBAR'), 'No value found in ini file');
    is_deeply( [ $ini->get('Foo:bar', 'FUBAR') ], [ undef ], 'No value in array context');

    # set and check if changes are written to file
    ok( $ini->set ('DE_EPAGES::Store', 'DefaultWebServicePassword', 'wsadmin2') , 'Setting existing parameter on existing section');
    $ini->fileName( $tmpFile1 );
    $ini->rewrite(); # converts all keys and sections to lower case!

    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    is( $ini->get('DE_EPAGES::Store', 'DefaultWebServicePassword'), 'wsadmin2', 'Check setting existing parameter on existing section');
    $ini->set ('DE_EPAGES::Store', 'DefaultWebServicePassword', 'wsadmin');

    ok ($ini->set ('DE_EPAGES::Store', 'Testkey1', 'Testvalue1'), 'Setting new parameter on existing section');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    ok( $ini->get('DE_EPAGES::Store', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on existing section');

    ok( $ini->set ('Foo:bar', 'Testkey1', 'Testvalue1'), 'Setting new parameter on new section');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    ok( $ini->get('Foo:bar', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on new section');
    $ini->delete('Foo:bar', 'Testkey1');
    $ini->rewrite();

    # delete key
    ok( defined $ini->get('DE_EPAGES::Store', 'Testkey1'), 'before deleting parameter');
    $ini->delete('DE_EPAGES::Store', 'Testkey1');
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), undef, 'Deleting parameter');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), undef, 'Check Deleting parameter');
    ok( ! exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check Deleting parameter 1');
    ok( ! exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check Deleting parameter 2');

    # empty key
    $ini->set('DE_EPAGES::Store', 'Testkey1', '' );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Set key to empty string');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    ok( exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check Set key to empty string 1');
    ok( !exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check Set key to empty string 2');
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Check Set key to empty string 3');

    # undefined key
    $ini->set('DE_EPAGES::Store', 'Testkey1', undef );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Set key to undef');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    ok( exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check  Set key to undef 1');
    ok( !exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check  Set key to undef 2');
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Check Set key to undef 3');

    # check existsSection
    ok( $ini->existsSection('DE_EPAGES::Store'), 'Checking existing section');
    ok( ! $ini->existsSection('Bla::Blub'), 'Checking nonexisting section');

    # add section
    ok( $ini->addSection('Bla:blubb') eq 'Bla:blubb', 'Adding section');
    ok( $ini->existsSection('Bla:blubb'), 'Check adding section');

    # get section names
    my $aSections = $ini->sectionNames ('DE_EPAGES::Store');
    ok( scalar @$aSections  == 7, 'Getting correct number of section names');

    $aSections = $ini->sectionNames ('de_epages::store');
    ok( scalar @$aSections  == 7, 'Getting correct number of section names - with lowercase');

    # adding and returning section values
    my $hSection = $ini->section('DE_EPAGES::Store');
    ok ($$hSection{lc 'DefaultWebServicePassword'} eq 'wsadmin' , 'Getting hash from section');
    # get keys case sensitive
    my $ini3 = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1, CaseSensitive => 1 );
    my $hStore = $ini3->section('DE_EPAGES::Store');
    is ( scalar keys %$hStore, 0, 'Mixed case section name does not exist anymore');
    ok ( !$ini3->existsSection('DE_EPAGES::Store'), 'Mixed case section name does not exist anymore');

    $hStore = $ini3->section('de_epages::store');
    is ($hStore->{'DefaultWebServicePassword'}, undef, 'Mixed case key name does not exist anymore');
    is ($hStore->{'defaultwebservicepassword'}, 'wsadmin' , 'Getting hash from section case sensitive');

    $hSection = {'newkey2' => 'newval2'};
    $ini->section('DE_EPAGES::Store', $hSection);
    my $hSection2 = $ini->section('DE_EPAGES::Store');
    ok ($hSection2->{'newkey2'} eq $$hSection{'newkey2'}, 'Getting new inserted value back in hash from section');

    # deleting section
    $ini->deleteSection('Bla:blubb');
    $ini->deleteSection('Foo:bar');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile1 );
    ok( ! $ini->existsSection('Foo:bar'), 'Check deleting section');

    # get content
    my $content = $ini->content();
    ok( $content, 'Content of ini file');

    my @comments = $content =~ /(#\w+)/g;
    ok(scalar @comments > 0, 'Content has some comments');

    # creating new file

    my $ini4 = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile2, Create => 1 , CaseSensitive => 1);
    ok( $ini4->set ('Foo:bar', 'Testkey1', 'Testvalue1'), 'Setting new parameter on new file');
    $ini4->rewrite();
    $ini4 = DE_EPAGES::Core::API::IniConfig->new( FileName => $tmpFile2, CaseSensitive => 1 );
    ok( $ini4->get('Foo:bar', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on new file');

    DeleteFile($tmpFile2);

    DE_EPAGES::Core::API::IniConfig->lock(
        FileName => $tmpFile2,
        CaseSensitive => 1,
        Access => 'w',
        Create => 1,
        Sub => sub {
            my ($IniConfig) = @_;
            my %Info = map { $_ => $_ } map { "A$_" } ( 1..100 ); ## no critic qw(ProhibitMagicNumbers)
            $IniConfig->section( 'AttributeValues', \%Info );
            $IniConfig->rewrite;
        }
    );

    DE_EPAGES::Core::API::IniConfig->lock(
        FileName => $tmpFile2,
        CaseSensitive => 1,
        Access => 'r',
        Create => 1,
        Sub => sub {
            my ($IniConfig) = @_;
            is( 100, scalar keys %{$IniConfig->section('AttributeValues')}, 'lock' ); ## no critic qw(ProhibitMagicNumbers)
        }
    );

    DeleteFile($tmpFile2);
    ok( !ExistsFile($tmpFile2), 'cleanup' );

    my $EPJConf = './Ini/epagesj.sample.conf';
    my $EPJfixed = './Ini/epagesj.fixed.conf';
    my $EPJsorted = './Ini/epagesj.sorted.conf';

    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $EPJConf );
    is( $ini->get('Task', 'corePoolSize'), '3', 'epagesj.sample.conf: Check "corePoolSize" in section "Task"');
    is( $ini->get('Task', 'maxLifeInSecs'), '864000', 'epagesj.sample.conf: Check "maxLifeInSecs" in section "Task"');

    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $EPJfixed );
    my $Expected = ";; -----------------------------------------------\n;; EPagesJ Server configuration";
    my $Found = join("\n", $ini->{'ini'}->GetSectionComment('EPagesJServer'));
    is( $Expected, $Found, 'epagesj.fixed.conf: Check comment of section "EPagesJServer"');

    $Expected = ';; Maximum lifetime in seconds before a completed task gets removed. Default 10 days';
    $Found = join("\n", $ini->{'ini'}->GetParameterComment('Task', 'maxLifeInSecs'));
    is( $Expected, $Found, 'epagesj.fixed.conf: Check comment of key "maxLifeInSecs" in section "Task"');

    SKIP: {
        skip 'not on console', 2 unless $ENV{'RUN_CONSOLE_TESTS'};
        ShellCommand("../Scripts/uncommentKeyValuePairs.sh $EPJConf > $tmpFile1");
        is(compare($tmpFile1, $EPJfixed), 0, "(fix $EPJConf) vs. $EPJfixed");
        ShellCommand("$ENV{PERL} ../Scripts/sortSchedulerConf.pl -file $tmpFile1 -inplace");
        is(compare($tmpFile1, $EPJsorted), 0, "(sort $EPJfixed) vs. $EPJsorted");
    }

    DeleteFile($tmpFile1);
});
