#========================================================================================
# package      DE_EPAGES::Customer::API::CustomerPatch
# state        public
#----------------------------------------------------------------------------------------
# description  Implements JSON-PATCH (RFC 6902) for the customer object.
#========================================================================================
package DE_EPAGES::Customer::API::CustomerPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToPerl );
use DE_EPAGES::Customer::API::WebService::CustomerService;
use DE_EPAGES::Shop::API::AddressValidation;
use DateTime::Format::ISO8601;

use strict;

## no critic (ProhibitUnusedPrivateSubroutines)



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Customer );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input         $Customer  | A Product | DE_EPAGES::Customer::API::Object::Customer
# return       $self | self | DE_EPAGES::Customer::API::CustomerPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Customer) = @_;
    my $self  = bless {
        'Customer' => $Customer,
    }, $class;
    return $self;
}



sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Customer = $self->{'Customer'};

    eval {
        if ($Path eq '/customerNumber') {
            $Customer->set({'Alias' => $Value});
        }
        elsif ($Path eq '/internalNote') {
            $Customer->set({'Comment' => $Value});
        }
        elsif ($Path eq '/billingAddress') {
            my $hBillingAddressData = ConvertAddressToPerl($Value);
            $self->_setBillingAddress($Customer, $hBillingAddressData);
        }
        else {
            Error('Adding in '.$Path.' is not supported!');
        }
    };
      if (ExistsError()) {
        my $Error = GetError();
        Error('BadRequest', {'reason'=> $Error->shortMessage }, $Error);
    }
    return;
}



sub _remove {
    my $self = shift;
    my ($Path) = @_;

    my $Customer = $self->{'Customer'};

    if($Path =~ m|^/internalNote|) {
        $Customer->set({'Comment' => undef});
        return;
    }

    Error('BadRequest', {'Reason' => "Removing $Path is not supported!"});
}



sub _setBillingAddress {
    my $self = shift;
    my ($Customer, $hAddressData) = @_;

    if ( isValid($hAddressData) ) {
        $self->_validateAddressData($Customer, $hAddressData);
        my $SOAPCustomerService = $self->_createSoapCustomerService();
        $SOAPCustomerService->setAddress( $Customer, $hAddressData, 'BillingAddress');
    }

    return;
}



sub _removeAddress {
    my $self = shift;
    my ($Customer) = @_;

    my $BillingAddress = $Customer->get('BillingAddress');
    $BillingAddress->delete();

    return;
}



sub _validateAddressData {
    my $self = shift;
    my ($Customer, $hAddressData) = @_;

    my $Shop = $Customer->getSite();
    my $LocaleID = $Shop->get('LocaleID');

    my $AddressValidation = $self->_createAddressValidation($Customer, $LocaleID);
    $AddressValidation->validate($hAddressData);
}



sub _createAddressValidation {
    my $self = shift;
    my ($Customer, $LocaleID) = @_;
    return DE_EPAGES::Shop::API::AddressValidation->new($Customer, $LocaleID);
}



sub _createSoapCustomerService {
    return DE_EPAGES::Customer::API::WebService::CustomerService->new();
}

1;
