#========================================================================================
# package      DE_EPAGES::DHL::API::DHL
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  This is the API class to access the DHL SOAP Webservices
#========================================================================================
package DE_EPAGES::DHL::API::DHL;

use base Exporter;
use strict;

use DE_EPAGES::DHL::API::Object::RemoteDHL;
use DE_EPAGES::DHL::API::SOAP::Address;
use DE_EPAGES::DHL::API::SOAP::Auth;
use DE_EPAGES::DHL::API::SOAP::BuyShoppingCart;
use DE_EPAGES::DHL::API::SOAP::CurPickupTime;
use DE_EPAGES::DHL::API::SOAP::Dimension;
use DE_EPAGES::DHL::API::SOAP::Filter;
use DE_EPAGES::DHL::API::SOAP::LabelItem;
use DE_EPAGES::DHL::API::SOAP::LoadBuyedShoppingCartRequest;
use DE_EPAGES::DHL::API::SOAP::Orderer;
use DE_EPAGES::DHL::API::SOAP::PartnerBackLinks;
use DE_EPAGES::DHL::API::SOAP::PaymentPartner;
use DE_EPAGES::DHL::API::SOAP::PaymentPaypal;
use DE_EPAGES::DHL::API::SOAP::PaymentDHLCheckout;
use DE_EPAGES::DHL::API::SOAP::PaymentNullpay;
use DE_EPAGES::DHL::API::SOAP::PaymentCreditcard;
use DE_EPAGES::DHL::API::SOAP::PaymentGiropay;
use DE_EPAGES::DHL::API::SOAP::PickupItem;
use DE_EPAGES::DHL::API::SOAP::ProductInfoRequest;
use DE_EPAGES::DHL::API::SOAP::ReceiverAddress;
use DE_EPAGES::DHL::API::SOAP::SenderAddress;
use DE_EPAGES::DHL::API::SOAP::ShoppingCart;
use DE_EPAGES::DHL::API::SOAP::ShoppingCartValidateRequest;
use DE_EPAGES::DHL::API::SOAP::ShoppingCartCheckoutDirectRequest;
use DE_EPAGES::DHL::API::SOAP::ShoppingCartCheckoutViaPaymentRequest;
use DE_EPAGES::DHL::API::SOAP::ShoppingCartIdentification;
use DE_EPAGES::DHL::API::Tools qw( GetDHLProductFromDB );
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObjectByPath );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::PerlTools qw (isValid);

our @EXPORT_OK = qw(
    ValidateShoppingCart
    ShoppingCartCheckoutViaPayment
    ShoppingCartCheckoutDirect
    LoadBoughtShoppingCart
    RequestProductInfo
    SelectPackstation
    CreatePackstationAddress
    DeletePackstationAddress
);

#------------------------------------------------------------------------------
# Error definitions
#------------------------------------------------------------------------------
sub ErrorDefinition {
    return {
        MissingShopName              => 'Missing shop owners name or company name',
        MissingShopStreet            => 'Missing shop street',
        MissingShopCountry           => 'Missing shop country',
        MissingShopCity              => 'Missing shop city',
        MissingShopZipCode           => 'Missing shop zip code',
        MissingShippingName          => 'Missing recipient name',
        MissingShippingStreet        => 'Missing recipient street',
        MissingShippingCountry       => 'Missing recipient county',
        MissingShippingCity          => 'Missing recipient city',
        MissingShippingZipCode       => 'Missing recipient zip code',
        DHLMaxiTransportNotSupported => 'The Maxi Transport is not supported',
        NoDHLShippingMethod          => 'The Shop \'#ShopAlias\' has no DHL shipping method. Store: #Store',
    }
}

#========================================================================================
# function     ValidateShoppingCart
# state        public
#----------------------------------------------------------------------------------------
# syntax       ValidateShoppingCart( $aDHLShippingLabels );
#----------------------------------------------------------------------------------------
# description  Calls ValidateShoppingCart webservice
#----------------------------------------------------------------------------------------
# input        $aDHLShippingLabels | shipping labels | ref.array.object
# return       $SoapResponse | SOAP response | object
#========================================================================================
sub ValidateShoppingCart {
    my ($aDHLShippingLabels) = @_;
    my $System             = LoadRootObject();
    my $Request            = DE_EPAGES::DHL::API::SOAP::ShoppingCartValidateRequest->new;

    return unless (scalar @{$aDHLShippingLabels});
    _FillCheckoutRequest($aDHLShippingLabels, $Request, { 'Validate' => 1 });

    my $Soap = DE_EPAGES::DHL::API::Object::RemoteDHL->new($System->get('DHL_Logging'));
    $Soap->testMode($aDHLShippingLabels->[0]->getSite->get('IsInternalTestShop'));
    return $Soap->ShoppingCartValidate($Request);
}


#========================================================================================
# function     ShoppingCartCheckoutViaPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       ShoppingCartCheckoutViaPayment( $aDHLShippingLabels );
#----------------------------------------------------------------------------------------
# description  Calls ShoppingCartCheckoutViaPayment webservice
#----------------------------------------------------------------------------------------
# input        $aDHLShippingLabels | shipping labels | ref.array.object
# input        $ReturnURL | return URL | string
# return       $SOAPResponse | SOAP response | object
#========================================================================================
sub ShoppingCartCheckoutViaPayment {
    my ($aDHLShippingLabels, $ReturnURL) = @_;
    my $Request = DE_EPAGES::DHL::API::SOAP::ShoppingCartCheckoutViaPaymentRequest->new;

    return unless (scalar @{$aDHLShippingLabels});
    _FillCheckoutRequest($aDHLShippingLabels, $Request, {});

    my $PartnerBackLinks                      = DE_EPAGES::DHL::API::SOAP::PartnerBackLinks->new;
    $PartnerBackLinks->{'paymentAcceptedUrl'} = $ReturnURL;
    $PartnerBackLinks->{'paymentRejectedUrl'} = $ReturnURL;
    $Request->{'PartnerBackLinks'} = $PartnerBackLinks;

    my $Soap = DE_EPAGES::DHL::API::Object::RemoteDHL->new(LoadRootObject()->get('DHL_Logging'));
    $Soap->testMode($aDHLShippingLabels->[0]->getSite->get('IsInternalTestShop'));
    return $Soap->ShoppingCartCheckoutViaPayment($Request);
}

#========================================================================================
# function     ShoppingCartCheckoutDirect
# state        public
#----------------------------------------------------------------------------------------
# syntax       ShoppingCartCheckoutDirect( $aDHLShippingLabels );
#----------------------------------------------------------------------------------------
# description  Calls ShoppingCartCheckoutDirect webservice
#----------------------------------------------------------------------------------------
# input        $aDHLShippingLabels | shipping labels | ref.array.object
# input        $UseNullpay | use nullpay as payment | boolean
# return       $SOAPResponse | SOAP response | object
#========================================================================================
sub ShoppingCartCheckoutDirect {
    my ($aDHLShippingLabels, $UseNullpay) = @_;

    my $Request = DE_EPAGES::DHL::API::SOAP::ShoppingCartCheckoutDirectRequest->new;
    return unless (scalar @{$aDHLShippingLabels});
    _FillCheckoutRequest($aDHLShippingLabels, $Request, { 'UseNullpay' => $UseNullpay });

    my $Soap = DE_EPAGES::DHL::API::Object::RemoteDHL->new(LoadRootObject()->get('DHL_Logging'));
    $Soap->testMode($aDHLShippingLabels->[0]->getSite->get('IsInternalTestShop'));
    return $Soap->ShoppingCartCheckoutDirect($Request);
}

#========================================================================================
# function     _FillCheckoutRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FillCheckoutRequest( $Input1 );
#----------------------------------------------------------------------------------------
# description  Fills the SOAP request with data
#----------------------------------------------------------------------------------------
# input        $aDHLShippingLabels | shiping labels | ref.array.object
# input        $Request | SOAP request | object
# input        $hOptions | options. Used keys are <ul>
#                 <li>UseNullpay - use nullpay for zero amount shopping carts</li>
#                 <li>Validate - insert only data relevant for validate request</li>
#               </ul> | ref.hash
# return       none
#========================================================================================
sub _FillCheckoutRequest {
    my ($aDHLShippingLabels, $Request, $hOptions) = @_;
    my $Shop               = $aDHLShippingLabels->[0]->getSite;
    TestObject('Shop',$Shop,'DE_EPAGES::Shop::API::Object::Shop');
    my $ShopAddress        = $Shop->get('Address');
    my $System             = LoadRootObject();
    my $UseNullpay         = $hOptions->{'UseNullpay'};
    my $Validate           = $hOptions->{'Validate'};

    my $Auth                                  = DE_EPAGES::DHL::API::SOAP::Auth->new;
    my $Orderer                               = DE_EPAGES::DHL::API::SOAP::Orderer->new;
    my $SenderAddress                         = DE_EPAGES::DHL::API::SOAP::SenderAddress->new;
    my $PaymentPaypal                         = DE_EPAGES::DHL::API::SOAP::PaymentPaypal->new;
    my $PaymentNullpay                        = DE_EPAGES::DHL::API::SOAP::PaymentNullpay->new;
    my $PaymentCreditcard                     = DE_EPAGES::DHL::API::SOAP::PaymentCreditcard->new;
    my $PaymentDHLCheckout                    = DE_EPAGES::DHL::API::SOAP::PaymentDHLCheckout->new;
    my $PaymentPartner                        = DE_EPAGES::DHL::API::SOAP::PaymentPartner->new;
    my $ShoppingCart;
    if ($Validate) {
        $ShoppingCart                      = DE_EPAGES::DHL::API::SOAP::ShoppingCart->new;
    } else {
        $ShoppingCart                      = DE_EPAGES::DHL::API::SOAP::BuyShoppingCart->new;
    }

    # Auth information
    $Auth->{'partnerId'}        = $System->get('DHL_WS_PartnerId');
    $Auth->{'subPartnerId'}     = $System->get('DHL_WS_SubPartnerId');
    $Auth->{'user'}             = $System->get('DHL_WS_User');
    $Auth->{'password'}         = $System->get('DHL_WS_Password');

    _CheckShopAddress($ShopAddress);
    # Orderer information
    _FillAddress($Orderer, $ShopAddress);
    # DHL invoice goes to shop owner...
    _FillAddress($SenderAddress, $ShopAddress);

    # build object structure like the xml structure
    $Request->{'Auth'}             = $Auth;
    $Request->{$Validate?'ShoppingCart':'BuyShoppingCart'}    = $ShoppingCart;
    $ShoppingCart->{'Orderer'}                                = $Orderer;

    my $DHLShippingFulfillmentMethod = _FindDHLShippingFulfillmentMethod($Shop);

    # error if no match.
    Error('NoDHLShippingMethod',{
        'ShopAlias' => $Shop->alias,
        'Storename' => GetCurrentDBHandle->cacheKey,
    }) unless (defined $DHLShippingFulfillmentMethod);

    if ($Validate) {
        $Request->{'ShoppingCart'}->{'PaymentPartner'} = $PaymentPartner;
        # a dummy payment has to be chosen to validate the shoppingcart
        $PaymentPartner->{'paymentId'}    = 'DUMMYPAY';
    } else {
        # get selected payment method.
        # Bei Warenkorbwert = 0 muss hier PaymentNullpay
        if ($UseNullpay) {
            $ShoppingCart->{'_UsedPaymentMethodElement'} = 'PaymentNullpay';
            $ShoppingCart->{'PaymentNullpay'}            = $PaymentNullpay;
        } else {
            my $PaymentMethod = $DHLShippingFulfillmentMethod->get('DHLPaymentMethod');
            if ($PaymentMethod eq 'Paypal') {
                $ShoppingCart->{'_UsedPaymentMethodElement'} = 'PaymentPaypal';
                $ShoppingCart->{'PaymentPaypal'}             = $PaymentPaypal;
            }
            elsif ($PaymentMethod eq 'CreditCard')  {
                $ShoppingCart->{'_UsedPaymentMethodElement'} = 'PaymentCreditcard';
                $ShoppingCart->{'PaymentCreditcard'}         = $PaymentCreditcard;
            }
            elsif ($PaymentMethod eq 'DHLCheckout') {
                $ShoppingCart->{'_UsedPaymentMethodElement'} = 'PaymentDHLCheckout';
                $ShoppingCart->{'PaymentDHLCheckout'}        = $PaymentDHLCheckout;
            }
        }
    }

    my @LabelItems;
    my $IsPickupService = 0;
    my $PickupDateStr = '';

    # build all LabelItems
    foreach my $DHLShippingLabel (@{$aDHLShippingLabels}) {
        my $LabelItem          = DE_EPAGES::DHL::API::SOAP::LabelItem->new;
        _FillLabelItem($DHLShippingLabel, $LabelItem, $SenderAddress);
        if ($DHLShippingLabel->get('PickupService')) {
            $IsPickupService = 1;
            my $PickupDate = $DHLShippingLabel->get('PickupDate');
            my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(locale => 'de_DE');
            # 2009-08-28T00:00:00.00
            $PickupDateStr = $Formatter->format_datetime( $PickupDate, 'date', '%Y-%m-%d' ).'T00:00:00.00';
        }
        push (@LabelItems,$LabelItem);
    }

    # build object structure like the xml structure
    $ShoppingCart->{'LabelItems'}     = \@LabelItems;

    # add the PickupItem if Pickup Service was chosen for one DHLShippingLabel
    if ($IsPickupService) {
        my $PickupItem    = DE_EPAGES::DHL::API::SOAP::PickupItem->new;
        _FillPickupItem($PickupItem, $PickupDateStr, $ShopAddress);
        $ShoppingCart->{'_IsPickupService'}    = 1;
        $ShoppingCart->{'PickupItem'}          = $PickupItem;
    }
    else {
        $ShoppingCart->{'_IsPickupService'} = 0;
    }

    # DHL TOS accepted?
    if (defined($DHLShippingFulfillmentMethod)) {
        $ShoppingCart->{'acceptAgb'} = 'true';
    } else {
        $ShoppingCart->{'acceptAgb'} = 'false';
    }

    # Accept emails from DHL to the Merchant for bought shipping labels?
    if (defined($DHLShippingFulfillmentMethod) &&
        $DHLShippingFulfillmentMethod->get('AcceptSendUserEmail')) {
        $ShoppingCart->{'sendUserEmail'} = 'true';
    } else {
        $ShoppingCart->{'sendUserEmail'} = 'false';
    }
}

#========================================================================================
# function     _FillAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FillAddress( $Input1 );
#----------------------------------------------------------------------------------------
# description  Transforms an address into a soap object structure
#----------------------------------------------------------------------------------------
# input        $Address | Target address object | object
# input        $DbAddress | Source address object (db) | object
# return       none
#========================================================================================
sub _FillAddress {
    my ($Address, $DbAddress) = @_;
    TestObject('DbAddress',$DbAddress,'DE_EPAGES::Address::API::Object::Address');
    #EPG-21217
    my $hInfo = $DbAddress->get([qw( Street StreetOnly HouseNumber AddressExtension Company FullName Zipcode City EMail Country  )]);
    use constant MAXFIELDLENGTH => 50;
    # this enables packstation handling:
    # Address.Street => "Packstation XY" => street + housenr
    # Address.AddressExtension "Postnumber" => name2
    if ($hInfo->{'AddressExtension'}) {
        $Address->{'name1'}                    = substr(($hInfo->{'Company'} ? $hInfo->{'Company'} . ' - ' : '') . $hInfo->{'FullName'}, 0, MAXFIELDLENGTH);
        $hInfo->{'AddressExtension'}           =~ s/^Postnr\.?\s*|Postnummer\s*//i; # Postnr. Postnummer PostNr
        $Address->{'name2'}                    = $hInfo->{'AddressExtension'};
    } else {
        $Address->{'name1'}                    = substr($hInfo->{'Company'} ? $hInfo->{'Company'} : $hInfo->{'FullName'}, 0, MAXFIELDLENGTH);
        $Address->{'name2'}                    = substr($hInfo->{'Company'} ? $hInfo->{'FullName'} : '', 0, MAXFIELDLENGTH);
    }
    #EPG-21217
    $Address->{'street'}                   = $hInfo->{'StreetOnly'};
    $Address->{'houseNumber'}              = $hInfo->{'HouseNumber'};
    $Address->{'postalCode'}               = $hInfo->{'Zipcode'};
    $Address->{'city'}                     = $hInfo->{'City'};
    my $Country = $hInfo->{'Country'};
    $Address->{'countryCode'}              = $Country?$Country->{'Code3'}:undef;
    $Address->{'email'}                    = $hInfo->{'EMail'};
}

#========================================================================================
# function     _FillPickupItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FillPickupItem( $PickupItem );
#----------------------------------------------------------------------------------------
# description  Fills the pickupitem object
#----------------------------------------------------------------------------------------
# input        $PickupItem | target object | object
# input        $PickupDateStr | pickup date | string
# input        $BillingAddress | billing address | object
# return       none
#========================================================================================
sub _FillPickupItem {
    my ($PickupItem, $PickupDateStr, $BillingAddress) = @_;
    my $Address       = DE_EPAGES::DHL::API::SOAP::Address->new;
    my $CurPickupTime = DE_EPAGES::DHL::API::SOAP::CurPickupTime->new;

    _FillAddress($Address, $BillingAddress);

    $CurPickupTime->{'pickupDate'}         = $PickupDateStr;
    $CurPickupTime->{'pickupType'}         = 'TAS';
    $PickupItem->{'address'}               = $Address;
    $PickupItem->{'curPickupTime'}         = $CurPickupTime;
}

#========================================================================================
# function     _FillLabelItem
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FillLabelItem( $DHLShippingLabel );
#----------------------------------------------------------------------------------------
# description  Fills one LabelItem object
#----------------------------------------------------------------------------------------
# input        $DHLShippingLabel | shipping label (source) | object
# input        $LabelItem | target label item | object
# input        $SenderAddress | sender address | object
# return       none
#========================================================================================
sub _FillLabelItem {
    my ($DHLShippingLabel, $LabelItem, $SenderAddress) = @_;
    my $Order              = $DHLShippingLabel->get('Order');
    my $ShippingAddress    = $Order->get('ShippingAddress');
    $ShippingAddress       = $Order->get('BillingAddress') if (! defined $ShippingAddress);
    my $ReceiverAddress    = DE_EPAGES::DHL::API::SOAP::ReceiverAddress->new;
    my $Dimension          = DE_EPAGES::DHL::API::SOAP::Dimension->new;

    _CheckShippingAddress($ShippingAddress);
    _FillAddress($ReceiverAddress, $ShippingAddress);

    use constant KG => 1000;

    $Dimension->{'length'}             = $DHLShippingLabel->get('Length');
    $Dimension->{'width'}              = $DHLShippingLabel->get('Width');
    $Dimension->{'depth'}              = $DHLShippingLabel->get('Depth');
    $Dimension->{'weight'}             = $DHLShippingLabel->get('EnteredOrCalculatedOverallWeight') * KG;

    $LabelItem->{'SenderAddress'}      = $SenderAddress;
    $LabelItem->{'ReceiverAddress'}    = $ReceiverAddress;
    $LabelItem->{'Dimension'}          = $Dimension;
    $LabelItem->{'CouponCode'}         = $DHLShippingLabel->get('CouponCode');

    # which DHL product will be bought?
    my $hDHLProduct                    = GetDHLProductFromDB($DHLShippingLabel);
    my $DHLProductID                   = $hDHLProduct->{'DHLProductID'};
    $LabelItem->{'productId'}          = $DHLProductID;

    $LabelItem->{'DHLShippingLabelId'} = $DHLShippingLabel->id;

    # Set the PreAdviceType
    my $PreAdviceType = $DHLShippingLabel->get('PreAdviceType');
    if ($PreAdviceType == 0) {
        $LabelItem->{'preAdviceType'}      = 'PA_DESTROY';
    }
    else {
        $LabelItem->{'preAdviceType'}      = 'PA_RETURN';
    }
}

#========================================================================================
# function     _FindDHLShippingFulfillmentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FindDHLShippingFulfillmentMethod( $Shop );
#----------------------------------------------------------------------------------------
# description  Finds a ShippingFulfillmentMethodDHL in the shop which has Accepted TermsOFService
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# return       $DHLShippingFulfillmentMethod | DHL shipping method | object
#========================================================================================
sub _FindDHLShippingFulfillmentMethod {
    my ($Shop) = @_;
    my $DHLShippingFulfillmentMethod;
    # find a DHL Shipping method which has AcceptTOS set.
    foreach my $ShippingMethod (@{$Shop->get('ShippingMethods')}){
        my $FulfillmentMethod = $ShippingMethod->get('SelectedShippingFulfillmentMethod');
        next unless (defined($FulfillmentMethod) && $FulfillmentMethod->instanceOf('ShippingFulfillmentMethodDHL'));
        next unless $FulfillmentMethod->get('AcceptTOS');
        # found match.
        $DHLShippingFulfillmentMethod = $FulfillmentMethod;
    }

    return $DHLShippingFulfillmentMethod;
}

#========================================================================================
# function     LoadBoughtShoppingCart
# state        public
#----------------------------------------------------------------------------------------
# syntax       LoadBoughtShoppingCart( $ShippingLabel, $ShoppingCartId );
#----------------------------------------------------------------------------------------
# description  Calls LoadBuyedShoppingCart webservice
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | shipping label | DE_EPAGES::DHL::API::Object::DHLShippingLabel
# input        $ShoppingCartId | Shopping cart ID | integer
# return       $SOAPResponse | SOAP response | object
#========================================================================================
sub LoadBoughtShoppingCart {
    my ($ShippingLabel, $ShoppingCartId)   = @_;
    my $System           = LoadRootObject();

    my $Auth                         = DE_EPAGES::DHL::API::SOAP::Auth->new;
    my $ShoppingCartIdentification   = DE_EPAGES::DHL::API::SOAP::ShoppingCartIdentification->new;
    my $LoadBuyedShoppingCartRequest = DE_EPAGES::DHL::API::SOAP::LoadBuyedShoppingCartRequest->new;

    # build object structure like the xml structure
    $LoadBuyedShoppingCartRequest->{'Auth'}                       = $Auth;
    $LoadBuyedShoppingCartRequest->{'ShoppingCartIdentification'} = $ShoppingCartIdentification;

    # Auth information
    $Auth->{'partnerId'}        = $System->get('DHL_WS_PartnerId');
    $Auth->{'subPartnerId'}     = $System->get('DHL_WS_SubPartnerId');
    $Auth->{'user'}             = $System->get('DHL_WS_User');
    $Auth->{'password'}         = $System->get('DHL_WS_Password');

    $ShoppingCartIdentification->{'shoppingCartId'} = $ShoppingCartId;
    my $Soap = DE_EPAGES::DHL::API::Object::RemoteDHL->new($System->get('DHL_Logging'));
    $Soap->testMode($ShippingLabel->getSite->get('IsInternalTestShop'));
    return $Soap->LoadBuyedShoppingCart($LoadBuyedShoppingCartRequest);
}

#========================================================================================
# function     RequestProductInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       RequestProductInfo();
#----------------------------------------------------------------------------------------
# description  Fetch DHL product list via web service call
#----------------------------------------------------------------------------------------
# return       $SOAPResponse | SOAP response | object
#========================================================================================
sub RequestProductInfo {
    my $Auth = DE_EPAGES::DHL::API::SOAP::Auth->new;
    my $Filter = DE_EPAGES::DHL::API::SOAP::Filter->new;
    my $ProductInfoRequest = DE_EPAGES::DHL::API::SOAP::ProductInfoRequest->new;
    my $System = LoadRootObject();

    $ProductInfoRequest->{'version'} = '2.0.0';
    $ProductInfoRequest->{'Auth'} = $Auth;
    $ProductInfoRequest->{'Filter'} = $Filter;

    $Auth->{'partnerId'}        = $System->get('DHL_WS_PartnerId');
    $Auth->{'subPartnerId'}     = $System->get('DHL_WS_SubPartnerId');
    $Auth->{'user'}             = $System->get('DHL_WS_User');
    $Auth->{'password'}         = $System->get('DHL_WS_Password');

    $Filter->{'productGroupFilter'} = 'STANDARD || COLLECTION';
    $Filter->{'viewPrice'} = 'true';
    #$Filter->{'viewFuturePrices'} = 'true';
    $Filter->{'viewDisplayText'} = 'true';
    $Filter->{'viewDimension'} = 'true';
    $Filter->{'viewFeatures'} = 'true';
    #$Filter->{'viewTransitions'} = 'true';

    my $Soap = DE_EPAGES::DHL::API::Object::RemoteDHL->new($System->get('DHL_Logging'));

    return $Soap->RequestProductInfo($ProductInfoRequest);
}

sub _CheckShippingAddress {
   my ($ShippingAddress) = @_;
   Error('MissingShippingName', { } )     unless $ShippingAddress->get('LastName');
   Error('MissingShippingStreet', { } )   unless $ShippingAddress->get('Street');
   Error('MissingShippingZipCode', { } )  unless $ShippingAddress->get('Zipcode');
   Error('MissingShippingCity', { } )     unless $ShippingAddress->get('City');
   Error('MissingShippingCountry', { } )  unless (defined $ShippingAddress && defined $ShippingAddress->get('Country'));
   return;
}


sub _CheckShopAddress {
    my ($ShopAddress) = @_;
    Error('MissingShopName', { } )        unless ($ShopAddress->get('Company') or $ShopAddress->get('LastName'));
    Error('MissingShopStreet', { } )      unless $ShopAddress->get('Street');
    Error('MissingShopZipCode', { } )     unless $ShopAddress->get('Zipcode');
    Error('MissingShopCity', { } )        unless $ShopAddress->get('City');
    Error('MissingShopCountry', { } )     unless (defined $ShopAddress && defined $ShopAddress->get('Country'));
    return;
}

#========================================================================================
# function     SelectPackstation
# state        public
#----------------------------------------------------------------------------------------
# syntax       SelectPackstation($Basket, $PostNumber, $Address);
#----------------------------------------------------------------------------------------
# description  Select a packstation to use
#----------------------------------------------------------------------------------------
# input        $Basket | the basket object | object
# input        $PostNumber | the PostNumber of the customer | string
# input        $Address | the shipping address | object
# return       -
#========================================================================================
sub SelectPackstation {
    my ($Basket, $PostNumber, $PackstationAddress) = @_;

    my ($PackstationId, $Title, $Zip, $City) = $PackstationAddress =~ /^(\d+)~~~(.+)~~~(\d+)~~~(.+)$/x;

    my $IsValidPackstation = isValid($PostNumber) && isValid($PackstationId) && isValid($Title)
        && isValid($Zip) && isValid($City);
    return _DeletePackstationAddress($Basket->get('ShippingAddress')) unless($IsValidPackstation);

    my $ShippingAddress = $Basket->get('ShippingAddress') // $Basket->get('BillingAddress');
    return unless(defined($ShippingAddress));

    my $PickupStation = $ShippingAddress->get('SelectedShippingPickupStation');
    # create a new shipping address only if necessary
    unless(defined($PickupStation) && $ShippingAddress->get('DHLPackstationId') eq $PackstationId) {
        my $hPackstationData = {
            'AddressExtension'              => $PostNumber,
            'Street'                        => $Title,
            'Zipcode'                       => $Zip,
            'City'                          => $City,
            'IsPickupAddress'               => 1,
            'SelectedShippingPickupStation' => LoadObjectByPath('/ShippingPickupStations/DHLPackstation'),
            'DHLPackstationId'              => $PackstationId,
        };

        my $hAddressData = $ShippingAddress->get([qw (FirstName LastName DisplayName CountryID)]);
        $hPackstationData->{$_} = $hAddressData->{$_} foreach (keys %$hAddressData);

        my $NewShippingAddress = CreatePackstationAddress($Basket, $hPackstationData);
        $Basket->set({'ShippingAddress' => $NewShippingAddress});
    }
}

#========================================================================================
# function     CreatePackstationAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreatePackstationAddress($Basket, $hAddress);
#----------------------------------------------------------------------------------------
# description  Create an address object for a Packstation
#----------------------------------------------------------------------------------------
# input        $Basket | the basket object | object
# input        $hAddress | the address of the Packstation | ref.hash
# return       $Address | address object | object
#========================================================================================
sub CreatePackstationAddress {
    my ($Basket, $hAddressData) = @_;

    my $User = $Basket->get('User');
    my $UserAddresses = $User->folder('Addresses');
    $hAddressData->{'Alias'} = $UserAddresses->nextAlias('PackstationAddress%d');

    my $NewAddress = $UserAddresses->insertChild('Address', $hAddressData);
    return $NewAddress;
}

#========================================================================================
# function     DeletePackstationAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeletePackstationAddress($Address);
#----------------------------------------------------------------------------------------
# description  Delete address if it's a Packstation address
#----------------------------------------------------------------------------------------
# input        $Address | the address of the Packstation | object
# return       -
#========================================================================================
sub DeletePackstationAddress {
    my ($ShippingAddress) = @_;
    $ShippingAddress->delete if(defined($ShippingAddress) && isValid($ShippingAddress->get('DHLPackstationId')));

    return;
}

1;
