#========================================================================================
# package      DE_EPAGES::DHL::UI::DHLShippingLabel
# state        private
#========================================================================================
package DE_EPAGES::DHL::UI::DHLShippingLabel;
use base DE_EPAGES::Order::UI::OrderDocument;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime ElementOf );
use DE_EPAGES::Core::API::String qw ( QuoteXML );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::DHL::API::DHL qw (
    LoadBoughtShoppingCart
    ShoppingCartCheckoutDirect
    ShoppingCartCheckoutViaPayment
    ValidateShoppingCart
);
use DE_EPAGES::DHL::API::DHLShippingLabel qw (
    CheckPartialShipment
    CreateNewDHLShippingLabel
    CreateNewSubDHLShippingLabel
);
use DE_EPAGES::DHL::API::Tools qw (
    ConvertFromDHLDate
    GetDHLProductFromDB
    GetDHLProductMaxDimensionsFromDB
    GetMaxShippingHeightFromDB
    GetMaxShippingLengthFromDB
    GetMaxShippingWeightFromDB
    GetMaxShippingWidthFromDB
    GetMinShippingHeightFromDB
    GetMinShippingLengthFromDB
    GetMinShippingWidthFromDB
    IsInteger
);
use DE_EPAGES::Object::API::Factory qw ( ExistsObject );
use DE_EPAGES::Presentation::API::Session qw (
    ExistsSessionAttribute
    GetSessionAttribute
    SetSessionAttribute
);
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use HTTP::Status;

#==============================================================================
# Event handlers
#==============================================================================

sub NewDHLShippingLabel {
    my $self = shift;
    my ($Servlet) = @_;
    my $Order = $Servlet->object();

    CreateNewDHLShippingLabel($Order);
    #Reset session attribute
    SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );

    return;
}


sub NewSubDHLShippingLabel {
    my $self = shift;
    my ($Servlet) = @_;
    #get Order out of the DHLShippingLabel
    my $Order = $Servlet->object()->get('Order');
    return CreateNewSubDHLShippingLabel($Order);
}


sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $DHLShippingLabel = $Servlet->object();

    #--- standard view
    $self->SUPER::View($Servlet);

    TestMaxShippingWeight($DHLShippingLabel, undef, $Servlet);

    #--- set max dimensions for the Pckchen as TLE-Variables
    my $hDHLProductMaxDimensions = GetDHLProductMaxDimensionsFromDB('PAECK.DEU');
    $Servlet->vars('MaxPAECKLengthCM', $hDHLProductMaxDimensions->{'MaxLengthCm'});
    $Servlet->vars('MaxPAECKWidthCM' , $hDHLProductMaxDimensions->{'MaxWidthCm'});
    $Servlet->vars('MaxPAECKHeightCM', $hDHLProductMaxDimensions->{'MaxHeightCm'});

    #--- set max dimensions for the Paket as TLE-Variables
    $hDHLProductMaxDimensions = GetDHLProductMaxDimensionsFromDB('PAK02.DEU');
    $Servlet->vars('MaxPAKLengthCM', $hDHLProductMaxDimensions->{'MaxLengthCm'});
    $Servlet->vars('MaxPAKWidthCM' , $hDHLProductMaxDimensions->{'MaxWidthCm'});
    $Servlet->vars('MaxPAKHeightCM', $hDHLProductMaxDimensions->{'MaxHeightCm'});

    #Check if a dhl product exists with the given attributes and set the TLE-Variables
    BlockWarnings(sub {
        eval {
            my $hDHLProduct = GetDHLProductFromDB($DHLShippingLabel);
            my $DHLProductID = $hDHLProduct->{'DHLProductID'};
            $DHLShippingLabel->set( {'ProductId' => $DHLProductID } );
            $Servlet->vars('DisplayName', $hDHLProduct->{'DisplayName'});
            $Servlet->vars('DisplayText', $hDHLProduct->{'DisplayText'});
            $Servlet->vars('OverallPriceAmountEUR', $hDHLProduct->{'OverallPriceAmountEUR'});
        };
        if (ExistsError()) {
            my $Error = GetError();
            Warning($Error);
        }
    }, [qw ( MISSING_ADDRESS MISSING_COUNTRY DHLMaxShippingDimensionExceeded GetProductsFromDBErrorNoSuchProduct )]);

    my $ToBeFinalisedDHLShippingLabelIDs = GetSessionAttribute(
        $Servlet->intSessionID, 'ToBeFinalisedDHLShippingLabelIDs' )//'';
    my @ToBeFinalisedDHLShippingLabelIDs = split(/,/,$ToBeFinalisedDHLShippingLabelIDs);
    $Servlet->vars('ToBeFinalisedDHLShippingLabelIDs',scalar @ToBeFinalisedDHLShippingLabelIDs);

    return;
}


sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $DHLShippingLabel = $Servlet->object();
    my $Shop = $DHLShippingLabel->getSite();
    my $LineItemForm = $Servlet->form->form($DHLShippingLabel, 'SaveDHL');

    #--- standard save
    $self->SUPER::Save($Servlet);

    TestMaxShippingWeight($DHLShippingLabel, $LineItemForm, $Servlet);

    #--- Test the shipping dimensions
    if (  defined($LineItemForm->{'Length'})
      && defined($LineItemForm->{'Width'})
      && defined($LineItemForm->{'Depth'}) ) {
        my $ShippingLength = $LineItemForm->{'Length'};
        my $ShippingWidth = $LineItemForm->{'Width'};
        my $ShippingHeight = $LineItemForm->{'Depth'};
        my $MinShippingLengthFromDB = GetMinShippingLengthFromDB();
        my $MinShippingWidthFromDB  = GetMinShippingWidthFromDB();
        my $MinShippingHeightFromDB = GetMinShippingHeightFromDB();
        my $MaxShippingLengthFromDB = GetMaxShippingLengthFromDB();
        my $MaxShippingWidthFromDB  = GetMaxShippingWidthFromDB();
        my $MaxShippingHeightFromDB = GetMaxShippingHeightFromDB();
        #Throw error if the min dimensions are not reached
        if ($ShippingLength < $MinShippingLengthFromDB
         || $ShippingWidth < $MinShippingWidthFromDB
         || $ShippingHeight < $MinShippingHeightFromDB) {
            $Servlet->form->addFormError({
                     'Name'      => 'DHLMinShippingDimensionNotReached',
                     'Reason'    => 'DHLMinShippingDimensionNotReached',
           });
           $Servlet->form->executeFormError;
        }
        #Throw error if the max dimensions are exceeded
        if ($ShippingLength > $MaxShippingLengthFromDB
         || $ShippingWidth > $MaxShippingWidthFromDB
         || $ShippingHeight > $MaxShippingHeightFromDB) {
            $Servlet->form->addFormError({
                     'Name'      => 'DHLMaxShippingDimensionExceeded',
                     'Reason'    => 'DHLMaxShippingDimensionExceeded',
           });
           $Servlet->form->executeFormError;
        }
    }

    #--- Save the PickupDate
    if ($DHLShippingLabel->get('PickupService')) {
        if ( defined($LineItemForm->{'PickupDateWithoutTime'}) && $LineItemForm->{'PickupDateWithoutTime'} ne '' ) {
            # Pickup date is set
            my $PickupDateTime = $LineItemForm->{'PickupDateWithoutTime'};
            #Check if the date is a sunday
            my $Day_of_week = $PickupDateTime->{'local_c'}->{'day_of_week'};
            if ($Day_of_week == 7) {
                $Servlet->form->executeFormError({
                          'Name'      => 'PickupDate',
                          'Reason'    => 'PickupDateIsSunday',
                });
            }

            $PickupDateTime->set_time_zone('floating');
            my $Delta = $PickupDateTime - GetCurrentDateTime()->truncate( to => 'day' );
            my $DeltaDays = $Delta->in_units('days');

            #Check if the date is in the past
            if ($DeltaDays < 0) {
                $Servlet->form->executeFormError({
                          'Name'      => 'PickupDate',
                          'Reason'    => 'PickupDateIsInThePast',
                });
            }
            #Check if the date is today
            elsif ($DeltaDays == 0) {
                $Servlet->form->executeFormError({
                          'Name'      => 'PickupDate',
                          'Reason'    => 'PickupDateIsToday',
                });
            }
            $DHLShippingLabel->set( {'PickupDate' => $PickupDateTime } );
        }
        else {
            #Pickup date is not set
            $Servlet->form->executeFormError({
                          'Name'      => 'PickupDate',
                          'Reason'    => 'PickupDateIsNotSet',
            });
        }
    }

    #--- save the entered product weights and weight units
    my $LineItemContainer = $DHLShippingLabel->get('LineItemContainer');
    my $aLineItems = $LineItemForm->{'LineItems'};
    foreach my $Index ( 0 .. scalar(@$aLineItems) - 1 ) {
        my $LineItemEntry = @$aLineItems[$Index];
        my $ObjectID = $LineItemEntry->{'LineItemID'};
        my $LineItem = $Shop->loadObject($ObjectID, 'LineItemProduct');
        die 'invalid LineItem:'.$LineItem->id.' used in DHLShippingLabel:'
          .$DHLShippingLabel->id if $LineItem->get('LineItemContainer')->id != $LineItemContainer->id;
        if ( $LineItemEntry->{'Weight'} &&
             $LineItemEntry->{'WeightUnitID'}) {
            my $NewProductWeight = $LineItemEntry->{'Weight'};
            my $NewProductWeightUnit = $Shop->loadObject($LineItemEntry->{'WeightUnitID'}, 'UnitOfMeasurement', {'AllowSystemSite' => 1});
            $LineItem->set( {'Weight'     => $NewProductWeight,
                             'WeightUnit' => $NewProductWeightUnit,
                           } );
        }
    }

    #Check if a dhl product exists with the given attributes
    eval {
        my $hDHLProduct = GetDHLProductFromDB($DHLShippingLabel);
        my $DHLProductID = $hDHLProduct->{'DHLProductID'};
        $DHLShippingLabel->set( {'ProductId' => $DHLProductID } );
        $Servlet->vars('DisplayName', $hDHLProduct->{'DisplayName'});
        $Servlet->vars('DisplayText', $hDHLProduct->{'DisplayText'});
        $Servlet->vars('OverallPriceAmountEUR', $hDHLProduct->{'OverallPriceAmountEUR'});
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ( $Error->code eq 'GetProductsFromDBError' ) {
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            $Servlet->form->executeFormError({
                'Reason' => 'GetProductsFromDBError',
                'Name'   => 'GetProductsFromDBError',
            });
        }
        elsif ( $Error->code eq 'GetProductsFromDBErrorNoSuchProduct' ) {
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            $Servlet->form->executeFormError({
                'Reason' => 'GetProductsFromDBErrorNoSuchProduct',
                'Name'   => 'GetProductsFromDBErrorNoSuchProduct',
            });
        }
    }

    #Issue the DHL shipping label if it has a manually entered shipmentid
    my $ShipmentID = $LineItemForm->{'ShipmentId'} // '';
    #delete all whitespaces
    $ShipmentID =~ s/\s+//g;
    if ($ShipmentID ne '') {
        $DHLShippingLabel->set({ 'PickupService' => 0,
                                 'PickupDate'    => undef,
                                 'Issued'        => 1,
                           });
    }

    return;
}


sub TestMaxShippingWeight {
    my ($DHLShippingLabel, $LineItemForm, $Servlet) = @_;

    my $OverallWeight = undef;
    # Function was called by the save fuction
    if ( defined($LineItemForm) ) {
        if ( defined( $LineItemForm->{'OverallWeight'} ) ) {
            $OverallWeight = $LineItemForm->{'OverallWeight'};
        }
    }
    # Function was called by the view function
    else {
        my $CalculatedOverallWeight = 0;
        my $aLineItems = $DHLShippingLabel->get('LineItemContainer')->get('LineItems');
        foreach my $LineItem ( @$aLineItems ) {
            my $Quantity       = $LineItem->get('Quantity');
            my $Weight         = $LineItem->get('Weight');
            my $WeightUnit     = $LineItem->get('WeightUnit');
            my $WeightQuantity = $LineItem->get('WeightQuantity');
            $WeightQuantity  ||= 1;

            if (defined $Weight and defined $WeightUnit) {
                #--- convert weights to gram and add weights for all line items
                if ($WeightUnit->alias ne 'gram') {
                    $Weight *= $WeightUnit->get('Factor');
                }
                $CalculatedOverallWeight += $Weight * $Quantity / $WeightQuantity;
            }
        }
        #Convert $CalculatedOverallWeight from gram to kilogram
        $OverallWeight = $CalculatedOverallWeight / 1000;;##no critic (ProhibitMagicNumbers)
    }

    #--- Test the shipping weight
    if ( defined($OverallWeight) ) {
        my $MaxWeightFromDB = GetMaxShippingWeightFromDB();
        #Throw an error if a DHL Maxi Transport would be needed
        if ( $OverallWeight > $MaxWeightFromDB ) {
            #Has to be a DHL Maxi Transport (currently not included)
            # Function was called by the save fuction
            if ( defined($LineItemForm) ) {
                $Servlet->form->executeFormError({
                      'Name'      => 'OverallWeightTooHigh',
                      'Reason'    => 'OverallWeightTooHigh',
                });
            }
            # Function was called by the view function
            else {
                $Servlet->vars('Error_OverallWeightTooHigh', '1');
            }
        }
    }
    return;
}


sub Send {
    my $self = shift;
    my ($Servlet) = @_;

    my $DHLShippingLabel = $Servlet->object();

    # save before send
    $self->Save($Servlet);

    #Set DHLShippingLabelIDs as SessionAttribute
    my $DHLShippingLabelIDs = '';
    my $IsDHLShippingLabelIDSet = 0;
    if (ExistsSessionAttribute('DHLShippingLabelIDs')) {
        if (GetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs') ne ' ') {
            $DHLShippingLabelIDs = GetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs');
            $IsDHLShippingLabelIDSet = _IsDHLShippingLabelIDSet($DHLShippingLabelIDs, $DHLShippingLabel->id);
            if (not $IsDHLShippingLabelIDSet) {
                $DHLShippingLabelIDs .= ',' if ($DHLShippingLabelIDs);
            }
        }
    }
    if (not $IsDHLShippingLabelIDSet) {
        $DHLShippingLabelIDs .= $DHLShippingLabel->id;
    }
    SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', $DHLShippingLabelIDs );

    #Check partial shipment
    #-----------------------------------
    my $IsPartialShipment = CheckPartialShipment($DHLShippingLabel->get('Order'));
    my @DHLShippingLabels;

    if($IsPartialShipment) {
        # redirect to ask whether a new shipping label shall be created or not
        $Servlet->vars('ViewAction', 'MBO-ViewNewDHLShippingLabelQuestion');
    }
    else {
        my $Shop = $DHLShippingLabel->getSite();
        my @DHLShippingLabelIDs = split(/,/,$DHLShippingLabelIDs);
        foreach my $DHLShippingLabelID (@DHLShippingLabelIDs) {
            if (IsInteger($DHLShippingLabelID) && ExistsObject($DHLShippingLabelID)) {
                my $DHLShippingLabelObj = $Shop->loadObject($DHLShippingLabelID, 'DHLShippingLabel');
                if ($DHLShippingLabelObj->instanceOf('DHLShippingLabel')) {
                    # Keine DHLShippingLabels an DHL senden, wenn eine ShipmentID (=Issued) eingetragen wurde.
                    push (@DHLShippingLabels, $DHLShippingLabelObj) unless ($DHLShippingLabelObj->get('Issued'));
                }
                else {
                    GetLog->warn('DHL Send: The object with DHLShippingLabelID: '.$DHLShippingLabelID
                        .' is not an instance of DHLShippingLabel.');
                }
            }
            else {
                GetLog->warn('DHL Send: DHLShippingLabelID: '.$DHLShippingLabelID
                    .' is no Integer or Object does not exist.');
            }
        }
        _ConnectToDHL(\@DHLShippingLabels,$Servlet);
    }
    #-----------------------------------

    return;
}


#---
#
#---
sub SendWithoutPartialShipmentCheck {
    my $self = shift;
    my ($Servlet) = @_;

    #Load DHLShippingLabelIDs from the session attributes
    my $DHLShippingLabelIDs = GetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs' );
    my @DHLShippingLabelIDs = split(/,/,$DHLShippingLabelIDs);
    my $Shop = $Servlet->object()->getSite();

    #Build array @DHLShippingLabels
    my @DHLShippingLabels;
    foreach my $DHLShippingLabelID (@DHLShippingLabelIDs) {
        if (IsInteger($DHLShippingLabelID) && ExistsObject($DHLShippingLabelID)) {
            my $DHLShippingLabel = $Shop->loadObject($DHLShippingLabelID, 'DHLShippingLabel');
            if ($DHLShippingLabel->instanceOf('DHLShippingLabel')) {
                # Keine DHLShippingLabels an DHL senden, wenn eine ShipmentID eingetragen wurde.
                push (@DHLShippingLabels, $DHLShippingLabel) unless ($DHLShippingLabel->get('Issued'));
            }
            else {
                GetLog->warn('DHL SendWithoutPartialShipmentCheck: The object with DHLShippingLabelID: '
                    .$DHLShippingLabelID.' is not an instance of DHLShippingLabel.');
            }
        }
        else {
            GetLog->warn('DHL SendWithoutPartialShipmentCheck: DHLShippingLabelID: '.$DHLShippingLabelID
                .' is no Integer or Object does not exist.');
        }
    }

    #Send DHLShippingLabels to DHL
    _ConnectToDHL(\@DHLShippingLabels,$Servlet);

    #Reset session attribute
    SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );

    return;
}


sub _ConnectToDHL {
    my ($aDHLShippingLabels, $Servlet) = @_;

    # quick back out.
    return unless (scalar @{$aDHLShippingLabels});

    my $rValidateContent = '';
    my $ValidateResponseStatus = 'undefined';
    eval {
        $rValidateContent = ValidateShoppingCart($aDHLShippingLabels);
        $ValidateResponseStatus = $rValidateContent->dataof('//ShoppingCartValidateResponse/Status')->attr->{'status'};
    };
    if (ExistsError()) {
        #Reset session attribute
        my $Error = GetError();
        my $ErrorCode = $Error->code;

        if (ElementOf($ErrorCode,
                qw(OverallWeightTooHigh DHLMinShippingDimensionNotReached
                   DHLMaxShippingDimensionExceeded
                   DHLShippingToThatCountryNotYetSupported
                   GetProductsFromDBError GetProductsFromDBErrorNoSuchProduct))) {
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            $Servlet->form->executeFormError({
                'Reason' => $Error->code,
                  'Name' => $ErrorCode,
            });
        }
        elsif (ElementOf($ErrorCode,
                   qw(MissingShopName MissingShopStreet MissingShopZipCode
                      MissingShopCity MissingShopCountry))) {
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            $Servlet->form->executeFormError({
                'Reason' => 'DHLShopAddressIncomlete',
                  'Name' => 'DHLShopAddressIncomlete',
            });
        }
        elsif (ElementOf($ErrorCode,
                   qw( MissingShippingName MissingShippingStreet MissingShippingZipCode
                       MissingShippingCity MissingShippingCountry))) {
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            $Servlet->form->executeFormError({
                'Reason' => 'DHLShippingAddressIncomlete',
                  'Name' => 'DHLShippingAddressIncomlete',
            });
        }
        else {
            # sample WebService error: (2) Invalid XML; cvc-maxLength-valid:
            # Value 'Max Max MaxMaxMaxMaxMax Max Max Max Max Max Mustermann-Mustermannmustermann' with
            # length = '75' is not facet-valid with respect to maxLength '50' for type 'null'. (env:Client)

            if ($ErrorCode eq 'WebServiceFault' &&
                ## no critic (ProhibitComplexRegexes RequireExtendedFormatting)
                $Error->message =~ /cvc-maxLength-valid: Value '(.*?)' with length .*? maxLength '(\d+)' for type/) {
                ## use critic
                $Servlet->form->executeFormError({
                    'Reason' => 'DHLShoppingCartValidate',
                    'Name' => 'DHLShoppingCartValidate',
                    'Value' => $1,
                    'MaxLength' => $2,
                });
            }

            $Servlet->form->executeFormError({
                'Reason' => 'DHLValidateShoppingCartTimedOut',
                  'Name' => 'DHLValidateShoppingCartTimedOut',
            });
        }
    }

    if ($ValidateResponseStatus == 0) {
        my $rCheckoutContent = '';
        my $CheckoutResponseStatus = 'undefined';
        # User return URL
        my %Params = (
            'ChangeAction'  => ['BlockMonitor', 'CallbackDHLShoppingCartCheckoutViaPayment'],
            'ViewAction'    => 'MBO-ViewDHLShoppingCartRedirect',
        );

        my $BruttoAmount = 0;
        # determine if DirectCheckout has to be used.
        foreach my $PriceComponents ($rValidateContent->dataof(
            '//ShoppingCartValidateResponse/ShoppingCart/LabelItem/Price/*')) {
            $BruttoAmount += $PriceComponents->attr()->{'bruttoAmount'};
        }
        foreach my $PriceComponents ($rValidateContent->dataof(
            '//ShoppingCartValidateResponse/ShoppingCart/PickupItem/Price/*')) {
            $BruttoAmount += $PriceComponents->attr()->{'bruttoAmount'};
        }
        my $UseDirectCheckout = ($BruttoAmount < 0.01 ? 1 : 0);##no critic (ProhibitMagicNumbers)

        eval {
            if ($UseDirectCheckout) {
                $rCheckoutContent = ShoppingCartCheckoutDirect($aDHLShippingLabels, 1);
                $CheckoutResponseStatus = $rCheckoutContent->dataof(
                    '//ShoppingCartCheckoutDirectResponse/Status')->attr->{'status'};

                # on success, jump directly to return url
                if ($CheckoutResponseStatus == 0) {
                    $Params{'scid'} = $rCheckoutContent->dataof(
                        '//ShoppingCartCheckoutDirectResponse/DownloadShoppingCart')->attr->{'shoppingCartId'};
                    my $ReturnURL = BuildShopUrl($aDHLShippingLabels->[0], \%Params, {
                        'UseSSL' => $Servlet->serverVariables->{'SERVER_PORT_SECURE'}, 'Type' => 'admin', 'UseObjectPath' => 1, 'AddToken' => 1}, $Servlet);
                    GetLog->debug('DHL Redirect URL',$ReturnURL);
                    my $Response = $Servlet->response;
                    $Response->code( RC_SEE_OTHER );
                    $Response->header( 'Location' => $ReturnURL);
                    $Response->content('<html><head><meta http-equiv="Refresh" content="0;URL='.QuoteXML($ReturnURL).'"/></head><body></body></html>');
                    $Servlet->vars('ExitAfterEvent',1);
                    return;
                }
            } else {
                my $ReturnURL = BuildShopUrl($aDHLShippingLabels->[0], \%Params, {'UseSSL' =>
                    $Servlet->serverVariables->{'SERVER_PORT_SECURE'}, 'Type' => 'admin', 'UseObjectPath' => 1, 'AddToken' => 1}, $Servlet);
                $rCheckoutContent = ShoppingCartCheckoutViaPayment($aDHLShippingLabels,$ReturnURL);
                $CheckoutResponseStatus = $rCheckoutContent->dataof('//ShoppingCartCheckoutViaPaymentResponse/Status')->attr->{'status'};
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            #Reset session attribute
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            LogDebug('DHLCheckoutShoppingCartViaPaymentTimedOut:',$Error);
            $Servlet->form->addFormError({
                'Name'      => 'DHLCheckoutShoppingCartViaPaymentTimedOut',
                'Reason'    => 'DHLCheckoutShoppingCartViaPaymentTimedOut',
            });
            $Servlet->form->executeFormError;
        }
        if ($CheckoutResponseStatus == 0) {
            return if ($UseDirectCheckout);
            my $paymentUrl = $rCheckoutContent->dataof('//ShoppingCartCheckoutViaPaymentResponse/PaymentForwardShoppingCart')->attr->{'paymentUrl'};

            my $redirectUrl = $paymentUrl;
            # redirect to payment platform
            $Servlet->vars('DHLPaymentRedirectURL', $redirectUrl);
            $Servlet->vars('ViewAction', 'MBO-ViewDHLPaymentRedirect');
        }
        else {
            #ShoppingCartCheckoutViaPayment error
            #Reset session attribute
            SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
            my $ErrorMessage = 'ShoppingCartCheckoutViaPaymentError';
            eval { ## no critic (RequireExistsError)
                $ErrorMessage = $rCheckoutContent->dataof(
                    '//BuyShoppingCart/LabelItem/ItemError/ItemError[@messageType=\'BLOCKINGERROR\']')->attr->{'message'}; ##no critic(RequireInterpolationOfMetachars)
            };
            $Servlet->vars('DHLShoppingCartCheckoutViaPaymentErrorMessage', $ErrorMessage);
            $Servlet->form->addFormError({
                'Name'      => 'DHLShoppingCartCheckoutViaPayment',
                'Reason'    => 'DHLShoppingCartCheckoutViaPayment',
            });
            $Servlet->form->executeFormError;
        }
    }
    else {
        #ShoppingCartValidate error
        #Reset session attribute
        SetSessionAttribute( $Servlet->intSessionID, 'DHLShippingLabelIDs', '' );
        my $ErrorMessage = '';
        my $DetailedErrorMessage = '';
        if ($ValidateResponseStatus == RC_UNAUTHORIZED) {
            eval { ## no critic (RequireExistsError)
                foreach my $ErrorElement ($rValidateContent->dataof('//ShoppingCartValidateResponse/ShoppingCart/*/ItemError/ItemError')) {
                    $ErrorMessage = $ErrorElement->attr->{'i18nkey'};
                    $DetailedErrorMessage = $ErrorElement->attr->{'message'};
                    my $hErrorMap = {
                        'pop.shoppingcart.items.errors.collectiondataincomplete'    => {
                            'reason_ext' => 'CollectionDataIncomplete',
                         },
                        'pop.shoppingcart.errors.nosenderleitcodetas'               => {
                            'reason_ext' => 'NoSenderLeitcodeTAS',
                        },
                        'pop.shoppingcart.items.errors.collectiondatenobusinessday' => {
                            'reason_ext' => 'CollectionDateNoBusinessDay',
                            'name'       => 'PickupService',
                            'reason'     => 'PickupService'
                        },
                        'web.now1.errors.voucher.novoucher'                         => {
                            'reason'     => 'CouponCode',
                            'name'       => 'CouponCode',
                        },
                        'web.now1.errors.voucher.promomaxuseslimit'                 => {
                            'reason'     => 'CouponCode',
                            'name'       => 'CouponCode',
                        },
                        'pop.maptos.unknown_produktkey'                             => {
                            'reason_ext' => 'UnknownProductKey',
                        },
                        'web.now1.errors.voucher.promomaxuseslimit'                 => {
                            'reason_ext' => 'CouponCodeNotValid',
                        },
                        'pop.shoppingcart.items.errors.senderaddrincomplete'        => {
                            'name'       => 'DHLShopAddressIncomlete',
                            'reason'     => 'DHLShopAddressIncomlete',
                        },
                        'pop.shoppingcart.items.errors.receiveraddrincomplete'      => {
                            'name'       => 'DHLShippingAddressIncomlete',
                            'reason'     => 'DHLShippingAddressIncomlete',
                        }
                    };

                    my $hError = $hErrorMap->{$ErrorMessage};
                    if($hError && ($hError->{'name'} || $hError->{'reason'})) {
                        $Servlet->form->addFormError({
                            'Name'     => $hError->{'name'} // $hError->{'reason'},
                            'Reason'   => $hError->{'reason'} // $hError->{'name'},
                        });
                    }

                    $Servlet->form->addFormError({
                        'Name'      => 'DHLShoppingCartValidate',
                        'Reason'    => 'DHLShoppingCartValidate'.$hError->{'reason_ext'},
                        'Detail'    => $DetailedErrorMessage,
                    }) if ($hError->{'reason_ext'});
                    $Servlet->vars('DHLShoppingCartValidateErrorMessage', $DetailedErrorMessage) if ($DetailedErrorMessage);
                }
            };
        }
        $Servlet->form->executeFormError;
    }
}


#---
# Callback Function (Callback from the payment platform)
#---
sub CallbackDHLShoppingCartCheckoutViaPayment {
    my $self = shift;
    my ($Servlet) = @_;
    my $ServletDHLShippingLabel = $Servlet->object();
    my $Shop                    = $ServletDHLShippingLabel->getSite;
    $self->BlockMonitor($Servlet);

    my $FormValues = $Servlet->params();
    my $ShoppingCartId = $FormValues->{'scid'};

    my $rLoadContent = '';
    my $LoadResponseStatus = 'undefined';
    eval {
        $rLoadContent = LoadBoughtShoppingCart($ServletDHLShippingLabel, $ShoppingCartId);
        $LoadResponseStatus = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/Status')->attr->{'status'};
    };
    if (ExistsError()) {
        $Servlet->form->addFormError({
            'Name'      => 'DHLLoadBoughtShoppingCartTimedOut',
            'Reason'    => 'DHLLoadBoughtShoppingCartTimedOut',
        });
        $Servlet->form->executeFormError;
    }
    if ($LoadResponseStatus == 0) {
        #Get the attributes of the element PaymentInfo
        my $PaymentInfo    = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/PaymentInfo');
        my $Status         = $PaymentInfo->attr->{'status'};
        my $StatusDate     = $PaymentInfo->attr->{'statusDate'};
        my $PaymentMethod  = $PaymentInfo->attr->{'paymentMethod'};
        my $ConfirmStatus  = $PaymentInfo->attr->{'confirmStatus'};
        my $PaymentId      = $PaymentInfo->attr->{'paymentId'};
        my $TransactionId1 = $PaymentInfo->attr->{'transactionId1'}//'-'; # needs to be not-null.
        my $TransactionId2 = $PaymentInfo->attr->{'transactionId2'};
        my $PaymentCode    = $PaymentInfo->attr->{'paymentCode'};
        my $PaymentMessage = '';
        eval { ## no critic (RequireExistsError)
            $PaymentMessage = $PaymentInfo->attr->{'paymentMessage'};
        };

        #Get the attributes of the element DownloadShoppingCart
        my $DownloadShoppingCart = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart');
        my $AppletUrl            = $DownloadShoppingCart->attr->{'appletUrl'};
        my $DownloadPageUrl      = $DownloadShoppingCart->attr->{'downloadPageUrl'};

        my $ShoppingCartAmount = 0.0;
        my $DHLShoppingCart = $Shop->folder('DHLShoppingCarts')->insertChild('DHLShoppingCart', {
                                      'Alias'             => $Shop->folder('DHLShoppingCarts')->nextAlias('DHL-ShoppingCart-%d'),
                                      'CurrencyID'        => 'EUR',
                                      'Amount'            => '0.0',
                                      'ShoppingCartID'    => $ShoppingCartId,
                                      'Status'            => $Status,
                                      'StatusDate'        => ConvertFromDHLDate($StatusDate),
                                      'AppletUrl'         => $AppletUrl,
                                      'DownloadPageUrl'   => $DownloadPageUrl,
                                      'PaymentMethod'     => $PaymentMethod,
                                      'ConfirmStatus'     => $ConfirmStatus,
                                      'PaymentId'         => $PaymentId,
                                      'TransactionId1'    => $TransactionId1,
                                      'TransactionId2'    => $TransactionId2,
                                      'PaymentCode'       => $PaymentCode,
                                      'PaymentMessage'    => $PaymentMessage,
        });

        # Get the PickupItem if it is defined in the response of DHL
        my $ShoppingCartPickupDate;
        my $IsPickupShoppingCart = 0;
        eval { ## no critic (RequireExistsError)
            # Load the price components of the PickupItem
            my $PickupItemPriceComponent = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/PickupItem/Price/PriceComponents');
            my $PickupItemBruttoAmount   = $PickupItemPriceComponent->attr()->{'bruttoAmount'};
            my $PickupItemVat            = $PickupItemPriceComponent->attr()->{'vat'};
            my $PickupItemCurPickupTime  = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/PickupItem/curPickupTime');
            my $PickupItemProductId      = $PickupItemCurPickupTime->attr()->{'pickupType'};
            $ShoppingCartPickupDate      = $PickupItemCurPickupTime->attr()->{'pickupDate'};
            # Update the shoppingcart amount
            $ShoppingCartAmount += $PickupItemBruttoAmount;
            # Set the PriceComponent attributes to the DHLShoppingCart
            $DHLShoppingCart->set({ 'PickupItemBruttoAmount' => $PickupItemBruttoAmount,
                                    'PickupItemVat'          => $PickupItemVat,
                                    'PickupItemProductId'   => $PickupItemProductId,
                                    'ShoppingCartPickupDate' => ConvertFromDHLDate($ShoppingCartPickupDate),
                                  });
            $IsPickupShoppingCart = 1;
        };

        #Update every DHLShippingLabel
        my $i = 0;
        foreach my $LabelItem ($rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/LabelItem')) {
            $i++;
            #Get the attributes of the element LabelItem
            my $ShipmentId         = $LabelItem->attr->{'shipmentId'};
            my $LabelId            = $LabelItem->attr->{'labelId'};
            my $ProductId          = $LabelItem->attr->{'productId'};
            my $AwtDownloadUrl     = $LabelItem->attr->{'awtDownloadUrl'};
            my $PdfDownloadUrl     = $LabelItem->attr->{'pdfDownloadUrl'};
            my $RoutingCode        = $LabelItem->attr->{'routingCode'};
            my $DHLShippingLabelID = $LabelItem->attr->{'customerReference'};

            #Get the attributes of the element PriceComponents from the array @PriceComponents
            my $Vat = 0;
            my $BruttoAmount = 0.0;
            foreach my $PriceComponents ($rLoadContent->dataof("//LoadBuyedShoppingCartResponse/DownloadShoppingCart/[$i]/Price/*")) {
                $Vat += $PriceComponents->attr()->{'vat'};
                $BruttoAmount += $PriceComponents->attr()->{'bruttoAmount'};
            }

            #Load DHLShippingLabel set the values
            if (IsInteger($DHLShippingLabelID) && ExistsObject($DHLShippingLabelID)) {
                my $DHLShippingLabel = $Shop->loadObject($DHLShippingLabelID, 'DHLShippingLabel');
                if ($DHLShippingLabel->instanceOf('DHLShippingLabel')) {
                    $DHLShippingLabel->set({'DHLShoppingCart'   => $DHLShoppingCart,
                                            'ShipmentId'        => $ShipmentId,
                                            'LabelId'           => $LabelId,
                                            'ProductId'         => $ProductId,
                                            'AwtDownloadUrl'    => $AwtDownloadUrl,
                                            'PdfDownloadUrl'    => $PdfDownloadUrl,
                                            'RoutingCode'       => $RoutingCode,
                                            'Vat'               => $Vat,
                                            'BruttoAmount'      => $BruttoAmount,
                                            'CurrencyID'        => 'EUR',
                                            'Issued'            => 1,
                    });
                    # Set the pickup date for the DHLShippingLabel if the DHLShoppingCart has a PickupItem
                    if($IsPickupShoppingCart) {
                        $DHLShippingLabel->set({'PickupService' => 1,
                                                'PickupDate'    => ConvertFromDHLDate($ShoppingCartPickupDate),
                        });
                    }
                }
                else {
                    GetLog->warn('DHL CallbackDHLShoppingCartCheckoutViaPayment:','The object with DHLShippingLabelID: '
                        .$DHLShippingLabelID.' is not an instance of DHLShippingLabel.');
                }
            }
            $ShoppingCartAmount += $BruttoAmount;
        }

        #Set calculated shoppingcart amount
        $DHLShoppingCart->set({ 'Amount' => $ShoppingCartAmount });
    }
    else {
        #LoadBuyedShoppingCart error
        my $ErrorMessage = 'LoadBuyedShoppingCartError';
        eval { ## no critic (RequireExistsError)
            $ErrorMessage = $rLoadContent->dataof('//LoadBuyedShoppingCartResponse/DownloadShoppingCart/LabelItem/ItemError/ItemError')->attr->{'message'};
        };
        $Servlet->vars('DHLLoadBuyedShoppingCartErrorMessage', $ErrorMessage);
        $Servlet->form->addFormError({
            'Name'      => 'DHLLoadBuyedShoppingCart',
            'Reason'    => 'DHLLoadBuyedShoppingCart',
        });
        $Servlet->vars('ViewAction','MBO-ViewGeneral');
        $Servlet->form->executeFormError;
    }
    return;
}


#---
# Redirect to the ViewAction 'MBO-ViewGeneral' of the DHLShoppingCart
#---
sub ViewDHLShoppingCartRedirect {
    my $self = shift;
    my ($Servlet) = @_;
    my $DHLShippingLabel = $Servlet->object();
    my $DHLShoppingCart = $DHLShippingLabel->get('DHLShoppingCart');

    if (!$DHLShoppingCart) {
        GetLog->warn( 'Shopping cart is null - this should not happen. ShippingLabel:'.$DHLShippingLabel->pathString);
        return;
    }

    # Build redirect URL to the DHLShoppingCart
    my %Params = (
        'ViewAction'    => 'MBO-ViewGeneral',
    );
    my $DHLShoppingCartRedirectURL = BuildShopUrl($DHLShoppingCart, \%Params, {'UseSSL' =>
        $Servlet->serverVariables->{'SERVER_PORT_SECURE'}, 'Type' => 'admin', 'UseObjectPath' => 1, 'AddToken' => 1}, $Servlet);

    # redirect to the DHLShoppingCart
    my $Response = $Servlet->response;
    $Response->code( RC_SEE_OTHER );
    $Response->header( 'Location' => $DHLShoppingCartRedirectURL);
    $Servlet->vars('ExitAfterEvent',1);

    return;
}


sub SendGeneratedDHLShippingLabels {
    my $self = shift;
    my ($Servlet) = @_;

    # save before send
    $self->Save($Servlet);

    #Load DHLShippingLabelIDs which have to be finalised from the session attributes
    my $ToBeFinalisedDHLShippingLabelIDs = GetSessionAttribute( $Servlet->intSessionID, 'ToBeFinalisedDHLShippingLabelIDs' );
    my @ToBeFinalisedDHLShippingLabelIDs = split(/,/,$ToBeFinalisedDHLShippingLabelIDs);
    #Build array @ToBeFinalisedDHLShippingLabels
    my @ToBeFinalisedDHLShippingLabels;
    my $Shop = $Servlet->object()->getSite();

    foreach my $ToBeFinalisedDHLShippingLabelID (@ToBeFinalisedDHLShippingLabelIDs) {
        if (IsInteger($ToBeFinalisedDHLShippingLabelID) && ExistsObject($ToBeFinalisedDHLShippingLabelID)) {
            my $ToBeFinalisedDHLShippingLabel = $Shop->loadObject($ToBeFinalisedDHLShippingLabelID, 'DHLShippingLabel');
            if ($ToBeFinalisedDHLShippingLabel->instanceOf('DHLShippingLabel')) {
                push (@ToBeFinalisedDHLShippingLabels, $ToBeFinalisedDHLShippingLabel);
            }
            else {
                GetLog->warn('DHL SendGeneratedDHLShippingLabels: The object with ToBeFinalisedDHLShippingLabelID: '
                    .$ToBeFinalisedDHLShippingLabelID.' is not an instance of DHLShippingLabel.');
            }
        }
        else {
            GetLog->warn('DHL SendGeneratedDHLShippingLabels: ToBeFinalisedDHLShippingLabelID: '
                .$ToBeFinalisedDHLShippingLabelID.' is no Integer or Object does not exist.');
        }
    }

    #delete the DHLShippingLabel, which is currently displayed
    shift(@ToBeFinalisedDHLShippingLabels);

    #Pass all generated DHLShippingLabels to DHL or finalise them before sending to DHL
    if ( scalar(@ToBeFinalisedDHLShippingLabels) == 0 ) {
        #Send all Labels to DHL
        #Load DHLShippingLabelIDs which will be sent to DHL from the session attributes
        my $GeneratedDHLShippingLabelIDs = GetSessionAttribute( $Servlet->intSessionID, 'GeneratedDHLShippingLabelIDs' );
        my @GeneratedDHLShippingLabelIDs = split(/,/,$GeneratedDHLShippingLabelIDs);
        #Build array @GeneratedDHLShippingLabels
        my @GeneratedDHLShippingLabels;
        foreach my $GeneratedDHLShippingLabelID (@GeneratedDHLShippingLabelIDs) {
            if (IsInteger($GeneratedDHLShippingLabelID) && ExistsObject($GeneratedDHLShippingLabelID)) {
                my $GeneratedDHLShippingLabel = $Shop->loadObject($GeneratedDHLShippingLabelID, 'DHLShippingLabel');
                if ($GeneratedDHLShippingLabel->instanceOf('DHLShippingLabel')) {
                    push (@GeneratedDHLShippingLabels, $GeneratedDHLShippingLabel) unless ($GeneratedDHLShippingLabel->get('Issued'));
                }
                else {
                    GetLog->warn('DHL SendGeneratedDHLShippingLabels:','The object with GeneratedDHLShippingLabelID: '
                        .$GeneratedDHLShippingLabelID.' is not an instance of DHLShippingLabel.');
                }
            }
            else {
                GetLog->warn('DHL SendGeneratedDHLShippingLabels:','GeneratedDHLShippingLabelID: '
                    .$GeneratedDHLShippingLabelID.' is no Integer or Object does not exist.');
            }
        }
        #Send DHLShippingLabels to DHL
        _ConnectToDHL(\@GeneratedDHLShippingLabels,$Servlet);
        #Reset session attribute
        SetSessionAttribute( $Servlet->intSessionID, 'GeneratedDHLShippingLabelIDs', '' );
    }
    else {
        #Finalise the rest of the DHLShippingLabels
        #Set session attribute
        my $FirstToBeFinalisedDHLShippingLabel = shift(@ToBeFinalisedDHLShippingLabels);
        $ToBeFinalisedDHLShippingLabelIDs = '';
        $ToBeFinalisedDHLShippingLabelIDs = $FirstToBeFinalisedDHLShippingLabel->id;
        foreach my $ToBeFinalisedDHLShippingLabel (@ToBeFinalisedDHLShippingLabels) {
            $ToBeFinalisedDHLShippingLabelIDs .= ','.$ToBeFinalisedDHLShippingLabel->id;
        }
        SetSessionAttribute( $Servlet->intSessionID, 'ToBeFinalisedDHLShippingLabelIDs', $ToBeFinalisedDHLShippingLabelIDs );

        #Build redirect URL to view the next DHLShippingLabels
        my %Params = (
            'ViewAction'    => 'MBO-ViewGeneratedDHLShippingLabels',
        );
        my $ToBeFinalisedDHLShippingLabelsRedirectURL = BuildShopUrl($FirstToBeFinalisedDHLShippingLabel, \%Params, {
            'UseSSL' => 1, 'Type' => 'admin', 'UseObjectPath' => 1, 'AddToken' => 1}, $Servlet);

        #redirect to the generated DHLShippingLabels
        my $Response = $Servlet->response;
        $Response->code( RC_SEE_OTHER );
        $Response->header( 'Location' => $ToBeFinalisedDHLShippingLabelsRedirectURL);
        $Servlet->vars('ExitAfterEvent',1);
    }

    return;
}


#---
# Check if the DHLShippingLabelID is in the given list DHLShippingLabelIDs
#---
sub _IsDHLShippingLabelIDSet {
    my ($DHLShippingLabelIDs,$DHLShippingLabelID) = @_;

    my @GivenDHLShippingLabelIDs = split(/,/,$DHLShippingLabelIDs);
    foreach my $GivenDHLShippingLabelID (@GivenDHLShippingLabelIDs) {
        if ($GivenDHLShippingLabelID eq $DHLShippingLabelID) {
            return 1;
        }
    }
    return 0;
}

1;
