#==============================================================================
# package      DE_EPAGES::DHL::UI::JSON
# base         DE_EPAGES::Presentation::UI::Object;
# state        private
#==============================================================================
package DE_EPAGES::DHL::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use if $] < 5.038, 'encoding' => qw(utf8); ##no critic qw(ProhibitMagicNumbers)
use strict;

use LWP;
use XML::Simple;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use Encode qw (encode);
use DE_EPAGES::Core::API::Log qw (GetLog);

#========================================================================================
# function     JSONViewPackstations
# state        public
#----------------------------------------------------------------------------------------
# syntax       JSONViewPackstations( $Servlet );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#----------------------------------------------------------------------------------------
# description  Get a list of packstations close to a given zip code
#========================================================================================
sub JSONViewPackstations {
    my $self = shift;
    my ($Servlet) = @_;

    my $ZipCode = $Servlet->{'Params'}->{'ZipCode'};

    my $RequestURL = 'http://www.dhl.de/maptos/pfind?PARTNER_ID=DHLDS&CHARACTER_ENCODING=utf-8&standorttyp=packstation_paketbox&pmtype=1&postleitzahl=' . $ZipCode;
    my $Response;

    eval {
        my $Request = HTTP::Request->new('GET', $RequestURL);
        my $UserAgent = LWP::UserAgent->new();
        $Response = $UserAgent->request($Request);
    };

    GetLog()->error(GetError()) if ExistsError();

    if (_isMalformedResponse($Response)) {
        $self->writeContentOutput({'StatusCode' => 'ERR'});
        return;
    }

    my $XMLData = encode('utf8', $Response->{'_content'} // '');
    my $StationData = XMLin($XMLData)->{'data'};
    $StationData->{'StatusCode'} = 'OK';
    $StationData = _FormatPackstationData($StationData);
    $self->writeContentOutput($StationData);

    return undef;
}

#========================================================================================
# function     _isMalformedResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isMalformedResponse( $Response );
#----------------------------------------------------------------------------------------
# description  Checks if the response exists and is valid.
#----------------------------------------------------------------------------------------
# input        $Response | The Response to check | HTTP::Response
# return       $ReturnVariable | is malformed Response | boolean
#========================================================================================
sub _isMalformedResponse {
    my ($Response) = @_;

    my $Content = $Response->{'_content'};
    if (!isValid($Content)) {
        return 1;
    }

    my $HtmlStatusCodeOK = 200; ## no critic (ProhibitMagicNumbers)
    my $HtmlStatus = $Response->{'_rc'} // '';
    if ($HtmlStatus ne $HtmlStatusCodeOK) {
        return 1;
    }

    my $PageNotFound = ($Content =~ /404.html/);
    if($PageNotFound) {
        return 1;
    }

    return 0;
}

#========================================================================================
# function     _FormatPackstationData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FormatPackstationData( $Data );
#----------------------------------------------------------------------------------------
# description  extracts relevant data for displaying the stations in Google Maps
#----------------------------------------------------------------------------------------
# input        $Data | station data | ref_hash
# return       $Result | extracted data | ref_hash
#========================================================================================
sub _FormatPackstationData {
    my ($Data) = @_;
    my $Result;

    $Result->{'StatusCode'} = $Data->{'StatusCode'};
    # customer's position
    $Result->{'long'} = $Data->{'position'}->{'longitude'};
    $Result->{'lat'}  = $Data->{'position'}->{'latitude'};

    my $Automats = $Data->{'result'}->{'automats'}->{'automat'};
    my $aLocations = [];

    foreach my $AutomatID (keys %$Automats) {
        my $SrcAutomat = $Automats->{$AutomatID};
        next if($SrcAutomat->{'automatType'} != 4); # only use packstations

        my $Automat = {
            'id'      => $AutomatID,
            'station' => substr($SrcAutomat->{'objectId'}, -3, 3), ## no critic(ProhibitMagicNumbers)
            'long'    => $SrcAutomat->{'location'}->{'longitude'},
            'lat'     => $SrcAutomat->{'location'}->{'latitude'},
            'dist'    => $SrcAutomat->{'location'}->{'distance'},
            'address' => $SrcAutomat->{'address'}->{'street'} . ' ' . ($SrcAutomat->{'address'}->{'streetno'} // ''),
            'zip'     => $SrcAutomat->{'address'}->{'zip'},
            'city'    => $SrcAutomat->{'address'}->{'city'},
            'title'   => $SrcAutomat->{'address'}->{'remark'} // '',
        };

        push(@$aLocations, $Automat);
    }

    # locations by distance
    @$aLocations = sort { $a->{'dist'} - $b->{'dist'}; } @$aLocations;

    $Result->{'locations'} = $aLocations;
    return $Result;
}

1;
