#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest
# state        public
# description  Merchant credentials validator. A request to getManifest DHLVersenden SOAP
#               service with merchant credentials going to be saved is sent in order to
#               validate them.
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest;

use DateTime;
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::DHLIntraship::API::Constants qw ( $DHLVERSENDEN_SHIPMENT_RELEASE );

use strict;

#========================================================================================
sub new {
    my $class = shift;

    my $self = bless { 'getManifest' => \&_createManifestRequest, }, $class;
    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       assemble( $operation, $aShippingLabels );
#----------------------------------------------------------------------------------------
# description  that is the "overloaded like" method that returns the request structure by
#               making a call to "_createManifestRequest" private method
#----------------------------------------------------------------------------------------
# input        $operation | DHLVersenden web service method name to call | String
# input        $hInputParameters | hash with credentials and optionally a date | ref.hash
# return       $Response | response from web servcie | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($operation, $hInputParameters) = @_;

    return unless (defined ($self->{$operation}));
    my $Response = $self->{$operation}($self, $hInputParameters);

    return $Response;
}

#========================================================================================
# function     _createManifestRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hManifestRequest = $self->_createManifestRequest($hInputParameters);
#----------------------------------------------------------------------------------------
# description  Create the hash with all necessary values for the request to getManifest
#               SOAP service:
#               {
#                 'header' => { 'user' => 'xyz', 'signature' => '***' },
#                 'part1' => {
#                   'Version' => { 'majorRelease' => '2', 'minorRelease' => '2' },
#                   'manifestDate' => 'yyyy-mm-dd'
#                  }
#               };
#----------------------------------------------------------------------------------------
# input        $hInputParameters | hash with credentials and optionally a date | ref.hash
# return       $hManifestRequest | manifest service request | ref.hash
#========================================================================================
sub _createManifestRequest {
    my $self = shift;
    my ($hInputParameters) = @_;

    my $hManifestDate = $self->_assembleManifestDate($hInputParameters);

    return {
        %{ $self->_assembleAuthentification($hInputParameters->{'user'}, $hInputParameters->{'signature'}) },
        'part1' => {
                 %{ $self->_assembleVersion() },
                 %{ $hManifestDate },
        }
    };
}

#========================================================================================
# function     _assembleAuthentification
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAuthentification = $self->_assembleAuthentification($User, $Signature);
#----------------------------------------------------------------------------------------
# description  Create the authentification data hash piece for the getManifest request
#----------------------------------------------------------------------------------------
# input        $User | user string | string
# input        $Signature | password string | string
# return       $hAuthentification | DHL Versenden authentification | ref.hash
#========================================================================================
sub _assembleAuthentification {
    my $self = shift;
    my ($User, $Signature) = @_;

    return {
        'header' => {
            'user'      => $User,
            'signature' => $Signature,
        }
    };
}

#========================================================================================
# function     _assembleVersion
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hVersion = $self->_assembleVersion();
#----------------------------------------------------------------------------------------
# description  Create the version data hash piece for the getManifest request
#----------------------------------------------------------------------------------------
# return       $hVersion | DHL Versenden shipping version | ref.hash
#========================================================================================
sub _assembleVersion {
    my $self = shift;

    my @VersionNumbers = ( $DHLVERSENDEN_SHIPMENT_RELEASE =~ /\d+/g );
    return {
        'Version' => {
            'majorRelease' => $VersionNumbers[0],
            'minorRelease' => $VersionNumbers[1],
        }
    };
}

#========================================================================================
# function     _assembleManifestDate
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hManifestDate = $self->_assembleManifestDate($hInputParameters);
#----------------------------------------------------------------------------------------
# description  Create manifest date hash piece for the getManifest request
#----------------------------------------------------------------------------------------
# input        $hInputParameters | hash with required data to generate request | ref.hash
# return       $hManifestDate | hash with formatted date | ref.hash
#========================================================================================
sub _assembleManifestDate {
    my $self = shift;
    my ($hInputParameters) = @_;

    my $FormattedDate = $hInputParameters->{'manifestDate'};

    unless ( defined($FormattedDate) ) {
        my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();

        $FormattedDate = $Formatter->format_datetime(DateTime->today(), 'date', '%Y-%m-%d');
    }

    return { 'manifestDate' => $FormattedDate };
}

1;
