#========================================================================================
# package      DE_EPAGES::DHLIntraship::UI::ShippingTransaction
# base         DE_EPAGES::ExternalShipping::UI::ShippingTransaction
# state        public
#----------------------------------------------------------------------------------------
# description  Handles the shipping transaction UI requests.
#========================================================================================
package DE_EPAGES::DHLIntraship::UI::ShippingTransaction;
use base DE_EPAGES::ExternalShipping::UI::ShippingTransaction;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::DHLIntraship::API::Constants qw ( %SHIPPING_LABEL_STATUS );
use DE_EPAGES::DHLIntraship::API::DHLIntrashipError qw ( FormErrorAndLogging );

use strict;

#========================================================================================
# function     SendShippingOrderHelper
# state        public
#----------------------------------------------------------------------------------------
# syntax       SendShippingOrderHelper( $Servlet );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------
# description  This Function will called if to create the Shippinglabels.
#----------------------------------------------------------------------------------------
# syntax       $Handler->SendShippingOrderHelper( $Servlet );
#========================================================================================
sub SendShippingOrderHelper {
    my $self        = shift;
    my ($Servlet)   = @_;
    my $Transaction = $Servlet->object();
    $self->ValidateShippingOrder($Servlet);
    return if ( $Servlet->form->existsFormError() );
    eval { $self->_doTransaction( $Servlet, $Transaction ); };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError($Error);
        FormErrorAndLogging( 'ExportShippingLabels', $Servlet, $Error );
    }
    $self->_transactionErrorHandling($Servlet);

    return;
}

#========================================================================================
# function     _doTransaction
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_doTransaction( $Servlet,$Transaction );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Transaction | a Transaction object| object
#----------------------------------------------------------------------------------------
# description  This private Function encapsulate the transaction process. Additional is there
#               a Error handling if the Function is called without Error handling.
#========================================================================================
sub _doTransaction {
    my $self = shift;
    my ( $Servlet, $Transaction ) = @_;
    my $ShippingFulfillmentMethod = $Transaction->get('ShippingFulfillmentMethod');

    Error(
        'NO_SHIPPING_FULFILLMENT_METHOD',{
            'name'      => 'Order without a ShippingFulfillmentMethod',
            'Parameter' => $Transaction->alias,
            'Message' => $Transaction->alias,
        }
    ) unless defined $ShippingFulfillmentMethod;

    $Transaction->prepareShippingOrder();
    $ShippingFulfillmentMethod->generateShippingLabels( $Transaction, $Servlet->languageID );
    $Transaction->issueShippingOrder();
    return;
}


#========================================================================================
# function     _transactionErrorHandling
# state        private
#----------------------------------------------------------------------------------------
# syntax       _transactionErrorHandling( $Servlet );
#----------------------------------------------------------------------------------------
# description  depending on the the TransStatus of the transaction it will choose the correct
#               viewaction and, if needed, Error handling
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _transactionErrorHandling {
    my $self = shift;
    my ($Servlet) = @_;

    my $Transaction = $Servlet->object;
    my $TransStatus = $Transaction->get('TransStatus');

    if ($TransStatus eq $SHIPPING_LABEL_STATUS{'ALL_OK'}) {
        $self->_changeViewActionToTransaction($Servlet);
    }
    elsif ($TransStatus eq $SHIPPING_LABEL_STATUS{'SOME_OK'}) {
        $self->_changeViewActionToTransactionAndShowErrorMessage($Servlet);
    }
    elsif ($TransStatus eq $SHIPPING_LABEL_STATUS{'WRONG_CREDENTIALS'}) {
        my $ErrorName = 'WRONG_CREDENTIALS';
        $self->_showErrorMessage($Servlet, $ErrorName);
    }
    elsif ($TransStatus eq $SHIPPING_LABEL_STATUS{'ALL_KO'}) {
        my $ErrorName = 'ALL_EXPORTED_FAILED';
        $self->_showErrorMessage($Servlet, $ErrorName);
    }
    return undef;
}


#========================================================================================
# function     _changeViewActionToTransaction
# state        private
#----------------------------------------------------------------------------------------
# syntax       _changeViewActionToTransaction( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _changeViewActionToTransaction {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'ViewAction',  $self->_getViewAction($Servlet, 'MBO-ViewGeneral'));

    return undef;
}

#========================================================================================
# function     _showErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _showErrorMessage( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#               change the objectID
#               add error message
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _showErrorMessage {
    my $self = shift;
    my ($Servlet, $ErrorName) = @_;

    $Servlet->vars( 'ViewAction', $self->_getViewAction($Servlet,'MBO-ViewDHLIntrashipShippingOrder'));
    $Servlet->vars( 'ObjectID',$Servlet->object()->getSite()->id() );
    $Servlet->vars( 'BatchActionError', $ErrorName );
    return undef;
}


#========================================================================================
# function     _changeViewActionToTransactionAndShowErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _changeViewActionToTransactionAndShowErrorMessage( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#               change the objectID
#               add error message
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _changeViewActionToTransactionAndShowErrorMessage {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShopObjectID = $Servlet->object()->getSite()->id();
    my $ViewAction = $self->_getViewAction($Servlet,'MBO-ViewDHLIntrashipShippingOrder');
    $Servlet->vars( 'BatchActionError', 'SOME_EXPORTED_FAILED' );
    $Servlet->vars( 'LinkToShippingOrders', '?ViewAction='.$ViewAction.'&ObjectID='.$ShopObjectID );
    $self->_changeViewActionToTransaction($Servlet);
    return undef;
}

sub _getViewAction {
    my $self = shift;
    my ($Servlet, $ViewActionAlias) = @_;

    my $CurrentViewAction = $Servlet->params('ViewAction');
    if ($CurrentViewAction =~ m/^Unity.*/) {
        $ViewActionAlias = 'Unity' . $ViewActionAlias;
    }

    return $ViewActionAlias;
}
1;
