use strict;
use DBI;
use Data::Dumper;
use Getopt::Long;
use Pod::Usage;

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );

##no critic qw(ProhibitCascadingIfElse)

my $VERSION = '2.0';

# global options
my $echo = 1;
my $xml = 0;
my $csv = 0;
my $separator = '^go\s*$';
my $encoding;

RunScript( Sub => sub {
    my $dsn;
    my $StoreName;
    my $help = 0;
    my $dbms;
    my $autocommit = 1;
    my @sql;
    my @file;
    my %options = (
        'help|?'   => \$help,
        'dsn=s'    => \$dsn,
        'storename=s' => \$StoreName,
        'dbms=s'      => \$dbms,
        'autocommit!' => \$autocommit,
        'echo!'       => \$echo,
        'xml'         => \$xml,
        'csv'         => \$csv,
        'separator=s' => \$separator,
        'sql=s'       => \@sql,
        'file=s'      => \@file,
        'encoding=s'  => \$encoding,
    );

    GetOptions( %options ) or pod2usage(1);
    pod2usage(-exitstatus => 0, -verbose => 2) if $help;

    my $dbh;
    my $MainSub = sub {
        $dbh->{PrintError} = 0;
        $dbh->{RaiseError} = 1;
        $dbh->{AutoCommit} = $autocommit;

        foreach( @sql ) {
            print "$_\n" if $echo;
            DoSQL( $dbh, $_ );
        }

        foreach( @file ) {
            DoFile( $dbh, $_ );
        }

        Shell( $dbh ) unless @sql || @file;
    };

    if( $StoreName ) {
        print "Connecting store $StoreName ...\n" if $echo;
        RunOnStore( 'Store' => $StoreName, 'Sub' => sub {
            $dbh = GetCurrentDBHandle()->dbh;
            $MainSub->();
        });
    }
    else {
        if( $dsn ) {
            print "Connecting $dsn ...\n" if $echo;
            $dbh = DBI->connect( $dsn ) or die "Can't connect $dsn";
        }
        else {
            print 'Connecting '. join(', ', @ARGV) ." ...\n" if $echo;
            $dbh = DBI->connect( @ARGV ) or die "Can't connect $ARGV[0]";
        }

        $MainSub->();

        $dbh->disconnect;
    }
});

sub Shell {
    my ( $dbh ) = @_;

    my $line;
    my @lines;

    do {
        my $i = scalar(@lines) + 1;
        print "$i> ";

        $line = <STDIN>; ##no critic (ProhibitExplicitStdin)
        $line = 'exit' unless defined $line; # break on EOF
        chomp $line;

        if( $line =~ /^exit$/i ) { ##no critic (ProhibitCascadingIfElse)
            0;
        }
        elsif( $line =~ /$separator/i ) {
            eval { DoSQL( $dbh, join( "\n", @lines ) ); };
            print STDERR GetError()->message . "\n" if ExistsError();

            @lines = ();
        }
        elsif( $line eq 'reset' ) {
            @lines = ();
        }
        elsif( $line =~ /^do\s+(.*)/ ) {
            @lines = ();
            eval { DoFile( $dbh, $1 ); };
            print STDERR GetError()->message . "\n" if ExistsError();
        }
        elsif( $line =~ /^\/(\S+)\s*(.*?)\s*$/x ) {
            @lines = ();
            my @params = map { ($_ eq 'undef' || $_ eq 'null') ? undef : $_ }
                         map { /^\s*(.*?)\s*$/ }
                         split /\,|\s+/, $2;
            eval { DoDBI( $dbh, $1, @params ); };
            print STDERR GetError()->message . "\n" if ExistsError();
        }
        elsif( $line =~ /^\!(.*)/ ) {
            system $1;
        }
        elsif( $line =~ /^perl\s+(.*)/i ) {
            eval $1; ##no critic (ProhibitStringyEval)
            print STDERR GetError()->message . "\n" if ExistsError();
        }
        else {
            push @lines, $line;
        }
    }
    while( $line !~ /^exit$/i );
}

sub DoFile {
    my $dbh = shift or die 'Missing database handle';
    my $fileName = shift or die 'Missing file name';

    my $File;
    ##no critic (RequireBriefOpen)
    if( defined $encoding ) {
        open $File, "<:encoding($encoding)", $fileName or die "Can't open $fileName: $!";
    }
    else {
        open $File, '<', $fileName or die "Can't open $fileName: $!";
    }

    foreach ( split /\ngo\s*\n/i, do { local $/ = undef; <$File> } ) {
        print "$_\n" if $echo;
        DoSQL( $dbh, $_ );
    }
    ##use critic
    close $File or die "Can't close file $fileName: $!";
}

sub DoDBI {
    my ($dbh, $command, @params) = @_;
    die 'Missing database handle' unless $dbh;
    die 'Missing DBI command' unless $command;
# print $command . ' ' . join(', ', map { "'$_'" } @params) . "\n";

    # commands that return a result set
    my $sth;
    if( $command eq 'table_info' ) {
        $sth = $dbh->table_info(@params) or return;
    }
    elsif( $command eq 'column_info' ) {
        $sth = $dbh->column_info( @params ) or return;
    }
    elsif( $command eq 'primary_key_info' ) {
        $sth = $dbh->primary_key_info( @params ) or return;
    }
    elsif( $command eq 'foreign_key_info' ) {
        $sth = $dbh->foreign_key_info( @params ) or return;
    }
    else {
        # commands that don't return a result set
        if($command eq 'help' ) {
            ShowHelp();
        }
        elsif($command eq  'attributes' ) {
            print map { "$_=$dbh->{$_}\n"} qw( AutoCommit ChopBlanks FetchHashKeyName
                                               LongReadLen LongTruncOk Name PrintError
                                               Profile RaiseError RowCacheSize
                                               ShowErrorStatement Taint TraceLevel );
        }
        elsif($command eq  'available_drivers' ) {
            print "$_\n" foreach DBI->available_drivers;
        }
        elsif($command eq  'data_sources' ) {
            my $driver = $params[0] || $dbh->{Driver}->{Name};
            print "$_\n" foreach DBI->data_sources($driver);
        }
        elsif($command eq  'driver' ) {
            print "$dbh->{Driver}->{Name}\n";
        }
        elsif($command eq  'get_info' ) {
            print $dbh->get_info( $params[0] ), "\n";
        }
        elsif($command eq  'ping' ) {
            print $dbh->ping ? "true\n" : "false\n";
        }
        elsif($command eq  'primary_key' ) {
            print "$_\n" foreach $dbh->primary_key(@params);
        }
        elsif($command eq  'tables' ) {
            my $s = $dbh->table_info(undef, undef, undef) or return;
            my $aaRows = $s->fetchall_arrayref;
            $s->finish;

            print "$_\n" foreach sort map { $_->[2] } grep { $_->[3] eq 'TABLE' } @$aaRows;
        }
        elsif($command eq  'output' ) {
            $xml = $params[0] eq 'xml' ? 1 : 0;
            $csv = $params[0] eq 'csv' ? 1 : 0;
            print 'output='. ($xml ? 'xml' : $csv ? 'csv' : 'text') . "\n" if $echo;
        }
        elsif($command eq  'trace' ) {
            print $dbh->trace( $params[0] ), "\n";
        }
        elsif($command eq  'type_info_all' ) {
            print Dumper( $dbh->type_info_all ), "\n";
        }
        elsif($command eq  'type_info' ) {
            print Dumper( $dbh->type_info( $params[0] ) ), "\n";
        }
        else {
            die "Unknown DBI command: $command";
        }
        return;
    }

    ShowResultSet( $sth );
    $sth->finish;
}

sub DoSQL {
    my ( $dbh, $sql ) = @_;

    # remove comments
    $sql =~ s/--.*?(\n|$)/$1/g;

    # special commands
    if( $sql =~ /^(begin_work|begin\ tran(saction)?)\s*$/ix ) {
        $dbh->begin_work;
        return 1;
    }
    if( $sql =~ /^commit(\stran(saction)?)?\s*$/ix ) {
        $dbh->commit;
        return 1;
    }
    elsif( $sql =~ /^rollback(\stran(saction)?)?\s*$/ix ) {
        $dbh->rollback;
        return 1;
    }

    my $sth = $dbh->prepare( $sql );
    $sth->execute;

    if( $sql !~ /^\s*SELECT|SHOW|DESC\W/six && $sql !~ /^\s*EXEC|SP/si ) {
        my $rowCount = $sth->rows;
        print "$rowCount rows affected\n" if $rowCount >= 0;
    }

    ShowResultSet( $sth );

    $sth->finish;
}

sub ShowResultSet {
    my $sth = shift or die 'Missing statement handle';

    if( $xml ) {
        ShowResultSetAsXML( $sth );
    }
    elsif( $csv ) {
        ShowResultSetAsCSV( $sth );
    }
    else {
        ShowResultSetAsText( $sth );
    }
}

sub ShowResultSetAsText {
    my $sth = shift or die 'Missing statement handle';

    if( 0 == $sth->{NUM_OF_FIELDS} ) {
        print ".\n";
        return;
    }

    my $aColumns = $sth->{NAME};
    my @columnWidth = map { length $_ } @$aColumns;
    my $columnCount = scalar @$aColumns;

    # fetch all rows
    my @rows;
    my $aValues;

    while( $aValues = $sth->fetchrow_arrayref ) {

        my @row;

        for ( my $i=0; $i<$columnCount; $i++ ) {

            my $value = $aValues->[$i];
            $value = '(null)' unless defined $value;

            if( length $value > $columnWidth[$i] ) {
                $columnWidth[$i] = length $value;
            }

            push @row, $value;
        }

        push @rows, \@row;
    }

    # print a neatly formatted table
    for ( my $i=0; $i<$columnCount; $i++ ) {
        print sprintf '%-'.$columnWidth[$i].'s', $aColumns->[$i];
        print( ($i+1 == $columnCount) ? "\n" : ' ');
    }

    for ( my $i=0; $i<$columnCount; $i++ ) {
        print '-' x $columnWidth[$i];
        print( ($i+1 == $columnCount) ? "\n" : ' ');
    }

    foreach my $aRow (@rows) {
        for ( my $i=0; $i<$columnCount; $i++ ) {
            print sprintf '%-'.$columnWidth[$i].'s', $aRow->[$i];
            print( ($i+1 == $columnCount) ? "\n" : ' ');
        }
    }
}

sub ShowResultSetAsXML {
    my $sth = shift or die 'Missing statement handle';

    if( 0 == $sth->{NUM_OF_FIELDS} ) {
        print ".\n";
        return;
    }

    my $aColumns = $sth->{NAME};

    print "<ResultSet>\n";

    # fetch all rows
    my $aValues;
    my $RowIndex = 0;
    while( $aValues = $sth->fetchrow_arrayref ) {

        print "  <Row Index=\"$RowIndex\">\n";
        $RowIndex++;

        foreach my $Index ( 0 .. scalar(@$aColumns)-1 ) {
            my $Column = $aColumns->[$Index];
            my $Value  = $aValues->[$Index];
            print "    <$Column>$Value</$Column>\n"
        }

        print "  </Row>\n";
    }

    print "</ResultSet>\n";
}

sub ShowResultSetAsCSV {
    my $sth = shift or die 'Missing statement handle';

    if( 0 == $sth->{NUM_OF_FIELDS} ) {
        print ".\n";
        return;
    }

    local $, = ',';

    my $aColumns = $sth->{NAME};

    print @$aColumns;
    print "\n";

    # fetch all rows
    my $aValues;
    while( $aValues = $sth->fetchrow_arrayref ) {

        my @Line;
        foreach my $Index ( 0 .. scalar(@$aColumns)-1 ) {
            my $Value  = $aValues->[$Index];
            if( $Value =~ /\s/ || $Value =~ /\"/ || $Value =~ /\,/ ) { # "
                $Value =~ s/\"/\"\"/g; # "
                $Value = '"'.$Value.'"';
            }

            push @Line, $Value;
        }

        print @Line;
        print "\n";
    }
}

sub ShowHelp {
    print <<END_HELP;
dbi_shell Commands
  exit
  go
  reset
  /help
  /output csv|xml|text
Transaction Commands
  begin_work
  commit
  rollback
DBI Commands
  /attributes
  /available_drivers
  /column_info catalog schema table column
  /data_sources [driver]
  /driver
  /foreign_key_info pk_catalog pk_schema pk_table fk_catalog fk_schema fk_table
  /get_info 17|18|29|41|114
  /tables
  /ping
  /primary_key catalog schema table
  /primary_key_info catalog schema table
  /table_info catalog schema table type
  /trace level
  /type_info_all
  /type_info [data_type]
END_HELP
}

=head1 NAME

dbi_shell - Interactive SQL shell

=head1 SYNOPSIS

  perl dbi_shell.pl -dsn DSN [options]

Deprecated syntax:

  perl dbi_shell.pl DSN [username [password]]

=head1 OPTIONS

=over 8

=item B<-dsn>

Specifies the database as DBI datasource name.

=item B<-storename>

Specifies the logical database name.

=item B<-dbms>

Specifies the database management system. Currenty MySQL and MSSQL are supported.

=item B<-noautocommit>

Disables autocommit. All INSERT, UPDATE and DELETE statements must be
commited explicitely.

=item B<-noecho>

Disables output of status messages.

=item B<-xml>

Prints result sets as XML.

=item B<-csv>

Prints result sets as comma separated values.

=item B<-separator>

Specifies the separator for SQL statements.

=item B<-sql>

Specifies one or more ad-hoc SQL statements to execute.

=item B<-file>

Specifies one or more files containing SQL statements.

=item B<-encoding>

Specifies the character encoding of the input file. default=binary

=item B<-help>

Prints the complete documentation.

=back

=head1 DESCRIPTION

B<dbi_shell> allows you to execute SQL statements interactively, from files
or via command line.

=head1 AUTHOR

Wolfgang Loch <wolo@wolosoft.com>

=cut
