use strict;

use Getopt::Long;
use Config::IniFiles;
use Sys::Hostname;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Database::API::Config qw(DatabaseConfig);

sub usage {
    print <<END_USAGE;
Description:
  Script runs all schedulers in given business unit.
  This script works only on Linux operating systems.

Usage:
  perl $0 [options] [flags]

Options:
  -storename       name of store (default: as defined in Scheduler.conf)
  -scheduler-conf  path to Scheduler.conf (default: $ENV{EPAGES_CONFIG}/Scheduler.conf)

Flags:
  -localhost  just run scheduler for my own machine
  -active     just run scheduler which are active
  -dryrun     perform a trial run with no changes made (default)
  -run        really run schedulers (either -dryrun or -run must be used)
  -help       show the command line options

Example:
  perl $0 -storename Monitor -dryrun
END_USAGE
    exit 2;
}

sub Main {
    local $| = 1;

    if ($^O ne 'linux') {
        print "$0: this script runs just on linux, not on $^O\n";
        exit 0;
    }

    my %Options;
    my @GetOpt = qw(storename=s scheduler-conf=s active localhost dryrun run help);
    GetOptions(\%Options, @GetOpt) or usage();

    $Options{dryrun} = 1 unless $Options{run};

    usage() if $Options{help};

    $Options{'scheduler-conf'} //= "$ENV{EPAGES_CONFIG}/Scheduler.conf";

    my $time1 = time;

    my $HostName = hostname();
    $HostName =~ s/\..*//;

    my %Ini;
    tie %Ini, 'Config::IniFiles', (-file => $Options{'scheduler-conf'})
        or die "$Options{'scheduler-conf'}: $!\n";

    my $ReturnCode = 0;
    foreach my $Section (sort keys %Ini) {
        # ignore section: LOOP
        next if $Section eq 'LOOPS';
        # skip scheduler with option -active and scheduler is not active
        if ($Options{active} and not $Ini{$Section}{IsActive}) {
            print "skiping: $Section (not active)\n\n";
            next;
        }
        # skip scheduler with option -localhost and scheduler runs on other host
        if ($Options{localhost}) {
            my $Machine = ",$Ini{$Section}{Machine},";
            if ($Ini{$Section}{Machine} =~ /\S/ and $Machine !~ /,\Q$HostName\E,/x) {
                print "skiping: $Section (host not found in: $Ini{$Section}{Machine})\n\n";
                next;
            }
        }

        my $PerlScript = _GetPerlCommand($Options{storename}, $Section, \%Ini);
        if ($Options{dryrun}) {
            print "dryrun: $Section\n$ENV{PERL} $PerlScript\n\n";
            next;
        }

        print "running: $Section\n";
        my $time0 = time;

        eval { SystemCommand("$ENV{PERL} $PerlScript", { Print => 1 }); };
        if (ExistsError) {
            printf "result: ERROR (needs %.1f seconds)\n\n", time-$time0;
            $ReturnCode = 1;
        } else {
            printf "result: SUCCESS (needs %.1f seconds)\n\n", time-$time0;
        }
    }

    printf "needs %.1f seconds\n", time-$time1;
    exit $ReturnCode;
}

#========================================================================================
# function     _ReplaceENV
# state        private
#----------------------------------------------------------------------------------------
# syntax       $OutString = _ReplaceENV( $InString );
#----------------------------------------------------------------------------------------
# description  Replaces '$ENV{Varname}' in $InString by the actual values of environment
#               variables and returns $OutString
#----------------------------------------------------------------------------------------
# input        $InString        | string $ENV{Varname} | string
# return       $OutString       | $InString with replaced $ENV{Varname} | string
#========================================================================================
sub _ReplaceENV {
    my ($String) = @_;
    $String =~ s/\$ENV\{(.*?)\}/$ENV{$1}/ge;
    return $String;
}

#========================================================================================
# function     _GetPerlCommand
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $PerlCommand = _GetPerlCommand($StoreName, $Section, $Ini);
#----------------------------------------------------------------------------------------
# description  Returns scheduler script with arguments as defined by
#               $StoreName, $Section and $Ini
#----------------------------------------------------------------------------------------
# input        $StoreName | name of store where scheduler runs | string
# input        $Section | section of ini file Scheduler.conf | string
# input        $Ini     | Scheduler.conf hash | ref.hash
# return       $PerlCommand  |  perl script with arguments | string
#========================================================================================
sub _GetPerlCommand {
    my ($StoreName, $Section, $Ini) = @_;

    my $Loop = $Ini->{$Section}{Loop};
    my $LoopArgs = (split /,/, $Ini->{LOOPS}{$Loop})[0];

    if ($StoreName and $LoopArgs) {
        if ($Loop ~~ [qw(Store StoreEnvironment)]) {
            # in loops Store and StoreEnvironment change -storename
            $LoopArgs =~ s,(-storename\s+)\S+,${1}$StoreName,;
        } elsif ($Loop ~~ [qw(Backup BackupDB)]) {
            # in loops Backup and BackupDB change -dbname
            my $DBName = DatabaseConfig($StoreName)->get('Connection', 'Database');
            $LoopArgs =~ s,(-dbname\s+)\S+,${1}$DBName, if $DBName;
        }
    }

    $LoopArgs = _ReplaceENV( $LoopArgs );

    my $Args = _ReplaceENV( $Ini->{$Section}{Options} );
    return _ReplaceENV( $Ini->{$Section}{Command} ). " $LoopArgs $Args";
}

RunScript(
    'Sub' => \&Main
);

1;
