use strict;

use Getopt::Long;
use DE_EPAGES::Core::API::File qw( ExistsDirectory ExistsFile CopyFile CreateDirectory fsplit GetFileNames WriteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Url qw( BuildParameterString );
use DE_EPAGES::Database::API::Config qw( StoreType );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::JQuery::API::JavaScriptDocBuild qw( RunDocuGenerator );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use HTTP::Cookies;
use LWP::UserAgent;
use WWW::Mechanize;
use XML::XPath;

sub Main {
    local $| = 1;
    my $Help;
    my ($StoreName, $Target, $NoOutput, $Password);
    GetOptions(
        'storename=s'   => \$StoreName,
        'target=s'      => \$Target,
        'password=s'    => \$Password,
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
    );

    usage() if $Help;

    unless($StoreName){
        print "The store name is not defined!\n";
        usage();
    }

    unless($Target){
        print "The target directory is not defined!\n";
        usage();
    }

    usage() if $Help;

    # remove (back)slash at the end of the target path
    $Target =~ s/[\/\\]$//;
    if( !ExistsDirectory($Target) ){
        CreateDirectory( $Target );
    }

    my $Version;
    my $URLDiagnostics = '';
    my $StoreType = StoreType( $StoreName );

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {
            my $RootObject = LoadRootObject();

            $Version = $RootObject->get('EpagesVersion');

            $URLDiagnostics .= $RootObject->get('ProtocolAndServer');
            $URLDiagnostics .= '/';
            $URLDiagnostics .= $RootObject->get('WebServerDirectory');
            $URLDiagnostics .= '/';
            $URLDiagnostics .= $StoreType;
            $URLDiagnostics .= '.';
            $URLDiagnostics .= $RootObject->get('URLDiagnosticsExtension');

            $URLDiagnostics = LoginToDiagnostics( $URLDiagnostics );

            $URLDiagnostics .= '?';
        },
    );

    my $SourceXML = "$ENV{'EPAGES_WEBROOT'}/StoreTypes/$Version/$StoreType/Diagnostics/jQueryAPI.xml";

    # try to generate API source XML if not exists
    if( !ExistsFile($SourceXML) ){
        RunDocuGenerator({
            'StoreType' => $StoreType,
            'Version'   => $Version,
            'Template'  => "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/JQuery/Scripts/jsDocTemplate.xml",
            'Target'    => $SourceXML,
        });
    }

    # check if API source XML exists
    if( ExistsFile($SourceXML) ){

        my @Pages = ('/snippet/all','/category/all');
        my $Document = XML::XPath->new(filename => $SourceXML);

        foreach my $FileName ( @{GetFileNames("$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Diagnostics/Data/Private/buildJsDocStatic/")} ){

            my (undef, $InfoFileName, $InfoFileExt) = fsplit($FileName);
            my $TargetFile = "$Target/$InfoFileName$InfoFileExt";
            CopyFile( $FileName, $TargetFile );

            print "\nwrite: $TargetFile" unless $NoOutput;
        }

        foreach my $Node ( $Document->findnodes('//category') ){
            my $Page = '/category/'.$Node->getAttribute('name');
            push( @Pages, $Page) unless $Page ~~ @Pages;
        }
        foreach my $Node ( $Document->findnodes('//added | //deprecated | //removed') ){
            my $str = $Node->toString();
            $str =~ s/^<.+?>(.*)<.+?>$/$1/sx;
            my $Page = '/category/version-'.$str;
            push( @Pages, $Page) unless $Page ~~ @Pages;
        }
        foreach my $Node ( $Document->findnodes('//entries/entry | //plugins/entry') ){
            my $Page = '/api/'.$Node->getAttribute('name');
            push( @Pages, $Page) unless $Page ~~ @Pages;
        }

        foreach my $Path ( @Pages ){
            my $TargetFile = $Path;

            $TargetFile =~ s/^\///;
            $TargetFile =~ s/\//-/g;
            $TargetFile = "$Target/$TargetFile.html";

            my $Mech = WWW::Mechanize->new();
            my $Params = BuildParameterString({
                'BuildJsDocStatic'   => '1',
                'ViewAction'         => 'ViewJQuery',
                'Path'               => $Path,
            });
            $Mech->get( "$URLDiagnostics$Params" );
            my $Content = $Mech->content();
            WriteFile( $TargetFile, \$Content, {'utf8' => 1} );

            print "\nwrite: $TargetFile" unless $NoOutput;
        }
    }
}

sub LoginToDiagnostics {
    my ($URL) = @_;

    my $Browser = LWP::UserAgent->new;
    # $Browser->proxy(['http'], 'http://127.0.0.1:8888/');    # fiddler

    $Browser->cookie_jar({});
    my $LoginResponse = $Browser->post(
        $URL,
        [
            'ViewAction'   => 'ViewDiagIndex',
            'ChangeAction' => 'SaveLoginForm',
            'Login'        => 'developer',
            'Password'     => 'developer'
        ]
    );
    my $LoginCookies = HTTP::Cookies->new;
    $LoginCookies->extract_cookies( $LoginResponse );

    $URL = $LoginResponse->header('content-base');

    return $URL;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Create static documentation files for all JavaScript *Docu.txt
    files found in the given WebRoot/StoreTypes/<Version>/<StoreType>
    directory.

Options:
    -storename  name of store                                  (required)
    -target     a target directory to save the generated files (required)
    -password   database password                              (optional)

Flags:
    -nooutput   be silent, only display errors                 (optional)
    -help       show the command line options                  (optional)

Examples:
    perl $0 -storename Store -target /tmp/docu

END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);



