#========================================================================================
# package      DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest
# state        public
# description  Builds hashes for SOAP requests.
#========================================================================================
package DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DocdataPay::API::Constants qw (
    DEFAULT_DESCRIPTION
    MAX_LENGTH_DESCRIPTION
    NUMBER_OF_DAYS_TO_PAY
    RATE_TO_INTEGER
);
use DE_EPAGES::DocdataPay::API::SOAP::Tools;
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest;
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {
        'hRequest'                  => undef,
        'invoiceData'               => undef,
        'Basket'                    => undef,
        'LineItemContainer'         => undef,
        'LineItems'                 => undef,
        'LineItemPaymentDocdataPay' => undef,
        'PaymentMethodDocdataPay'   => undef,
        'BillingAddress'            => undef,
        'User'                      => undef,
        'Shipping'                  => undef
    }, $class;
    return $self;
}

#========================================================================================
# function     buildCreateRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildCreateRequest( $Basket );
#----------------------------------------------------------------------------------------
# description  Creates a hash containing the create request message for the given
#              Basket.
#----------------------------------------------------------------------------------------
# input        $Basket   | Basket object           | Object
# return       $hRequest | Request message to send | refHash
#========================================================================================
sub buildCreateRequest {
    my $self = shift;
    my ($Basket) = @_;

    $self->{'Basket'} = $Basket;
    $self->_initializeClassAttributes();

    $self->{'hRequest'} = {
        'version'                => $self->{'LineItemPaymentDocdataPay'}->get('APIVersion'),
        'merchant'               => $self->_assembleMerchantData(),
        'merchantOrderReference' => $self->_getMechantOrderReference(),
        'paymentPreferences'     => $self->_assemblePaymentPreferencesData(),
        'shopper'                => $self->_assembleShopperData(),
        'totalGrossAmount'       => $self->_assembleTotalGrossAmount(),
        'billTo'                 => $self->_assembleBillToData(),
        'invoice'                => $self->_assembleInvoiceData
    };

    return $self->{'hRequest'};
}

#========================================================================================
# function     _initializeClassAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _initializeClassAttributes($Basket);
#----------------------------------------------------------------------------------------
# description  Initializes all undef class attributes.
#----------------------------------------------------------------------------------------
# input        $Basket | DocData Basket | Object
#========================================================================================
sub _initializeClassAttributes {
    my $self = shift;

    $self->{'LineItemContainer'} = $self->{'Basket'}->get('LineItemContainer');
    $self->_TestObject( 'LineItemContainer', $self->{'LineItemContainer'}, 'DE_EPAGES::Order::API::Object::LineItemContainer' );

    $self->{'LineItems'} = $self->{'LineItemContainer'}->get('LineItems');
    Error( 'NoProductsInTheBasket', { 'Message' => 'There are no items in LineItemContainer.' } )
      unless ( isValid( $self->{'LineItems'} ) );

    $self->{'LineItemPaymentDocdataPay'} = $self->{'LineItemContainer'}->get('Payment');
    $self->_TestObject( 'LineItemPaymentDocdataPay', $self->{'LineItemPaymentDocdataPay'}, 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay' );

    $self->{'PaymentMethodDocdataPay'} = $self->{'LineItemPaymentDocdataPay'}->get('PaymentMethod');
    $self->_TestObject( 'PaymentMethodDocdataPay', $self->{'PaymentMethodDocdataPay'}, 'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay' );

    $self->{'BillingAddress'} = $self->{'Basket'}->get('BillingAddress');
    $self->_TestObject( 'Address', $self->{'BillingAddress'}, 'DE_EPAGES::Address::API::Object::Address' );

    $self->{'User'} = $self->{'Basket'}->get('User');
    Error( 'UserDoesNotExist', { 'Message' => 'The User who created the Basket is missing.' } )
      unless ( isValid( $self->{'User'} ) && $self->{'User'}->instanceOf('User') );

    $self->{'Shipping'} = $self->{'LineItemContainer'}->get('Shipping');
    # Don't use TestObject here; Shipping if basket contains downloadable products only.

    return;
}

#========================================================================================
# function     _assembleMerchantData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleMerchantData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing merchant data to be sent on create request.
#----------------------------------------------------------------------------------------
# return       $hMerchantData | Merchant data to send | refHash
#========================================================================================
sub _assembleMerchantData {
    my $self = shift;

    Error( 'MerchantNameNotSet',
        { 'Message' => 'The merchant name from DocdataPay is not set in the PaymentMethodDocdataPay' } )
      unless ( isValid( $self->{'PaymentMethodDocdataPay'}->get('MerchantName') ) );
    Error( 'MerchantPasswordNotSet',
        { 'Message' => 'The merchant password from DocdataPay is not set in the PaymentMethodDocdataPay' } )
      unless ( $self->{'PaymentMethodDocdataPay'}->get('MerchantPassword') );

    my $hMerchant = {
        'name'     => $self->{'PaymentMethodDocdataPay'}->get('MerchantName'),
        'password' => $self->{'PaymentMethodDocdataPay'}->get('MerchantPassword')
    };

    return $hMerchant;
}

#========================================================================================
# function     _getMechantOrderReference
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getMechantOrderReference();
#----------------------------------------------------------------------------------------
# description  Obtains the alias of a Basket.
#----------------------------------------------------------------------------------------
# return       $Alias | Basket's alias | String
#========================================================================================
sub _getMechantOrderReference {
    my $self = shift;

    return $self->_removeDashesFromGUID($self->{'LineItemPaymentDocdataPay'}->get('GUID'));
}

#========================================================================================
# function     _assemblePaymentPreferencesData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assemblePaymentPreferencesData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing payment preferences to be sent on create
#              request.
#----------------------------------------------------------------------------------------
# return       $hPaymentPreferences | PaymentPreferences data to send | refHash
#========================================================================================
sub _assemblePaymentPreferencesData {
    my $self = shift;

    my $hPaymentPreferences = {
        'profile'           => $self->{'PaymentMethodDocdataPay'}->get('ProfileName'),
        'numberOfDaysToPay' => NUMBER_OF_DAYS_TO_PAY
    };

    return $hPaymentPreferences;
}

#========================================================================================
# function     _assembleShopperData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleShopperData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing shopper data to be sent on create
#              request.
#----------------------------------------------------------------------------------------
# return       $hShopper | Shopper data to send | refHash
#========================================================================================
sub _assembleShopperData {
    my $self = shift;

    my $hShopper = {
        'id'   => $self->{'User'}->id(),
        'name' => {
            'first' => $self->_truncateAttribute($self->{'BillingAddress'}->get('FirstName'), 'FirstName'),
            'last'  => $self->_truncateAttribute($self->{'BillingAddress'}->get('LastName'), 'LastName')
        },
        'email'    => $self->{'BillingAddress'}->get('EMail'),
        'language' => { 'code' => $self->_GetCodeByLanguageID() },
        'gender'   => $self->_getGender()
    };

    return $hShopper;
}

#========================================================================================
# function     _getGender
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getGender();
#----------------------------------------------------------------------------------------
# description  Returns 'M' if gender is male, 'F' if gender is female and 'U' if it's not
#              defined.
#----------------------------------------------------------------------------------------
# return       $Gender | The gender | String
#========================================================================================
sub _getGender {
    my $self = shift;

    my ($Gender) = $self->{'BillingAddress'}->get('Gender');
    if (defined ($Gender)) {
        return ($Gender eq '0') ? 'M' : 'F';
    }

    return 'U';
}

#========================================================================================
# function     _assembleTotalGrossAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleTotalGrossAmount();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing the sum of all gross amounts in a given
#              LineItemContainer plus shipping cost.
#----------------------------------------------------------------------------------------
# return       $htotalGrossAmount | Total gross amount (and unit) to send | refHash
#========================================================================================
sub _assembleTotalGrossAmount {
    my $self = shift;

    my $htotalGrossAmount = {
        'currency' => $self->{'LineItemContainer'}->get('CurrencyID'),
        '_'        => $self->_convertNumberToDocdataFormat( $self->{'LineItemContainer'}->get('GrandTotal') )
    };

    return $htotalGrossAmount;
}

#========================================================================================
# function     _assembleBillToData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleBillToData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing billing data to be sent on create request.
#----------------------------------------------------------------------------------------
# return       $hBillTo | Billing data to send | refHash
#========================================================================================
sub _assembleBillToData {
    my $self = shift;

    my $hBillTo = {
        'name' => {
            'first' => $self->_truncateAttribute($self->{'BillingAddress'}->get('FirstName'), 'FirstName'),
            'last'  => $self->_truncateAttribute($self->{'BillingAddress'}->get('LastName'), 'LastName')
        },
        'address' => {
            'street'      => $self->_truncateAttribute($self->{'BillingAddress'}->get('StreetOnly'), 'StreetOnly'),
            'houseNumber' => $self->_getHouseNumber($self->{'BillingAddress'}),
            'postalCode'  => $self->_removeAllWhiteSpaces( $self->{'BillingAddress'}->get('Zipcode') ),
            'city'        => $self->_truncateAttribute($self->{'BillingAddress'}->get('City'), 'City'),
            'country'     => { 'code' => $self->{'BillingAddress'}->get('Country')->{'Code2'} }
        }
    };

    return $hBillTo;
}

#========================================================================================
# function     _assembleInvoiceData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleInvoiceData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing invoice data to be sent on create request.
#----------------------------------------------------------------------------------------
# return       $hInvoice | Invoice data to send | refHash
#========================================================================================
sub _assembleInvoiceData {
    my $self = shift;

    my $hInvoice = {
        'totalNetAmount' => {
            'currency' => $self->{'LineItemContainer'}->get('CurrencyID'),
            '_'        => $self->_convertNumberToDocdataFormat( $self->{'LineItemContainer'}->get('TotalBeforeTax') )
        },
        'totalVatAmount' => $self->_assembleTotalVatAmounts(),
        'item'           => $self->_assembleInvoiceItems(),
        'shipTo'         => $self->_assembleShipToData()
    };

    return $hInvoice;
}

#========================================================================================
# function     _assembleShipToData
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleShipToData();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing shipping data to be sent on create request.
#----------------------------------------------------------------------------------------
# return       $hShipTo | Shipping data to send | refHash
#========================================================================================
sub _assembleShipToData {
    my $self = shift;

    my $hShipTo;
    my $ShippingAddress;
    if ( defined( $ShippingAddress = $self->{'Basket'}->get('ShippingAddress') ) ) {
        $hShipTo = {
            'name' => {
                'first' => $self->_truncateAttribute($ShippingAddress->get('FirstName'), 'FirstName'),
                'last'  => $self->_truncateAttribute($ShippingAddress->get('LastName'), 'LastName')
            },
            'address' => {
                'street'      => $self->_truncateAttribute($ShippingAddress->get('StreetOnly'), 'StreetOnly'),
                'houseNumber' => $self->_getHouseNumber($ShippingAddress),
                'postalCode'  => $self->_removeAllWhiteSpaces( $ShippingAddress->get('Zipcode') ),
                'city'        => $self->_truncateAttribute($ShippingAddress->get('City'), 'City'),
                'country'     => { 'code' => $ShippingAddress->get('Country')->{'Code2'} }
            }
        };
        if ( defined( $ShippingAddress->get('Company') ) ) {
            $hShipTo->{'address'}->{'company'} = $self->_truncateAttribute($ShippingAddress->get('Company'), 'Company');
        }
        if ( defined( $self->{'Basket'}->get('ShopAddress')->get('VATID') ) ) {
            $hShipTo->{'address'}->{'vatNumber'} = $self->{'Basket'}->get('ShopAddress')->get('VATID');
        }
    }
    else {
        $hShipTo = $self->_assembleBillToData();
    }

    return $hShipTo;
}

#========================================================================================
# function     _assembleTotalVatAmounts
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleTotalVatAmounts();
#----------------------------------------------------------------------------------------
# description  Creates an Array containing TotalVatAmounts.
#----------------------------------------------------------------------------------------
# return       @aTotalVatAmounts | Array of TotalVatAmounts | Array
#========================================================================================
sub _assembleTotalVatAmounts {
    my $self = shift;

    my @aTotalVatAmounts;

    my $LineItemShipping = $self->{'Shipping'};
    if (isValid($LineItemShipping)) {
        push @aTotalVatAmounts, $self->_buildTotalVatAmount( $self->{'Shipping'} );
    }

    foreach my $LineItemProduct ( @{ $self->{'LineItems'} } ) {
        push @aTotalVatAmounts, $self->_buildTotalVatAmount($LineItemProduct);
    }

    return \@aTotalVatAmounts;
}

#========================================================================================
# function     _buildTotalVatAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildTotalVatAmount();
#----------------------------------------------------------------------------------------
# description  Creates a hash containing a TotalVatAmount.
#----------------------------------------------------------------------------------------
# input        $LineItem        | LineItem whose TotalVat will be calculated | Object
# return       $hTotalVatAmount | Hash containing a TotalVatAmount           | Array
#========================================================================================
sub _buildTotalVatAmount {
    my $self = shift;
    my ($LineItem) = @_;

    my $hTotalVatAmount = {
        'currency' => $self->{'LineItemContainer'}->get('CurrencyID'),
        'rate'     => $self->_convertRateToInteger( $LineItem->get('TaxRate') ),
        '_'        => $self->_getDocdataVat( $LineItem->get('LineItemPrice'), $LineItem->get('TaxRate') )
    };

    return $hTotalVatAmount;
}

#========================================================================================
# function     _assembleInvoiceItems
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleInvoiceItems();
#----------------------------------------------------------------------------------------
# description  Creates an Array containing Items.
#----------------------------------------------------------------------------------------
# return       @aTotalVatAmounts | Array of Items | Array
#========================================================================================
sub _assembleInvoiceItems {
    my $self = shift;

    my $ItemNumber = 1;
    my @aItems;

    my $LineItemShipping = $self->{'Shipping'};
    if (isValid($LineItemShipping)) {
        my $Price = $LineItemShipping->get('LineItemPrice');
        push @aItems, $self->_buildLineItem($LineItemShipping, $ItemNumber, $Price, $Price);
        ++$ItemNumber;
    }

    foreach my $LineItem ( @{ $self->{'LineItems'} } ) {
        my $UnitPrice = $LineItem->get('BasePrice');
        my $TotalPrice = $LineItem->get('LineItemPrice');
        push @aItems, $self->_buildLineItem($LineItem, $ItemNumber, $UnitPrice, $TotalPrice);
        ++$ItemNumber;
    }

    return \@aItems;
}

#----------------------------------------------------------------------------------------
sub _buildLineItem {
    my $self = shift;
    my ( $LineItem, $ItemNumber, $UnitPrice, $TotalPrice ) = @_;

    my $CurrencyID = $self->{'LineItemContainer'}->get('CurrencyID');
    my $TaxRate = $LineItem->get('TaxRate');
    my $hItem = {
        'number'   => $ItemNumber,
        'name'     => $self->_truncateAttribute($LineItem->get('NameOrAlias'), 'ItemName'),
        'code'     => $self->_getCode($LineItem),
        'quantity' => {
            'unitOfMeasure' => 'PCS',
            '_'             => $self->_getQuantity($LineItem)
        },
        'description' => $self->_getDescription($LineItem),
        'netAmount'   => {
            'currency' => $CurrencyID,
            '_'        => $self->_getDocdataNet( $UnitPrice, $TaxRate )
        },
        'grossAmount' => {
            'currency' => $CurrencyID,
            '_'        => $self->_convertNumberToDocdataFormat( $UnitPrice )
        },
        'vat' => {
            'rate'   => $self->_convertRateToInteger( $TaxRate ),
            'amount' => {
                'currency' => $CurrencyID,
                '_' => $self->_getDocdataVat( $UnitPrice, $TaxRate )
            },
        },
        'totalNetAmount' => {
            'currency' => $CurrencyID,
            '_' => $self->_getDocdataNet( $TotalPrice, $TaxRate )
        },
        'totalGrossAmount' => {
            'currency' => $CurrencyID,
            '_'        => $self->_convertNumberToDocdataFormat( $TotalPrice )
        },
        'totalVat' => {
            'rate'   => $self->_convertRateToInteger( $TaxRate ),
            'amount' => {
                'currency' => $CurrencyID,
                '_' => $self->_getDocdataVat( $TotalPrice, $TaxRate )
            },
        }
    };

    return $hItem;
}

#----------------------------------------------------------------------------------------
sub _getCode {
    my $self = shift;
    my ($LineItem) = @_;

    if ($LineItem->instanceOf('LineItemProduct')) {
        return $LineItem->get('Product')->get('Alias');
    } else {
        return $LineItem->get('GUID');
    }
}

#----------------------------------------------------------------------------------------
sub _getDescription {
    my $self = shift;
    my ($LineItem) = @_;

    if ($LineItem->instanceOf('LineItemProduct')) {
        return $self->_getProductDescription( $LineItem->get('Product'));
    } else {
        my $NameOrAlias = $LineItem->get('NameOrAlias');
        return isValid($NameOrAlias) ? substr($NameOrAlias, 0, MAX_LENGTH_DESCRIPTION) : DEFAULT_DESCRIPTION;
    }
}

#----------------------------------------------------------------------------------------

sub _getProductDescription {
    my $self = shift;
    my ($Product) = @_;

    my $ProductDescription;
    if(defined( $Product->get('SuperProduct') )) {
        $ProductDescription = $Product->get('SuperProduct')->get('DisplayDescription')
    } else {
        $ProductDescription = $Product->get('DisplayDescription');
    }

    return (length($ProductDescription) > 0) ? substr($ProductDescription, 0, MAX_LENGTH_DESCRIPTION) : DEFAULT_DESCRIPTION;
}

#========================================================================================
# function     _getHouseNumber
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getHouseNumber( $Address );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Address     | Address                          | object
# return       $HouseNumber | A house number or 0 if it's null | integer
#========================================================================================
sub _getHouseNumber {
    my $self = shift;
    my ($Address) = @_;

    my $HouseNumber = $Address->get('HouseNumber');

    return (isValid($HouseNumber)) ? $self->_truncateAttribute($HouseNumber, 'HouseNumber') : 0;
}

#========================================================================================
# function     _convertRateToInteger
# state        private
#----------------------------------------------------------------------------------------
# syntax       _convertRateToInteger($Number);
#----------------------------------------------------------------------------------------
# description  Converts a rate to an integer, extracting the decimal part.
#----------------------------------------------------------------------------------------
# input       $Number | Number to convert | Integer
# return       $DocdataNumber | The converted number | Integer
#========================================================================================
sub _convertRateToInteger {
    my $self = shift;
    my ($Number) = @_;

    return $Number * RATE_TO_INTEGER;
}

#========================================================================================
# function     _getDocdataVat
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getDocdataVat($Price, $TaxRate);
#----------------------------------------------------------------------------------------
# description  Obtains Vat from a certain purchase.
#----------------------------------------------------------------------------------------
# input       $Price | Price of the item | Integer
# input       $TaxRate | Tax rate | Integer
# return       $Vat | Vat in Doctada format | Integer
#========================================================================================
sub _getDocdataVat {
    my $self = shift;
    my ( $Price, $TaxRate ) = @_;

    return $self->_convertNumberToDocdataFormat( $Price - ( $Price / ( 1 + $TaxRate ) ) );
}

#========================================================================================
# function     _getDocdataNet
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getDocdataNet($Price, $TaxRate);
#----------------------------------------------------------------------------------------
# description  Obtains Net price from a certain purchase.
#----------------------------------------------------------------------------------------
# input       $Price | Price of the item | Integer
# input       $TaxRate | Tax rate | Integer
# return       $Vat | Vat in Doctada format | Integer
#========================================================================================
sub _getDocdataNet {
    my $self = shift;
    my ( $Price, $TaxRate ) = @_;

    return $self->_convertNumberToDocdataFormat( $Price / ( 1 + $TaxRate ) );
}

#========================================================================================
# function     _removeAllWhiteSpaces
# state        private
#----------------------------------------------------------------------------------------
# syntax       _removeAllWhiteSpaces($String);
#----------------------------------------------------------------------------------------
# description  Removes all whitespaces from a String.
#----------------------------------------------------------------------------------------
# input       $String | The String whose whitespaces have to be removed. | String
# return       $String | A String with no whitespaces. | String
#========================================================================================
sub _removeAllWhiteSpaces {
    my $self = shift;
    my ($String) = @_;

    $String =~ s/\s+//g;
    return $String;
}

#========================================================================================
# function     _getQuantity
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getQuantity( $LineItemProduct );
#----------------------------------------------------------------------------------------
# description  Obtains a valid quantity for the given LineItem, taking
#               into account that Docdata only accepts integer numbers. If quantity is
#               a decimal number, return 1.
#----------------------------------------------------------------------------------------
# input        $LineItemProduct | Product whose quantity will be returned | object
# return       $Quantity        | Product quantity                        | integer
#========================================================================================
sub _getQuantity {
    my $self = shift;
    my ($LineItem) = @_;

    if ($LineItem->instanceOf('LineItemProduct') or
        $LineItem->instanceOf('LineItemShippingOption')) {
        my $Quantity = $LineItem->get('Quantity');
        return ($Quantity =~ /^\d+$/) ? $Quantity : 1;
    } else {
        return 1;
    }

}

# Wrappers
#----------------------------------------------------------------------------------------
sub _TestObject {
    my $self = shift;
    my ( $ClassName, $Object, $Package ) = @_;

    return TestObject( $ClassName, $Object, $Package );
}

#----------------------------------------------------------------------------------------
sub _GetCodeByLanguageID {
    my $self = shift;

    return GetCodeByLanguageID( $self->{'Basket'}->get('User')->get('LanguageID') );
}

#----------------------------------------------------------------------------------------
sub _convertNumberToDocdataFormat {
    my $self = shift;
    my ($Number) = @_;

    my $Tools = DE_EPAGES::DocdataPay::API::SOAP::Tools->new();
    return $Tools->convertNumberToDocdataFormat($Number);
}

#----------------------------------------------------------------------------------------
sub _removeDashesFromGUID {
    my $self = shift;
    my ($GUID) = @_;

    my $Tools = DE_EPAGES::DocdataPay::API::SOAP::Tools->new();

    return $Tools->removeDashesFromGUID($GUID);
}

#----------------------------------------------------------------------------------------
sub _truncateAttribute {
    my $self = shift;
    my ($Attribute, $FieldName) = @_;

    my $Tools = DE_EPAGES::DocdataPay::API::SOAP::Tools->new();

    return $Tools->truncateAttribute($Attribute, $FieldName);
}

1;
