#========================================================================================
# package      DE_EPAGES::Dojo::API::Shrinksafe
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides an API for creating dojo shrinksafe builds.
#========================================================================================
package DE_EPAGES::Dojo::API::Shrinksafe;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    RunDojoShrinksafe
);

use Config;
use JSON qw( from_json to_json );
use DE_EPAGES::Core::API::PerlTools qw (unique);
use DE_EPAGES::Core::API::File qw( ExistsDirectory ExistsFile GetInfoDirectory GetFileContent
                                   WriteFile CopyFile CreateDirectory DeleteFile GetFileNames
                                   SetAccessRights RealPath GetFileContentLines AbsolutePath
                                   fsplit RunOnDirectory );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );
use DE_EPAGES::Core::API::Program::java;

##no critic qw(ProhibitMagicNumbers)

#========================================================================================
# function     RunDojoShrinksafe
# state        public
#----------------------------------------------------------------------------------------
# syntax       RunDojoShrinksafe($hParams);
#----------------------------------------------------------------------------------------
# description  This procedure creates the dojo shrinksafe build depending on its params.
#----------------------------------------------------------------------------------------
# input        $hParams | build parameters
#               <ul>
#                 <li>StoreType           - store type (required) - string
#                 <li>Profile             - the used profile file (optional) - string
#                 <li>Version             - dojo.version will contain this value (optional, default: 0.0.0.dev) - string
#                 <li>CustomReleaseDir    - sets the release directory (optional) - string
#                 <li>HFFiles             - file with list of changed js files or directory with changed files (optional) - string
#                 <li>ConsultingCartridge - cartridge with js files to override (optional) - string
#                 <li>ConfigFile          - configuration file name (optional) - string
#                 <li>User                - username for owner, i.e. 'ep_appl' (optional) - string
#                 <li>Group               - username for owner, i.e. 'ep_web' (optional) - string
#               </ul> | ref.hash
#========================================================================================
sub RunDojoShrinksafe {
    my ($hParams) = @_;

    my $hBuildParams = _getBuildParams($hParams);

    if($hBuildParams->{'HFFiles'}){
        _addHotFixFiles($hBuildParams);
    }

    _createReleaseDirectory($hBuildParams);

    # Make temporary profile file.
    _makeProfileFile($hBuildParams);

    # Use build script and shrinksafe of Dojo Toolkit.
    if($hBuildParams->{'Profile'}){
        _addLayersToBuild($hBuildParams);
    }

    _processDojoShrinksafe($hBuildParams);

    # Delete temporary profile file.
    _deleteProfileFile($hBuildParams);

    # Copy files for hotfix
    if($hBuildParams->{'HFFiles'}){
        _copyHotfixFiles($hBuildParams);
    }

    # SetAccessRights
    _setAccessRights($hBuildParams);
}

sub _addHotFixFiles {
    my ($hBuildParams) = @_;

    my %HotFix;
    $HotFix{'files'} = _getHotfixFiles($hBuildParams);
    $hBuildParams->{'HotFix'} = \%HotFix;
}

sub _addLayersToBuild {
    my ($hBuildParams) = @_;

    my $Profile = $hBuildParams->{'Profile'};
    my $hProfileLayerMap = $hBuildParams->{'ProfileLayerMap'};

    foreach my $Layer (keys %$hProfileLayerMap){
        push @{$hBuildParams->{'BuildLayers'}}, $hProfileLayerMap->{$Layer} if $Layer eq $Profile;
    }
}

sub _deleteProfileFile {
    my ($hBuildParams) = @_;

    my $ReleaseDir  = $hBuildParams->{'ReleaseDir'};
    my $ProfileFile = $hBuildParams->{'ProfileFile'};

    DeleteFile(join('/',$ReleaseDir,$ProfileFile));
}

sub _createReleaseDirectory {
    my ($hBuildParams) = @_;

    my $ReleaseDir = $hBuildParams->{'ReleaseDir'};
    CreateDirectory($ReleaseDir) unless ExistsDirectory($ReleaseDir);
}

sub _makeProfileFile {

    my ($hBuildParams) = @_;

    my $HFFiles             = $hBuildParams->{'HFFiles'};
    my $EpagesCartridges    = $hBuildParams->{'EpagesCartridges'};
    my $ConsultingCartridge = $hBuildParams->{'ConsultingCartridge'};
    my $ProfileFile         = $hBuildParams->{'ProfileFile'};
    my $ReleaseDir          = $hBuildParams->{'ReleaseDir'};
    my $hHotFix             = $hBuildParams->{'HotFix'};

    my ( @Prefixes, @hJsonLayers );
    push @Prefixes, ( [ 'dojo', join('/',$EpagesCartridges,'DE_EPAGES/Dojo/Data/Public/javascript/dojo') ],
                      [ 'dijit', join('/',$EpagesCartridges,'DE_EPAGES/Dojo/Data/Public/javascript/dijit') ],
                      [ 'dojox', join('/',$EpagesCartridges,'DE_EPAGES/Dojo/Data/Public/javascript/dojox') ],
                      [ 'epages', join('/',$EpagesCartridges,'DE_EPAGES/Dojo/Data/Public/javascript/epages') ] );

    foreach my $CurrentCartridge ( @{$hBuildParams->{'CartridgeList'}} ){
        my $Package = $CurrentCartridge;
        $Package =~ s/::/\//g;
        my $ProfilesPath = join('/',$EpagesCartridges,$Package,'Data/DojoBuild');
        my $JavaScriptPath = join('/',$EpagesCartridges,$Package,'Data/javascript');

        if(ExistsDirectory($JavaScriptPath)){
            my $Prefix = 'epages.cartridges.' . lc $Package;
            $Prefix =~ s/\/|\\/\./g;
            push @Prefixes, [ $Prefix, $JavaScriptPath,
                              #join('/',$ProfilesPath,'copyright.txt')
                            ];
        }

        if(ExistsDirectory($ProfilesPath)){
            foreach my $ProfileFile (grep {/\.profile\.js$/} @{GetInfoDirectory($ProfilesPath)->{'Files'}}){
                my $hLayer = from_json(${GetFileContent($ProfileFile)});
                # push this profile into list for package building
                $hBuildParams->{'ProfileLayerMap'}{((split(/\./,((split(/\//,$ProfileFile))[-1])))[0])} = $hLayer->{'name'};
                $hLayer->{'layerDependencies'} = [] unless exists $hLayer->{'layerDependencies'};
                push @hJsonLayers, $hLayer;

                # get list of files to copy in hotfix dir
                if($HFFiles){
                    _addHofixFilesToCopyList($hBuildParams, $hLayer);
                }
            }
        }
    }

    # EPG-13714: add all additional files which are not contained in the shrinksafe packages.
    # They will be packer-compressed as single js files.
    foreach my $RemainingFile ( keys %{$hHotFix->{'files'}}){
        next if $hHotFix->{'files'}->{$RemainingFile} == 1;
        if($RemainingFile =~ /[\\\/]javascript[\\\/]/x ) { # path contains /javascript/
            my $CartridgeName = getCartridgeName($RemainingFile);
            my $RelPathToWidget = $RemainingFile;
            $RelPathToWidget =~ s/^.*javascript\///;
            my $ChangedFile;

            # widget class naming differs if from dojo cartridge
            if($CartridgeName =~ /dojo/ig) {
                $ChangedFile = join('/',
                    'javascript',
                    $hBuildParams->{'ConfigValues'}{'releaseName'},
                    $RelPathToWidget
                );
            } else {
                $ChangedFile = join('/',
                    'javascript',
                    $hBuildParams->{'ConfigValues'}{'releaseName'},
                    'epages',
                    'cartridges',
                    lc($CartridgeName),
                    $RelPathToWidget
                );
            }
            push @{$hHotFix->{'changedFiles'}}, $ChangedFile;
            $hHotFix->{'files'}{$RemainingFile} = 1;
        } else {
            push(@{$hHotFix->{'ignoredFiles'}}, $RemainingFile);
        }
    }

    if($ConsultingCartridge){
        _addConsultingCartridge($hBuildParams, \@Prefixes);
    }

    @hJsonLayers = sort { exists $b->{'discard'} <=> exists $a->{'discard'}
                          or scalar @{$a->{'layerDependencies'}} <=> scalar @{$b->{'layerDependencies'}}
                          or exists $a->{'name'} <=> scalar grep { /\Q$a->{'name'}\E/ } @{$b->{'layerDependencies'}}
                        } @hJsonLayers;

    WriteFile( join('/',$ReleaseDir,$ProfileFile),
               \join('=','dependencies',to_json( {'layers' => \@hJsonLayers,'prefixes' => \@Prefixes}, {'pretty'=>1} )
              ));

}

sub _addHofixFilesToCopyList {
    my ($hBuildParams, $hLayer) = @_;

    my $ConfigValues     = $hBuildParams->{'ConfigValues'};
    my $EpagesCartridges = $hBuildParams->{'EpagesCartridges'};
    my $hHotFix          = $hBuildParams->{'Hotfix'};

    foreach my $HFFile (keys %{$hHotFix->{'files'}}){
        if($HFFile =~ /\.js$/) { # js files
            $hHotFix = UpdateChangedFiles($HFFile, $hLayer, $ConfigValues, $hHotFix);
        }
        elsif ($HFFile =~ /templates(\\|\/).*\.html$/x) { # js templates
            my ($TemplateJSPath,undef,undef) = fsplit($HFFile);         # Cartridges/DE_EPAGES/Dojo/Data/Public/javascript/epages/widget/templates
            $TemplateJSPath =~ s/^Cartridges/$EpagesCartridges/;        # %EPAGES_CARTRIDGES%/DE_EPAGES/Dojo/Data/Public/javascript/epages/widget/templates
            $TemplateJSPath = RealPath($TemplateJSPath.'/..');          # %EPAGES_CARTRIDGES%/DE_EPAGES/Dojo/Data/Public/javascript/epages/widget

            # look inside all js files where this template is used
            my $TemplateFileName = (split(/\//,$HFFile))[-1];

            my @AllJSFilesInCartridge = grep { /\.js$/ } @{GetFileNames( $TemplateJSPath, {'recursive' => 1} )};
            foreach my $JSFile (@AllJSFilesInCartridge) {
                my $Content = ${GetFileContent($JSFile)};
                # two versions in dojo's template mapping
                if($Content =~ /templatePath.*templates\/$TemplateFileName/ix or
                   $Content =~ /templateString.*dojo.cache.*templates\/$TemplateFileName/ix) {
                    # using that name of the js class, look inside the layer if it's one of the dependencies.
                    # if yes, this adds corresponding shrinksafe package and js file to changed list
                    $hHotFix = UpdateChangedFiles($JSFile, $hLayer, $hBuildParams->{'ConfigValues'}, $hHotFix);
                    # add template itselt to list as well
                    my $ChangedTemplateString = widgetFilePathToJSClass($HFFile);
                    $ChangedTemplateString =~ s/\./\//g;
                    push @{$hHotFix->{'changedFiles'}}, join('/','javascript',$hBuildParams->{'ConfigValues'}{'releaseName'}, $ChangedTemplateString) . '.html';
                    $hHotFix->{'files'}{$HFFile} = 1;
                }
            }
        }
    }
}


sub _getBuildParams {
    my ($hParams) = @_;

    my $StoreType        = $hParams->{'StoreType'};
    my $CustomReleaseDir = $hParams->{'CustomReleaseDir'};
    my $ConfigFile       = $hParams->{'ConfigFile'};
    my $Version          = $hParams->{'Version'};

    $ConfigFile = "$ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES/Dojo/Scripts/build.conf" unless $ConfigFile;

    my $hCartridges = GetCartridges();
    die "database type $StoreType does not exist in Cartridges.xml" unless defined $hCartridges->{$StoreType};
    my @CartridgeList = @{$hCartridges->{$StoreType}};

    unless ( grep { $_ eq 'DE_EPAGES::Dojo' } @{$hCartridges->{$StoreType}} ) {
        print "DE_EPAGES::Dojo does not exist in database type $StoreType\n";
        return;
    }

    my $Webroot = $ENV{'EPAGES_WEBROOT'};
    my $EpagesCartridges = $ENV{'EPAGES_CARTRIDGES'};

    my $Java = DE_EPAGES::Core::API::Program::java->new;
    my $ScriptDir = join('/',$EpagesCartridges,'DE_EPAGES/Dojo/Scripts');
    $_ =~ s/\\/\//g foreach ($Webroot, $EpagesCartridges, $CustomReleaseDir);

    my $hConfigValues = DE_EPAGES::Core::API::IniConfig->new( FileName => $ConfigFile, CaseSensitive => 1 )->section('BuildParams');

    my $ProfileFile = $StoreType.'.profile.js';

    my $ReleaseDir = $CustomReleaseDir
                     ? join('/',($CustomReleaseDir, 'StoreTypes', $Version, $StoreType, 'javascript'))
                     : join('/',($Webroot, 'StoreTypes', $Version, $StoreType,'javascript'));
    $ReleaseDir = AbsolutePath($ReleaseDir);

    my %BuildParams = (
        'Cartridges'        => $hCartridges,
        'CartridgeList'     => \@CartridgeList,
        'WebRoot'           => $Webroot,
        'EpagesCartridges'  => $EpagesCartridges,
        'Java'              => $Java->command,
        'JavaVersion'       => $Java->version,
        'ScriptDir'         => $ScriptDir,
        'ConfigValues'      => $hConfigValues,
        'ProfileFile'       => $ProfileFile,
        'ReleaseDir'        => $ReleaseDir,
        'BuildLayers'       => [],
        'ProfileLayerMap'   => {},
        %$hParams,
    );

    return \%BuildParams;
}

sub _addConsultingCartridge {
    my ($hBuildParams, $aPrefixes) = @_;

    my $EpagesCartridges    = $hBuildParams->{'EpagesCartridges'};
    my $ConsultingCartridge = $hBuildParams->{'ConsultingCartridge'};

    my $JavaScriptPath = join('/',$EpagesCartridges, $ConsultingCartridge,'Data/javascript');
    my $ChangedFilesPath = join('/',$EpagesCartridges, $ConsultingCartridge,'Data/DojoBuild/changedCartridges.js');
    if(ExistsDirectory($JavaScriptPath) and ExistsFile($ChangedFilesPath)){
        my $hFileList = from_json(${GetFileContent($ChangedFilesPath)});
        push @$aPrefixes, [ $_, $JavaScriptPath,
                            #join('/',$ProfilesPath,'copyright.txt')
                          ] foreach @{$hFileList->{'changedCartridges'}};
    }
}

sub _processDojoShrinksafe {
    my ($hBuildParams) = @_;

    my $hConfigValues = $hBuildParams->{'ConfigValues'};
    my $ScriptDir     = $hBuildParams->{'ScriptDir'};
    my $Java          = $hBuildParams->{'Java'};
    my $JavaVersion   = $hBuildParams->{'JavaVersion'};
    my $ProfileFile   = $hBuildParams->{'ProfileFile'};
    my $ReleaseDir    = $hBuildParams->{'ReleaseDir'};
    my $Version       = $hBuildParams->{'Version'};
    my $aBuildLayers  = $hBuildParams->{'BuildLayers'};

    push my @ConfigValues, map { $_.'='.$hConfigValues->{$_} } keys %$hConfigValues;
    push @ConfigValues, (
                        'profileFile='.join('/',$ReleaseDir,$ProfileFile),
                        'releaseDir='.$ReleaseDir.'/',
                        'buildLayers='.join(',',@$aBuildLayers)
                        );

    push @ConfigValues, 'version='.$Version if $Version;
    RunOnDirectory( "$ScriptDir/util/buildscripts", sub {
        #-- Illegal reflective access operations will be denied in the future (Java 16)
        #-- https://docs.oracle.com/en/java/javase/11/migrate/index.html#GUID-7BB28E4D-99B3-4078-BDC4-FC24180CE82B
        #-- But we can still explicitly allow it with --add-opens, even if completely denied with '--illegal-access'
        #-- However, '--illegal-access' is deprecated, so don't use this anymore, assume '--illegal-access=deny'
        #-- js.jar accesses private sun.nio.ch.* members
        #-- To check which module provides what: (e.g. 'java.base' provides 'sun.nio'
        #-- java --list-modules | tr @ " " | awk '{ print $1 }' | xargs -n1 java -d
        my @AdditionalParams = version->parse($JavaVersion) >= 11 ? ('--add-opens=java.base/sun.nio.ch=ALL-UNNAMED') : ();

        my @Params = ( @AdditionalParams, '-classpath', '../shrinksafe/js.jar:../shrinksafe/shrinksafe.jar','org.mozilla.javascript.tools.shell.Main', 'build.js', @ConfigValues );
        # redirect standard output
        my $LogFileName = "$ENV{EPAGES_LOG}/DojoBuild.log";
        open my $oldout, '>&', STDOUT or die "Can't duplicate STDOUT: $!"; ##no critic (RequireBriefOpen)
        close STDOUT or die "Can't close STDOUT: $!";
        open STDOUT, '>>', $LogFileName or die "Can't redirect STDOUT to $LogFileName: $!";

        SystemCommand([$Java, @Params]);

        # restore the original standard output
        close STDOUT or die "Can't close STDOUT: $!";
        open STDOUT, '>&', $oldout or die "Can't restore old STDOUT: $!";
    });

    # fix white pages: create combined sf+dojo and bo+dojo packages
    if(ExistsFile($ReleaseDir.'/build/epages/ep_sf.js') && ExistsFile($ReleaseDir.'/build/dojo/dojo.js')) {
        CopyFile($ReleaseDir.'/build/dojo/dojo.js', $ReleaseDir.'/build/epages/ep_djsf.js');
        WriteFile($ReleaseDir.'/build/epages/ep_djsf.js', GetFileContent($ReleaseDir.'/build/epages/ep_sf.js'), { append => 1 });
    }
    if(ExistsFile($ReleaseDir.'/build/epages/ep_base.js') && ExistsFile($ReleaseDir.'/build/dojo/dojo.js')) {
        my @EpPkgs = (
            'build/dojo/dojo.js',
            'build/epages/ep_base.js',
            'build/epages/ep_shop.js',
            'build/epages/ep_shop_tree.js',
            'build/epages/ep_shop_preview.js',
            'build/epages/ep_shop_contenteditor.js',
            'build/epages/ep_content.js',
            'build/epages/ep_shop_upload.js',
            'build/epages/ep_mediagallery.js',
            'build/epages/ep_product.js',
            'build/epages/ep_design.js',
            'build/epages/ep_calendar.js',
            'build/epages/ep_externalcontent.js',
            'build/epages/ep_shopwizard.js',
        );

        my $Uncompressed = '.uncompressed.js';

        my $TContent;
        my $TContentUncompressed;

        foreach my $PkgFile (@EpPkgs) {
            # Add compressed content
            if (ExistsFile($ReleaseDir.'/'.$PkgFile)) {
                my (undef, $PkgName, undef) = fsplit($PkgFile);
                $TContent .= "\n/* package: $PkgName */\n";
                $TContent .= ${GetFileContent($ReleaseDir.'/'.$PkgFile)};
            }

            # Add uncompressed content
            if (ExistsFile($ReleaseDir.'/'.$PkgFile.$Uncompressed)) {
                my (undef, $PkgName, undef) = fsplit($PkgFile.$Uncompressed);
                $TContentUncompressed .= "\n/* package: $PkgName */\n";
                $TContentUncompressed .= ${GetFileContent($ReleaseDir.'/'.$PkgFile.$Uncompressed)};
            }
        }

        WriteFile($ReleaseDir.'/build/epages/ep_djbo.js', \$TContent);
        WriteFile($ReleaseDir.'/build/epages/ep_djbo.js'.$Uncompressed, \$TContentUncompressed);
    }

}

sub _setAccessRights {
    my ($hBuildParams) = @_;

    my $User             = $hBuildParams->{'User'};
    my $Group            = $hBuildParams->{'Group'};
    my $HFFiles          = $hBuildParams->{'HFFiles'};
    my $CustomReleaseDir = $hBuildParams->{'CustomReleaseDir'};
    my $StoreType        = $hBuildParams->{'StoreType'};
    my $ReleaseDir       = $hBuildParams->{'ReleaseDir'};

    my $hInfo = {};
    $hInfo->{User} = $User if defined $User;
    $hInfo->{Group} = $Group if defined $Group;

    my $hDir = GetInfoDirectory( $ReleaseDir, { recursive => 1 } );
    SetAccessRights($_, $hInfo) foreach ( @{$hDir->{Directories}}, @{$hDir->{Files}}, @{$hDir->{Other}} );
    SetAccessRights($ReleaseDir, $hInfo);

    if( $HFFiles && $CustomReleaseDir ) {
        my $HotfixDir = "$CustomReleaseDir/hotfix_".$StoreType;
        my $hDirHF = GetInfoDirectory( $HotfixDir, { recursive => 1 } );
        SetAccessRights($_, $hInfo) foreach ( @{$hDirHF->{Directories}}, @{$hDirHF->{Files}}, @{$hDirHF->{Other}} );
        SetAccessRights($HotfixDir, $hInfo);
    }
}

sub _copyHotfixFiles {
    my ($hBuildParams) = @_;

    my $StoreType        = $hBuildParams->{'StoreType'};
    my $CustomReleaseDir = $hBuildParams->{'CustomReleaseDir'};
    my $hHotFix          = $hBuildParams->{'HotFix'};

    # delete duplicates
    @{$hHotFix->{'changedFiles'}} = sort(unique( @{$hHotFix->{'changedFiles'}} ));

    if($CustomReleaseDir){
        for(my $i=0; $i<scalar(@{$hHotFix->{'changedFiles'}}); $i++) {
            my $ChangedFile = ${$hHotFix->{'changedFiles'}}[$i];
            my $SourceFile = join('/',$CustomReleaseDir,$StoreType,$ChangedFile);
            if(ExistsFile($SourceFile)) {
                CopyFile(
                  join('/',$CustomReleaseDir,$StoreType,$ChangedFile),
                  join('/',$CustomReleaseDir,'hotfix_'.$StoreType,$ChangedFile),
                  { 'create_directory' => 1, 'stat' => 'preserve' }
                );
            } else {
                push(@{$hHotFix->{'ignoredFiles'}},$SourceFile);
                ${$hHotFix->{'changedFiles'}}[$i] = undef;
            }
        }
    }

    @{$hHotFix->{'changedFiles'}} = grep { defined $_; } @{$hHotFix->{'changedFiles'}};
    print HotFixMessage($hHotFix);
}

sub _getHotfixFiles {
    my ($hBuildParams) = @_;

    my $aCartridgeList = $hBuildParams->{'CartridgeList'};
    my $HFFiles        = $hBuildParams->{'HFFiles'};

    # cartridge list for fast access { "$Commpany/$Cartridge" => 1, ... }
    my %Cartridges = map { join('/', /(.*)::(.*)/) => 1 } @$aCartridgeList;

    my $aFiles = ExistsDirectory( $HFFiles )
        ? GetFileNames( $HFFiles, {'recursive' => 1} )
        : [ map { chomp; $_; } @{GetFileContentLines($HFFiles)} ];

    return {
        map { $_ => 0 }
        grep { exists $Cartridges{ getCartridgeName($_) } }
        map { /\b(Cartridges[\\\/]\S+)/x }
        grep { !m,/CVS/, }
        grep { m,/javascript/, }
        @$aFiles
    };
}

#========================================================================================
# function     widgetFilePathToJSClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $JSClassName = widgetFilePathToJSClass($JSFileName);
#----------------------------------------------------------------------------------------
# description  Converts a file name like:
#   "Cartridges/DE_EPAGES/Dojo/Data/Public/javascript/epages/widget/Elementlist.js" or
#   "Cartridges/DE_EPAGES/MediaGallery/Data/javascript/widget/Filelistitem.js" or
#   "U Cartridges/DE_EPAGES/MediaGallery/Data/javascript/widget/Filetree.js"
# into the corresponding javascript class name used in the layer dependencies like:
#   "epages.widget.Elementlist" or
#   "epages.cartridges.de_epages.mediagallery.widget.Filelistitem"
#   "epages.cartridges.de_epages.mediagallery.widget.Filetree"
#----------------------------------------------------------------------------------------
# input        $AbsFileName | string with the file name | string
# return       $Target | javascript class name | string
#========================================================================================
sub widgetFilePathToJSClass {
    my ($AbsFileName) = @_;

    my $Target = $AbsFileName;
    my $FirstPart;
    if($Target !~ /(\/|\\)dojo(\/|\\)/igx) {
        # all other cartridges except dojo
        $FirstPart = $Target; # Cartridges/DE_EPAGES/MediaGallery/Data/javascript/widget/Filelistitem.js
        $FirstPart =~ s/(\/|\\)Data(\/|\\).*//igx;   # U Cartridges/DE_EPAGES/MediaGallery
        $FirstPart =~ s/^.*Cartridges/Cartridges/i;   # Cartridges/DE_EPAGES/MediaGallery
        $FirstPart =~ s/(\/|\\)/\./g; # Cartridges.DE_EPAGES.MediaGallery
        $FirstPart = lc($FirstPart); # cartridges.de_epages.mediaGallery
    }

    $Target =~ s/^.*javascript(\/|\\|)//x; # epages/widget/Elementlist.js
    $Target =~ s/\..*$//; # epages/widget/Elementlist
    $Target =~ s/(\/|\\)/\./g; # epages.widget.Elementlist

    $Target = 'epages.' . $FirstPart.'.'.$Target if $FirstPart; # epages. + cartridges.de_epages.mediagallery + . + widget.Filelistitem

    return $Target;
}

#========================================================================================
# function     UpdateChangedFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       %Hash = %{UpdateChangedFiles($FileName, $hLayer, $hConfigValues, \%Hash)};
# example      %HotFix = %{UpdateChangedFiles($HFFile, $hLayer, $hConfigValues, \%HotFix)};
#----------------------------------------------------------------------------------------
# description  Updates the content of the given $hHotFix hash ref. After aquiring the
#               javascript class name from the given $HFFile javascript file name this
#               string is searched for inside the list of dependencies inside the given
#               shrinksafe layer. If found, the corresponding layer name and the actual
#               javascript class is added to the changedFiles array - which later marks
#               the list of changed files for the hotfix.
#----------------------------------------------------------------------------------------
# input        $HFFile | javascript file name | string
# input        $hLayer | hash with the shrinksafe layer information | ref.hash.string
# input        $hConfigValues | configuration parameters of this script | ref.hash.string
# input        $hHotFix | hash with files and changedFiles list | ref.hash.array
# return       $hHotFix | changed hash with files and changedFiles list | ref.hash.array
#========================================================================================
sub UpdateChangedFiles {
    my ($HFFile, $hLayer, $hConfigValues, $hHotFix) = @_;

    my $Name = widgetFilePathToJSClass($HFFile);
    my ($Result) = grep { $_ =~ /\Q$Name\E$/ } @{$hLayer->{'dependencies'}};
    if($Result){
        push @{$hHotFix->{'changedFiles'}}, RealPath(join('/','javascript',$hConfigValues->{'releaseName'},'dojo',$hLayer->{'name'})) ;
        push @{$hHotFix->{'changedFiles'}}, RealPath(join('/','javascript',$hConfigValues->{'releaseName'},'dojo',$hLayer->{'name'} . '.uncompressed.js')) ;

        $Result =~ s/\./\//g;
        push @{$hHotFix->{'changedFiles'}}, join('/','javascript',$hConfigValues->{'releaseName'},$Result) . '.js';
        $hHotFix->{'files'}->{$HFFile} = 1;
    }

    return $hHotFix;
}


sub HotFixMessage{
    my ($hData) = @_;
    my $ReturnString;

    if($hData->{'changedFiles'}) {
        $ReturnString .=
            "\n\nChanged files for hotfix\n".
            ('-' x 24) . "\n" .
            join("\n",@{$hData->{'changedFiles'}})."\n\n";
    }
    if($hData->{'ignoredFiles'}) {
        $ReturnString .=
            "\nIgnored files for hotfix (not found in sources)\n".
            ('-' x 47) . "\n" .
            join("\n",@{$hData->{'ignoredFiles'}})."\n\n";
    }

    return $ReturnString;
}

sub getCartridgeName {
    my ($File) = @_;
    my @CartridgeNameParts = split(/\//, $File);
    shift @CartridgeNameParts if $CartridgeNameParts[0] =~ /^.*Cartridges/;
    my $CartridgeName = join('/', $CartridgeNameParts[0], $CartridgeNameParts[1]); # creates sth. like DE_EPAGES_TONLINE/TOnline
    return $CartridgeName;
}

1;
