#========================================================================================
# package      DE_EPAGES::Ebay::Hooks::Product
# state        public
# description  interface for table ReferenceAttribute
#========================================================================================
package DE_EPAGES::Ebay::Hooks::Product;
use base qw( Exporter );

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Ebay::API::Constants;

#========================================================================================
# function     OnDeleteProduct
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct(@PKeys);
#----------------------------------------------------------------------------------------
# description  OnDeleteProduct is triggered when a product is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Shop, key is
#               <ul>
#                   <li>ProductID
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;

    my $arProductAuctions = LoadClassByAlias('EbayAuction')->getObjects('ByProductID',[$hParams->{'ProductID'}]);

    foreach my $auction (@$arProductAuctions) {
        $auction->set({'Product' => undef});
    }
}

#========================================================================================
# function     OnAfterUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateProduct is triggered when a product is updated.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - product object       - object
#                 <li>ProductID     - product identifier   - int
#                 <li>Values        - updated values       - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    return unless defined $Product;
    my $Shop = $Product->getSite();
    return unless $Shop->get('EbayStockDecreaseType') eq 'OnAuctionEnd';
    return if $Product->get('IsMaster');

    # only react if stocklevel changed.
    return unless defined $hParams->{'Values'};
    my $StockLevel = $hParams->{'Values'}->{'StockLevel'};
    return unless defined $StockLevel;

    my $Class = LoadClassByAlias('EbayAuction');
    my $Auctions = $Class->getObjects( 'ByProductID', [ $hParams->{'ProductID'} ] );

    foreach my $Auction (@$Auctions){
        my $Status = $Auction->get('Status');

        my $doQuantityUpdate = $Auction->get('Quantity') != $StockLevel ? 1 : 0;
        # force Stocklevel update if parameter "IgnoreQuantityDifferences" is set
        $doQuantityUpdate = 1 if ((exists $hParams->{'IgnoreQuantityDifferences'}) && $hParams->{'IgnoreQuantityDifferences'});

        if ( ($Status == AUCTION_RUNNING)
         and ($Auction->get('ListingType') eq 'FixedPriceItem')
         and ($Auction->get('IsReviseInventory'))
         and ($doQuantityUpdate)) {
            eval {

                # stop an action which has an end date and stocklevel is decreased to 0 or lower.
                if ( $StockLevel <= 0 and $Auction->get('Duration') != DURATION_GOOD_TILL_CANCELED ) {
                    $Auction->set({'Quantity' => 0});
                    $Auction->end();
                }
                else {
                    if ( $StockLevel <= 0 ) {
                        $Auction->set({'Quantity' => 0});
                    }
                    else {
                        $Auction->set({'Quantity' => $StockLevel});
                    }
                    my $EbayShopEnv = $Auction->get('EbayShopEnvironment');
                    my $EbayOutOfStockPreference = $EbayShopEnv->get('EbayOutofStockPreference');
                    my $RemoteEbay = $Auction->remoteEbay();

                    # We only can to send 0 as stocklevel if OutOfStocklevelControlPreference is set to true,
                    # otherwise we are not able to set the stocklevel to 0 at ebay and want to reduce not needed requests
                    if ( $Auction->get('Quantity') > 0 or (($Auction->get('Quantity') == 0) and ($EbayOutOfStockPreference))) {
                        $RemoteEbay->reviseInventoryStatus($Auction->get('ExternalAuctionId'), undef, $Auction->get('Quantity'));
                    }
                }
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
            }
        }
    }

    my $MasterProduct = $Product->get('SuperProduct');
    if ($MasterProduct) {
        foreach my $Auction (@$Auctions){
            my $ParentAuction = $Auction->get('ParentAuction');
            next unless defined $ParentAuction;
            my $Status = $ParentAuction->get('Status');

            if (( $Status == AUCTION_RUNNING)
             && ($ParentAuction->get('ListingType') eq 'FixedPriceItem' && $ParentAuction->get('IsReviseInventory'))
             && $Auction->get('Quantity') != $StockLevel)
            {
                eval {
                    my $OldStockLevel = $Auction->get('Quantity') < 0 ? 0 : $Auction->get('Quantity');
                    my $NewStockLevel = $StockLevel < 0 ? 0 : $StockLevel;

                    my $MasterStockLevel = $ParentAuction->get('Quantity') - $OldStockLevel + $NewStockLevel;
                    $ParentAuction->set({'Quantity' => $MasterStockLevel});
                    $Auction->set({'Quantity' => $NewStockLevel});
                    my $RemoteEbay = $Auction->remoteEbay();
                    $RemoteEbay->reviseInventoryStatus($ParentAuction->get('ExternalAuctionId'),$Product->get('Alias'),$StockLevel);
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogError($Error);
                }
            }
        }
    }
}

1;