use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error                 qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script                qw ( RunScript );
use DE_EPAGES::Core::API::Warning               qw ( Warning );
use DE_EPAGES::Core::API::Log                   qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools             qw ( GetLocalTimeZone );
use DE_EPAGES::Database::API::Connection        qw ( RunOnStore );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Object::API::Factory             qw ( LoadObjectByPath JoinObjectPath );
use DE_EPAGES::Ebay::API::Ebay                  qw ( UpdateTransactionList UpdateRunningAuctions );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName = 'Store';
    my $EnvironmentAlias = 'SANDBOX';
    my $ShopAlias;
    my $Date;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'environment=s' => \$EnvironmentAlias,
        'shop=s'        => \$ShopAlias,
        'date=s'        => \$Date,
    );

    usage() if $Help;
    print "\nMissing shop!\n" and usage() unless $ShopAlias;
    print "\nMissing date!\n" and usage() unless $Date;

    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            # make sure that the cartridge is installed
            unless (ExistsCartridgeByPackage('DE_EPAGES::Ebay')) {
                print "Store: $StoreName : Cartridge Ebay is not installed.\n" unless $NoOutput;
                return;
            }
            CheckGetSellerTransactions( ($NoOutput)?0:1, $EnvironmentAlias, $ShopAlias, $Date )
        },
    );

    my $logmsg = sprintf 'need %.1f seconds', time()-$time1;
    print $logmsg."\n" unless $NoOutput;
    GetLog()->debug("Reset auction count: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub CheckGetSellerTransactions {
    my ($printOutput, $EnvironmentAlias, $ShopAlias, $Date) = @_;

    my $Shop = LoadObjectByPath(JoinObjectPath(['','Shops',$ShopAlias]));
    my $aEbayShopEnvironments = $Shop->get('EbayShopEnvironments');
    my @ShopEnv = grep {$_->alias eq $EnvironmentAlias} @$aEbayShopEnvironments;
    for (my $i=0; $i<scalar(@ShopEnv); $i++) {
        my $ShopEnv = $ShopEnv[$i];

        my $RemoteEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('ShopEnvironment'=>$ShopEnv);

        my ($Year,$Month,$Day) = split /-/, $Date;
        return unless defined ($Year && $Month && $Day);

        my $lastUpdate = DateTime->new(
             year   => $Year,
             month  => $Month,
             day    => $Day,
             hour   => 0,
             minute => 0,
             time_zone => GetLocalTimeZone(),
        );
        # eBay needs UTC
        $lastUpdate->set_time_zone('UTC');

        # get transactions
        my $time1 = time();
        my $aTransactions = $RemoteEbay->getSellerTransactions($lastUpdate);

        UpdateRunningAuctions( $ShopEnv, $printOutput );
        UpdateTransactionList( $ShopEnv, $aTransactions, 1);

        my $logmsg = sprintf 'need %.1f seconds', time()-$time1;
        print $logmsg."\n" if ($printOutput);
    }
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias                          ()
    -environment  ebay environment                    (SANDBOX)
    -date         retrieve transactions since         ()

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -environment SANDBOX -date 2006-10-21
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
