#========================================================================================
# package      DE_EPAGES::Ebay::UI::EbayShopEnvironment
# state        private
# base         DE_EPAGES::Presentation::UI::Object
#========================================================================================
package DE_EPAGES::Ebay::UI::EbayShopEnvironment;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Core::API::PerlTools qw ( VerifySecureCode CreateSecureCode ElementOf);
use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Core::API::Url qw(UrlEncodeValue);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Table::Country qw ( GetAllInfoCountries );
use DE_EPAGES::Ebay::API::Constants qw (EBAY_OAUTH_SCOPE);
use DE_EPAGES::Ebay::API::Object::RemoteRESTEbay;
use DE_EPAGES::Ebay::API::Table::EbayRegion qw ( GetInfoEbayRegionsByCountryID );
use URI::Escape qw( uri_unescape );
use MIME::Base64;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'EbaySiteNeedsCurrencyThatIsNotEnabledOnShop' => 'Currencies do not match.'
            . ' The choosen eBay Site needs #EbayCurrency as currency, in the shop'
            . ' are only these currencies avialable: #ShopCurrencies.',
        'ProviderSettingsIncomplete' => 'Your Provider\'s eBay settings are not complete.'
            . ' Please consult your provider.'
    };
}
#ERRORDEFINITION END

#==============================================================================
# Event handlers
#==============================================================================

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopEnv     = $Servlet->object;
    my $Environment = $ShopEnv->get('EbayEnvironment');

    # get the Runames (Live and SandBox) from Store
    my $hEbayAttributes = LoadRootObject()->get( [qw (EbayAppId EbaySandBoxAppId EbayRuName EbaySandBoxRuName ProviderPath) ] );
    my $IsSandbox  = $Environment->get('IsSandBox');
    my $EbayRuName = $IsSandbox ? $hEbayAttributes->{'EbaySandBoxRuName'} : $hEbayAttributes->{'EbayRuName'};
    my $EbayAppId  = $IsSandbox ? $hEbayAttributes->{'EbaySandBoxAppId'} : $hEbayAttributes->{'EbayAppId'};    
    my $ShopEnvURL = $ShopEnv->get('WebUrlAdminSSL');

    # use new urls for getting REST tokens
    # see https://developer.ebay.com/api-docs/static/oauth-permissions-grant-request.html
    my $OAuthSignInUrl = $Environment->get('OAuthGrandApplicationAccessEndpoint');

    eval {
        # should never happen
        Error('ProviderSettingsIncomplete') if (not $EbayRuName);

        #rebuild complete url as state parameter and create the security code with the Environment GUID
        my $ShopRedirectURL = $ShopEnvURL   ."&ViewAction=MBO-SaveEbayToken";
        my $SecurityCode = CreateSecureCode($ShopRedirectURL.$ShopEnv->get('GUID'));

        $Servlet->vars('EbaySignInUrl', $OAuthSignInUrl);
        
        # add the scope and sign in params for ebay authentication site
        # the state is the target url for fetching and saving the token after granting permissions on ebay
        # that means the Site is forwarding to the url in state
        my $SignInParams = '?client_id='     . $EbayAppId .
                           '&redirect_uri='  . $EbayRuName .
                           '&response_type=' . 'code' .
                           '&scope='         . EBAY_OAUTH_SCOPE.
                           '&state='         . UrlEncodeValue($ShopRedirectURL.'&sec='.$SecurityCode,'-'); 

        $Servlet->vars('EbaySignInParams',$SignInParams);
    };

    if (ExistsError()) {
        my $Error = GetError();
        # re-throw error if not settings implete error
        Error($Error) unless $Error->code eq 'ProviderSettingsIncomplete';
        $Servlet->vars('ERR_PROVIDERSETTINGSINCOMPLETE',1);
    }

    #load regions
    my $ahCountries = GetAllInfoCountries();
    foreach my $country (@$ahCountries) {
        $country->{'EbayRegions'} = GetInfoEbayRegionsByCountryID($country->{'CountryID'});
        if (not scalar @{$country->{'EbayRegions'}}) {
            $country->{'EbayRegions'} = [{'EbayRegionId' => 0, 'Name' => ''}];
        }
    }
    $Servlet->vars('Countries', $ahCountries);
}

# called from the MBO
sub NewShopEnvironment {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Shop    = $Servlet->object;
    my $IsUnityShop = $Shop->get('IsUnityRegistered');

    my $EnvID = $Servlet->form->formValue('EbayEnvironmentID');
    $Servlet->form->addFormError({
        'Name'      => 'EbayEnvironmentID',
        'Reason'    => 'WrongInputs',
    }) unless $EnvID;

    my $Name = $Servlet->form->formValue('Name');
    $Servlet->form->addFormError({
        'Name'      => 'Name',
        'Reason'    => 'WrongInputs',
    }) unless $Name;

    $Servlet->form->executeFormError;

    foreach my $ShopEnv (@{$Shop->folder('EbayShopEnvironments')->children}){
        if ($ShopEnv->get('Name') eq $Name) {
            $Servlet->form->executeFormError({
                'Name'   => 'Name',
                'Reason' => 'DuplicateName'
            });
        }
    }

    my $Env = LoadRootObject()->loadObject($EnvID, 'EbayEnvironment');

    eval {
        #check if currency id is available
        my $Currencies = $Shop->get('Currencies');
        my $currencyExists = 0;
        foreach my $cur (@$Currencies) {
            $currencyExists = 1 if ($cur->{'CurrencyID'} eq $Env->get('CurrencyID'));
        }
        if (not $currencyExists) {
            Error('EbaySiteNeedsCurrencyThatIsNotEnabledOnShop', {
                'EbayCurrency' => $Env->get('CurrencyID'),
                'ShopCurrencies' => join (', ', map {$_->{'CurrencyID'}} @$Currencies),
            });
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'EbaySiteNeedsCurrencyThatIsNotEnabledOnShop';

        $Servlet->form->mapFormError({'Name' => 'EbayEnvironmentID'}, $Error);
    } else {
        my $Folder  = $Shop->folder('EbayShopEnvironments');
        my $newShopEnv = $Folder->insertChild('EbayShopEnvironment',{
            'Alias'                 => $Folder->nextAlias('EbayShopEnvironment%s'),
            'EbayEnvironment'       => $Env,
            'Name'                  => $Name,
            'LastUpdate'            => GetCurrentDBHandle()->currentDateTime,
        });

        # if this is the first shop env
        # -> set it as the shop env of the default auction
        if (scalar @{$Folder->children} == 1) {
            $Shop->get('EbayDefaultAuction')->set({'EbayShopEnvironment' => $newShopEnv});
        }
        if ($IsUnityShop) {
            $Servlet->vars('ViewAction', 'MBO-ViewGeneral');
            my $ObjID = $newShopEnv -> get('ObjectID');
            $Servlet->vars('ObjectID', $ObjID);
        }

    }

    $Servlet->form->executeFormError;
    return;
}

# called from the MBO
sub CheckEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopEnv = $Servlet->object;

    eval {
        my $RemoteRESTEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('ShopEnvironment'   => $ShopEnv,);
        my $Success  = $RemoteRESTEbay->getCategories(1); 
        Error('TestNotSuccessful') unless $Success;
    };

    if (ExistsError()) {
        my $Error = GetError();
        my $Message = $Error->message;

        GetLog()->debug('EbayToken not valid: '.$Message);
        $Servlet->vars('ERR_TOKEN',$Message);

        # remove Token because if the test fails we have problems also with the refresh token
        $ShopEnv->set({
            'EbayToken' => undef,
            'EbayTokenExpiryDate' => undef,
            'EbayOAuthToken' => undef,
            'EbayOAuthTokenExpiryDate' => undef,
            'EbayUserName' => undef,
        });
    }
    $Servlet->vars('TOKENTESTED',1);

    return;
}

# called from the MBO
sub CopyEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopEnv = $Servlet->object;
    my $Shop = $ShopEnv->getSite();

    my $CopyFromShopEnvID = $Servlet->form->formValue('EbayShopEnvironment');
    return unless $CopyFromShopEnvID;

    my $CopyFromShopEnv = $Shop->loadObject( $CopyFromShopEnvID, 'EbayShopEnvironment' );

    $ShopEnv->set({
        'EbayToken'                => $CopyFromShopEnv->get('EbayToken'),
        'EbayTokenExpiryDate'      => $CopyFromShopEnv->get('EbayTokenExpiryDate'),
        'EbayOAuthToken'           => $CopyFromShopEnv->get('EbayOAuthToken'),
        'EbayOAuthTokenExpiryDate' => $CopyFromShopEnv->get('EbayOAuthTokenExpiryDate'),
        'EbayUserName'             => $CopyFromShopEnv->get('EbayUserName'),
    });

    return;
}

# called from the MBO
sub DeleteEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopEnv = $Servlet->object;

    my $hFormValues = $Servlet->form->formValues;

    if (not $hFormValues->{'ACCEPT'}->[0]) {
        $Servlet->vars('DELETE',1);
        return;
    } else {
        my $now = GetCurrentDBHandle()->currentDateTime;
        $ShopEnv->set({
            'EbayToken'             => undef ,
            'EbayTokenExpiryDate'   => $now,
            'EbayOAuthToken'        => undef,
            'EbayOAuthTokenExpiryDate' => $now,
            'EbayUserName'          => undef,
        });
        $Servlet->vars('TOKEN_DELETED',1);
    }

    return;
}

sub DeleteShopEnvironments {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my @DeleteEnvironmentIDs = $Servlet->form->formValue('ListObjectID');
    #Nothing to do
    return unless scalar(@DeleteEnvironmentIDs); 

    my $EbayDefaultAuction     = $Shop->get('EbayDefaultAuction');
    my $DefaultShopEnvironment = $EbayDefaultAuction->get('EbayShopEnvironment');
    #If we delete the default environment, display hint.
    if (ElementOf($DefaultShopEnvironment->id(), @DeleteEnvironmentIDs)) {
        $Servlet->vars('DeleteDefaultEnvironment', 1);
    }

    # standard delete function
    $self->DeleteListedObjects($Servlet);
    #set a remaining EbayShopEnvironment as default if the former default was deleted
    $Shop = $Servlet->object();
    my $hAttributeValues = $Shop->get(['EbayShopEnvironments', 'EbayDefaultAuction']);
    my $aAllEnvironments = $hAttributeValues->{'EbayShopEnvironments'};
    my $EbayDefaultAuctionAfterDelete = $hAttributeValues->{'EbayDefaultAuction'};
    my $EbayDefaultEnvironmentAfterDelete = $EbayDefaultAuctionAfterDelete->get('EbayShopEnvironment');
    $EbayDefaultAuctionAfterDelete->set({'EbayShopEnvironment' => $aAllEnvironments->[0]}) unless $EbayDefaultEnvironmentAfterDelete ;

    return;
}

# called from the Site for getting a token by the given code!
# Sample:
#https://<server>/epages/DemoShop.admin/?ObjectPath=/Shops/DemoShop/EbayShopEnvironments/EbayShopEnvironment3
#&ViewAction=MBO-SaveEbayToken
#&sec=55ce0e878ea9f31ed0616c2ee87d5d9c
#&code=v%5E1.1%23i%5E1%23I%5E3%23p%5E3%23f%5E0%23r%5E1%23t%5EUl41Xzk6QjhCQTg5OTJFQUMzMDJBNjExNjI4Mjc2QjNDN0E4QzlfMl8xI0VeMTI4NA#%3D%3D
#&expires_in=299' 
sub SaveEbayToken {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShopEnv     = $Servlet->object;
    my $ShopEnvURL  = $ShopEnv->get('WebUrlAdminSSL');
    my $Form        = $Servlet->form;

    my $hFormValues = {};
    foreach my $Values (qw(sec code expires_in)) {
        $hFormValues->{$Values} = $Form->queryValue($Values);
        $self->_showEnvPage($Servlet, $ShopEnv, 1)  unless $hFormValues->{$Values};
    }

    #verify security code
    my $ShopRedirectURL = $ShopEnvURL   ."&ViewAction=MBO-SaveEbayToken";

    $self->_showEnvPage($Servlet, $ShopEnv, 1) unless VerifySecureCode($ShopRedirectURL.$ShopEnv->get('GUID'), $hFormValues->{'sec'});

    # get the token by calling a RemoteEbay function
    my $hResult = {};
    my $RemoteRESTEbay;
    eval {
        $RemoteRESTEbay = DE_EPAGES::Ebay::API::Object::RemoteRESTEbay->new('ShopEnvironment' => $ShopEnv);
        $hResult = $RemoteRESTEbay->getOAuthTokensByAuthCode( uri_unescape($hFormValues->{'code'}) );
    };
    if (ExistsError()){
         my $Error = GetError();
         my $Message = $Error->shortMessage;
         GetLog()->debug('getOAuthTokensByAuthCode failed: '.$Message);
         $Servlet->vars('ERR_FETCH_TOKEN',$Message);
         $self->_showEnvPage($Servlet, $ShopEnv, 1);
    } else {

        $RemoteRESTEbay->saveOAuthToken($hResult);

        if ($RemoteRESTEbay->existsOAuthToken()) {
            $Servlet->vars('SUCCESS_SAVE_TOKEN', 1);
            # force ebay checkout enabled!
            eval {
                $RemoteRESTEbay->setUserPreferencesDisplayPayNowButton('ShowPayNowButtonForAllPaymentMethods');
            };
            if (ExistsError()) {
                my $Error = GetError();
                Warning($Error);
            }
        }
    }

    # Get the OutOfStock setting from ebay and set its value to EbayOutOfStockPreference attribute
    $ShopEnv->setEbayOutOfStockPreference();

    $self->_showEnvPage($Servlet, $ShopEnv, 0);


    return;

}

sub _showEnvPage {
    my $self = shift;
    my ($Servlet, $ShopEnv, $IsError) = @_;

    $Servlet->vars('TOKEN_NOT_SAVED', 1) if $IsError;
    $Servlet->vars('ERR_FETCH_TOKEN', 1) if $IsError;

    # show the ebay environment page
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $ShopEnv->id,
        },
        'ViewAction' => 'MBO-ViewGeneral'
    });
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::Save($Servlet);

    if (not $Servlet->params('EbayLocation')) {
        $Servlet->form->executeFormError({
            'Name'      => 'EbayLocation',
            'Reason'    => 'NoEbayLocation',
        });
    }

    $Servlet->vars('SETTINGS_SAVED', 1);

}

1;
