use strict;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle Transaction RunOnStore);
use Getopt::Long;
use MIME::Base64 qw (decode_base64);
use Time::HiRes;
use JSON;

##no critic qw(ProhibitCascadingIfElse)

sub ReplaceGadgets {
    my ($DBI) = @_;

    Transaction ( sub {

        # Get string attributes with gadgets.
        my $aStringValues = $DBI->execute( <<END_SQL );
            SELECT
                stringattribute.value,
                stringattribute.attributeid,
                stringattribute.objectid
            FROM
                stringattribute
            WHERE
                stringattribute.value LIKE '%BEGIN_GADGET%ep_info%';
END_SQL

        # Get localized string attributes with gadgets.
        my $aLocalizedStringValues = $DBI->execute( <<END_SQL );
            SELECT
                localizedstringattribute.value,
                localizedstringattribute.attributeid,
                localizedstringattribute.objectid,
                localizedstringattribute.languageid
            FROM
                localizedstringattribute
            WHERE
                localizedstringattribute.value LIKE '%BEGIN_GADGET%ep_info%';
END_SQL

        # Replace the old gadgets with the new one. Mark not replacable gadgets with "outdated".
        _ReplaceOldGadgetsStringAttributes($aStringValues, $DBI);
        _ReplaceOldGadgetsLocalizedStringAttributes($aLocalizedStringValues, $DBI);

    });
    return ;
}

sub _ReplaceOldGadgetsStringAttributes {
    my ($aStringAttr, $DBI) = @_;

    foreach my $aString (@$aStringAttr) {
        my $AttributeValue = $aString->[0];

        # Replace old gadgets in current string attribute.
        my ($Changed, $NewAttributeValue) = _ReplaceOldGadgetString($AttributeValue);

        # Update current string attribute if there are some replacements.
        if($Changed) {
            my $ObjectID = $aString->[2];
            my $AttributeID = $aString->[1];

            $DBI->do(<<END_SQL, [ $NewAttributeValue, $AttributeID, $ObjectID ]);
            UPDATE
                stringattribute AS attr
            SET
                attr.value = ?
            WHERE
                attr.attributeid = ? AND objectid = ?;
END_SQL
        }
    }
    return;
}

sub _ReplaceOldGadgetsLocalizedStringAttributes {
    my ($aStringAttr, $DBI) = @_;

    foreach my $aString (@$aStringAttr) {
        my $AttributeValue = $aString->[0];

        # Replace old gadgets in current localized string attribute.
        my ($Changed, $NewAttributeValue) = _ReplaceOldGadgetString($AttributeValue);

        # Update current localized string attribute if there are some replacements.
        if($Changed) {
            my $LanguageID = $aString->[3];
            my $ObjectID = $aString->[2];
            my $AttributeID = $aString->[1];

            $DBI->do(<<END_SQL, [ $NewAttributeValue, $AttributeID, $ObjectID, $LanguageID ]);
            UPDATE
                localizedstringattribute AS attr
            SET
                attr.value = ?
            WHERE
                attr.attributeid = ? AND objectid = ? AND languageid = ?;
END_SQL
        }
    }
    return;
}

sub _ReplaceOldGadgetString {
    my ($AttributeValue) = @_;

    my $Changed = 0;
    # Search for old gadgets.
    while($AttributeValue =~ m/(<!--\s(BEGIN_GADGET).*?(ep_info:|ep_info=")(.*?)".*?(END_GADGET)\s-->)/xsg) {
        # Complete gadget string.
        my $GadgetLine = $1;
        # Start comment.
        my $GadgetStartComment = $2;
        # Content of "ep-info" (base64 string with gadget settings).
        my $GadgetInfo = $4;
        # End comment.
        my $GadgetEndComment = $5;

        # Build replace string, if there is no new gadget for the old one.
        # Change comment "<!-- BEGIN_GADGET ... END_GADGET -->" to "<!-- BEGIN_OUTDATED_GADGET ... END_OUTDATED_GADGET -->".
        my $OutDatedString = $GadgetLine;
        $OutDatedString =~ s/$GadgetStartComment/BEGIN_OUTDATED_GADGET/x;
        $OutDatedString =~ s/$GadgetEndComment/END_OUTDATED_GADGET/x;

        # Check if there are gadget info.
        if(defined $GadgetInfo and $GadgetInfo ne ''){
            # Decode the base64 string.
            my $DecodedString = decode_base64($GadgetInfo);

            # Do some magic to make the string compatible for the from_json methode.
            $DecodedString =~ s/\\\n/\\n/gx;
            $DecodedString =~ s/\'/'/gx;
            $DecodedString =~ s/undefined|undef/""/gx;

            # Make perl hash.
            my $hFinal = from_json($DecodedString);

            # Get contentclass from the gadget perl hash to identify the gadgets.
            my $ContentClass = $hFinal->{'contentClass'};

            my $ReplaceString;
            if   ($ContentClass =~ /KlicktelMap$/)        { $ReplaceString = _ReplaceKlicktel($hFinal) }
            elsif($ContentClass =~ /GoogleMap$/)          { $ReplaceString = _ReplaceGoogleMaps($hFinal) }
            elsif($ContentClass =~ /BingMap$/)            { $ReplaceString = _ReplaceBingMaps($hFinal) }
            elsif($ContentClass =~ /GoogleTranslate$/)    { $ReplaceString = _ReplaceGoogleTranslate() }
            elsif($ContentClass =~ /Babelfish$/)          { $ReplaceString = _ReplaceBabelFishTranslate() }
            elsif($ContentClass =~ /RSSReader$/)          { $ReplaceString = _ReplaceRSSFeed($hFinal) }
            elsif($ContentClass =~ /SocialBookmarks$/)    { $ReplaceString = _ReplaceSocialBookmarks($hFinal) }
            elsif($ContentClass =~ /googleplus$/)         { $ReplaceString = _ReplaceGooglePlus($hFinal) }
            elsif($ContentClass =~ /fblike$/)             { $ReplaceString = _ReplaceFaceBookLikeButton($hFinal) }
            elsif($ContentClass =~ /fblikebox$/)          { $ReplaceString = _ReplaceFaceBookLikeBox($hFinal) }
            elsif($ContentClass =~ /Twitter$/)            { $ReplaceString = _ReplaceTwitterProfile($hFinal) }
            elsif($ContentClass =~ /TwitterFollowMe$/)    { $ReplaceString = _ReplaceTwitterFollowMe($hFinal) }
            elsif($ContentClass =~ /TwitterTweetThis$/)   { $ReplaceString = _ReplaceTwitterTweetThis($hFinal) }
            elsif($ContentClass =~ /TwitterSearch$/)      { $ReplaceString = _ReplaceTwitterSearch($hFinal) }
            elsif($ContentClass =~ /YouTube$/)            { $ReplaceString = _ReplaceYouTube($hFinal) }
            elsif($ContentClass =~ /MyVideo$/)            { $ReplaceString = _ReplaceMyVideo($hFinal) }
            elsif($ContentClass =~ /DailyMotion$/)        { $ReplaceString = _ReplaceDailyMotion($hFinal) }
            elsif($ContentClass =~ /Vimeo$/)              { $ReplaceString = _ReplaceVimeo($hFinal) }
            elsif($ContentClass =~ /Wideo$/)              { $ReplaceString = _ReplaceWideo($hFinal) }
            elsif($ContentClass =~ /Metacafe$/)           { $ReplaceString = _ReplaceMetacafe($hFinal) }
            # Default: Set outdated comments instead of ordinary gadget begin- and endcomment.
            else { $ReplaceString = $OutDatedString }

            # Replace the old gadget with the new one in the current attribute.
            $AttributeValue =~ s/\Q$GadgetLine\E/$ReplaceString/sx;
        }
        # There has something changed... in the neighborhood... who you gonna call?
        $Changed = 1;
    }

    return ($Changed, $AttributeValue);
}

# MAPS
# Klicktel
sub _ReplaceKlicktel {
    my ($hFinal) = @_;

    my $Width = $hFinal->{'MapWidth'};
    my $Height = $hFinal->{'MapHeight'};
    my $ContactPerson = $hFinal->{'ContactPerson'};
    my $Address = $hFinal->{'Address'};
    my $EMail = $hFinal->{'EMail'};
    my $LayoutLR = $hFinal->{'LayoutLR'};
    my $LargeNavigation = $hFinal->{'LargeNavigation'};
    my $UseRoute = $hFinal->{'UseRoute'};
    my $ShowAddress = $hFinal->{'ShowAddress'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;klicktel&quot;,&quot;address&quot;:&quot;'.$Address.'&quot;,&quot;contactPerson&quot;:&quot;'.$ContactPerson.'&quot;,&quot;email&quot;:&quot;'.$EMail.'&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;useRoute&quot;:'.$UseRoute.',&quot;showAddress&quot;:'.$ShowAddress.',&quot;largeNavigation&quot;:'.$LargeNavigation.',&quot;layoutLR&quot;:'.$LayoutLR.'},&quot;alias&quot;:&quot;maps&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';;
}
#Google
sub _ReplaceGoogleMaps {
    my ($hFinal) = @_;

    my $MapSize = $hFinal->{'formData'}->{'_data'}->{'mapsize'}->[0];
    my ( $Width, $Height ) = split(/\sx\s/, $MapSize);
    my $Controls = $hFinal->{'controls'};
    my $Zoom = int($hFinal->{'zoom'});
    my $Lat = $hFinal->{'centerLat'};
    my $Lng = $hFinal->{'centerLng'};
    my $Title = $hFinal->{'formData'}->{'_data'}->{'googlemaps:text:0'}->[0];
    $Title =~ s/\n//;

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;google&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;controls&quot;:&quot;'.$Controls.'&quot;,&quot;zoom&quot;:'.$Zoom.',&quot;controls&quot;:&quot;'.$Controls.'&quot;,&quot;title&quot;:&quot;'.$Title.'&quot;,&quot;lat&quot;:&quot;'.$Lat.'&quot;,&quot;lng&quot;:&quot;'.$Lng.'&quot;,&quot;fixedView&quot;:0},&quot;alias&quot;:&quot;maps&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Bing
sub _ReplaceBingMaps {
    my ($hFinal) = @_;

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};
    my $Controls = $hFinal->{'controls'};
    my $Zoom = $hFinal->{'zoom'};
    my $Lat = $hFinal->{'centerLat'};
    my $Lng = $hFinal->{'centerLng'};
    my $Title = $hFinal->{'formData'}->{'markertext'}->[0];
    $Title =~ s/\n//;
    my $FixedView = $hFinal->{'mapFixed'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;bing&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;controls&quot;:&quot;'.$Controls.'&quot;,&quot;zoom&quot;:&quot;'.$Zoom.'&quot;,&quot;controls&quot;:&quot;'.$Controls.'&quot;,&quot;title&quot;:&quot;'.$Title.'&quot;,&quot;lat&quot;:&quot;'.$Lat.'&quot;,&quot;lng&quot;:&quot;'.$Lng.'&quot;,&quot;fixedView&quot;:&quot;'.$FixedView.'&quot;},&quot;alias&quot;:&quot;maps&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}

# Translate
# Google
sub _ReplaceGoogleTranslate {
    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;service&quot;:&quot;google&quot;},&quot;alias&quot;:&quot;translation&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Microsoft (Babelfish)
sub _ReplaceBabelFishTranslate {
    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;service&quot;:&quot;microsoft&quot;},&quot;alias&quot;:&quot;translation&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}

# RSS
sub _ReplaceRSSFeed {
    my ($hFinal) = @_;

    my $NumberOfEntries = $hFinal->{'formData'}->{'RSSReaderEntrys'}->[0];
    my $OnlyHeadlines = _checkBoolean($hFinal->{'onlyheadlines'});
    my $Layout = $hFinal->{'style'} eq 'classic' ? 'default' : $hFinal->{'style'};
    my $URL = uri_unescape($hFinal->{'formData'}->{'RSSReaderFeedUrl'}->[0]);
    my $RemoveImages = _checkBoolean($hFinal->{'stripimages'});
    my $ShowImages = !$RemoveImages;
    my $OnlySnippets = _checkBoolean($hFinal->{'textlength'});

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;url&quot;:&quot;'.$URL.'&quot;,&quot;numberOfEntries&quot;:&quot;'.$NumberOfEntries.'&quot;,&quot;onlyHeadlines&quot;:'.$OnlyHeadlines.',&quot;onlySnippets&quot;:'.$OnlySnippets.',&quot;showImages&quot;:'.$ShowImages.',&quot;layout&quot;:&quot;'.$Layout.'&quot;},&quot;alias&quot;:&quot;rss&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}

sub _checkBoolean {
    my ($Value) = @_;

    if ($Value eq 'true' or $Value eq '1') {
        return 1;
    } elsif ($Value eq 'false' or $Value eq '0') {
        return 0;
    } else {
        return $Value;
    }
}

# SocialMedia
# SocialBookmarks
sub _ReplaceSocialBookmarks {
    my ($hFinal) = @_;

    my $ButtonString;
    foreach my $SocialButton (qw(digg delicious stumbleupon)) {
        if($hFinal->{$SocialButton} eq 'true') {
            $ButtonString .= defined $ButtonString ? ",&quot;$SocialButton&quot;" : "&quot;$SocialButton&quot;";
        }
    }
    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;services&quot;:['.$ButtonString.']},&quot;alias&quot;:&quot;otherbuttons&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# GooglePlus
sub _ReplaceGooglePlus {
    my ($hFinal) = @_;

    my $Type = $hFinal->{'counter'} ? 'bubble' : 'none';
    my $Layout = defined $hFinal->{'size'} && length($hFinal->{'size'}) > 0 ? $hFinal->{'size'} : 'standard';
    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;layout&quot;:&quot;'.$Layout.'&quot;,&quot;type&quot;:&quot;'.$Type.'&quot;},&quot;alias&quot;:&quot;googleplus&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# FaceBook
# FBLikeButton
sub _ReplaceFaceBookLikeButton {
    my ($hFinal) = @_;

    my $Action = $hFinal->{'fb_action'};
    my $Layout = $hFinal->{'fb_layout'};
    my $ShowFaces = $hFinal->{'fb_show_faces'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;button&quot;,&quot;action&quot;:&quot;'.$Action.'&quot;,&quot;layout&quot;:&quot;'.$Layout.'&quot;,&quot;showFaces&quot;:&quot;'.$ShowFaces.'&quot;},&quot;alias&quot;:&quot;facebook&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# FBLikeBox
sub _ReplaceFaceBookLikeBox {
    my ($hFinal) = @_;

    my $URL = $hFinal->{'url'};
    my $Color = $hFinal->{'colorscheme'};
    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};
    my $Header = $hFinal->{'header'};
    my $Stream = $hFinal->{'stream'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;box&quot;,&quot;url&quot;:&quot;'.$URL.'&quot;,&quot;color&quot;:&quot;'.$Color.'&quot;,&quot;showFaces&quot;:&quot;true&quot;,&quot;showStream&quot;:&quot;'.$Stream.'&quot;,&quot;showHeader&quot;:&quot;'.$Header.'&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;},&quot;alias&quot;:&quot;facebook&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Twitter
# TwitterProfile
sub _ReplaceTwitterProfile {
    my ($hFinal) = @_;

    my $UserName = $hFinal->{'formData'}->{'twitterUserName'}->[0];
    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;profile&quot;,&quot;username&quot;:&quot;'.$UserName.'&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;},&quot;alias&quot;:&quot;twitter&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# TwitterFollowMe
sub _ReplaceTwitterFollowMe {
    my ($hFinal) = @_;

    my $UserName = $hFinal->{'formData'}->{'twitterUserName'}->[0];
    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;profile&quot;,&quot;username&quot;:&quot;'.$UserName.'&quot;,&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;},&quot;alias&quot;:&quot;twitter&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# TwitterTweetThis
sub _ReplaceTwitterTweetThis {
    my ($hFinal) = @_;

    my $Count = $hFinal->{'ButtonCount'};
    my $Text = $hFinal->{'formData'}->{'TweetText_Insert'}->[0];
    my $UserName = $hFinal->{'formData'}->{'Twitter_AccountName'}->[0];
    my $URL = $hFinal->{'formData'}->{'TweetURL_Insert'}->[0];

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;share&quot;,&quot;count&quot;:&quot;'.$Count.'&quot;,&quot;text&quot;:&quot;'.$Text.'&quot;,&quot;username&quot;:&quot;'.$UserName.'&quot;,&quot;url&quot;:&quot;'.$URL.'&quot;},&quot;alias&quot;:&quot;twitter&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# TwitterSearch
sub _ReplaceTwitterSearch {
    my ($hFinal) = @_;

    my $SearchTerm = $hFinal->{'query_tags'}->[0];
    $SearchTerm = 'ePages' unless defined $SearchTerm or length($SearchTerm) == 0;
    my $Title = $hFinal->{'title'}->[0];
    my $Caption = $hFinal->{'caption'}->[0];

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;type&quot;:&quot;search&quot;,&quot;searchterm&quot;:&quot;'.$SearchTerm.'&quot;,&quot;title&quot;:&quot;'.$Title.'&quot;,&quot;subject&quot;:&quot;'.$Caption.'&quot;},&quot;alias&quot;:&quot;twitter&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}

# ExternalVideo (regular expressions taken from ExternalContent/Data/Public/GUI/de_epages/externalcontent/gadget.js)
# Youtube
sub _ReplaceYouTube {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'youTubeInput'}->[0];
    if ($ID =~ m/v=([\w-]+)|be\/([\w-]+)|embed\/([\w-]+)/x) {
        $ID = $1;
    }

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;youtube&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Vimeo
sub _ReplaceVimeo {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'vimeoInput'}->[0];

    if ($ID =~ m/vimeo.com\/?[video]*?\/([\w-]+)/x) {
        $ID = $1;
    }
    $ID =~ s/v=//;

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;vimeo&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# DailyMotion
sub _ReplaceDailyMotion {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'dailyMotionInput'}->[0];
    if($ID =~ m/video\/([\w-]+)/x) {
        $ID = $1;
    }
    $ID =~ s/v=//;

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;dailymotion&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# MyVideo
sub _ReplaceMyVideo {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'myVideoInput'}->[0];
    if($ID =~ m/watch\/([\w-]+)/x) {
        $ID = $1;
    }
    $ID =~ s/movie=//;

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:'.$Width.',&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;myvideo&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Wideo
sub _ReplaceWideo {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'wideoInput'}->[0];
    if($ID =~ m/wideo\.fr\/video\/([\w-]+)/x) {
        $ID = $1;
    }
    $ID =~ s/v=//;

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;wideofr&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}
# Metacafe
sub _ReplaceMetacafe {
    my ($hFinal) = @_;

    my $ID = $hFinal->{'formData'}->{'metacafeInput'}->[0];
    if($ID =~ m/watch\/([\w-]+\/[\w-]+)/x) {
        $ID = $1;
    }

    my $Width = $hFinal->{'width'};
    my $Height = $hFinal->{'height'};

    return '<div class="ep-gadget" data-ep-gadget="{&quot;options&quot;:{&quot;width&quot;:&quot;'.$Width.'&quot;,&quot;height&quot;:&quot;'.$Height.'&quot;,&quot;provider&quot;:&quot;metacafe&quot;,&quot;id&quot;:&quot;'.$ID.'&quot;},&quot;alias&quot;:&quot;externalvideo&quot;,&quot;cartridge&quot;:&quot;externalcontent&quot;}"></div>';
}

sub Main {
     local $| = 1;

    my $Password;
    my $StoreName = 'Store';
    my $Help;
    my $NoOutput = 0;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'nooutput=s'     => \$NoOutput,
    );

    usage() if $Help;
    my $time1 = time;

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            ReplaceGadgets(GetCurrentDBHandle());
        }
    );

    printf( "need %.1f seconds\n", time-$time1 ) if $NoOutput;
}

sub usage {
    print <<END_USAGE;
Description:
    With 6.15.0 ePages introduced the new gadgets.
    Therefore this script replaces the old with the new ones direct in all string-, localizedstringattributes values.
    Some gadgets does not exist in the 6.15.0 anymore. Those will marked as "outdated".

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password              ()
    -storename  name of store                       (Store)

flags:
    -help       show the command line options

Example:
    perl $0
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);
