#========================================================================================
# package      DE_EPAGES::ExternalContent::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to get all gadget information.
#========================================================================================
package DE_EPAGES::ExternalContent::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;
use JSON;

use strict;
use DE_EPAGES::Core::API::File qw ( GetFileContent fsplit RelativePath );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use Image::Magick;
use MIME::Base64;

#========================================================================================
# function     JSONGetGadgetInformation
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONGetGadgetInformation( $Servlet );
#----------------------------------------------------------------------------------------
# description  JSON ViewAction which returns all gadget information for the shop.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------
# return       $ReturnValue | Gadget information for the current shop | string
#========================================================================================
sub JSONGetGadgetInformation {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite();

    my $System = LoadRootObject();
    my @GadgetGroupsInfo;

    my $HasRss = $Shop->featureMaxValue('Rss');
    if($Shop->featureMaxValue('Gadget')) {
        my $aGadgetGroups = $System->child('ContentElementGroups')->childrenByClass('GadgetGroup');
        foreach my $GadgetGroup (@$aGadgetGroups) {
            my $aGadgets = $GadgetGroup->children();
            my @GadgetsInfo = map { {'Alias' => $_->alias} } grep { ($HasRss && $_->alias eq 'RSS' || $_->alias ne 'RSS') } @$aGadgets;
            push @GadgetGroupsInfo, {
                'Alias' => $GadgetGroup->alias,
                'Gadgets' => \@GadgetsInfo
            };
        }
    } elsif ($HasRss) {
        push @GadgetGroupsInfo, {
            'Alias' => 'Information',
            'Gadgets' => {
                'Alias' => 'RSS'
            }
        };
    }

    return $self->writeContentOutput({
        'GadgetGroups' => \@GadgetGroupsInfo
    });
}

#========================================================================================
# function     JSONUploadContentSliderImage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONUploadContentSliderImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  JSON ChangeAction which saves a dataURL image string or image for contentslider.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------
# return       $ReturnValue | File name and filepath in MediaGallery | string
#========================================================================================
sub JSONUploadContentSliderImage {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite();
    # Get MediaGallery object.
    my $MediaGallery = $Shop->get('MediaGallery');
    # Get both possible params.
    my $DataURL = $Servlet->form->value('DataURL');
    my $Image = $Servlet->form->value('Path');
    # Split filepath in name and extension.
    my (undef, $Name, $Extension ) = fsplit($Image);

    # Check if the a correct dataURL was sent and save the raw content.
    my $ImageString;
    if (defined $DataURL && $DataURL =~ m/base64,(.*)/) {
        $ImageString = $1;
    }

    my $File;
    my $hResponseContent;
    # Check if the MediaGallery is installed.
    if(defined $MediaGallery) {
        my $FileContent;
        # Get correct filecontent (dataURL or file path).
        if( !defined $ImageString || $ImageString eq '' ) {
            $FileContent = ${GetFileContent($ENV{'EPAGES_SHARED'}.'/'.$Image)};
        } else {
            $FileContent = decode_base64($ImageString);
            $Extension = '.png';
        }
        # Set flags hash for the MediaGallery uploader.
        my $hFlags = {
             'forceUpload'         => 1, # Even if the file exists and the MD5Checksum is equal upload the file.
             'hiddenDirectory'     => 1, # It is a hidden folder, so allo ',' in the path/directory name.
             'increaseGallerySize' => 0, # Do not increase the gallery size (cause hidden folder).
             'createThumnail'      => 0, # Do not create a thumbnail for the image (no thumbnail needed).
        };
        # Upload file.
        $File = $MediaGallery->addUploadedFile(',ContentSlider', \$FileContent, $Name.$Extension, $hFlags);
        # Split new file in name and extension.
        my (undef, $NewName, $NewExtension) = fsplit($File);
        # Set json response content.
        $hResponseContent = {
            'filename' => $NewName.$NewExtension,
            'fullpath' => $MediaGallery->get('WebPath').'/'.$File,
        }
    }
    # Return response content.
    return $self->writeContentOutput(
        $hResponseContent
    );
}

#========================================================================================
# function     JSONDeleteContentSliderImage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONDeleteContentSliderImage( $Servlet );
#----------------------------------------------------------------------------------------
# description  JSON ChangeAction which deletes an contentslider image.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#----------------------------------------------------------------------------------------
# return       $ReturnValue | File name and filepath in MediaGallery | string
#========================================================================================
sub JSONDeleteContentSliderImage {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object->getSite();
    # Get MediaGallery object.
    my $MediaGallery = $Shop->get('MediaGallery');
    # Get image path to delete this image (e.g. '/Webroot/Store/Shops/DemoShop/MediaGallery/,ContentSlider/image.png').
    my $Image = $Servlet->form->value('Path');
    # Check if the MediaGallery is installed.
    if (defined $MediaGallery) {
        # Generate relative path to the MediaGallery folder (e.g. ',ContentSlider/image.png').
        my $MediaGalleryWebPath = $MediaGallery->get('WebPath');
        my $RelativeImagePath = RelativePath($MediaGalleryWebPath, $Image);
        # Delete image without recalculating the gallerysize and deleting the thumbnail (cause no thumbnail was generated).
        $MediaGallery->deleteFile($RelativeImagePath, { 'decreaseGallerySize' => 0, 'deleteThumnail' => 0 });
    }

    return $self->writeContentOutput({});
}


#========================================================================================
# function     JSONHandleGoogleMapsAPIKey
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONHandleGoogleMapsAPIKey( $Servlet );
#----------------------------------------------------------------------------------------
# description  JSON ChangeAction which saves or deletes API key
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================

sub JSONHandleGoogleMapsAPIKey {
    my $self = shift;
    my ($Servlet) = @_;

    my $APIKeyData = $Servlet->form->value('data');
    my $Operation = $Servlet->form->value('Operation');

    my $Shop = $Servlet->object();

    my $hResponseContent = {
            'success' => JSON::true
        };

    if ($Operation eq 'save') {
        my $JSONData = from_json($APIKeyData);
        my $JSONAPIKeyData = $JSONData->{'APIKeyData'};

        $Shop->set({'GoogleMapsAPIKey' => $JSONAPIKeyData });
    }

    elsif ($Operation eq 'delete') {
        $Shop->set({'GoogleMapsAPIKey' => ''});
    }

    else {
        die "no valid Operation '$Operation'\n";
    }
    return $self->writeContentOutput($hResponseContent);

}

1;
