#==============================================================================
# package      DE_EPAGES::FedEx::UI::ShippingTransactionFedEx
# state        private
#==============================================================================
package DE_EPAGES::FedEx::UI::ShippingTransactionFedEx;
use base DE_EPAGES::ExternalShipping::UI::ShippingTransaction;

use strict;

use DE_EPAGES::Core::API::Log qw (GetLog LogDebug);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::FedEx::API::Constants qw (
    %SHIPPING_LABEL_STATUS
);


#========================================================================================
# function     SendShippingOrderHelper
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SendShippingOrderHelper( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to send shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SendShippingOrderHelper {
    my $self = shift;
    my ($Servlet) = @_;
    my $Transaction = $Servlet->object;

    LogShippingFulfillment('FedEx','Transaction',$Transaction,'SendShippingOrder');

    my $variable = 'ShippingTransaction';
    LogDebug('This is the entry point', $variable);

    $self->ValidateShippingOrder($Servlet);
    return if ($Servlet->form->existsFormError());

    eval { $self->_doTransaction( $Servlet, $Transaction ); };

    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'NativeDie') {
            Warning($Error);
        } elsif ( $Error->message =~ /401 Authorization Required/ ) {
            $Servlet->form->addFormError({
                'Name'   => 'ExportShippingLabels',
                'Form'   => 'ExportShippingLabels',
                'Reason' => 'WebserviceAuthorizationFailed',
            });
        } else {
            GetLog->debug('Error: '.$Error->longMessage);
            $Servlet->form->mapFormError({
                'Form'  => 'ExportShippingLabels',
                'Name'  => 'ExportShippingLabels',
            }, $Error);
        }
        $Servlet->form->executeFormError;
    }
    $self->_transactionErrorHandling($Servlet);

    return;
}

#========================================================================================
# function     _doTransaction
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_doTransaction( $Servlet,$Transaction );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Transaction | a Transaction object| object
#----------------------------------------------------------------------------------------
# description  This private Function encapsulate the transaction process. Additional is there
#               a Error handling if the Function is called without Error handling.
#========================================================================================
sub _doTransaction {
    my $self = shift;
    my ( $Servlet, $Transaction ) = @_;
    my $ShippingFulfillmentMethod = $Transaction->get('ShippingFulfillmentMethod');

    Error(
        'NO_SHIPPING_FULFILLMENT_METHOD',{
            'name'      => 'Order without a ShippingFulfillmentMethod',
            'Parameter' => $Transaction->alias,
            'Message' => $Transaction->alias,
        }
    ) unless defined $ShippingFulfillmentMethod;

    $Transaction->prepareShippingOrder();
    $ShippingFulfillmentMethod->generateShippingLabels($Transaction);
    $Transaction->issueShippingOrder();

    return;
}

#========================================================================================
# function     _transactionErrorHandling
# state        private
#----------------------------------------------------------------------------------------
# syntax       _transactionErrorHandling( $Servlet );
#----------------------------------------------------------------------------------------
# description  depending on the the TransStatus of the transaction it will choose the correct
#               viewaction and, if needed, Error handling
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _transactionErrorHandling {
    my $self = shift;
    my ($Servlet) = @_;

    my $Transaction = $Servlet->object;
    my $TransStatus = $Transaction->get('TransStatus');

    if ($TransStatus eq $SHIPPING_LABEL_STATUS{'ALL_OK'}) {
       $self->_changeViewActionToTransaction($Servlet);
    }
    elsif ($TransStatus eq $SHIPPING_LABEL_STATUS{'SOME_OK'}) {
        $self->_changeViewActionToTransactionAndShowErrorMessage($Servlet);
    }
    elsif ($TransStatus eq $SHIPPING_LABEL_STATUS{'ALL_KO'}) {
       $self->_showErrorMessage($Servlet);
    }
    return undef;
}

#========================================================================================
# function     _changeViewActionToTransaction
# state        private
#----------------------------------------------------------------------------------------
# syntax       _changeViewActionToTransaction( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _changeViewActionToTransaction {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'ViewAction',  $self->_getViewAction($Servlet, 'MBO-ViewGeneral'));

    return undef;
}

#========================================================================================
# function     _showErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _showErrorMessage( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#               change the objectID
#               add error message
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _showErrorMessage {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars( 'ViewAction', $self->_getViewAction($Servlet, 'MBO-ViewFedExShippingOrder') );
    $Servlet->vars( 'ObjectID',$Servlet->object()->getSite()->id() );
    $Servlet->vars( 'BatchActionError', 'ALL_EXPORTED_FAILED' );
    return undef;
}

#========================================================================================
# function     _changeViewActionToTransactionAndShowErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _changeViewActionToTransactionAndShowErrorMessage( $Servlet );
#----------------------------------------------------------------------------------------
# description  change the viewaction
#               change the objectID
#               add error message
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _changeViewActionToTransactionAndShowErrorMessage {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShopObjectID = $Servlet->object()->getSite()->id();
    my $ViewAction = $self->_getViewAction($Servlet, 'MBO-ViewFedExShippingOrder');
    $Servlet->vars( 'BatchActionError', 'SOME_EXPORTED_FAILED' );
    $Servlet->vars( 'LinkToShippingOrders', '?ViewAction='.$ViewAction.'&ObjectID='.$ShopObjectID );
    $self->_changeViewActionToTransaction($Servlet);
    return undef;
}

sub _getViewAction {
    my $self = shift;
    my ($Servlet, $ViewActionAlias) = @_;

    my $CurrentViewAction = $Servlet->params('ViewAction');
    if ($CurrentViewAction =~ m/^Unity.*/) {
        $ViewActionAlias = 'Unity' . $ViewActionAlias;
    }

    return $ViewActionAlias;
}

1;
