#========================================================================================
# package      DE_EPAGES::Forum::API::Attributes::BlogPost_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Attribute interface for BlogPost.
#========================================================================================
package DE_EPAGES::Forum::API::Attributes::BlogPost_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::Html qw ( CloseOpenHTMLTags RemoveHtmlTags );
use DE_EPAGES::Core::API::String qw ( ConvertNewlines );
use DE_EPAGES::Forum::DAL::Forum qw( DAL_CountRepliesInPost
                                     DAL_CountVisibleRepliesInPost );
use DE_EPAGES::SEO::API::SEO qw ( GetSEOCheckValues CalculateSEOQuality GetSEOQualityClass );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use HTML::Entities qw( encode_entities );

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->( $Object, $aNames );
# example      # never call getAttributes directly!
#               $Value = $Object->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Returns attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;
    my $hInfo = {};
    foreach my $Name (@$aNames) {
        if( $Name eq 'BlogPostExcerpt' ) {
            my $Value = $Object->get('Text');
            if($Value && $Value =~ /(.*)<!--\s+more\s+-->/xsi) {  # find more html comment
                if($1) {
                    $hInfo->{$Name} = CloseOpenHTMLTags($1);
                } else {
                    # empty if no
                    $hInfo->{$Name} = undef;
                }
            }
        }
        elsif( $Name eq 'BlogPostMainText' ) {
            my $Value = $Object->get('Text');
            if($Value && $Value =~ /<!--\s+more\s+-->(.*)/xsi) {   # find more html comment
                if($1) {
                    $hInfo->{$Name} = CloseOpenHTMLTags($1);
                } else {
                    $hInfo->{$Name} = $Value;
                }
            } else {
                $hInfo->{$Name} = $Value;
            }
        }
        elsif( $Name eq 'Message' ) {
            # Message is used to fetch content in shared templates for forum,blog and gb
            $hInfo->{$Name} = $Object->get('Text');
        }
        elsif( $Name eq 'Subject' ) {
            $hInfo->{$Name} = $Object->get('NameOrAlias');
        }
        elsif( $Name eq 'DisplayTitle' ) {
            #IF(#Title)
                #Title
            #ELSE
                #NameOrAlias[nohtml] - #Parent.NameOrAlias[nohtml] - #Shop.Name[nohtml]
            #ENDIF
            my $TitleAttribute = $Object->get('Title', $LanguageID);
            if($TitleAttribute) {
                $hInfo->{$Name} = $TitleAttribute;
            } else {
                my $ObjectName = $Object->get('Name', $LanguageID);
                my $BlogName = $Object->parent->get('Name', $LanguageID);
                my $ShopName = $Object->getSite()->get('Name', $LanguageID);
                $hInfo->{$Name} =   RemoveHtmlTags($ObjectName) . ' - ' .
                                    RemoveHtmlTags($BlogName) . ' - ' .
                                    RemoveHtmlTags($ShopName);
            }
        }
        elsif( $Name eq 'DisplayDescription' ) {
            my $Excerpt = $Object->get('BlogPostExcerpt', $LanguageID);
            if($Excerpt) {
                # == [nohtml,nobreak,html] formatters
                $hInfo->{$Name} = encode_entities( ConvertNewlines( RemoveHtmlTags($Excerpt) ), '<>&"' );
            } else {
                my $Text = $Object->get('Text', $LanguageID);
                if($Text) {
                    $hInfo->{$Name} = encode_entities( ConvertNewlines( RemoveHtmlTags($Text) ), '<>&"' );
                }
            }
        }
        elsif( $Name eq 'SeoCheckRatings' ) {
            $hInfo->{$Name} = {
                'Title'             => 0.35,
                'Url'               => 0.15,
                'AdvDescription'    => 0.4,
                'CategoryImage'     => 0.1,
            };
        }
        elsif( $Name eq 'CalculatedSEOCheckValues' ) {
            $hInfo->{$Name} = _getBlogSEOValues($Object, $LanguageID);
        }
        elsif( $Name eq 'VisibleRepliesCount' ) {
            $hInfo->{$Name} = DAL_CountVisibleRepliesInPost( $Object->id );
        }
        elsif( $Name eq 'RepliesCount' ) {
            $hInfo->{$Name} = DAL_CountRepliesInPost( $Object->id );
        }
    }
    return $hInfo;
}

#========================================================================================
# function     setAttributes
# state        private
# syntax       $Value = $Object->set($AttributeName, $Value);
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hAttributes, $LanguageID) = @_;

    foreach my $Name (keys %$hAttributes) {
        if ($Name eq 'Subject') {
            $Object->set( { 'Name' => $hAttributes->{$Name} } );
        }
    }

}

sub _getBlogSEOValues {
    my ($Object, $LanguageID) = @_;

    my $Site = $Object->getSite;
    my $ShopDomain = $Site->get('DomainName');
    my $ShopWebUrl = $Site->get('WebUrl');

    my $hValues;

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => $Object->get('SEOQuality') || 0,
        SiteUrl => $ShopWebUrl,
        Domain => $ShopDomain,
    };

    TriggerHook('SEOCheckTests', {
        'Object'        => $Object,
        'Shop'          => $Site,
        'Values'        => $hGeneralValues,
    });

    my $hLanguageCheckValues = GetSEOCheckValues($Site, $Object, $LanguageID);
    $hValues->{$LanguageID} = {
        %$hGeneralValues,
        %$hLanguageCheckValues,
    };

    # use only specified tests for blog post
    my $hCurVals = $hValues->{$LanguageID};
    $hValues->{$LanguageID} = {
        'CategoryImage' => $hCurVals->{CategoryImage},   # 0.1
        'Title' => $hCurVals->{Title},                   # 0.35
        'Url' => $hCurVals->{Url},                       # 0.15
    };

    # reuse AdvDescription for Text, rating 0.4
    my $AdvDescRating = _getAdvDescriptionRating($hCurVals->{'AdvDescription'}->{'Value'});
    $hValues->{$LanguageID}->{'AdvDescription'} = {
        'Value' => $hCurVals->{'AdvDescription'}->{'Value'},
        'Rating' => $AdvDescRating,
        'RatingClass' => qw ( green yellow orange orange red )[$AdvDescRating-1],
    };
    foreach my $Key (qw (Domain FormerSEOQuality Name NewSEOQuality SiteUrl)) {
        $hValues->{$LanguageID}->{$Key} = $hCurVals->{$Key};
    }

    my $LanguageCheckQuality = CalculateSEOQuality($hValues->{$LanguageID}, $Object);
    $hValues->{$LanguageID}->{'NewSEOQuality'} = $LanguageCheckQuality;

    $hValues->{'NewSEOQuality'} = $LanguageCheckQuality;
    $hValues->{'NewSEOQualityClass'} = GetSEOQualityClass($LanguageCheckQuality);

    return $hValues;
}

sub _getAdvDescriptionRating {
    my ($AdvDescription) = @_;
    return 5 if !$AdvDescription || $AdvDescription eq '';

    while($AdvDescription =~ /(<img\s.*?>)/isgx) {
        my ($Match) = $1 =~ /(?:alt|title)="(.*?)"/ix;
        return 3 unless defined $Match and $Match =~ /\S/;
    }

    return 2 if $AdvDescription !~ /<(?:strong|b|i|em)>/isg &&
                $AdvDescription !~ /<(?:ul|ol)>/isg;

    return 1;
}

1;
