#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the GoogleSmartShopping view setup
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String Number );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::More;

use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( ERROR_ACCEPT_TERMS ERROR_FORM_SUBMIT ERROR_TOO_MANY_SUBACCOUNTS ERROR_TOKEN_EXPIRED_OR_REVOKED );

use HTTP::Status qw ( HTTP_BAD_REQUEST HTTP_UNAUTHORIZED HTTP_FORBIDDEN );
use JSON::MaybeXS qw ( encode_json );
use HTTP::Response;

use constant MERCHANT_EMAIL => 'example@email.com';
use constant CONNECT_OAUTH_URL => 'https://www.example.com';
use constant GOOGLE_OAUTH_ERROR => 'access_denied';
use constant GOOGLE_SINGLE_QUOTE_ERROR => 'quo\'te';
use constant GOOGLE_OAUTH_TOKENS_EXCHANGE_ERROR => 'INTERNAL_SERVER_ERROR';
use constant SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    code   => [ '4/0AX4XfWgoOUV99TEXPVNpq7YxcwyTNMWhvSNRWBpjj17qUeaEv6sSebt9OXtrHhZyhIrLTA' ],
    scope  => [ 'email https://www.googleapis.com/auth/content https://www.googleapis.com/auth/siteverification https://www.googleapis.com/auth/adwords openid https://www.googleapis.com/auth/userinfo.email' ],
    prompt => [ 'consent' ],
};
use constant SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE_BUT_SCOPES_MISSING => {
    code   => [ '4/0AX4XfWgoOUV99TEXPVNpq7YxcwyTNMWhvSNRWBpjj17qUeaEv6sSebt9OXtrHhZyhIrLTA' ],
    scope  => [ 'email https://www.googleapis.com/auth/siteverification openid https://www.googleapis.com/auth/userinfo.email' ],
    prompt => [ 'consent' ],
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE => {
    error  => [ GOOGLE_OAUTH_ERROR ]
};
use constant GOOGLE_OAUTH_CALLBACK_UNKNOWN_PARAMETER => {
    unknown => 'asdf'
};
use constant GOOGLE_OAUTH_CALLBACK_ERROR_SINGLE_QUOTE => {
    error  => [ GOOGLE_SINGLE_QUOTE_ERROR ]
};
use constant SUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    access_token  => 'some-google-access-token',
    refresh_token => 'some-google-refresh-token',
    id_token      => 'some-long-encrypted-key',
    expires_in    => 3599,
    token_type    => 'Bearer',
    scope         => 'space-separated-list-of-scopes-string'
};
use constant UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE => {
    "error" => "invalid_grant",
    "error_description" => "Malformed auth code."
};
use constant UNSUCCESSFUL_REFRESH_TOKEN_REVOKED_ERROR_RESPONSE => {
    "error" => "invalid_grant",
    "error_description" => "Token has been expired or revoked."
};
use constant TOKENS_EXCHANGE_SINGLE_QUOTE_ERROR_RESPONSE => {
    error => [ GOOGLE_SINGLE_QUOTE_ERROR ]
};
use constant EXAMPLE_EMAIL => 'google-account-email';
use constant SUCCESSFUL_USER_INFO_RESPONSE => {
    email => EXAMPLE_EMAIL
};
use constant SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE => {
    name => 'google-merchant-center-account-name',
    id   => 123456789
};
use constant SUCCESSFUL_GOOGLE_ADS_ACCOUNTS_RESPONSE => {
    name => 'google-ads-account-name',
    id   => 555778899
};
use constant FAILED_USER_INFO_RESPONSE => { };
use constant FAILED_USER_INFO_RESPONSE_WITH_EMAIL => {
    is_token_expired => 1,
    email => EXAMPLE_EMAIL,
};
use constant ACCESS_DENIED_CALLBACK_ERROR => "access_denied";
use constant UNKNOWN_ERROR => "unknown_error";
use constant MISSING_API_SCOPES_ERROR => "missing_api_scopes";

use constant REVOKE_TOKEN_RESPONSE_ERROR => {
    'error' => 'invalid_token',
    'error_description' => 'poggers'
};

use constant ACCOUNT_ID => 89879879;
use constant MULTI_CLIENT_ACCOUNT_ID => 398123456;
use constant MERCHANT_CENTER_ID => 458599058;
use constant GOOGLE_ADS_ID => 555778899;

use constant MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_HOME_PAGE => {
    "id" => "home_page_issue",
    "documentation"=> "https://support.google.com/merchants/answer/176793",
    "severity"=> "error",
    "title"=> "Website not claimed"
};

use constant MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_QUALITY_LOW => {
    "documentation"=> "https://support.google.com/merchants/answer/9849758",
    "id" => "merchant_quality_low",
    "severity"=> "error",
    "title"=> "Account isn't eligible for enhanced free listings"
};

use constant MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_MISSING_AD_WORDS_LINK => {
    "documentation"=> "https://support.google.com/merchants/answer/6159060",
    "id" => "missing_ad_words_link",
    "severity"=> "error",
    "title"=> "No Google Ads account linked"
};

use constant MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE => {
    'id' => 'pending_address_and_phone',
    'documentation' => 'https://support.google.com/merchants/answer/10139041',
    'severity' => 'critical',
    'title' => 'You need to enter your business address and phone number',
    'detail' => 'Go to the business information page to enter your business address and phone number',
};

use constant MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_UNKNOWN => {
    "documentation"=> "https://support.google.com/merchants/answer/123456789",
    "id" => "unknown_issue",
    "severity"=> "error",
    "title"=> "Something is wrong but we don't know what is it"
};

use constant MERCHANT_CENTER_ACCOUNT_STATUSES_UNKNOWN_ISSUE_RESPONSE => {
    "accountId"=> "414922848",
    "accountLevelIssues"=> [
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_UNKNOWN
    ],
    "kind"=> "content#accountStatus",
    "websiteClaimed"=> "true",
    "accountManagement"=> "manual"
};
use constant UNSUCCESSFUL_MERCHANT_CENTER_ACCOUNT_STATUSES_RESPONSE => {
    "accountId"=> "414922848",
    "accountLevelIssues"=> [
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_QUALITY_LOW,
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE,
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_MISSING_AD_WORDS_LINK,
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_HOME_PAGE
    ],
    "kind"=> "content#accountStatus",
    "websiteClaimed"=> "false",
    "accountManagement"=> "manual"
};

use constant EXPECTED_TOO_MANY_SUBACCOUNTS_RESPONSE => {
    "errorId" => "google-api-error",
    "message" => "Maximum number of subaccounts reached",
    "details" => {
        "code" => 403,
        "errors" => [
            {
                "domain" => "content.ContentErrorDomain",
                "message" => "Maximum number of subaccounts reached",
                "reason" => "quota/too_many_subaccounts"
            }
        ],
        "message" => "Maximum number of subaccounts reached"
    }
};

use constant EXPECTED_ACCESS_DENIED_RESPONSE => {
    "errorId" => "google-api-error",
    "message" => "access denied",
    "details" => {
        "code" => 401,
        "errors" => [
            {
                "domain" => "global",
                "message" => "access denied",
                "reason" => "auth/account_access_denied"
            }
        ],
        "message" => "access denied"
    }
};
use constant ADS_ACCOUNT_ID => 899423423432879;
use constant GOOGLE_SITE_VERIFICATION_TOKEN => 'google96d1fc6bdb3dfb48.html';
use constant GOOGLE_SITE_VERIFICATION_TOKEN_FAIL => 'googlea1b2f3.html';
use constant GOOGLE_SITE_VERIFICATION_RESPONSE_SUCCESS => 'google-site-verification: '.GOOGLE_SITE_VERIFICATION_TOKEN;

#ProductFeedTest const
use constant MERCHANT_CENTER_ID_PRODUCT_FEED_TEST => 503220055;
use constant GOOGLE_PORTAL_ALIAS => 'SomeAlias_de_DE';
use constant DEFAULT_SHIPPING_METHOD_ID => '22528';
use constant DEFAULT_SHIPPING_METHOD_ALIAS => 'Express';
use constant PORTAL_CONFIG_ID => 78910;
use constant PORTAL_SITE_LANGUAGE_CODE => 'ger';

sub test {
    my $self = shift;

    # $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig();
    # $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken();
    # $self->test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken();
    # $self->test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail();
    # $self->test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsFirstTime();
    # $self->test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsTokenExpiredOrRevoked();
    # $self->test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingApproved();
    # $self->test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingUnapproved();
    # $self->test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingError();
    # $self->test_DisconnectConnectedAccount_RevokeToken_NoConfig();
    # $self->test_DisconnectConnectedAccount_RevokeToken_Successful();
    # $self->test_DisconnectConnectedAccount_RevokeToken_Error();
    # $self->test_DisconnectMerchantAccount_Successful_SubAccount();
    # $self->test_DisconnectMerchantAccount_Successful_Not_SubAccount();
    # $self->test_DisconnectMerchantAccount_Failed_SubAccount_delete();
    # $self->test_DisconnectAdsAccount_Successful_No_Campaign();
    # $self->test_DisconnectAdsAccount_Successful_With_Campaign();
    # $self->test_DisconnectAdsAccount_Failed_deleteAdsAccount();
    # $self->test_DisconnectAdsAccount_Failed_deleteAdsCampaign();
    # $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange();
    # $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_But_ScopesMissing();
    # $self->test_GoogleOAuthCallback_On_OnboardingCallbackError();
    # $self->test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange();
    # $self->test_GoogleOAuthCallback_On_OnboardingCallbackUnknownParameter();
    # $self->test_ConnectedAccount_MerchantCenterRequirements_Not_Met();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts_NoAccounts();
    # $self->test_CreateMerchantCenterAccount_Success();
    # $self->test_CreateMerchantCenterAccount_Fail_Form();
    # $self->test_CreateMerchantCenterAccount_Fail_Api();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Subaccount_Success();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Standalone_Merchant_Account_Success();
    # $self->test_CreateGoogleAdsAccount_Success();
    # $self->test_CreateGoogleAdsAccount_Fail_Form_Requirements_Not_Met();
    # $self->test_CreateGoogleAdsAccount_Fail_Form_No_MerchantCenter();
    # $self->test_CreateGoogleAdsAccount_Fail_Form_Already_Has_Ads_Account();
    # $self->test_CreateGoogleAdsAccount_Fail_Api();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Unknown_Issue();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Success_When_Ads_Account_IS_Connected();
    # $self->test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Success_When_No_Ads_Account();
    # $self->test_UnitySFViewGoogleSmartShoppingHTML_Success();
    # $self->test_UnitySFViewGoogleSmartShoppingHTML_Not_Found();
    # $self->test_ViewGoogleSmartShopping_On_MerchantCenterConnected_CreatePortalConfig();
    # $self->test_ViewGoogleSmartShopping_On_MerchantCenterConnected_ExistingPortalConfig();
    # $self->test_LinkMerchantCenterAndAdsAccount_Success();
    # $self->test_LinkMerchantCenterAndAdsAccount_Fail_AccountLinking();
    # $self->test_LinkMerchantCenterAndAdsAccount_Fail_LabelCreation();

    #PRODUCTFEED
    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics();
    $self->test_UnityViewGoogleSmartShoppingProductFeed_Show_MerchantCenterProductsTable();

    done_testing();
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoGoogleSmartShoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->ViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when there is no GoogleSmartShoppingConfig object"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoAccessToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => undef,
        RefreshToken => undef,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->ViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig access token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_OnboardingStart_NoRefreshToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => undef,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $SUT = $self->_createSUT();

    $SUT->ViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleOAuthConnect'),
        CONNECT_OAUTH_URL,
        "$SubTestName - GoogleSmartShopping setup view has OAuth URL when GoogleSmartShoppingConfig refresh token is empty"
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
}

sub test_ViewGoogleSmartShopping_On_GettingConnectedAccountEmail {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 0,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    is(
        $Servlet->vars('MerchantCenterAccounts'),
        SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        "$SubTestName - Verify that MerchantCenterAccounts correctly returned."
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}


sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccounts_NoAccounts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 0,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => []
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );
    ok(
        WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    is_deeply(
        $Servlet->vars('MerchantCenterAccounts'),
        [],
        "$SubTestName - Verify that MerchantCenterAccounts returns empty response when no accounts found."
    );
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsFirstTime {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => FAILED_USER_INFO_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that lookup of Google account email is attempted'
    );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAccountEmailFailsTokenExpiredOrRevoked {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 0,
        AccountEmail => EXAMPLE_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $MockedError = $self->_createError(HTTP_UNAUTHORIZED, UNSUCCESSFUL_REFRESH_TOKEN_REVOKED_ERROR_RESPONSE);
    my $ApiService = $self->_createGoogleApiService({
        getOAuthEmail => { 'throw_error' => 1 }
    });
    my $SUT = $self->_createSUT($ApiService, $MockedError);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that lookup of Google account email is attempted'
    );
    ok(
        !WasCalled($ApiService, 'getMerchantCenterAccounts'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was not called when AccountInfo fails'
    );
    ok( WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was built' );
    is(
        $Servlet->vars('TokenExpiredOrRevoked'),
        1,
        $SubTestName . ' - Ensure servlet flag is set for expired or revoked RefreshToken.'
    );
    is(
        $Servlet->vars('ConnectedGoogleAccountEmail'),
        EXAMPLE_EMAIL,
        $SubTestName . ' - Check that saved email was retrieved.'
    );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingApproved {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        AccountEmail => EXAMPLE_EMAIL,
        GoogleAdsAccountID => ADS_ACCOUNT_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
    });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($AdsApiService, 'isAdsAccountBillingApproved'),
        $SubTestName . ' - Verify that isAdsAccountBillingApproved was called'
    );

    is(
        $Servlet->vars('IsAdsAccountBillingApproved'),
        1,
        $SubTestName . ' - Check that correct IsAdsAccountBillingApproved is sent to the template.'
    );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingUnapproved {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        AccountEmail => EXAMPLE_EMAIL,
        GoogleAdsAccountID => ADS_ACCOUNT_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
    });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 0
        }
    });
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($AdsApiService, 'isAdsAccountBillingApproved'),
        $SubTestName . ' - Verify that isAdsAccountBillingApproved was called'
    );

    is(
        $Servlet->vars('IsAdsAccountBillingApproved'),
        0,
        $SubTestName . ' - Check that correct IsAdsAccountBillingApproved is sent to the template.'
    );
}

sub test_ViewGoogleSmartShopping_On_ConnectedAdsAccount_BillingError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        AccountEmail => EXAMPLE_EMAIL,
        GoogleAdsAccountID => ADS_ACCOUNT_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
    });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($AdsApiService, 'isAdsAccountBillingApproved'),
        $SubTestName . ' - Verify that isAdsAccountBillingApproved was called'
    );

    is(
        $Servlet->vars('IsAdsAccountBillingApproved'),
        0,
        $SubTestName . ' - Check that correct IsAdsAccountBillingApproved is sent to the template.'
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_SuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService();
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);

    ok(WasCalled($ApiService, 'getAndSaveOAuthTokens'), "$SubTestName - Ensure getAndSaveOAuthTokens was called");
    is(
        $Servlet->vars('GoogleOAuthCallbackError'),
        undef,
        "$SubTestName - Ensure GoogleOAuthCallbackError is undefined in servlet vars"
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_But_ScopesMissing {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE_BUT_SCOPES_MISSING;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService();
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);

    ok(!WasCalled($ApiService, 'getAndSaveOAuthTokens'), "$SubTestName - Ensure getAndSaveOAuthTokens was NOT called");
    is(
        $Servlet->vars('GoogleOAuthCallbackError'),
        MISSING_API_SCOPES_ERROR,
        "$SubTestName - Ensure GoogleOAuthCallbackError was set to 'missing_api_scopes' in the Servlet"
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackError {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = UNSUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService();
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);

    ok(!WasCalled($ApiService, 'getAndSaveOAuthTokens'), "$SubTestName - Ensure getAndSaveOAuthTokens was NOT called");
    is(
        $Servlet->vars('GoogleOAuthCallbackError'),
        ACCESS_DENIED_CALLBACK_ERROR,
        "$SubTestName - Ensure GoogleOAuthCallbackError was set to 'access_denied' in the Servlet"
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackSuccess_And_UnsuccessfulTokensExchange {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = SUCCESSFUL_GOOGLE_OAUTH_CALLBACK_RESPONSE;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        getAndSaveOAuthTokens => {'throw_error' => 1}
    });
    my $MockedError = $self->_createError(HTTP_BAD_REQUEST, UNSUCCESSFUL_GOOGLE_OAUTH_TOKENS_EXCHANGE_RESPONSE);
    my $SUT = $self->_createSUT($ApiService, $MockedError);

    $SUT->GoogleOAuthCallback($Servlet);

    ok(WasCalled($ApiService, 'getAndSaveOAuthTokens'), "$SubTestName - Ensure getAndSaveOAuthTokens was called");
    is(
        $Servlet->vars('GoogleOAuthCallbackError'),
        UNKNOWN_ERROR,
        "$SubTestName - Ensure GoogleOAuthCallbackError was set to 'unknown_error' in the Servlet"
    );
}

sub test_GoogleOAuthCallback_On_OnboardingCallbackUnknownParameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = GOOGLE_OAUTH_CALLBACK_UNKNOWN_PARAMETER;
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService();
    my $SUT = $self->_createSUT($ApiService);

    $SUT->GoogleOAuthCallback($Servlet);

    ok(!WasCalled($ApiService, 'getAndSaveOAuthTokens'), "$SubTestName - Ensure getAndSaveOAuthTokens was NOT called");
    is(
        $Servlet->vars('GoogleOAuthCallbackError'),
        UNKNOWN_ERROR,
        "$SubTestName - Ensure GoogleOAuthCallbackError was set to 'unknown_error' in the Servlet"
    );
}


sub test_DisconnectConnectedAccount_RevokeToken_NoConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = undef;
    my $AlreadyExistingConfig = 0;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService(undef);

    # given: 'On-boarding NOT done, GoogleSmartShoppingConfig not defined '
    my $SUT = $self->_createSUT($ApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: 'Make request to google API to revoke the token, delete gss config from shop'
    ok(!WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was NOT called');
    ok(!WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was NOT called');
    ok(!WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was NOT called');

}

sub test_DisconnectConnectedAccount_RevokeToken_Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountID => ACCOUNT_ID,
        IsMerchantCenterSubAccount => 1,
        IsAdsAccountSubAccount => 1,
        GoogleAdsAccountID => GOOGLE_ADS_ID,
        AdsCampaignId => 743242423
    });
    my $AlreadyExistingConfig = 1;

    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({ deleteMerchantCenterSubAccount => {} });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 0
        },
    });
    my $GooglePortalService = $self->_createGooglePortalService({
        getSmartShoppingPortalConfig => {
            thenReturn => $self->_createPortalConfig()
        },
    });

    # setup
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService, undef, $GooglePortalService);

    # given: 'On-boarding done, connected account email successful retrieved '
    $SUT->ViewGoogleSmartShopping($Servlet);
    ok(WasCalled($ApiService, 'getOAuthEmail'), $SubTestName . ' - Verify getOAuthEmail was called');
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: 'Make request to google API to revoke the token, delete gss config from shop'
    ok(WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was called');
    ok(WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was called');
    ok(WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify Ads service deleteAdsCampaign was called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify Ads service unlinkGoogleAdsAccount was called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'delete'),
        $SubTestName . ' - Verify $GoogleSmartShoppingConfig->delete was called');
    ok(WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was called');
}

sub test_DisconnectConnectedAccount_RevokeToken_Error {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountID => ACCOUNT_ID,
        IsMerchantCenterSubAccount => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService( { 'revokeAccessToken' => { 'throw_error' => 1 } });

    # given: 'On-boarding done, connected account email successful retrieved '
    my $SUT = $self->_createSUT($ApiService, $self->_createError(HTTP_BAD_REQUEST, REVOKE_TOKEN_RESPONSE_ERROR));
    $SUT->ViewGoogleSmartShopping($Servlet);
    ok(WasCalled($ApiService, 'getOAuthEmail'), $SubTestName . ' - Verify getOAuthEmail was called');
    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );

    # when: 'Merchant clicks Disconnect button', service call returns an error
    $SUT->DisconnectPersonalAccount($Servlet);

    # then: Delete config, call revokeAccessToken, executeFormError should be called
    ok(WasCalled($GoogleSmartShoppingConfig, 'delete'),
        $SubTestName . ' - Verify GoogleSmartShoppingConfig->delete was called');
    ok(WasCalled($ApiService, 'revokeAccessToken'),
        $SubTestName . ' - Verify service revokeAccessToken was called');
    ok(WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was called');
    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");
    ok(WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was called');
}

sub test_DisconnectMerchantAccount_Successful_SubAccount {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountID => ACCOUNT_ID,
        IsMerchantCenterSubAccount => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({  });

    # setup
    my $SUT = $self->_createSUT($ApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectMerchantCenterAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteMerchantCenterAccount'),
        $SubTestName . ' - Verify deleteMerchantCenterAccount was called');
    ok(WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was called');
}

sub test_DisconnectMerchantAccount_Successful_Not_SubAccount {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountID => ACCOUNT_ID,
        IsMerchantCenterSubAccount => 0,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({  });

    # setup
    my $SUT = $self->_createSUT($ApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectMerchantCenterAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(!WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was NOT called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteMerchantCenterAccount'),
        $SubTestName . ' - Verify deleteMerchantCenterAccount was called');
    ok(WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was called');
}

sub test_DisconnectMerchantAccount_Failed_SubAccount_delete {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountID => ACCOUNT_ID,
        IsMerchantCenterSubAccount => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $ApiService = $self->_createGoogleApiService({
        'deleteMerchantCenterSubAccount' => {
            'throw_error' => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT($ApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectMerchantCenterAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(WasCalled($ApiService, 'deleteMerchantCenterSubAccount'),
        $SubTestName . ' - Verify service deleteMerchantCenterSubAccount was NOT called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteMerchantCenterAccount'),
        $SubTestName . ' - Verify deleteMerchantCenterAccount was called');
    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");
    ok(WasCalled($SUT, '_deleteGooglePortalConfig'),
        $SubTestName . ' - Verify service _deleteGooglePortalConfig was called');
}

sub test_DisconnectAdsAccount_Successful_No_Campaign {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        GoogleAdsAccountID => ACCOUNT_ID,
        IsAdsAccountSubAccount => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectGoogleAdsAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(!WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify service deleteAdsCampaign was NOT called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify service unlinkGoogleAdsAccount was called');

    ok(!WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify deleteAdsCampaign was NOT called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteAdsAccount'),
        $SubTestName . ' - Verify deleteAdsAccount was called');
}

sub test_DisconnectAdsAccount_Successful_With_Campaign {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        GoogleAdsAccountID => ACCOUNT_ID,
        IsAdsAccountSubAccount => 1,
        AdsCampaignId => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectGoogleAdsAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify service deleteAdsCampaign was called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify service unlinkGoogleAdsAccount was called');

    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify deleteAdsCampaign was called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteAdsAccount'),
        $SubTestName . ' - Verify deleteAdsAccount was called');
}

sub test_DisconnectAdsAccount_Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        GoogleAdsAccountID => ACCOUNT_ID,
        IsAdsAccountSubAccount => 0,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectGoogleAdsAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(!WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify service deleteAdsCampaign was NOT called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify service unlinkGoogleAdsAccount was called');

    ok(!WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify deleteAdsCampaign was NOT called');
    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteAdsAccount'),
        $SubTestName . ' - Verify deleteAdsAccount was called');
}

sub test_DisconnectAdsAccount_Failed_deleteAdsAccount {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        GoogleAdsAccountID => ACCOUNT_ID,
        IsAdsAccountSubAccount => 0,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        deleteAdsAccount => {
            throw_error => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectGoogleAdsAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(!WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify service deleteAdsCampaign was NOT called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify service unlinkGoogleAdsAccount was called');

    ok(!WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify deleteAdsCampaign was NOT called');
    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");
}

sub test_DisconnectAdsAccount_Failed_deleteAdsCampaign {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        GoogleAdsAccountID => ACCOUNT_ID,
        IsAdsAccountSubAccount => 0,
        AdsCampaignId => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        deleteAdsCampaign => {
            throw_error => 1
        }
    });

    # setup
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    # when: 'The Merchant clicks disconnect button'
    $SUT->DisconnectGoogleAdsAccount($Servlet);

    # then: 'Make request to google API to delete the subAccount, delete gss config from shop'
    ok(WasCalled($AdsApiService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify service deleteAdsCampaign was called');
    ok(WasCalled($AdsApiService, 'unlinkGoogleAdsAccount'),
        $SubTestName . ' - Verify service unlinkGoogleAdsAccount was called');

    ok(WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify deleteAdsCampaign was called');
    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");
}

sub test_ConnectedAccount_MerchantCenterRequirements_Not_Met {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        MerchantCenterAccountName => 'MerchantCenterAccountName',
        MerchantCenterAccountID => ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'MerchantCenterAccountName'},
        "MerchantCenterAccountName",
        "$SubTestName - MerchantCenterAccountName is assigned to the Servlet"
    );

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'MerchantCenterAccountID'},
        ACCOUNT_ID,
        "$SubTestName - MerchantCenterAccountID is assigned to the Servlet"
    );

    is(
        $Servlet->vars('GoogleSmartShoppingConfig')->{'ChecklistBillingTOC'},
        0,
        "$SubTestName - Google Merchant Center billing terms and conditions requirement is not met"
    );
    is(
        $Servlet->vars('GoogleMerchantCenterRequirementsMet'),
        0,
        "$SubTestName - Google Merchant Center requirements not met warning is shown"
    );
}

sub test_CreateMerchantCenterAccount_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        HasAdultContent => [0],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID
        }
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->CreateMerchantCenterAccount($Servlet);

    is_deeply(
        $Servlet->vars('ResponseContent'),
        {id => ACCOUNT_ID},
        "$SubTestName - MerchantCenter SubAccount is successfully created"
    );
}

sub test_CreateMerchantCenterAccount_Fail_Form {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        HasAdultContent => [""],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID
        }
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->CreateMerchantCenterAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

    is_deeply(
        $aExecuteFormErrorParams->[0],
        {
            'Name'   => 'MerchantCenterAccountCreationFailed',
            'Reason' => ERROR_FORM_SUBMIT
        },
        $SubTestName . ' - Ensure Form error gets thrown'
    );

}

sub test_CreateMerchantCenterAccount_Fail_Api {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        HasAdultContent => [1],
        newMerchantAccountAcceptTerms => [1],
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        createMerchantCenterSubAccount => {
            id => ACCOUNT_ID,
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT($ApiService, $self->_createError(HTTP_FORBIDDEN, EXPECTED_TOO_MANY_SUBACCOUNTS_RESPONSE));

    $SUT->CreateMerchantCenterAccount($Servlet);

    ok(WasCalled($SUT, '_logError'), "$SubTestName - Verify specific subaccount exceeded quota error was logged.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

    is_deeply(
        $aExecuteFormErrorParams->[0],
        {
            'Name'   => 'MerchantCenterAccountCreationFailed',
            'Reason' => ERROR_TOO_MANY_SUBACCOUNTS
        },
        $SubTestName . ' - Ensure Api error too_many_accounts gets thrown'
    );
}

sub test_CreateGoogleAdsAccount_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService();
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->CreateGoogleAdsAccount($Servlet);

    ok(!WasCalled($Servlet->form(), 'executeFormError'), "$SubTestName - ensure executeFormError was NOT called");

    ok(WasCalled($AdsApiService, 'createGoogleAdsSubAccount'), "$SubTestName - ensure createGoogleAdsLink was called");
    ok(WasCalled($AdsApiService, 'createAndEnableGoogleAdsLink'), "$SubTestName - ensure createAndEnableGoogleAdsLink was called");
    ok(WasCalled($AdsApiService, 'createAdsAccountResellerLabel'), "$SubTestName - ensure createAdsAccountResellerLabel was called");
}

sub test_CreateGoogleAdsAccount_Fail_Form_Requirements_Not_Met {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 0,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService();
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->CreateGoogleAdsAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'AdsAccountCreationFailed',
                'Reason' => ERROR_ACCEPT_TERMS
            },
            $SubTestName . ' - Ensure form error not_accepted_terms gets thrown if google requirements are not met'
        );
    }
}

sub test_CreateGoogleAdsAccount_Fail_Form_No_MerchantCenter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService();
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->CreateGoogleAdsAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'AdsAccountCreationFailed',
                'Reason' => ERROR_FORM_SUBMIT
            },
            $SubTestName . ' - Ensure Api error form_submit_failed gets thrown if no Merchant Center account'
        );
    }
}

sub test_CreateGoogleAdsAccount_Fail_Form_Already_Has_Ads_Account {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken        => 'some-access-token',
        RefreshToken       => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        GoogleAdsAccountID => 12342314,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService();
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->CreateGoogleAdsAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'AdsAccountCreationFailed',
                'Reason' => ERROR_FORM_SUBMIT
            },
            $SubTestName . ' - Ensure Api error form_submit_failed gets thrown if user already has Ads account'
        );
    }
}

sub test_CreateGoogleAdsAccount_Fail_Api {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        createGoogleAdsSubAccount => {
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT(undef, $self->_createError(HTTP_BAD_REQUEST, EXPECTED_TOO_MANY_SUBACCOUNTS_RESPONSE), $AdsApiService);

    $SUT->CreateGoogleAdsAccount($Servlet);

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'AdsAccountCreationFailed',
                'Reason' => ERROR_TOO_MANY_SUBACCOUNTS
            },
            $SubTestName . ' - Ensure Api error too_many_accounts gets thrown'
        );
    }
}

sub test_LinkMerchantCenterAndAdsAccount_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({});
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->LinkMerchantCenterAndAds($Servlet);

    ok(WasCalled($AdsApiService, 'createAndEnableGoogleAdsLink'), "$SubTestName - Verify createAndEnableGoogleAdsLink was called.");
    ok(WasCalled($AdsApiService, 'createAdsAccountResellerLabel'), "$SubTestName - Verify createAdsAccountResellerLabel was called.");
}

sub test_LinkMerchantCenterAndAdsAccount_Fail_AccountLinking {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        createAndEnableGoogleAdsLink => {
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->LinkMerchantCenterAndAds($Servlet);

    ok(WasCalled($AdsApiService, 'createAndEnableGoogleAdsLink'), "$SubTestName - Verify createAndEnableGoogleAdsLink was called.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'MerchantCenterAdsLinkingFail',
                'Reason' => ERROR_TOO_MANY_SUBACCOUNTS
            },
            $SubTestName . ' - Ensure Api error MerchantCenterAdsLinkingFail gets thrown'
        );
    }
}

sub test_LinkMerchantCenterAndAdsAccount_Fail_LabelCreation {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        AccountEmail                           => MERCHANT_EMAIL,
    });
    my $AlreadyExistingConfig = 1;
    my $hQueryParameters = {};
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $ServletForm = $self->_createServletForm($hQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $AdsApiService = $self->_createGoogleAdsApiService({
        createAdsAccountResellerLabel => {
            throw_error => 1
        }
    });
    my $SUT = $self->_createSUT(undef, undef, $AdsApiService);

    $SUT->LinkMerchantCenterAndAds($Servlet);

    ok(WasCalled($AdsApiService, 'createAndEnableGoogleAdsLink'), "$SubTestName - Verify createAndEnableGoogleAdsLink was called.");
    ok(WasCalled($AdsApiService, 'createAdsAccountResellerLabel'), "$SubTestName - Verify createAdsAccountResellerLabel was called.");

    ok(WasCalled($Servlet->form(), 'executeFormError'),
        "$SubTestName - Verify form error was executed.");

    if (WasCalled($Servlet->form(), 'executeFormError')) {
        my $aExecuteFormErrorParams = GetParametersFromMockifyCall($Servlet->form(), 'executeFormError');

        is_deeply(
            $aExecuteFormErrorParams->[0],
            {
                'Name'   => 'CreateAdsAccountResellerLabelFail',
                'Reason' => ERROR_TOO_MANY_SUBACCOUNTS
            },
            $SubTestName . ' - Ensure Api error CreateAdsAccountResellerLabelFail gets thrown'
        );
    }
}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Subaccount_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID  => MULTI_CLIENT_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        MerchantCenterAccountStatuses => $self->_createSuccessfulMerchantCenterAccountStatusesResponse()
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);


    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );

    ok(
        WasCalled($SUT, '_getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that _getMerchantCenterAccountStatuses was called'
    );

    ok(
        WasCalled($ApiService, 'getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    my $ExpectedAccountLevelIssues = [ MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE ];


    is_deeply($Servlet->vars('MerchantCenterAccountStatuses')->{'AccountLevelIssues'}, $ExpectedAccountLevelIssues,
        "$SubTestName - Verify that MerchantCenterAccountLevelIssues correctly set.");

    is($Servlet->vars('MerchantCenterAccountStatuses')->{'IsWebsiteClaimed'}, 1,
        "$SubTestName - Verify that IsWebsiteClaimed set as true in servlet.");

    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
    ok( WasCalled($SUT, '_removeIgnoredIssues'), $SubTestName . ' - Verify that _removeIgnoredIssues was called' );
}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Standalone_Merchant_Account_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID  => undef,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        MerchantCenterAccountStatuses => $self->_createSuccessfulMerchantCenterAccountStatusesResponse()
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);


    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );

    ok(
        WasCalled($SUT, '_getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that _getMerchantCenterAccountStatuses was called'
    );

    ok(
        WasCalled($ApiService, 'getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    my $ExpectedAccountLevelIssues = [ MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE ];

    is_deeply($Servlet->vars('MerchantCenterAccountStatuses')->{'AccountLevelIssues'}, $ExpectedAccountLevelIssues,
        "$SubTestName - Verify that MerchantCenterAccountLevelIssues correctly set.");

    is($Servlet->vars('MerchantCenterAccountStatuses')->{'IsWebsiteClaimed'}, 1,
        "$SubTestName - Verify that IsWebsiteClaimed set as true in servlet.");

    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Unknown_Issue {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID  => MULTI_CLIENT_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        MerchantCenterAccountStatuses => MERCHANT_CENTER_ACCOUNT_STATUSES_UNKNOWN_ISSUE_RESPONSE
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);


    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );

    ok(
        WasCalled($SUT, '_getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that _getMerchantCenterAccountStatuses was called'
    );

    ok(
        WasCalled($ApiService, 'getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    my $ExpectedAccountLevelIssues = [ MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_UNKNOWN ];

    is_deeply($Servlet->vars('MerchantCenterAccountStatuses')->{'AccountLevelIssues'}, $ExpectedAccountLevelIssues,
        "$SubTestName - Verify that MerchantCenterAccountLevelIssues correctly set (no translationKey should be available).");

    is($Servlet->vars('MerchantCenterAccountStatuses')->{'IsWebsiteClaimed'}, 1,
        "$SubTestName - Verify that IsWebsiteClaimed set as true in servlet.");

    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );
}



sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Success_When_Ads_Account_IS_Connected {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID     => MULTI_CLIENT_ACCOUNT_ID,
        GoogleAdsAccountID                     => ADS_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        MerchantCenterAccountStatuses => $self->_createSuccessfulMerchantCenterAccountStatusesResponse()
    });

    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);


    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );

    ok(
        WasCalled($SUT, '_getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that _getMerchantCenterAccountStatuses was called'
    );

    ok(
        WasCalled($ApiService, 'getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    my $ExpectedAccountLevelIssues = [
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_MISSING_AD_WORDS_LINK,
        MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE,

    ];

    ok( WasCalled($SUT, '_removeIgnoredIssues'), $SubTestName . ' - Verify that _removeIgnoredIssues was called' );
    ok( WasCalled($SUT, '_isGoogleAdsAccountConnected'), $SubTestName . ' - Verify that _isGoogleAdsAccountConnected was called' );

    is_deeply($Servlet->vars('MerchantCenterAccountStatuses')->{'AccountLevelIssues'}, $ExpectedAccountLevelIssues,
        "$SubTestName - Verify missing_ad_words_link IS NOT removed when Ads account IS connected ");

    is($Servlet->vars('MerchantCenterAccountStatuses')->{'IsWebsiteClaimed'}, 1,
        "$SubTestName - Verify that IsWebsiteClaimed set as true in servlet.");

    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );

}

sub test_ViewGoogleSmartShopping_On_Getting_MerchantCenterAccount_Statuses_Success_When_No_Ads_Account {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID     => MULTI_CLIENT_ACCOUNT_ID,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
        MerchantCenterAccounts => SUCCESSFUL_MERCHANT_CENTER_ACCOUNTS_RESPONSE,
        MerchantCenterAccountStatuses => $self->_createSuccessfulMerchantCenterAccountStatusesResponse()
    });
    my $SUT = $self->_createSUT($ApiService);

    $SUT->ViewGoogleSmartShopping($Servlet);


    ok(
        WasCalled($ApiService, 'getOAuthEmail'),
        $SubTestName . ' - Verify that Google account email is being retrieved'
    );

    ok(
        WasCalled($SUT, '_getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that _getMerchantCenterAccountStatuses was called'
    );

    ok(
        WasCalled($ApiService, 'getMerchantCenterAccountStatuses'),
        $SubTestName . ' - Verify that getMerchantCenterAccounts was called when AccountInfo call is successful'
    );
    my $ExpectedAccountLevelIssues = [ MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE ];

    ok( WasCalled($SUT, '_removeIgnoredIssues'), $SubTestName . ' - Verify that _removeIgnoredIssues was called' );
    ok( WasCalled($SUT, '_isGoogleAdsAccountConnected'), $SubTestName . ' - Verify that _isGoogleAdsAccountConnected was called' );
    is_deeply($Servlet->vars('MerchantCenterAccountStatuses')->{'AccountLevelIssues'}, $ExpectedAccountLevelIssues,
        "$SubTestName - Verify missing_ad_words_link is removed when Ads account is not connected ");

    is($Servlet->vars('MerchantCenterAccountStatuses')->{'IsWebsiteClaimed'}, 1,
        "$SubTestName - Verify that IsWebsiteClaimed set as true in servlet.");

    ok( !WasCalled($SUT, '_buildConnectOAuthURL'), $SubTestName . ' - Verify that OAuth connect URL was NOT built' );

}

sub test_UnitySFViewGoogleSmartShoppingHTML_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken           => 'some-access-token',
        SiteVerificationToken => GOOGLE_SITE_VERIFICATION_TOKEN,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $HttpResponse = $self->_createHttpResponse();
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $HttpResponse);
    my $SUT = $self->_createSUT(undef, undef, undef, 'URI='.GOOGLE_SITE_VERIFICATION_TOKEN);

    $SUT->UnitySFViewGoogleSmartShoppingHTML($Servlet);

    my $Response = $Servlet->response;

    ok( WasCalled($SUT, '_getServerVariables'), $SubTestName . ' - Verify that _getServerVariables was called' );
    is($Response->content, GOOGLE_SITE_VERIFICATION_RESPONSE_SUCCESS, $SubTestName . ' - Verify that correct site verification string is returned');

}


sub test_UnitySFViewGoogleSmartShoppingHTML_Not_Found {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken           => 'some-access-token',
        SiteVerificationToken => undef,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $HttpResponse = $self->_createHttpResponse();
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm, $HttpResponse);
    my $SUT = $self->_createSUT(undef, undef, undef, 'URI='.GOOGLE_SITE_VERIFICATION_TOKEN);

    $SUT->UnitySFViewGoogleSmartShoppingHTML($Servlet);

    my $Response = $Servlet->response;

    ok( WasCalled($SUT, '_getServerVariables'), $SubTestName . ' - Verify that _getServerVariables was called' );
    is($Response->content, 'No google*.html available.', $SubTestName . ' - Verify that an error message is returned');

}

sub test_ViewGoogleSmartShopping_On_MerchantCenterConnected_CreatePortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        AccountEmail => EXAMPLE_EMAIL,
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        GoogleAdsAccountID => ADS_ACCOUNT_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
    });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });
    my $GooglePortalService = $self->_createGooglePortalService({
        getPortalConfigCreationParams => {
            thenReturn => {
                'PortalSite' => $self->_createPortalSite(),
                'Class' => 'GoogleSmartShopping',
                'Folder' => [],
            }
        },
    });
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService, undef, $GooglePortalService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($GooglePortalService, 'getSmartShoppingPortalConfig'),
        $SubTestName . ' - Verify that getSmartShoppingPortalConfig was called'
    );

    ok(
        WasCalled($GooglePortalService, 'getPortalConfigCreationParams'),
        $SubTestName . ' - Verify that getPortalConfigCreationParams was called'
    );

    is($Servlet->vars('PortalConfigID'), 12345, $SubTestName . ' - Verify that PortalConfigID is set on the Servlet');
}


sub test_ViewGoogleSmartShopping_On_MerchantCenterConnected_ExistingPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken => 'some-access-token',
        RefreshToken => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        AccountEmail => EXAMPLE_EMAIL,
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        GoogleAdsAccountID => ADS_ACCOUNT_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiService = $self->_createGoogleApiService({
        UserInfo => SUCCESSFUL_USER_INFO_RESPONSE,
    });
    my $AdsApiService = $self->_createGoogleAdsApiService({
        isAdsAccountBillingApproved => {
            thenReturn => 1
        }
    });
    my $GooglePortalService = $self->_createGooglePortalService({
        getSmartShoppingPortalConfig => {
            thenReturn => $self->_createPortalConfig()
        },
    });
    my $SUT = $self->_createSUT($ApiService, undef, $AdsApiService, undef, $GooglePortalService);

    $SUT->ViewGoogleSmartShopping($Servlet);

    ok(
        WasCalled($GooglePortalService, 'getSmartShoppingPortalConfig'),
        $SubTestName . ' - Verify that getSmartShoppingPortalConfig was called'
    );

    ok(
        !WasCalled($GooglePortalService, 'getPortalConfigCreationParams'),
        $SubTestName . ' - Verify that getPortalConfigCreationParams was NOT called'
    );

    is($Servlet->vars('PortalConfigID'), 12345, $SubTestName . ' - Verify that PortalConfigID is set on the Servlet');
}

###PRODUCTFEED
sub test_UnityViewGoogleSmartShoppingProductFeed_success_newPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig);


    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUTForProductFeedTest(
        {
            'GetInfoLanguage' => $hReturnFromInfoLanguage,
            'PortalConfig' => $MockedPortalConfig
        },
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoShopAddress {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = undef;
    my $MockedAddress = $self->_createAddress($hCountry);
    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUTForProductFeedTest(
        {
            'GetInfoLanguage' => $hReturnFromInfoLanguage,
            'PortalConfig' => $MockedPortalConfig
        },
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );

}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_NoSmartshoppingConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig, );

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUTForProductFeedTest(
        {
            'GetInfoLanguage' => $hReturnFromInfoLanguage,
            'PortalConfig' => $MockedPortalConfig
        },
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );
    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_fail_MerchantCenterRequirementsNotMet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 0,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServlet($MockedShop, $MockedPortalConfig);

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUTForProductFeedTest(
        {
            'GetInfoLanguage' => $hReturnFromInfoLanguage,
            'PortalConfig' => $MockedPortalConfig
        },
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), undef, $SubTestName . ' - Verify that upload is flagged forbidden.' );
    is ($MockedServlet->vars('ShippingMethodId'), undef, $SubTestName . ' - Verify that Servlet contains no Shipping Method.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_Success_ProductsStatistics {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0,
        total               => 5,
        active              => 5,
        pending             => 0,
        disapproved         => 0
    };

    my $GoogleApiService = $self->_createGoogleApiServiceForProductFeedTest({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });
    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $hParams = {
        'GoogleApiService'          => $GoogleApiService,
        'GetInfoLanguage'           => $hReturnFromInfoLanguage,
        'PortalConfig'              => $MockedPortalConfig,
    };

    my $SUT = $self->_createSUTForProductFeedTest($hParams);

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok(WasCalled($GoogleApiService, 'getRatedProductsStatistics'),
        $SubTestName . ' - Verify that service method _getProductStatistics was called.');
    is_deeply(
        $MockedServlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_success_existingPortalConfig {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        ID         => PORTAL_CONFIG_ID,
        LanguageID => 1
    });

    my $hPostData = {};
    my $ServletForm = $self->_createServletFormForProductFeedTest($hPostData);

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig, $ServletForm);
    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $SUT = $self->_createSUTForProductFeedTest(
        {
            'GetInfoLanguage'           => $hReturnFromInfoLanguage,
            'PortalConfig'              => $MockedPortalConfig,
        }
    );

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    ok( WasCalled($SUT, '_areProductUploadRequirementsMet'), $SubTestName . ' - Verify that SUT "_areProductUploadRequirementsMet" method was called.' );

    is ($MockedServlet->vars('IsAllowedProductUpload'), 1, $SubTestName . ' - Verify that upload is flagged allowed.' );
    is ($MockedServlet->vars('PortalConfigID'), PORTAL_CONFIG_ID, $SubTestName . ' - Verify that PortalConfigID is set.' );
    is ($MockedServlet->vars('PortalSiteLanguageCode3'), PORTAL_SITE_LANGUAGE_CODE, $SubTestName . ' - Verify that PortalSiteLanguageCode3 is set.' );
}

sub test_UnityViewGoogleSmartShoppingProductFeed_Show_MerchantCenterProductsTable {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hGoogleConfigParams = {
        MerchantCenterAccountID                => MERCHANT_CENTER_ID_PRODUCT_FEED_TEST,
        AreGoogleMerchantCenterRequirementsMet => 1,
    };
    my $MockedGoogleConfig = $self->_createGoogleConfigForProductFeedTest($hGoogleConfigParams);

    my $hCountry = {'Code2' => 'DE'};
    my $MockedAddress = $self->_createAddress($hCountry);

    my $hShopParams = {
        MockedAddress      => $MockedAddress,
        MockedGoogleConfig => $MockedGoogleConfig,
    };
    my $MockedShop = $self->_createShopForProductFeedTest($hShopParams);

    my $MockedPortalConfig = $self->_createPortalConfigForProductFeedTest({
        MockedShop => $MockedShop,
        LanguageID => 1
    });

    my $MockedServlet = $self->_createServletForProductFeedTest($MockedShop, $MockedPortalConfig);

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0,
        total               => 6,
        active              => 4,
        pending             => 0,
        disapproved         => 2
    };

    my $GoogleApiService = $self->_createGoogleApiServiceForProductFeedTest({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $PagerMerchantCenterProducts = $self->_createPagerMerchantCenterProducts();

    my $hReturnFromInfoLanguage = {
        Code3 => PORTAL_SITE_LANGUAGE_CODE
    };
    my $hParams = {
        'GoogleApiService'            => $GoogleApiService,
        'PagerMerchantCenterProducts' => $PagerMerchantCenterProducts,
        'GetInfoLanguage'             => $hReturnFromInfoLanguage,
        'PortalConfig'                => $MockedPortalConfig,
    };

    my $SUT = $self->_createSUTForProductFeedTest($hParams);

    $SUT->ViewGoogleSmartShoppingProductFeed($MockedServlet);

    is_deeply(
        $MockedServlet->vars('ProductStatuses'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that ProductStatistics was correctly set in servlet."
    );
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($ApiService, $MockedError, $AdsApiService, $ServerQueryString, $GooglePortalService) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Shop');
    $MockifySut->mock('_buildConnectOAuthURL')
        ->when(Object('DE_EPAGES::Presentation::Servlet::Permission'), Object('DE_EPAGES::Shop::API::Object::Shop'))
        ->thenReturn(CONNECT_OAUTH_URL);

    if(defined($ServerQueryString)) {
        $MockifySut->mock('_getServerVariables')
            ->when(Object(), String('QUERY_STRING'))
            ->thenReturn($ServerQueryString);
    }

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();
    if (defined($MockedError)) {
        $MockifySut->mockImported('DE_EPAGES::Core::API::Error', 'GetError')
            ->whenAny()
            ->thenReturn($MockedError);
        $MockifySut->mockImported('DE_EPAGES::Core::API::Log', 'LogError')
            ->whenAny()
            ->thenReturnUndef();
    }
    $MockifySut->mock('_deleteGooglePortalConfig')
        ->whenAny()
        ->thenReturnUndef();
    $MockifySut->spy('_getMerchantCenterAccountStatuses')->whenAny();
    $MockifySut->spy('_removeIgnoredIssues')->whenAny();
    $MockifySut->spy('_isGoogleAdsAccountConnected')->whenAny();
    $MockifySut->spy('_logError')->whenAny();

    if (defined($ApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(HashRef())
            ->thenReturn($ApiService);
    }

    if (defined($AdsApiService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService::new')
            ->when(HashRef())
            ->thenReturn($AdsApiService);
    }

    if (defined($GooglePortalService)) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GooglePortalService::new')
            ->when(HashRef())
            ->thenReturn($GooglePortalService);
    }

    $MockifySut->mockStatic('DE_EPAGES::Presentation::UI::Object::NewObject')->whenAny()->thenReturn($self->_createPortalConfig());

    return $MockifySut->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        AccountEmail               => $hConfig->{'AccountEmail'} // undef,
        AccessToken                => $hConfig->{'AccessToken'},
        RefreshToken               => $hConfig->{'RefreshToken'},
        ChecklistBillingTOC        => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo       => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice      => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy      => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountName => $hConfig->{'MerchantCenterAccountName'},
        MerchantCenterAccountID => $hConfig->{'MerchantCenterAccountID'},
        IsMerchantCenterSubAccount => $hConfig->{'IsMerchantCenterSubAccount'},
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'},
        GoogleAdsAccountID => $hConfig->{'GoogleAdsAccountID'} // undef ,
        GoogleAdsAccountName => $hConfig->{'GoogleAdsAccountName'} // undef ,
        IsAdsAccountSubAccount => $hConfig->{'IsAdsAccountSubAccount'} // undef ,
        AreMerchantCenterAndAdsLinked => $hConfig->{'AreMerchantCenterAndAdsLinked'} // undef ,
        AdsCampaignId => $hConfig->{'AdsCampaignId'} // undef ,
        SiteVerificationToken => $hConfig->{'SiteVerificationToken'} // undef ,
        Site                               => $self->_createShop({}, {}) // {} ,
    };
    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );
    $MockifyConfig->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);

    $MockifyConfig->addMockWithReturnValue('delete', undef);
    $MockifyConfig->addMockWithReturnValue('deleteMerchantCenterAccount', undef);
    $MockifyConfig->addMockWithReturnValue('deleteAdsAccount', undef);
    $MockifyConfig->addMockWithReturnValue('deleteAdsCampaign', undef);

    return $MockifyConfig->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');
    if ($hServiceResponse->{'getAndSaveOAuthTokens'}->{'throw_error'}) {
        $Mockify->mock('getAndSaveOAuthTokens')
            ->when(String())
            ->thenThrowError('oauth_failed');
    } else {
        $Mockify->mock('getAndSaveOAuthTokens')
            ->when(String())
            ->thenReturnUndef();
    }
    if ($hServiceResponse->{'getOAuthEmail'}->{'throw_error'}) {
        $Mockify->mock('getOAuthEmail')
            ->when()
            ->thenThrowError(ERROR_TOKEN_EXPIRED_OR_REVOKED);
    } else {
        $Mockify->mock('getOAuthEmail')
            ->when()
            ->thenReturn($hServiceResponse->{'UserInfo'} // {});
    }

    if ($hServiceResponse->{'revokeAccessToken'}->{'throw_error'}) {
        $Mockify->mock('revokeAccessToken')
            ->when(String())
            ->thenThrowError('lol');
    } else {
        $Mockify->mock('revokeAccessToken')
            ->when(String())
            ->thenReturn($hServiceResponse // {});
    }
    $Mockify->mock('getMerchantCenterAccounts')
        ->when()
        ->thenReturn($hServiceResponse->{'MerchantCenterAccounts'} // {});

    $Mockify->mock('getMerchantCenterAccountStatuses')
        ->when()
        ->thenReturn($hServiceResponse->{'MerchantCenterAccountStatuses'} // {});

    if ($hServiceResponse->{'deleteMerchantCenterSubAccount'}->{'throw_error'}) {
        $Mockify->mock('deleteMerchantCenterSubAccount')
            ->when(Number(ACCOUNT_ID))
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('deleteMerchantCenterSubAccount')
            ->when(Number(ACCOUNT_ID))
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'createMerchantCenterSubAccount'}->{'throw_error'}) {
        $Mockify->mock('createMerchantCenterSubAccount')
            ->when(Number())
            ->thenThrowError('throw!');
    } else {
        $Mockify->mock('createMerchantCenterSubAccount')
            ->when(Number())
            ->thenReturn($hServiceResponse->{'createMerchantCenterSubAccount'} // {});
    }

    return $Mockify->getMockObject();
}

sub _createGoogleAdsApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService');

    if ($hServiceResponse->{'isAdsAccountBillingApproved'}->{'throw_error'}) {
        $Mockify->mock('isAdsAccountBillingApproved')
            ->when()
            ->thenThrowError('access_denied');
    } elsif (defined $hServiceResponse->{'isAdsAccountBillingApproved'}->{'thenReturn'}) {
        $Mockify->mock('isAdsAccountBillingApproved')
            ->when()
            ->thenReturn($hServiceResponse->{'isAdsAccountBillingApproved'}->{'thenReturn'});
    }

    if ($hServiceResponse->{'createGoogleAdsSubAccount'}->{'throw_error'}) {
        $Mockify->mock('createGoogleAdsSubAccount')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('createGoogleAdsSubAccount')
            ->whenAny()
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'createAndEnableGoogleAdsLink'}->{'throw_error'}) {
        $Mockify->mock('createAndEnableGoogleAdsLink')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('createAndEnableGoogleAdsLink')
            ->whenAny()
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'unlinkGoogleAdsAccount'}->{'throw_error'}) {
        $Mockify->mock('unlinkGoogleAdsAccount')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('unlinkGoogleAdsAccount')
            ->whenAny()
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'deleteAdsCampaign'}->{'throw_error'}) {
        $Mockify->mock('deleteAdsCampaign')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('deleteAdsCampaign')
            ->whenAny()
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'getGoogleAdsAccounts'}->{'throw_error'}) {
        $Mockify->mock('getGoogleAdsAccounts')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('getGoogleAdsAccounts')
            ->whenAny()
            ->thenReturnUndef();
    }

    if ($hServiceResponse->{'createAdsAccountResellerLabel'}->{'throw_error'}) {
        $Mockify->mock('createAdsAccountResellerLabel')
            ->whenAny()
            ->thenThrowError('access_denied');
    } else {
        $Mockify->mock('createAdsAccountResellerLabel')
            ->whenAny()
            ->thenReturnUndef();
    }

    return $Mockify->getMockObject();
}

sub _createGooglePortalService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GooglePortalService');

    if (defined $hServiceResponse->{'getSmartShoppingPortalConfig'}->{'thenReturn'}) {
        $Mockify->mock('getSmartShoppingPortalConfig')
            ->when()
            ->thenReturn($hServiceResponse->{'getSmartShoppingPortalConfig'}->{'thenReturn'});
    } else {
        $Mockify->mock('getSmartShoppingPortalConfig')
            ->when()
            ->thenReturnUndef();
    }

    if (defined $hServiceResponse->{'getPortalConfigCreationParams'}->{'thenReturn'}) {
        $Mockify->mock('getPortalConfigCreationParams')
            ->when()
            ->thenReturn($hServiceResponse->{'getPortalConfigCreationParams'}->{'thenReturn'});
    } else {
        $Mockify->mock('getPortalConfigCreationParams')
            ->when()
            ->thenReturnUndef();
    }

    return $Mockify->getMockObject();
}

sub _createError {
    my $self = shift;
    my ($Status, $hResponse) = @_;

    my $Error = DE_EPAGES::Core::API::Error->new(
        'Code' => 'Error',
        'Vars' => {
            'Status' => $Status,
            'Response' => encode_json($hResponse)
        }
    );
    return $Error;
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm, $HttpResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);

    if(defined($HttpResponse)) {
        $MockifyServlet->addMockWithReturnValue('response', $HttpResponse);
    }

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;

    my $hAttributes = {
        'ID'                          => undef,
        'GoogleSiteVerificationToken' => 'VerificationToken',
        'GoogleSmartShoppingConfig'   => $GoogleSmartShoppingConfig,
        'IsUnityRegistered'           => 1
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    my $Shop  = $MockifyShop->getMockObject();
    $Shop->set({'Site' => $Shop});

    return $Shop;
}

sub _createPortalConfig {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'NameOrAlias' => 'NameOrAlias' ,
        'PortalConfigClass' => 'GoogleSmartShopping',
        'ID' => '12345',
    };

    my $MockifyPortal = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig', $hAttributes
    );

    return $MockifyPortal->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('queryValues')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hQueryParameters);


    $Mockify->mock('executeFormError')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    $Mockify->mock('addFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    $Mockify->mock('existsFormError')
        ->when()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createSuccessfulMerchantCenterAccountStatusesResponse {
    my $self = shift;

    return {
        "accountId"=> "414922848",
        "accountLevelIssues" => [
            MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_QUALITY_LOW,
            MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_MISSING_AD_WORDS_LINK,
            MERCHANT_CENTER_ACCOUNT_LEVEL_ISSUE_PENDING_ADDRESS_AND_PHONE
        ],
        "kind"=> "content#accountStatus",
        "websiteClaimed"=> "true",
        "accountManagement"=> "manual"
    };
}

sub _createHttpResponse {
    my $self = shift;

    return HTTP::Response->new();
}


sub _createPortalSite {
    my $self = shift;

    my $hAttributes = {
        'LocaleID' => 'en_GB',
        'Alias' => 'GoogleSmartShopping'
    };

    my $MockifyPortalSite = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ProductPortal::API::Object::PortalSite', $hAttributes
    );

    return $MockifyPortalSite->getMockObject();
}

### Setup Methods for ProductFeedView tests
sub _createShippingMethod {
    my $self = shift;
    my $hParameterList = {
        'ID'          => DEFAULT_SHIPPING_METHOD_ID,
        'NameOrAlias' => DEFAULT_SHIPPING_METHOD_ALIAS
    };
    my $MockifyShippingMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::ShippingTypes::API::Object::ShippingMethodWeightOfBasket',
        $hParameterList
    );

    return $MockifyShippingMethod->getMockObject();
}

sub _createPagerMerchantCenterProducts {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts');

    $Mockify->mock('execute')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('page')
        ->when()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hCountry) = @_;

    my $hParameterList = {
        'Country'   => $hCountry ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}

sub _createSUTForProductFeedTest {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Shop');

    $MockifySut->mock('_loadPortal')->whenAny()->thenReturn($hParams->{PortalConfig});

    $MockifySut->spy('_areProductUploadRequirementsMet')->when(Object());

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->whenAny()
            ->thenReturn($hParams->{'GoogleApiService'});
    }
    if (defined $hParams->{'ProductBatchService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::ProductBatchService::new')
            ->when(String(), Object())
            ->thenReturn($hParams->{'ProductBatchService'});
    }
    if (defined $hParams->{'PagerMerchantCenterProducts'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::Object::PagerMerchantCenterProducts::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'PagerMerchantCenterProducts'});
    }
    if (defined $hParams->{'GooglePortalService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GooglePortalService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GooglePortalService'});
    }

    if (defined $hParams->{'GetInfoLanguage'}) {
        $MockifySut->mockImported('DE_EPAGES::Object::API::Table::Language', 'GetInfoLanguage')
            ->whenAny()
            ->thenReturn($hParams->{'GetInfoLanguage'});
    }

    if (defined($hParams->{'NewGooglePortalService'})) {
        $MockifySut->mock('_createPortalConfig')
            ->whenAny()
            ->thenReturn($hParams->{'NewGooglePortalService'});
    }

    if(defined($hParams->{'SpyViewPagedChildren'})) {
        $MockifySut->spy('_viewPagedChildren')->whenAny();
    } else {
        $MockifySut->mock('_viewPagedChildren')
            ->whenAny()
            ->thenReturnUndef();
    }

    return $MockifySut->getMockObject();
}

sub _createPortalConfigForProductFeedTest {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'ExportAllProducts' => $hParams->{'ExportAllProducts'} // undef,
        'ShippingMethod'    => $self->_createShippingMethod(),
        'NameOrAlias'       => 'NameOrAlias',
        'PortalConfigClass' => 'GoogleSmartShopping',
        'ID'                => $hParams->{'ID'} // '12345',
        'Site'              => $hParams->{'MockedShop'} // {},
        'Products'          => $hParams->{'Products'} // [],
        'LanguageID'        => $hParams->{'LanguageID'} // undef
    };

    my $MockifyPortal = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingPortalConfig', $hAttributes
    );
    $MockifyPortal->addMockWithReturnValueAndParameterCheck('set', undef, [ 'hashref' ]);
    $MockifyPortal->addMockWithReturnValue('removeAllProducts', undef);
    $MockifyPortal->addMockWithReturnValueAndParameterCheck('removeProducts', undef, [ 'arrayref' ]);

    return $MockifyPortal->getMockObject();
}

sub _createShopForProductFeedTest {
    my $self = shift;
    my ($hParams) = @_;

    my $hAttributes = {
        'Address'                   => $hParams->{'MockedAddress'} // undef,
        'GoogleSmartShoppingConfig' => $hParams->{'MockedGoogleConfig'} // undef,
        'IsUnityRegistered'         => 1
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('existsChild', defined($hParams->{'MockedGoogleConfig'}), [{'string'=>'GoogleSmartShoppingConfig'}]);
    $MockifyShop->addMockWithReturnValueAndParameterCheck('loadObject',$self->_createShippingMethod(), ['string', 'string']);

    return $MockifyShop->getMockObject();
}

sub _createGoogleConfigForProductFeedTest {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC                => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo               => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies         => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID            => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID                 => $hConfig->{'GoogleAdsAccountID'} // undef,
        AdsCampaignId                      => $hConfig->{'AdsCampaignId'} // undef,
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'} // undef,
        AreMerchantCenterAndAdsLinked      => $hConfig->{'AreMerchantCenterAndAdsLinked'} // 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $Mockify->getMockObject();
}

sub _createServletForProductFeedTest {
    my $self = shift;
    my ($Shop, $PortalConfig, $ServletForm, $ServletResponse) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('params', { 'PortalConfigID' => PORTAL_CONFIG_ID });

    if (defined $ServletForm && defined $ServletResponse) {
        $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
        $MockifyServlet->addMockWithReturnValue('response', $ServletResponse);
        $MockifyServlet->addMockWithReturnValue('params', { 'PortalConfigID' => PORTAL_CONFIG_ID });
        $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, [ 'hashref' ]);
        # $MockifyServlet->addMockWithReturnValueAndParameterCheck('params', undef, [ 'string', 'string' ]);
    }


    return $MockifyServlet->getMockObject();
}

sub _createServletFormForProductFeedTest {
    my $self = shift;
    my ($hPostData) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');

    if( defined $hPostData->{'ListObjectID'}) {
        $Mockify->mock('value')
            ->when(String('ListObjectID'))
            ->thenReturn($hPostData->{'ListObjectID'});
    }

    if( defined $hPostData->{'PortalID'}) {
        $Mockify->mock('PortalID')
            ->when(String('PortalID'))
            ->thenReturn($hPostData->{'PortalID'});
    }

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hPostData);

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenThrowError('Something bad happened! fail;<');

    return $Mockify->getMockObject();
}

sub _createGoogleApiServiceForProductFeedTest {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('getRatedProductsStatistics')
        ->when()
        ->thenReturn($hServiceResponse->{'RatedProductsStatistics'} // {}
    );

    $Mockify->mock('productsStatuses')
        ->when(Number(), Number())
        ->thenReturn($hServiceResponse->{'ProductsStatuses'} // {}
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();

