#========================================================================================
# package      DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a csv driver to export object Idealo products
#
# example      use DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver;
#
#               my $Driver = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->exportCSV;
#========================================================================================
package DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver;
use base qw(DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver Exporter);

use strict;
use Algorithm::CheckDigits;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Template qw ( ProcessFileTemplate );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE );
use DE_EPAGES::Idealo::API::Constants qw ($PAYMENT_PERCETAGE_FEE $PAYMENT_FIX_FEE);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use Scalar::Util qw ( blessed );

use constant ISBN_LENGTH => 10;
use constant DYNAMIC_ATTRIBUTE_PREFIX => 'idealo';

#========================================================================================
sub ErrorDefinition {
    return {
        'ProductNoDesc' => 'Product( #ProductID ) has no description, can not export to portal.',
        'ProductNoCat' => 'Product( #ProductID ) has no category, can not export to portal.',
        'ProductNoMPNR' => 'Product( #ProductID ) has no manufacturer SKU, can not export to portal.',
        'ProductNoDeliver' => 'Product( #ProductID ) has no delivery time, can not export to portal.',
        'ProductNoStock' => 'Product( #ProductID ) has no stock, can not export to portal.',
    }
}

#========================================================================================
my @MAP_Idealo_ATTRIBUTES = qw (
    aid
    brand
    mpnr
    ean
    pzn
    isbn
    name
    desc
    shop_cat
    price
    link
    image
    downloadable
    ref_price
    dlv_time
    dlv_cost
    increment
    guid
    energyEfficiencyClass
    energySpectrum
    energyLabelVersion
    energyLabelUrl
    datasheetUrl
);

my $hMAP_Idealo_COLUMNNAMES = {
    'aid'                => 'Alias',
    'brand'              => 'Brand',
    'mpnr'               => 'Prod.No',
    'ean'                => 'EAN',
    'pzn'                => 'PZN',
    'isbn'               => 'ISBN',
    'name'               => 'Name',
    'desc'               => 'Description',
    'shop_cat'           => 'Categories',
    'price'              => 'Price',
    'link'               => 'URL',
    'image'              => 'Image.URL',
    'downloadable'       => 'IsDownload',
    'ref_price'          => 'Ref.price',
    'dlv_time'           => 'Deliverytime',
    'dlv_cost'           => 'DeliveryCosts',
    'increment'          => 'Increment',
    'guid'               => 'GUID',
    'energyEfficiencyClass' => 'EEC_efficiencyClass',
    'energySpectrum' => 'EEC_spectrum',
    'energyLabelVersion' => 'EEC_version',
    'energyLabelUrl' => 'EEC_labelUrl',
    'datasheetUrl' => 'EEC_dataSheetUrl'
};

my $PREFIX = 'shipping';

#========================================================================================
# function     exportCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exporter->exportCSV($FileName, $hFormat, $aColumns, $ViewObject, $aObjects );
#----------------------------------------------------------------------------------------
# description  exports data (2 dim. array) into a file.
#               sets default file format parameters :
#               sep_char = ";" , always_quote = 1 , encoding = utf8
#----------------------------------------------------------------------------------------
# input        $FileName |  export file | string
# input        $hFormat |  export file format | ref.hash
# input        $aColumns | column attributes (used for header and values) | attribute object
# input        $ViewObject | object to view monitor should be the shop | object
# input        $aObjects | list of objects | ref.array.object
#========================================================================================
sub exportCSV {
    my $self = shift;
    my ($FileName, $hFileFormat, $aColumnNames, $ViewObject, $aObjects) = @_;
    $hFileFormat = {} unless defined $hFileFormat;
    $hFileFormat->{'sep_char'}     = ';' if not defined $hFileFormat->{'sep_char'};
    $hFileFormat->{'always_quote'} = '1' if not defined $hFileFormat->{'always_quote'};
    $hFileFormat->{'encoding'}     = 'utf8' unless defined $hFileFormat->{'encoding'};

    #add the driver variables here, because needed for columns
    my $hPortalConfig = $ViewObject->get([qw (
        PortalSite
        CurrencyID
        ShippingMethod
    )]);
    $self->{$_} = $hPortalConfig->{$_} foreach keys %$hPortalConfig;
    $self->{'LanguageCode'} = $hPortalConfig->{'PortalSite'}->get('Language')->{'Code2'};
    $self->{'Object'} = $ViewObject;
    $self->{'sep_char'} = $hFileFormat->{'sep_char'};
    my $PortalSite = $hPortalConfig->{'PortalSite'};
    $self->{'TaxModel'} = $PortalSite->get('TaxModel');

    $aColumnNames = [ @MAP_Idealo_ATTRIBUTES ];

    my @DynamicAttributesColumns = $self->_getDynamicAttributesColumns($ViewObject);
    push(@$aColumnNames, @DynamicAttributesColumns);

    my $aAvailableShippingColumns = $self->_availableShippingColumns($ViewObject);
    push(@$aColumnNames, @$aAvailableShippingColumns);

    $self->SUPER::exportCSV($FileName, $hFileFormat,$aColumnNames ,$ViewObject, $aObjects);
    return;
}

#========================================================================================
# function     getExportValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getExportValues($Object);
#----------------------------------------------------------------------------------------
# description  get values for one line.
#----------------------------------------------------------------------------------------
# input        $Object |  the object to export | object
# return       array ref
#========================================================================================
sub getExportValues {
    my $self = shift;
    my ($Object) = @_;
    my @Values;

    foreach my $Name (@{$self->columns}) {
        my $Value ;

        if ($self->_hasDynamicAttributePrefix($Name)) {
            $Value = $self->dynamicAttribute( $Object, $Name );
        } elsif ($Name =~ /^$PREFIX(.*)/ ) {
            $Value = $self->shipping( $Object, $1 );
        } else {
            $Value = $self->$Name( $Object );
        }
        $Value = '' unless defined $Value;
        push @Values, $Value;
    }
    return \@Values;
}

#========================================================================================
# function     isProductValidPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->isProductValidPortal($Product);
#----------------------------------------------------------------------------------------
# description  return 1 if product is visible and has name and has a price
#----------------------------------------------------------------------------------------
# input        $Product |  the product to export | object
#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    my $ProductID = $Product->id();
    #test for description
    Error('ProductNoDesc', {'ProductID'=>$ProductID, 'Product'=>$Product})
        unless $self->descriptionAttribute();
    my $DescriptionAttribute = $self->descriptionAttribute();

    if ( defined $DescriptionAttribute ) {
        my $DescAlias = $DescriptionAttribute->alias();
        Error('ProductNoDesc', {'ProductID'=>$ProductID, 'Product'=>$Product})
           unless $Product->getWithParent($DescAlias,$self->languageID());
    }

    #test for categories
    Error('ProductNoCat', {'ProductID'=>$ProductID, 'Product'=>$Product})
       unless $Product->getWithParent('Categories');
    my $aCategories = $Product->getWithParent('Categories');

    my @CategoryNames;
    foreach my $Category (@$aCategories) {
        push @CategoryNames, $Category->get('NameOrAlias', $self->languageID())
           unless $Category->instanceOf('StartPage') or $Category->instanceOf('CategorySpecialOffer');
    }

    my $hasCategories = (scalar(@CategoryNames) > 0);
    Error('ProductNoCat', {'ProductID'=>$ProductID, 'Product'=>$Product}) unless $hasCategories;

    #test for manufacturer no
    my $UPCEAN = $Product->getWithParent('UPCEAN');
    if (! defined($UPCEAN)) {
        Error('ProductNoMPNR', {'ProductID'=>$ProductID, 'Product'=>$Product})
            unless $Product->getWithParent('ManufacturerSKU');
    } elsif ( !_IsISBN13($UPCEAN) && !_IsISBN($UPCEAN) && !_IsPZN($UPCEAN)) {
         Error('ProductNoMPNR', {'ProductID'=>$ProductID, 'Product'=>$Product})
            unless $Product->getWithParent('ManufacturerSKU');
    }

    #test for delivery time
    if (! $Product->getWithParent('IsDownloadProduct')) {
       Error('ProductNoDeliver', {'ProductID'=>$Product->id, 'Product'=>$Product})
          unless $Product->getWithParent('DeliveryPeriod');
    }

    #test for stock availability
    unless ($self->_isInStock($Product)) {
        Error('ProductNoStock', {'ProductID'=>$Product->id, 'Product'=>$Product});
    }

    $self->isProductValidPortalExt1($Product);

    return;
}

#========================================================================================
sub dynamicAttribute {
    my $self = shift;
    my ($Product, $Name) = @_;

    return '' unless $Product->class()->existsAttribute($Name);

    my $Value = $Product->getWithParent($Name, $self->languageID);

    # value is a blessed object if it is a predefstring with option picked
    my $ParsedValue = blessed($Value) ? $Value->get('Value', $self->languageID) : $Value;

    return ($ParsedValue // '');
}
#========================================================================================
sub aid {
    my $self = shift;
    my ($Product) = @_;

    return $Product->alias;
}

#========================================================================================
sub brand {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('Manufacturer');
}

#========================================================================================
sub mpnr {
    my $self = shift;
    my ($Product) = @_;

    my $UPCEAN = $Product->getWithParent('UPCEAN');
    if (defined($UPCEAN)) {
      return '' if ( _IsISBN13($UPCEAN) || _IsISBN($UPCEAN) || _IsPZN($UPCEAN));
    }

    return $Product->getWithParent('ManufacturerSKU');
}

#========================================================================================
sub ean {
    my $self = shift;
    my ($Product) = @_;

    my $UPCEAN = $Product->getWithParent('UPCEAN');
    if (defined($UPCEAN)) {
      return '' if ( _IsISBN13($UPCEAN) || _IsISBN($UPCEAN) || _IsPZN($UPCEAN));
    }

    return $UPCEAN;
}

#========================================================================================
# Export column "pzn", the German "Pharmazentralnummer" of the offer.
# Numeric value (maxlength=7)
sub pzn {
    my $self = shift;
    my ($Product) = @_;

    my $UPCEAN = $Product->getWithParent('UPCEAN');
    my $pzn = '';

    if (defined($UPCEAN) && _IsPZN($UPCEAN)) {
        $pzn = $UPCEAN;
    }

    return $pzn;
}

#========================================================================================
# Export column "isbn", the ISBN-No. of the offer.
# String of the ISBN-10 or ISBN-13
sub isbn {
    my $self = shift;
    my ($Product) = @_;

    my $UPCEAN = $Product->getWithParent('UPCEAN');
    my $isbn = '';

    if (defined($UPCEAN)) {
        if ( _IsISBN13($UPCEAN) || _IsISBN($UPCEAN) ) {
            $isbn = $UPCEAN;
        }
    }

    return $isbn;
}

#========================================================================================
sub name {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('NameOrAlias', $self->languageID);
}

#========================================================================================
sub desc {
    my $self = shift;
    my ($Product) = @_;

    my $DescriptionAttribute = $self->descriptionAttribute;
    if ( defined $DescriptionAttribute ) {
        my $DescAlias = $DescriptionAttribute->alias;
        return $Product->getWithParent($DescAlias,$self->languageID);
    }
}

#========================================================================================
sub shop_cat {
    my $self = shift;
    my ($Product) = @_;

    my $aCategories = $Product->getWithParent('Categories');
    my @CategoryNames;
    my $TempCategory;

    foreach my $Category (@$aCategories) {
        my @CategoryName;
        $TempCategory = $Category;
        # As long as category is not Home page (has Alias named Categories)
        while (defined($TempCategory) && ($TempCategory->get('Alias') ne 'Categories')) {
            my $temp = $TempCategory->get('NameOrAlias', $self->languageID);
               $temp =~ tr/ /_/;

            push @CategoryName,$temp;
            $TempCategory = $TempCategory->parent;
        }
        $TempCategory = join('>', reverse(@CategoryName));

        push @CategoryNames, $TempCategory
            unless $Category->instanceOf('StartPage') or $Category->instanceOf('CategorySpecialOffer');
    }
    my $CategoryNamesString = join(' ', @CategoryNames);
    return $CategoryNamesString;
}

#========================================================================================
sub price {
    my $self = shift;
    my ($Product) = @_;

    return $self->formatPrice($self->productPrice($Product));
}

#========================================================================================
sub link {
    my $self = shift;
    my ($Product) = @_;

    return $self->productUrl($Product);
}

#========================================================================================
sub image {
    my $self = shift;
    my ($Product) = @_;

    my $Result;

    if($self->imageAttribute->alias eq 'ImagesSlideShow') {
        # EPG-22571: export more than one product image
        my $URLPath = $self->_GetProductUrl($Product);
        my $GalleryImages = $Product->get('ImagesSlideShow');
        my @ResultURLs;
        foreach my $Image (@$GalleryImages) {
            push(@ResultURLs, $URLPath . $Image->{'ImageName'});
        }
        $Result = join('|', @ResultURLs); # Idealo parses multiple images separated by |
    }
    else {
        $Result = $self->imageUrl($Product, $self->imageAttribute);
    }

    return $Result;
}

#========================================================================================
sub dlv_time {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('DeliveryPeriod').$self->_dlvTimeUnit();
}
#========================================================================================
sub dlv_cost {
    my $self = shift;
    my ($Product) = @_;

    my $ShippingPrice = $self->shippingPrice($Product);
    return $self->formatPrice($ShippingPrice);
}
#========================================================================================
sub increment {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('IntervalOrder');
}

#========================================================================================
# function     _getDynamicAttributesColumns
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_getDynamicAttributesColumns($ViewObject);
#----------------------------------------------------------------------------------------
# description  returns user defined dynamic attributtes (idealo prefix)
#----------------------------------------------------------------------------------------
# input        $ViewObject | object to view monitor should be the shop | object
# return       @DymanicAttributes | user defined idealo Dynamic Attributes | perl array
#========================================================================================
sub _getDynamicAttributesColumns {
    my $self = shift;
    my ($ViewObject) = @_;

    my @DymanicAttributes;
    my $hInsertedDymanicAttributes = {};

    my $Shop = $ViewObject->getSite();

    return unless $Shop->get('AllProductTypes');

    foreach my $ProductType (@{$Shop->get('AllProductTypes')}) {

        next unless $ProductType->existsChild('Attributes');

        foreach my $ProductType (@{$ProductType->child('Attributes')->children()}) {

            my $AttributeAlias = $ProductType->alias();
            if ($self->_hasDynamicAttributePrefix($AttributeAlias) && !defined($hInsertedDymanicAttributes->{$AttributeAlias})) {
                push(@DymanicAttributes, $AttributeAlias);
                $hMAP_Idealo_COLUMNNAMES->{$AttributeAlias} = $AttributeAlias;
                $hInsertedDymanicAttributes->{$AttributeAlias} = $AttributeAlias;
            }
        }
    }
    return @DymanicAttributes;
}

sub _hasDynamicAttributePrefix {
    my $self = shift;
    my ($Alias) = @_;

    my $AliasPrefix = substr($Alias, 0, 6);
    return ( $AliasPrefix eq  DYNAMIC_ATTRIBUTE_PREFIX );
}

sub _availableShippingColumns {
    my $self = shift;
    my ($PortalConfig) = @_;

    my $ShippingMethod = $self->shippingMethod;
    my $hVars = {
        'CurrencyID'=> $self->currencyID,
        'TaxModel'  => $self->taxModel,
        'BillingCountryID' => $self->portalSite->get('Country')->{'CountryID'},
    };
    my $aPaymentMethods = $ShippingMethod->get('PaymentMethods');

    # insert default payment
    my @PaymentMethods;
    foreach my $PaymentMethod ( @$aPaymentMethods) {
      push @PaymentMethods, $PaymentMethod if $PaymentMethod->canAddToBasket($hVars);
    }
    my @shippingColumns = map {'shipping' . $_->alias} @PaymentMethods;
    return \@shippingColumns;
}

# get the name of the column
sub shippingName {
    my $self = shift;
    my ($PaymentAlias) = @_;

    my ($PaymentMethod)= grep {$_->alias eq $PaymentAlias } @{$self->object->getSite()->get('PaymentMethods')};
    return $PaymentMethod->get('NameOrAlias', $self->languageID);
}

# export column "shipping"
sub shipping {
    my $self = shift;
    my ($Product, $PaymentAlias) = @_;

    my $Price = $self->productPrice($Product);
    my $ShippingPrice = $self->shippingPrice($Product);
    my ($PaymentMethod)= grep {$_->alias eq $PaymentAlias } @{$Product->getSite()->get('PaymentMethods')};
    my $PaymentPrice  = $self->getPriceForPayment($PaymentMethod, $Price+$ShippingPrice);
    my $TotalPrice = $Price + $ShippingPrice + $PaymentPrice;
    my $Difference = $TotalPrice - $Price - $ShippingPrice;

    if ( $Difference<0 ) {
       $Difference = 0; #idealo can not process discounts smaller than 0
    }

    return $self->formatPrice($Difference);
}

#========================================================================================
sub getPriceForPayment{
    my $self = shift;
    my ($PaymentMethod, $Price) = @_;
    my $DiscountMode = $PaymentMethod->get('CashDiscountMode');

    if ($DiscountMode == $PAYMENT_PERCETAGE_FEE){
        my $Discount = $PaymentMethod->get('CashDiscount');
        return $Price * $Discount / 100; ## no critic (ProhibitMagicNumbers)
    } elsif ($DiscountMode == $PAYMENT_FIX_FEE){
        my $aDiscounts = $PaymentMethod->get('Discounts');
        foreach my $CurrencyDiscount (@$aDiscounts){
            if ( $CurrencyDiscount->{'CurrencyID'} eq $self->currencyID ){
                return $CurrencyDiscount->{'PriceValue'};
            }
        }
    }

    return 0;

}

#========================================================================================
sub downloadable {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('IsDownloadProduct');
}

#========================================================================================
sub ref_price {
    my $self = shift;
    my ($Product) = @_;

    my $RefAmount = $Product->getWithParent('RefAmount');
    my $RefContentAmount = $Product->getWithParent('RefContentAmount');

    if (defined($RefAmount) && defined($RefContentAmount)) {
     if ($RefAmount>0 && $RefContentAmount>0) {
      my $refprice = $Product->getWithParent('RefAmount')/$Product->getWithParent('RefContentAmount')*$self->productPrice($Product);
      if ($Product->getWithParent('RefAmount')==1) {
        return $self->formatPrice($refprice).$self->currencyID.'/'.$Product->getWithParent('RefUnit')->get('Abbreviation');
      }
      else {
        return $self->formatPrice($refprice).$self->currencyID.'/'.$Product->getWithParent('RefAmount').$Product->getWithParent('RefUnit')->get('Abbreviation');
      }
     }
    }
    return '';
}

#========================================================================================
sub guid {
    my $self = shift;
    my ($Product) = @_;

    return $Product->get('GUID');
}

sub energyEfficiencyClass {
    my $self = shift;
    my ($Product) = @_;

    my $EfficiencyClass = '';

    if (!$self->_IsEnergyLabelRange($Product)) {
        $EfficiencyClass = $Product->get('EnergyLabelsString');
    }

    return $EfficiencyClass;
}

sub energySpectrum {
    my $self = shift;
    my ($Product) = @_;

    my $EnergySpectrum = '';

    if ($self->_IsEnergyLabelRange($Product)) {
        my @Labels = map { $_->{'Value'} } @{$Product->get('EnergyLabels')};
        $EnergySpectrum = join('-', @Labels);
    }

    return $EnergySpectrum;
}

sub energyLabelVersion {
    my $self = shift;
    my ($Product) = @_;

    my $EnergyLabelVersion = 0;
    my $HighestEfficiencyClass = $Product->get('EnergyLabelHighestEfficiencyClass');

    if ($HighestEfficiencyClass eq 'A') {
        $EnergyLabelVersion = 1;
    }

    return $EnergyLabelVersion;
}

sub energyLabelUrl {
    my $self = shift;
    my ($Product) = @_;

    my $EnergyLabelUrl = '';
    my $EnergyLabelFilename = $Product->get('EnergyLabelSourceFile');

    if (defined($EnergyLabelFilename)) {
        $EnergyLabelUrl = $self->_GetProductUrl($Product) . $EnergyLabelFilename;
    }

    return $EnergyLabelUrl;
}

sub datasheetUrl {
    my $self = shift;
    my ($Product) = @_;

    my $DatasheetUrl = '';
    my $DatasheetFilename = $Product->get('ProductDataSheet', $self->languageID());

    if (defined($DatasheetFilename)) {
        $DatasheetUrl = $self->_GetProductUrl($Product) . $DatasheetFilename;
    }

    return $DatasheetUrl;
}

#========================================================================================
# function     _IsPZN
# state        Private
#----------------------------------------------------------------------------------------
# syntax       $IsPZN = _IsPZN('0123456');
#----------------------------------------------------------------------------------------
# description  Subroutine to check, if the given UPCEAN is a PZN.
#----------------------------------------------------------------------------------------
# input        $UPCEAN | EAN number of the product | string
#----------------------------------------------------------------------------------------
# return       true / false
#========================================================================================
sub _IsPZN {
    my ($UPCEAN) = @_;

    #remove spaces and - in the code
    $UPCEAN =~ s/-//ig;
    $UPCEAN =~ s/ //ig;

    my $pzn = CheckDigits('pzn');

    return $pzn->is_valid($UPCEAN);
}

#========================================================================================
# function     _IsISBN13
# state        Private
#----------------------------------------------------------------------------------------
# syntax       $IsISBN13 = _IsISBN13('978-3831716630');
#----------------------------------------------------------------------------------------
# description  Subroutine to check, if the given UPCEAN is a ISBN13.
#----------------------------------------------------------------------------------------
# input        $UPCEAN | EAN number of the product | string
#----------------------------------------------------------------------------------------
# return       true / false
#========================================================================================
sub _IsISBN13 {
    my ($UPCEAN) = @_;

    #remove spaces and - in the code
    $UPCEAN =~ s/-//ig;
    $UPCEAN =~ s/ //ig;

    my $isbn13 = CheckDigits('isbn13');

    return $isbn13->is_valid($UPCEAN);
}

#========================================================================================
# function     _IsISBN
# state        Private
#----------------------------------------------------------------------------------------
# syntax       $IsISBN = _IsISBN('3831716633');
#----------------------------------------------------------------------------------------
# description  Subroutine to check, if the given UPCEAN is a ISBN.
#----------------------------------------------------------------------------------------
# input        $UPCEAN | EAN number of the product | string
#----------------------------------------------------------------------------------------
# return       true / false
#========================================================================================
sub _IsISBN {
    my ($UPCEAN) = @_;

    #remove spaces and - in the code
    $UPCEAN =~ s/-//ig;
    $UPCEAN =~ s/ //ig;

    if (length($UPCEAN) == ISBN_LENGTH) {
        my $isbn = CheckDigits('isbn');
        return $isbn->is_valid($UPCEAN);
    }
    else {
        return 0;
    }
}

sub _IsEnergyLabelRange {
    my $self = shift;
    my ($Product) = @_;

    my $EnergyLabels = $Product->get('EnergyLabels');

    my $CountLabels = @{$EnergyLabels};

    if ($CountLabels > 1) {
        return 1;
    }
    return 0;
}

sub _GetProductUrl {
    my $self = shift;
    my ($Product) = @_;

    my $Url = $Product->getSite()->get('ProtocolAndServer') . $Product->get('WebPath') . '/';

    return $Url;
}



sub writeHeader {
    my $self = shift;

    my @columnNames ;
    foreach my $Name (@{$self->columns}) {
        my $NewName = $hMAP_Idealo_COLUMNNAMES->{$Name};
        if ($Name =~ /^$PREFIX(.*)/ ) {
            $NewName = $self->shippingName($1);
        }
        push @columnNames, $NewName;
    }
    $self->writeLine(\@columnNames );
    return;
}

#========================================================================================
# function     _dlvTimeUnit
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $dlvTimeUnit = $Object->_dlvTimeUnit();
#----------------------------------------------------------------------------------------
# description  getter 'dlvTimeUnit'
#----------------------------------------------------------------------------------------
# return       $_dlvTimeUnit | value of _dlvTimeUnit | String
#========================================================================================
sub _dlvTimeUnit {
    my $self = shift;
    my $FileName = $ENV{EPAGES_CARTRIDGES}.'/DE_EPAGES/Idealo/Templates/MBO/INC_DeliveryDays.txt';
    return ProcessFileTemplate ( $FileName, $self->{'LanguageCode'}, {});
}

#========================================================================================
# function     _isInStock
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_isInStock( $Product );
#----------------------------------------------------------------------------------------
# description  Returns 1 if the product is in stock, 0 if not in stock
#----------------------------------------------------------------------------------------
# input        $Product | Product Object | object
# return       boolean
#========================================================================================)
sub _isInStock {
    my $self = shift;
    my ($Product) = @_;

    my $StockLevel = $Product->getWithParent('StockLevel');
    my $StockLevelAction = $Product->getSite()->get('ProductLowOnStockAction');

    # The stock level could be empty (if the merchant didn't fill it) or above zero
    return 1 unless isValid($StockLevel);
    return 1 if $StockLevelAction == STOCK_IS_ORDERABLE;
    return 1 if $StockLevel > 0;

    return 0;
}

1;
