package TestIdealoProductsExportDriver;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use Test::More;
use Test::Exception;
use DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify;
use Test::Mockify::Matcher qw ( Object );
use Test::Mockify::Verify qw ( WasCalled );
use Test::MockObject;
use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE STOCK_NO_ADDTOBASKET );


use constant PRODUCT_URL => 'https:\\product.url\\';
use constant DESCRIPTION_ATTRIBUTTE_ALIAS => 'ID';
use constant LANGUAGE_ID => 1;

sub Main {
    testgetPriceForPayment();
    testEnergyLabelVersion();
    testEnergyEfficiencyClass();
    testEnergySpectrum();
    testEnergyLabelUrl();
    testDatasheetUrl();
    testEmptyDatasheetUrl();
    testIsProductValidPortal();
    done_testing();
}

sub testgetPriceForPayment {

    my $TestName = (caller(0))[3];
    my $mySelf = _getSelfObject('GBP');

    my $paymentMethod = _getPaymentMethodObject('');
    my $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 0,"$TestName - test behavoir of getPriceForPayment with undef discount mode - 0");

    $paymentMethod = _getPaymentMethodObject(0);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 0,"$TestName - test behavoir of getPriceForPayment without discount or fee - 0");

    $paymentMethod = _getPaymentMethodObject(1);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 0,"$TestName - test behavoir of getPriceForPayment with percentage discount - 0");

    $paymentMethod = _getPaymentMethodObject(2);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 0,"$TestName - test behavoir of getPriceForPayment with fix discount value - 0");

    $paymentMethod = _getPaymentMethodObject(3);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,666);
    is($result, 66.6,"$TestName - test behavoir of getPriceForPayment with percentage of fee - 66.6");

    $paymentMethod = _getPaymentMethodObject(4);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 42,"$TestName - test behavoir of getPriceForPayment with fix fee (GBP)- 42");

    $mySelf = _getSelfObject('EUR');
    $paymentMethod = _getPaymentMethodObject(4);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 23,"$TestName - test behavoir of getPriceForPayment with fix fee (EUR)- 23");

    $paymentMethod = _getPaymentMethodObject(5);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::getPriceForPayment($mySelf,$paymentMethod,10);
    is($result, 0,"$TestName - test behavoir of getPriceForPayment with to high CashDiscountMode - 0");
}


sub testEnergyLabelVersion {

    my $TestName = (caller(0))[3];
    my $mySelf = _getSelfObject();

    my $hProductAttributes = {
        'EnergyLabelHighestEfficiencyClass' => 'A',
        'EnergyLabels' => [],
    };

    my $ProductObject = _getProductObject($hProductAttributes);
    my $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::energyLabelVersion($mySelf, $ProductObject);
    is($result, 1,"$TestName - test behavior of energyLabelVersion of product with HighestEfficiencyClass == A");

    $hProductAttributes->{'EnergyLabelHighestEfficiencyClass'} = 'A+';

    $ProductObject = _getProductObject($hProductAttributes);
    $result = DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver::energyLabelVersion($mySelf, $ProductObject);
    is($result, 0,"$TestName - test behavior of energyLabelVersion of product with HighestEfficiencyClass == A+");
}


sub testEnergyEfficiencyClass {

    my $TestName = (caller(0))[3];

    my $hProductAttributes = {
        'EnergyLabelHighestEfficiencyClass' => 'A',
        'EnergyLabelsString' => 'A - D',
        'EnergyLabels' => [
            {'Value' => 'A'},
            {'Value' => 'D'}
        ]
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    my $result = $SUT->energyEfficiencyClass($ProductObject);
    is($result, '',"$TestName - test if energyLabel is Range then EfficiencyClass is empty");

    $hProductAttributes->{'EnergyLabels'} = [{'Value' => 'A'}];
    $hProductAttributes->{'EnergyLabelsString'} = 'A';

    $ProductObject = _getProductObject($hProductAttributes);
    $result = $SUT->energyEfficiencyClass($ProductObject);

    is($result, 'A',"$TestName - test if energyLabel is Class then EfficiencyClass is correctly filled");
}


sub testEnergySpectrum {

    my $TestName = (caller(0))[3];

    my $hProductAttributes = {
        'EnergyLabelHighestEfficiencyClass' => 'A',
        'EnergyLabelsString' => 'A - D',
        'EnergyLabels' => [
            {'Value' => 'A'},
            {'Value' => 'D'}
        ]
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    my $result = $SUT->energySpectrum($ProductObject);
    is($result, 'A-D',"$TestName - test if energyLabel is Range then Spectrum is correctly filled");

    $hProductAttributes->{'EnergyLabels'} = [{'Value' => 'A'}];
    $hProductAttributes->{'EnergyLabelsString'} = 'A';

    $ProductObject = _getProductObject($hProductAttributes);
    $result = $SUT->energySpectrum($ProductObject);

    is($result, '',"$TestName - test if energyLabel is Class then Spectrum is empty");
}


sub testEnergyLabelUrl {

    my $TestName = (caller(0))[3];

    my $hProductAttributes = {
        'EnergyLabelSourceFile' => 'EnergyLabelSourceFile.pdf'
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    my $ActualResult = $SUT->energyLabelUrl($ProductObject);
    my $ExpectedResult = PRODUCT_URL . $hProductAttributes->{'EnergyLabelSourceFile'};

    ok(WasCalled($SUT, '_GetProductUrl'), "$TestName - test if _GetProductUrl was called");

    is($ActualResult, $ExpectedResult,"$TestName - test if energyLabelUrl is correctly exported");
}

sub testDatasheetUrl {

    my $TestName = (caller(0))[3];

    my $hProductAttributes = {
        'ProductDataSheet' => 'ProductDataSheet.pdf'
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    my $ActualResult = $SUT->datasheetUrl($ProductObject);
    my $ExpectedResult = PRODUCT_URL . $hProductAttributes->{'ProductDataSheet'};

    ok(WasCalled($SUT, '_GetProductUrl'), "$TestName - test if _GetProductUrl was called");
    is($ActualResult, $ExpectedResult,"$TestName - test if datasheetUrl is correctly exported");
}

sub testEmptyDatasheetUrl {

    my $TestName = (caller(0))[3];

    my $hProductAttributes = {
        'ProductDataSheet' => undef
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    my $ActualResult = $SUT->datasheetUrl($ProductObject);

    ok(!WasCalled($SUT, '_GetProductUrl'), "$TestName - test if when undef datasheet _GetProductUrl was NOT called");
    is($ActualResult, '',"$TestName - test if when undef datasheet datasheetUrl is empty");
}

sub testIsProductValidPortal {

    my $TestName = (caller(0))[3];

    my $MockSite = Test::MockObject->new();
    $MockSite->mock('get', sub {
        return STOCK_NO_ADDTOBASKET;
    });

     my $MockCategory = Test::MockObject->new();
    $MockCategory->mock('instanceOf', sub {
        return 1;
    });
    $MockCategory->mock('get', sub {
        return 'Product Category NameOrAlias';
    });
    $MockCategory->mock('instanceOf', sub {
        return 0;
    });

    my $hProductAttributes = {
        'StockLevel' => 3,
        'Site' => $MockSite,
        'Categories' => [$MockCategory],
        'ID' => 123,
        'UPCEAN' => undef,
        'ManufacturerSKU' => 'SKU',
        'IsDownloadProduct' => 1,
    };

    my $ProductObject = _getProductObject($hProductAttributes);

    my $SUT = _createSUT();

    lives_ok(
        sub { $SUT->isProductValidPortal($ProductObject);},
        "$TestName - test that the product is valid"
    );

    $hProductAttributes->{'StockLevel'} = undef;
    $ProductObject = _getProductObject($hProductAttributes);

    lives_ok (
        sub { $SUT->isProductValidPortal($ProductObject);},
        "$TestName - test that stock undef is valid"
    );

    $hProductAttributes->{'StockLevel'} = 0;
    $ProductObject = _getProductObject($hProductAttributes);

    throws_ok (
        sub { $SUT->isProductValidPortal($ProductObject);},
        qr/ProductNoStock/, 
        "$TestName - test that stock 0 is invalid"
    );

    $MockSite = Test::MockObject->new();
    $MockSite->mock('get', sub {
        return STOCK_IS_ORDERABLE;
    });

    $hProductAttributes->{'StockLevel'} = 0;
    $hProductAttributes->{'Site'} = $MockSite;

    $ProductObject = _getProductObject($hProductAttributes);

    lives_ok (
        sub { $SUT->isProductValidPortal($ProductObject);},
        "$TestName - test that stock 0 but STOCK_IS_ORDERABLE works"
    );

}

sub _createSUT {
    my $self = shift;

    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::Idealo::API::CSV::IdealoProductsExportDriver',
        []
    );

    $Mockify->mock('_GetProductUrl')
        ->when(Object('DE_EPAGES::Product::API::Object::Product'))
        ->thenReturn(PRODUCT_URL);

    my $MockDescriptionAttribute = Test::MockObject->new();
    $MockDescriptionAttribute->mock('alias', sub {
        return DESCRIPTION_ATTRIBUTTE_ALIAS;
    });

    $Mockify->mock('descriptionAttribute')
        ->when()
        ->thenReturn($MockDescriptionAttribute);

    $Mockify->mock('isProductValidPortalExt1')
        ->when(Object('DE_EPAGES::Product::API::Object::Product'))
        ->thenReturn(1);

    return $Mockify->getMockObject();
}

sub _getPaymentMethodObject {
    my ($DiscountMode) = @_;
    my $paymentMethodObject = Test::MockObject->new();
    $paymentMethodObject->mock('get', sub {
            my $self = shift;
            my ($ArgumentName) = @_;
            if ($ArgumentName eq 'CashDiscountMode'){
                return $DiscountMode;
            }
            if ($ArgumentName eq 'CashDiscount'){
                return 10;
            }
            if ($ArgumentName eq 'Discounts'){
                return [
                    { CurrencyID=>'EUR', PriceValue=>23 },
                    { CurrencyID=>'GBP', PriceValue=>42 }
                ];
            }
    });
    return $paymentMethodObject;
}


sub _getProductObject {
    my ($hProductAttributes) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Product::API::Object::Product', $hProductAttributes);

    return $Mockify->getMockObject();
}

sub _getSelfObject {
    my ($currencyID) = @_;
    my $selfObject = Test::MockObject->new();
    $selfObject->mock('currencyID', sub {
        return $currencyID;
    });
    return $selfObject;
}

RunScript( Sub => \&Main );