#========================================================================================
# package      DE_EPAGES::Installer::API::SystemInfoReport::SystemInfoReportHandlerDBContent
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for the DB content
#========================================================================================
package DE_EPAGES::Installer::API::SystemInfoReport::SystemInfoReportHandlerDBContent;

use strict;
use warnings;
use diagnostics;
use Time::HiRes qw (time);

use DE_EPAGES::Shop::API::ShopChildOrderHandler qw ( GetShopChildOrderList GetDependencyList );
use DBI;
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Table::Locale qw ( GetAllInfoLocales );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->new( $aFiles, $aStore );
#----------------------------------------------------------------------------------------
# description  saves the report of passed sections in class variables
#----------------------------------------------------------------------------------------
# input        $aFiles | list of all sections | string
# input        $aStore | list of all stores | ref array
#========================================================================================

sub new {
  my $class = shift;
  my ( $aFiles, $aStore, $hOptions ) = @_;
  $hOptions = {} unless defined $hOptions;
  my $self = bless $hOptions, $class;

  #foreach passed db content section
  foreach my $Section (@$aFiles) {

    #get time
    my $time1 = time;

    print "fetching $Section data\n";

    #save info for current section
    $self->_getInfo( $Section, $aStore );

    #get runtime
    my $runtime = time - $time1;

    #save runtime in class variable
    $self->{'runtime'}->{$Section} = sprintf( '%.3f', $runtime );
  }
  return $self;
}

#========================================================================================
# function     _getInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_getInfo( $Section, $aStore );
#----------------------------------------------------------------------------------------
# description  saves the report of the dbcontent section in class variables
#----------------------------------------------------------------------------------------
# input        $Section | subsection, regulate the section to be saved | string
# input        $aStore | list of all stores | ref array
#========================================================================================
sub _getInfo {
    my $self = shift;
    my ( $Section, $aStore ) = @_;
    my @StoresTemp = grep { $_ ne 'Site' } @$aStore;

    if ($Section eq 'ShopChildDependencies') {
        foreach my $Store (@StoresTemp) {
            $self->_getInfoShopChildDependencies($Store);
        }
    }
    elsif ($Section eq 'BCP') {
        foreach my $Store (@$aStore) {
            $self->_getInfoBCPSQL($Store);
        }
    }
    elsif ($Section eq 'MySQLConfig') {
        foreach my $Store (@StoresTemp) {
            $self->_getInfoMySQLConfig($Store);
        }
    }
    elsif ($Section eq 'Locales'){
        foreach my $Store (@$aStore) {
            $self->_getInfoLocales($Store);
        }
    }
}

#========================================================================================
# function     getReport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->getReport( $Section );
#----------------------------------------------------------------------------------------
# description  export the report of the DBStructure
#----------------------------------------------------------------------------------------
# input        $Section | name of section to be exported | string
# return       $hResult | hash containing RND, QA and AM report of passed section as keys | hash ref
#========================================================================================
sub getReport {
  my $self = shift;
  my ($Section) = @_;

  #get result in dependence on section
  my $hReturn = $self->{ $Section . 'Hash' };
  return $hReturn;
}

#========================================================================================
# function     _getInfoShopChildDependencies
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_getInfoShopChildDependencies();
#----------------------------------------------------------------------------------------
# description  saves the report of the ShopChildDependencies
#               for RND shop child order list and dependency list
#               for QA and AM only dependency list
#----------------------------------------------------------------------------------------
# input        $Store | name of store e.g. 'Site' | string
#========================================================================================
sub _getInfoShopChildDependencies {
  my $self = shift;
  my ($Store) = @_;
  my $aOrderList;
  my $Result1;
  my $Result2;
  my $hDependencyList;

  #andere stores einfgen

  RunOnStore(
    Store      => "$Store",
    DBPassword => undef,
    Sub        => sub {

      #call subroutines in DE_EPAGES::Shop::API::ShopChildOrderHandler
      $aOrderList      = GetShopChildOrderList;
      $hDependencyList = GetDependencyList;
    }
  );

  #changes array to hash
  for ( my $i = 0; $i < @$aOrderList; $i++ ) {
    $Result1->{'ShopChildOrderList'}->{ $i + 1 }->{ $aOrderList->[$i] } = 1;
  }
  $Result2->{'DependencyList'} = $hDependencyList;

  #merge Result1 and Result2 to one hash
  my %Result = ( %$Result1, %$Result2 );

  #save shop child order list and dependency list
  $self->{'ShopChildDependenciesHash'}->{'RND'}->{$Store} = \%Result;

  #save dependency list
  $self->{'ShopChildDependenciesHash'}->{'QA'}->{$Store} = $Result2;
  $self->{'ShopChildDependenciesHash'}->{'AM'}->{$Store} = $Result2;
}

#========================================================================================
# function     _getInfoLocales
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_getInfoLocales();
#----------------------------------------------------------------------------------------
# description  saves the report of the Locales
#----------------------------------------------------------------------------------------
# input        $Store | name of store e.g. 'Site' | string
#========================================================================================
sub _getInfoLocales {
    my $self = shift;
    my ($Store) = @_;

    my $aResult;
    my $hResult;

    #run on store
    RunOnStore(
        Store      => "$Store",
        DBPassword => undef,
        Sub        => sub {

            #get all infos for Locales
            $aResult = GetAllInfoLocales();
        }
    );

    #transform array ref to hash ref
    foreach my $hInfoHash (@$aResult){
        my $LocaleID = $hInfoHash->{'LocaleID'};

        $hResult->{$LocaleID} = $hInfoHash;
    }

    #save locales info
    $self->{'LocalesHash'}->{'RND'}->{$Store} = $hResult;
    $self->{'LocalesHash'}->{'QA'}->{$Store} = $hResult;
    $self->{'LocalesHash'}->{'AM'}->{$Store} = $hResult;
}

#========================================================================================
# function     _getInfoBCPSQL
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_getInfoBCPSQL( $StoreName );
#----------------------------------------------------------------------------------------
# description  saves the report of the BCP (country, currency, language, locale)
#----------------------------------------------------------------------------------------
# input        $StoreName | name of store | string
#========================================================================================
sub _getInfoBCPSQL {
  my $self = shift;
  my ($StoreName) = @_;
  my $Dbh;
  my $hResult;
  my $hSQL;

  #save sql statements in hash
  $hSQL->{'country'}  = 'SELECT * FROM country ORDER BY countryid';
  $hSQL->{'currency'} = 'SELECT * FROM currency ORDER BY currencyid';
  $hSQL->{'language'} = <<END_SQL ;
SELECT code3,
    isused,
    code2,
    english,
    french,
    native,
    defaultlocaleid,
    isforbackoffice
FROM language
ORDER BY code3
END_SQL
  $hSQL->{'locale'} = <<END_SQL ;
SELECT lo.localeid,
    l.code3 AS languagecode,
    lo.isused,
    lo.countryid,
    lo.currencyid,
    lo.decimalsep,
    lo.thousandsep,
    lo.negformatnumbers,
    lo.posformatcurrency,
    lo.negformatcurrency,
    lo.listcolumnseparator,
    lo.listrowseparator,
    lo.listquotcharacter,
    lo.defaultcharset,
    lo.firstdayofweek,
    lo.mindaysfirstweek
FROM locale lo
INNER JOIN language l ON l.languageid=lo.languageid
ORDER BY lo.localeid
END_SQL

  if ($StoreName) {
    RunOnStore(
      'Store' => $StoreName,
      'DBPassword' => undef,
      'Sub'   => sub {

        $Dbh = GetCurrentDBHandle()->dbh;

        $Dbh->{PrintError} = 0;
        $Dbh->{RaiseError} = 1;
        $Dbh->{AutoCommit} = 1;

        #for each sql statement
        foreach my $Key ( keys %$hSQL ) {

          #get result array
          my $aTempArray = $self->_doSQL( $Dbh, $hSQL->{$Key} );

          #save result array with identifier in hash
          $hResult->{$Key} = $aTempArray;
        }
      }
    );
  }
  else { die 'No store name'; }

  #save result in class variable
  $self->{'BCPHash'}->{'RND'}->{$StoreName} = $hResult;
  $self->{'BCPHash'}->{'QA'}->{$StoreName}  = $hResult;
  $self->{'BCPHash'}->{'AM'}->{$StoreName}  = $hResult;

}

#========================================================================================
# function     _getInfoMySQLConfig
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_getInfoMySQLConfig( $StoreName );
#----------------------------------------------------------------------------------------
# description  export the report of the MySQL information
#----------------------------------------------------------------------------------------
# input        $StoreName | name of store | string
# return       $hResult | hash containing variable name as key and variable value as value
#                          key = Column_Names returns column names  | ref hash
#========================================================================================
sub _getInfoMySQLConfig {
  my $self = shift;
  my ($StoreName) = @_;
  my $Dbh;
  my %Result;

  #save sql statements in a string
  my $Sql = <<END_SQL ;
SHOW VARIABLES
WHERE Variable_name NOT LIKE "%dir"
  AND Variable_name NOT LIKE "%file"
  AND Variable_name NOT IN ("language", "log_error", "hostname")
go
END_SQL

  if ($StoreName) {
    RunOnStore(
      'Store' => $StoreName,
      'DBPassword' => undef,
      'Sub'   => sub {
        $Dbh = GetCurrentDBHandle()->dbh;

        $Dbh->{PrintError} = 0;
        $Dbh->{RaiseError} = 1;
        $Dbh->{AutoCommit} = 1;

        #get result array
        my $hTempHash = $self->_mySQLSeperate( $Dbh, $Sql );

        #transform result to a hash
        @Result{ keys %$hTempHash } = values %$hTempHash;
      }
    );
  }
  else { die 'No store name'; }

  #save rnd result
  $self->{'MySQLConfigHash'}->{'RND'} = \%Result;

  #create deep copy
  use Storable 'dclone';
  my $hResult = dclone \%Result;

  #edit result to fit QA and AM needs
  #delete irrelevant information
  delete $hResult->{'pseudo_thread_id'};
  delete $hResult->{'timestamp'};

  #save QA result
  $self->{'MySQLConfigHash'}->{'QA'} = $hResult;

  #save AM result
  $self->{'MySQLConfigHash'}->{'AM'} = $hResult;
}

#========================================================================================
# function     _mySQLSeperate
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_mySQLSeperate( $Dbh, $Sql );
#----------------------------------------------------------------------------------------
# description  seperates SQL command at go and executes every seperated command
#----------------------------------------------------------------------------------------
# input        $Dbh | database handle | object
# input        $Sql | SQL command to be seperated | string
# return       $hResult | hash containing variable name as key and variable value as value
#                          key = Column_Names returns column names  | ref hash
#========================================================================================
sub _mySQLSeperate {

  #(original: DoFile
  # epages/Cartridges/DE_EPAGES/Database/Scripts/dbi_shell.pl)
  my $self = shift;
  my ( $Dbh, $Sql ) = @_;
  my %Result;

  #split all sql statements at every go
  foreach my $Command (
    split /\ngo\s*\n/i,
    do { local $/ = undef; $Sql }
  ) {

    #execute sql command
    my $hTempHash = $self->_doSQLHash( $Dbh, $Command );
    @Result{ keys %$hTempHash } = values %$hTempHash;
  }

  return \%Result;
}

#========================================================================================
# function     _doSQL
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_doSQL( $Dbh, $Sql );
#----------------------------------------------------------------------------------------
# description  executes SQL command and returns result as a two dimensional array
#----------------------------------------------------------------------------------------
# input        $Dbh | database handle | object
# input        $Sql | SQL command to be seperated | string
# return       $aResult | two-dimensional array containing sql table | ref array
#========================================================================================
sub _doSQL {

  #epages/Cartridges/DE_EPAGES/Database/Scripts/dbi_shell.pl
  my $self = shift;
  my ( $Dbh, $Sql ) = @_;

  #remove comments
  $Sql =~ s/--.*?(\n|$)/$1/g;

  my $Sth = $Dbh->prepare($Sql);
  $Sth->execute;

  my $aResult = $self->_showResultSetAsTwoDimArray($Sth);
  $Sth->finish;
  return $aResult;
}

#========================================================================================
# function     _doSQLHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_doSQLHash( $Dbh, $Sql );
#----------------------------------------------------------------------------------------
# description  executes SQL command and returns result as a hash
#----------------------------------------------------------------------------------------
# input        $Dbh | database handle | object
# input        $Sql | SQL command to be seperated | string
# return       $hResult | hash containing variable name as key and variable value as value
#                          key = Column_Names returns column names  | ref hash
#========================================================================================
sub _doSQLHash {

  #epages/Cartridges/DE_EPAGES/Database/Scripts/dbi_shell.pl
  my $self = shift;
  my ( $Dbh, $Sql ) = @_;

  #remove comments
  $Sql =~ s/--.*?(\n|$)/$1/g;

  my $Sth = $Dbh->prepare($Sql);
  $Sth->execute;

  my $hResult = $self->_showResultSetAsHash($Sth);
  $Sth->finish;
  return $hResult;
}

#========================================================================================
# function     _showResultSetAsTwoDimArray
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_showResultSetAsTwoDimArray( $Sth );
#----------------------------------------------------------------------------------------
# description  returns SQL result converted in a two dimensional array
#----------------------------------------------------------------------------------------
# input        $Sth | DBI statement handle | object
# return       $aResult | two-dimensional array containing sql table | ref array
#========================================================================================
sub _showResultSetAsTwoDimArray {
  my $self = shift;
  my ($Sth) = @_;
  my @Result;

  #return . if statemend is empty
  if ( 0 == $Sth->{NUM_OF_FIELDS} ) {
    push @Result, ".\n";
    return \@Result;
  }

  local $, = ',';

  #get column names
  my $aColumns = $Sth->{NAME};

  #add names to array
  push @{ $Result[0] }, @$aColumns;

  #fetch all rows
  my $aValues;
  my $i = 1;

  #save rows in an array
  while ( $aValues = $Sth->fetchrow_arrayref ) {

    #every line is an array of single values
    my @Line;

    #for each column
    foreach my $Index ( 0 .. scalar(@$aColumns) - 1 ) {

      $aValues->[$Index] = '' unless (defined $aValues->[$Index]);

      #get value at position $Index
      my $Value = $aValues->[$Index];

      #if value contains whitespaces or quotation marks or commas
      if ( $Value =~ /\s/ || $Value =~ /\"/ || $Value =~ /\,/ ) {

        #replace single quotation mark with two quotation marks
        $Value =~ s/\"/\"\"/g;

        #put value in quotation marks
        $Value = '"' . $Value . '"';
      }

      #fill current line with values
      push @Line, $Value;
    }

    #fill result array with lines
    push @{ $Result[$i] }, @Line;
    $i = $i + 1;
  }
  return \@Result;
}

#========================================================================================
# function     _showResultSetAsHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       $DBContentReportHandler->_showResultSetAsHash( $Sth );
#----------------------------------------------------------------------------------------
# description  returns SQL result converted in a two dimensional array
#----------------------------------------------------------------------------------------
# input        $Sth | DBI statement handle | object
# return       $hResult | hash containing variable name as key and variable value as value
#                          key = Column_Names returns column names  | ref hash
#========================================================================================
sub _showResultSetAsHash {
  my $self = shift;
  my ($Sth) = @_;
  my %Result;

  #return if statemend is empty
  if ( 0 == $Sth->{NUM_OF_FIELDS} ) {
    return \%Result;
  }

  local $, = ',';

  #get column names
  my $aColumns = $Sth->{NAME};

  #add names to hash
  $Result{'Column_Names'} = "$aColumns->[0],$aColumns->[1]";

  #fetch all rows
  my $aValues;

  #save rows in hash
  while ( $aValues = $Sth->fetchrow_arrayref ) {

    #every line is an array of single values
    my @Line;

    #for each column
    foreach my $Index ( 0 .. scalar(@$aColumns) - 1 ) {

      #get value at position $Index
      my $Value = $aValues->[$Index];

      #if value contains whitespaces or quotation marks or commas
      if ( $Value =~ /\s/ || $Value =~ /\"/ || $Value =~ /\,/ ) {

        #replace single quotation mark with two quotation marks
        $Value =~ s/\"/\"\"/g;

        #put value in quotation marks
        $Value = '"' . $Value . '"';
      }

      #fill current line with values
      push @Line, $Value;
    }

    #fill result hash
    $Result{ $Line[0] } = $Line[1];

  }
  return \%Result;
}

1;
