#========================================================================================
# package      DE_EPAGES::Installer::API::SystemInfoReport::SystemInfoReportHandlerFiles
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for the files
#========================================================================================
package DE_EPAGES::Installer::API::SystemInfoReport::SystemInfoReportHandlerFiles;

use strict;
use warnings;
use Time::HiRes qw ( time );

use DE_EPAGES::Core::API::File qw ( CompareMD5File CopyDirectory ExistsDirectory
    ExistsFile GetFileContent GetFileMD5Digest RunOnDirectory CopyFile );
use DE_EPAGES::Core::API::CartridgeConfig qw ( CartridgeConfig );
use DE_EPAGES::Database::API::Config qw ( StoreType );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw ( GetStoreTypeCartridges );
use Digest::MD5 qw ( md5_hex );
use File::Basename;
use File::Find;
use Cwd;

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles = DE_EPAGES::Installer::API::SystemInfoReport::SystemInfoReportHandlerFiles->new( $aFiles, $OutputFolder, $aStore );
#----------------------------------------------------------------------------------------
# description  saves the report of passed sections in class variables
#----------------------------------------------------------------------------------------
# input        $aFiles | list of all sections | string
# input        $OutputFolder | target folder for output | ref array
# input        $aStore | list of all stores | ref array
# input        $hOptions | options | ref hash
# return       $SystemInfoReportHandlerFiles | system report handler object | object
#========================================================================================
sub new {
    my $class = shift;
    my ( $aFiles, $OutputFolder, $aStore, $hOptions ) = @_;
    $hOptions = {} unless defined $hOptions;
    my $self = bless $hOptions, $class;

    #foreach passed db structure section
    foreach my $Section (@$aFiles) {

      #get time
      my $time1 = time;

      print "fetching $Section data\n";

      #save info for current section
      $self->_getInfo( $Section, $OutputFolder, $aStore );

      #get runtime
      my $runtime = time - $time1;

      #save runtime in class variable
      $self->{'runtime'}->{$Section} = sprintf( '%.3f', $runtime );
    }
    return $self;
}

#========================================================================================
# function     _getInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_getInfo( $Section, $OutputFolder, $aStore );
#----------------------------------------------------------------------------------------
# description  saves the report of the file section in class variables
#----------------------------------------------------------------------------------------
# input        $Section | subsection, regulate the section to be saved | string
# input        $OutputFolder | target folder for output | string
# input        $aStore | list of all stores | ref array
#========================================================================================
sub _getInfo {
    my $self = shift;
    my ( $Section, $OutputFolder, $aStore ) = @_;
    my $SourceFolder;

    #switch through section to set sourcefolder
    if ($Section eq 'Cartridges') {
      $SourceFolder = $ENV{'EPAGES_CARTRIDGES'};
    }
    elsif ($Section eq 'Perl') {
      $SourceFolder = $ENV{'EPAGES_PERL'};
    }
    elsif ($Section eq 'RequestRouter') {
      $SourceFolder = $ENV{'EPAGES'} . "/$Section";
    }
    elsif ($Section eq 'Service') {
      $SourceFolder = $ENV{'EPAGES'} . "/$Section";
    }
    elsif ($Section eq 'Stores') {
      $SourceFolder = $ENV{'EPAGES_STORES'};
    }
    elsif ($Section eq 'WebAdapter') {
      $SourceFolder = $ENV{'EPAGES'} . "/$Section";
    }
    elsif ($Section eq 'WebRoot') {
      $SourceFolder = $ENV{'EPAGES_WEBROOT'};
    }
    elsif ($Section eq 'CTMPLFolder') {
      $SourceFolder = $ENV{'EPAGES_STATIC'} . '/Templates.d';
    }
    elsif ($Section eq 'var-epages') {
      $SourceFolder = '/var/epages';
    }
    elsif ($Section eq 'j'){
      $SourceFolder = $ENV{'EPAGESJ'};

      #copy config files to tempfolder
      my $TempFolder = "$OutputFolder/Temp/Files/$Section";
      CopyDirectory( "$SourceFolder/$_", "$TempFolder/$_" ) foreach qw(conf solr/conf);
      RunOnDirectory($SourceFolder,
                sub { CopyFile($_, "$TempFolder/$_", {create_directory => 1})
                          foreach glob 'webapps/*.version';
                     }
      );
    }

    elsif ($Section eq 'Config') {

      #set outputfolder as sourcefolder
      $SourceFolder = "$OutputFolder/Temp/Files/$Section";

      #copy directory to outputfolder
      CopyDirectory( $ENV{'EPAGES'} . "/Shared/$Section", $SourceFolder );
    }

    elsif ($Section eq 'Monitor') {

      #set outputfolder as sourcefolder
      $SourceFolder = "$OutputFolder/Temp/Files/$Section";

      #copy directory to outputfolder
      CopyDirectory( $ENV{'EPAGES'} . "/Shared/$Section", $SourceFolder );
    }

    elsif ($Section eq 'DiffCartridgesWebRoot') {
      my $haResult;

      #for each store
      foreach my $Store (@$aStore) {

        #store points to array
        $haResult = $self->_diffCartridgesWebRoot( $Store, 1 );
        $self->{ "$Section" . 'Hash' }->{'RND'}->{$Store} = $haResult;
        $self->{ "$Section" . 'Hash' }->{'QA'}->{$Store}  = $haResult;
        $self->{ "$Section" . 'Hash' }->{'AM'}->{$Store}  = $haResult;

      }
      return;
    }

    else {
        die "no valid section '$Section'\n";
    }

    return unless -d $SourceFolder;

    my $aFiles;
    RunOnDirectory(
        $SourceFolder,
        sub {
            #find all files fast and sorted
            find(
                {
                    wanted => sub {
                        push @$aFiles, $File::Find::name if -f $_;
                    },
                    preprocess => sub {
                        return ( sort @_ );
                    },
                },
                ($SourceFolder)
            );
        }
    );

    #counter to save position
    my $count = -1;

    #replace all backslashes with forward slashes
    $SourceFolder =~ s/\\/\//go;

    #generate fingerprint for each file
    foreach my $Filename (@$aFiles) {

        #skip no longer existing files (e.g. temporary files)
        next unless -e $Filename;

        #replace all backslashes with forward slashes
        $Filename =~ s/\\/\//go;

        #set empty default value for windows
        my $Perm = '';
        if ($^O ne 'MSWin32') {
            my ( $Mode, $Uid, $Gid ) = ( lstat $Filename )[ 2, 4, 5 ];
            $Perm = $self->_getPermissions($Mode). ' '. (getpwuid($Uid))[0].
                ' '. (getgrgid($Gid))[0]. ' ';
        }

        #create fingerprint for filename
        my $Fingerprint = $self->_fingerprint($Filename);
        chomp($Fingerprint);

        #delete sourcefolder part from path
        my $Path = $Filename;
        $Path =~ s,\Q$SourceFolder\E/,,;

        #increase position counter
        $count++;

        #save as hash RND and AM not in QA
        $self->{ "$Section" . 'Hash' }->{'RND'}->{$count}->{$Path} = "$Perm$Fingerprint";
        $self->{ "$Section" . 'Hash' }->{'AM'}->{$count}->{$Path}  = "$Perm$Fingerprint";

        #skip Perl exceptions
        next if $Section eq 'Perl' and $Filename =~ m,\.obsolete$,;
        next if $Section eq 'Perl' and $Filename =~ m,/\.meta/,;

        #skip Cartridges exceptions
        next if $Section eq 'Cartridges' and $Filename =~ m,makefile$,;

        #save as hash QA
        $self->{ "$Section" . 'Hash' }->{'QA'}->{$count}->{$Path} = "$Perm$Fingerprint";
    }
}

use Fcntl ':mode';

#========================================================================================
# function     _getPermissions
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_getPermissions( $Mode );
#----------------------------------------------------------------------------------------
# description  translates numeric permissions to string
#----------------------------------------------------------------------------------------
# input        $Mode | permission of a file in numeric code | integer
# return       $Permissions | permission of a file in string | string
#========================================================================================
sub _getPermissions {
  my $self   = shift;
  my ($Mode) = @_;

  my $Result = '';

  #directory
  if (($Mode & S_IFMT) == S_IFDIR) { $Result = 'd' }

  #character special
  elsif (($Mode & S_IFMT) == S_IFCHR) { $Result = 'c' }

  #block special
  elsif (($Mode & S_IFMT) == S_IFBLK) { $Result = 'b' }

  #regular
  elsif (($Mode & S_IFMT) == S_IFREG) { $Result = '-' }

  #symbolic link
  elsif (($Mode & S_IFMT) == S_IFLNK) { $Result = 'l' }

  #socket
  elsif (($Mode & S_IFMT) == S_IFSOCK) { $Result = 's' }

  #fifo
  elsif (($Mode & S_IFMT) == S_IFIFO) { $Result .= 'p' }

  #unknown
  else { $Result .= '?' }

  my $Owner;
  #user part
  $Result .= ( $Mode & S_IRUSR ) ? 'r' : '-';
  $Result .= ( $Mode & S_IWUSR ) ? 'w' : '-';
  $Owner = $Mode & ( S_IXUSR | S_ISUID );
  if ($Owner == 0)                      { $Result .= '-' }
  elsif ($Owner == S_IXUSR)             { $Result .= 'x' }
  elsif ($Owner == S_ISUID)             { $Result .= 'S' }
  elsif ($Owner ==  (S_IXUSR | S_ISUID) ) { $Result .= 's' }

  #group part
  $Result .= ( $Mode & S_IRGRP ) ? 'r' : '-';
  $Result .= ( $Mode & S_IWGRP ) ? 'w' : '-';
  $Owner = $Mode & ( S_IXGRP | S_ISGID );
  if ($Owner == 0)                      { $Result .= '-' }
  elsif ($Owner == S_IXGRP)             { $Result .= 'x' }
  elsif ($Owner == S_ISGID)             { $Result .= 'S' }
  elsif ($Owner ==  (S_IXGRP | S_ISGID) ) { $Result .= 's' }

  #other part
  $Result .= ( $Mode & S_IROTH ) ? 'r' : '-';
  $Result .= ( $Mode & S_IWOTH ) ? 'w' : '-';
  $Owner = $Mode & ( S_IXOTH | S_ISVTX );
  if ($Owner == 0)                      { $Result .= '-' }
  elsif ($Owner == S_IXOTH)             { $Result .= 'x' }
  elsif ($Owner == S_ISVTX)             { $Result .= 'T' }
  elsif ($Owner ==  (S_IXOTH | S_ISVTX) ) { $Result .= 't' }

  return $Result;
}

#========================================================================================
# function     getReport
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->getReport( $Section );
#----------------------------------------------------------------------------------------
# description  exports the report of the passed file section
#----------------------------------------------------------------------------------------
# input        $Section | subsection, regulate the type of output | string
# return       $hResult | hash containing RND, QA and AM report of passed section as keys | hash ref
#========================================================================================
sub getReport {
  my $self = shift;
  my ($Section) = @_;

  #get result in dependence on section
  my $hReturn = $self->{ $Section . 'Hash' };

  return $hReturn;
}

#========================================================================================
# function     _fingerprint
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_fingerprint( $FileName );
#----------------------------------------------------------------------------------------
# description  convert windows newlines '\r\n' to UNIX newlines '\n', calutate MD5sum
#----------------------------------------------------------------------------------------
# input        $FileName | path to file | string
# return       md5_hex($$rContent) | MD5 value in hexadecimal format | integer
#========================================================================================
sub _fingerprint {

  #Checkout/Patch/PatchDiff.fingerprint.pl
  my $self = shift;
  my ($FileName) = @_;

  #read file
  my $rContent = GetFileContent($FileName);

  #convert all windows newlines to unix ones
  #only if file is not binary
  $$rContent =~ s/\r\n/chr(10)/ges if -T $FileName;

  #create md5 for fileconten in hex
  return md5_hex($$rContent);
}

#========================================================================================
# function     _diffCartridgesWebRoot
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_diffCartridgesWebRoot( $StoreName, $NoOutput, $LanguageCodes );
#----------------------------------------------------------------------------------------
# description  finds all pairs of files in webroot and cartridges which differ,
#               transforms passed language codes
#----------------------------------------------------------------------------------------
# input        $StoreName | name of store to use for webroot | string
# input        $NoOutput | enables/disables printing errors only | boolean
# input        $LanguageCodes | comma separated list, ignores files of this languages e.g. en,de | string
# return       $aResult | array containing all files that differ | ref array
#========================================================================================
sub _diffCartridgesWebRoot {
  my $self = shift;
  my ( $StoreName, $NoOutput, $LanguageCodes ) = @_;

  $LanguageCodes = '' unless defined $LanguageCodes;

  #get list of cartridges of the passed store
  my $haCartridges = GetStoreTypeCartridges( StoreType($StoreName) );

  #transforms passed language codes, for example de,en to \.de\.|\.en\.
  my $RegExpr = join '|', map { '\.' . $_ . '\.' } split( /,/, $LanguageCodes );

  #if language code is empty set $RegExpr to undef
  $RegExpr = undef if $RegExpr eq '';

  #return ref array, get result array
  return $self->_testStore( $StoreName, $haCartridges, $RegExpr, $NoOutput );
}

#========================================================================================
# function     _testStore
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_testStore($StoreName, $aCartridges, $RegExpr, $NoOutput);
#----------------------------------------------------------------------------------------
# description  finds all pairs of files in webroot and cartridges which differ
#----------------------------------------------------------------------------------------
# input        $StoreName | name of store to use for webroot | string
# input        $aCartridges | list of cartridges | ref.array
# input        $RegExpr | regular expression to find file | string
# input        $NoOutput | enables/disables printing errors only | boolean
# return       $aResult | array containing all files that differ | ref array
#========================================================================================
sub _testStore {
  my $self = shift;
  my ( $StoreName, $aCartridges, $RegExpr, $NoOutput ) = @_;
  my @Types = qw ( Public Private WebRoot javascript );
  my @Result;

  #get md5 sum for cartridge files
  my $hDigests = $self->_collectMD5FromCartridges( $aCartridges, \@Types );

  #get store type
  my $StoreType = StoreType($StoreName);

  #get version
  my $Version = CartridgeConfig($StoreName)->get( 'Cartridges', 'DE_EPAGES::Core' );
  die "Can't get ePages version from Cartridges.d/$StoreName.conf" unless defined $Version;

  #for each type
  foreach my $Type (@Types) {
    my $BaseDirectory;

    #set base directory in dependence on type
    if ( $Type eq 'Public' ) {
      $BaseDirectory = join '/', $ENV{'EPAGES_WEBROOT'}, 'StoreTypes', $Version, $StoreType;
    }
    elsif ( $Type eq 'Private' ) {
      $BaseDirectory = join '/', $ENV{'EPAGES_STORES'}, $StoreName;
    }
    elsif ( $Type eq 'WebRoot' ) {
      $BaseDirectory = join '/', $ENV{'EPAGES_WEBROOT'};
    }
    elsif ( $Type eq 'javascript' ) {
      $BaseDirectory = join '/', $ENV{'EPAGES_WEBROOT'}, 'StoreTypes', $Version, $StoreType, 'javascript/epages/cartridges';
    }

    push @Result, sprintf "\ncompare %d $Type files.\n", scalar keys %{ $hDigests->{$Type} } unless $NoOutput;
    my $Counter = 0;

    #for each file of current type
    foreach my $Name ( sort keys %{ $hDigests->{$Type} } ) {

      $Counter++;

      #set filename
      my $FileName
      = $Type eq 'javascript'
      ? $BaseDirectory . '/' . lc( $hDigests->{$Type}{$Name}{'CartridgeDir'} ) . $Name
      : $BaseDirectory . $Name;

      #case 1: file does not exist
      if ( not ExistsFile($FileName) ) {
        push @Result, sprintf "\r%s not exists. CartridgeFile: %s \n", $FileName, $hDigests->{$Type}{$Name}{'File'}
            if !defined $RegExpr || $FileName !~ /($RegExpr)/;##no critic qw(RegularExpressions::ProhibitUnusedCapture)
      }

      #case 2: file 1 is not equal to file 2
      elsif ( not CompareMD5File( $FileName, $hDigests->{$Type}{$Name}{'MD5'} ) ) {
        push @Result, sprintf "\r%s not the same like %s\n", $FileName, $hDigests->{$Type}{$Name}{'File'};
      }

      #case 3: file 1 is equal to file 2
      else {
        push @Result, sprintf "\r$Counter" unless $NoOutput;
      }
    }
  }
  print "\r";
  return \@Result;
}

#========================================================================================
# function     _collectMD5FromCartridges
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SystemInfoReportHandlerFiles->_collectMD5FromCartridges( $aCartridges, $aTypes );
#----------------------------------------------------------------------------------------
# description  returns a hash of hashes, the first hash contains the cartridge files as keys,
#               the second hash contains the md5 sum, file name and cartridge dir as values
#----------------------------------------------------------------------------------------
# input        $aCartridges | list of cartridge packages | ref array
# input        $aTypes | list of types | ref array
# return       $hResult | result hash:
# type => {
#         target1 => {
#                     'MD5' => md5 sum
#                     'File' => file name
#                     'CartridgeDir' => current directory
#                     }
#        } | ref hash
#========================================================================================
sub _collectMD5FromCartridges {
  my $self = shift;
  my ( $aCartridges, $aTypes ) = @_;
  my %Digests;

  #for each cartridge package
  foreach my $Package ( map { s/::/\//g; $_ } @$aCartridges ) {

    #for each type
    foreach my $Type (@$aTypes) {

      #set current directory path
      my $Directory = join '/', $ENV{'EPAGES_CARTRIDGES'}, $Package, 'Data', $Type;
      if ( ExistsDirectory($Directory) ) {

        #finds all files and directorys in $Directory. For each file or directory found, calls the passed subroutine.
        find(
          sub {

            #return if current path is not a file
            return unless -f $File::Find::name;

            #skip all cvs files
            return if $File::Find::name =~ /\/CVS\//;

            #set target name to current path
            my $TargetName = $File::Find::name;

            #delete path to the file
            $TargetName =~ s/\Q$Directory\E//;

            #save data in result hash
            $Digests{$Type}{$TargetName}{'MD5'}          = GetFileMD5Digest($File::Find::name);
            $Digests{$Type}{$TargetName}{'File'}         = $File::Find::name;
            $Digests{$Type}{$TargetName}{'CartridgeDir'} = $Package;
          },
          $Directory
        );
      }
    }
  }
  return \%Digests;
}

1;
