use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw (
    DeleteFile
    GetFileContent
    GetFileContentLines
    GetSubDirectories
    ExistsDirectory
    ExistsFile
    GetFileNames
    RelativePath
);
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Object::API::SQLStructureImportHandler;
my $CURRENT_WORKING_DIR = File::Spec->rel2abs(File::Spec->curdir());
my $ERR_OUT_FILE = File::Spec->catfile($CURRENT_WORKING_DIR, 'command.err');
my $STD_OUT_FILE = File::Spec->catfile($CURRENT_WORKING_DIR, 'command.out');

## no critic qw(ProhibitDeepNests)

sub Main {

    local $| = 1;

    my $NewReleaseVersion;
    my $OldReleaseVersion;
    my $OldReleaseTag;
    my $NewReleaseTag;
    my $Help;
    my $New;
    my $Old;
    my $FolderUnpackedPatch;

    my $NewReleaseReached = 0;
    my $OldReleaseExists = 0;

    GetOptions(
        'help'    => \$Help,
        'new=s'   => \$New,
        'old=s'   => \$Old,
        'unpackedpatch=s'    => \$FolderUnpackedPatch,
    );

    usage() if $Help;

    unless( defined $Old) {
        print STDERR "missing old version.\n\n";
        usage();
    }

    unless( defined $New) {
        print STDERR "missing new version.\n\n";
        usage();
    }

    unless( defined $FolderUnpackedPatch) {
        print STDERR "missing path of unpacked patch.\n\n";
        usage();
    }

    unless( ExistsDirectory($FolderUnpackedPatch)) {
        print STDERR "path of unpacked patch doesnt exists.\n\n";
        usage();
    }

    $NewReleaseVersion = 'release_' . $New;
    $OldReleaseVersion = 'release_' . $Old;
    $OldReleaseTag = 'release_' . $Old;
    $NewReleaseTag = 'release_' . $New;

    my $time1 = time;

    my $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES';
    my $aSubDirectories;
    $aSubDirectories = GetSubDirectories($CartridgeDir) if ExistsDirectory($CartridgeDir);

    my $XMLStructImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    my $hResult = {};

    foreach my $CartridgeName (@$aSubDirectories){#for every CartridgeFolder in Cartridge Directory
        next if ($CartridgeName =~ /CVS/);#ignore cvs folders

        my $PatchDir = $CartridgeName . '/Patches';
        my $aPatchDirectories;
        #get all folders under CartridgeXY/Patches/..
        $aPatchDirectories = GetSubDirectories($PatchDir) if ExistsDirectory($PatchDir);

        my $TempOldRelease = $OldReleaseVersion;

        foreach my $PatchFolder (@$aPatchDirectories){#for each folder under CartridgeXY/Patches/..

            next if ($PatchFolder =~ /CVS/);#ignore cvs folders
            next if ! ExistsFile($PatchFolder . '/patch.xml');#empty patch did nothing
            next if $NewReleaseReached;

            #build xml structure of patch.xml file to look what is the new release
            my $PatchXMLStruct = $XMLStructImportHandler->getXMLStructure( $PatchFolder . '/patch.xml' );

            #start with old release tag, patch could have steps like release_6_0_8_pre and release_6_0_8
            if ($PatchXMLStruct->getIDValue("/PatchCartridge{OldRelease->$TempOldRelease}", 'OldRelease') eq $TempOldRelease){
                #patch.xml is for patch from old release
                if ($PatchXMLStruct->getIDValue("/PatchCartridge{OldRelease->$TempOldRelease}", 'OldRelease') eq $OldReleaseVersion){
                    $OldReleaseExists = 1;
                    print "\nstarting at " . $PatchFolder . "/patch.xml\n";
                }
                else{
                    print 'continue at ' . $PatchFolder . "/patch.xml\n";
                };

                #new release is reached
                if ($NewReleaseVersion eq $PatchXMLStruct->getTagAttribute("/PatchCartridge{OldRelease->$TempOldRelease}", 'NewRelease')){
                    $NewReleaseReached = 1;
                }

                #$NewReleaseReached = 1 if ($NewReleaseVersion eq $PatchXMLStruct->getIDValue("/PatchCartridge{OldRelease->$TempOldRelease}", 'NewRelease'));

                my $CheckDir = $PatchFolder . '/Database/XML';

                my $Cartridge = $CartridgeName;
                $Cartridge =~ s/^.*\/\QDE_EPAGES\E\///;

                #get infos from Patch..\Database\XML\ files
                CollectInfosFromPatchFiles($hResult,$CheckDir);

                #get infos from patch..\patch.xml
                CollectInfosFromPatchXML($hResult, $PatchXMLStruct,$Cartridge);

                #register the new "old release tag"
                $TempOldRelease = $PatchXMLStruct->getTagAttribute("/PatchCartridge{OldRelease->$TempOldRelease}",'NewRelease');

                my $CheckSQLDir = $PatchFolder . '/Database/mysql/Tables';
                #get infos from Patch..\Database\mysql\ files
                CollectInfosFromPatchSQLFiles($hResult,$CheckSQLDir);
            };
        }
        print "\nERROR: release tag is not reached for $CartridgeName\n" if (not $NewReleaseReached and $OldReleaseExists);
        print "complete for $CartridgeName\n" if ($NewReleaseReached and $OldReleaseExists);
        print "\nno patch changes found for $CartridgeName\n" if (not $NewReleaseReached and not $OldReleaseExists);
        $NewReleaseReached = 0;
        $OldReleaseExists = 0;
    }

    #register deleted or added API functions
    RegisterAPIChanges($hResult,$OldReleaseTag,$NewReleaseTag);

    #register added or changed Template Files
    RegisterAddedOrChangedTemplateFiles($hResult, $FolderUnpackedPatch);

    #write changes to file
    WriteChangesToFile($hResult,"PatchChanges_Patch_$New.txt");

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub RegisterAPIChanges{
    my ($hResult,$OldReleaseTag,$NewReleaseTag) = @_;

    #get changes for API functions
    use Cwd;
    my $Dir = getcwd;
    my $RelativeFilePath = RelativePath($Dir, $ENV{'EPAGES_CARTRIDGES'} . '\DE_EPAGES\XML\Scripts\findUseProblems.dat'); ##no critic (RequireInterpolationOfMetachars)

    #update file from old release tag
    UpdateFile($RelativeFilePath,$OldReleaseTag);
    #get old API function list
    my $hOldAPIList = GetAPIList($RelativeFilePath);

    #update file from new release tag
    UpdateFile($RelativeFilePath,$NewReleaseTag);
    #get new API function list
    my $hNewAPIList = GetAPIList($RelativeFilePath);

    #register changes in API functions
    my $FirstLevel = 'API-functions';

    foreach my $Package (sort keys %$hOldAPIList){
        foreach my $Function (sort keys %{$hOldAPIList->{$Package}}){
            if (not exists $hNewAPIList->{$Package}->{$Function}){
                $hResult->{$FirstLevel}->{'Deleted'}->{$Package . '::' . $Function} = 1;
            }
        }
    }
    foreach my $Package (sort keys %$hNewAPIList){
        foreach my $Function (sort keys %{$hNewAPIList->{$Package}}){
            if (not exists $hOldAPIList->{$Package}->{$Function}){
                $hResult->{$FirstLevel}->{'Added'}->{$Package . '::' . $Function} = 1;
            }
        }
    }
}

#help function to get the variable saved in '\DE_EPAGES\XML\Scripts\findUseProblems.dat'
sub GetAPIList{
    my ($FileName) = @_;
    my $hResult;

    #read file content make variable visible
    my $rFunctions1;
    my $rPackages1;## no critic qw(ProhibitUnusedVariables)
    my $rContent = GetFileContent($FileName);
    eval $$rContent or die $@;## no critic qw(ProhibitStringyEval)

    #generate  result hash
    foreach my $Function (keys %$rFunctions1 ){
        my $aTemp = $rFunctions1->{$Function};
        foreach my $Package (@$aTemp){
            $hResult->{$Package}->{$Function} = 1;
        }
    }

    return $hResult;
}

#help function to execute a cvs command
sub systemCommand ($;$) {
    my ($Command, $hOptions) = @_;
    print "$Command\n";
    my %out;
    $out{'return'}      = system "$Command 1>$STD_OUT_FILE 2>$ERR_OUT_FILE";
    $out{'outlines'}    = GetFileContentLines($STD_OUT_FILE) if $hOptions->{'outlines'};
    $out{'errlines'}    = GetFileContentLines($ERR_OUT_FILE) if $hOptions->{'errlines'};
    $out{'outcontent'}  = GetFileContent($STD_OUT_FILE) if $hOptions->{'outcontent'};

    $out{'errcontent'}  = GetFileContent($ERR_OUT_FILE);
    die "command failed: $out{'return'} ${$out{'errcontent'}}" if $out{'return'} && !$hOptions->{'skiperrors'};
    print ${$out{'errcontent'}} if ${$out{'errcontent'}};

    DeleteFile($STD_OUT_FILE);
    DeleteFile($ERR_OUT_FILE);

    return \%out;
}

#help function updates a file from cvs by given tag
sub UpdateFile ($$) {
    my ($File, $Tag) = @_;
    my $cvs_update = "cvs update -d -P -r $Tag";
    systemCommand ("$cvs_update $File");
    return;
}

#gets deleted templates listed in Patch../patch.xml
sub CollectInfosFromPatchXML {
    my ($hResult,$PatchXMLStruct,$Cartridge) = @_;

    my $FirstLevel = 'TemplateFiles';
    my $SecondLevel = 'Deleted';

    foreach my $Element (@{$PatchXMLStruct->getElements()}){
        if ($Element =~/\/\QPatchCartridgeDir{PatchCartridgeDir}\E\/\QPatchFunction{Method->onPatchDeleteCartridgeFile}\E\//){
            my $ThirdLevel = $Cartridge . '/' . $PatchXMLStruct->getIDValue($Element,'_Tag-Text');

            $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = 1 if (
                $ThirdLevel =~ /\/\QTemplates\E\//
                and not $ThirdLevel =~ /\...\.xml$/
            );
        }
    }
}

#search in an unpacked patch for template files, these files are added or changed templates
sub RegisterAddedOrChangedTemplateFiles {
    my ($hResult, $FolderUnpackedPatch) = @_;

    my $FirstLevel = 'TemplateFiles';
    my $SecondLevel = 'Added/Changed';
    my $aFiles = GetFileNames($FolderUnpackedPatch, {'recursive' => 1});

    foreach my $File (@$aFiles){
        if ($File =~ /\/\QTemplates\E\// and not $File =~ /\...\.xml$/){
            my $CuttetFileName = $File;
            $CuttetFileName =~ s/^.*\/\QDE_EPAGES\E\///;
            my $ThirdLevel = $CuttetFileName;
            $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = 1;
        }
    }
}

#run through $hResult and write all infos of hash into file
sub WriteChangesToFile{
    my ($hResult,$FileName) = @_;

    #print out patch changes
    my $ChangesFile = $FileName;
    open(my $CHANGES, '>', $ChangesFile) or die "Can't write '$ChangesFile': $!"; ##no critic (RequireBriefOpen)
    foreach my $FirstLevel (sort keys %{$hResult}){
        print $CHANGES '* ' . $FirstLevel . "\n";
        foreach my $SecondLevel (sort keys %{$hResult->{$FirstLevel}}){
            print $CHANGES '    ';
            if ($FirstLevel eq 'Changed' ){print $CHANGES '* ';}
                elsif ($FirstLevel eq 'Added' ){print $CHANGES '+ ';}
                elsif ($FirstLevel eq 'Deleted' ){print $CHANGES '- ';}
                else{print $CHANGES '>';}
                print $CHANGES $SecondLevel . "\n";
            foreach my $ThirdLevel (sort keys %{$hResult->{$FirstLevel}->{$SecondLevel}}){
                print $CHANGES '        ';
                if ($SecondLevel eq 'Changed' ){print $CHANGES '* ';}
                elsif ($SecondLevel eq 'Added' ){print $CHANGES '+ ';}
                elsif ($SecondLevel eq 'Deleted' ){print $CHANGES '- ';}
                elsif ($SecondLevel eq 'Added/Changed' ){print $CHANGES '+/* ';}
                else{print $CHANGES '>';}
                print $CHANGES $ThirdLevel . "\n";
                if (ref $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} eq 'HASH'){
                    foreach my $FourthLevel (sort keys %{$hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}}){
                        print $CHANGES '            ';
                        if ($ThirdLevel eq 'Changed' ){print $CHANGES '* ';}
                        elsif ($ThirdLevel eq 'Added' ){print $CHANGES '+ ';}
                        elsif ($ThirdLevel eq 'Deleted' ){print $CHANGES '- ';}
                        else{print $CHANGES '>';}
                        print $CHANGES $FourthLevel . "\n";
                        if (ref $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel} eq 'HASH'){
                            foreach my $FifthLevel (sort keys %{$hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}}){
                                print $CHANGES '                ';
                                if ($FourthLevel eq 'Changed' ){print $CHANGES '* ';}
                                elsif ($FourthLevel eq 'Added' ){print $CHANGES '+ ';}
                                elsif ($FourthLevel eq 'Deleted' ){print $CHANGES '- ';}
                                else{print $CHANGES '>';}
                                print $CHANGES $FifthLevel . "\n";
                                if (ref $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}->{$FifthLevel} eq 'HASH'){
                                    foreach my $SixthLevel (sort keys %{$hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}->{$FifthLevel}}){
                                         print $CHANGES '                    ';
                                         if ($FifthLevel eq 'Changed' ){print $CHANGES '* ';}
                                        elsif ($FifthLevel eq 'Added' ){print $CHANGES '+ ';}
                                        elsif ($FifthLevel eq 'Deleted' ){print $CHANGES '- ';}
                                        else{print $CHANGES '>';}
                                        print $CHANGES $SixthLevel . "\n";
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    close($CHANGES) or die "Can't close '$ChangesFile': $!";
}

#run through all patch files and collect infos like added Actions, Menus etc. fill results into $hResult
sub CollectInfosFromPatchFiles {
    my ($hResult,$CheckDir) = @_;

    my $aFileNames;
    my $XMLStructImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();

    #get all import and delete files
    $aFileNames = GetFileNames($CheckDir) if ExistsDirectory($CheckDir);
    foreach my $PatchFileName (@$aFileNames){#for all import and delete files
        #ignore cvs files
        next if ($PatchFileName =~ /\/\.\#/);
        next if ($PatchFileName =~ /\.bak$/);
        my $IsDeleteFile = $PatchFileName =~ /del_/;
        print '    ' . $PatchFileName . "\n";

        #get xml structure from import/delete file to rgeister changes
        my $CheckStruct = $XMLStructImportHandler->getXMLStructure($PatchFileName);

        foreach my $ElementName (@{$CheckStruct->getElements()}){#for each element in xml structure

            #register changes in page types
            if ($CheckStruct->getClass($ElementName) eq 'PageType'){
                RegisterPageTypeChanges($ElementName, $CheckStruct, $hResult, $IsDeleteFile);
            }

            #register changes for actions
            elsif ($CheckStruct->getClass($ElementName) eq 'Action'){
                RegisterItemChanges($ElementName, $CheckStruct, $hResult, 'Actions', $IsDeleteFile);
            }

            #register changes for attributes
            elsif ($CheckStruct->getClass($ElementName) eq 'Attribute'){
                RegisterItemChanges($ElementName, $CheckStruct, $hResult, 'Attributes', $IsDeleteFile);
            }

            #register changes for features
            elsif ($CheckStruct->getClass($ElementName) eq 'Feature'){
                RegisterFeatureChanges($ElementName, $CheckStruct, $hResult, 'Features', $IsDeleteFile);
            }

            #register changes for forms
            elsif ($CheckStruct->getClass($ElementName) eq 'Form'){
                RegisterFormChanges($ElementName, $CheckStruct, $hResult, 'Forms', $IsDeleteFile);
            }

            #register changes for hooks
            elsif ($CheckStruct->getClass($ElementName) eq 'Hook'){
                RegisterHookChanges($ElementName, $CheckStruct, $hResult, $IsDeleteFile)
            }
        }
    }

}

sub RegisterPageTypeChanges {
    my ($ElementName, $CheckStruct, $hResult, $IsDeleteFile) = @_;

    my $FirstLevel = 'PageTypes';
    my $SecondLevel;
    if ($CheckStruct->getTagAttribute($ElementName, 'delete') eq '1' and $IsDeleteFile){
        $SecondLevel = 'Deleted';
    }
    elsif ($CheckStruct->getTagAttribute($ElementName, 'reference') eq '1'){
        $SecondLevel = 'Changed';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        not $IsDeleteFile
    ){
        $SecondLevel = 'Added';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        $CheckStruct->getTagAttribute($ElementName, 'delete') ne '1' and
        $IsDeleteFile
    ){
        $SecondLevel = 'Changed';
    }

    my $Alias = $CheckStruct->getIDValue($ElementName,'Alias');
    my $ThirdLevel = $Alias;
    #$hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = {};

    foreach my $SubElement (@{$CheckStruct->getChildElements($ElementName)}){
        if ($CheckStruct->getClass($SubElement) eq 'Menu'){
            my $FourthLevel = 'Menus';
            my $FifthLevel;
            if ($CheckStruct->getTagAttribute($SubElement, 'delete') eq '1' and $IsDeleteFile){
                $FifthLevel = 'Deleted';
            }
            elsif ($CheckStruct->getTagAttribute($SubElement, 'reference') eq '1'){
                $FifthLevel = 'Changed';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                not $IsDeleteFile
            ){
                $FifthLevel = 'Added';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                $CheckStruct->getTagAttribute($SubElement, 'delete') ne '1' and
                $IsDeleteFile
            ){
                $FifthLevel = 'Changed';
            }
            my $MenuName = $CheckStruct->getIDValue($SubElement,'Name');
            my $SixthLevel = $MenuName;
            $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}->{$FifthLevel}->{$SixthLevel} = 1;
        }
        elsif ($CheckStruct->getClass($SubElement) eq 'Template'){
            my $FourthLevel = 'Templates';
            my $FifthLevel;
            if ($CheckStruct->getTagAttribute($SubElement, 'delete') eq '1' and $IsDeleteFile){
                $FifthLevel = 'Deleted';
            }
            elsif ($CheckStruct->getTagAttribute($SubElement, 'reference') eq '1'){
                $FifthLevel = 'Changed';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                not $IsDeleteFile
            ){
                $FifthLevel = 'Added';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                $CheckStruct->getTagAttribute($SubElement, 'delete') ne '1' and
                $IsDeleteFile
            ){
                $FifthLevel = 'Changed';
            }

            my $TemplateName = $CheckStruct->getIDValue($SubElement,'Name');
            my $SixthLevel = $TemplateName;
            $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}->{$FifthLevel}->{$SixthLevel} = 1;
        }
    }
}

sub RegisterHookChanges {
    my ($ElementName, $CheckStruct, $hResult, $IsDeleteFile) = @_;

    my $FirstLevel = 'Hooks';
    my $SecondLevel;
    if ($CheckStruct->getTagAttribute($ElementName, 'delete') eq '1' and $IsDeleteFile){
        $SecondLevel = 'Deleted';
    }
    elsif ($CheckStruct->getTagAttribute($ElementName, 'reference') eq '1'){
        $SecondLevel = 'Changed';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        not $IsDeleteFile
    ){
        $SecondLevel = 'Added';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        $CheckStruct->getTagAttribute($ElementName, 'delete') ne '1' and
        $IsDeleteFile
    ){
        $SecondLevel = 'Changed';
    }

    my $Name = $CheckStruct->getIDValue($ElementName,'Name');
    my $ThirdLevel = $Name;
    #my $ThirdLevel = $ElementName;
    $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = {};
    foreach my $SubElement (@{$CheckStruct->getChildElements($ElementName)}){
        if ($CheckStruct->getClass($SubElement) eq 'HookFunction'){
            my $FourthLevel = 'HookFunctions';
            my $FifthLevel;
            if ($CheckStruct->getTagAttribute($SubElement, 'delete') eq '1' and $IsDeleteFile){
                $FifthLevel = 'Deleted';
            }
            elsif ($CheckStruct->getTagAttribute($SubElement, 'reference') eq '1'){
                $FifthLevel = 'Changed';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                not $IsDeleteFile
            ){
                $FifthLevel = 'Added';
            }
            elsif (
                $CheckStruct->getTagAttribute($SubElement, 'reference') ne '1' and
                $CheckStruct->getTagAttribute($SubElement, 'delete') ne '1' and
                $IsDeleteFile
            ){
                $FifthLevel = 'Changed';
            }

            my $FunctionName = $CheckStruct->getIDValue($SubElement,'FunctionName');
            my $SixthLevel = $FunctionName;
            $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel}->{$FourthLevel}->{$FifthLevel}->{$SixthLevel} = 1;
        }
    }
}

sub RegisterFormChanges{
    my ($ElementName, $CheckStruct, $hResult, $FirstLevel, $IsDeleteFile) = @_;


    my $SecondLevel;
    if ($CheckStruct->getTagAttribute($ElementName, 'delete') eq '1' and $IsDeleteFile){
        $SecondLevel = 'Deleted';
    }
    elsif ($CheckStruct->getTagAttribute($ElementName, 'reference') eq '1'){
        $SecondLevel = 'Changed';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        not $IsDeleteFile
    ){
        $SecondLevel = 'Added';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        $CheckStruct->getTagAttribute($ElementName, 'delete') ne '1' and
        $IsDeleteFile
    ){
        $SecondLevel = 'Changed';
    }

    my $Name = $CheckStruct->getIDValue($ElementName,'Name');

    my $Class = $ElementName;
    $Class =~ s/^[\/\\]Classes[\/\\]//;
    $Class =~ s/[\/\\].*$//;
    my $ThirdLevel = "$Class.$Name";
    $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = 1;
}

sub RegisterFeatureChanges {
    my ($ElementName, $CheckStruct, $hResult, $FirstLevel, $IsDeleteFile) = @_;


    my $SecondLevel;
    if ($CheckStruct->getTagAttribute($ElementName, 'delete') eq '1' and $IsDeleteFile){
        $SecondLevel = 'Deleted';
    }
    elsif ($CheckStruct->getTagAttribute($ElementName, 'reference') eq '1'){
        $SecondLevel = 'Changed';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        not $IsDeleteFile
    ){
        $SecondLevel = 'Added';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        $CheckStruct->getTagAttribute($ElementName, 'delete') ne '1' and
        $IsDeleteFile
    ){
        $SecondLevel = 'Changed';
    }

    my $Alias = $CheckStruct->getIDValue($ElementName,'Alias');
    my $ThirdLevel = "$Alias";
    $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = 1;
}

sub RegisterItemChanges {
    my ($ElementName, $CheckStruct, $hResult, $FirstLevel, $IsDeleteFile) = @_;


    my $SecondLevel;
    if ($CheckStruct->getTagAttribute($ElementName, 'delete') eq '1' and $IsDeleteFile){
        $SecondLevel = 'Deleted';
    }
    elsif ($CheckStruct->getTagAttribute($ElementName, 'reference') eq '1'){
        $SecondLevel = 'Changed';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        not $IsDeleteFile
    ){
        $SecondLevel = 'Added';
    }
    elsif (
        $CheckStruct->getTagAttribute($ElementName, 'reference') ne '1' and
        $CheckStruct->getTagAttribute($ElementName, 'delete') ne '1' and
        $IsDeleteFile
    ){
        $SecondLevel = 'Changed';
    }

    my $Alias = $CheckStruct->getIDValue($ElementName,'Alias');
    my $Class = $ElementName;
    $Class =~ s/^[\/\\]?Classes[\/\\]//;
    $Class =~ s/[\/\\].*$//;
    my $ThirdLevel = "$Class.$Alias";
    $hResult->{$FirstLevel}->{$SecondLevel}->{$ThirdLevel} = 1;
}

#run through all patch files and collect infos like added Actions, Menus etc. fill results into $hResult
sub CollectInfosFromPatchSQLFiles {
    my ($hResult,$CheckDir) = @_;

    my $aFileNames;
    my $SQLStructImportHandler = DE_EPAGES::Object::API::SQLStructureImportHandler->new();

    #get all import and delete files
    $aFileNames = GetFileNames($CheckDir) if ExistsDirectory($CheckDir);

    foreach my $NewSQLFile (@$aFileNames){#for all import and delete files
        #ignore cvs files
        next if ($NewSQLFile =~ /del_/);
        next if ($NewSQLFile =~ /\/\.\#/);
        next if ($NewSQLFile =~ /\.bak$/);

        print '    ' . $NewSQLFile . "\n";

        #get sql structure from new sql files
        my $CheckStruct = $SQLStructImportHandler->getSQLStructure($NewSQLFile);

        my $aNewTables = $CheckStruct->getTables();

        foreach my $Table (@$aNewTables){
            RegisterNewTable($Table,$hResult);
        }
    }

}

sub RegisterNewTable {
    my ($TableName, $hResult) = @_;

    my $FirstLevel = 'Tables';
    my $SecondLevel = 'Added';

    $hResult->{$FirstLevel}->{$SecondLevel}->{$TableName} = 1;
}

sub usage {
    print <<END_USAGE;
Usage:
    Notice: it is important that the files '/XML/Scripts/findUseProblems.dat' in specific release tags
    are consistent with the API function in perl modules tagged with specific release tag.
    A actual unpacked patch is needed to get a list of added and changed template files.
    perl $0 [options] [flags] datafiles

options:
    -old              old release tag version                      ()
    -new              new release tag version                      ()
    -unpackedpatch    folder of unpacked patch                ()
flags:
    -help       show the command line options

Example:
    perl $0 -new 6_12_0 -old 6_11_3 -unpackedpatch /tmp/Temp/patch_6_12_0/Cartridges.v/6.12.0
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
