use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf UsePackage );
use DE_EPAGES::Core::API::File qw( GetSubDirectories ExistsFile RealPath );
use DE_EPAGES::Database::API::Config qw( StoreType );
use DE_EPAGES::Core::API::CartridgeConfig qw( CartridgeConfig );

##no critic qw(ProhibitCascadingIfElse)

sub CopyCartridgeData ($$$$$) {
    my ($Package, $Type, $StoreType, $Version, $StoreName) = @_;

    printf "copy %s Data of Cartridge %s \n",  $Type, $Package;
    my $CartridgePackage = $Package . '::API::Cartridge';
    UsePackage($CartridgePackage);
    my $Cartridge = $CartridgePackage->new('StoreName'=>$StoreName,'StoreType'=>$StoreType,'StoreTypeVersion'=>$Version);

    if($Type eq 'Private'){
        $Cartridge->copyPrivateFiles;
    }
    elsif($Type eq 'Public'){
        $Cartridge->copyPublicFiles;
    }
    elsif($Type eq 'WebRoot'){
        $Cartridge->copyWebRootFiles;
    }
    elsif($Type eq 'javascript'){
        $Cartridge->copyJavascriptFiles;
    }
    elsif($Type eq 'binary'){
        $Cartridge->copyBinaryFiles;
    }
    elsif($Type eq 'StoreRoot'){
        $Cartridge->copyPublicFiles;
        $Cartridge->copyJavascriptFiles;
    }
    elsif($Type eq 'all'){
        $Cartridge->copyPublicFiles;
        $Cartridge->copyPrivateFiles;
        $Cartridge->copyWebRootFiles;
        $Cartridge->copyJavascriptFiles;
        $Cartridge->copyBinaryFiles;
    }

    return;
}

sub Main {

    local $| = 1;

    my ($Help, $StoreName, $Type, $StoreType, $Version);

    GetOptions(
        'help'          => \$Help,
        'storename=s'   => \$StoreName,
        'type=s'        => \$Type,
        'storetype=s'   => \$StoreType,
        'version=s'     => \$Version,
    );

    usage() if $Help;
    unless( scalar @ARGV ) {
        print STDERR "No cartridge specified.\n\n";
        usage();
    }
    if (not defined $Type or not ElementOf ($Type , (qw (Private Public WebRoot javascript binary all StoreRoot)) )) {
        print STDERR "Type unknown.\n\n";
        usage();
    }

    if (($Type eq 'Private' or $Type eq 'all') and not $StoreName){
        print STDERR "Types 'Private' and 'all' need option '-storename'.\n\n";
        usage();
    }

    #---
    # backward compatibility mode
    unless($StoreType){
        $StoreType = StoreType($StoreName) if $StoreName;
        unless($StoreType){
            print STDERR "No storetype specified.\n\n";
            usage();
        }
    }

    unless($Version){
        $Version = CartridgeConfig($StoreName)->section('Cartridges')->{'DE_EPAGES::Core'} if $StoreName;
        unless($Version){
            print STDERR "No version specified.\n\n";
            usage();
        }
    }
    #---

    my $time1 = time;

    my $aPackages = [ @ARGV ];
    if (scalar(@$aPackages) == 1 and $aPackages->[0] eq 'ALL' ) {
        #get all existing cartridges
        $aPackages = GetAllCartridgePackages();
    }

    foreach my $Package (@$aPackages) {
        CopyCartridgeData($Package, $Type, $StoreType, $Version, $StoreName);
    }

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub GetAllCartridgePackages {

    my @Packages;
    my $CartridgesDir = RealPath($ENV{'EPAGES_CARTRIDGES'});
    my $aVendorDirs = GetSubDirectories($CartridgesDir, {'recursive' => 0});
    foreach my $VendorDir ( @$aVendorDirs ) {
        my $aCartridgeDirs = GetSubDirectories($VendorDir, {'recursive' => 0});
        foreach my $CartridgeDir (@$aCartridgeDirs) {
            if ( ExistsFile($CartridgeDir . '/API/Cartridge.pm') ) {
                my $Package = $CartridgeDir;
                $Package =~ s/$CartridgesDir//;
                $Package =~ s/^\///;
                $Package =~ s/\//::/g;
                push @Packages, $Package;
            }
        }
    }
    return \@Packages;
}

sub usage {
    print <<END_USAGE;
Description:
    Copies data files from the cartridge to the given store's directory.
    Be careful, existing files in target directory will be overwritten.
    Use copyCartridgeDataFile.pl to copy only a single file.

Usage:
    perl $0 [options] [flags] cartridges

options:
    -storename  name of store          (used for -type Private)
    -storetype  database type                                ()
    -version    epages version                               ()
    -type       type of Cartridge-Data                       ()
                possible values are:
                  Private ...... [Cartridge]/Data/Private
                  Public ....... [Cartridge]/Data/Public
                  WebRoot ...... [Cartridge]/Data/WebRoot
                  javascript ... [Cartridge]/Data/javascript
                  binary ....... [Cartridge]/bin
                  all .......... copy all above (needs -storename)
                  StoreRoot .... includes Public and javascript
flags:
    -help       show the command line options

Example:
    perl $0 DE_EPAGES::Design -type Public -storetype Store -version 6.10.3
    perl $0 ALL -type StoreRoot -storetype Store -version 6.10.3
    perl $0 ALL -type Private -storename Store
    perl $0 ALL -type all -storename Store

END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;

