use strict;
use Getopt::Long;
use File::Basename;
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Object::API::XMLStructureExportDriver;
use DE_EPAGES::Installer::API::LangDiffBuilder qw (
    PostProcessXMLFile
    AddFilePrefix
    ChangeStructLanguage
    GetMainLanguage
);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw (
    ExistsFile
    RunOnDirectory
    GetFileNames
    CopyFile
);

# save a translation XML structure to disk. automatically detects encoding to use
sub _SaveStructure {
    my ($Structure, $FileName) = @_;

    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    $ExportDriver->exportXML($FileName, $Structure, { encoding => 'utf8', quotecharacter => '"' });

    return;
}

# change quotes and encoding
sub _PreProcessFile {
    my ($XMLFile, $TargetFile) = @_;

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $OriginalStructure = $ImportHandler->getXMLStructure($XMLFile, { 'register_notrans' => 1} );
    _SaveStructure($OriginalStructure, $TargetFile);

    return;
}

# merge SourceFile into TargetFile, overwrite existing nodes in target
sub _MergeFiles {
    my ($SourceFile, $TargetFile, $Language) = @_;

    print "Merging $SourceFile into $TargetFile\n";

    #get info and objects
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $OriginalStructure = $ImportHandler->getXMLStructure($TargetFile, { 'register_notrans' => 1} );
    my $MergeStructure = $ImportHandler->getXMLStructure($SourceFile, { 'register_notrans' => 1} );
    my $OutputFile = AddFilePrefix($TargetFile, 'transexp_');

    ChangeStructLanguage($MergeStructure, GetMainLanguage($Language));

    #add merge structure elements at the end, set position to a high value
    foreach my $Element (@{$MergeStructure->getElements()}){
        #set to very high number, to add new attributes at the end
        $MergeStructure->setXMLStructAttribute($Element, 'Position', 99999999999); ##no critic (ProhibitMagicNumbers)
    }

    #merge structures
    $OriginalStructure->assimilateXMLStructure($MergeStructure, {'overwrite' => '1', 'overwrite_empty_tagtext' => '0'});

    #export merged structure into file
    _SaveStructure($OriginalStructure, $OutputFile);

    #post process for xml file
    PostProcessXMLFile($OutputFile);

    unlink($TargetFile);
    rename($OutputFile, $TargetFile);

    return;
}

# delete nodes listed in SourceFile from TargetFile
sub _DeleteNodes {
    my ($SourceFile, $TargetFile, $Language) = @_;
    print "Deleting nodes listed in $SourceFile from $TargetFile\n";

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $OriginalStructure = $ImportHandler->getXMLStructure($TargetFile, { 'register_notrans' => 1});
    my $DeleteStructure = $ImportHandler->getXMLStructure($SourceFile, { 'register_notrans' => 1});

    ChangeStructLanguage($DeleteStructure, $Language);

    my $ElementsToRemove = $DeleteStructure->getElements();
    foreach my $Element (@$ElementsToRemove) {
        my $RemoveElement = 0;
        my $ElementClass = $DeleteStructure->getClass($Element);

        if($ElementClass eq 'Translation') {
            $RemoveElement = 1;
        }
        elsif($ElementClass eq 'Object') {
            $RemoveElement = 1;
        }
        elsif($ElementClass eq 'TranslationAttribute') {
            $RemoveElement = 1;
        }
        elsif($Element =~ /\/.*/) {
            print "Unknown element $Element of class $ElementClass\n";
        }

        if($RemoveElement == 1) {
            $OriginalStructure->deleteElement($Element);
        }
    }

    my $OutputFile = AddFilePrefix($TargetFile, 'transexp_');
    _SaveStructure($OriginalStructure, $OutputFile);
    PostProcessXMLFile($OutputFile);

    unlink($TargetFile);
    rename($OutputFile, $TargetFile);

    return;
}

# copy and postprocess xml file
sub _CopyNewFile {
    my ($PreprocessedFile, $TargetFile) = @_;

    print "Copying new file $PreprocessedFile to $TargetFile\n";
    CopyFile($PreprocessedFile, $TargetFile, { 'create_directory' => 1, 'unlink' => 1 } );
    PostProcessXMLFile($TargetFile);

    return;
}

sub _ProcessFiles {
    my ($aSrcFiles, $NamePattern, $SourceDir, $cSub) = @_;

    foreach my $SrcFile (@$aSrcFiles) {
        my $SrcDir = dirname($SrcFile);
        my $SrcFileName = basename($SrcFile);
        my $AbsSrcPath = $SrcFile;
        $AbsSrcPath =~ s/^\.(.*)/$SourceDir$1/g;
        $cSub->($SrcFileName, $SrcDir, $AbsSrcPath) if($SrcFileName =~ /$NamePattern/);
    }
}

sub _ProcessDirectory {
    my ($SourceDir, $TargetDir, $Language) = @_;
    print "Source: $SourceDir\n";
    print "Language: $Language\n";

    my $SrcFiles = RunOnDirectory($SourceDir, sub { GetFileNames(undef , {'recursive' => 1}) });
    _ProcessFiles($SrcFiles, "^transimp_.*\.$Language\.xml\$", $SourceDir, sub {
        my ($SrcFile, $SrcDir, $AbsSrcPath) = @_;

        $SrcFile =~ s/transimp_(.*)\.$Language\.xml$/$TargetDir\/$SrcDir\/$1\.$Language\.xml/gx;
        my $PreprocessedFile = AddFilePrefix($AbsSrcPath, 'transprep_');

        _PreProcessFile($AbsSrcPath, $PreprocessedFile);

        if(ExistsFile($SrcFile)) {
            _MergeFiles($PreprocessedFile, $SrcFile, $Language);
        } else {
            _CopyNewFile($PreprocessedFile, $SrcFile);
        }

        unlink($PreprocessedFile);
    });

    _ProcessFiles($SrcFiles, '^transdel_.*\.xx\.xml$', $SourceDir, sub {
        my ($SrcFile, $SrcDir, $AbsSrcPath) = @_;

        $SrcFile =~ s/transdel_(.*)\.xx\.xml$/$TargetDir\/$SrcDir\/$1\.$Language\.xml/g;
        _DeleteNodes($AbsSrcPath, $SrcFile, $Language);
    });

    _ProcessFiles($SrcFiles, '^transdelfile_.*\.xx\.xml$', $SourceDir, sub {
        my ($SrcFile, $SrcDir, $AbsSrcPath) = @_;

        $SrcFile =~ s/transdelfile_(.*)\.xx\.xml$/$TargetDir\/$SrcDir\/$1\.$Language\.xml/g;
        unlink($SrcFile);
    });

    return;
}

sub Main {
    local $| = 1;

    my $Help;
    my $SourceDir;
    my $TargetDir;
    my $Language;

    GetOptions(
        'help'       => \$Help,
        'source=s'   => \$SourceDir,
        'target=s'   => \$TargetDir,
        'lang=s'     => \$Language
    );

    usage() if $Help;

    unless(defined $SourceDir) {
        print STDERR "missing source\n\n";
        usage();
    }

    unless(defined $TargetDir) {
        print STDERR "missing target\n\n";
        usage();
    }

    $Language = 'de' unless defined($Language);

    $SourceDir =~ s/[\/\\]$//;
    $TargetDir =~ s/[\/\\]$//;

    _ProcessDirectory($SourceDir, $TargetDir, $Language);

    return;
}

sub usage {
    print <<END_USAGE;
Usage:
    This script merges language file diffs into stock language files
    perl $0 [options] [flags]

options:
    -source                  directory to read diff files from
    -target                  directory to write output files into
    -lang                    language to use (de)
flags:
    -help                    show the command line options

Example:
    perl $0 -lang de -source diff_de_6_9_0 -target d:\\epages\\Cartridges
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
