#========================================================================================
# package      DE_EPAGES::Ipayment::API::Order::Payment
# state        public
#----------------------------------------------------------------------------------------
# description  Class to handle the payment type.
#========================================================================================
package DE_EPAGES::Ipayment::API::Order::Payment;

use strict;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Ipayment::API::CGI::RequestFactory  qw ( CreateDefaultCapture CreateDefaultCancel );
use DE_EPAGES::Ipayment::API::Constants qw (%CGI_MESSAGES %TRANSLATED_MESSAGES);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Ipayment::API::Order::Payment
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     setCancelObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       setCancelObject( $CancelObject );
#----------------------------------------------------------------------------------------
# description  store a cancel object in this class
#----------------------------------------------------------------------------------------
# input        $CancelObject | a cancel object| DE_EPAGES::Ipayment::API::CGI::Cancel
#========================================================================================
sub setCancelObject($) {
    my $self = shift;
    my ($CancelObject) = @_;
    TestObject('CancelObject',$CancelObject,'DE_EPAGES::Ipayment::API::CGI::Cancel');
    $self->{'CancelObject'} = $CancelObject;
    return;
}

#========================================================================================
# function     setCaptureObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       setCaptureObject( $CaptureObject );
#----------------------------------------------------------------------------------------
# description  store a capture object in this class
#----------------------------------------------------------------------------------------
# input        $CaptureObject | a cancel object| DE_EPAGES::Ipayment::API::CGI::Capture
#========================================================================================
sub setCaptureObject($) {
    my $self = shift;
    my ($CaptureObject) = @_;
    TestObject('CaptureObject',$CaptureObject,'DE_EPAGES::Ipayment::API::CGI::Capture');
    $self->{'CaptureObject'} = $CaptureObject;
    return;
}

#========================================================================================
# function     _getCgiCancel
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getCgiCancel( );
#----------------------------------------------------------------------------------------
# description  It will return at least a default Cancel object or an object which is set by
#               $self->setRequestObject();
#----------------------------------------------------------------------------------------
# return       $RequestObject | a request object or one of its children | DE_EPAGES::Ipayment::API::CGI::Cancel
#========================================================================================
sub _getCgiCancel {
    my $self = shift;
    my $CancelObject = undef;
    if(defined $self->{'CancelObject'}){
        $CancelObject =  $self->{'CancelObject'};
    }else{
        $CancelObject = CreateDefaultCancel();
    }
    return $CancelObject;
}

#========================================================================================
# function     _GetCgiCapture
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetCgiCapture( );
#----------------------------------------------------------------------------------------
# description  It will return at least a default capture object or an object which is set by
#               $self->setRequestObject();
#----------------------------------------------------------------------------------------
# return       $RequestObject | a request object or one of its children | DE_EPAGES::Ipayment::API::CGI::Capture
#========================================================================================
sub _GetCgiCapture {
    my $self = shift;
    my $CaptureObject = undef;
    if(defined $self->{'CaptureObject'}){
        $CaptureObject =  $self->{'CaptureObject'};
    }else{
        $CaptureObject = CreateDefaultCapture();
    }
    return $CaptureObject;
}

#========================================================================================
# function     cancel
# state        public
#----------------------------------------------------------------------------------------
# syntax       cancel();
#----------------------------------------------------------------------------------------
# description  This method cancel the order
#----------------------------------------------------------------------------------------
# input        $Order      | an epages order   | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $self->getStatus() | Return the status of the cancel (ok or error) | String
#========================================================================================
sub cancel {
    my $self = shift;
    my ($Order) = @_;
    TestObject('Order',$Order,'DE_EPAGES::Order::API::Object::CustomerOrder');

    my $CgiCancel = $self->_getCgiCancel();

    my $Status = $CgiCancel->cancelMoney($Order);

    $self->_setStatus($Status);
    $self->_setMessage($Status);

    return $self->getStatus();
}

#========================================================================================
# function     capture
# state        public
#----------------------------------------------------------------------------------------
# syntax       capture();
#----------------------------------------------------------------------------------------
# description  This method cancel the order
#----------------------------------------------------------------------------------------
# input        $Order      | an epages order   | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $self->getStatus() | Return the status of the cancel (ok or error) | String
#========================================================================================
sub capture {
    my $self = shift;
    my ($Order) = @_;
    TestObject('Order',$Order,'DE_EPAGES::Order::API::Object::CustomerOrder');

    my $CgiCapture = $self->_GetCgiCapture();

    my $Status = $CgiCapture->captureMoney($Order);

    $self->_setStatus($Status);
    $self->_setMessage($Status);

    return $self->getStatus();
}

#========================================================================================
# function     _setStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setStatus( $Status );
#----------------------------------------------------------------------------------------
# description  If the status input is ok set self status as ok, otherwise set self status as error
#----------------------------------------------------------------------------------------
# input        $Status | Status of the payment from the CGI (ok or some error) | String
#========================================================================================
sub _setStatus {
    my $self = shift;
    my ($Status) = @_;

    if (lc($Status) eq 'ok' ) {
       $self->{'status'} = 'ok';
    } else {
       $self->{'status'} = 'error';
    }

    return;
}

#========================================================================================
# function     _setMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setMessage( $Status );
#----------------------------------------------------------------------------------------
# description  Preparing the different kind of messages for JSON response
#----------------------------------------------------------------------------------------
# input        $Status | Status of the payment from the CGI (ok or some errror) | String
#========================================================================================
sub _setMessage {
    my $self = shift;
    my ($Status) = @_;

    if ($Status eq $CGI_MESSAGES{'OK'} ) {
        $self->{'message'} = '';
    }
    elsif ($Status eq $CGI_MESSAGES{'NO_IPAYMENT_LINEITEM'} ) {
        $self->{'message'} = $TRANSLATED_MESSAGES{'INVALID_ORDER'};
    }
    elsif ($Status eq $CGI_MESSAGES{'UNSUPPORTED_ACTION'}) {
        $self->{'message'} = $TRANSLATED_MESSAGES{'INVALID_ORDER'};
    }
    elsif ($Status eq $CGI_MESSAGES{'NO_HASH'} ) {
        $self->{'message'} = $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'};
    }
    elsif ($Status eq $CGI_MESSAGES{'NO_REDIRECT_URL'}) {
        $self->{'message'} = $TRANSLATED_MESSAGES{'WEBSERVICE_ERROR'};
    }
    else {
        $self->{'message'} = "$TRANSLATED_MESSAGES{'UNKNOWN_ERROR'}: $Status";
    }

    return;
}

#========================================================================================
# function     getStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       getStatus();
#----------------------------------------------------------------------------------------
# description  Return the Status from the CGI request
#----------------------------------------------------------------------------------------
# return       $self->{'status'} | Return the Status | String
#========================================================================================
sub getStatus {
    my $self = shift;
    return $self->{'status'};
}

#========================================================================================
# function     getMessage
# state        public
#----------------------------------------------------------------------------------------
# syntax       getMessage();
#----------------------------------------------------------------------------------------
# description  Return the Message from the CGI request
#----------------------------------------------------------------------------------------
# return       $self->{'message'} | Return the Message | String
#========================================================================================
sub getMessage {
    my $self = shift;
    return $self->{'message'};
}

1;
