#========================================================================================
# package      DE_EPAGES::Mail::API::Mail
# state        public
# example      my $Connection = DE_EPAGES::Mail::API::SMTP->new( 'mail', 25 );
#               $Connection->connection( sub {
#                   my $Mail = DE_EPAGES::Mail::API::Mail->new( {
#                       'FROM' => 'Mailer <sender@domain.com>',
#                       'TO'   => ['Recipient1@provider1.com','Name <Recipient2@provider2.com>', ],
#                       'SUBJECT' => 'Test Subject'
#                   });
#                   $Mail->addText({'content'   => 'This is Plain Text'});
#                   $Mail->send( $Connection );
#               });
#----------------------------------------------------------------------------------------
# description  This package is used for processing e-mails.
#               It simplifies creation of HTML mails and mails with attachments.
#========================================================================================
package DE_EPAGES::Mail::API::Mail;

use strict;
use Encode qw ( encode );
use MIME::Lite;
use DE_EPAGES::Mail::API::Constants qw(MIME_AUTOCONTENTTYPE);
use DE_EPAGES::Core::API::TestParameter qw ( TestHash);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail = DE_EPAGES::Mail::API::Mail->new($hParams);
# syntax       my $Mail = DE_EPAGES::Mail::API::Mail->new( {
#                        'HEADER_ENC'   => 'iso-8859-1'
#                        'SUBJECT'      => 'the Subject'
#                        'FROM'         => 'Sender <mailer@epages.de>',
#                        'NOTIFY'       => 1,
#                        'REPLY'        => 'Replyer <replyaddress@epages.de>',
#                        'SkipBadRecipients'   => 1,
#                        'TO'           => ['Recipient <rcpt@epages.de>',
#                                            'Recipient1 <rcpt1@epages.de>'
#                                          ],
#                        'CC'           => ['CC1 <ccrcpt1@epages.de>',
#                                            'CC2 <ccrcpt2@epages.de>'
#                                          ],
#                        'BCC'          => ['BCC1 <bccrcpt1@epages.de>',
#                                            'BCC2 <bccrcpt2@epages.de>'
#                                          ]
#                       });
#
#----------------------------------------------------------------------------------------
# description  Creates a new object of DE_EPAGES::Mail::API::Mail with header parameters.
#----------------------------------------------------------------------------------------
# input        $hParams | e-mail header parameters with following keys:
#               <ul>
#                   <li>HEADER_ENC  - utf8, iso-8859-1,...                          (optional) default='utf-8' - string
#                   <li>SUBJECT     - the mail subject                              - string
#                   <li>FROM        - the sender "Phrase <name@host.domain>"        - string
#                   <li>NOTIFY      - read notification to sender 1/0               (optional)- boolean
#                   <li>REPLY       - the reply address 'Phrase <name@host.domain>' (optional)- string
#                   <li>TO          - the recipients 'Phrase <name@host.domain>'    (optional)- array
#                   <li>SkipBadRecipients  - 1/0 ignore recipient addresses like
#                                     'name@epa@ges.de'                             (optional)- boolean
#                   <li>CC          - the CC recipients 'Phrase <name@host.domain>' (optional)- array
#                   <li>BCC         - the BCC recipients 'Phrase <name@host.domain>'(optional)- array
#               </ul> | reference to hash
# return       $Mail | mail object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($hParams) = @_;

    TestHash('hParams', $hParams, {
        'FROM'=>'String;NotNull',
    });

    return bless {
        Header  => $hParams, # headers
        Parts   => {},       # hash of parts
    }, $class;
}

#========================================================================================
# function     addText
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail->addText( $hParams );
# syntax       $Mail->addText({
#                   'content'     => 'Das ist Plain Text',
#                   'encoding'    => 'quoted-printable',
#                   'type'        => 'text/plain',
#                   'type.charset'=> 'utf-8',
#               });
#----------------------------------------------------------------------------------------
# description  Adds a text element to the mail.
#----------------------------------------------------------------------------------------
# input        $hParams | with following keys:
#               <ul>
#                   <li>content     - text to add              - string
#                   <li>encoding    - encoding      (optional, default:quoted-printable) - string
#                   <li>type        - content type  (optional, default:text/plain) - string
#                   <li>type.charset - character encoding (optional, default:utf-8) - string
#               </ul> | reference to hash
#========================================================================================
sub addText {
    my $self = shift;
    my ($hParams) = @_;
    TestHash('hParams', $hParams);

    # add  mail headers fields...
    $self->{'Parts'}{'Text'} = $hParams;

    return;
}

#========================================================================================
# function     addHTML
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail->addHTML( $hParams );
# example      $Mail->addHTML({
#                   'content'         => '<body>content</body>',
#                   'encoding'        => 'quoted-printable',
#                   'type'            => 'text/html',
#                   'type.charset'    => 'utf-8',
#                   'html2plain'      => 1,
#                   'alttext'         => '',
#                   'altencoding'     => 'quoted-printable',
#                   'alt.type.charset'=> 'utf-8',
#               });
#----------------------------------------------------------------------------------------
# description  Adds an HTML element to the mail.
#----------------------------------------------------------------------------------------
# input        $hParams | with following keys:
#               <ul>
#                   <li>content     - HTML text - string
#                   <li>encoding    - encoding      (optional, default:quoted-printable) - string
#                   <li>type        - content type  (optional, default:text/html) - string
#                   <li>type.charset- content type charset (optional, default:utf8) - string
#                   <li>html2plain  - create alternate text from html
#                                     flag 1/0      (optional) - boolean
#                   <li>alttext     - alternate text if not html2plain
#                                     (optional, default: '') - string
#                   <li>altencoding - encoding of the alternate text
#                                     (optional, default: quoted-printable) - string
#                   <li>alt.type.charset - content type charset for alternate text
#                                     (optional, default:utf8) - string
#               </ul> | reference to hash
#========================================================================================
sub addHTML {
    my $self = shift;
    my ($hParams) = @_;
    TestHash('hParams', $hParams);

    # add  mail headers fields...
    $self->{'Parts'}{'HTML'} = $hParams;
    return;
}

#========================================================================================
# function     addHTMLAttachment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail->addHTMLAttachment( $ahParams );
# example      my $hStyle = {
#                   'filename'   => '/tmp/style.css',
#                   'includeid'  => '__reference_id_style_',
#                   'type'       => 'text/css',
#               };
#               my $hImage = {
#                   'filename'   => '/tmp/image.jpg',
#                   'includeid'  => '__reference_id_image_',
#                   'type'       => 'image/jpg',
#               };
#               $Mail->addHTMLAttachment( [ $hStyle, $hImage ] );
#----------------------------------------------------------------------------------------
# description  Adds inline attachments (images, style sheets etc.) to an HTML e-mail.
#----------------------------------------------------------------------------------------
# input        $ahParams | with following keys:
#               <ul>
#                   <li>filename    - the filename                           - string
#                   <li>includeid   - unique id for the file ref. in html    - string
#                   <li>type        - content type   (optional)
#                                     default:Application/Octet-stream       - string
#               </ul> | reference to array of hashes
#========================================================================================
sub addHTMLAttachment {
    my $self = shift;
    my ($ahParams) = @_;

    # filename and includeid fields are mandatory !
    TestHash('Attachment', $_, {
        'filename'=>'String;NotNull',
        'includeid'=>'String;NotNull'
     }) foreach @$ahParams;


    # add  mail headers fields...

    if (! $self->isHtmlAttachment()){
        $self->{'Parts'}{'HTML_Attachment'} = $ahParams;
    } else {
        push (@{$self->{'Parts'}{'HTML_Attachment'}}, @$ahParams);
    }

    return;
}

#========================================================================================
# function     addAttachment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail->addAttachment( $ahParams );
# example      my $hFile = {
#                   'filename' => '/tmp/Setup.exe',
#                   'type'     => 'Application/Octet-stream',
#               };
#               $Mail->addAttachment( [ $hFile ] );
#----------------------------------------------------------------------------------------
# description  Adds attachments to the mail.
#----------------------------------------------------------------------------------------
# input        $ahParams | with following keys:
#               <ul>
#                   <li>filename    - path to file or filename               - string
#                   <li>type        - content type   (optional)
#                                     default:Application/Octet-stream       - string
#                   <li>data        - document to attach (optional)          - binary
#                   <li>encoding    - document's encoding (optional)         - string
#                                     default:base64
#               </ul> | reference to array of hashes
#========================================================================================
sub addAttachment {
    my $self = shift;
    my ($ahParams) = @_;

    # filename field are mandatory !
    TestHash('Attachment', $_, {'filename'=>'String;NotNull'}) foreach @$ahParams;

    # add  mail headers fields...
    if (! $self->isAttachment()){
        $self->{'Parts'}{'Attachment'} = $ahParams;
    } else {
        push(@{$self->{'Parts'}{'Attachment'}},@{$ahParams});
    }

    return;
}

#========================================================================================
# function     send
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Mail->send($Connection);
#----------------------------------------------------------------------------------------
# description  Sends the mail to the given SMTP Server Connection.
#----------------------------------------------------------------------------------------
# input        $Connection | SMTP server Connection, see
#                               <package DE_EPAGES::Mail::API::SMTP> | object
#========================================================================================
sub send {
    my $self = shift;
    my ($Connection) = @_;

    $Connection->sendDataToSMTP( $self->createMail(), $self->getHeader());
    return;
}

#========================================================================================
# function     getHeader
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hParams = $Mail->getHeader();
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the e-mail header.
#----------------------------------------------------------------------------------------
# return       $hHeader | header parameters, see <function new> | ref.hash
#========================================================================================
sub getHeader {
    my $self = shift;
    return $self->{'Header'};
}

#========================================================================================
# function     createMail
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MailBody = $Mail->createMail();
# example      $Connection->sendDataToSMTP( $Mail->createMail(), $Mail->getHeader() );
#----------------------------------------------------------------------------------------
# description  Creates the MIME body of the e-mail from the previously added text and
#               attachment elements.
#----------------------------------------------------------------------------------------
# return       $MailBody  | the Mail content |  string
#========================================================================================
sub createMail {
    my $self = shift;

    # set the contenttype to autodetect if not given
    local $MIME::Lite::AUTO_CONTENT_TYPE = MIME_AUTOCONTENTTYPE ? 1:0;

    my $MasterContentType = $self->getMasterContainerType;
    my $MasterContainer;

    if($MasterContentType eq 'text/plain') {
        $MasterContainer = MIME::Lite->new(
            Type => 'text/plain',
            $self->getTextElements(),
        );
        $MasterContainer->attr('Content-type.charset' => $self->getTextCharset);
    }
    elsif($MasterContentType eq 'text/html') {
        $MasterContainer = MIME::Lite->new(
            Type => 'text/html',
            $self->getHTMLElements(),
        );
        $MasterContainer->attr('Content-type.charset' => $self->getHtmlCharset);
    }
    elsif($MasterContentType eq 'multipart/mixed') {
        $MasterContainer = $self->_createMultiPartMixedContainer;
    }
    elsif($MasterContentType eq 'multipart/alternative') {
        $MasterContainer = $self->_createMultiPartAlternativeContainer;
    }
    elsif($MasterContentType eq 'multipart/related') {
        $MasterContainer = $self->_createMultiPartRelatedContainer;
    }
    elsif($MasterContentType eq 'Application/Octet-stream') {
        $MasterContainer = MIME::Lite->new(
            Type => 'application/octet-stream',
            map { @$_ } $self->getATTElements(),
        );
    }
    else {
        die "Can't handle content type: '$MasterContentType'";
    }

    # get the message as string
    return $MasterContainer->as_string;
}

sub _createMultiPartRelatedContainer {
    my $self = shift;

    my $MasterContainer =  MIME::Lite->new( Type => 'multipart/related' );

    # html with images inline
    my $Part = $MasterContainer->attach( $self->getHTMLElements );
    $Part->attr('Content-type.charset' => $self->getHtmlCharset);

    $MasterContainer->attach( @$_ ) foreach $self->getHTMLATTElements;

    return $MasterContainer;
}

sub _createMultiPartAlternativeContainer {
    my $self = shift;

    my $MasterContainer =  MIME::Lite->new( Type => 'multipart/alternative' );

    my $SubContainerType = $self->getSubContainerType();
    if (! $SubContainerType) {
        # alternative text or html
        $MasterContainer->attach( $self->getALTElements );
        $MasterContainer->attr('Content-type.charset' => ( $self->isHtmlToPlain ? $self->getHtmlCharset : $self->getHtmlAltCharset ));
        my $Part = $MasterContainer->attach( $self->getHTMLElements );
        $Part->attr('Content-type.charset' => $self->getHtmlCharset);
        $MasterContainer->attr('Content-type.charset' => $self->getHtmlCharset);

    }
    elsif( $SubContainerType eq 'multipart/related' ) {
        # html content with inline elements + alternate text
        my $Part = $MasterContainer->attach( $self->getALTElements );
        $Part->attr('Content-type.charset' => ( $self->isHtmlToPlain ? $self->getHtmlCharset : $self->getHtmlAltCharset ));

        my $SubContainer =  MIME::Lite->new( Type => $SubContainerType );
        my $SubPart = $SubContainer->attach( $self->getHTMLElements );
        $SubPart->attr('Content-type.charset' => $self->getHtmlCharset);
        $SubContainer->attach( @$_ ) foreach $self->getHTMLATTElements;
        $MasterContainer->attach($SubContainer);
    }

    return $MasterContainer;
}

sub _createMultiPartMixedContainer {
    my $self = shift;

    my $MasterContainer =  MIME::Lite->new( Type => 'multipart/mixed' );

    my $SubContainerType = $self->getSubContainerType();
    if (! $SubContainerType) { # attachments with text only
        my $Part = $MasterContainer->attach( $self->getTextElements );
        $Part->attr('Content-type.charset' => $self->getTextCharset);
        if( $self->isHtml && !$self->isHtmlAlternate && !$self->isHtmlToPlain ) {
            $Part = $MasterContainer->attach( $self->getHTMLElements );
        }
        $Part->attr('Content-type.charset' => $self->getHtmlCharset);
        $MasterContainer->attach(@{$_}) foreach $self->getATTElements;
    }
    else {
        my $SubContainer = MIME::Lite->new( Type => $SubContainerType );

        if($SubContainerType eq 'multipart/related') {
            # attachments and html content with inline elements
            my $Part = $SubContainer->attach( $self->getHTMLElements );
            $Part->attr('Content-type.charset' => $self->getHtmlCharset);
            $SubContainer->attach(@{$_}) foreach $self->getHTMLATTElements;
            $MasterContainer->attach($SubContainer);
            $MasterContainer->attach(@{$_}) foreach $self->getATTElements;
        }
        elsif($SubContainerType eq 'multipart/alternative') {
            my $ALTPart = $SubContainer->attach( $self->getALTElements );
            my $ALTCharSet = $self->isHtmlToPlain ? $self->getHtmlCharset : $self->getHtmlAltCharset;
            $ALTPart->attr('Content-type.charset' => $ALTCharSet);

            my $SubSubContainerType = $self->getSubSubContainerType;
            if ($SubSubContainerType eq 'multipart/related') {
                # attachments , html content with inlines and alternative text
                my $SubSubContainer = MIME::Lite->new(
                    Type => $SubSubContainerType,
                );
                my $SubPart = $SubSubContainer->attach( $self->getHTMLElements );
                $SubPart->attr('Content-type.charset' => $self->getHtmlCharset);

                $SubSubContainer->attach(@{$_}) foreach $self->getHTMLATTElements;
                $SubContainer->attach($SubSubContainer);
            }
            else {
                # attachments with alternative txt and html content
                my $Part = $SubContainer->attach( $self->getHTMLElements );
                $Part->attr('Content-type.charset' => $self->getHtmlCharset);
            }
            $MasterContainer->attach($SubContainer);
            $MasterContainer->attach(@{$_}) foreach $self->getATTElements;
        }
    }

    return $MasterContainer;
}

#========================================================================================
# function     convertHtml2Plain
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Plain = $Mail->convertHtml2Plain($HTML);
#----------------------------------------------------------------------------------------
# description  Creates plain text from HTML.
#----------------------------------------------------------------------------------------
# input        $HTML | html string | string
# return       $Plain | plain text | string
#========================================================================================
sub convertHtml2Plain {
    my $self = shift;
    local $_ = shift;
    s/\r|\n/ /g;                     # remove newlines
    s/<style\b.*?<\/style>/ /igs;    # remove style tags
    s/<script\b.*?<\/script>/ /igs;  # remove script tags
    s/<h\d\b.*?>/\n/igs;             # replace <h(x)> by \n
    s/<\/h\d\b.*?>/\n/igs;           # replace </h(x)> by \n
    s/<br\b.*?>/\n/igs;              # replace <br> by \n
    s/<p\b.*?>/\n\n/igs;             # replace <p> by \n\n
    s/<div\b.*?>/\n/igs;             # replace <div> by \n
    s/<tr\b.*?>/\n/igs;             # replace <tr> by \n
    s/&nbsp;/ /g;    # replace &nbsp;
    s/<a\b(.*?)>(.*?)<\/a>/$self->_convertLink2Plain($1,$2)/xigse;
    s/<(.*?)>/ /gs;  # remove HTML tags
    s/\t/ /g;        # convert tabs to spaces
    s/ +/ /g;        # remove multiple spaces
    s/^\s+//;        # remove leading spaces
    s/\n +/\n/gs;    # remove leading spaces per line
    s/\n(\n+)/\n\n/gs; # collapse continous empty lines
    s/&gt;/>/g;      # replace >
    s/&lt;/</g;      # replace <
    s/&quot;/"/g;    # replace "
    s/&amp;/&/g;     # replace &amp;

    return $_;
}

#========================================================================================
# function     _convertLink2Plain
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Plain = $self->_convertLink2Plain( $Attributes, $LinkText );
# example      my $Plain = $self->_convertLink2Plain( 'href="http://inter.net"', 'The Internet' );
#----------------------------------------------------------------------------------------
# description  Converts a HTML link to a readable URL.
#----------------------------------------------------------------------------------------
# input        $Attributes | attributes of the <a> tag | string
# input        $LinkText | content of the <a> tag | string
# return       $Plain | readable URL, example: "The Internet &lt;http://inter.net&gt;" | string
#========================================================================================
sub _convertLink2Plain {
    my $self = shift;
    my ($Attributes, $LinkText) = @_;

    $LinkText =~ s/<(.*?)>/ /gs;  # remove HTML tags
    return '' unless $LinkText =~ /\S/;

    if( $Attributes =~ /\bhref=['"](.*?)['"]/ix ) { #"
        my $URL = $1;
        return $URL =~ /^#/
            ? $LinkText
            : "$LinkText &lt;$URL&gt;";
    }
    else {
        return $LinkText;
    }
}

sub getTextCharset {
    my $self = shift;
    my $typecharset = $self->{'Parts'}{'Text'}->{'type.charset'}
                      ? $self->{'Parts'}{'Text'}->{'type.charset'} : 'utf-8';
    return $typecharset;
}

sub getHtmlCharset {
    my $self = shift;
    my $typecharset = $self->{'Parts'}{'HTML'}->{'type.charset'}
                      ? $self->{'Parts'}{'HTML'}->{'type.charset'} : 'utf-8';
    return $typecharset;
}

sub getHtmlAltCharset {
    my $self = shift;
    my $typecharset = $self->{'Parts'}{'HTML'}->{'alt.type.charset'}
                      ? $self->{'Parts'}{'HTML'}->{'alt.type.charset'} : 'utf-8';
    return $typecharset;
}

sub isText {
    my $self = shift;
    return 1 if defined $self->{'Parts'}{'Text'}->{'content'};
    return 0;
}

sub isHtml {
    my $self = shift;
    return 1 if defined $self->{'Parts'}{'HTML'}->{'content'};
    return 0;
}

sub isHtmlAttachment {
    my $self = shift;
    return 1 if defined $self->{'Parts'}{'HTML_Attachment'}->[0]->{'filename'};
    return 0;
}

sub isAttachment {
    my $self = shift;
    return 1 if defined $self->{'Parts'}{'Attachment'}->[0]->{'filename'};
    return 0;

}

sub isHtmlAlternate {
    my $self = shift;
    return 1 if $self->{'Parts'}{'HTML'}{'alttext'};
    return 0;
}

sub isHtmlToPlain {
    my $self = shift;
    return 1 if $self->{'Parts'}{'HTML'}{'html2plain'};
    return 0;
}

sub getMasterContainerType {
    my $self = shift;

    my $MasterContainerType;
    $MasterContainerType = 'text/plain'             if $self->isText();
    $MasterContainerType = 'text/html'              if $self->isHtml();
    $MasterContainerType = 'multipart/alternative'  if $self->isHtml() && ($self->isHtmlAlternate()
                                                                       || $self->isHtmlToPlain());
    #$MasterContainerType = 'Application/Octet-stream' if $self->isAttachment();
    $MasterContainerType = 'multipart/mixed'        if $self->isAttachment();
    $MasterContainerType = 'multipart/mixed'        if $self->isAttachment() && ($self->isText()
                                                                             || $self->isHtml() );
    $MasterContainerType = 'multipart/related'      if $self->isHtml() && $self->isHtmlAttachment()
                                                                       && !$self->isAttachment()
                                                                       && !($self->isHtmlAlternate()
                                                                       || $self->isHtmlToPlain());
    return $MasterContainerType;
}

sub getSubContainerType {
    my $self = shift;

    my $SubContainerType;
    $SubContainerType = 'multipart/related'         if ($self->getMasterContainerType eq 'multipart/mixed')
                                                        && $self->isHtml() && $self->isHtmlAttachment();
    $SubContainerType = 'multipart/related'         if ($self->getMasterContainerType eq 'multipart/alternative')
                                                        && $self->isHtml() && $self->isHtmlAttachment();
    $SubContainerType = 'multipart/alternative'     if ($self->getMasterContainerType eq 'multipart/mixed')
                                                        && $self->isHtml()
                                                        && ($self->isHtmlAlternate()
                                                        || $self->isHtmlToPlain());

    return $SubContainerType;
}

sub getSubSubContainerType {
    my $self = shift;

    return undef unless defined $self->getSubContainerType;

    my $SubSubContainerType;
    $SubSubContainerType = 'multipart/related'      if ($self->getSubContainerType eq 'multipart/alternative')
                                                    && $self->isHtml()
                                                    && $self->isHtmlAttachment();

    return $SubSubContainerType;
}

sub getTextElements {
    my $self = shift;

    my $hTextPart = $self->{'Parts'}{'Text'};

    # encode the Text
    my $Text = encode($self->getTextCharset(),$hTextPart->{'content'});
    $Text='' unless defined $Text;

    # replace DOS line feeds (otherwise MS Outlook shows 2 line breaks)
    $Text =~ s/\015\012/\012/gs;

    # push all Elements of the mail into arrays
    return (
        'Data'     => $Text,
        'Encoding' => $hTextPart->{'encoding'} ? $hTextPart->{'encoding'} : 'quoted-printable',
        'Type'     => $hTextPart->{'type'} ? $hTextPart->{'type'} : 'text/plain',
    );
}

sub getHTMLElements {
    my $self = shift;

    my $hHtmlPart = $self->{'Parts'}{'HTML'};

    # encode the Html
    my $HtmlText = encode( $self->getHtmlCharset(), $hHtmlPart->{'content'} );

    # replace DOS line feeds
    $HtmlText =~ s/\015\012/\012/gs if defined $HtmlText;

    return (
        'Data'     => $HtmlText,
        'Encoding' => $hHtmlPart->{'encoding'} ? $hHtmlPart->{'encoding'} : 'quoted-printable',
        'Type'     => $hHtmlPart->{'type'} ? $hHtmlPart->{'type'} : 'text/html',
    );
}

sub getALTElements {
    my $self = shift;

    my $hHtmlPart = $self->{'Parts'}{'HTML'};

    # convert html2plain
    my $Text = $self->isHtmlToPlain()
        ? $self->convertHtml2Plain( $hHtmlPart->{'content'} )
        : $hHtmlPart->{'alttext'};

    # encode the alternate Text
    $Text = encode( $self->getHtmlAltCharset(), $Text );

    # replace DOS line feeds (otherwise MS Outlook show 2 line breaks)
    $Text =~ s/\015\012/\012/gs if defined $Text;

    return (
        'Data'     => $Text,
        'Encoding' => $hHtmlPart->{'altencoding'} ? $hHtmlPart->{'altencoding'} : 'quoted-printable',
        'Type'     => 'text/plain'
    );
}

sub getATTElements {
    my $self = shift;

    my @ATTElements;
    if ( $self->isAttachment() ){
        foreach my $hAttachment (@{$self->{'Parts'}{'Attachment'}}){
            my $Type = $hAttachment->{'type'} ? $hAttachment->{'type'} : 'Application/Octet-stream';
            if(exists $hAttachment->{'data'}){
                push(@ATTElements, [ 'Data'     => $hAttachment->{'data'},
                                     'Filename' => $hAttachment->{'filename'},
                                     'Encoding' => $hAttachment->{'encoding'} || 'base64',
                                     'Type'     => $Type
                                      ]);
            } else {
                push (@ATTElements, [ 'Type' => $Type,
                                      'Path' => $hAttachment->{'filename'} ]);
            }
        }
    }

    return @ATTElements;
}

sub getHTMLATTElements {
    my $self = shift;

    my @HTMLATTElements;
    if ( $self->isHtmlAttachment() ){
        push (@HTMLATTElements, ['Type' => $_->{'type'} ? $_->{'type'} : 'Application/Octet-stream' ,
                                 'Path' => $_->{'filename'},
                                 'Id'   => $_->{'includeid'}
                                ]
             ) foreach @{$self->{'Parts'}{'HTML_Attachment'}};
    }
    return @HTMLATTElements;
}

1;
