#========================================================================================
# package      DE_EPAGES::MultiStore::Hooks::ObjectImport
# state        public
# description  Listens to events on objects and triggers appropriate functions.
#========================================================================================
package DE_EPAGES::MultiStore::Hooks::ObjectImport;

use strict;
use DE_EPAGES::Catalog::API::Catalog qw( AssignProductToCategory );
use DE_EPAGES::Object::API::Factory qw( SplitObjectPath );
use DE_EPAGES::MultiStore::API::Synchronization qw( ClassAliasFromObjectPath ImportProduct ImportProductType SetProductType );

#========================================================================================
# function     OnTransformImportXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnTransformImportXML($hParams);
#----------------------------------------------------------------------------------------
# description  API_MS_TransformImportXML triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Object</li>
#                   <li>TagName - String</li>
#                   <li>hAttributes - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnTransformImportXML {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'}->getSite;
    my $hAttributes = $hParams->{'hAttributes'};

    if ($hParams->{'TagName'} eq 'BundleProductMap') {
        my $aBundledProductPath = SplitObjectPath( $hAttributes->{'BundledProduct'} );
        ImportProduct($Shop, join('/', 'Products', $aBundledProductPath->[1]) );
    }
    elsif ($hParams->{'TagName'} eq 'Product' and exists $hAttributes->{'ProductType'}) {
        # ProductType does not exists for variations
        ImportProductType($Shop, $hAttributes->{'ProductType'});

        # Set a new product type if the imported one differs from the existing object.
        my $ProductFolder = $Shop->child('Products');
        my $Alias = $hAttributes->{'Alias'};
        return unless $ProductFolder->existsChild($Alias);

        my $Product = $ProductFolder->child($Alias);
        my $NewProductTypeAlias = $hAttributes->{'ProductType'};
        SetProductType($Product, $NewProductTypeAlias)
            if $Product->class->alias ne $NewProductTypeAlias;
    }

    return;
}

#========================================================================================
# function     OnAddSubShopObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAddSubShopObject($hParams);
#----------------------------------------------------------------------------------------
# description  API_MS_AddSubShopObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>SubShopCategory - DE_EPAGES::MultiStore::API::Object::SubShopCategory</li>
#                   <li>MasterObjectPath - String</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAddSubShopObject {
    my ($hParams) = @_;

    my $SubShopCategory = $hParams->{'SubShopCategory'};
    my $MasterObjectPath = $hParams->{'MasterObjectPath'};

    my $Shop = $SubShopCategory->getSite;
    my $SubShopObject;
    if( ClassAliasFromObjectPath($MasterObjectPath) eq 'Product' ){

        $SubShopObject = ImportProduct( $Shop, $MasterObjectPath );

        # assign it to its category if the object is a product
        AssignProductToCategory($SubShopObject, $SubShopCategory);

        #Pass results to caller.
        @{$hParams}{'SubShopObject','Done'} = ($SubShopObject,1)
            if defined $SubShopObject;
    }

    return;
}

1;
