#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreCategories
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for a single shipping method.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreCategories;
use base DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;

use DE_EPAGES::MultiStoreConnector::API::CategorySynchronization;

#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    return LoadObjectByPath('/RestServices/MultiStoreCategories');
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    return 'System';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.complete.v1+json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}


#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/multi_stores/ms_categories/?$|;
    return 1;
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    return [ $Object ];
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->post($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub post {
    my $self = shift;
    my ($Shop, $Request) = @_;

    #get data from request
    my $aMultiStoreCategories = $Request->{ms_categories}->{categories};
    my $aMultiStoreSubShops = $Request->{ms_categories}->{subshops};
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::CategorySynchronization->new();
    my $hResult;
    my $aShops;

    #for each multistore shop in data base
    foreach my $ShopInfo (@$aMultiStoreSubShops){
        #get info
        my $ShopAlias = $ShopInfo->{'alias'};

        my $aProductsResult = $Synchronizer->addCategoriesToShop($ShopAlias, $aMultiStoreCategories);

        #call synchronization
        push (@$aShops, {'alias' => $ShopAlias, 'categories' => $aProductsResult});
    }

    $hResult->{'ms_categories'} = $aShops;

    #return results
    return $hResult;
}


1;
