#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProductTypeAttributes
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the sub resource for multistore product type attributes.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProductTypeAttributes;
use base DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;

use strict;
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization;
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);


#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    return LoadObjectByPath('/RestServices/MultiStoreProductTypeAttributes');
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    return 'ProductType';
}

#========================================================================================
# function     producedContentTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProducedContentTypes = $Resource->producedContentTypes($RequestMethod);
#----------------------------------------------------------------------------------------
# description  Returns the produced content types of this resource, depending on the
#               provided method.
#----------------------------------------------------------------------------------------
# input        $RequestMethod | options for synchronization
#               <ul>
#                 <li>POST</li>
#               </ul>
#               | scalar
#----------------------------------------------------------------------------------------
# return       $ProducedContentTypes | produced content types | ref.array
#========================================================================================
sub producedContentTypes {
    my $self = shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.complete.v1+json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
        PUT => [
            ['put', 'application/json'],
            ['put', 'application/vnd.epages.complete.v1+json'],
            ['put', 'application/vnd.epages.v1+json'],
        ],
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.complete.v1+json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
        DELETE => [
            ['delete', 'application/json'],
            ['delete', 'application/vnd.epages.complete.v1+json'],
            ['delete', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}

#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/multi_stores/ms_product_types/([^/]+)/attributes/?$|;
    return $1;
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'post' or $ActionName eq 'put' ) {
        return [ $Object ];
    }
    elsif( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'guid'}->[0] ];
    }
    elsif( $ActionName eq 'delete' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'guid'}->[0] ];
    }
    else {
        return [ $Object ];
    }
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->post($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the post result for ms_product_type_attributes.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_product_types</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub post {
    my $self = shift;
    my ($ProductType, $Request) = @_;

    #get data from request
    my $aMultiStoreProductTypeAttributes = $Request->{ms_product_type_attributes};
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeAttributeSynchronization->new();
    my $hResult;
    my $hProductTypeAttributesResult;
    my $ProductTypeGUID = $ProductType->get('GUID');

    my $Shop = $ProductType->parent()->parent();

    #get info
    my $StoreName = GetCurrentDBHandle()->storeName();

    # store GUID of product type together with attributes
    foreach my $hMultiStoreProductTypeAttribute (@$aMultiStoreProductTypeAttributes) {
        $hMultiStoreProductTypeAttribute->{'product_type'} = $ProductType->get('GUID');
    }



    #try call synchronization
    eval{
        #call synchronization
        $hProductTypeAttributesResult = $Synchronizer->addProductTypeAttributes($Shop->alias, $aMultiStoreProductTypeAttributes);
    };

    #check for errors
    if(ExistsError()){
        my $MyError = GetError();

        #raise error if not correct
        if ( $MyError->code eq 'AttributeTypeNotDefined' ){
            Error('BadRequest', {'Message' => 'Attribute type \'' . $MyError->vars->{'Type'} . '\' is not valid in product type attribute \'' . $MyError->vars->{'Attribute'} . '\'' } );
        }
        elsif ( $MyError->code eq 'ProductTypeNotFoundInStore'){
            Error('ResourceNotFound', { 'Reason' => 'empty result set for product type with GUID \'' . $MyError->vars->{'GUID'} . '\'', 'Class' => 'ProductType', 'Site' => $StoreName} );
        }
        elsif ( $MyError->code eq 'ShopNotFoundInStore'){
            Error('ResourceNotFound', { 'Reason' => 'empty result set for shop \'' . $MyError->vars->{'Shop'} . '\'', 'GUID' => $MyError->vars->{'Shop'}, 'Class' => 'Shop', 'Site' => $StoreName} );
        }
        elsif( $MyError->code eq 'MissingMandatoryAttribute'){
            Error('BadRequest', {'Message' => 'mandatory attribute \'' . $MyError->vars->{'Attribute'} . '\' in product type attribute \'' . $MyError->vars->{'Object'} . '\' is missing.' } );
        }
        elsif( $MyError->code eq 'AttributeValueVerificationError'){
            Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product type attribute \'' . $MyError->vars->{'Object'} . '\' is not a \'' . $MyError->vars->{'ParameterType'} . '\'' } );
        }
        else{
            Error($MyError);
        }
    }

    #synchronize AS caches
    $Synchronizer->synchronizeCache();

    #save result
    $hResult->{'ms_product_type_attributes'} = $hProductTypeAttributesResult->{$ProductTypeGUID};

    #return results
    return $hResult;
}

1;
