#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProductTypes
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for multistore product types.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProductTypes;
use base DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;

use strict;
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization;
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    return LoadObjectByPath('/RestServices/MultiStoreProductTypes');
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    return 'System';
}

#========================================================================================
# function     producedContentTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProducedContentTypes = $Resource->producedContentTypes($RequestMethod);
#----------------------------------------------------------------------------------------
# description  Returns the produced content types of this resource, depending on the
#               provided method.
#----------------------------------------------------------------------------------------
# input        $RequestMethod | options for synchronization
#               <ul>
#                 <li>POST</li>
#               </ul>
#               | scalar
#----------------------------------------------------------------------------------------
# return       $ProducedContentTypes | produced content types | ref.array
#========================================================================================
sub producedContentTypes {
    my $self = shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.complete.v1+json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
        PUT => [
            ['put', 'application/json'],
            ['put', 'application/vnd.epages.complete.v1+json'],
            ['put', 'application/vnd.epages.v1+json'],
        ],
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.complete.v1+json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
        DELETE => [
            ['delete', 'application/json'],
            ['delete', 'application/vnd.epages.complete.v1+json'],
            ['delete', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}

#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/multi_stores/ms_product_types/?$|;
    return 1;
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'post' or $ActionName eq 'put' ) {
        return [ $Object ];
    }
    elsif( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'guid'}->[0] ];
    }
    elsif( $ActionName eq 'delete' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'guid'}->[0] ];
    }
    else {
        return [ $Object ];
    }
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->post($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the post result for ms_product_types.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_product_types</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub post {
    my $self = shift;
    my ($System, $Request) = @_;

    #get data from request
    my $aMultiStoreProductTypes = $Request->{ms_product_types}->{product_types};
    my $aMultiStoreSubShops = $Request->{ms_product_types}->{subshops};
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();
    my $hResult;
    my $aShops;

    #get info
    my $StoreName = GetCurrentDBHandle()->storeName();

    #check if all shops exists
    $Synchronizer->verifySubshops($aMultiStoreSubShops, $StoreName);

    #for each multistore shop in data base
    foreach my $ShopInfo (@$aMultiStoreSubShops){
        #get info
        my $ShopAlias = $ShopInfo->{'alias'};
        my $aProductTypesResult;

        #try call synchronization
        eval{
            #call synchronization
            $aProductTypesResult = $Synchronizer->addProductTypesToShop($ShopAlias, $aMultiStoreProductTypes);
        };

        #check for errors
        if(ExistsError()){
            my $MyError = GetError();

            #raise error if not correct
            if ( $MyError->code eq 'NotSupportedProductTypeAttribute' ){
                Error('BadRequest', {'Message' => $MyError->vars->{'Attribute'} . ' is not supported in this version.' } );
            }
            elsif( $MyError->code eq 'MissingGUID'){
                Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product type \'' . $MyError->vars->{'ProductType'} . '\' is missing.' } );
            }
            elsif( $MyError->code eq 'MissingMandatoryAttribute'){
                Error('BadRequest', {'Message' => 'mandatory attribute \'' . $MyError->vars->{'Attribute'} . '\' in product type \'' . $MyError->vars->{'ProductType'} . '\' is missing.' } );
            }
            elsif( $MyError->code eq 'AttributeValueVerificationError'){
                Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product type \'' . $MyError->vars->{'ProductType'} . '\' is not a \'' . $MyError->vars->{'ParameterType'} . '\'' } );
            }
            elsif ( $MyError->code eq 'TemplateNotFound'){
                Error('BadRequest', { 'Reason' => 'Invalid layout key \'' . $MyError->vars->{'Param'} . '\' specified in product type \'' . $MyError->vars->{'ProductType'} . '\' '} );
            }
            elsif ( $MyError->code eq 'TemplateTypeNotFound'){
                Error('BadRequest', { 'Reason' => 'Invalid layout value \'' . $MyError->vars->{'Param'} . '\' specified in product type \'' . $MyError->vars->{'ProductType'} . '\' '} );
            }
            elsif ( $MyError->code eq 'AttributeTypeNotDefined' ){
                Error('BadRequest', {'Message' => 'Attribute type \'' . $MyError->vars->{'Type'} . '\' is not valid in Attribute \'' . $MyError->vars->{'Attribute'} . '\'' } );
            }
            elsif ( $MyError->code eq 'GUIDObjectClassNotProductType'){
                Error('ResourceNotFound', { 'Reason' => 'empty result set for GUID \'' . $MyError->vars->{'GUID'} . '\'', 'GUID' => $MyError->vars->{'GUID'}, 'Class' => 'ProductType', 'Site' => $StoreName} );
            }
            elsif ( $MyError->code eq 'ShopNotFoundInStore'){
                Error('ResourceNotFound', { 'Reason' => 'empty result set for shop \'' . $MyError->vars->{'Shop'} . '\'', 'GUID' => $MyError->vars->{'Shop'}, 'Class' => 'Shop', 'Site' => $StoreName} );
            }
            elsif ( $MyError->code eq 'ProductTypeNotFoundInStore'){
                Error('ResourceNotFound', { 'Reason' => 'empty result set for product type with GUID \'' . $MyError->vars->{'GUID'} . '\'', 'Class' => 'ProductType', 'Site' => $StoreName} );
            }
            else{
                Error($MyError);
            }
        }

        #save result
        push (@$aShops, {'alias' => $ShopAlias, 'product_types' => $aProductTypesResult});
    }

    #synchronize AS caches
    $Synchronizer->synchronizeCache();

    #save result
    $hResult->{'ms_product_types'} = $aShops;

    #return results
    return $hResult;
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->get($System, $GUIDs);
#----------------------------------------------------------------------------------------
# description  Returns the get result for ms_product_types.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $GUIDs | guids seperated with ',' | string
# return       $hResult | result:
#                   <ul><li>ms_product_types</li>
#                           <ul>
#                               <li>
#                                   <ul>
#                                       <li>list containing the requested product type objects</li>
#                                   </ul>
#                               </li>
#                           </ul>
#                   </ul> | ref.array.hash
#========================================================================================
sub get {
    my $self = shift;
    my ($Object, $GUIDs) = @_;

    #raise error if not correct
    Error('BadRequest', {'Message' => 'missing URL parameter \'GUID\''} ) unless ($GUIDs);

    #get store name
    my $StoreName = GetCurrentDBHandle()->storeName();

    #get info
    my @GUIDs = split /,/, $GUIDs;
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductTypeSynchronization->new();

    my $aProductTypesResult;

    eval{
        #call synchronization
        $aProductTypesResult = $Synchronizer->getProductTypesContent(\@GUIDs);
    };

    if(ExistsError()) {

        my $MyError = GetError();

        if ( $MyError->code eq 'NoResultForGUID' ){
            Error('ResourceNotFound', { 'Reason' => 'empty result set for GUID \'' . $MyError->vars->{'GUID'} . '\'', 'GUID' => $MyError->vars->{'GUID'}, 'Class' => 'ProductType', 'Site' => $StoreName} );
        }
        else {
            Error($MyError);
        }
    }


    #check if result is empty
    unless($aProductTypesResult){

        #raise 404 error in response
        Error('ResourceNotFound', { 'Reason' => 'empty result set', 'GUID' => $GUIDs, 'Class' => 'ProductType', 'Site' => $StoreName}) ;
    }
    else{
        #build result
        my $hResult = {
            'ms_product_types' => $aProductTypesResult
        };

        #return results
        return $hResult;
    }
}

1;
