#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProducts
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for multistore products.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreProducts;
use base DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;

use strict;
use DE_EPAGES::Core::API::Error qw (Error GetError ExistsError);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::MultiStoreConnector::API::ProductSynchronization;
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    return LoadObjectByPath('/RestServices/MultiStoreProducts');
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    return 'System';
}

sub producedContentTypes {
    my $self = shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.complete.v1+json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
        PUT => [
            ['put', 'application/json'],
            ['put', 'application/vnd.epages.complete.v1+json'],
            ['put', 'application/vnd.epages.v1+json'],
        ],
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.complete.v1+json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
        DELETE => [
            ['delete', 'application/json'],
            ['delete', 'application/vnd.epages.complete.v1+json'],
            ['delete', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}


#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/multi_stores/ms_products/?$|;
    return 1;
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'post' or $ActionName eq 'put' ) {
        return [ $Object ];
    }
    elsif( $ActionName eq 'get' ) {

        # parse query
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );

        # check params
        Error('BadRequest', {'Message' => 'missing URL parameter \'guid\' or \'shop\''} ) if  ((not defined $hQuery->{'guid'}) and (not defined $hQuery->{'shop'}));
        Error('BadRequest', {'Message' => 'concurrent URL parameter \'guid\' and \'shop\' are not allowed together'} ) if  ((defined $hQuery->{'guid'}) and (defined $hQuery->{'shop'}));
        Error('BadRequest', {'Message' => 'missing URL parameter \'perpage\' , expected because \'page\' is defined'} ) if  ((defined $hQuery->{'page'}) and (not defined $hQuery->{'perpage'}));
        Error('BadRequest', {'Message' => 'missing URL parameter \'page\' , expected because \'perpage\' is defined'} ) if  ((defined $hQuery->{'perpage'}) and (not defined $hQuery->{'page'}));

        # prepare params
        my $hParams = {};

        $hParams->{'shop'} = $hQuery->{'shop'}->[0] if (defined $hQuery->{'shop'});
        $hParams->{'guid'} = $hQuery->{'guid'}->[0] if (defined $hQuery->{'guid'});
        $hParams->{'page'} = $hQuery->{'page'}->[0] if (defined $hQuery->{'page'});
        $hParams->{'perpage'} = $hQuery->{'perpage'}->[0] if (defined $hQuery->{'perpage'});

        # return params
        return [ $Object, $hParams ];
    }
    elsif( $ActionName eq 'delete' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'guid'}->[0] ];
    }
    else {
        return [ $Object ];
    }
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->post($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the post result for ms_products.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_products</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub post {
    my $self = shift;
    my ($System, $Request) = @_;

    #get data from request
    my $aMultiStoreProducts = $Request->{ms_products}->{products};
    my $aMultiStoreSubShops = $Request->{ms_products}->{subshops};
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    my $hResult;
    my $aShops;

    #get info
    my $StoreName = GetCurrentDBHandle()->storeName();

    #check if all shops exists
    foreach my $ShopInfo (@$aMultiStoreSubShops){
        #get info
        my $ShopAlias = $ShopInfo->{'alias'};

        #check if shop exists
        eval{
            LoadObjectByPath('/Shops/' . $ShopAlias);
        };
        if(ExistsError){
            my $MyError = GetError();

            if($MyError->code eq 'PATH_NOT_FOUND'){
                Error('ResourceNotFound', {
                    'Reason' => 'empty result set for shop \'' . $ShopAlias . '\'',
                    'GUID' => $ShopAlias,
                    'Class' => 'Shop',
                    'Site' => $StoreName
                });
            }
            else{
                Error($MyError);
            }
        }
    }

    #for each multistore shop in data base
    foreach my $ShopInfo (@$aMultiStoreSubShops){
        #get info
        my $ShopAlias = $ShopInfo->{'alias'};
        my $aProductsResult;

        #try call synchronization
        eval{
            #call synchronization
            $aProductsResult = $Synchronizer->addProductsToShop($ShopAlias, $aMultiStoreProducts);
        };

        #check for errors
        if(ExistsError()){
            my $MyError = GetError();

            #raise error if not correct
            if ( $MyError->code eq 'NotSupportedAttribute' ){
                Error('BadRequest', {'Message' => $MyError->vars->{'Attribute'} . ' is not supported in this version.' } );
            }
            elsif( $MyError->code eq 'MissingGUID'){
                Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Product'} . '\' is missing.' } );
            }
            elsif( $MyError->code eq 'MissingMandatoryAttribute'){
                Error('BadRequest', {'Message' => 'mandatory attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Object'} . '\' is missing or empty.' } );
            }
            elsif( $MyError->code eq 'AttributeValueVerificationError'){
                Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Object'} . '\' is not a \'' . $MyError->vars->{'ParameterType'} . '\'' } );
            }
            elsif ( $MyError->code eq 'GUIDObjectClassNotProduct'){
                Error('ResourceNotFound', { 'Reason' => 'empty result set for GUID \'' . $MyError->vars->{'GUID'} . '\'', 'GUID' => $MyError->vars->{'GUID'}, 'Class' => 'Product', 'Site' => $StoreName} );
            }
            elsif ( $MyError->code eq 'ShopNotFoundInStore'){
                Error('ResourceNotFound', { 'Reason' => 'empty result set for shop \'' . $MyError->vars->{'Shop'} . '\'', 'GUID' => $MyError->vars->{'Shop'}, 'Class' => 'Shop', 'Site' => $StoreName} );
            }
            elsif ( $MyError->code eq 'NoResultForProductType'){
                Error('BadRequest', {'Message' => 'producttype \'' . $MyError->vars->{'ProductType'} . '\' in shop \'' . $MyError->vars->{'Shop'} . '\' does not exist.' } );
            }
            elsif ( $MyError->code eq 'DBIExecute'){

                if( index($MyError->message, 'Duplicate entry') != -1){ # does message contain this stubstring? # WARNING - FRAGILE.. exact message depends on database (and even its version)){
                     Error('BadRequest', {'Message' => ' Product already exists! Database error: ' .  $MyError->message} );
                }
                # Possible other tests here...
                else{
                    Error($MyError);
                }

            }
            else{
                Error($MyError);
            }
        }

        #save result
        push (@$aShops, {'alias' => $ShopAlias, 'products' => $aProductsResult});
    }

    #synchronize AS caches
    $Synchronizer->synchronizeCache();

    #save result
    $hResult->{'ms_products'} = $aShops;

    #return results
    return $hResult;
}

#========================================================================================
# function     put
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->put($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the put result for ms_products.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_products</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub put {
    my $self = shift;
    my ($System, $Request) = @_;

    #get data from request
    my $aMultiStoreProducts = $Request->{ms_products}->{products};
    my $hMultiStoreSubShop = $Request->{ms_products}->{subshop};
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    my $hResult;
    my $aProductsResult;

    #get info
    my $ShopAlias = $hMultiStoreSubShop->{'alias'};
    my $StoreName = GetCurrentDBHandle()->storeName();

    eval{
        #call synchronization
        $aProductsResult = $Synchronizer->updateProductsContentToShop($ShopAlias, $aMultiStoreProducts);
    };
    if(ExistsError()){
        my $MyError = GetError();

        #raise error if not correct
        if ( $MyError->code eq 'NotSupportedAttribute' ){
            Error('BadRequest', {'Message' => $MyError->vars->{'Attribute'} . ' is not supported in this version.' } );
        }
        elsif( $MyError->code eq 'MissingGUID'){
            Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Product'} . '\' is missing.' } );
        }
        elsif( $MyError->code eq 'MissingMandatoryAttribute'){
            Error('BadRequest', {'Message' => 'mandatory attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Product'} . '\' is missing.' } );
        }
        elsif( $MyError->code eq 'AttributeValueVerificationError'){
            Error('BadRequest', {'Message' => 'attribute \'' . $MyError->vars->{'Attribute'} . '\' in product \'' . $MyError->vars->{'Product'} . '\' is not a \'' . $MyError->vars->{'ParameterType'} . '\'' } );
        }
        elsif ( $MyError->code eq 'GUIDObjectClassNotProduct'){
            Error('ResourceNotFound', { 'Reason' => 'empty result set for GUID \'' . $MyError->vars->{'GUID'} . '\'', 'GUID' => $MyError->vars->{'GUID'}, 'Class' => 'Product', 'Site' => $StoreName} );
        }
        elsif ( $MyError->code eq 'ShopNotFoundInStore'){
            Error('ResourceNotFound', { 'Reason' => 'empty result set for shop \'' . $MyError->vars->{'Shop'} . '\'', 'GUID' => $MyError->vars->{'Shop'}, 'Class' => 'Shop', 'Site' => $StoreName} );
        }
        else{
            Error($MyError);
        }
    }

    #synchronize AS caches
    $Synchronizer->synchronizeCache();

    #save result
    $hResult->{'ms_products'} = $aProductsResult;

    #return results
    return $hResult;
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->get($System, $GUIDs);
#----------------------------------------------------------------------------------------
# description  Returns the get result for ms_products.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $GUIDs | guids seperated with ',' | string
# return       $hResult | result:
#                   <ul><li>ms_products</li>
#                           <ul>
#                               <li>
#                                   <ul>
#                                       <li>alias - object alias - string
#                                       <li>taxclass - taxclass name - string
#                                       <li>isnew - value - bool
#                                       <li>isdailyprice - value - bool
#                                       <li>orderunit - string
#                                       <li>pricequantity - float
#                                       <li>minorder - float
#                                       <li>intervalorder - float
#                                       <li>weightunit - string
#                                       <li>weight - float
#                                       <li>refunit - string
#                                       <li>refamount - int
#                                       <li>refcontentamount - float
#                                       <li>stocklevel - float
#                                       <li>length - float
#                                       <li>height - float
#                                       <li>width - float
#                                       <li>isavailable - bool
#                                       <li>imagemediumsmall - string
#                                       <li>imagemediumlarge - string
#                                       <li>imagelarge - string
#                                       <li>imagehotdeal - string
#                                       <li>hassubownprices - bool
#                                       <li>upcean - string
#                                       <li>isdownloadproduct - bool
#                                       <li>position - int
#                                   </ul>
#                               </li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub get {
    my $self = shift;
    my ($Object, $hParams) = @_;

    #get info
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    my $aProductsResult;

    #check if products adressed by guids or all products from shop is required
    if (defined $hParams->{'shop'} and defined $hParams->{'page'} and defined $hParams->{'perpage'}){
        #call synchronization
        $aProductsResult = $Synchronizer->getProductsContentByShopAliasPaged($hParams->{'shop'}, $hParams->{'page'},$hParams->{'perpage'});
    }
    elsif (defined $hParams->{'shop'}){
        #call synchronization
        $aProductsResult = $Synchronizer->getProductsContentByShopAlias($hParams->{'shop'});
    }
    else{
        # get params
        my @GUIDs = split /,/, $hParams->{'guid'};

        #call synchronization
        $aProductsResult = $Synchronizer->getProductsContentByGUIDs(\@GUIDs);
    }

    #check if result is empty
    unless($aProductsResult){
        #get store name
        my $StoreName = GetCurrentDBHandle()->storeName();

        #raise 404 error in response
        Error('ResourceNotFound', { 'Reason' => 'empty result set', 'GUID' => $hParams->{'guid'}, 'Class' => 'Product', 'Site' => $StoreName}) ;
    }
    else{
        #build result
        my $hResult = {
            'ms_products' => $aProductsResult
        };

        #return results
        return $hResult;
    }
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->delete($System, $GUIDs);
#----------------------------------------------------------------------------------------
# description  Returns the delete result for ms_products.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $GUIDs | guids seperated with ',' | string
# return       $hResult | result:
#                   <ul><li>ms_products</li>
#                           <ul>
#                               <li>guid - object object</li>
#                               <li>status - delete status 'fail' or success</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub delete {
    my $self = shift;
    my ($System, $GUIDs) = @_;

    #raise error if not correct
    Error('BadRequest', {'Message' => 'missing URL parameter \'GUID\''} ) unless ($GUIDs);

    #get info
    my @GUIDs = split /,/, $GUIDs;
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::ProductSynchronization->new();
    my $StoreName = GetCurrentDBHandle()->storeName();
    my $aProductsResult;

    eval{
        #call synchronization
        $aProductsResult = $Synchronizer->deleteProductsByGUIDs(\@GUIDs);
    };
    if(ExistsError()){
        my $Error = GetError();

        #raise error if not correct
        Error('ResourceNotFound', { 'Reason' => 'empty result set', 'GUID' => $Error->vars->{'GUID'}, 'Class' => 'Product', 'Site' => $StoreName} ) if ( $Error->code eq 'NoResultForGUID' );
        Error('ResourceNotFound', { 'Reason' => 'empty result set for GUID \'' . $Error->vars->{'GUID'} . '\'', 'GUID' => $Error->vars->{'GUID'}, 'Class' => 'Product', 'Site' => $StoreName} ) if ( $Error->code eq 'GUIDObjectClassNotProduct');
    }

    #check if result is empty
    unless($aProductsResult){
        #raise 404 error in response
        Error('ResourceNotFound', { 'Reason' => 'empty result set', 'GUID' => $GUIDs, 'Class' => 'Product', 'Site' => $StoreName}) ;
    }
    else{
        #build result
        my $hResult = {
            'ms_products' => $aProductsResult
        };

        #return results
        return $hResult;
    }
}

1;
