#========================================================================================
# package      DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreSubShopProductMaps
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for multistore products.
#========================================================================================
package DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreSubShopProductMaps;
use base DE_EPAGES::MultiStoreConnector::API::Resource::MultiStoreBaseObjectResource;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::MultiStoreConnector::API::SubShopProductMapSynchronization;
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);

#========================================================================================
# function     service
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RestService = $Resource->service;
#----------------------------------------------------------------------------------------
# description  Returns the RestService object that contains the RestServiceMethod
#               objects which implement the actual actions.
#----------------------------------------------------------------------------------------
# return       $RestService | rest service object | DE_EPAGES::WebService::API::Object::RestService
#========================================================================================
sub service {
    return LoadObjectByPath('/RestServices/MultiStoreSubShopProductMaps');
}

#========================================================================================
# function     class
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Resource->class;
#----------------------------------------------------------------------------------------
# description  Returns the class name of objects that are represented by this resource.
#               The value is passed to $Site->loadObject($ObjectID, $Class);
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | string
#========================================================================================
sub class {
    return 'System';
}

sub producedContentTypes {
    my $self = shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/json'],
            ['post', 'application/vnd.epages.complete.v1+json'],
            ['post', 'application/vnd.epages.v1+json'],
        ],
        PUT => [
            ['put', 'application/json'],
            ['put', 'application/vnd.epages.complete.v1+json'],
            ['put', 'application/vnd.epages.v1+json'],
        ],
        GET => [
            ['get', 'application/json'],
            ['get', 'application/vnd.epages.complete.v1+json'],
            ['get', 'application/vnd.epages.v1+json'],
        ],
        DELETE => [
            ['delete', 'application/json'],
            ['delete', 'application/vnd.epages.complete.v1+json'],
            ['delete', 'application/vnd.epages.v1+json'],
        ],
    }->{$RequestMethod};
}


#========================================================================================
# function     match
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Match = $Resource->match($Request);
# example      $Match = $Resource->match($Request);
#               if (defined $Match) {
#                   my $hRequestContext = $Resource->requestContext($Site, $Match, $Request);
#               }
#----------------------------------------------------------------------------------------
# description  Tests if the resource can handle the request.
#               If no, returns an empty list.
#               If yes, returns a defined value. In some cases this value can be
#               interpreted as object identifier for the actual resource.
#----------------------------------------------------------------------------------------
# input        $Request | request object | HTTP::Request
# return       $Match | object identifier | string
#========================================================================================
sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/multi_stores/ms_subshop_product_maps/?$|;
    return 1;
}

#========================================================================================
# function     params
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->params($ActionName, $Match, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the parameters for the action function.
#----------------------------------------------------------------------------------------
# input        $ActionName | name of the action  | DE_EPAGES::Object::API::Object::Site
# input        $Match | result of <function match> | string
# input        $Request | request object | HTTP::Request
# return       $hRequestContext | request data with keys:
#                   <ul><li>Object - business object - DE_EPAGES::Object::API::Object::Object</li>
#                       <li>Action - action object or sub ref to handle the request - ref.code or DE_EPAGES::Permission::API::Object::Action</li>
#                       <li>Params - parameters to the action function - ref.array</li>
#                       <li>RequestCodec - </li>
#                       <li>ResponseCodec - </li>
#                   </ul> | ref.hash
#========================================================================================
sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'post' or $ActionName eq 'put' ) {
        return [ $Object ];
    }
    elsif( $ActionName eq 'get' ) {
        # parse query
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );

        # check params
        Error('BadRequest', {'Message' => 'missing URL parameter \'guid\' or \'shop\''} ) if  ((not defined $hQuery->{'guid'}) and (not defined $hQuery->{'shop'}));
        Error('BadRequest', {'Message' => 'concurrent URL parameter \'guid\' and \'shop\' are not allowed together'} ) if  ((defined $hQuery->{'guid'}) and (defined $hQuery->{'shop'}));
        Error('BadRequest', {'Message' => 'missing URL parameter \'perpage\' , expected because \'page\' is defined'} ) if  ((defined $hQuery->{'page'}) and (not defined $hQuery->{'perpage'}));
        Error('BadRequest', {'Message' => 'missing URL parameter \'page\' , expected because \'perpage\' is defined'} ) if  ((defined $hQuery->{'perpage'}) and (not defined $hQuery->{'page'}));

        # prepare params
        my $hParams = {};

        $hParams->{'shop'} = $hQuery->{'shop'}->[0] if (defined $hQuery->{'shop'});
        $hParams->{'guid'} = $hQuery->{'guid'}->[0] if (defined $hQuery->{'guid'});
        $hParams->{'page'} = $hQuery->{'page'}->[0] if (defined $hQuery->{'page'});
        $hParams->{'perpage'} = $hQuery->{'perpage'}->[0] if (defined $hQuery->{'perpage'});

        # return params
        return [ $Object, $hParams ];
    }
    elsif( $ActionName eq 'delete' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        return [ $Object, $hQuery->{'shop'}->[0] ];
    }
    else {
        return [ $Object ];
    }
}

#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->post($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the post result for ms_subshop_product_maps.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_subshop_product_maps</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub post {
    my $self = shift;

    # get store name
    my $StoreName = GetCurrentDBHandle()->storeName();

    Error('ResourceNotFound', { 'Reason' => 'post not supported', 'Database' => $StoreName} );
}

#========================================================================================
# function     put
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->put($System, $Request);
#----------------------------------------------------------------------------------------
# description  Returns the put result for ms_subshop_product_maps.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $Request | request object | HTTP::Request
# return       $hResult | result:
#                   <ul><li>ms_subshop_product_maps</li>
#                           <ul>
#                               <li>alias - object alias</li>
#                               <li>guid - object object</li>
#                               <li>epages6_objectid - object id in epages6 store context</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub put {
    my $self = shift;

    # get store name
    my $StoreName = GetCurrentDBHandle()->storeName();

    Error('ResourceNotFound', { 'Reason' => 'put not supported', 'Database' => $StoreName} );
}

#========================================================================================
# function     get
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->get($System, $GUIDs);
#----------------------------------------------------------------------------------------
# description  Returns the get result for ms_subshop_product_maps.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# input        $GUIDs | guids seperated with ',' | string
# return       $hResult | result:
#                   <ul><li>ms_subshop_product_maps</li>
#                           <ul>
#                               <li>
#                                   <ul>
#                                       <li>alias - object alias - string
#                                       <li>taxclass - taxclass name - string
#                                       <li>isnew - value - bool
#                                       <li>isdailyprice - value - bool
#                                       <li>orderunit - string
#                                       <li>pricequantity - float
#                                       <li>minorder - float
#                                       <li>intervalorder - float
#                                       <li>weightunit - string
#                                       <li>weight - float
#                                       <li>refunit - string
#                                       <li>refamount - int
#                                       <li>refcontentamount - float
#                                       <li>stocklevel - float
#                                       <li>length - float
#                                       <li>height - float
#                                       <li>width - float
#                                       <li>isavailable - bool
#                                       <li>imagemediumsmall - string
#                                       <li>imagemediumlarge - string
#                                       <li>imagelarge - string
#                                       <li>imagehotdeal - string
#                                       <li>hassubownprices - bool
#                                       <li>upcean - string
#                                       <li>isdownloadproduct - bool
#                                       <li>position - int
#                                   </ul>
#                               </li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub get {
    my $self = shift;
    my ($Object, $hParams) = @_;

    #get info
    my $Synchronizer = DE_EPAGES::MultiStoreConnector::API::SubShopProductMapSynchronization->new();
    my $aSubShopProductMapsResult;

    #check if products adressed by guids or all products from shop is required
    if (defined $hParams->{'shop'} and defined $hParams->{'page'} and defined $hParams->{'perpage'}){
        #call synchronization
        $aSubShopProductMapsResult = $Synchronizer->getSubShopProductMapsContentByShopNamePaged($hParams->{'shop'}, $hParams->{'page'},$hParams->{'perpage'});
    }
    elsif (defined $hParams->{'shop'}){
        #call synchronization
        $aSubShopProductMapsResult = $Synchronizer->getSubShopProductMapsContentByShopName($hParams->{'shop'});
    }
    else{
        # get params
        my @GUIDs = split /,/, $hParams->{'guid'};

        #call synchronization
        $aSubShopProductMapsResult = $Synchronizer->getProductsContentByGUIDs(\@GUIDs);
    }

    #build result
    my $hResult = {
        'ms_subshop_product_maps' => $aSubShopProductMapsResult
    };

    #return results
    return $hResult;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestContext = $Resource->delete($System);
#----------------------------------------------------------------------------------------
# description  Returns the delete result for ms_subshop_product_maps.
#----------------------------------------------------------------------------------------
# input        $System | system object  | $Object
# return       $hResult | result:
#                   <ul><li>ms_subshop_product_maps</li>
#                           <ul>
#                               <li>guid - object object</li>
#                               <li>status - delete status 'fail' or success</li>
#                           </ul>
#                   </ul> | ref.hash
#========================================================================================
sub delete {
    my $self = shift;

    # get store name
    my $StoreName = GetCurrentDBHandle()->storeName();

    Error('ResourceNotFound', { 'Reason' => 'delete not supported', 'Database' => $StoreName} );
}

1;
