#========================================================================================
# package      DE_EPAGES::MultiStoreMaster::API::Attributes::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  Implements MultiStore attributes for shop
#========================================================================================
package DE_EPAGES::MultiStoreMaster::API::Attributes::Shop;
use base DE_EPAGES::Object::API::BaseAttribute;

use strict;
use DE_EPAGES::MultiStoreMaster::API::AffiliateVisibility qw(
    AddShopsToAffiliateVisibility
    RemoveAllShopsFromAffiliateVisibility
);
use DE_EPAGES::MultiStoreMaster::API::AffiliateOverwritable qw( GetAffiliateOverwritableAttributes );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::WebService::API::Client;

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAttributes( $Object, $aNames );
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               Use $Object->get() instead.
#----------------------------------------------------------------------------------------
# input        $Object     | object          | object
# input        $aNames     | list of attribute names | array ref
# input        $LanguageID | language id | int
# return       $hValues    | attribute name => value hash | hash ref
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ( $Object, $aNames, $LanguageID ) = @_;
    my $hInfo;

    foreach my $Name ( @{$aNames} ) {
        if($Name eq 'AssignedSubShops'){
            my $ShopAlias = $Object->alias;
            my $hWebServiceConfig = $Object->get( [qw( MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword )] );

            my $WebServiceURL = $hWebServiceConfig->{'MS_WebServiceURL'}{'Site'};
            my $WebServiceUser = $hWebServiceConfig->{'MS_WebServiceUser'}{'Site'};
            my $WebServicePassword = $hWebServiceConfig->{'MS_WebServicePassword'}{'Site'};

            # TODO: Send warning to provider mail if configuration is missing!
            #       Will be done with EPG-28613.
            return if !$WebServiceURL || !$WebServiceUser || !$WebServicePassword;

            my $MSConfigService = DE_EPAGES::WebService::API::Client
                ->uri( 'urn://epages.de/WebService/MultiStoreConfigService/2011/08' )
                ->proxy( $WebServiceURL )
                ->userinfo( $WebServiceUser . ':' . $WebServicePassword )
                ->xmlschema('2001');

            my $ProviderPath = LoadRootObject()->get('ProviderPath');
            $hInfo->{$Name} = $MSConfigService->getInfo({
                'ProviderPath' => $ProviderPath,
                'MasterShopAlias' => $ShopAlias,
            })->result()->{'SubShops'};
        }
        elsif($Name eq 'MS_OverwritableAttributes'){
            my %Values;
            foreach my $Info ( @{ GetAffiliateOverwritableAttributes($Object) } ){
                my $Key = $Info->get('ClassName');
                my $AttributeName = $Info->get('AttributeName');
                $Values{$Key} =
                    ref $Values{$Key} eq 'ARRAY'
                    ? [ @{$Values{$Key}}, $AttributeName ]
                    : [ $AttributeName ]
                ;
            }
            $hInfo->{$Name} = \%Values;
        }
    }
    return $hInfo;
}

#========================================================================================
# function     setAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttribute( $Object, $Name, $Value, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Sets a single attribute value. Don't call this function directly.
#               Use $Object->set() instead.
#               The default implementation calls <function setAttribute>.
#----------------------------------------------------------------------------------------
# input        $Object     | object | object
# input        $Name       | attribute name  | string
# input        $Value      | attribute value | string
# input        $LanguageID | language id | int
#========================================================================================
sub setAttributes {
    my $self = shift;
    my ( $Object, $hValues ) = @_;

    foreach my $Key ( keys %$hValues ) {
        if ( $Key eq 'AssignedSubShops' ) {
            RemoveAllShopsFromAffiliateVisibility($Object);
            AddShopsToAffiliateVisibility( $Object, $hValues->{$Key} );
        }
    }
}

1;
