#========================================================================================
# package      DE_EPAGES::MultiStoreMaster::Hooks::ContentObject
# state        private
#========================================================================================
package DE_EPAGES::MultiStoreMaster::Hooks::ContentObject;

use strict;
use XML::Simple;
use JSON::XS;

use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::File qw (
    fsplit
    WriteFile
);
use DE_EPAGES::Object::API::Factory qw (
    GetRootObjectID
    LoadClassByAlias
    SplitObjectPath
);
use DE_EPAGES::MultiStoreCommon::API::MultiStoreLog qw (
    MultiStoreLogDebug
    MultiStoreLogDebugObjectPath
);
use DE_EPAGES::MultiStoreCommon::API::Synchronization qw (
    GetCategoryObjXML
    GetDefaultValues
);
use DE_EPAGES::MultiStoreCommon::API::ObjectUpdate qw (
    InsertObjectUpdate
    TestObjectClass
    IsSuperProduct
    ObjectPathToFilePath
); ##no critic qw(RequireCorrectImports)

#========================================================================================
# function     OnExportContentObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnExportContentObjects($hParams);
#----------------------------------------------------------------------------------------
# description  API_MS_ExportContentObjects triggered
#               Implements additional functionality for importing categories.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments and return values, used keys are
#               <ul>
#                   <li>MainFolder - String</li>
#                   <li>FileName - String</li>
#                   <li>TmpFile - String</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnExportContentObjects {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnExportContentObjects() START');

    #Treat Categories a different way splitting them into two parts: Category, CategoryProductMap.
    if( $hParams->{'MainFolder'} eq 'Categories' ){

        MultiStoreLogDebug('Hook OnExportContentObjects() Process category');
        my $XMLheader = q(<?xml version='1.0' encoding='utf-8'?>);
        my $FileName = $hParams->{'FileName'};
        my $TmpFile = $hParams->{'TmpFile'};

        my $CatObjXML = XMLout(
            GetCategoryObjXML(
                XMLin( $TmpFile, 'KeepRoot' => 1 )
            ),
            'KeepRoot' => 1,
            'XMLDecl' => $XMLheader
        );
        MultiStoreLogDebug('Hook OnExportContentObjects() Write File > '.$FileName);
        WriteFile($FileName, \$CatObjXML, { 'create_directory' => 1, 'flock'=> 1, 'utf8' => 1 } );

        my ($Path, $Name) = fsplit($FileName);
        my $CatProdMapXML = XMLout(
            _GetCatProductMapXML(
                XMLin($TmpFile, 'KeepRoot' => 1)
            ),
            'KeepRoot' => 1,
            'XMLDecl' => $XMLheader
        );

        $FileName = "$Path/$Name.cpm";
        MultiStoreLogDebug('Hook OnExportContentObjects() Write File > '.$FileName);
        WriteFile($FileName, \$CatProdMapXML, { 'create_directory' => 1, 'flock'=> 1, 'utf8' => 1 } );
        # Set done to avoid overwriting category.xml.
        $hParams->{'Done'} = 1;
    }

    MultiStoreLogDebug('Hook OnExportContentObjects() END');

    return;
}

#========================================================================================
# function     _GetCatProductMapXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetCatProductMapXML($hXML);
#----------------------------------------------------------------------------------------
# description  Returns the category product map from an exported category objects xml file.
#----------------------------------------------------------------------------------------
# input        $hXML | Category object | ref.hash
# return       $hXML | Category product map | ref.hash
#========================================================================================
sub _GetCatProductMapXML {
    my ($hXML) = @_;

    my $CatType = exists $hXML->{'epages'}{'CategorySpecialOffer'}
                ? 'CategorySpecialOffer'
                : 'Category';

    foreach my $key ( keys %{ $hXML->{'epages'}{$CatType} } ){
        if( $key ne 'Alias' && $key ne 'CategoryProductMap' && $key ne 'Class' ){
            delete $hXML->{'epages'}{$CatType}{$key};
        }
    }
    return $hXML;
}

#========================================================================================
# function     OnInitObjectUpdate
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInitObjectUpdate($hParams);
#----------------------------------------------------------------------------------------
# description  API_MS_InitObjectUpdate triggered
#               Implements additional functionality for importing categories with '-expotall'.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments and return values, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnInitObjectUpdate {
    my ($hParams) = @_;

    my $Object = $hParams->{'Object'};
    MultiStoreLogDebugObjectPath('Hook OnInitObjectUpdate() START with object > %s', $Object);

    if( $Object->instanceOf('Category') ){
        MultiStoreLogDebug('Hook OnInitObjectUpdate() Process Category');
        foreach my $SubChild ( @{ $Object->get('RecursiveSubCategories') } ){
            InsertObjectUpdate($SubChild) if TestObjectClass($SubChild, GetDefaultValues()->{'MASTER_CLASSES_LISTENED_TO'});
        }
    }

    MultiStoreLogDebug('Hook OnInitObjectUpdate() END');

    return;
}

#========================================================================================
# function     OnGetAllAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetAllAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_OnGetAllAttributes triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Attribute</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetAllAttributes {
    my ($hParams) = @_;

    return unless $DE_EPAGES::MultiStoreCommon::API::Synchronization::IS_MULTISTORE_EXIMPORT;

    my $Object = $hParams->{'Object'};
    my $aNames = $hParams->{'Names'};
    my $hValues = $hParams->{'Values'};

    MultiStoreLogDebugObjectPath('Hook OnGetAllAttributes() START on object %s', $Object);

    my $aAttributes = [ grep {
        $_->get('IsExportable')
        && $_->alias ~~ $aNames
    } @{ $Object->class->attributes } ];

    foreach my $Attr (@$aAttributes){
        my $AttrAlias = $Attr->alias;
        my $AttrType = $Attr->get('Type');

        if (ElementOf($AttrType, qw( LocalizedString LocalizedFile ))) {
            $hValues->{$AttrAlias} = {} unless exists $hValues->{$AttrAlias};
            foreach my $Language ( @{ $Object->getSite()->get('Languages') } ){
                my $LanguageID = $Language->{'LanguageID'};
                $hValues->{$AttrAlias}{$LanguageID} = '' unless exists $hValues->{$AttrAlias}{$LanguageID};
            }
        }
        elsif (ElementOf($AttrType,
                   qw( String File EMail ExternalPassword SpaceString Integer
                       Float Money DateTime Date Time Duration ))) {
            $hValues->{$AttrAlias} //= '';
        }
        elsif ($AttrType eq 'Boolean') {
            $hValues->{$AttrAlias} //= 0;
        }
    }

    MultiStoreLogDebug('Hook OnGetAllAttributes() END');

    return;
}

#========================================================================================
# function     OnAfterUpdateAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateAttribute($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_AfterUpdateAttribute triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - DE_EPAGES::Object::API::Object::Attribute</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateAttribute {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnAfterUpdateAttribute() START');

    OnAfterUpdateContentObject({
        'Object' => $hParams->{'Object'}->get('AttributeClass'),
        'Values' => {},
    });

    MultiStoreLogDebug('Hook OnAfterUpdateAttribute() END');

    return;
}

#========================================================================================
# function     OnInsertContentObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertContentObject($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_InsertContentObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertContentObject {
    my ($hParams) = @_;

    my $Object = $hParams->{'Object'};

    MultiStoreLogDebugObjectPath('Hook OnInsertContentObject() START on object %s', $Object);

    my $ClassName = _FilterContentObject($Object);
    return unless $ClassName;

    MultiStoreLogDebug('Hook OnInsertContentObject() Valid ClassName ... continue ...');

    # Set FilePathPart
    my $Shop = $Object->getSite;

    # Quite effective, but poorly efficent.
    # Set parent object to be updated, too, unless it's a root folder.
    # If a sub category is added within the object tree, the SubShop doesn't know about
    # it and has to be notified that the parent (resp. one child) has been changed.
    my $Parent;
    if ($ClassName eq 'SubProduct') {
        $Parent = $Object->get('SuperProduct');
    }
    else {
        my $ObjectPath = $Shop->relativePath($Object);
        $Object->set({ 'FilePathPart' => ObjectPathToFilePath($ObjectPath) });
        InsertObjectUpdate($Object);
        $Parent = $Object->parent;
    }
    return unless defined $Parent;

    InsertObjectUpdate($Parent) if scalar @{ SplitObjectPath( $Shop->relativePath( $Parent ) ) } > 1;

    MultiStoreLogDebug('Hook OnInsertContentObject() END');

    return;
}

#========================================================================================
# function     OnAfterUpdateContentObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateContentObject($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_AfterUpdateContentObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateContentObject {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnAfterUpdateContentObject() START');

    my $Object = $hParams->{'Object'};
    my $ClassName = _FilterContentObject($Object);
    return unless $ClassName;

    if ($ClassName eq 'SubProduct') {
        $Object = $Object->get('SuperProduct');
    }

    my $hUpdates = _UpdateValues($hParams->{'Values'}, $ClassName);
    return if ref $hUpdates eq 'HASH' && 0 == scalar keys %$hUpdates;

    my $UpdateString = defined $hUpdates ? encode_json($hUpdates) : undef;
    InsertObjectUpdate($Object, undef, undef, $UpdateString, $ClassName);

    MultiStoreLogDebug('Hook OnAfterUpdateContentObject() END');

    return;
}

# returns the updated values as JSON string
# returns undef if all values need to be updated
sub _UpdateValues {
    my ($hValues, $ClassName) = @_;

    my %AttributeUpdates = map { $_ => undef } @{ GetDefaultValues()->{'AttributeUpdate'}->{$ClassName} };

    my %Values;
    foreach my $AttributeAlias (keys %$hValues) {
        if (exists $AttributeUpdates{$AttributeAlias}) {
            $Values{$AttributeAlias} = $hValues->{$AttributeAlias};
        }
        elsif ($AttributeAlias ne '_StockLevel') { # EPG-34512
            return undef; # requires re-import of the complete object
        }
    }

    return \%Values;
}

#========================================================================================
# function     OnAfterInsertTemplateMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterInsertTemplateMap($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_OnAfterInsertTemplateMap triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterInsertTemplateMap{
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnAfterInsertTemplateMap() START');

    my $Shop = $hParams->{'Object'}->get('Object');
    return if $Shop->id == GetRootObjectID();

    InsertObjectUpdate($Shop);

    MultiStoreLogDebug('Hook OnAfterInsertTemplateMap() END');

    return;
}

#========================================================================================
# function     OnDeleteContentObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteContentObject($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_DeleteContentObject triggered
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteContentObject {
    my ($hParams) = @_;

    my $Object = $hParams->{'Object'};
    MultiStoreLogDebugObjectPath('Hook OnDeleteContentObject() START on object %s', $Object);

    # Remove from VisibilityMap.
    my $AffiliateMapClass = LoadClassByAlias('AffiliateVisibilityMap');
    $AffiliateMapClass->deleteObjects( 'ByMasterShopContentObjectID', [ $Object->id ] );

    my $ClassName = _FilterContentObject($Object);
    return unless defined $ClassName;

    # Set 'Remove' flag for sync script.
    InsertObjectUpdate($Object, 1);

    MultiStoreLogDebug('Hook OnDeleteContentObject() END');

    return;
}

#========================================================================================
# function     _FilterContentObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _FilterContentObject( $Object );
#----------------------------------------------------------------------------------------
# description  Tests if object shall be processed.
#----------------------------------------------------------------------------------------
# input        $Object | content object | DE_EPAGES::Content::API::Object::ContentObject
# return       $ClassName | return the objects class name or else 'undef' | string
#========================================================================================
sub _FilterContentObject {
    my ($Object) = @_;

    my $ClassName = TestObjectClass($Object, GetDefaultValues()->{'MASTER_CLASSES_LISTENED_TO'});
    return unless $ClassName;
    return 'SubProduct' if $ClassName eq 'Product' && !IsSuperProduct($Object);
    return $ClassName;
}

#========================================================================================
# function     OnInsertCategoryProductMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertCategoryProductMap($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_InsertCategoryProductMap triggered
#               CategoryProductMap has been changed -> bookmark category for export
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertCategoryProductMap {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnInsertCategoryProductMap() START');

    InsertObjectUpdate( $hParams->{'Object'}->get('Category') );

    MultiStoreLogDebug('Hook OnInsertCategoryProductMap() END');

    return;
}

#========================================================================================
# function     OnDeleteCategoryProductMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteCategoryProductMap($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_DeleteCategoryProductMap triggered
#               CategoryProductMap has been changed -> bookmark category for export
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteCategoryProductMap {
    my ($hParams) = @_;

    MultiStoreLogDebug('Hook OnDeleteCategoryProductMap() START');

    InsertObjectUpdate( $hParams->{'Object'}->get('Category') );

    MultiStoreLogDebug('Hook OnDeleteCategoryProductMap() END');

    return;
}

#========================================================================================
# function     OnUpdateCategoryProductMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnUpdateCategoryProductMap($hParams);
#----------------------------------------------------------------------------------------
# description  OBJ_AfterUpdateCategoryProductMap triggered
#               CategoryProductMap has been changed -> bookmark category for export
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - local object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateCategoryProductMap {
    my ($hParams) = @_;

    InsertObjectUpdate( $hParams->{'Object'}->get('Category') );
    return;
}


1;
