#========================================================================================
# package      DE_EPAGES::Order::API::Object::LineItemPaymentDiscount
# state        public
# base         DE_EPAGES::Order::API::Object::LineItem
# description  object interface for LineItemPaymentDiscount.
#========================================================================================
package DE_EPAGES::Order::API::Object::LineItemPaymentDiscount;
use base qw( DE_EPAGES::Order::API::Object::LineItem );

use strict;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Payment::API::Constants qw ( CASHDISCOUNTMODE_NO_DISCOUNT CASHDISCOUNTMODE_PERCENT_DISCOUNT CASHDISCOUNTMODE_FIX_DISCOUNT
                                            CASHDISCOUNTMODE_FIX_FEE CASHDISCOUNTMODE_PERCENT_FEE );

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDiscount->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentDiscount' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentDiscount | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID - line item paymentdiscount id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentDiscount', {'Object' => $self, 'LineItemID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     lineItemInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hLineItem = $Object->lineItemInfo( $Container );
#----------------------------------------------------------------------------------------
# description  Build a lineitem like hash for all lineiteminformation. Return value can
#               be undef if no container is submitted or payment method provides no discount.
#----------------------------------------------------------------------------------------
# input        $Container | line item container (optional) | object
# return       $hLineItem |  hash for lineitem insert, keys <ul>
#                   <li>Quantity
#                   <li>BasePrice
#                   <li>LineItemPrice
#               </ul> | ref.hash
#========================================================================================
sub lineItemInfo {
    my $self = shift;
    my ($Container) = @_;
    return undef unless defined $Container;

    my $PaymentLineItem = $Container->get('Payment');
    return undef unless defined $PaymentLineItem;

    # test if we have a per cent discount/fee or a fix discount/fee

    # try to get the discount mode
    my $PaymentLineItemMethod = $PaymentLineItem->get('PaymentMethod');
    return undef unless defined $PaymentLineItemMethod;

    my $CashDiscountMode = $PaymentLineItemMethod->get('CashDiscountMode');

    my $DiscountValue = 0;
    my $Discount = 0;

    # use only disountable lineitems for calculation
    my $BasePrice = $Container->get('PaymentSubTotal');
    my $DiscountableSumDiff = $Container->get('LineItemsSum') - $Container->get('LineItemsSubTotalDiscountable');
    $BasePrice -= $DiscountableSumDiff;

    if( $CashDiscountMode == CASHDISCOUNTMODE_NO_DISCOUNT ) {
        return undef;
    }
    # per cent discount or fee handling
    elsif( $CashDiscountMode == CASHDISCOUNTMODE_PERCENT_DISCOUNT or
           $CashDiscountMode == CASHDISCOUNTMODE_PERCENT_FEE ) {
        # is a per cent discount or fee defined?
        my $PerCentDiscountOrFee = defined $PaymentLineItem? $PaymentLineItem->get('PaymentMethod')->get('CashDiscount') : 0;
        return undef if $PerCentDiscountOrFee == 0;

        # do we have a discount or a fee
        ## no critic qw(ProhibitMagicNumbers)
        my $Div = $CashDiscountMode == CASHDISCOUNTMODE_PERCENT_DISCOUNT ? -100 : 100;
        ## use critic qw(ProhibitMagicNumbers)
        $DiscountValue = $BasePrice * $PerCentDiscountOrFee / $Div;
        $Discount = $PerCentDiscountOrFee;
    }
    # fix discount or fee handling
    elsif( $CashDiscountMode == CASHDISCOUNTMODE_FIX_DISCOUNT or
           $CashDiscountMode == CASHDISCOUNTMODE_FIX_FEE ) {
        # is a fix discount or fee defined?

        my $CurrencyID = $Container->get('CurrencyID') ;
        my $FixDiscountOrFee = defined $PaymentLineItem? $PaymentLineItem->get('PaymentMethod')->getPaymentMethodDiscount($CurrencyID) : 0;
        return undef if (not defined $FixDiscountOrFee) or ($FixDiscountOrFee == 0);

        # do we have a discount or a fee
        $DiscountValue = $CashDiscountMode == CASHDISCOUNTMODE_FIX_DISCOUNT ?  - $FixDiscountOrFee  : $FixDiscountOrFee;
        $Discount = $FixDiscountOrFee;
    }

    return undef if $DiscountValue == 0;

    my %Info = (
        'Quantity'              => 1,
        'BasePrice'             => $BasePrice ,
        'LineItemPrice'         => $Container->formatter->round($DiscountValue),
        'Discount'              => $Discount,
        'DiscountMode'          => $CashDiscountMode,
    );
    return \%Info;
}

#========================================================================================
# function     recalculate
#----------------------------------------------------------------------------------------
# syntax       $LineItem->recalculate($Container);
#----------------------------------------------------------------------------------------
# description  Recalculate global value discount item or delete it, if it is not valid anymore.
#----------------------------------------------------------------------------------------
# input        $Container | container | object
#========================================================================================
sub recalculate {
    my $self = shift;
    my ($Container) = @_;

    if ($self->get('IsPatched')) {
        return;
    }
    my $hInfo = $self->lineItemInfo($Container);
    if (defined $hInfo) {
        $self->set($hInfo);
    }else {
        #not valid
        $self->delete;
    }
    return;
}

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDiscount = DE_EPAGES::Order::API::Object::LineItemPaymentDiscount->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentDiscount');
#               my $LineItemPaymentDiscount = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentDiscount' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentDiscount | new object | object
# hook         OBJ_InsertLineItemPaymentDiscount | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemID    - line item payment discount id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertLineItemPaymentDiscount', {'Object' => $self, 'LineItemID' => $self->id });
    return $self;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDiscount->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentDiscount' and 'OBJ_AfterUpdateLineItemPaymentDiscount'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentDiscount | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDiscountID - line item payment discount id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentDiscount | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDiscountID - line item payment discount id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentDiscount', {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentDiscountID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemPaymentDiscount',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentDiscountID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

1;
