#========================================================================================
# package      DE_EPAGES::Order::API::OtherLineItemsData
# state        public
#----------------------------------------------------------------------------------------
# description  Class to manipulate secondary LineItems 
#              (BasketDiscount, Shipping Options, Deposit and eco participation)
#========================================================================================
package DE_EPAGES::Order::API::OtherLineItemsData;

use DE_EPAGES::Dictionary::API::Template;
use DE_EPAGES::Dictionary::API::Localizer qw ( Replace );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Order::API::LineItemPriceCalculator;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Shop::API::Resource::Formatter;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Order::API::OtherLineItemsData
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     getOtherLineItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       getOtherLineItems($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem );
#----------------------------------------------------------------------------------------
# description  Return secondary LineItems of the Container
#----------------------------------------------------------------------------------------
# input        $Container | DE_EPAGES::Product::API::Object::LineItemProduct
# input        $Formatter | DE_EPAGES::Shop::API::Resource::Formatter
# input        $CurrencyID | int
# input        $IncludeTaxInLineItem | Flag to decide if the LineItem price should be show in Net or Gross |String
# return       refhash
#========================================================================================
sub getOtherLineItems {
    my $self = shift;
    my ($Container, $Formatter, $LanguageID, $IncludeTaxInLineItem) = @_;

    my $CurrencyID = $Container->get('CurrencyID');
    my $Context = $self->_createShopRequestContext($Container->getSite);
    return [
        map { $self->_createLineItem($_, $CurrencyID, $Formatter, $Context, $LanguageID, $IncludeTaxInLineItem) }
        @{ $Container->children() }
    ];
}

sub _createLineItem {
    my $self = shift;
    my ($LineItem, $CurrencyID, $Formatter, $Context, $LanguageID, $IncludeTaxInLineItem) = @_;

    return unless ($LineItem->instanceOf('LineItemDeposit')
         || $LineItem->instanceOf('LineItemEcoParticipation')
         || $LineItem->instanceOf('LineItemShippingOption')
         || $LineItem->instanceOf('LineItemGlobalValueDiscount'));

    my $LineItemPrice = $LineItem->get('LineItemPrice');
    my $TaxModelLineItem = $LineItem->get('Parent')->get('TaxModel');

    my $LineItemType;
    my $LineItemName;
    my $LineItemProductGUID;
    if($LineItem->instanceOf('LineItemDeposit')){
       $LineItemName = $self->_createDepositName($LanguageID);
       $LineItemType = 'deposit';
       $LineItemProductGUID = $self->_getLineItemProductGuidFromDeposit($LineItem);
    }
    if($LineItem->instanceOf('LineItemEcoParticipation')){
       $LineItemName = $self->_createEcoParticipationName($LanguageID);
       $LineItemType = 'ecoParticipation';
    }
    if($LineItem->instanceOf('LineItemGlobalValueDiscount')){
       $LineItemName = $self->_createGlobalValueDiscountName($LanguageID);
       $LineItemType = 'basketDiscount';
    }
    if($LineItem->instanceOf('LineItemShippingOption')){
       $LineItemName = $self->_createShippingOptionName($LanguageID);
       $LineItemType = 'shippingOption';
    }
    my $hValues = { 
       type => $LineItemType,
       name => $LineItemName,
       lineItemPrice => $Formatter->price($LineItemPrice, $CurrencyID, $TaxModelLineItem),
    };
    $hValues->{'taxClass'} = $self->_createTaxClass($LineItem, $LanguageID);
    if(defined $LineItemProductGUID) {$hValues->{'lineItemProductGuid'} = $LineItemProductGUID;}
    return $hValues;
}

sub _getLineItemProductGuidFromDeposit{
    my $self = shift;
    my ($LineItem) = @_;

    my $aProductLines = $LineItem->container->childrenByClass('LineItemProduct');
    foreach ( @$aProductLines ) {
      if ( $_->get('LineItemDeposit') && $_->get('LineItemDeposit')->guid eq $LineItem->guid ) {
        return $_->guid;
      }
    }

    return '';
}

sub _createTaxClass{
    my $self = shift;
    my ($LineItem, $LanguageID) = @_;

    my $TaxClass = $LineItem->get('TaxClassDigital') // $LineItem->get('TaxClass');
    return unless defined $TaxClass;

    return {
        'taxClassId' => $TaxClass->guid(),
        'name' => $TaxClass->get('Name', $LanguageID),
        'percentage' => $LineItem->get('TaxRate') * 100,
    };
}

sub _createShippingOptionName {
    my $self = shift;
    my ($LanguageID) = @_;
    my $hLangInfo = $self->_GetInfoLanguage($LanguageID);
    my $NameTemplate = '{ShippingOption}';
    my $CartridgeName = 'ShippingOption';
    return $self->_getNameFromTranslationFiles($NameTemplate, $hLangInfo ,$CartridgeName);
}

sub _createGlobalValueDiscountName {
    my $self = shift;
    my ($LanguageID) = @_;
    my $hLangInfo = $self->_GetInfoLanguage($LanguageID);
    my $NameTemplate = '{GlobalValueDiscount}';
    my $CartridgeName = 'Dictionary';
    return $self->_getNameFromTranslationFiles($NameTemplate, $hLangInfo ,$CartridgeName);
}

sub _createEcoParticipationName {
    my $self = shift;
    my ($LanguageID) = @_;
    my $hLangInfo = $self->_GetInfoLanguage($LanguageID);
    my $NameTemplate = '{EcoParticipation}';
    my $CartridgeName = 'Product';
    return $self->_getNameFromTranslationFiles($NameTemplate, $hLangInfo ,$CartridgeName);
}

sub _createDepositName {
    my $self = shift;
    my ($LanguageID) = @_;
    my $hLangInfo = $self->_GetInfoLanguage($LanguageID);
    my $NameTemplate = '{Deposit}';
    my $CartridgeName = 'Product';
    return $self->_getNameFromTranslationFiles($NameTemplate, $hLangInfo ,$CartridgeName);
}

sub _getNameFromTranslationFiles {
    my $self = shift;
    my ($NameTemplate, $hLangInfo, $CartridgeName) = @_;

    my $LanguageCode2 = $hLangInfo->{'Code2'};
    my $DictionaryXml = $ENV{'EPAGES_CARTRIDGES'} . "/DE_EPAGES/$CartridgeName/Templates/Dictionary.$LanguageCode2.xml";
    my $Template = $self->_createTemplate();
    my $Dictionary = $Template->dictionary( $DictionaryXml, $LanguageCode2 );
    my $hTranslationStrings = $Dictionary->{$LanguageCode2};

    my $NameFromTranslationFiles = $self->_replace($NameTemplate, $hTranslationStrings);
    return $NameFromTranslationFiles;
}

sub _GetInfoLanguage {
    my $self = shift;
    my ($LanguageID) = @_;
    return GetInfoLanguage($LanguageID);
}

sub _replace {
    my $self = shift;
    my ($NameTemplate, $hTranslationStrings) = @_;
    return Replace($NameTemplate, $hTranslationStrings);
}

sub _createTemplate {
    my $self = shift;
    my $Template = DE_EPAGES::Dictionary::API::Template->new();
    return $Template;
}

sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}

1;