#========================================================================================
# package      DE_EPAGES::Order::API::Resource::CartCollection
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource
# description  REST resource for the carts collection.
#========================================================================================
package DE_EPAGES::Order::API::Resource::CartCollection;
use base DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource;

use DE_EPAGES::Core::API::Url qw( ParseQueryString );
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::TestParameter qw( TestInteger );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Order::API::CartSearchService;
use DE_EPAGES::Order::API::CartService;
use DE_EPAGES::Order::API::Resource::Cart;
use DE_EPAGES::Search::API::Object::PagerSearch qw();
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DateTime::Format::Strptime;
use JSON;

use strict;

use constant DEFAULT_PAGE_SIZE => 10;
use constant DEFAULT_MAX_PAGE_SIZE => 100;


sub service {
    return LoadObjectByPath('/RestServices/CartCollection');
}

sub class {
    return 'Shop';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            [ 'get', 'application/vnd.epages.v1+json' ],
            [ 'get', 'application/json' ],
        ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/carts/?$|;
    return 1;
}


sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if ($ActionName eq 'get') {
        my $hQuery = ParseQueryString($Request->uri->query, 'utf-8');
        return [ $Object, $hQuery ];
    }
    else {
        return [ $Object ];
    }
}

sub get {
    my $self = shift;
    my ($Shop, $hQuery) = @_;

    my $ResultsPerPage = $hQuery->{'resultsPerPage'}->[0];
    my $Page = $hQuery->{'page'}->[0];
    my $IncludeTax = $hQuery->{'includeTax'}->[0];

    $Page //= 1;
    $ResultsPerPage //=  DEFAULT_PAGE_SIZE;

    eval {
        TestInteger('page', $Page, 1);
        TestInteger('resultsPerPage', $ResultsPerPage, 1, DEFAULT_MAX_PAGE_SIZE);
    };
    if ( ExistsError() ) {
        return;
    }

    my $CartSearchService = $self->_createCartSearchService($Shop, $ResultsPerPage, $Page, $hQuery);

    $CartSearchService->ConvertEpagesJValuesToEpages6();
    $CartSearchService->DefineSearchParameters();

    my $hSearchParameters = $CartSearchService->GetSearchParameters();

    # Get all the Cart-ids meeting search options.
    my $Search = $self->_createPagerSearch($hSearchParameters);
    # Get total Cart count meeting search options.
    my $Count = $Search->{'ItemCount'};
    # Filter Cart-ids which belong to the asked page
    my $aCarts = $Search->items($Page);

    # Get Cart-details for selected Cart-ids
    my @aResultCarts = ();
    my $CartService = $self->_createCartService();

    foreach my $Cart (@{$aCarts}){
        my $hResponse = $CartService->getCartData($Cart, $IncludeTax);
        push(@aResultCarts, $hResponse);
    }

    return {
        results         => $Count,
        page            => $Page,
        resultsPerPage  => $ResultsPerPage,
        items           => \@aResultCarts,
    };

}

sub _createCartSearchService {
    my $self = shift;
    my ($Shop, $ResultsPerPage, $Page, $hQuery) = @_;

    return DE_EPAGES::Order::API::CartSearchService->new($Shop, $ResultsPerPage, $Page, $hQuery);
}

sub _createPagerSearch {
    my $self = shift;
    my ($hOptions) = @_;
    return DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
}

sub _createCartService {
    my $self = shift;
    return DE_EPAGES::Order::API::CartService->new();
}


1;
