#========================================================================================
# package      DE_EPAGES::Order::API::Resource::OrderCollection
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource
# description  REST resource for get Orders
#========================================================================================
package DE_EPAGES::Order::API::Resource::OrderCollection;
use base DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource;

use DE_EPAGES::Order::API::OrderSearchService;
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Core::API::Error qw (ExistsError);
use DE_EPAGES::Core::API::TestParameter qw (TestInteger);
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Order::API::WebService::OrderService;
use DE_EPAGES::Order::API::Object::CustomerOrder;
use DE_EPAGES::Order::API::OrderService;
use DE_EPAGES::Search::API::Object::PagerSearch qw ();
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DateTime::Format::Strptime;
use JSON;

use strict;

use constant DEFAULT_PAGE_SIZE => 10;
use constant DEFAULT_MAX_PAGE_SIZE => 100;
use constant ZERO_VALUE => 0;


sub service {
    return LoadObjectByPath('/RestServices/OrderCollection');
}



sub class {
    return 'Shop';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/orders/?$|;
    return 1;
}



sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if ( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $ResultsPerPage = $hQuery->{'resultsPerPage'}->[0];
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $Page = $hQuery->{'page'}->[0];
        return [ $Object, $LocaleID, $ResultsPerPage, $Page, $hQuery];
    }
    else {
        return [ $Object ];
    }
}

sub get {
    my $self = shift;
    my ($Shop, $LocaleID, $ResultsPerPage, $Page, $hQuery) = @_;

    $Page //= 1;
    $ResultsPerPage //=  DEFAULT_PAGE_SIZE;

    eval {
        TestInteger('page', $Page, 1);
        TestInteger('resultsPerPage', $ResultsPerPage, 1, DEFAULT_MAX_PAGE_SIZE);
    };
    if ( ExistsError() ) {
        return;
    }

    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $OrderSearchService = $self->_createOrderSearchService($Shop, $ResultsPerPage, $Page, $hQuery);

    $OrderSearchService->ConvertEpagesJValuesToEpages6();
    $OrderSearchService->DefineSearchParameters();

    my $hSearchParameters = $OrderSearchService->GetSearchParameters();

    # Get all the order-ids meeting search options.
    my $Search = $self->_createPagerSearch($hSearchParameters);
    # Get total order count meeting search options.
    my $Count = $Search->{'ItemCount'};
    # Filter order-ids which belong to the asked page
    my $aOrders = $Search->items($Page);

    # Get Order-details for selected order-ids
    my @aResultOrders = ();
    my $OrderService = $self->_createOrderService();

    foreach my $Order (@{$aOrders}){
        my $hResponse = $OrderService->getOrderData($Order, $LocaleID);
        push(@aResultOrders, $hResponse);
    }

    return {
        results         => $Count,
        page            => $Page,
        resultsPerPage  => $ResultsPerPage,
        items           => \@aResultOrders,
    };
}

sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}

sub _createPagerSearch {
    my $self = shift;
    my ($hOptions) = @_;
    return DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
}

sub _createOrderService {
    my $self = shift;
    return DE_EPAGES::Order::API::OrderService->new();
}

sub _createOrderSearchService {
    my $self = shift;
    my ($Shop, $ResultsPerPage, $Page, $hQuery) = @_;

    return DE_EPAGES::Order::API::OrderSearchService->new($Shop, $ResultsPerPage, $Page, $hQuery);
}


1;
