#========================================================================================
# package      DE_EPAGES::Order::API::Resource::OrderDocumentResource
# base         DE_EPAGES::Order::API::Resource::BaseOrderResource
# state        public
#----------------------------------------------------------------------------------------
# description  Implements the REST resource for order documents of an order
#========================================================================================
package DE_EPAGES::Order::API::Resource::OrderDocumentResource;
use base DE_EPAGES::Order::API::Resource::BaseOrderResource;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Order::API::OrderDocumentService;
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;

use strict;

use constant DEFAULT_PAGE_SIZE => 10;
use constant DEFAULT_MAX_RESULTS_PER_PAGE_SIZE => 100;


sub service {
    return LoadObjectByPath('/RestServices/OrderDocument');
}

sub match {
    my $self = shift;
    my ($Request) = @_;
    return unless $Request->uri->path =~ m|^/api/orders/(.+)/documents/?$|;
    return $1;
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $ResultsPerPage = $hQuery->{'resultsPerPage'}->[0];
        my $Page = $hQuery->{'page'}->[0];
        return [ $Object, $ResultsPerPage, $Page, $hQuery];
    }
    else {
        return [ $Object ];
    }
}

sub get {
    my $self = shift;
    my ($Order, $ResultsPerPage, $Page, $hQuery) = @_;

    LogDebug('Query Parameters', $hQuery);

    my $Shop = $Order->getSite();
    my $Context = $self->_createShopRequestContext($Shop);

    my $LocaleID = $Context->validLocale($hQuery->{'locale'}->[0]);
    my $IncludePdf = ( lc($hQuery->{'includePdf'}->[0]) eq 'true' ) ? 1 : 0;

    $Page //= 1;
    $ResultsPerPage //=  DEFAULT_PAGE_SIZE;

    eval {
        TestInteger('page', $Page, 1);
        TestInteger('resultsPerPage', $ResultsPerPage, 1, DEFAULT_MAX_RESULTS_PER_PAGE_SIZE);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        Error('BadRequest', {
            'Reason' => $Error->shortMessage(),
        }, $Error);
    }

    my $hSearchQueryParams = $self->_getSearchQueryParameters($Order, $ResultsPerPage);

    LogDebug('Search Query Parameters', $hSearchQueryParams->{'Parameters'});

    my $hOrderDocumentsPage = $self->_generateOrderDocumentsPage($hSearchQueryParams, $Page, $LocaleID, $IncludePdf);

    return $hOrderDocumentsPage;
}

#========================================================================================
# function     _getSearchQueryParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hSearchQueryParams = $self->_getSearchQueryParameters( $Order, $ResultsPerPage );
#----------------------------------------------------------------------------------------
# description  It returns query parameters hash ready to make a search on DB
#----------------------------------------------------------------------------------------
# input        $Order | Order object | object
# input        $ResultsPerPage | number of items per page to return | integer
# return       $hSearchQueryParams | DB search query parameters hash | refhash
#========================================================================================
sub _getSearchQueryParameters {
    my $self = shift;
    my ($Order, $ResultsPerPage) = @_;

    my $SearchName = 'RESTSearch';
    my $OrderBy = 'CreationDate';
    my $SearchDefinition = $self->_GetSearchDefinition($self->_LoadClassByAlias('OrderDocument'), $SearchName);

    return {
        'Object' => $Order,
        'ObjectID' => $Order->id(),
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'OrderID' => $Order->id(),
            'IsVisible' => 1,
            'Search' => $SearchName,
            'Issued' => 1, # boolean attribute used when order documents are finalized
            'OrderDocumentTypes' => ['Invoice', 'CreditNote']
        },
        'PageSize'=> $ResultsPerPage,
        'OrderBy' => $OrderBy,
        'OrderDesc' => 1,
    };
}

#========================================================================================
# function     _generateOrderDocumentsPage
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hOrderDocumentsPage = $self->_generateOrderDocumentsPage( $hSearchQueryParams, $Page, $LocaleID, $IncludePdf );
#----------------------------------------------------------------------------------------
# description  It iterates through selected order documents and returns collected order
#               documents for requested page.
#----------------------------------------------------------------------------------------
# input        $hSearchQueryParams |query parameters to get targeted order documents | refhash
# input        $Page | page number of items to show | integer
# input        $LocaleID | locale indentifier | integer
# input        $IncludePdf | include PDF encoded documents or not | boolean
# return       $hOrderDocumentsPage | collected order documents | refhash
#========================================================================================
sub _generateOrderDocumentsPage {
    my $self = shift;
    my ($hSearchQueryParams, $Page, $LocaleID, $IncludePdf) = @_;

    # Get order documents SQL searcher object.
    my $Searcher = $self->_createPagerSearch($hSearchQueryParams);
    # Get total order document count meeting search options.
    my $Count = $Searcher->countItems();
    # Get order documents which belong to the asked page
    my $aDocuments = $Searcher->items($Page);

    # Get order document-details for selected order documents
    my @OrderDocumentCollection;
    my $OrderDocumentService = $self->_createOrderDocumentService();
    foreach my $Document (@{$aDocuments}){
        my $hOrderDocumentData = $OrderDocumentService->getOrderDocumentData($Document, $LocaleID, $IncludePdf);
        push(@OrderDocumentCollection, $hOrderDocumentData);
    }

    return {
        results         => $Count,
        page            => $Page,
        resultsPerPage  => $hSearchQueryParams->{'PageSize'},
        items           => \@OrderDocumentCollection,
    };
}

#=========================================================================================
# Wrappers
#=========================================================================================
sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}

sub _createPagerSearch {
    my $self = shift;
    my ($hOptions) = @_;
    return DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
}

sub _LoadClassByAlias {
    my $self = shift;
    my ($ClassName) = @_;
    return LoadClassByAlias($ClassName);
}

sub _GetSearchDefinition {
    my $self = shift;
    my ($ResultClass, $SearchName) = @_;
    return GetSearchDefinition($ResultClass, $SearchName);
}

sub _createOrderDocumentService {
    my $self = shift;
    return DE_EPAGES::Order::API::OrderDocumentService->new();
}

1;
