#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository
# state        public
#----------------------------------------------------------------------------------------
# description  Collects data for Webservice call and writes to epages6 businessobjects
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Repository;

use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime IsA );
use DE_EPAGES::Core::API::TestParameter qw ( TestArray TestHash TestObject TestString TestFloat);
use DE_EPAGES::PayPalPlus::API::Constants qw (
    PAYPALPLUS_STATUS_COMPLETED
    PAYPALPLUS_STATUS_PENDING
    TRANSSTATUS_PAID
    TRANSSTATUS_PENDING
    PAYPALPLUS_PAY_UPON_INVOICE
);
use DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO;
use DE_EPAGES::Core::API::Object::DateTimeFormatter;

use strict;
#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'UNSUPPORTED_ARRAY_SIZE' => 'PayPal plus only support one element for #ElementName, actual size: #ActualSize',
    }
}
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     buildRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildRequest( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  builds the request data
#               Throws Error if data is not complete
#----------------------------------------------------------------------------------------
# input        $OrderOrBasket | OrderOrBasket | DE_EPAGES::Order::API::Object::CustomerOrder or
#                                                DE_EPAGES::Order::API::Object::Basket
#========================================================================================
sub buildRequest {
    my $self = shift;
    my ( $OrderOrBasket ) = @_;

    if(not IsA( $OrderOrBasket, 'DE_EPAGES::Order::API::Object::CustomerOrder' ) and not IsA( $OrderOrBasket, 'DE_EPAGES::Order::API::Object::Basket' )) {
        Error('WRONG_CLASS');
    }

    my $LineItemContainer = $OrderOrBasket->container();
    TestObject('LineItemContainer', $LineItemContainer, 'DE_EPAGES::Order::API::Object::LineItemContainer');

    my $LineItemPaymentPayPalPlus = $LineItemContainer->get('Payment');
    TestObject('LineItemPaymentPayPalPlus', $LineItemPaymentPayPalPlus, 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus');

    my $PaymentMethodPayPalPlus = $LineItemPaymentPayPalPlus->get('PaymentMethod');
    $self->buildCredentialDTO($PaymentMethodPayPalPlus);

    my $PayerID = $LineItemPaymentPayPalPlus->get('PayerID');
    TestString('PayerID', $PayerID);

    my $PaymentID = $LineItemPaymentPayPalPlus->get('TransID2');
    TestString('TransID2', $PaymentID);

    my $RequestDTO = $self->_createRequestDTO();
    $RequestDTO->payerId($PayerID);
    $RequestDTO->paymentId($PaymentID);
    $self->{'RequestDTO'} = $RequestDTO;

    return;
}

#========================================================================================
# function     getRequestDTO
# state        public
#----------------------------------------------------------------------------------------
# syntax       getRequestDTO( );
#----------------------------------------------------------------------------------------
# description  returns RequestDTO
#----------------------------------------------------------------------------------------
# return       Request DTO | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO
#========================================================================================#
sub getRequestDTO {
    my $self = shift;
    return $self->{'RequestDTO'};
}

#========================================================================================
# function     writeResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       writeResponse( $hVerifiedResponse, $CustomerOrder  );
#----------------------------------------------------------------------------------------
# description  writes Response to the CustomerOrder Object
#----------------------------------------------------------------------------------------
# input        $hVerifiedResponse  | verified Response hash | hash_ref
# input        $CustomerOrder | CustomerOrder | DE_EPAGES::Order::API::Object::CustomerOrder
#========================================================================================
sub writeResponse {
    my $self = shift;
    my ( $hVerifiedResponse, $CustomerOrder ) = @_;

    TestObject('CustomerOrder',$CustomerOrder,'DE_EPAGES::Order::API::Object::CustomerOrder');
    my $PaidOn = $CustomerOrder->get('PaidOn');

    if(defined $PaidOn) {
        Error( 'OrderAlreadyPaid' );
    }

    my $PaymentMethod = $hVerifiedResponse->{'PaymentMethod'};
    my $State = $hVerifiedResponse->{'State'};
    my $TransactionId = $hVerifiedResponse->{'TransactionId'};
    my $InvoiceData = $hVerifiedResponse->{'InvoiceData'};

    my $LineItemContainer = $CustomerOrder->container();
    my $LineItemPaymentPayPalPlus  = $LineItemContainer->get('Payment');
    TestObject('LineItemPaymentPayPalPlus',$LineItemPaymentPayPalPlus,'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus');
    my $DateTime = $self->_GetCurrentDateTime();
    my $OrderAmount = $LineItemContainer->get('GrandTotal');
    my $OrderCurrency = $LineItemContainer->get('CurrencyID');
    my $PayPalAmount = $hVerifiedResponse->{'Amount'};
    my $PayPalCurrency = $hVerifiedResponse->{'Currency'};

    my $PaidOnSelector = 'PaidOn';
    my $IsSameAmount = $self->_isSameAmount($OrderAmount, $PayPalAmount);
    if( not $IsSameAmount or $OrderCurrency ne $PayPalCurrency) {
        $PaidOnSelector = 'PartlyPaidOn';
    }

    $LineItemPaymentPayPalPlus->set( { 'InternalMethod' => $PaymentMethod} );
    $LineItemPaymentPayPalPlus->set( { 'TransID' => $TransactionId, });
    $LineItemPaymentPayPalPlus->set( { 'Amount' => $PayPalAmount } );
    $LineItemPaymentPayPalPlus->set( { 'CurrencyID' => $PayPalCurrency } );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceReferenceNumber' => $InvoiceData->{'ReferenceNumber'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoicePaymentDueDate' => $InvoiceData->{'PaymentDueDate'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceBankName' => $InvoiceData->{'BankName'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceAccountHolderName' => $InvoiceData->{'AccountHolderName'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceIBAN' => $InvoiceData->{'IBAN'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceBIC' => $InvoiceData->{'BIC'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceAmount' => $InvoiceData->{'Amount'}} );
    $LineItemPaymentPayPalPlus->set( { 'InvoiceCurrency' => $InvoiceData->{'Currency'}} );

    if($State eq PAYPALPLUS_STATUS_COMPLETED) {
        $CustomerOrder->set({'PendingOn' => undef});
        $CustomerOrder->set({$PaidOnSelector => $DateTime});
        if($PaidOnSelector eq 'PaidOn') {
            $LineItemPaymentPayPalPlus->set( {
                'TransStatus' => TRANSSTATUS_PAID,
                'TransTime' => $DateTime,
            });
        }
    }
    elsif($State eq PAYPALPLUS_STATUS_PENDING) {
        $CustomerOrder->set({'PendingOn' => $DateTime});
        $LineItemPaymentPayPalPlus->set( {
            'TransStatus' => TRANSSTATUS_PENDING,
        });
    }
    else {
        Error('UnexpectedState');
    }
    return;
}

#========================================================================================
# function     verifyResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       verifyResponse( $hResponse );
#----------------------------------------------------------------------------------------
# description  verifies response from webservice
#----------------------------------------------------------------------------------------
# input        $hResponse | response from webservice | refhash
# return       Verified response | refhash
#========================================================================================
sub verifyResponse {
    my $self = shift;
    my ( $hResponse ) = @_;

    my $hSale = $self->_extractSaleHash($hResponse);
    my $hAmount = $self->_extractAmount($hResponse);

    my $State = $hSale->{'state'};
    TestString('State', $State);
    my $TransactionId = $hSale->{'id'};
    TestString('TransactionId', $TransactionId);
    my $Amount = $hAmount->{'total'};
    TestFloat('Amount', $Amount);
    my $Currency = $hAmount->{'currency'};
    TestString('Currency', $Currency);

    my $hInvoiceData = $self->_extractInvoiceData($hResponse);
    my $PaymentMethod = $self->_extractPaymentMethod($hResponse);

    my $hVerifiedResponse = {
        'State' => $State,
        'TransactionId' => $TransactionId,
        'Amount' => $Amount,
        'Currency' => $Currency,
        'InvoiceData' => $hInvoiceData,
        'PaymentMethod' => $PaymentMethod
    };
    return $hVerifiedResponse;
}

sub _isPayUponInvoice {
    my $self = shift;
    my ($hResponse) = @_;

    my $PaymentInstruction = $hResponse->{'payment_instruction'};
    return 0 unless defined $PaymentInstruction;

    return $PaymentInstruction->{'instruction_type'} eq PAYPALPLUS_PAY_UPON_INVOICE;
}

sub _extractInvoiceData {
    my $self = shift;
    my ($hResponse) = @_;

    my $hInvoiceData = {
        'ReferenceNumber' => undef,
        'PaymentDueDate' => undef,
        'BankName' => undef,
        'AccountHolderName' => undef,
        'IBAN' => undef,
        'BIC' => undef,
        'Amount' => undef,
        'Currency' => undef,
    };

    if($self->_isPayUponInvoice($hResponse)) {

        my $PaymentInstruction = $hResponse->{'payment_instruction'};

        my $ReferenceNumber = $PaymentInstruction->{'reference_number'};
        my $PaymentDueDate = $PaymentInstruction->{'payment_due_date'};

        TestString('ReferenceNumber', $ReferenceNumber);
        TestString('PaymentDueDate', $PaymentDueDate);

        $PaymentDueDate = $self->_parseToDateTime($PaymentDueDate);

        my $Amount = $PaymentInstruction->{'amount'};

        my $InvoiceAmount = $Amount->{'value'};
        my $Currency = $Amount->{'currency'};

        TestString('Amount', $InvoiceAmount);
        TestString('Currency', $Currency);

        my $BankingInstructions = $PaymentInstruction->{'recipient_banking_instruction'};

        my $BankName = $BankingInstructions->{'bank_name'};
        my $AccountHolderName = $BankingInstructions->{'account_holder_name'};
        my $IBAN = $BankingInstructions->{'international_bank_account_number'};
        my $BIC = $BankingInstructions->{'bank_identifier_code'};

        TestString('BankName', $BankName);
        TestString('AccountHolderName', $AccountHolderName);
        TestString('IBAN', $IBAN);
        TestString('BIC', $BIC);

        $hInvoiceData->{'ReferenceNumber'} = $ReferenceNumber;
        $hInvoiceData->{'PaymentDueDate'} = $PaymentDueDate;
        $hInvoiceData->{'BankName'} = $BankName;
        $hInvoiceData->{'AccountHolderName'} = $AccountHolderName;
        $hInvoiceData->{'IBAN'} = $IBAN;
        $hInvoiceData->{'BIC'} = $BIC;
        $hInvoiceData->{'Amount'} = $InvoiceAmount;
        $hInvoiceData->{'Currency'} = $Currency;
    }

    return $hInvoiceData;
}

#========================================================================================
# function     _extractState
# state        private
#----------------------------------------------------------------------------------------
# syntax       _extractState( $hResponse );
#----------------------------------------------------------------------------------------
# description  extract state from response
#----------------------------------------------------------------------------------------
# input        $hResponse | response | refhash
# return       the sale | refhash
#========================================================================================
sub _extractSaleHash {
    my $self = shift;
    my ($hResponse) = @_;

    my $aTransactions = $hResponse->{'transactions'};
    $self->_testArrayMaxSize('Transactions', $aTransactions);
    my $hTransaction = $aTransactions->[0];
    TestHash('Transaction', $hTransaction);
    my $aRelatedResources = $hTransaction->{'related_resources'};
    $self->_testArrayMaxSize('related_resources', $aRelatedResources);
    my $hResource = $aRelatedResources->[0];
    TestHash('resource', $hResource);
    my $hSale = $hResource->{'sale'};
    TestHash('sale', $hSale);

    return $hSale;
}

#========================================================================================
# function     _extractAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _extractAmount( $hResponse );
#----------------------------------------------------------------------------------------
# description  extract amount from response
#----------------------------------------------------------------------------------------
# input        $hResponse | response | refhash
# return       the amount | refhash
#========================================================================================
sub _extractAmount {
    my $self = shift;
    my ($hResponse) = @_;

    my $aTransactions = $hResponse->{'transactions'};
    $self->_testArrayMaxSize('Transactions', $aTransactions);
    my $hTransaction = $aTransactions->[0];
    TestHash('Transaction', $hTransaction);
    my $hAmount = $hTransaction->{'amount'};
    TestHash('amount', $hAmount);

    return $hAmount;
}

sub _testArrayMaxSize {
    my $self = shift;
    my ($Name, $aArray) = @_;

    TestArray($Name, $aArray);
    my $SizeOfArray = scalar(@{$aArray});
    if ($SizeOfArray != 1){
        Error('UNSUPPORTED_ARRAY_SIZE',{ 'ElementName' => $Name, 'ActualSize' => $SizeOfArray });
    }

    return;
}

#========================================================================================
# function     _isSameAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isSameAmount( $ShopAmount, $NotificationAmount );
#----------------------------------------------------------------------------------------
# description  Truncate 2 floats with 2 decimals after comma and return compare them
#----------------------------------------------------------------------------------------
# input        $ShopAmount | float value | float
# input        $NotificationAmount | float value | float
# return       return boolean flag if are the same value
#========================================================================================
sub _isSameAmount {
    my $self = shift;
    my ($ShopAmount, $NotificationAmount) = @_;
    my $ShopAmount2DecimalDigits = sprintf ("%0.2f", $ShopAmount); ## no critic (ProhibitInterpolationOfLiterals)
    my $NotificationAmount2DecimalDigits = sprintf ("%0.2f", $NotificationAmount); ## no critic (ProhibitInterpolationOfLiterals)

    if( $ShopAmount2DecimalDigits eq $NotificationAmount2DecimalDigits) {
        return 1;
    } else {
        return 0;
    }
}

sub _parseToDateTime {
    my $self = shift;
    my ($DateFormattedString) = @_;

    my $DateTime;
    my $DateTimeFormatter = $self->_createDateTimeFormatter();
    my $TargetDateTimeFormat = 'date';
    my $ExpectedDateFormat = '%Y-%m-%d';

    eval {
        $DateTime = $DateTimeFormatter->parse_datetime($DateFormattedString, $TargetDateTimeFormat, $ExpectedDateFormat);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_logError($Error);

        $DateTime = $DateFormattedString;
    }

    return $DateTime;
}

sub _createDateTimeFormatter {
    my $self = shift;
    return DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
}

#========================================================================================
# function     _createRequestDTO
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createRequestDTO( );
#----------------------------------------------------------------------------------------
# description  create a request DTO
#----------------------------------------------------------------------------------------
# return       Request DTO | DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO
#========================================================================================
sub _createRequestDTO {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO->new();
}

#========================================================================================
# function     _GetCurrentDateTime
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetCurrentDateTime( );
#----------------------------------------------------------------------------------------
# description  wrapps the static function call of DateTime
#----------------------------------------------------------------------------------------
# return       $DateTime | current time | DateTime object
#========================================================================================
sub _GetCurrentDateTime {
    my $self = shift;
    return GetCurrentDateTime();
}

sub _logError {
    my $self = shift;
    my ($Error) = @_;

    LogError($Error);
    return;
}

sub _extractPaymentMethod {
    my $self      = shift;
    my ($hResponse) = @_;

    my $hPayer = $hResponse->{'payer'};
    TestHash('hPayer', $hPayer);
    my $PaymentMethod = $hPayer->{'payment_method'};
    if($self->_isPayUponInvoice($hResponse)) {
        $PaymentMethod = 'pay_upon_invoice';
    }
    return $PaymentMethod;
}

1;
