#==============================================================================
# §package      DE_EPAGES::Presentation::API::Form
# §state        public
# §base         DE_EPAGES::WebInterface::API::Form
#------------------------------------------------------------------------------
# §description  This package provides functions for validating input data
#               submitted by HTML forms.
#------------------------------------------------------------------------------
# §example      my $Form = $Servlet->form;
#               my $hValues = $Form->form( $User, 'LoginData' );
#               $User->set( $hValues );
#
# §example      # read unchecked values
#               my @Quantities = $Servlet->form->value('Quantity');
#
#               # check each individual value
#               my $Index = 0;
#               foreach ( @Quantities ) {
#                   unless ( /^\d+$/ ) {
#                       $Form->addFormError( {
#                           Reason => 'NoInteger',
#                           Name => 'Quantity',
#                           Index => $Index,
#                           LoopName => 'BasketProducts',
#                           Form => 'Basket',
#                       } );
#                   }
#                   $Index++;
#               }
#
#               # die if an input error occured
#               $Form->executeFormError;
#==============================================================================
package DE_EPAGES::Presentation::API::Form;
use base DE_EPAGES::WebInterface::API::Form;

use strict;
use Scalar::Util qw( blessed );

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw( IsInPath );
use DE_EPAGES::Core::API::Html qw( CloseOpenHTMLTags RemoveSuspiciousChars );
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf GetCallerStack trim );
use DE_EPAGES::Core::API::TestParameter qw( TestInteger TestString TestDataURLString TestHash TestFloat TestIpAddress TestEMailAddress TestMultipleEMailAddresses );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Presentation::API::Constants qw( MAX_IMAGE_UPLOAD_SIZE WRITEABLE_PATH_ATTRIBUTES );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );
use DE_EPAGES::TLE::API::TLE qw( TestSyntax );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NO_FORM'               => 'The form #FormName is not defined for class #Class',
        'INVALID_FORM'          => 'the input field #Name is not correct (#Reason).',
        'INVALID_FORM_ERROR'    => 'the error parameter is invalid (#RefError, #Error).',
        'LOOP_NOT_SAME'         => 'the loop field #Parameter has not correct count (#Count0, #CountKey).',
        'FILE_TOO_LONG'         => 'the uploaded file #Parameter is too long  (#Length > #Limit).',
        'FILE_TOO_SMALL'        => 'the uploaded file #Parameter is too small (#Length < #Limit).',
        'FILE_NOT_UPLOADED'     => 'the file #Parameter was not uploaded.',
        'DATETIME_TOO_LARGE'    => 'datetime too large: #Value',
        'DATETIME_TOO_SMALL'    => 'datetime too small: #Value',
        'FORMAT_NOT_INTEGER'    => '\'#Value\' is not an integer',
        'FORMAT_NOT_FLOAT'      => '\'#Value\' is not a float number',
        'FORMAT_NOT_MONEY'      => '\'#Value\' is not a valid money format',
        'TLE_ERROR'             => '\'#Value\' is not a valid TLE syntax',
        'NoCurrencyID'          => 'Can\'t get a CurrencyID via object #ObjectPath'
    };
}

##no critic qw(ProhibitCascadingIfElse ProhibitExcessComplexity)

#========================================================================================
# §function     new
#----------------------------------------------------------------------------------------
# §syntax       $Form = DE_EPAGES::Presentation::API::Form->new($HttpRequest);
#----------------------------------------------------------------------------------------
# §description  Creates a form object on a HTTP::Request object.
#----------------------------------------------------------------------------------------
# §input        $HttpRequest | HTTP::Request object | object
# §return       $Form | form | object
#========================================================================================
sub new {
    my $class = shift;
    my ($HttpRequest) = @_;

    my $self = bless DE_EPAGES::WebInterface::API::Form->new($HttpRequest), $class;
    $self->{'FormErrors'}  = [];
    $self->{'CurrentForm'} = undef;
    $self->{'Object'}      = undef;
    return $self;
}

#========================================================================================
# §function     formName
#----------------------------------------------------------------------------------------
# §syntax       $FormName = $self->formName;
#----------------------------------------------------------------------------------------
# §description  Returns the name of the current form.
#----------------------------------------------------------------------------------------
# §return       $FormName | form name | string
#========================================================================================
sub formName {
    my $self = shift;
    return $self->{'CurrentForm'};
}

#========================================================================================
# §function     object
#----------------------------------------------------------------------------------------
# §syntax       $Object = $self->object;
#----------------------------------------------------------------------------------------
# §description  Returns the object that this form is based on.
#               This object is used to determine format and curreny settings if they
#               are not available otherwise.
#----------------------------------------------------------------------------------------
# §return       $Object | object | object
#========================================================================================
sub object {
    my $self = shift;
    return $self->{'Object'};
}

#========================================================================================
# §function     locale
#----------------------------------------------------------------------------------------
# §syntax       $Locale = $self->locale;
#----------------------------------------------------------------------------------------
# §description  Returns the locale that is used for input validation of numbers and
#               date/time values.
#----------------------------------------------------------------------------------------
# §return       $Locale | locale string, e.g. 'en_US' | string
#========================================================================================
sub locale {
    my $self = shift;
    return $self->{'Locale'};
}

#========================================================================================
# §function     setlocale
#----------------------------------------------------------------------------------------
# §syntax       $self->setlocale($Locale);
#----------------------------------------------------------------------------------------
# §description  Sets the locale that is used for input validation of numbers and
#               date/time values.
#----------------------------------------------------------------------------------------
# §input        $Locale | locale string, e.g. 'en_US' | string
#========================================================================================
sub setlocale {
    my $self = shift;
    my ($Locale) = @_;
    $self->{'Locale'} = $Locale;
    return;
}

#========================================================================================
# §function     formErrors
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aFormErrors = $self->formErrors;
#----------------------------------------------------------------------------------------
# §description  Returns all form errors that were added previously by <function addFormError>.
#----------------------------------------------------------------------------------------
# §return       $aFormError | list of form errors. The list elements are hash references
#                       that were passed to <function addFormError> | ref.array.hash
#========================================================================================
sub formErrors {
    my $self = shift;
    LogDebug('FormErrors', $self->{'FormErrors'});
    return $self->{'FormErrors'};
}

#========================================================================================
# §function     resetFormErrors
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->resetFormErrors;
#----------------------------------------------------------------------------------------
# §description  Resets the collected form errors.
#========================================================================================
sub resetFormErrors {
    my $self = shift;
    $self->{'FormErrors'} = [];
    return;
}

#========================================================================================
# §function     addFormError
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->addFormError($hFormError);
#----------------------------------------------------------------------------------------
# §description  Adds a form error to the list of form errors. Use <function formErrors>
#               to get all errors that have been added.
#----------------------------------------------------------------------------------------
# §input        $hFormError | keys:
#               <ul>
#                   <li>Reason - reason error code - string
#                   <li>Name - input name - string
#                   <li>LoopName - loop name (optional)- string
#                   <li>Index - index of error in loop (optional default=0)- integer
#                   <li>Form - form name (optional default=NoForm)- string
#               </ul> | ref.hash.string
#========================================================================================
sub addFormError {
    my $self = shift;
    my ($hFormError) = @_;
    LogDebug( 'addFormError', $hFormError );

    TestString( 'Reason', $hFormError->{'Reason'} );
    TestString( 'Name', $hFormError->{'Name'} );
    $hFormError->{'Form'} = 'NoForm' unless defined $hFormError->{'Form'};
    my @Modules = grep { $_ ne 'DE_EPAGES::Presentation::API::Form'} map { $_->[0] } @{GetCallerStack()};
    $hFormError->{'ErrorPackage'} = shift @Modules unless (defined $hFormError->{'ErrorPackage'});

    push @{$self->formErrors}, $hFormError;
    return;
}

#========================================================================================
# §function     existsFormError
#----------------------------------------------------------------------------------------
# §syntax       $Exists = $self->existsFormError;
#----------------------------------------------------------------------------------------
# §description  Returns true if a form error has been added by <function addFormError>.
#----------------------------------------------------------------------------------------
# §return       $Exists | true if an error exists | boolean
#========================================================================================
sub existsFormError {
    my $self = shift;
    return scalar @{$self->formErrors} ? 1 : 0;
}

#========================================================================================
# §function     formData
#----------------------------------------------------------------------------------------
# §syntax       $hFormData = $self->formData( $ObjectClass, $FormName )
# §example      $hFormData = $self->formData( LoadClassByAlias('User'), 'LoginData' )
#----------------------------------------------------------------------------------------
# §description  Gets form field defininitions from database.
#----------------------------------------------------------------------------------------
# §input        $ObjectClass | class | object
# §input        $FormName | form name | string
# §return       $hFormData | hash with form field definitions, keys are form field names,
#                   values are hashes containing field definition,
#                   see <function formField> | ref.hash.hash
#========================================================================================
sub formData {
    shift; # static method
    my ( $ObjectClass, $FormName ) = @_;

    my $Form = undef;
    my $FormClass = $ObjectClass;
    my $ClassForm = LoadClassByAlias('Form');
    while (not defined $Form and defined $FormClass) {
        my $Forms = $ClassForm->getObjects('ByFormClassIDName', [$FormClass->id, $FormName]);
        $Form = $Forms->[0] if scalar @$Forms;
        $FormClass = $FormClass->baseClass;
    }
    Error('NO_FORM', {
        'Class'     => $ObjectClass->alias,
        'FormName'  => $FormName
    }) unless defined $Form;

    return $Form->formData;
}

#========================================================================================
# §function     executeFormError
#----------------------------------------------------------------------------------------
# §syntax       $self->executeFormError;
# §syntax       $self->executeFormError($FormError);
#----------------------------------------------------------------------------------------
# §description  Throws an 'INVALID_FORM' Error if a form error exists.
#               If $FormError is defined this error will be added and then executed.
#----------------------------------------------------------------------------------------
# §input        $FormError | (optional) keys:
#               <ul>
#                   <li>Reason - reason error code - string
#                   <li>Index - index of error in loop (optional) - string
#                   <li>LoopName - loop name (optional) - string
#                   <li>Form - form name - string
#                   <li>Name - input name - string
#               </ul> | ref.hash.string
#========================================================================================
sub executeFormError {
    my $self = shift;
    my ($FormError) = @_;
    $self->addFormError($FormError) if defined $FormError;
    Error('INVALID_FORM', $self->formErrors->[0]) if $self->existsFormError;
}

#========================================================================================
# §function     form
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hValues = $self->form($Object, $FormName, $skipExecuteFormError);
#----------------------------------------------------------------------------------------
# §example      my $hValues = $Servlet->form->form($Shop, 'SearchMask');
# §example      my $hValues = $Servlet->form->form($Basket, 'Update');
#----------------------------------------------------------------------------------------
# §description  Reads form field specifications from the database using the object class
#               and form name. Then checks the form data and converts them to the
#               internal representation using <function checkValues>.
#----------------------------------------------------------------------------------------
# §input        $Object | form based on object | object
# §input        $FormName | name of form (used only for error parameter) | string
# §input        $skipExecuteFormError | skip "executeFormError" or not | boolean
# §return       $hValues |  checked form values in native Perl format (possible copy of
#                           input $rIn) | ref.hash.string
#========================================================================================
sub form {
    my $self = shift;
    my ($Object, $FormName, $skipExecuteFormError) = @_;

    local $self->{'CurrentForm'} = $FormName; # for error messages ?
    my $hFormData = $self->formData( $Object->class, $FormName );
    return $self->checkValues( $Object, $hFormData, $skipExecuteFormError );
}

#========================================================================================
# §function     checkValues
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hValues = $self->checkValues($Object, $hFormData, $skipExecuteFormError);
#----------------------------------------------------------------------------------------
# §example      my $hFormData = $Form->formData( $Object->class, 'SearchMask' );
#               my $hValues = $Form->checkValues( $Object, $hFormData );
#----------------------------------------------------------------------------------------
# §description  Checks the form data and converts them from regional-specific formatting
#               to the internal (Perl) representation using the given form field
#               specifications.
#----------------------------------------------------------------------------------------
# §input        $Object | form based on object | object
# §input        $hFormData | from field specifications, see <function formData> | ref.hash
# §input        $skipExecuteFormError | skip "executeFormError" or not | boolean
# §return       $hValues | checked form values in internal representation | ref.hash.string
#========================================================================================
sub checkValues {
    my $self = shift;
    my ($Object, $hFormData, $skipExecuteFormError) = @_;

    local $self->{'Object'} = $Object; # used to get formatters

    my %Out;

    # collect non loop input fields
    my @CurrencyIDs = $self->value('CurrencyID');
    $hFormData->{'Fields'}{'CurrencyID'} = {'Type'=>'String', 'MinValue'=>3, 'MaxValue'=>3} if scalar @CurrencyIDs == 1 and defined $CurrencyIDs[0];
    foreach my $InputName (keys %{$hFormData->{'Fields'}}) {
        my @Values = $self->value($InputName);
        $Out{$InputName} = $self->formField($Values[0], $hFormData->{'Fields'}{$InputName});
    }

    # collect loop input fields
    foreach my $LoopName (keys %{$hFormData->{'Loops'}}) {
        my $ListCount = undef;
        my $FirstName = undef;
        my $Fields = $hFormData->{'Loops'}{$LoopName};
        foreach my $InputName (keys %$Fields) {
            next unless $self->existsValue($InputName);
            my @Values = $self->value($InputName);
            my $CountKey = scalar @Values;
            if (not defined $ListCount) {
                $ListCount = $CountKey;
                $FirstName = $InputName;
            }
            Error('LOOP_NOT_SAME', {'Parameter'=> $InputName, 'Count0'=>$ListCount, 'CountKey'=>$CountKey, 'Name0' => $FirstName }) if $ListCount != $CountKey;
        }
        my @aListValues;
        $ListCount = 0 unless defined $ListCount;
        for( my $i=0;$i < $ListCount; $i++ ) {
            my %Row;
            foreach my $InputName (keys %$Fields) {
                my $FieldData = $Fields->{$InputName};
                $FieldData->{'LoopName'} = $LoopName;
                $FieldData->{'Index'} = $i;
                $FieldData->{'CurrencyID'} = ($self->value('CurrencyID'))[$i] if defined (($self->value('CurrencyID'))[$i]);
                my @Values = $self->value($InputName);
                $Row{$InputName} = $self->formField($Values[$i], $FieldData);
            }
            push @aListValues, \%Row;
        }
        # error if form loop count < min count
        if (scalar @aListValues < $hFormData->{'LoopInfo'}{$LoopName}{'MinCount'}) {
            $self->addFormError({
                'Reason' => 'MINCOUNT_NOT_REACHED',
                'Form' => $hFormData->{'FormName'},
                'Loop' => $LoopName,
                #'Name' => join (',', keys %$Fields),
                'Name' => $LoopName,
            });
        }
        $Out{$LoopName} = \@aListValues;
    }
    LogDebug('Out', \%Out);
    $self->executeFormError unless $skipExecuteFormError;
    return \%Out;
}

#========================================================================================
# §function     field
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Value = $self->field($Object, $FormValue, $hFieldData);
#----------------------------------------------------------------------------------------
# §example      my $Value = $Servlet->form->field($Shop, $QuantityData, { 'Type'=>'reg_integer', 'Name'=>$InputName });
#----------------------------------------------------------------------------------------
# §description  Tests form data and converts from external to internal representation.
#----------------------------------------------------------------------------------------
# §input        $Object | form based on object | object
# §input        $FormValue | value of input field in form | string
# §input        $hFieldData | field data keys:
#               <ul>
#                   <li>Name
#                   <li>Type
#                   <li>Mandatory
#                   <li>MinValue
#                   <li>MaxValue
#                   <li>LoopName
#                   <li>Index
#                   <li>CurrencyID
#               </ul> | ref.hash
# §return       $Value | unformatted value | integer,float,DateTime,string
#========================================================================================
sub field {
    my $self = shift;
    my ( $Object, $FormValue, $hFieldData ) = @_;
    local $self->{'Object'} = $Object;
    local $self->{'CurrentForm'} = 'Field';

    my $PathToSite = $Object->get('Site')->get('Path');
    _checkNewPath( $PathToSite, $FormValue, $hFieldData );

    return $self->formField( $FormValue, $hFieldData );
}

#========================================================================================
# §function     _checkNewPath
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       _checkNewPath($PathToSite, $FormValue, $hFieldData);
#----------------------------------------------------------------------------------------
# §description  Checks whether the Path from the input is valid, e.g. is
#               within the path of the shop. Mainly to prevent possible directory
#               traversal attacks.
#----------------------------------------------------------------------------------------
# §example      _checkNewPath(
#                   '/Shops/DemoShop', '/Shops/DemoShop/Products/cg_0101104470', $hFieldData
#               );
#               Will evaluate to true
# §example      _checkNewPath(
#                   '/Shops/DemoShop', '../../../../../../../etc/apache2-epages', $hFieldData
#              );
#               Will evaluate to false
#----------------------------------------------------------------------------------------
# §input        $PathToSite | path to site | string
# §input        $FormValue | value of input field in form | string
# §input        $hFieldData | field data keys | hash
#========================================================================================
sub _checkNewPath {
    my ( $PathToSite, $FormValue, $hFieldData ) = @_;
    for ( $PathToSite, $FormValue, $hFieldData ) { return unless defined $_ }
    my $FieldName = exists $hFieldData->{'Name'} ? $hFieldData->{'Name'} : return;

    if ( $FieldName ~~ WRITEABLE_PATH_ATTRIBUTES && ! IsInPath( $FormValue, $PathToSite ) ) {
        Error( 'INVALID_PATH', { $FieldName => $FormValue } );
    }
}

#========================================================================================
# §function     formField
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Value = $self->formField($OrgValue, $hFieldData);
# §example      my @Values = $self->value('Quantity')); # get values in array context
#               $Value = $self->formField($Values[0], {
#                   'Type'     => 'reg_integer',
#                   'MaxValue' => 999,
#                   'Name'     => 'Quantity',
#                   'LoopName' => 'BasketItems',
#                   'Index'    => 0
#               });
#----------------------------------------------------------------------------------------
# §description  Tests form data field and converts from external to internal representation.
#----------------------------------------------------------------------------------------
# §input        $OrgValue | value of field | string
# §input        $hFieldData | field data keys:
#               <ul>
#                   <li>Name
#                   <li>Type
#                   <li>Mandatory
#                   <li>MinValue
#                   <li>MaxValue
#                   <li>LoopName
#                   <li>Index
#                   <li>CurrencyID
#               </ul> | ref.hash
# §return       $Value | unformatted value | integer,float,DateTime,string
#========================================================================================
sub formField {
    my $self = shift;
    my ($OrgValue, $hFieldData) = @_;
    my $Value = undef;

    eval {
        my ($Name, $Type, $MinValue, $MaxValue) = @$hFieldData{qw (Name Type MinValue MaxValue)};
        die 'no type for form field.' unless defined $Type; # don't use TestParameter here, because then it would look like an input error

        if ($Type eq 'file') {
            my $filelength = 0;
            my $FileIndex = defined $hFieldData->{'Index'} ? $hFieldData->{'Index'} : 0;
            $filelength = length $self->files->{$Name}[$FileIndex] if defined $self->files->{$Name}[$FileIndex];
            return if (not $hFieldData->{'Mandatory'} and $filelength <= 0);
            Error('FILE_NOT_UPLOADED', {'Parameter' => $Name }) if $filelength == 0 and $self->files->{$Name} =~ /\S/;
            Error('FILE_TOO_LONG',  {'Parameter' => $Name, 'Length'=> $filelength, 'Limit' => $MaxValue }) if defined $MaxValue and $filelength > $MaxValue;
            Error('FILE_TOO_SMALL', {'Parameter' => $Name, 'Length'=> $filelength, 'Limit' => $MinValue }) if defined $MinValue and $filelength < $MinValue;
            $Value = $OrgValue;
            return; # return out of eval
        }
        elsif ($Type eq 'password') {
            $Value = $OrgValue;
        }
        elsif (defined $OrgValue) {
            ($Value) = trim($OrgValue);
        }

        if (not defined $Value or $Value !~ /\S/) {
            if ($Type eq 'checkbox') {
                $Value = 0;
            }
            elsif (not $hFieldData->{'Mandatory'}) {
                $Value = undef;
            }
            else {
                Error('STRING_EMPTY', {
                    'Parameter' => $Name,
                    'Value'     => $Value,
                });
            }
            return; # return out of eval
        }

        if ($Type eq 'string') {
            $MaxValue = GetCurrentDBHandle()->maxTextLength() unless defined $MaxValue;
            my $IsHTML = $hFieldData->{'IsHTML'} // 0;
            TestString($Name, $Value, $MinValue, $MaxValue);
            $Value = CloseOpenHTMLTags($Value) if $IsHTML;
        }
        elsif ($Type eq 'password') {
            # like "string", but throws an error for leading and trailing spaces
            $MaxValue = GetCurrentDBHandle()->maxTextLength() unless defined $MaxValue;
            TestString($Name, $Value, $MinValue, $MaxValue);
            Error('PasswordWithSpaces') if $Value =~ /^\s|\s$/
        }
        elsif ($Type eq 'sf_string') {
            $MaxValue = GetCurrentDBHandle()->maxTextLength() unless defined $MaxValue;
            TestString($Name, $Value, $MinValue, $MaxValue);
            $Value = RemoveSuspiciousChars($Value);
        }
        elsif ($Type eq 'integer') {
            TestInteger($Name, $Value, $MinValue,$MaxValue);
        }
        elsif ($Type eq 'float')   {
            TestFloat($Name, $Value, $MinValue,$MaxValue);
        }
        elsif (ElementOf($Type, qw(reg_datetime reg_date reg_time))) {
            $Value = $self->convertDateTime($Value, $hFieldData);
        }
        elsif ($Type eq 'reg_money') {
            $Value = $self->convertMoney($Value, $hFieldData);
        }
        elsif ($Type eq 'reg_integer') {
            $Value = $self->convertInteger($Value, $hFieldData);
        }
        elsif ($Type eq 'reg_float') {
            $Value = $self->convertFloat($Value, $hFieldData);
        }
        elsif ($Type eq 'checkbox') {
            # if not defined returned before
            $Value = $Value ? 1 : 0;
        }
        elsif ($Type eq 'ip_address') {
            TestIpAddress($Name, $Value);
        }
        elsif ($Type eq 'email_address') {
            TestEMailAddress($Name, $Value);
        }
        elsif ($Type eq 'multiple_email_address') {
            TestMultipleEMailAddresses($Name, $Value);
        }
        elsif ($Type eq 'html') {
            $MaxValue = GetCurrentDBHandle()->maxTextLength() unless defined $MaxValue;
            TestString($Name, $Value, $MinValue, $MaxValue);
            $Value = CloseOpenHTMLTags($Value);
        }
        elsif ($Type eq 'tle_string')  {
            $MaxValue = GetCurrentDBHandle()->maxTextLength() unless defined $MaxValue;
            TestString($Name, $Value, $MinValue, $MaxValue);
            my $Error = TestSyntax(\$Value);
            Error($Error) if $Error;
        }
        elsif ($Type eq 'data_url') {
            # we fetch the max length in kB from the Config.pm
            $MaxValue = MAX_IMAGE_UPLOAD_SIZE // undef ;
            $MaxValue = $MaxValue * 1024 if defined $MaxValue; ##no critic qw(ProhibitMagicNumbers)

           # test the Data-URL image for the correct max size
            TestDataURLString($Name, $Value, $MinValue, $MaxValue);
        }
        else {
            $hFieldData->{'Value'} = $Value;
            TriggerHook('TestFormField', $hFieldData);
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code ~~ [qw(
            DATETIME_TOO_LARGE
            DATETIME_TOO_SMALL
            FORMAT_NOT_TIME
            FORMAT_NOT_DATE
            FORMAT_NOT_DATETIME
            FORMAT_NOT_FLOAT
            FORMAT_NOT_INTEGER
            FORMAT_NOT_MONEY
            FORMAT_NOT_IPADDRESS
            FORMAT_NOT_EMAILADDRESS
            FORMAT_NOT_URL
            FORMAT_NOT_DOMAIN
            INTEGER_TOO_LARGE
            INTEGER_TOO_SMALL
            FLOAT_TOO_LARGE
            FLOAT_TOO_SMALL
            STRING_EMPTY
            STRING_TOO_LONG
            STRING_TOO_SHORT
            FILE_TOO_LONG
            FILE_TOO_SMALL
            FILE_NOT_UPLOADED
            SMTP_FALSE_MAIL_FORMAT
            TLE_ERROR
            PasswordWithSpaces
        )];
        $self->mapFormError($hFieldData, $Error);
    }
    LogDebug($hFieldData->{'Name'}, $Value);
    return $Value;
}

sub formatter {
    my $self = shift;

    return $self->{'Formatter'} if defined $self->{'Formatter'};

    my $SiteLocale = $self->object->getSite()->siteLocale( $self->locale );

    $self->{'Formatter'} = $SiteLocale;
    return $SiteLocale;
}

sub convertInteger {
    my $self = shift;
    my ($Value, $hFieldData) = @_;

    my $IntValue = eval {
        $self->formatter('Number')->parse($Value);
    };
    if( ExistsError() ) {
        my $Error = GetError();
        Error( 'FORMAT_NOT_INTEGER', { Value => $Value } ) if $Error->code eq 'FORMAT_NOT_FLOAT';
        Error( $Error );
    }

    Error( 'FORMAT_NOT_INTEGER', { Value => $Value } ) unless defined $IntValue;
    TestInteger($hFieldData->{'Name'}, $IntValue, $hFieldData->{'MinValue'}, $hFieldData->{'MaxValue'});
    return $IntValue;
}

sub convertFloat {
    my $self = shift;
    my ($Value, $hFieldData) = @_;

    my $FloatValue = $self->formatter('Number')->parse($Value);
    Error( 'FORMAT_NOT_FLOAT', { Value => $Value } ) unless defined $FloatValue;
    TestFloat($hFieldData->{'Name'}, $FloatValue, $hFieldData->{'MinValue'}, $hFieldData->{'MaxValue'});
    return $FloatValue;
}

#========================================================================================
# §function     convertMoney
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->convertMoney( $Value, $hFieldData );
#----------------------------------------------------------------------------------------
# §description  Parses a formatted currency value and returns the numeric value.
#               The accepted currency (and the format) can be specified by the hash value
#               of $hFieldData->{'CurrencyID'}.
#               If the currency is not specified in the field definition, the currency
#               and the current object has a 'CurrencyID' attribute, then this currency
#               is used ($self->object->get('CurrencyID')), otherwise the currency is
#               the default site currency ($self->object->getSite()->get('CurrencyID')).
#----------------------------------------------------------------------------------------
# §input        $Value | value in display format, e.g. 'EUR 19,99' | string
# §input        $hFieldData | field definition data, see <function field> | ref.hash
# §return       $MoneyValue | value in internal number format, e.g. 19.99 | float
#========================================================================================
sub convertMoney {
    my $self = shift;
    my ($Value, $hFieldData) = @_;

    my $CurrencyID = $hFieldData->{'CurrencyID'};
    if( !defined $CurrencyID ) {
        die 'no Object for CurrencyID' unless defined $self->object; # don't use TestParameter here
        if( $self->object->class->existsAttribute('CurrencyID') ) {
            $CurrencyID = $self->object->get('CurrencyID');
            Error( 'NoCurrencyID', {
                ObjectPath => $self->object->pathString
            }) unless defined $CurrencyID;
        }
        else {
            $CurrencyID = $self->object->getSite()->get('CurrencyID');
            Error( 'NoCurrencyID', {
                ObjectPath => $self->object->getSite()->pathString
            }) unless defined $CurrencyID;
        }
    }
    my $MoneyValue = $self->formatter('Number')->parse($Value, $CurrencyID);
    Error( 'FORMAT_NOT_MONEY', { Value => $Value } ) unless defined $MoneyValue;
    TestFloat($hFieldData->{'Name'}, $MoneyValue, $hFieldData->{'MinValue'}, $hFieldData->{'MaxValue'});
    return $MoneyValue;
}

sub convertDateTime {
    my $self = shift;
    my ($Value, $hFieldData) = @_;

    my $Type = $hFieldData->{'Type'};
    my $Formatter = $self->formatter('DateTime');
    $Type =~ s/^reg_//;
    my @Pattern = ('%Y-%m-%dT%T', qw (full long medium short) );
    if ($Type eq 'time') {
        unshift @Pattern, '%{hour_12}:%M %P', '%{hour_12} %P', '%{hour_12}:%M %p', '%{hour_12} %p';
        push @Pattern, '%H';
        # allow "24" and "24:00", will be converted to 00:00 (DateTime::Format::Strptime does not know %k ..)
        push @Pattern, '24:%M';
        push @Pattern, '24';
    }
    my $LastError = undef;
    foreach my $pattern (@Pattern) {
        my $DateTimeValue = undef;
        eval {
            $DateTimeValue = $Formatter->parseDateTime($Value, $Type, $pattern);
            $DateTimeValue->set( year => 2000, month => 1, day => 1) if $Type eq 'time';
            GetCurrentDBHandle()->testDateTimeRange($DateTimeValue);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $DateTimeValue = undef;
            Error($Error) unless $Error->code ~~ [qw(FORMAT_NOT_DATETIME  DATETIME_TOO_LARGE DATETIME_TOO_SMALL)];
            $LastError = $Error;
        }
        return $DateTimeValue if defined $DateTimeValue;
    }
    Error($LastError);
}

#========================================================================================
# §function     mapFormError
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->mapFormError($hFieldData, $Error);
#----------------------------------------------------------------------------------------
# §description  Builds form error from standard error.
#----------------------------------------------------------------------------------------
# §input        $hFieldData | data for input field ()| string
# §input        $Error | error of parameter see <function DE_EPAGES::Core::API::Error::GetError> | object
#========================================================================================
sub mapFormError {
    my $self = shift;
    my ($hFieldData, $Error) = @_;
    my %Vars = %{$Error->vars};
    $Vars{'Form'}   = $self->formName;
    $Vars{'Reason'} = $Error->code;
    $Vars{'ErrorPackage'} = $Error->package;
    $Vars{'Error'} = $Error;
    $Vars{$_} = $hFieldData->{$_} foreach keys %{$hFieldData};
    $Vars{$_} = $Vars{$_}->tleHash foreach grep {defined $Vars{$_} and blessed $Vars{$_} and $Vars{$_}->can('tleHash')} keys %Vars;
    $Vars{'Vars'} = $Error->vars;
    $self->addFormError(\%Vars);
}

#========================================================================================
# §function     errors
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hVars = $self->errors;
# §example      $Servlet->vars('FormError', 1);
#               $Servlet->vars('FormErrors', $self->form->errors);
# §example      #LOOP(#FormErrors.Errors)
#                 Reason: #Reason
#                 Name: #Name
#                 Value: #Value
#                 Index: #Index
#                 LoopName: #LoopName
#                 <br />
#               #ENDLOOP
# §example      #IF(#FormErrors.Reason.FORMAT_NOT_INTEGER) ... #ENDIF
#----------------------------------------------------------------------------------------
# §description  The errors of form vars will be converted to accessable objects.
#----------------------------------------------------------------------------------------
# §return       $hVars | empty hash if no error else keys:
#               <ul>
#                   <li>Errors - list of all form errors
#                                (keys: Reason, Name, Value, LoopName, Index)
#                               - ref.array.hash
#                   </li>
#                   <li>Error - form errors by input filen name and loop index
#                                (keys: Reason, Name, Value, LoopName, Index)
#                               - ref.hash.hash.hash
#                   </li>
#                   <li>Reasons - list of all form errors by reason. error code
#                                (keys: Reason, Errors, ErrorCount)
#                               - ref.array.hash
#                   </li>
#                   <li>Reason - form errors by reason code
#                                (keys: Reason, Errors, ErrorCount)
#                               - ref.hash.hash
#                   </li>
#                   <li>Form - form errors by form name
#                                (keys: Form, Reason, Error, ErrorCount)
#                               - ref.hash.hash
#                   </li>
#               </ul> | ref.hash.string
#========================================================================================
sub errors {
    my $self = shift;
    my %Vars = (
        'Form'      => {},
        'Error'     => {},
        'Reason'    => {},
    );
    my $hFormValues = $self->formValues;
    foreach my $InputName (grep {defined $hFormValues->{$_}} keys %$hFormValues) {
        my @Values = @{$hFormValues->{$InputName}};
        for (my $i=0;$i<scalar @Values;$i++) {
            $Vars{'Value'}{$InputName}{$i} = $Values[$i];
        }
    }
    foreach my $hFormError (@{$self->formErrors}) {
        # Error Vars
        if (defined $hFormError->{'LoopName'}) {
            $Vars{'Error'}{$hFormError->{'Name'}}{$hFormError->{'Index'}} = $hFormError;
        } else {
            $Vars{'Error'}{$hFormError->{'Name'}}{0} = $hFormError;
        }

        # Form Vars
        my $VarsForm;
        if (exists $Vars{'Form'}{$hFormError->{'Form'}}) {
            $VarsForm = $Vars{'Form'}{$hFormError->{'Form'}};
        } else {
            $Vars{'Form'}{$hFormError->{'Form'}} = {};
            $VarsForm = $Vars{'Form'}{$hFormError->{'Form'}};
            $VarsForm->{'ErrorCount'} = 0;
            $VarsForm->{'Form'}       = $hFormError->{'Form'};
            $VarsForm->{'Reason'}     = {};
            $VarsForm->{'ErrorCount'} = 0;
        }
        $VarsForm->{'ErrorCount'}++;
        $VarsForm->{'Reason'}{$hFormError->{'Reason'}} = 0 unless exists $VarsForm->{'Reason'}{$hFormError->{'Reason'}};
        $VarsForm->{'Reason'}{$hFormError->{'Reason'}}++;
        push @{$VarsForm->{'Errors'}}, $hFormError;

        # Reason Vars
        if (not exists $Vars{'Reason'}{$hFormError->{'Reason'}}) {
            $Vars{'Reason'}{$hFormError->{'Reason'}}{'ErrorCount'} = 0;
            $Vars{'Reason'}{$hFormError->{'Reason'}}{'Reason'} = $hFormError->{'Reason'};
        }
        $Vars{'Reason'}{$hFormError->{'Reason'}}{'ErrorCount'}++;
        push @{$Vars{'Reason'}{$hFormError->{'Reason'}}{'Errors'}}, $hFormError;
    }
    @{$Vars{'Reasons'}} = values %{$Vars{'Reason'}};
    $Vars{'Errors'} = $self->formErrors;
    return \%Vars;
}

#========================================================================================
# §function     MandatoryForm
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hVars = MandatoryForm($hFormData);
#----------------------------------------------------------------------------------------
# §example      my $hFormData = $self->formData( LoadClassByObject('User', 'LoginData' )
#               $hVars = { %$hVars, %{ MandatoryForm( $hFormData ) } };
#----------------------------------------------------------------------------------------
# §description  Returns TLE variables containging information about the form definition.
#               They can be used to indicate required fields or accepted ranges.
#----------------------------------------------------------------------------------------
# §input        $hFormData | format definition, see <function formData> | ref.hash.hash.string
# §return       $hVars | TLE variables, keys:
#               <ul>
#                   <li>TYPE_xxx - type of data</li>
#                   <li>MANDATORY_xxx - true if field is required</li>
#                   <li>MIN_xxx - min length (string) min amount (integer,float)</li>
#                   <li>MAX_xxx - max length (string) max amount (integer,float)</li>
#                   <li>EXAMPLE_xxx - example (for regional dependent types)</li>
#               </ul> | ref.hash.string
#========================================================================================
use constant EXAMPLE_INT => 1234;
use constant EXAMPLE_FLOAT => 1234.56;
use constant EXAMPLE_MONEY => 1234.56;
use constant EXAMPLE_DATE => '19991231';
my %FORMAT_EXAMPLES = (
    'reg_integer'  => ['integer', EXAMPLE_INT],
    'reg_float'    => ['float', EXAMPLE_FLOAT],
    'reg_money'    => ['money', EXAMPLE_MONEY],
    'reg_date'     => ['date', EXAMPLE_DATE],
);
sub MandatoryForm {
    my ($rFormData) = @_;
    TestHash('rFormData', $rFormData);

    my %Vars;
    foreach my $Name (keys %$rFormData) {
        $Vars{'TYPE_'     .$Name} = $rFormData->{$Name}->{'Type'};
        $Vars{'MANDATORY_'.$Name} = $rFormData->{$Name}->{'Mandatory'};
        $Vars{'MIN_'      .$Name} = $rFormData->{$Name}->{'MinValue'};
        $Vars{'MAX_'      .$Name} = $rFormData->{$Name}->{'MaxValue'};
        if (defined $FORMAT_EXAMPLES{$Vars{'TYPE_'.$Name}}) {
            my $Format = $FORMAT_EXAMPLES{$Vars{'TYPE_'.$Name}};
            $Vars{'EXAMPLE_'.$Name} = FormatOutput($Format->[0],$Format->[1]);
        }
    }
    return \%Vars;
}

#========================================================================================
# §function     setValues
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->setValues($hValues);
#----------------------------------------------------------------------------------------
# §description  adds form values and transform input hash into correct format (hash of arrays)
#----------------------------------------------------------------------------------------
# §input        $hValues | form values; values may be strings or arrays of strings | ref.hash
#========================================================================================
sub setValues() {
    my $self = shift;
    my ($hValues) = @_;

    my %TransformedValues;
    foreach my $key (keys %$hValues) {
        my $rValues = $hValues->{$key};
        $TransformedValues{$key} = ref($rValues) eq 'ARRAY' ? [ @$rValues ] : [ $rValues ];
    }

    $self->{'Values'}  = \%TransformedValues;
    return;
}

1;

