#========================================================================================
# package      DE_EPAGES::Presentation::API::Scripts::HttpToHttps
# state        public
# description  api for changeHttpToHttps script
#========================================================================================
package DE_EPAGES::Presentation::API::HttpToHttps;
use strict;

use DE_EPAGES::Presentation::DAL::HttpToHttps qw (
  DAL_GetDataLocalizedStringAttr
  DAL_GetDataStringAttr
  DAL_GetShopIds
);

use DE_EPAGES::Core::API::Alarm qw (Alarm);
use File::Path qw (make_path);
use Data::Dumper;
use DE_EPAGES::Object::API::Factory qw (LoadObject);
use LWP::UserAgent;

use constant TIMEOUT => 2;

no warnings 'once';  ## no critic (ProhibitNoWarnings)

#========================================================================================
# function     new
#----------------------------------------------------------------------------------------
# syntax       $Object = DE_EPAGES::Presentation::API::Scripts::HttpToHttps->new();
#----------------------------------------------------------------------------------------
# description  Creates a new object of DE_EPAGES::Presentation::API::Scripts::HttpToHttps.
#----------------------------------------------------------------------------------------
# return       $Object | object
#========================================================================================
sub new {
    use IO::Socket::SSL;
    $Net::HTTPS::SSL_SOCKET_CLASS = 'IO::Socket::SSL';

    my $class = shift;
    my $ua      = LWP::UserAgent->new(
        ssl_opts => { verify_hostname => 1 },
        timeout => TIMEOUT
    );
    my $self = bless {
            'UserAgent' => $ua
        }, $class;

    return $self;
}

#========================================================================================
# function     getTargetList
#----------------------------------------------------------------------------------------
# syntax       $aTargetHashList = $Package->GetTargetList();
#----------------------------------------------------------------------------------------
# description  Gets list of hashes which contains values with http to replace.
#----------------------------------------------------------------------------------------
# return       $aData | array of following hashes:
#               <ul>
#                  <li>Table      - string value</li>
#                  <li>Shops      - array of following hashes
#                       <ul>
#                           <li>ShopId</li>
#                           <li>Sources - array of following hashes
#                               <ul>
#                                   <li>Links - array of http links to be replaced</li>
#                                   <li>Value - string value which contains links above</li>
#                                   <li>ObjectId - int id of the object</li>
#                                   <li>AttributeId - int id of the attribute of corresponding object</li>
#                                   <li>LanguageId - int id of the language of corresponding value (optional)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                  </li>
#               </ul>
#========================================================================================
sub getTargetList {
    my $self = shift;

    # init
    my $aHttpLinksViaShopHash = [];

    # get all shops
    my $aShopIds = DAL_GetShopIds();

    # get localized string values
    my $hHttpLinksViaShopLSA = {};
    $hHttpLinksViaShopLSA->{'Table'} = 'localizedstringattribute';

    push( @{$aHttpLinksViaShopHash}, $self->_buildHash( $hHttpLinksViaShopLSA, $aShopIds ) );

    # get string values
    my $hHttpLinksViaShopSA = {};
    $hHttpLinksViaShopSA->{'Table'} = 'stringattribute';
    push( @{$aHttpLinksViaShopHash}, $self->_buildHash( $hHttpLinksViaShopSA, $aShopIds ) );

    return $aHttpLinksViaShopHash;
}

#========================================================================================
# function     updateTargetList
#----------------------------------------------------------------------------------------
# syntax       $Package->UpdateTargetList($Direction, $aData);
#----------------------------------------------------------------------------------------
# description  Replaces http->https or https->http depending on input parameter.
#---------------------------------------------------------------------------------------
# input       $Direction | string has one of the following values ['http2https', 'https2http']
# input       $aData | array of following hashes:
#               <ul>
#                  <li>Table      - string value</li>
#                  <li>Shops      - array of following hashes
#                       <ul>
#                           <li>ShopId</li>
#                           <li>Sources - array of following hashes
#                               <ul>
#                                   <li>Links - array of http links to be replaced</li>
#                                   <li>Value - string value which contains links above</li>
#                                   <li>ObjectId - int id of the object</li>
#                                   <li>AttributeId - int id of the attribute of corresponding object</li>
#                                   <li>LanguageId - int id of the language of corresponding value (optional)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                  </li>
#               </ul>
#========================================================================================
sub updateTargetList {
    my $self = shift;

    my ( $Direction, $aHttpLinksViaShopHash ) = @_;

    for my $hHttpLinksViaTable ( @{$aHttpLinksViaShopHash} ) {
        for my $hLinksViaShop ( @{ $hHttpLinksViaTable->{'Shops'} } ) {
            for my $hLinksViaObject ( @{ $hLinksViaShop->{'Sources'} } ) {
                for my $httpLink ( @{ $hLinksViaObject->{'Links'} } ) {

                    my $httpsLink;
                    if ( $Direction eq 'http2https' ) {

                        # http->https
                        ( $httpsLink = $httpLink ) =~ s/http:\/\//https:\/\//g;
                    }
                    elsif ( $Direction eq 'https2http' ) {

                        # http->https
                        ( $httpsLink = $httpLink ) =~ s/https:\/\//http:\/\//g;
                    }
                    else {

                        die "[ERROR]: Can't convert with direction: $Direction";
                    }

                    # replace link in original value
                    $httpLink = quotemeta($httpLink);
                    ( my $changedValue = $hLinksViaObject->{'Value'} ) =~ s/$httpLink/$httpsLink/g;

                    my $Object    = LoadObject( $hLinksViaObject->{'ObjectId'} );
                    my $AttrAlias = LoadObject( $hLinksViaObject->{'AttributeId'} )->get('Alias');

                    if ( $hLinksViaObject->{'LanguageId'} ) {

                        $Object->set( { $AttrAlias => $changedValue }, $hLinksViaObject->{'LanguageId'} );
                    }
                    else {
                        $Object->set( { $AttrAlias => $changedValue } );
                    }
                }    # end iteration over links
            }    # end iteration over sources
        }    # end iteration over shops
    }    # end iteration over hashes

    return;
}

#========================================================================================
# function     backupData
#----------------------------------------------------------------------------------------
# syntax       BackupData($aTargetHashList)
#----------------------------------------------------------------------------------------
# description  Stores data into a file.
#----------------------------------------------------------------------------------------
# return       $BackupFile | string | full path to backup file
#========================================================================================
sub backupData {
    my $self = shift;

    my ($hData) = @_;

    my $path = '/tmp/http2https';
    make_path($path);
    my $backupFile = "$path/HttpToHttpsBackup_" . time . '.txt';

    open( my $FH, '>', "$backupFile" ) or die q([ERROR]: Couldn't write backup.);
    print( $FH Dumper($hData) );
    close $FH
      or warn $!
      ? "[ERROR]: closing file handler: $!\n"
      : "Exit status $? \n";

    print "[INFO]: Backup is done in file '$backupFile' \n\n";

    return $backupFile;
}

#========================================================================================
# function     readBackupData
#----------------------------------------------------------------------------------------
# syntax       $aTargetHashList = ReadBackupData()
#----------------------------------------------------------------------------------------
# description  Reads data from backup and converts it back to corresponding perl type.
#========================================================================================
sub readBackupData {
    my $self = shift;

    my ($backupFile) = @_;
    return do($backupFile);
}

#========================================================================================
# function     _buildHash
#----------------------------------------------------------------------------------------
# syntax       $aTargetHash = self->__BuildHash($hData, $aShopIds)
#----------------------------------------------------------------------------------------
# description  Builds and array of hashes.
#----------------------------------------------------------------------------------------
# input        $aHashData | hash array which contains initially table
# input        $aShopIDs | array of shop ids
# return       following hashes:
#               <ul>
#                  <li>Table      - string value</li>
#                  <li>Shops      - array of following hashes
#                       <ul>
#                           <li>ShopId</li>
#                           <li>Sources - array of following hashes
#                               <ul>
#                                   <li>Links - array of http links to be replaced</li>
#                                   <li>Value - string value which contains links above</li>
#                                   <li>ObjectId - int id of the object</li>
#                                   <li>AttributeId - int id of the attribute of corresponding object</li>
#                                   <li>LanguageId - int id of the language of corresponding value (optional)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                  </li>
#               </ul>
#========================================================================================
sub _buildHash {
    my $self = shift;

    my ( $aHashData, $aShopIDs ) = @_;

    for my $ShopId ( @{$aShopIDs} ) {

        my $aLinksViaIds = $self->_getResourceList( $aHashData->{'Table'}, $ShopId );
        if ( @{$aLinksViaIds} ) {

            push(
                @{ $aHashData->{'Shops'} },
                {
                    ShopId  => $ShopId,
                    Sources => $aLinksViaIds
                }
            );
        }
    }

    return $aHashData;
}

#========================================================================================
# function     _getResourceList
#----------------------------------------------------------------------------------------
# syntax       $aTargetHash = self->__GetResourceList($hData, $aShopIds)
#----------------------------------------------------------------------------------------
# description  Builds and array of hashes vor specific shop.
#----------------------------------------------------------------------------------------
# input        $Table | string - table name where to search for http's
# input        $ShopId | int - shop id
# return       array of following hashes:
#               <ul>
#                  <li>Table      - string value</li>
#                  <li>Shops      - array of following hashes
#                       <ul>
#                           <li>ShopId</li>
#                           <li>Sources - array of following hashes
#                               <ul>
#                                   <li>Links - array of http links to be replaced</li>
#                                   <li>Value - string value which contains links above</li>
#                                   <li>ObjectId - int id of the object</li>
#                                   <li>AttributeId - int id of the attribute of corresponding object</li>
#                                   <li>LanguageId - int id of the language of corresponding value (optional)</li>
#                               </ul>
#                           </li>
#                       </ul>
#                  </li>
#               </ul>
#========================================================================================
sub _getResourceList {
    my $self = shift;

    my ( $Table, $ShopId ) = @_;

    my $aResources = [];
    if ( $Table eq 'localizedstringattribute' ) {

        $aResources = DAL_GetDataLocalizedStringAttr($ShopId);
    }
    elsif ( $Table eq 'stringattribute' ) {

        $aResources = DAL_GetDataStringAttr($ShopId);
    }
    else {

        die "[ERROR]: Unknown table: $Table \n";
    }

    my $hValuesWithHttp = {};
    my $aValuesWithHttp = [];

    # iterate over data values with http's and if it can be replaced with https
    # put add it to target list
    my $i = 0;
    foreach my $res ( @{$aResources} ) {
        my @matches = $res->[0] =~ /(?<=src=['"])http:.+?(?=['"])/gix;
        if (@matches) {

            my @validLinks = grep {
                ( my $link = $_ ) =~ s/http:\/\//https:\/\//g;
                if ( $self->_getTargetResponse($link)->is_success ) { $link }
            } @matches;

            if (@validLinks) {

                $hValuesWithHttp = {
                    ObjectId    => $res->[1],
                    AttributeId => $res->[2],
                    Links       => \@validLinks,
                    Value       => $res->[0]
                };

                $hValuesWithHttp->{'LanguageId'} = $res->[3] if $res->[3];
                $aValuesWithHttp->[ $i++ ] = $hValuesWithHttp;
            }
        }
        else {
            #die "[ERROR]: RegEx is didn't matched but it should, check: \n" . $res->[0];
        }
    }

    return $aValuesWithHttp;
}

sub _getTargetResponse {
    my $self = shift;

    my ($HttpTargetName) = @_;
    my $response;

    Alarm(
        Sub => sub {
            $response = $self->{'UserAgent'}->head($HttpTargetName);
        },
        Timeout   => TIMEOUT,
        OnTimeout => sub {
        },
    );

    #print 'HTTP-Statuscode: ' . $response->code . ' (' . $HttpTargetName . ")\n";
    #print 'StatusLine: ' . $response->status_line . "\n";

    return $response;
}

1;
