#========================================================================================
# SCRIPT WHICH SEARCHES FOR MISSING AND REDUNDANT LANGUAGE TAGS
#========================================================================================

use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::CartridgeConfig qw (CartridgeConfig);
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( ExistsFile ExistsDirectory GetFileContent GetFileNames CorrectPathSlashes fsplit );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw (unique UsePackage );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle RunOnStore );
use DE_EPAGES::Dictionary::API::XML::DictionaryImportHandler qw ( ImportDictionary );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Presentation::API::Template;
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );

##no critic qw(ProhibitExcessComplexity)

my @Warnings;

sub Main {

    my $StoreName;
    my $Password;
    my $LanguageCode = 'de';
    my $Help;
    my $File;
    my $JsLocalisation;
    my $CartridgeName;

    GetOptions(
        'help'              => \$Help,
        'passwd=s'          => \$Password,
        'storename=s'       => \$StoreName,
        'lang=s'            => \$LanguageCode,
        'file=s'            => \$File,
        'js'                => \$JsLocalisation,
        'cartridge=s'       => \$CartridgeName,
    );

    usage() if $Help;
    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    unless ($ENV{'EPAGES_CARTRIDGES'}) {
        die "Environment variable \$EPAGES_CARTRIDGES is not set. Please set it using the command: \n"
        . '. /etc/default/epages6';
    }

    if ($CartridgeName eq 'CodeGenerator' || $CartridgeName eq 'Installer') {
        print STDERR 'Skip test for Cartridge: '.$CartridgeName."\n";
        return
    }

    my $time1 = time;
    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            if ($JsLocalisation) {
                checkLanguageTagsJS($LanguageCode, $File);
            } else {
                checkLanguageTags($LanguageCode, $File, $CartridgeName, $StoreName);
            }
        },
    );
    my $time2 = time;
    printf "\n\nneed %.3f seconds\n", $time2-$time1;
    return;
}

sub checkLanguageTags ($;$$) {
    #INPUT
    my ($LanguageCode, $File, $CartridgeName, $StoreName) = @_;

    #VARS
    my $IncludeHandler = DE_EPAGES::Presentation::API::Template->new;
    my $DictionaryFile = undef;

    my $hAllTemplates = {};
    my @AllDictFiles;

    # first check language tags in JS lib files
    my ($hUnusedJSKeywords, $hTakenJSTags)
        = checkLanguageTagsLib($LanguageCode, $File, $CartridgeName, $StoreName);

    my @TemplateFileNames;
    if (defined $CartridgeName) {
        @TemplateFileNames = @{ _getCartridgeTemplateFiles($CartridgeName) };
    } else {
        foreach my $hTemplate ( @{ LoadClassByAlias('TemplateType')->infoObjects() } ) {

            my $FileName = $hTemplate->{'FileName'};
            next if defined $File and $FileName ne $File;

            my $CompleteFileName;
            eval {
                $CompleteFileName = $IncludeHandler->templateFileName($hTemplate);
            };
            #errorhandling
            if (ExistsError()) {
                my $Error = GetError();
                if( $Error->code eq 'NoTemplateFile' ) {
                    LogError($Error);
                    print "WARNING: $Error\n\n";
                }
                else {
                    $Error->addVars({ 'FileName' => $FileName });
                    Error( $Error );
                }
            } else {
                push @TemplateFileNames, $CompleteFileName;
            }
        }
    }

    #PROCESS
    foreach my $CompleteFileName ( @TemplateFileNames ) {

        if (defined $File) {
            $DictionaryFile = $CompleteFileName;
            $DictionaryFile =~ s/html$/$LanguageCode.xml/;
            $DictionaryFile =~ s/\\/\//g;
        }
        my @DictFiles = grep { ExistsFile($_) } @{$IncludeHandler->dictionaryFiles($CompleteFileName, $LanguageCode)};
        push @AllDictFiles, @DictFiles;
        $hAllTemplates->{$CompleteFileName} = \@DictFiles;
    }


    my $GlobalDictionaryPath = $ENV{'EPAGES_CARTRIDGES'}
        . '/DE_EPAGES/Dictionary/Templates/Dictionary.' . $LanguageCode . '.xml';
    unless (scalar @AllDictFiles) {
        # no templates in cartridge --> add cartridge and global dictionary
        push @AllDictFiles, $GlobalDictionaryPath;

        if (defined $CartridgeName) {
            my $CartridgeDictionaryPath = $ENV{'EPAGES_CARTRIDGES'}. '/DE_EPAGES/'
                . $CartridgeName . '/Templates/Dictionary.' . $LanguageCode . '.xml';
            push @AllDictFiles, $CartridgeDictionaryPath if ExistsFile($CartridgeDictionaryPath);
            # exception for JQuery cartridge because it is the global dict for JS tags
            # exclude global dictionary from unsused tags in case a cartridge name is given
            $DictionaryFile = lc $CartridgeName eq 'jquery' ? '0' : $CartridgeDictionaryPath;
        }
    }

    #normalize
    @AllDictFiles = unique @AllDictFiles;

    my $hDictionaryKeywords = {};
    my $hDictionaryJSKeywords = {};

    foreach my $DictFile (@AllDictFiles) {
        my $Dictionary = {};

        ImportDictionary($DictFile, $Dictionary);
        $Dictionary = $Dictionary->{substr $LanguageCode, 0, 2};

        my $aDictionaryKeywords;
        push @$aDictionaryKeywords, {$_ => {'Value' => $Dictionary->{$_}, 'Counter' => 0}} foreach keys %$Dictionary;

        #$hDictionaryKeywords contains all DictFiles and all keywords that belongs to them
        $hDictionaryKeywords->{$DictFile} = $aDictionaryKeywords;

        # import tags with JS="1" only
        my $DictionaryJS = {};

        ImportDictionary($DictFile, $DictionaryJS, $LanguageCode, 1);
        $DictionaryJS = $DictionaryJS->{substr $LanguageCode, 0, 2};

        my %DictionaryJSKeywords;
        $DictionaryJSKeywords{$_} = {'Value' => $Dictionary->{$_}, 'Counter' => 0} foreach keys %$DictionaryJS;
        #$hDictionaryJSKeywords contains all DictFiles and all keywords with JS="1" that belongs to them
        $hDictionaryJSKeywords->{$DictFile} = \%DictionaryJSKeywords if scalar keys %DictionaryJSKeywords;
    }

    my %MissingKeywords = ();
    my %MissingJSKeywords = ();
    my $GlobalDictionary = '/DE_EPAGES/Dictionary/Templates/Dictionary.'.$LanguageCode.'.xml';


    foreach my $FileName (keys %$hAllTemplates) {

        #$aDictionaries list of all dictionaries belongs to $FileName
        my $aDictionaries = $hAllTemplates->{$FileName};
        my $Content = GetFileContent($FileName);

        # remove javascript variables ${var}.
        $$Content =~ s/\$\{\w+\}//gx;                                            # ${var}
        $$Content =~ s/\$\{\w+?\[[^\]]+\]\}//gx;                                 # ${var[]}
        $$Content =~ s/\$\{\w+?:\[([^\]]+)\]\}//gx;                              # ${var()}
        $$Content =~ s/\{\s*.*:\[.*\].*\s*\}//gx;                                # { hash: ... }
        $$Content =~ s/\{\{\/?\w+(| [^\}]*?)\}\}//gx;                            # {{if}} {{else}} {{/if}} {{html $item.fn('')}}
        $$Content =~ s/\{[0-9]+\}//gx;                                           # {8}

        # look for language tags
        my @TemplateLanguageTags =  $$Content =~ /\{(\w+)}/gx;                   # {Hello} {World}
        push @TemplateLanguageTags, $$Content =~ /\{(\w+)?\[[^\]]+\]\}/gx;       # {Hallo[js]}
        push @TemplateLanguageTags, $$Content =~ /\{(\w+)?:\[([^\]]+)\]\}/gx;    # {Hallo[]}
        push @TemplateLanguageTags, $$Content =~ /\#TRANSLATE\("(\w+)"\)/gx;     # #TRANSLATE("Hello")

        my $hTakenTags = {};

        $hTakenTags->{$_} = {'Tag' => $_, 'Taken' => 0} foreach @TemplateLanguageTags;

        foreach my $Dictionary (reverse @$aDictionaries) {

            my @DictPath = split /Cartridges/, $Dictionary;

            my $IsGlobalDictionary = $DictPath[1] eq $GlobalDictionary ? 1 : 0;

            #$ahDictionaryKeywords all keywords that belongs to $Dictionary
            my $ahDictionaryKeywords = $hDictionaryKeywords->{$Dictionary};

            foreach my $TemplateLanguageTag (@TemplateLanguageTags) {
                my $IsInXML = 0;
                foreach my $hKeyword (@$ahDictionaryKeywords) {
                    my ($XMLKeyword) = keys %$hKeyword;
                    if ($TemplateLanguageTag eq $XMLKeyword) {
                        $hKeyword->{$XMLKeyword}->{'Counter'}++;
                        $IsInXML = 1;
                        $hTakenTags->{$TemplateLanguageTag} = {'Tag' => $TemplateLanguageTag, 'Taken' => 1};
                    }
                }
                if ($IsGlobalDictionary and not $IsInXML and $hTakenTags->{$TemplateLanguageTag}->{'Taken'} == 0) {
                    $MissingKeywords{$FileName}{$TemplateLanguageTag} = undef;
                }
            }
            $hDictionaryKeywords->{$Dictionary} = $ahDictionaryKeywords if scalar @TemplateLanguageTags;
        }
    }

    # check all JS tags that have not been taken yet
    foreach my $Cartridge (keys %$hTakenJSTags) {
        my $CartridgeDictionaryPath = $ENV{'EPAGES_CARTRIDGES'}. '/DE_EPAGES/'
            . $Cartridge. '/Templates/Dictionary.' . $LanguageCode . '.xml';
        my $hTakenCartridgeJSTags = $hTakenJSTags->{$Cartridge};
        foreach my $TemplateLanguageTag (grep { !$hTakenCartridgeJSTags->{$_}->{'Taken'} } keys %$hTakenCartridgeJSTags) {
            # start with cartridge dictionary
            if (defined $hDictionaryJSKeywords->{$CartridgeDictionaryPath}
                && defined $hDictionaryJSKeywords->{$CartridgeDictionaryPath}->{$TemplateLanguageTag}) {
                $hDictionaryJSKeywords->{$CartridgeDictionaryPath}->{$TemplateLanguageTag}->{'Counter'}++;
            # continue with global dictionary
            } elsif (defined $hDictionaryJSKeywords->{$GlobalDictionaryPath}
                && defined $hDictionaryJSKeywords->{$GlobalDictionaryPath}->{$TemplateLanguageTag}) {
                $hDictionaryJSKeywords->{$GlobalDictionaryPath}->{$TemplateLanguageTag}->{'Counter'}++;
            # not in the global dictionary? --> missing keyword
            } else {
                $MissingJSKeywords{$hTakenCartridgeJSTags->{$TemplateLanguageTag}->{'FileName'}}{$TemplateLanguageTag} = undef;
            }
        }
    }

    print ("\nFor language $LanguageCode, following language tags don't have an xml substitution:\n") if scalar keys %MissingKeywords;
    printf ("\nTemplate:\t%s\nTags:\t\t %s\n", $_, join (' ', keys %{$MissingKeywords{$_}})) foreach sort keys %MissingKeywords;

    print "\nFor language $LanguageCode, following JS language tags don't have an xml substitution ",
        "(or JS=\"1\" attribute is missing in dictionary):\n" if scalar keys %MissingJSKeywords;
    printf ("\nTemplate:\t%s\nJS Tags:\t %s\n", $_, join (' ', keys %{$MissingJSKeywords{$_}})) foreach sort keys %MissingJSKeywords;

    my %UnusedKeywords = ();
    foreach my $Dictionary
        (grep {(not defined $DictionaryFile and (not defined $CartridgeName or $_ ne $GlobalDictionaryPath)) or $DictionaryFile eq $_}
        keys %$hDictionaryKeywords) {
        foreach my $hEntry (@{$hDictionaryKeywords->{$Dictionary}}) {
            $UnusedKeywords{$Dictionary}{$_} = undef foreach grep {!$hEntry->{$_}->{'Counter'}} keys %$hEntry;
        }
    }
    print  ("\n\nFor language $LanguageCode, following keywords are unused:\n") if scalar keys %UnusedKeywords;
    printf ("\nDictionary:\t%s\nTags:\t\t %s\n", $_, join (' ', keys %{$UnusedKeywords{$_}})) foreach sort keys %UnusedKeywords;

    print  ("\n\nFor language $LanguageCode, following JS keywords are unused:\n") if scalar keys %$hUnusedJSKeywords;
    printf ("\nDictionary:\t%s\nTags:\t\t %s\n", $_, join (' ', keys %{$hUnusedJSKeywords->{$_}})) foreach sort keys %$hUnusedJSKeywords;

    # get all keywords that have the JS="1" attribute but are never used in any JS template
    my $hUnusedXMLKeywords = _identifyUnusedKeywords($hDictionaryJSKeywords, $GlobalDictionaryPath, $CartridgeName);
    print  ("\n\nFor language $LanguageCode, following keywords have the JS=\"1\" attribute too much:\n") if scalar keys %$hUnusedXMLKeywords;
    printf ("\nDictionary:\t%s\nTags:\t\t %s\n", $_, join (' ', keys %{$hUnusedXMLKeywords->{$_}})) foreach sort keys %$hUnusedXMLKeywords;

    if (scalar @Warnings) {
        print "The following warnings occured:\n";
        print $_ . "\n" foreach @Warnings;
    }
    return;
}

sub checkLanguageTagsJS ($;$) {
    my ($LanguageCode, $File) = @_;

    # get all templates
    my $StoreName = GetCurrentDBHandle()->storeName;
    my @JsTemplateFiles;
    my $hCartridges = CartridgeConfig($StoreName)->section('Cartridges');
    foreach my $Package (keys %$hCartridges) {
        my $CartridgePackage = $Package . '::API::Cartridge';
        UsePackage($CartridgePackage);
        my $Cartridge = $CartridgePackage->new(StoreName => $StoreName);
        my $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Cartridge->cartridgeDirectory;
        my $SourceDir = $CartridgeDir.'/Data/javascript';
        if( -d $SourceDir ) {
            push @JsTemplateFiles,
                grep { /\/templates\// && /\.html$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
        $SourceDir = $CartridgeDir.'/Data/Public/javascript/epages';
        if( -d $SourceDir ) {
            push @JsTemplateFiles,
                grep { /\/templates\// && /\.html$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
    }

    my $GlobalDictionary = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/Dojo/Data/Public/javascript/epages/translation.'.$LanguageCode.'.xml';
    $GlobalDictionary =~ s/\\/\//g;

    my %DictionaryMap = ();
    my $Dictionary = ImportDictionary($GlobalDictionary, undef, $LanguageCode);
    $DictionaryMap{$GlobalDictionary} = $Dictionary->{$LanguageCode};
    $DictionaryMap{$GlobalDictionary}{$_} = 0 foreach keys %{$DictionaryMap{$GlobalDictionary}};

    foreach my $FileName (grep { !/LocalizedWidgetTest\.html$/ } @JsTemplateFiles) {
        next if defined $File and $FileName ne $File;

        # fill dictionary
        my $DictionaryFile = $FileName;
        $DictionaryFile =~ s/\/[^\/]+$/\/translation.$LanguageCode.xml/;
        $DictionaryFile =~ s/\\/\//g;
        unless (exists $DictionaryMap{$DictionaryFile}) {
            my $hLocalDictionary = ImportDictionary($DictionaryFile, undef, $LanguageCode);
            $DictionaryMap{$DictionaryFile} = $hLocalDictionary->{$LanguageCode};
            $DictionaryMap{$DictionaryFile}{$_} = 0 foreach keys %{$DictionaryMap{$DictionaryFile}};
        }
        my $rContent = GetFileContent($FileName);
        my @TemplateLanguageTags = unique $$rContent =~ /[^\$]\{(\w+)}/gx; # {Hello} {World}
        my @NewTags = ();
        foreach my $Tag (@TemplateLanguageTags) {
            if (exists $DictionaryMap{$DictionaryFile}{$Tag}) {
                $DictionaryMap{$DictionaryFile}{$Tag}++;
            } elsif (exists $DictionaryMap{$GlobalDictionary}{$Tag}) {
                $DictionaryMap{$GlobalDictionary}{$Tag}++;
            } else {
                push @NewTags, $Tag;
            }
        }
        if(scalar @NewTags) {
            printf "\nTemplate:\t%s\nTags:\t\t %s\n", CorrectPathSlashes($FileName), join' ', sort unique @NewTags;
        }
    }

    # this.translate('Comment') aus js dateien
    my @JsFiles = ();
    foreach my $Package (keys %$hCartridges) {
        my $CartridgePackage = $Package . '::API::Cartridge';
        UsePackage($CartridgePackage);
        my $Cartridge = $CartridgePackage->new(StoreName => $StoreName);
        my $CartridgeDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$Cartridge->cartridgeDirectory;
        my $SourceDir = $CartridgeDir.'/Data/javascript';
        if( -d $SourceDir ) {
            push @JsFiles,
                grep { /\.js$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
        $SourceDir = $CartridgeDir.'/Data/Public/javascript/epages';
        if( -d $SourceDir ) {
            push @JsFiles,
                grep { /\.js$/ }
                @{ GetFileNames( $SourceDir, {'recursive' => 1} ) };
        }
    }

    foreach my $FileName (@JsFiles) {
        next if defined $File and $FileName ne $File;

        # fill dictionary
        my $DictionaryFile = $FileName;
        $DictionaryFile =~ s/\/[^\/]+$/\/templates\/translation.$LanguageCode.xml/;
        $DictionaryFile =~ s/\\/\//g;
        my $ExistsDirectionaryFile = ExistsFile($DictionaryFile);
        if ($ExistsDirectionaryFile and not exists $DictionaryMap{$DictionaryFile}) {
            my $hLocalDictionary = ImportDictionary($DictionaryFile, undef, $LanguageCode);
            $DictionaryMap{$DictionaryFile} = $hLocalDictionary->{$LanguageCode};
            $DictionaryMap{$DictionaryFile}{$_} = 0 foreach keys %{$DictionaryMap{$DictionaryFile}};
        }
        my $rContent = GetFileContent($FileName);
        # find js direct translations this|me|_this.translate:
        my @TemplateLanguageTags = unique $$rContent =~ /\.translate\('(\w+)'\)/gx;
        # find js comments // comment.translate:
        push @TemplateLanguageTags, split /\s*,\s*/ foreach $$rContent =~ /comment\.translate:([^\n]+)/g;
        my @NewTags = ();
        foreach my $Tag (@TemplateLanguageTags) {
            if ($ExistsDirectionaryFile and exists $DictionaryMap{$DictionaryFile}{$Tag}) {
                $DictionaryMap{$DictionaryFile}{$Tag}++;
            } elsif (exists $DictionaryMap{$GlobalDictionary}{$Tag}) {
                $DictionaryMap{$GlobalDictionary}{$Tag}++;
            } else {
                push @NewTags, $Tag;
            }
        }
        if(scalar @NewTags) {
            printf "\nJsFile:\t%s\nTags:\t\t %s\n", CorrectPathSlashes($FileName), join' ', sort unique @NewTags;
        }
    }

    foreach my $DictionaryFile (keys %DictionaryMap) {
        my @Tags = grep { !$DictionaryMap{$DictionaryFile}{$_} } keys %{$DictionaryMap{$DictionaryFile}};
        if (scalar @Tags) {
            printf ("\nDictionary:\t%s\nTags:\t\t %s\n", CorrectPathSlashes($DictionaryFile), join ' ', sort @Tags);
        }
    }
    return;
}

sub checkLanguageTagsLib ($;$$) {
    my ($LanguageCode, $File, $GivenCartridgeName, $StoreName) = @_;

    # use given cartridge name or all cartridges if not given
    my $aCartridges = defined $GivenCartridgeName
        ? [ 'DE_EPAGES::' . $GivenCartridgeName ] : GetCartridges()->{$StoreName};

    # $hDictionaryKeywords contains all DictFiles and all keywords that belongs to them
    my $hDictionaryKeywords = {};
    # $hTakenTags include all language tags font in the files, their path and cartdige and
    # if the are already taken in a dictionary
    my $hTakenTags = {};

    # import global JS dictionary (ep.dict)
    my $JQueryDictionaryPath = $ENV{'EPAGES_CARTRIDGES'}
        . '/DE_EPAGES/JQuery/Templates/Dictionary.'  . $LanguageCode . '.xml';
    $hDictionaryKeywords
        = _importJSDictionary($JQueryDictionaryPath, $hDictionaryKeywords, $LanguageCode);

    foreach my $Cartridge (@{ $aCartridges }) {
        # construct path of the jquery files
        my ($Package, $CartridgeName) = split(/::/, $Cartridge);
        my $LibPath = join('/', $ENV{'EPAGES_CARTRIDGES'}, $Package, $CartridgeName, 'Data/Public/lib');
        next unless ExistsDirectory($LibPath);

        my $CartridgePath = $LibPath . '/' . lc $Package . '/' . lc $CartridgeName;

        # get all files in the cartridge Data/Public/lib directory
        my $aCartridgeJSLibFiles
            = _getCartridgeJSLibFiles($LibPath, $CartridgePath, $CartridgeName, $LanguageCode);

        # import cartridge JS dictionary (de_epages.<cartridge>.dictionary)
        my $DictionaryPath = $CartridgePath . '/Dictionary.' . $LanguageCode . '.xml';
        $hDictionaryKeywords
            = _importJSDictionary($DictionaryPath, $hDictionaryKeywords, $LanguageCode)
            # do not import jquery dictionary twice
            unless lc $CartridgeName eq 'jquery';

        my $hTakenCartridgeTags = {};

        foreach my $FileName (@$aCartridgeJSLibFiles) {

            my ($Path, $Name) = fsplit ($FileName);
            # remove .tmpl ending, too
            $Name =~ s/\.tmpl$//;
            # exclude test files
            next if $Name =~ m/\-test/;

            # import cartridge JS dictionary (de_epages.<cartridge>.dictionary)
            my $IndividualDictionaryPath = $Path . '/' . $Name . '.' . $LanguageCode . '.xml';
            $hDictionaryKeywords
                = _importJSDictionary($IndividualDictionaryPath, $hDictionaryKeywords, $LanguageCode);

            # parse file content and grep all language tags mentioned in the
            # initial commentaries with the annotation @translation
            # and notated in curly brackets ({<language tag>})
            my $Content = GetFileContent($FileName);
            my @TemplateLanguageTags;
            if ($FileName =~ m/\.js$/) {
                my $Translations = join("\n", $$Content =~ /\@translation([^\@]+?)(?:\@|\*\/)/gx);
                @TemplateLanguageTags = $Translations =~ /\{(\w+)}/gx;
            } else {
                # remove javascript variables ${var}.
                $$Content =~ s/\$\{\w+\}//gx;                                            # ${var}
                $$Content =~ s/\$\{\w+?\[[^\]]+\]\}//gx;                                 # ${var[]}
                $$Content =~ s/\$\{\w+?:\[([^\]]+)\]\}//gx;                              # ${var()}
                $$Content =~ s/\{\s*.*:\[.*\].*\s*\}//gx;                                # { hash: ... }
                $$Content =~ s/\{\{\/?\w+(| [^\}]*?)\}\}//gx;                            # {{if}} {{else}} {{/if}} {{html $item.fn('')}}
                $$Content =~ s/\{[0-9]+\}//gx;                                           # {8}

                # look for language tags
                @TemplateLanguageTags =  $$Content =~ /\{(\w+)}/gx;                   # {Hello} {World}
                push @TemplateLanguageTags, $$Content =~ /\{(\w+)?\[[^\]]+\]\}/gx;       # {Hallo[js]}
                push @TemplateLanguageTags, $$Content =~ /\{(\w+)?:\[([^\]]+)\]\}/gx;    # {Hallo[]}
                push @TemplateLanguageTags, $$Content =~ /\#TRANSLATE\("(\w+)"\)/gx;     # #TRANSLATE("Hello")
            }

            # $hTakenTags include all language tags font in the files, their path and cartdige and
            # if the are already taken in a dictionary
            $hTakenCartridgeTags->{$_} = {
                'Taken' => 0,
                'FileName' => $FileName
            } foreach @TemplateLanguageTags;

            # check for each language tag if it has a substitution in a related JS dictionary
            foreach my $TemplateLanguageTag (@TemplateLanguageTags) {
                my $TmpDictionaryPath;
                # start with individual dictionary
                if (exists $hDictionaryKeywords->{$IndividualDictionaryPath}
                    && defined $hDictionaryKeywords->{$IndividualDictionaryPath}->{$TemplateLanguageTag}) {
                    $TmpDictionaryPath = $IndividualDictionaryPath;
                # check cartridge dictionary next
                } elsif (exists $hDictionaryKeywords->{$DictionaryPath}
                    && defined $hDictionaryKeywords->{$DictionaryPath}->{$TemplateLanguageTag}) {
                    $TmpDictionaryPath = $DictionaryPath;
                # at last check global JS dictionary (in JQuery cartridge)
                } elsif (exists $hDictionaryKeywords->{$JQueryDictionaryPath}
                    && defined $hDictionaryKeywords->{$JQueryDictionaryPath}->{$TemplateLanguageTag}) {
                    $TmpDictionaryPath = $JQueryDictionaryPath;
                } else {
                    # tag not found
                    $TmpDictionaryPath = undef;
                }

                if (defined $TmpDictionaryPath) {
                    $hTakenCartridgeTags->{$TemplateLanguageTag}->{'Taken'} = 1;
                    $hDictionaryKeywords->{$TmpDictionaryPath}->{$TemplateLanguageTag}->{'Counter'}++;
                }
            }
        }
        $hTakenTags->{$CartridgeName} = $hTakenCartridgeTags;
    }

    # identify all keywords from the dictionaries that are not used in any template
    my $hUnusedKeywords
        = _identifyUnusedKeywords($hDictionaryKeywords, $JQueryDictionaryPath, $GivenCartridgeName);

    return ($hUnusedKeywords, $hTakenTags);
}

# import all keywords of a given JS dictionary
sub _importJSDictionary {
    my ($Path, $hDictionaryKeywords, $LanguageCode) = @_;

    if (ExistsFile($Path)) {
        my $Dictionary = {};
        ImportDictionary($Path, $Dictionary);
        $Dictionary = $Dictionary->{substr $LanguageCode, 0, 2};
        if (defined $Dictionary) {
            # do not overwrite already imported dictionaries
            unless (exists $hDictionaryKeywords->{$Path}) {
                my %DictionaryKeywords;
                $DictionaryKeywords{$_} = {'Value' => $Dictionary->{$_}, 'Counter' => 0} foreach keys %$Dictionary;
                #$hDictionaryKeywords contains all DictFiles and all keywords that belongs to them
                $hDictionaryKeywords->{$Path} = \%DictionaryKeywords;
            }
        } else {
            push @Warnings, "Dictionary $Path does not contain keywords in the language \'$LanguageCode\'.";
        }
    }

    return $hDictionaryKeywords;
}

sub _getCartridgeTemplateFiles {
    my ($CartridgeName) = @_;

    my @CartridgeTemplateFiles;
    my $TemplatesPath = join('/', $ENV{'EPAGES_CARTRIDGES'}, 'DE_EPAGES', $CartridgeName, 'Templates');
    # exception for SimpleError.html as this does only exist for four languages
    @CartridgeTemplateFiles = grep { $_ !~ /(\.xml|SimpleError\.html)$/ } @{ GetFileNames( $TemplatesPath, {'recursive' => 1} ) }
        if ExistsDirectory($TemplatesPath);
    return \@CartridgeTemplateFiles;
}

sub _getCartridgeJSLibFiles {
    my ($Path, $CartridgePath, $CartridgeName, $LanguageCode) = @_;

    return undef if lc $CartridgeName eq 'dojo';

    my @CartridgeLibFiles;
    my $epPath = $Path . '/ep';
    # include /ep directory only for the jquery cartridge
    if (lc $CartridgeName eq 'jquery' && ExistsDirectory($epPath)) {
        # get all files ignoring the 'i18n' directory
        my @epFiles = grep { /\.(js|tmpl\.html)$/ } @{ GetFileNames( $epPath, {'recursive' => 1, 'ignore' => [ 'i18n' ]} ) };
        push @CartridgeLibFiles, @epFiles;
    }
    my $ckeditorPluginsPath = $Path . '/ckeditor/plugins';
    # include /ep directory only for the jquery cartridge
    if (ExistsDirectory($ckeditorPluginsPath)) {
        # get all files ignoring the 'i18n' directory
        my @ckeditorFiles = grep { /\.(js|tmpl\.html)$/ } @{ GetFileNames( $ckeditorPluginsPath, {'recursive' => 1} ) };
        push @CartridgeLibFiles, @ckeditorFiles;
    }
    # do not include 'guidedtour' cartridge as it includes only external content
    if ((lc $CartridgeName ne 'guidedtour') && ExistsDirectory($CartridgePath)) {
        my @CartridgeFiles = grep { /\.(js|tmpl\.html)$/ } @{ GetFileNames( $CartridgePath, {'recursive' => 1} ) };
        push @CartridgeLibFiles, @CartridgeFiles;
    }
    # include files in root directory
    if (ExistsDirectory($Path)) {
        my @LibFiles = grep { /\.(js|tmpl\.html)$/ } @{ GetFileNames( $Path ) };
        push @CartridgeLibFiles, @LibFiles;
    }

    return \@CartridgeLibFiles;
}

sub _identifyUnusedKeywords {
    my ($hDictionaryKeywords, $GlobalDictionaryPath, $CartridgeName) = @_;

    my %UnusedKeywords;

    foreach my $Dictionary (keys %$hDictionaryKeywords) {
        foreach my $key (keys %{$hDictionaryKeywords->{$Dictionary}}) {
            $UnusedKeywords{$Dictionary}{$key} = undef
                unless ($hDictionaryKeywords->{$Dictionary}->{$key}->{'Counter'}
                # exclude global dictionary in case of a single cartridge
                    || (defined $CartridgeName && $Dictionary eq $GlobalDictionaryPath));
        }
    }

    return \%UnusedKeywords;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password              ()
    -storename  name of store (required)            ()
    -lang       language code, e.g. en, de, fr      (de)
    -file       template file name                  ()
    -cartridge  cartridge name e.g Amazon, PayPal   ()

flags:
    -help       show the command line options
    -js         test javascript templates

Example:
    perl $0 -storename Store -lang en
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
    Debug => 1
);
