use Config;
use DE_EPAGES::Core::API::File qw (
    ExistsFile CorrectPathSlashes GetFileSize ExistsDirectory
    GetFileNames GetSubDirectories MoveFile CopyFile fsplit DeleteFile );
use DE_EPAGES::Core::API::ImageOptimization qw( OptiPng );
use DE_EPAGES::Core::API::PerlTools qw (GenerateShortGUID);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::TestParameter qw (TestString);
use Getopt::Long;
use Image::Magick;
use strict;
use Test::More;


sub _doOptiPng {
    my ($ActualFullFileName, $hOptions) = @_;

    TestString('Image to optimize', $ActualFullFileName);
    TestString('OptiPng method', $hOptions->{'optipngmethod'});

    # backup copy
    my $RandomString = GenerateShortGUID();

    my ($ActualPath, $ActualFileName, $ActualExtension) = fsplit($ActualFullFileName);
    my $DirtyFullFileName = join(CorrectPathSlashes('/'), ($ActualPath, $ActualFileName.'_'.$RandomString.$ActualExtension));

    # Make copy
    CopyFile($ActualFullFileName, $DirtyFullFileName);

    OptiPng($DirtyFullFileName);

    return -1 if !ExistsFile($DirtyFullFileName);

    # replace unoptimized with optimized one only if size reduces
    my $DirtyFileSize = GetFileSize($DirtyFullFileName);
    my $ActualFileSize = GetFileSize($ActualFullFileName);
    if($DirtyFileSize < $ActualFileSize and $DirtyFileSize > 0) {
        my $Percentage = 100.0 - $DirtyFileSize * 100.0 / $ActualFileSize;  ## no critic ( ProhibitMagicNumbers )

        if(not $hOptions->{'lowerlimit'} or ($hOptions->{'lowerlimit'} and $Percentage > $hOptions->{'lowerlimit'})) {
            # COMPARE IMAGE CONTENT
            my $OriginalImage = Image::Magick->new;
            $OriginalImage->Read($ActualFullFileName);
            my $OptimizedImage = Image::Magick->new;
            $OptimizedImage->Read($DirtyFullFileName);
            my $ErrorValue = $OriginalImage->Compare('image' => $OptimizedImage, 'metric' => 'rmse')->Get('error');
            undef $OriginalImage;
            undef $OptimizedImage;

            if($ErrorValue != 0) {
                DeleteFile($DirtyFullFileName) if ExistsFile($DirtyFullFileName);
                # error -2: image bitmap content does NOT match original!
                return -2;    ## no critic ( ProhibitMagicNumbers )
            }

            print "deleting old file: $ActualFullFileName\n" if $hOptions->{'debug'} eq 'true';
            DeleteFile($ActualFullFileName);
            print "renaming new file from $DirtyFullFileName to $ActualFullFileName\n" if $hOptions->{'debug'} eq 'true';
            MoveFile($DirtyFullFileName, $ActualFullFileName);
            return $Percentage;
        }
    }

    DeleteFile($DirtyFullFileName) if ExistsFile($DirtyFullFileName);
    return 0;
}

sub Main {
    my ($Help, $LowerLimit, $ShowSummary, $Cartridge, $CompanyName);
    $LowerLimit = 0;
    $CompanyName = '';

    GetOptions(
        'help'              => \$Help,
        'lowerlimit=i'      => \$LowerLimit,
        'companyname=s'     => \$CompanyName,
        'cartridge=s'       => \$Cartridge,
        'summary'           => \$ShowSummary
    );

    usage() if $Help;

    #verbose is 1 if quiet and test are 0
    $CompanyName = 'DE_EPAGES' if not $CompanyName;

    # composer helps
    my $SavedPercentage = 0.0;
    my $SumSavedPercentage = 0.0;
    my $NumImages = 0;
    my $NumOptimizedImages = 0;
    my $NumUnchangedImages = 0;
    my $NumErrorImages = 0;
    my $SumImageSizesBefore = 0;
    my $SumImageSizesAfter = 0;
    my $aErrorImages = ();
    my $hOptimizedImages = {};


    #start process

    # for all cartridges
    foreach my $CartridgePath ( @{GetSubDirectories($ENV{'EPAGES_CARTRIDGES'} . '/' . $CompanyName)} ){
        #next if cartridge path is not matched to target cartridge if $Cartridge is given
        next if ($Cartridge ne '' and not $CartridgePath =~m{\/$Cartridge$}x );

        #generate path to png folder
        my $DataPath = $CartridgePath . '/Data';

        #next if cartridge have no png files
        next unless ExistsDirectory($DataPath);

        my $aCartridgeDataFiles = GetFileNames($DataPath, { 'recursive' => 1});

        foreach my $DataFile (@$aCartridgeDataFiles) {
            #ignore cvs generated files
            next if ($DataFile =~ m{\/\.\#[a-zA-Z\.\_0-9]*}x);

            #take notice only for *.png files
            next unless $DataFile =~ m{\.png$}x;
            my $PngFile = $DataFile;

            #increase image counter
            $NumImages++;

            #compute saved percentage
            $SavedPercentage = _doOptiPng(
                $PngFile, {
                    'optipngmethod' => '-o7',
                    'quiet' => 'true',
                    'debug' => 'false',
                    'lowerlimit' => $LowerLimit                 #check if lower limit is defined
                }
            );

            #count size of all png images before optimization
            $SumImageSizesBefore += GetFileSize($PngFile);

            #check percentge
            if($SavedPercentage > 0) {

                #count size of all png images after optimization
                $SumImageSizesAfter += GetFileSize($PngFile);

                #list optimized images
                $hOptimizedImages->{$PngFile}->{'SavedPercentage'} = $SavedPercentage if $ShowSummary;

                #count optimized images
                $NumOptimizedImages++;

                #count saved percentage
                $SumSavedPercentage += $SavedPercentage;
                ok($SavedPercentage >= 0, "optimize '$PngFile'");
            } elsif($SavedPercentage == 0) {
                $NumUnchangedImages++;
            } else {
                ok($SavedPercentage >= 0, "optimize '$PngFile'");
                push(@$aErrorImages, $PngFile);
                $NumErrorImages++;
            }
        }
    }
    my $AvgSavedPercentage = 0;
    $AvgSavedPercentage = sprintf('%.2f%', $SumSavedPercentage / $NumOptimizedImages) if $NumOptimizedImages > 0;
    my $TotalSavedBytes = $SumImageSizesBefore-$SumImageSizesAfter;

    if($ShowSummary) {
    print <<END_SUMMARY ;
Summary:
    Total number of images found:           $NumImages
    Number of images optimized:             $NumOptimizedImages
    Average saved file size in per cent:    $AvgSavedPercentage
    Saved file size in bytes:               $TotalSavedBytes
    Number of images already optimimal:     $NumUnchangedImages
    Number of images skipped due to errors: $NumErrorImages

END_SUMMARY

        if(scalar(keys %$hOptimizedImages) > 0) {
            print "\nThe following images were optimized:\n";
            foreach my $OptImage (keys %$hOptimizedImages) {
                my $FileSavedPercentage = sprintf('%.2f%', $hOptimizedImages->{$OptImage}->{'SavedPercentage'});
                print '    ', $OptImage, ' (-', $FileSavedPercentage, ")\n";
            }
        }
    }
    if(defined $aErrorImages and scalar(@$aErrorImages) > 0) {
        print "\nErrors occured for these images:\n";
        foreach my $ErrImage (@$aErrorImages) {
            print '    ', $ErrImage, "\n";
        }
    }

    return;
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    For all png images in the /Data/ directory of each cartridge for the given database type
    apply the optipng optimization application. Only images where the optimization leads to
    smaller file sizes are changed.

    Currently this script does only work with PNG images!

Options:
    -lowerlimit         min. optimzation level in per cent                      (optional)
    -cartridge          optimize images for specific cartridge, i.e. Shop       (optional)
    -companyname        name of company e.g "DE_EPAGES" (default), "ES_EPAGES"  (optional)

Flags:
    -help               show the command line options                           (optional)
    -summary            show summary at the end                                 (optional)

Examples:
    perl $0
    perl $0 -cartridge Shop
    perl $0 -cartridge Shop -companyname ES_EPAGES

END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);

done_testing();

1;
