#==============================================================================
# package      DE_EPAGES::Presentation::UI::Site
# state        private
#==============================================================================
package DE_EPAGES::Presentation::UI::Site;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use HTTP::Status qw(HTTP_OK);

use DE_EPAGES::Object::API::Factory     qw ( ExistsObject LoadObjectByGUID ExistsObjectByGUID );
use DE_EPAGES::Presentation::API::DereferrerToken qw(ValidateDereferrerToken);
use DE_EPAGES::Trigger::API::Trigger    qw ( TriggerHook );
use DE_EPAGES::Core::API::Error         qw ( ExistsError Error GetError );
use DE_EPAGES::Core::API::Password      qw ( CryptPassword );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestDateTime );
use DE_EPAGES::Presentation::API::User  qw ( GetUserFolder ExistsUserByEMail CreateUser );
use DE_EPAGES::Core::API::Net           qw ( GetHttpResponse );
use DE_EPAGES::Core::API::File          qw ( WriteFile );
use URI::Escape                         qw ( uri_unescape );

sub ErrorDefinition {
    return {
        'UnknownAttribute' => 'Attribute "#Attribute" is unknown in class "#Class"',
        'ValueCountNotEqual' => 'Form value count for attribute "#Attribute" is not equal, expected #Expected, given #Given',
        'NotFromSameSite' => 'Site (#ObjectSiteID) of object #ObjectGUID differs from expected Site (#SiteID)',
    }
}

#==============================================================================
# Event handlers
#==============================================================================

#========================================================================================
# function     SaveGUIDList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveGUIDList( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction handler for page preview and advanced design.
#               Extracts form input data from the formloop together with localized
#               strings. Updates text, positions and parent objects and deletes objects.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SaveGUIDList {
    my $self = shift;
    my ($Servlet) = @_;

    my $Site = $Servlet->object;

    my $Form = $Servlet->form;
    # save the shop association for the featurepack
    my $hValues = $Form->form($Site, 'SaveGUIDList');

    # delete objects
    if( $hValues->{'DeleteObjects'} ) {
        my @GUIDs = map { $_->{'DeleteGUID'} } @{$hValues->{'DeleteObjects'}};
        $self->_deleteGUIDList($Servlet, \@GUIDs);
    }

    # set new attribute values for objects and objects with localized attributes
    # collect values to object
    my %Objects;
    foreach my $hElement (@{$hValues->{'ListedObjects'}}, @{$hValues->{'LocalizedValues'}}) {
        my $LanguageID = exists $hElement->{'LanguageID'} ? $hElement->{'LanguageID'} : undef;
        my $GUID  = $hElement->{(defined $LanguageID ? 'Localized' : '').'GUID'};
        my $Name  = $hElement->{(defined $LanguageID ? 'Localized' : '').'Name'};
        my $Value = $hElement->{(defined $LanguageID ? 'Localized' : '').'Value'};

        my $hObject;
        if( exists $Objects{$GUID} ) {
            $hObject = $Objects{$GUID};
        }
        else {
            my $Object = _LoadObject($Servlet, $GUID); # load object with objectid or guid
            if ($Object) {
                if( !$Servlet->hasPermission($Object, 'Save') ) {
                    $Form->addFormError({
                        'Reason' => 'PermissionDenied',
                        'Name'   => 'GUID',
                        'Form'   => 'SaveGUIDList',
                        'Value'  => $GUID
                    });
                    $Object = undef;
                }
            }

            $hObject = { 'Object' => $Object };
            $Objects{$GUID} = $hObject; # setting Object to undef prevents additional checks for other attributes
        }

        next unless defined $hObject->{'Object'};

        my $Attribute = $hObject->{'Object'}->class->attribute($Name);

        # test parameter
        eval {
            if( $Attribute->get('Type') eq 'Integer' ) {
                TestInteger($Name, $Value);
            } elsif( $Attribute->get('Type') eq 'DateTime' ) {
                TestDateTime($Name, $Value);
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            $Form->addFormError({
                'Reason'=> $Error->code,
                'Form'  => 'SaveGUIDList',
                'Name'  => $Name,
            }) ;
            next;
        }

        # load object by id for attribute which is object
        if ($Attribute->get('IsObject')) {
            $Value = eval {
                TestInteger( $Name, $Value );
                $Site->loadObject($Value, 'Object', {'AllowSystemSite' => 1});
            };
            if (ExistsError()) {
                my $Error = GetError();
                $Form->addFormError({
                    'Reason'=> $Error->code,
                    'Form'  => 'SaveGUIDList',
                    'Name'  => $Name,
                    'Value' => $Value,
                }) ;
                next;
            }

            if (not $Servlet->hasPermission($Value, 'View')) {
                $Form->addFormError({
                    'Reason' => 'PermissionDenied',
                    'Name'   => $Name,
                    'Form'   => 'SaveGUIDList',
                    'Name'   => $GUID
                });
                next;
            }
            elsif ($Name eq 'Parent' && !$Value->isAllowedChild($hObject->{'Object'}) ) {
                $Form->addFormError({
                    'Reason' => 'WrongType',
                    'Form'   => 'SaveGUIDList',
                    'Name'   => $Name,
                    'Value'  => $Value->id,
                });
                next;
            }
        }

        if (defined $LanguageID) {
            $hObject->{'LocalizedValues'}{$LanguageID}{$Name} = $Value;
        }
        else {
            $hObject->{'Values'}{$Name} = $Value;
        }
    }
    $Form->executeFormError;

    # grep existing objects
    my @Objects = grep { defined $_->{'Object'} } values %Objects;

    $self->updateGUIDListWrapper(\@Objects);

    TriggerHook('UI_AfterSaveGUIDList', {
        'Shop' => $Site,
        'Objects' => \@Objects,
    });

    return;
}

#========================================================================================
# function     _deleteGUIDList
# state        private
#----------------------------------------------------------------------------------------
# syntax       _deleteGUIDList($Servlet, $aGUIDs);
#----------------------------------------------------------------------------------------
# description  Deletes all objects from list
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $aGUIDs | list of GUIDs of objects to delete | ref.array.string
#========================================================================================
sub _deleteGUIDList {
    my $self = shift;
    my ($Servlet, $aGUIDs) = @_;

    my $Form = $Servlet->form;

    foreach my $GUID ( @$aGUIDs ) {
        my $Object = _LoadObject($Servlet, $GUID);
        if ($Object) {
            if ($Servlet->hasPermission($Object, 'Save')) {
                $self->deleteGUIDListWrapper ($Object);
            } else {
                $Form->addFormError({
                    'Reason' => 'PermissionDenied',
                    'Name'   => 'GUID',
                    'Form'   => 'SaveGUIDList',
                    'Value'  => 'Delete'
                });
            }
        }
    }
}

#========================================================================================
# function     deleteGUIDListWrapper
# state        protected
#----------------------------------------------------------------------------------------
# syntax       $self->deleteGUIDListWrapper( $Object );
#----------------------------------------------------------------------------------------
# description  Delete the Object from the list
#----------------------------------------------------------------------------------------
# input        $Object | Object | object
#========================================================================================
sub deleteGUIDListWrapper {
    my $self = shift;
    my ($Object) = @_;

    return $Object->delete();
}

#========================================================================================
# function     updateGUIDListWrapper
# state        protected
#----------------------------------------------------------------------------------------
# syntax       $self->updateGUIDListWrapper( $aObjects );
#----------------------------------------------------------------------------------------
# description  Update the Objects from the list
#----------------------------------------------------------------------------------------
# input        $aObjects | array ref
#========================================================================================
sub updateGUIDListWrapper {
    my $self = shift;
    my ($aObjects) = @_;

    # update objects
    foreach my $hObject ( @{$aObjects} ) {

        my $Object = $hObject->{'Object'};
        $Object->set($hObject->{'Values'}) if exists $hObject->{'Values'};
        if (exists $hObject->{'LocalizedValues'}) {
            foreach my $LanguageID ( keys %{$hObject->{'LocalizedValues'}} ) {
                $Object->set( $hObject->{'LocalizedValues'}{$LanguageID},  $LanguageID );
            }
        }
    }
    return;
}

#========================================================================================
# function     _LoadObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = _LoadObject($Servlet, $ID);
#----------------------------------------------------------------------------------------
# description  Returns the object for the given ObjectID or GUID.
#               Adds a form error and returns undef if the ObjectID or GUID does not exist.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $ID | id | integer
# return       $Object | object | object
#========================================================================================
sub _LoadObject {
    my ($Servlet, $ID) = @_;

    my $Form = $Servlet->form;

    # ID is ObjectID
    if ($ID =~ /^\d+$/) {
        if (not ExistsObject($ID)) {
            $Form->addFormError({
                'Reason' => 'NoResults_ID',
                'Name'   => 'ID',
                'Form'   => 'SaveGUIDList',
                'Value'  => $ID
            });
            return undef;
        }
        return $Servlet->object->loadObject($ID, 'Object', {'AllowSystemSite' => 1});
    # ID is GUID
    } else {
        if (not ExistsObjectByGUID($ID)) {
            $Form->addFormError({
                'Reason' => 'NoResults_GUID',
                'Name'   => 'GUID',
                'Form'   => 'SaveGUIDList',
                'Value'  => $ID
            });
            return undef;
        }
        return LoadObjectByGUID($ID);
    }
}

sub SaveUserRegistration {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $Site = $Servlet->object;

    my $hValues = $Form->form($Site, 'UserRegistration', 1);

    _checkRegistration($Servlet, $hValues);

    my $CryptedPassword = eval { CryptPassword($hValues->{'Password'}); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'NativeDie' and $Error->shortMessage eq 'Wide character in crypt';
        $Servlet->form->executeFormError({
            'Reason'=> 'PASSWORD_WRONG',
            'Form'  => 'SaveRegistration',
            'Name'  => 'Password',
        }) ;
    }

    $hValues->{'Name'} = $hValues->{'Alias'}
        unless (defined $hValues->{'Name'} && $hValues->{'Name'} =~ /\S+/);

    unless ($Form->existsFormError) {
        # create an inactive user
        # the user has to activate himself by clicking on the activation link
        # in the email
        my $User = CreateUser($Site, {
            Alias => $hValues->{'Alias'},
            Name => $hValues->{'Name'},
            EMail => $hValues->{'EMail'},
            Password => $CryptedPassword,
            LocaleID => $Servlet->locale,
            LanguageID => $Servlet->languageID,
            CurrencyID => $Servlet->currencyID,
            IsActivated => 0,
        });
        my $hArgs = {
            'MailFunction' => sub{},
            'User' => $User,
            'Site' => $Site,
            'Form' => $hValues,
        };
        TriggerHook('UI_UserRegistration', $hArgs);
    }
    $Form->executeFormError;

    return;
}

#========================================================================================
# function     _checkRegistration
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkRegistration($Servlet, $hValues);
#----------------------------------------------------------------------------------------
# description  Evaluates registration information such as Login and Password. Adds
#               form errors either if the Login is missing or if it exists already or if
#               the password exists or if both passwords don't match. Checks if all data
#               which were set mandatory due to the registration set have been entered.
#               Used at storefront only.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# input        $hValues | registration data | ref.hash
#========================================================================================
sub _checkRegistration {
    my ($Servlet, $hValues) = @_;
    my $Site = $Servlet->object;
    my $Form = $Servlet->form;

    # skip form errors, to show all form errors together
    if (not defined $hValues->{'Alias'} or $hValues->{'Alias'} !~ /\S/) {
        $Form->addFormError({
            'Reason'=> 'NO_USER_NAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'Alias',
        });
    }
    elsif (ExistsUserByEMail($Site, $hValues->{'Alias'})) {
        $Form->addFormError({
            'Reason'=> 'LOGIN_EXISTS',
            'Form'  => 'SaveRegistration',
            'Name'  => 'Alias',
        });
    }

    foreach my $child (@{GetUserFolder($Site)->children}) {
        if ($child->get('EMail') eq $hValues->{'EMail'}) {
            $Form->addFormError({
                'Reason'=> 'EMAIL_ADDRESS_EXISTS',
                'Form'  => 'SaveRegistration',
                'Name'  => 'EMail',
            });
            last;
        }
    }

    # compare passwords
    $hValues->{$_} = '' foreach grep { not defined $hValues->{$_} } (qw (PasswordConf Password));
    if ($hValues->{'Password'} !~ /\S/) {
        $Form->addFormError({
            'Reason'=> 'NO_USER_PWD',
            'Form'  => 'SaveRegistration',
            'Name'  => 'Password',
        });
    }
    elsif ($hValues->{'Password'} ne $hValues->{'PasswordConf'}) {
        $Form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'PasswordConf',
        });
        $Form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'SaveRegistration',
            'Name'  => 'Password',
        });
    }

    return;
}
#========================================================================================
# function     GetFeed
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->GetFeed($Servlet);
#----------------------------------------------------------------------------------------
# description  fetches a RSS Feed, stores it and writes it content back to the servlet
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub GetFeed {
    my $self = shift;
    my ($Servlet) = @_;
    my $Url = $Servlet->params('feedurl'); # 'http://rss.golem.de/rss.php?tp=av&feed=RSS2.0';

    my $Response = GetHttpResponse($Url,undef,'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; WOW64; Trident/4.0;)');
    my $Content = $Response->content;
    if ($Response->code == HTTP_OK) {
        my $FileName = $Url;
        $FileName =~ s/[^0-9a-zA-Z]/_/g;
        $FileName = $ENV{'EPAGES_WEBROOT'}.'/RssFeeds/'.$FileName.'.xml';
        WriteFile($FileName, \$Content, { create_directory=>1 } );
    }
    # change Content-Type of every Response to xml to avoid parsing error in rssreader when content is delivered as text/html
    $Servlet->response->header('Content-Type','text/xml');
    $Servlet->response->code($Response->code);
    $Servlet->response->content($Content);
    $Servlet->params('ExitAfterEvent', 1);
}

#========================================================================================
# function     DereferredRedirect
# state        private
#----------------------------------------------------------------------------------------
# syntax       DereferredRedirect($Servlet);
#----------------------------------------------------------------------------------------
# description  dereferes header and redirects to given url
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub DereferredRedirect {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    eval {
        my $Url = $Servlet->params('URL'); # 'http://facebook.de/'
        my $nonce = $Servlet->params('nonce'); # base 64 encoded token

        # test if we have a valid url by checking the token
        # token = base64_encode(sha256_hex(StoreName.ShopAlias.URL.ProviderSecret.timestamp)."_".timestamp)
        ValidateDereferrerToken($nonce,$Url);

        my $Response = $Servlet->response;
        $Response->code(HTTP_OK);
        $Response->header( 'Content-type' => 'text/html' );
        $Response->content( '<html><head>
            <meta http-equiv="Refresh" content="0;URL=' . uri_unescape($Url) . '"/>
            <meta http-equiv="Referer" content="">
            </head><body></body></html>' );
        $Servlet->vars( 'ExitAfterEvent', 1 );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        Error($Error);
    }

    return;
}

1;

