#========================================================================================
# package      DE_EPAGES::PriceMinister::API::Object::PriceMinisterOffer
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for PriceMinisterOffer.
#========================================================================================
package DE_EPAGES::PriceMinister::API::Object::PriceMinisterOffer;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Core::API::TestParameter qw ( TestFloat TestString );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::PriceMinister::API::Constants qw ( %PRICEMINISTER_ONLINESTATUS );
use DE_EPAGES::PriceMinister::API::Object::PriceMinisterUpload qw (UploadPriceMinisterProduct);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
##no critic (ProhibitLongLines)


sub ErrorDefinition {
    return {
            'MissingUPCEAN' => 'UPC or EAN code missing. You cannot list a product without UPC or EAN code.',
            'MissinPSIN'    => 'PSIN code missing. You cannot list a product without an assigned PSIN.',
    };
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $PriceMinisterOffer->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'PriceMinisterOffer' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $PriceMinisterOffer->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PriceMinisterOffer = DE_EPAGES::PriceMinister::API::Object::PriceMinisterOffer->insert($hInfo);
# example      my $Class = LoadClassByAlias('PriceMinisterOffer');
#               my $PriceMinisterOffer = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertPriceMinisterOffer' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $PriceMinisterOffer | new object | object
# hook         OBJ_InsertPriceMinisterOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PriceMinisterOfferID - object identifier    - int
#               </ul>
#========================================================================================
sub insert {
#LogDebug (caller(0),0);
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertPriceMinisterOffer', {'Object' => $self, 'PriceMinisterOfferID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Product CurrencyID PortalConfig ParentOfferID PriceMinisterCondition PriceMinisterConditionNote PriceMinisterPrice LastStockLevel LastStockUpdate OnlineStatus LastBestPriceUpdate BestPrice IsPriceChanged PSIN ErrorCode ErrorMessage ImportID ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PriceMinisterOffer->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeletePriceMinisterOffer' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeletePriceMinisterOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PriceMinisterOfferID - object identifier    - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeletePriceMinisterOffer', {'Object' => $self, 'PriceMinisterOfferID' => $self->id });
    LoadClassByAlias('PriceMinisterOffer')->deleteObjects('ByParentOfferID', [$self->id]);

    $self->set({ 'ParentOfferID' => undef });

    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PriceMinisterOffer->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdatePriceMinisterOffer' and 'OBJ_AfterUpdatePriceMinisterOffer'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdatePriceMinisterOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PriceMinisterOfferID - object identifier    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdatePriceMinisterOffer | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PriceMinisterOfferID - object identifier    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
#LogDebug (caller(0),0);
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    # find child offers and set there as well.
    foreach my $Child (@{$self->children}){
        $Child->set($hValues, $LanguageID);
    }
    TriggerHook('OBJ_BeforeUpdatePriceMinisterOffer', {'Object'=>$self, 'Values'=> $hValues, 'PriceMinisterOfferID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdatePriceMinisterOffer',  {'Object'=>$self, 'Values'=> $hValues, 'PriceMinisterOfferID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     findChildren
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PriceMinisterOffer->findChildren
#----------------------------------------------------------------------------------------
# description  This function returns a list of child offers if they exist.
#----------------------------------------------------------------------------------------
# return       $aPriceMinisterOrders | list reference on PriceMinisterOffers | ref.array.object
#========================================================================================
sub findChildren {
#LogDebug (caller(0),0);

    my $self = shift;
    return [
        map  { $_->{'ObjectID'} }
        # sort by Position and if Position is equal, sort by Alias
        sort { $a->{'Position'} <=> $b->{'Position'} || $a->{'Alias'} cmp $b->{'Alias'} }
        @{__PACKAGE__->dal->get('ByParentOfferID', [$self->id])}];
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::PriceMinister::API::Object::PriceMinisterOffer->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
#LogDebug (caller(0),0);

    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'CurrencyID' => '',
        'OnlineStatus' => $PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'},
        'IsPriceChanged' => 0,
    };
}

#========================================================================================
# function     checkNextState
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::PriceMinister::API::Object::PriceMinisterOffer->checkNextState( $event, $hInfo );
# example      $Offer->checkNextState('start', {}); # trigger event 'assign' to transit to next state.
#----------------------------------------------------------------------------------------
# description  Sets PriceMinisterOffer to a new state depending on the existing state and the
#               triggered event
#----------------------------------------------------------------------------------------
# input        $event | event key | string
#               valid events are:
#               <li>start                - start offer (Transition to OFFER_PROCESSING)
#               <li>processResultFailure - failed upload (Transition to OFFER_FAILED)
#               <li>processResultSuccess - successful upload (.. to OFFER_ONLINE)
#               <li>disable              - disable offer (.. to OFFER_HIDDEN)
#               <li>enable               - enable offer (.. to OFFER_ONLINE)
# return       $NewStatus | new status value, undef if there is no change. | integer
#========================================================================================
sub checkNextState {
#LogDebug (caller(0),0);
    my $self = shift;
    my ($event) = @_;
    my $NewStatus;
    # force valid state changes
    my $OnlineStatus = $self->get('OnlineStatus');

    LogDebug ('old Status', $OnlineStatus);
    if(ElementOf($OnlineStatus,
           ($PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'},
            $PRICEMINISTER_ONLINESTATUS{'OFFER_FAILED'},
            $PRICEMINISTER_ONLINESTATUS{'OFFER_HIDDEN'}))) {
        if ($event eq 'start' and $self->get('PSIN')) {
            $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_PROCESSING'})});
        }
    }
    elsif($OnlineStatus eq $PRICEMINISTER_ONLINESTATUS{'OFFER_PROCESSING'}) {
        if ($event eq 'processResultSuccess') {
            $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_ONLINE'})});
        } elsif ($event eq 'processResultFailure') {
            $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_FAILED'})});
        } elsif ($event eq 'disable') {
#           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_HIDDEN'})});
           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'})}); #TODO Evaluate Hidden Offers
        }
    }
    elsif($OnlineStatus eq $PRICEMINISTER_ONLINESTATUS{'OFFER_ONLINE'}) {
        if ($event eq 'disable') {
#           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_HIDDEN'})});
           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'})}); #TODO Evaluate Hidden Offers
        } elsif ($event eq 'addPending') {
           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_PROCESSING'})});
        }
    }
    elsif($OnlineStatus eq $PRICEMINISTER_ONLINESTATUS{'OFFER_HIDDEN'}) { # should not appear, because hiding is disabled
        if ($event eq 'enable') {
           $self->set({'OnlineStatus' => ($NewStatus = $PRICEMINISTER_ONLINESTATUS{'OFFER_ONLINE'})});
        }
    }
    LogDebug ($event.' '.'Offer='.$self->id, 'New status='.$NewStatus);
    return $NewStatus;
}

sub languageID { return shift->{'LanguageID'}; }

#========================================================================================
# function     updateValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->updateValues( $hVormValues );
#----------------------------------------------------------------------------------------
# description  Updates PriceMinisterOffer with new form values and communicates changes to
#               PriceMinister platform.
#----------------------------------------------------------------------------------------
# input        $hFormValues | input form values | hash.ref
#               possible keys are:
#               <li>PriceMinisterPrice</li>
#               <li>PriceMinisterCondition</li>
#               <li>PriceMinisterConditionNote</li>
#               <li>PriceMinisterPrice</li>
# return       null
#========================================================================================
sub updateValues {
#LogDebug (caller(0),0);
    my $self = shift;
    my ($hFormValues) = @_;
    my $hValuesOffer = {};

    my $OnlineStatus = $self->get('OnlineStatus');
    my $OldPriceMinisterPrice = $self->get('PriceMinisterPrice') // 0.00;
    my $NewPriceMinisterPrice = $hFormValues->{'PriceMinisterPrice'} // 0.00;
    my $OldPriceMinisterCondition = $self->get('PriceMinisterCondition') // '';
    my $NewPriceMinisterCondition = $hFormValues->{'PriceMinisterCondition'} // '';
    my $OldPriceMinisterConditionNote = $self->get('PriceMinisterConditionNote') // '';
    my $NewPriceMinisterConditionNote = $hFormValues->{'PriceMinisterConditionNote'} // '';

    if ($OldPriceMinisterPrice != $NewPriceMinisterPrice) {
        $hValuesOffer->{'PriceMinisterPrice'} = $self->_roundPrice($hFormValues->{'PriceMinisterPrice'}, $self->get('CurrencyID'));
    }

    if(ElementOf($OnlineStatus,($PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'},$PRICEMINISTER_ONLINESTATUS{'OFFER_FAILED'}))) {
        if ($OldPriceMinisterCondition ne $NewPriceMinisterCondition) {
            $hValuesOffer->{'PriceMinisterCondition'} = $NewPriceMinisterCondition; #ConditionNumberToCode($NewPriceMinisterCondition, $oC->get('LanguageID'));
        }
        if ($OldPriceMinisterConditionNote ne $NewPriceMinisterConditionNote) {
            $hValuesOffer->{'PriceMinisterConditionNote'} = $NewPriceMinisterConditionNote;
        }
    }

    $hValuesOffer->{'ErrorCode'} = undef;
    $hValuesOffer->{'ErrorMessage'} = undef;

    # Saving values
    $self->set($hValuesOffer) if (scalar keys %$hValuesOffer);

    if ( $OnlineStatus == $PRICEMINISTER_ONLINESTATUS{'OFFER_PROCESSING'} or
         $OnlineStatus == $PRICEMINISTER_ONLINESTATUS{'OFFER_ONLINE'} ) {
        return $self->startOffer();
    }
    else {
        return;
    }
}

#========================================================================================
# function     startOffer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->startOffer;
#----------------------------------------------------------------------------------------
# description  Uploads an PriceMinisterOffer
#----------------------------------------------------------------------------------------
# return       null
#========================================================================================
sub startOffer {
#LogDebug (caller(0),0);
    my $self = shift;
#    return unless (
#        $self->get('OnlineStatus') == $PRICEMINISTER_ONLINESTATUS{'OFFER_OFFLINE'} or
#        $self->get('OnlineStatus') == $PRICEMINISTER_ONLINESTATUS{'OFFER_PROCESSING'}
#    );

    if (not defined $self->get('PSIN')) {
        Error('MissingPSIN',{});
    }

#    if (not defined $self->get('UPCEAN')) {
#        Error('MissingUPCEAN',{});
#    }

    my $importid;
    eval {
        $importid = UploadPriceMinisterProduct($self, 0);
        if (defined $importid) {
            $self->set({'ImportID' => $importid});
            $self->checkNextState('start');
        }
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogDebug ('Error while exporting', $Error->code);
        Warning($Error);
        return;
    }
    return $importid;
}

#========================================================================================
# function     _roundPrice
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->_roundPrice($Price,$CurrencyID);
#----------------------------------------------------------------------------------------
# description  Correcly formats price before saving it to the database.
#----------------------------------------------------------------------------------------
# input        $Price | unformatted price | float
# input        $CurrencyID | CurrencyID   | currency
#----------------------------------------------------------------------------------------
# return       $RoundedPrice | rounded price | float
#========================================================================================
sub _roundPrice {
#LogDebug (caller(0),0);

  my $self = shift;
  my ($Price, $CurrencyID) = @_;

  TestFloat('Price',$Price);
  TestString('CurrencyID',$CurrencyID);

  # round price
  my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
  my $MinorUnit = GetInfoCurrency($CurrencyID)->{'MinorUnit'};
  return $Formatter->round($Price, $MinorUnit);
}

1;
