#========================================================================================
# package      DE_EPAGES::Product::API::Attributes::Shop
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Implements the Shop attributes Products, ProductTypes and ProductClass.
#========================================================================================
package DE_EPAGES::Product::API::Attributes::Shop;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::TestParameter qw( TestInteger );
use DE_EPAGES::Content::API::ShortUrl qw( GetShortUrlByClass );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Product::API::Constants qw ( PRODUCT_LARGE_IMAGE_SIZES );
use DE_EPAGES::Search::API::SearchDefinition qw( GetSearchDefinition );

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     getAttributes
# state        private
# syntax       $Value = $Object->get($AttributeName);
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        if( $Name eq 'ProductFolderID' ) {
            $Values{$Name} = $Object->findChild('Products');
        }
        elsif( $Name eq 'AppointmentFolderID' ) {
            $Values{$Name} = $Object->findChild('Appointments');
        }
        elsif( $Name eq 'Products' ) {
            if ($Object->existsChild('Products')) {
                $Values{$Name} = $Object->child('Products')->children;
            } else {
                $Values{$Name} = [];
            }
        }
        elsif( $Name eq 'VisibleAppointments' ) {
            if ($Object->existsChild('Appointments')) {
                $Values{$Name} = [grep { $_->get('IsVisible') } @{$Object->child('Appointments')->children}];
            } else {
                $Values{$Name} = [];
            }
        }
        elsif( $Name eq 'ProductTypesID' ) {
            $Values{$Name} = $Object->existsChild('ProductTypes') ? $Object->child('ProductTypes')->id : undef;
        }
        elsif( $Name eq 'ProductClassID' ) {
            $Values{$Name} = $Object->existsChild('ProductClass') ? $Object->child('ProductClass')->id : undef;
        }
        elsif( $Name eq 'AllProductTypes' ) {
            my $hInfo = $Object->get(['ProductClass', 'ProductTypes']);
            my @AllProductTypes;
            push @AllProductTypes, $hInfo->{'ProductClass'};
            push @AllProductTypes, @{ $hInfo->{'ProductTypes'}->children };
            $Values{$Name} = \@AllProductTypes;
        }
        elsif( $Name eq 'ProductExamples') {
            $Values{$Name} = _ProductExamples( $Object, $LanguageID );
        }
        elsif( $Name eq 'BusinessHoursID') {
            $Object->businessHours;
            $Values{$Name} = $Object->findChild('BusinessHours');
        }
        elsif( $Name eq 'AvailableProductImageLargeSizes') {
            my @Sizes;
            foreach my $SizeString (@{PRODUCT_LARGE_IMAGE_SIZES()}) {
                push(@Sizes, {
                    'Size' => $SizeString
                });
            }
            $Values{$Name} = \@Sizes;
        }
        elsif( $Name eq 'ProductImageLargeSize') {
            my $hValues = $Object->get(['ProductImageLargeWidth', 'ProductImageLargeHeight']);
            $Values{$Name} = $hValues->{'ProductImageLargeWidth'}.' x '.$hValues->{'ProductImageLargeHeight'};
        }
    }
    return \%Values;
}

#========================================================================================
# function     setAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Package->setAttributes( $Object, $hValues );
# example      # never call setAttributes directly!
#               $Object->set( { $Name => $Value } );
#----------------------------------------------------------------------------------------
# description  Sets attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::set>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hUpdate, $LanguageID) = @_;
    foreach my $Name ( keys %$hUpdate ) {
        if ($Name eq 'ProductImageLargeSize') {
            my $Value = $hUpdate->{$Name};
            Error(
                'INVALID_PRODUCT_IMAGE_SIZE',
                {
                    'Value' => $Value
                }
            ) unless $Value ~~ PRODUCT_LARGE_IMAGE_SIZES();
            my ($Width, $Height) = split(/ x /, $Value);
            $Object->set({
                'ProductImageLargeWidth'  => $Width,
                'ProductImageLargeHeight' => $Height,
            });
        }
    }
    return;
}

sub deleteObject {
    my $self = shift;
    my ($Object) = @_;
    $Object->child('Products')->delete if $Object->existsChild('Products');
}

sub _ProductExamples {
    my ($Shop, $LanguageID) = @_;
    TestInteger('LanguageID', $LanguageID);

    my $ProductClass = LoadClassByAlias('Product');
    my $Folder = $Shop->get('ProductFolder');
    my $SearchDefinition = GetSearchDefinition($ProductClass, 'MBO-AllStrings');
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID'  => $Folder->id,
        'SearchDefinition' => $SearchDefinition,
        'Page' => 1,
        'PageSize' => 2,
        'OrderBy' => 'Alias',
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
            'SearchLanguageID' => $LanguageID,
            'NameAttributeID'  => $ProductClass->attribute('Name')->id,
            'IsAppointment'    => 0,
            'IsNotSubProduct'  => 1,
            'IsVisible'        => 1,
            'CurrencyID'       => $Shop->get('CurrencyID')
        },
    });
    $Search->statistic;
    my $aProducts = $Search->items(1);

    my $ProtocolAndServer = $Shop->get('DomainName');
    return [
        map {
            {
                'Example' => $ProtocolAndServer.'/'. GetShortUrlByClass($Folder, $_->class, $_->get('NameOrAlias', $LanguageID))
            }
        }
        @$aProducts
    ];
}

1;
