#========================================================================================
# package      DE_EPAGES::Product::API::Resource::CustomAttributes
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for getting values of user-defined
#               product attributes.
#========================================================================================
package DE_EPAGES::Product::API::Resource::CustomAttributes;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use strict;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;

##no critic qw(ProhibitCascadingIfElse)

sub service {
    return LoadObjectByPath('/RestServices/ProductCustomAttributes');
}

sub class {
    return 'Product';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
        PATCH => [
            ['patch', 'application/vnd.epages.v1+json'],
            ['patch', 'application/json'],
        ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/products/([^/]+)/custom-attributes/?$|;
    return $1;
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        return [ $Object, $LocaleID ];
    }
    elsif( $ActionName eq 'patch' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        return [ $Object, $LocaleID ];
    }
    else {
        return [ $Object ];
    }
}

sub href {
    my $self = shift;
    my ($APIBaseURL, $ProductGUID) = @_;
    return "$APIBaseURL/api/products/$ProductGUID/custom-attributes";
}

sub get {
    shift;
    my ($Product, $LocaleID) = @_;

    my $Shop = $Product->getSite;
    my $Context = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);

    # validate query parameters
    $LocaleID = $Context->validLocale($LocaleID);
    my $Locale= DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $ValueFormatter = DE_EPAGES::Product::API::Resource::CustomAttributes::ValueFormatter->new(
        LocaleID => $LocaleID,
        LanguageID => $LanguageID,
        Shop => $Shop,
        ImageBaseUrl => $Context->imageBaseUrl . $Product->get('WebPath'),
    );

    return {
        items => [
            map { $ValueFormatter->attributeValue($_->{'Attribute'}, $_->{'Value'}) }
            grep { $_->{'Attribute'}->get('IsVisible') && $_->{'Attribute'}->type ne 'PreDefCustomerString' }
            @{ $Product->get('UserDefinedAttributeValues', $LanguageID) }
        ],
    };
}

sub patch {
    my $self = shift;
    my ($Product, $LocaleID, $ahRequestBody) = @_;

    _validateBody($ahRequestBody);

    my $Shop = $Product->getSite();
    my $Context = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
    $LocaleID = $Context->validLocale($LocaleID);
    my $Locale = DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $ProductService = $self->_createProductCustomAttributesService();

    $ProductService->updateProductCustomAttributePartially($Product, $LanguageID, $ahRequestBody);

    return $self->get($Product, $LocaleID);

}

sub _createProductCustomAttributesService {
    return DE_EPAGES::Product::API::ProductService->new();
}

sub _validateBody {
   my  ($ahRequestBody) = @_;

    foreach my $BodyItem (@{$ahRequestBody}) {

        Error (q('op' value can not be empty)) if ($BodyItem->{'op'} eq '');

        Error (q( operation 'replace' mot allowed. Use 'add' instead)) if ($BodyItem->{'op'} eq 'replace');

        Error (q('path' value can not be empty)) if ($BodyItem->{'path'} eq '');

        Error (q('value' must be of type hash)) unless ref $BodyItem->{'value'} eq 'HASH';

        Error (q(no value for 'key' given)) if $BodyItem->{'value'}->{'key'} eq '';

    }

    return;
}


# private package for formatting the values
package DE_EPAGES::Product::API::Resource::CustomAttributes::ValueFormatter; ## no critic qw(ProhibitMultiplePackages)

use strict;
use DE_EPAGES::Core::API::IDN qw (EncodeDomainInUrl);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf IsA);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Shop::API::Resource::Formatter;

sub new {
    my $class = shift;
    my %Options = @_;

    return bless \%Options, $class;
}

sub shop {
    return shift->{'Shop'};
}

sub localeID {
    return shift->{'LocaleID'};
}

sub languageID {
    return shift->{'LanguageID'};
}

sub imageBaseUrl {
    return shift->{'ImageBaseUrl'};
}

sub formatter {
    my $self = shift;

    return $self->{'Formatter'} //= DE_EPAGES::Shop::API::Resource::Formatter->new($self->shop, $self->localeID);
}

sub siteLocale {
    return shift->formatter->locale;
}

sub attributeValue {
    my $self = shift;
    my ($Attribute, $Value) = @_;

    return {
        key => $Attribute->alias,
        displayKey => $Attribute->get('NameOrAlias', $self->languageID),
        singleValue => $self->formatter->bool( not $Attribute->isArray ),
        type => $self->type($Attribute),
        values => $self->values($Attribute, $Value),
    }
}

sub values {
    my $self = shift;
    my ($Attribute, $Value) = @_;

    if( $Attribute->isArray ) {
        return [ map { $self->value($Attribute, $_) } @$Value ];
    }
    else { # single value
        return [ $self->value($Attribute, $Value) ];
    }
}

sub value {
    my $self = shift;
    my ($Attribute, $Value) = @_;

    return {
        value => $self->rawValue($Attribute, $Value),
        displayValue => $self->displayValue($Attribute, $Value),
    };
}

sub rawValue { # single value struct
    my $self = shift;
    my ($Attribute, $Value) = @_;

    return undef unless defined $Value;

    # non-localized value key for pre-defined attribute value
    return $Value->alias if IsA($Value, 'DE_EPAGES::Object::API::Object::Object');

    my $Type = $Attribute->type;
    if(ElementOf($Type, qw(File LocalizedFile))) {
        return EncodeDomainInUrl($self->imageBaseUrl.'/'.$Value);
    }
    elsif($Type eq 'Boolean' ) {
        return $Value ? 'true' : 'false';
    }
    elsif($Type eq 'DateTime' ) {
        my $TimeZone = LoadRootObject()->get('TimeZone');
        my $DateTimeFormat = DE_EPAGES::XML::API::DateTimeFormatXML->new;
        return $DateTimeFormat->format_datetime($Value, $TimeZone);
    }
    elsif($Type eq 'Date' ) {
        return $Value->ymd;
    }
    elsif($Type eq 'Time' ) {
        return $Value->hms;
    }
    else {
        return "$Value"; # convert numbers to strings
    }
}

sub displayValue { # single value struct
    my $self = shift;
    my ($Attribute, $Value) = @_;

    return '' unless defined $Value;

    return $Value->get('ValueOrAlias', $self->languageID) if IsA($Value, 'DE_EPAGES::Object::API::Object::PreDefAttribute');

    my $Type = $Attribute->type;
    if($Type eq 'Integer' ) {
        return $self->siteLocale->formatInteger($Value);
    }
    elsif(ElementOf($Type, qw(Float Money))) {
        return $self->siteLocale->formatFloat($Value);
    }
    elsif(ElementOf($Type, qw(File LocalizedFile))) {
        return $self->imageBaseUrl.'/'.$Value;
    }
    elsif($Type eq 'Boolean' ) {
        return $Value ? 'true' : 'false';
    }
    elsif($Type eq 'DateTime' ) {
        return $self->siteLocale->formatDateTime($Value, 'datetime');
    }
    elsif($Type eq 'Date' ) {
        return $self->siteLocale->formatDateTime($Value, 'date');
    }
    elsif($Type eq 'Time' ) {
        return $self->siteLocale->formatDateTime($Value, 'time');
    }
    else {
        die "value $Value is not a scalar" if ref $Value;
        return $Value;
    }
}

sub type {
    shift;
    my ($Attribute) = @_;

    my $Type = $Attribute->type;
    if(ElementOf($Type, qw(Integer Float Money))) {
        return 'number';
    }
    elsif(ElementOf($Type, qw(File LocalizedFile))) {
        return 'url';
    }
    elsif($Type eq 'Boolean') {
        return 'boolean';
    }
    elsif($Type eq 'DateTime') {
        return 'datetime';
    }
    elsif($Type eq 'Date') {
        return 'date';
    }
    elsif($Type eq 'Time') {
        return 'time';
    }
    else {
        return 'string';
    }
}

1;
