#========================================================================================
# package      DE_EPAGES::Product::API::Resource::Product
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for a single product.
#========================================================================================
package DE_EPAGES::Product::API::Resource::Product;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Core::API::Error qw (GetError Error ExistsError);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Constants qw (Text2TaxModel);
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Shop::API::Resource::Formatter;
use DE_EPAGES::Product::API::Resource::CustomAttributes;
use DE_EPAGES::Product::API::Resource::ProductSlideshow;
use DE_EPAGES::Product::API::Resource::ProductVariation;
use DE_EPAGES::Product::API::Resource::ProductLowestPrice;
use DE_EPAGES::Product::API::Resource::ProductCategories;
use DE_EPAGES::Product::API::ProductService;
use DE_EPAGES::Product::API::ProductPatch;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

use strict;



sub service {
    return LoadObjectByPath('/RestServices/Product');
}



sub class {
    return 'Product';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get_v2', 'application/vnd.epages.product+json; version=2.0'],
            ['get', 'application/json'],
            ['get_v2', 'application/x-yaml'],
        ],
        PATCH => [
            ['patch', 'application/vnd.epages.v1+json'],
            ['patch', 'application/json'],
        ],
        DELETE => [
            ['delete', undef],
        ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/products/([^/]+)/?$|;

    return $1;
}



sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $CurrencyID = $hQuery->{'currency'}->[0];
        my $SessionID = $Request->header('X-EPAGES-SESSIONID');
        my $SellingCountryID = $Request->header('X-EPAGES-SELLINGCOUNTRYID');
        return [ $Object, $LocaleID, $CurrencyID, $SessionID, $SellingCountryID ];
    }
    elsif( $ActionName eq 'get_v2' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0] or Error('BadRequest', {'MissingQueryParameter' => 'locale'});
        my $CurrencyID = $hQuery->{'currency'}->[0] or Error('BadRequest', {'MissingQueryParameter' => 'currency'});
        my $TaxModelName = $hQuery->{'tax-model'}->[0] or Error('BadRequest', {'MissingQueryParameter' => 'tax-model'});
        return [ $Object, $LocaleID, $CurrencyID, $TaxModelName ];
    }
    elsif( $ActionName eq 'patch' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $CurrencyID = $hQuery->{'currency'}->[0];
        return [ $Object, $LocaleID, $CurrencyID ];
    }
    else {
        return [ $Object ];
    }
}



sub href {
    my $self = shift;
    my ($APIBaseURL, $ProductGUID) = @_;
    return "$APIBaseURL/api/products/$ProductGUID";
}

sub hrefToProductCrossSelling {
    my $self = shift;
    my ($APIBaseURL, $ProductGUID) = @_;
    return "$APIBaseURL/api/products/$ProductGUID/crossselling";

}

sub get {
    my $self = shift;
    my ($Product, $LocaleID, $CurrencyID, $SessionID, $SellingCountryID) = @_;

    my $Shop = $Product->getSite();
    my $Context = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
    $CurrencyID = $Context->validCurrency($CurrencyID);
    $LocaleID = $Context->validLocale($LocaleID);

    my $ProductService = DE_EPAGES::Product::API::ProductService->new();

    my $hProductData = {};
    eval {
        $hProductData = $ProductService->getProductDetailsData($Shop, $Product, $LocaleID, $CurrencyID, $SessionID, $SellingCountryID);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        $Error->addVars({'ShopAlias' => $Shop->alias()});
        $Error->addVars({'ProductGUID' => $Product->guid()});
        Error($Error);
    }

    return $hProductData;
}

sub patch {
    my $self = shift;
    my ($Product, $LocaleID, $CurrencyID, $ahRequestBody) = @_;

    my $Shop = $Product->getSite();
    my $Context = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
    $CurrencyID = $Context->validCurrency($CurrencyID);
    $LocaleID = $Context->validLocale($LocaleID);
    my $Locale = DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    eval {
        $self->_RunBatchProcess('UpdateProducts', sub {
            my $Patch = DE_EPAGES::Product::API::ProductPatch->new($Product, $LanguageID, $CurrencyID);
            foreach my $hPatchOperation (@{$ahRequestBody}) {
                $Patch->patch($hPatchOperation);
            }
        });
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code() eq 'FORMAT_NOT_FLOAT' ) {
            Error('BadRequest', {
                'Reason' => '\'value\' was not a float',
                'ShopAlias' => $Shop->alias(),
            });
        } else {
            Error($Error);
        }
    }

    return $self->get($Product, $LocaleID, $CurrencyID);
}

sub delete {
    my $self = shift;
    my ($Product) = @_;

    $self->_RunBatchProcess('DeleteProducts', sub {
        my $ProductService = DE_EPAGES::Product::API::ProductService->new();
        $ProductService->deleteProduct($Product);
    });

    return;
}

sub get_v2 {
    my $self = shift;
    my ($Product, $LocaleID, $CurrencyID, $TaxModelName) = @_;

    my $TaxModel = Text2TaxModel($TaxModelName);
    my $Locale= DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $Formatter = DE_EPAGES::Shop::API::Resource::Formatter->new($Product->getSite, $LocaleID);

    my @Attributes = qw(
        Weight WeightUnit
        Text NameOrAlias
        ImageMedium ImageLarge
        GUID
        PriceQuantity OrderUnit
        RefUnit RefAmount RefContentAmount
        TaxClass
    );
    my $hProduct = $Product->get(\@Attributes, $LanguageID);

# remarks
# higher precision of net prices for back-office requests
# customer-specific prices for SF

    my $ListPrice = $Product->getListPrice($CurrencyID, $TaxModel);
    my $ManufacturerPrice = $Product->getManufacturerPrice($CurrencyID, $TaxModel);
    my $EcoParticipation = $Product->getEcoParticipation($CurrencyID, $TaxModel);
    my $Deposit = $Product->getDeposit($CurrencyID, $TaxModel);
    my $PriceWithDeposits = defined $ListPrice ? $ListPrice + ($EcoParticipation // 0) + ($Deposit // 0) : undef;
    my $ReferencePrice = $Product->getReferencePrice($CurrencyID, $ListPrice);

    return {
        deliveryWeight => $Formatter->quantity( $hProduct->{'Weight'}, $hProduct->{'WeightUnit'} ),
        description => $hProduct->{'Text'},
        images => [], # TODO [ { "name": String, "url": String } ... ]
        name => $hProduct->{'NameOrAlias'},
        priceInfo => {
            basePrice => {
                refPrice => $ReferencePrice,
                refQuantity => $Formatter->quantity( $hProduct->{'RefAmount'}, $hProduct->{'RefUnit'} ),
            },
            currency => $CurrencyID,
            depositPrice => $Deposit,
            ecoParticipationPrice => $EcoParticipation,
            manufacturerPrice => $ManufacturerPrice,
            price => $ListPrice,
            priceWithDeposits => $PriceWithDeposits,
            quantity => $Formatter->quantity( $hProduct->{'PriceQuantity'}, $hProduct->{'OrderUnit'} ),
        },
        productId => $hProduct->{'GUID'},
        taxClass => $hProduct->{'TaxClass'} ? $hProduct->{'TaxClass'}->alias : undef,
        taxModel => $TaxModelName,
    };
}

sub _RunBatchProcess {
    my $self = shift;
    my ($BatchTaskName, $cSub) = @_;
    RunBatchProcess($BatchTaskName, $cSub);
    return;
}

1;
