#========================================================================================
# package      DE_EPAGES::Product::API::Resource::ProductCollection
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource
# description  implements the REST resource for the products collection.
#========================================================================================
package DE_EPAGES::Product::API::Resource::ProductCollection;
use base DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Tax::API::Shop qw ( TaxInfoBySellingCountryID );
use DE_EPAGES::Product::API::ProductService;
use DE_EPAGES::Product::API::Resource::Product;
use DE_EPAGES::WebService::API::ResourcePager;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

use strict;



sub service {
    return LoadObjectByPath('/RestServices/ProductCollection');
}



sub class {
    return 'Shop';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
        POST => [
            ['post', 'application/vnd.epages.v1+json'],
            ['post', 'application/json'],
        ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/products/?$|;

    return 1;
}



sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $CurrencyID = $hQuery->{'currency'}->[0];
        my $aProductGUIDs = $hQuery->{'id'};
        my $SessionID = $Request->header('X-EPAGES-SESSIONID');
        my $SellingCountryID = $Request->header('X-EPAGES-SELLINGCOUNTRYID');
        return [ $Object, $LocaleID, $CurrencyID, $aProductGUIDs, $SessionID, $SellingCountryID ];
    }
    elsif( $ActionName eq 'post' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $CurrencyID = $hQuery->{'currency'}->[0];
        return [ $Object, $LocaleID, $CurrencyID ];
    }
    else {
        return [ $Object ];
    }
}



sub get {
    my $self = shift;
    my ($Shop, $LocaleID, $CurrencyID, $aProductGUIDs, $SessionID, $SellingCountryID) = @_;
    $aProductGUIDs //= [];

    if(scalar @$aProductGUIDs > 100) { ## no critic qw(ProhibitMagicNumbers)
        Error('BadRequest', {
            'Reason' => 'TooManyProductIds',
            'ShopAlias' => $Shop->alias(),
        });
    }

    my $Context = $self->_createShopRequestContext($Shop);
    $CurrencyID = $Context->validCurrency($CurrencyID);
    $LocaleID = $Context->validLocale($LocaleID);

    my $Pager = DE_EPAGES::WebService::API::ResourcePager->new({
        'Page' => 1,
        'PageSize' => 100,
    });

    my $ProductService = $self->_createProductService();

    my $ProductClass = LoadClassByAlias('Product');
    my @Products = map { _LoadProductByGUID($Shop, $_, $ProductClass) } @$aProductGUIDs;
    my $hShopData = $ProductService->getShopData($Shop,$LocaleID, $CurrencyID);
    my $hSellingCountryData = TaxInfoBySellingCountryID($Shop, $SellingCountryID );

    return $Pager->pagedItems(\@Products, sub {
        $ProductService->getProductData($_[0], $hShopData, $SessionID, $hSellingCountryData);
    });
}



sub post {
    my $self = shift;
    my ($Shop, $LocaleID, $CurrencyID, $hRequestBody) = @_;

    my $Context = $self->_createShopRequestContext($Shop);
    $CurrencyID = $Context->validCurrency($CurrencyID);
    $LocaleID = $Context->validLocale($LocaleID);

    my $ProductData;
    $self->_RunBatchProcess('CreateProducts', sub {
        my $ProductService = $self->_createProductService();
        $ProductData = $ProductService->createProduct($Shop, $LocaleID, $CurrencyID, $hRequestBody);
    });

    return $ProductData;
}



# returns an empty list if the product does not exists
sub _LoadProductByGUID {
    my ($Shop, $GUID, $ProductClass) = @_;

    my $Product = eval { $Shop->loadObjectByGuid( $GUID, $ProductClass ) };
    if (ExistsError()) {
        my ($Error) = GetError();
        return if $Error->code ~~ [qw(
            ClassMismatch
            NoResults_ObjectByGUID
            SiteMismatch
        )];
        Error($Error);
    }

    return $Product;
}



sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}



sub _createProductService {
    return DE_EPAGES::Product::API::ProductService->new();
}



sub _RunBatchProcess {
    my $self = shift;
    my ($BatchTaskName, $cSub) = @_;
    RunBatchProcess($BatchTaskName, $cSub);
    return;
}

1;
