#========================================================================================
# package      DE_EPAGES::Product::API::Resource::ProductTypeCollection
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for producttypes.
#========================================================================================
package DE_EPAGES::Product::API::Resource::ProductTypeCollection;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Core::API::Url qw ( ParseQueryString );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Shop::API::Resource::Formatter;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::WebService::API::ResourcePager;
use DE_EPAGES::Product::API::ProductTypeService;

use strict;

sub service {
    return LoadObjectByPath('/RestServices/ProductTypeCollection');
}

sub class {
    return 'Shop';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
        POST => [
        ['post', 'application/vnd.epages.v1+json'],
        ['post', 'application/json'],
    ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/producttypes?$|;
    return 1;
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    if( $ActionName eq 'get' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        my $ResultsPerPage = $hQuery->{'resultsPerPage'}->[0];
        my $Page = $hQuery->{'page'}->[0];
        return [ $Object, $LocaleID, $ResultsPerPage, $Page ];
    }
    elsif( $ActionName eq 'post' ) {
        my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
        my $LocaleID = $hQuery->{'locale'}->[0];
        return [ $Object, $LocaleID ];
    }
    else {
        return [$Object];
    }
}

sub get {
    my $self = shift;
    my ($Shop, $LocaleID, $ResultsPerPage, $Page) = @_;

    my $hPageOptions = {
        'Page' => $Page,
        'PageSize' => $ResultsPerPage,
    };
    my $Pager = $self->_createResourcePager($hPageOptions);

    my $ProductTypeService = $self->_createProductTypeService;
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $aProductTypes = $ProductTypeService->getProductTypes($Shop, $LocaleID);
    #Add Standard producttype if defined
    push @{$aProductTypes}, $Shop->get('ProductClass') if defined($Shop->get('ProductClass'));

    return $Pager->pagedItems($aProductTypes, sub {
        my $ProductType = $_;
        $ProductTypeService->getProductTypeData($ProductType, $LocaleID, $Shop);
    });
}

sub post {
    my $self = shift;
    my ($Shop, $LocaleID, $hRequestBody) = @_;


    #!!! we can not handle an array by epagesj at the moment!!!
    my $ahRequestBody = [$hRequestBody];
    die 'Invalid Body parameter' if $@ || ref $ahRequestBody ne 'ARRAY';

    #check if the POST body is structured correctly
    _analyzeBodyStructure($ahRequestBody);

    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $ProductTypeService = $self->_createProductTypeService();
    my @CreatedProductTypes =  $ProductTypeService->createProductTypes($Shop, $LocaleID, $ahRequestBody);

    my @NewProductTypes;
    foreach my $ProductType (@CreatedProductTypes) {
        push @NewProductTypes, $ProductTypeService->getProductTypeData($ProductType, $LocaleID, $Shop);
    }

    # return the created product type
    return @NewProductTypes[0];

}


sub _createProductTypeService {
    return DE_EPAGES::Product::API::ProductTypeService->new();
}

sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}

sub _createResourcePager {
    my $self = shift;
    my ($hPageOptions) = @_;
    return DE_EPAGES::WebService::API::ResourcePager->new($hPageOptions);
}

sub _analyzeBodyStructure{
    my ($ahParams) = @_;

    foreach my $Item (@$ahParams) {
        my $ProductTypeAlias = $Item->{'alias'};
        Error('BadRequest', {
            'Reason' => 'alias is a required parameter'}) unless $ProductTypeAlias =~ /\S/;
        TestString('alias', $ProductTypeAlias);

        # check structure of values for key 'attributes'
        foreach my $AttributesItem (@{$Item->{'attributes'}}) {

            Error('BadRequest', {
                'Reason' => 'Invalid attributes value, must be of type array'}) if ref $AttributesItem->{'values'} ne 'ARRAY';

            #check if name is set for Attribute
            Error('BadRequest', {
                'Reason' => 'name is a required for attribute'}) unless $AttributesItem->{'name'} =~ /\S/;

            Error('BadRequest', {
                'Reason' =>  'Attribute \'values\' invalid, must be of type array'}) if ref $AttributesItem->{'values'} ne 'ARRAY';

            #check if values exists for PreDefinedAttribute
            if ($AttributesItem->{'isPreDefAttribute'}) {
                Error('BadRequest', {
                    'Reason' => 'No values given for attribute'
                }) unless scalar(@{$AttributesItem->{values}});
            };
        }
    }

    return;
}

1;
