#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::IndexStatus
# base         Exporter
# description  Index Status API
#========================================================================================
package DE_EPAGES::RemoteSearch::API::IndexStatus;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );

use base Exporter;

use strict;

our @EXPORT_OK = qw(
  GetNextState
  IndexStatusString
  IndexStatusID
  GetActiveStates
  INDEXSTATUS_UNITIALIZED
  INDEXSTATUS_PENDING
  INDEXSTATUS_COMPLETE
  INDEXSTATUS_ERROR
  INDEXSTATUS_DIRTY
  INDEXSTATUS_OFF
);

# global        INDEXSTATUS_UNITIALIZED | status: Uninitialized | string
use constant INDEXSTATUS_UNITIALIZED => 'Uninitialized';

# global        INDEXSTATUS_PENDING | status: Pending | string
use constant INDEXSTATUS_PENDING     => 'Pending';

# global        INDEXSTATUS_COMPLETE | status: Complete | string
use constant INDEXSTATUS_COMPLETE    => 'Complete';

# global        INDEXSTATUS_ERROR | status: Error | string
use constant INDEXSTATUS_ERROR      => 'Error';

# global        INDEXSTATUS_DIRTY | status: Dirty | string
use constant INDEXSTATUS_DIRTY       => 'Dirty';

# global        INDEXSTATUS_OFF | status: Off | string
use constant INDEXSTATUS_OFF         => 'Off';

# private | index status map. mapping status id to strings | hash
my %INDEX_INDEXSTATUS_MAP = (
    0 => INDEXSTATUS_UNITIALIZED,
    1 => INDEXSTATUS_PENDING,
    2 => INDEXSTATUS_COMPLETE,
    3 => INDEXSTATUS_ERROR,
    4 => INDEXSTATUS_DIRTY,
    5 => INDEXSTATUS_OFF,
);

#========================================================================================
# function     IndexStatusString
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IndexStatusStrig = IndexStatusString( $StatusID );
#----------------------------------------------------------------------------------------
# description  Return Status string from id
#----------------------------------------------------------------------------------------
# input        $StatusID | status id | integer
# return       $String | status string | string
#========================================================================================
sub IndexStatusString {
    my ($ID) = @_;
    Error( 'INDEXSTATUS_ID_MISSING', { StatusID => $ID } ) unless ( exists $INDEX_INDEXSTATUS_MAP{$ID} );
    return $INDEX_INDEXSTATUS_MAP{$ID};
}

#========================================================================================
# function     GetActiveStates
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aStatusStrings = GetActiveStates();
#----------------------------------------------------------------------------------------
# description  Returns index status values that match to an active (i.e. usable in storefront)
#               index
#----------------------------------------------------------------------------------------
# return       $aStatusStrings | status string values | ref.array.string
#========================================================================================
sub GetActiveStates {
    return [ INDEXSTATUS_COMPLETE, INDEXSTATUS_DIRTY, INDEXSTATUS_PENDING ];
}

#========================================================================================
# function     IndexStatusID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $StatusID = IndexStatusID( $String );
#----------------------------------------------------------------------------------------
# description  Return StatusID from string
#----------------------------------------------------------------------------------------
# input        $String | status string | string
# return       $StatusID | status id | integer
#========================================================================================
sub IndexStatusID {
    my ($String) = @_;
    foreach my $ID ( keys %INDEX_INDEXSTATUS_MAP ) {
        return $ID if ( $INDEX_INDEXSTATUS_MAP{$ID} eq $String );
    }
    Error( 'INDEXSTATUS_STRING_MISSING', { StatusString => $String } );
}

#========================================================================================
# function     GetNextState
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetNextState( $currentState, $event );
#----------------------------------------------------------------------------------------
# description  state machine implementation. changes the index status along the defined
#               event path.
#----------------------------------------------------------------------------------------
# input        $currentState | status string. see list of index status | string
# input        $event | Event based on TaskStatus. See com.epages.task.api.TaskStatus from epagesj | string
# return       $newstate | new state | string
#========================================================================================
sub GetNextState {
    my ( $currentState, $event ) = @_;
    if ($event eq 'Completed') {
        if ($currentState ne INDEXSTATUS_OFF ) {
            return INDEXSTATUS_COMPLETE;
        }
    }
    elsif (ElementOf($event, qw(Init Pending Delayed Recovering Unscheduled))) {
        return INDEXSTATUS_PENDING;
    }
    elsif ($event eq 'Drop') {
        return INDEXSTATUS_OFF;
    }
    elsif (ElementOf($event, qw(Cancelled Error Rejected))) {
        return INDEXSTATUS_ERROR;
    }

    # default: do not change state.
    return $currentState;
}
1;
