#========================================================================================
# package      DE_EPAGES::SagePay::API::Payment
# state        public
#----------------------------------------------------------------------------------------
# description  Payment functions for SagePay.
#========================================================================================
package DE_EPAGES::SagePay::API::Payment;
use base Exporter;
use strict;

our @EXPORT_OK = qw (
    IsOnePaymentMethodInLiveMode
    GetSagePayPaymentMethods
    GetRegisterURLByCountryCode2
    GetAdminURLByCountryCode2
    RegisterTransaction
    ProcessSagePayCallback
);
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByGUID
    LoadObjectByGUID
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguagesByCode2 );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Presentation::API::PageType qw ( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage );
use DE_EPAGES::SagePay::API::Constants qw (
    PAYMENT_METHOD_LOW_PROFILE
    SIMULATOR_VENDOR
    %PARAMETERS_VALIDATION_LENGTH_LIMITS
);
use DE_EPAGES::SagePay::API::Object::RemoteSagePay;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use Digest::MD5 qw ( md5_hex );
use HTML::Entities qw ( encode_entities );

use constant REGPARAMS_DESCRIPTION_MAX_LENGTH => 99;
use constant REGPARAMS_DESCRIPTION_SUBSTR => 96;
use constant RESPONSE_CODE => 200;


#========================================================================================
# function     GetLanguageIDByCountryCode2
# state        private
#----------------------------------------------------------------------------------------
# syntax       GetLanguageIDByCountryCode2( $CountryCode2 );
#----------------------------------------------------------------------------------------
# description  Maps a country code to a language id, default is english
#----------------------------------------------------------------------------------------
# input        $CountryCode2 | country code type 2 | String
# return       $LanguageID | language id | integer
#========================================================================================
sub _GetLanguageIDByCountryCode2{
    my ( $CountryCode2 ) = @_;
    my $LanguageID;
   if ( $CountryCode2 eq 'DE' ) {
       $LanguageID =   @{GetInfoLanguagesByCode2('de')}[0]->{'LanguageID'};
   } else {
       $LanguageID =   @{GetInfoLanguagesByCode2('en')}[0]->{'LanguageID'};
   }
    return $LanguageID;
}

#========================================================================================
# function     GetRegisterURLByCountryCode2
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetRegisterURLByCountryCode2( $CountryCode2 );
#----------------------------------------------------------------------------------------
# description  gives the the register url depending to the country code
#----------------------------------------------------------------------------------------
# input        $CountryCode2 | country code type 2 | String
# return       $RegisterURL | Register URL | String
#========================================================================================
sub GetRegisterURLByCountryCode2 {
    my ( $CountryCode2 ) = @_;
    my $LanguageID = _GetLanguageIDByCountryCode2( $CountryCode2 );
    return LoadRootObject()->get('SagePay_RegisterURL',$LanguageID);
}

#========================================================================================
# function     GetAdminURLByCountryCode2
# state        public_private
#----------------------------------------------------------------------------------------
# syntax       GetAdminURLByCountryCode2( $CountryCode2,$aPaymentMethods );
#----------------------------------------------------------------------------------------
# description  returns the admin URL depending on country code and Operational mode (test/live)
#----------------------------------------------------------------------------------------
# input        $CountryCode2 | country code type 2 | String
# input        $aPaymentMethods | payment methods | refarray
# return       $AdminURL | admin URL | String
#========================================================================================
sub GetAdminURLByCountryCode2 {
    my ($CountryCode2,$aPaymentMethods) = @_;
    my $LanguageID = _GetLanguageIDByCountryCode2( $CountryCode2 );
    my $AdminURL;
    my $System = LoadRootObject();
    if( IsOnePaymentMethodInLiveMode($aPaymentMethods) ){
        $AdminURL = $System->get('SagePay_AdminURL_Live',$LanguageID);
    }else{
        $AdminURL = $System->get('SagePay_AdminURL_Test',$LanguageID);
    }
    return $AdminURL;
}

#========================================================================================
# function     IsOnePaymentMethodInLiveMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsOnePaymentMethodInLiveMode( $Servlet );
#----------------------------------------------------------------------------------------
# description  returns a 1 (true) if one of all sagepay payment methods which belongs to this shop
#               is actualy in live mode
#----------------------------------------------------------------------------------------
# input        $aSagePayPaymentMethods | sagepay payment methods | refArray
# return       $isInLiveMode | is one payment method in live mode | boolean
#========================================================================================
sub IsOnePaymentMethodInLiveMode {
    my ($aSagePayPaymentMethods) = @_;
    my $isInLiveMode = 0;
    foreach my $PaymentMethod (@{$aSagePayPaymentMethods}){
        if ( not $PaymentMethod->get('IsInTestMode')){
            $isInLiveMode = 1;
            last;
        }
    }
    return $isInLiveMode;
}

#========================================================================================
# function     GetSagePayPaymentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetSagePayPaymentMethods( $Servlet );
#----------------------------------------------------------------------------------------
# description  give a array reference with all Sagepay payment methods wich belog to this
#               current shop
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet | object
# return       \@SagePayPaymentMethods | SagePay payment methods | refarray
#========================================================================================
sub GetSagePayPaymentMethods{
    my ($Shop) = @_;
    my @SagePayPaymentMethods;
    my $aPaymentMethods = $Shop->folder('PaymentMethods')->children;
    foreach my $PaymentMethod (@{$aPaymentMethods}){
        if( $PaymentMethod->class()->alias() eq 'PaymentMethodSagePay'){
            push(@SagePayPaymentMethods,$PaymentMethod);
        }
    }
    return \@SagePayPaymentMethods;
}

#========================================================================================
# function     RegisterTransaction
#----------------------------------------------------------------------------------------
# syntax       RegisterTransaction( $PaymentLineItem );
#----------------------------------------------------------------------------------------
# description  send the registration request to SagePay to initiate a payment transaction
#----------------------------------------------------------------------------------------
# input        $PaymentLineItem | the payment lineitem | object
# return       $NextURL         | URL where the buyer must be redirected   | string
#========================================================================================
sub RegisterTransaction {
    my ($Servlet) = @_;

    my $Object      = $Servlet->object;     # may be a basket or a customerorder
    my $Shop        = $Object->getSite;
    my $PaymentLineItem      = $Object->container->get('Payment');

    my $hPaymentLineItemInfo =
        $PaymentLineItem->get([qw(
            PaymentMethod
            GUID
            Amount
            CurrencyID
            NextURL
            CardType
    )]);

    # If we know the CreditCard the payment is already done!
    return undef if $hPaymentLineItemInfo->{'CardType'};

    my $NextURL = $hPaymentLineItemInfo->{'NextURL'};

    unless ($NextURL) {
        my $hPaymentMethodInfo =
            $hPaymentLineItemInfo->{'PaymentMethod'}->get([qw(
                IsInTestMode
                UseSimulator
                SagePayVendor
                SagePayTransactionType
                SagePayAllowGiftAid
                SagePayAVSCV2Level
                SagePay3DSecureLevel
                SagePayPaymentMode
            )]);

        if ( not defined $hPaymentMethodInfo->{'SagePayPaymentMode'} ) {

            # The payment mode it's not defined for this PaymentMethod !!!
            # This means the PaymentMethod was created before SagePay API version 3.0 update ( EPG-24490 )
            # and update this cartridge.

            # We set the default method to initialize this new attribute
            $hPaymentLineItemInfo->{'PaymentMethod'}->set( { 'SagePayPaymentMode' => PAYMENT_METHOD_LOW_PROFILE } ) ;

            # and the correct value for the attribute retrieved for this operation ...
            $hPaymentMethodInfo->{'SagePayPaymentMode'} = PAYMENT_METHOD_LOW_PROFILE ;
        }

        my $LanguageID = $Servlet->languageID;
        my $ObjectHash = $Object->tleHash;
        my %Templates  = (
            'Description' => 'SagePayTransactionDescription',
            'Basket'      => 'SagePayBasketContents'
        );

        foreach my $key (keys %Templates) {
            $Templates{$key} = _processTemplate($Templates{$key}, $Object, $ObjectHash, $LanguageID);
        }

        my $hObjectInfo = $Object->get([qw (BillingAddress ShippingAddress)]);

        my @AddressFields = qw (
                                LastName
                                FirstName
                                MiddleName
                                Street
                                Street2
                                City
                                Zipcode
                                CountryID
                                Country
                                State
                                Phone
                            );

        my $hBillingAddressInfo = $hObjectInfo->{'BillingAddress'}->get(\@AddressFields);

        my $hShippingAddressInfo =  defined($hObjectInfo->{'ShippingAddress'}) ?
                                        $hObjectInfo->{'ShippingAddress'}->get(\@AddressFields) :
                                        $hBillingAddressInfo;

        my $hRegisterParameters  = {
            'TxType'             => $hPaymentMethodInfo->{'SagePayTransactionType'},
            'VendorTxCode'       => $hPaymentLineItemInfo->{'GUID'},
            'Amount'             => sprintf('%.2f', $hPaymentLineItemInfo->{'Amount'}),
            'Currency'           => $hPaymentLineItemInfo->{'CurrencyID'},
            'Description'        => _formatTemplateDescription($Templates{'Description'}),
            'NotificationURL'    => BuildShopUrl(
                                        $Shop,
                                        { 'ViewAction' => 'PaymentCallbackSagePay' },
                                        {
                                            'UseSSL' => 1,
                                            'UseObjectPath' => 1
                                        }
                                    ),
            'BillingSurname'     => $hBillingAddressInfo->{'LastName'},
            'BillingFirstnames'  => join(' ', grep { $_ } ($hBillingAddressInfo->{'FirstName'}, $hBillingAddressInfo->{'MiddleName'})),
            'BillingAddress1'    => $hBillingAddressInfo->{'Street'},
            'BillingCity'        => $hBillingAddressInfo->{'City'},
            'BillingPostCode'    => uc $hBillingAddressInfo->{'Zipcode'},
            'BillingCountry'     => $hBillingAddressInfo->{'CountryID'} ? $hBillingAddressInfo->{'Country'}->{'Code2'} : '',
            'BillingState'       => defined($hBillingAddressInfo->{'Country'}) && $hBillingAddressInfo->{'Country'}->{'Code2'} eq 'US'  ?  $hBillingAddressInfo->{'State'} : '',
            'DeliverySurname'    => $hShippingAddressInfo->{'LastName'},
            'DeliveryFirstnames' => join(' ', grep { $_ } ($hShippingAddressInfo->{'FirstName'}, $hShippingAddressInfo->{'MiddleName'})),
            'DeliveryAddress1'   => $hShippingAddressInfo->{'Street'},
            'DeliveryCity'       => $hShippingAddressInfo->{'City'},
            'DeliveryPostCode'   => uc $hShippingAddressInfo->{'Zipcode'},
            'DeliveryCountry'    => $hShippingAddressInfo->{'CountryID'} ? $hShippingAddressInfo->{'Country'}->{'Code2'} : '',
            'DeliveryState'      => defined($hShippingAddressInfo->{'Country'}) && $hShippingAddressInfo->{'Country'}->{'Code2'} eq 'US'  ?  $hShippingAddressInfo->{'State'} : '',
        #    'Basket'             => $Templates{'Basket'},
            'AllowGiftAid'       => $hPaymentMethodInfo->{'SagePayAllowGiftAid'} ? 1 : 0,
            'ApplyAVSCV2'        => $hPaymentMethodInfo->{'SagePayAVSCV2Level'},
            'Apply3DSecure'      => $hPaymentMethodInfo->{'SagePay3DSecureLevel'},
            'Profile'            => $hPaymentMethodInfo->{'SagePayPaymentMode'}
        };

        my $Form = $Servlet->form;
        my @MandatoryAddressFields = qw (
            BillingSurname
            BillingFirstnames
            BillingAddress1
            BillingCity
            BillingPostCode
            BillingCountry
            DeliverySurname
            DeliveryFirstnames
            DeliveryAddress1
            DeliveryCity
            DeliveryPostCode
            DeliveryCountry
        );

        foreach my $Field (@MandatoryAddressFields) {
            $Form->executeFormError({
                'Name'       => 'MissingAddressField',
                'Reason'     => $Field,
                'ViewAction' => 'ViewSagePayError',
            }) unless $hRegisterParameters->{$Field};
        };

        # --- State is mandatory for country US
        $Form->executeFormError({
            'Name'       => 'MissingAddressField',
            'Reason'     => 'BillingState',
            'ViewAction' => 'ViewSagePayError',
        }) if ($hRegisterParameters->{'BillingCountry'} eq 'US' && !$hRegisterParameters->{'BillingState'});

        $Form->executeFormError({
            'Name'       => 'MissingAddressField',
            'Reason'     => 'DeliveryState',
            'ViewAction' => 'ViewSagePayError',
        }) if ($hRegisterParameters->{'DeliveryCountry'} eq 'US' && !$hRegisterParameters->{'DeliveryState'});

        $hRegisterParameters->{'BillingAddress2'}  = $hBillingAddressInfo->{'Street2'}  if ($hBillingAddressInfo->{'Street2'});
        $hRegisterParameters->{'BillingPhone'}     = $hBillingAddressInfo->{'Phone'}    if ($hBillingAddressInfo->{'Phone'});
        $hRegisterParameters->{'DeliveryAddress2'} = $hShippingAddressInfo->{'Street2'} if ($hShippingAddressInfo->{'Street2'});
        $hRegisterParameters->{'DeliveryPhone'}    = $hShippingAddressInfo->{'Phone'}   if ($hShippingAddressInfo->{'Phone'});

        _validateRegistrationParameters($hRegisterParameters);

        my $LogInfo =   sprintf(
                            'Shop-ID: %d, %s-ID: %d, LineItemSagePay-ID: %d',
                            $Shop->id,
                            $Object->className,
                            $Object->id,
                            $PaymentLineItem->id
                        );

        # --- call register transaction service
        my $RemoteSagePay = DE_EPAGES::SagePay::API::Object::RemoteSagePay->new('LogInfo' => $LogInfo);
        $RemoteSagePay->setVendor( $hPaymentMethodInfo->{'SagePayVendor'} );
        $RemoteSagePay->setTestMode( $hPaymentMethodInfo->{'UseSimulator'} ? 2 : $hPaymentMethodInfo->{'IsInTestMode'} ? 1 : 0 );

        $RemoteSagePay->register( $hRegisterParameters );

        my $Response = $RemoteSagePay->getResponse;

        $Form->executeFormError({
            'Name'       => 'NoResult',
            'Reason'     => 'NoResult',
            'ViewAction' => 'ViewSagePayError',
        }) unless $Response;

        if ($Response->{'Status'} eq 'OK' ||
            $Response->{'Status'} eq 'OK REPEATED' ||
            $Response->{'Status'} eq 'REGISTERED')  {
            my $hValues = {
                'TransID'           => $Response->{'VPSTxId'},
                'TransTime'         => GetCurrentDateTime(),
                'TransStatus'       => $Response->{'Status'},
                'TransStatusDetail' => $Response->{'StatusDetail'},
                'SecurityKey'       => $Response->{'SecurityKey'},
                'NextURL'           => $Response->{'NextURL'}
            };
            $hValues->{'IsTestTransaction'} = 1 if $hPaymentMethodInfo->{'UseSimulator'};
            $PaymentLineItem->set($hValues);
            $NextURL = $Response->{'NextURL'};
        }
        else {
            LogPayment('SagePay', $LogInfo . ' Bad Response status:', {
                'Status'       => $Response->{'Status'},
                'StatusDetail' => $Response->{'StatusDetail'}
            });
            if ($hPaymentMethodInfo->{'UseSimulator'}) {
                # --- direct output of error message in simulator mode
                $Form->executeFormError({ 'Name'       => 'SimulatorStatus',
                                          'Reason'     => $Response->{'StatusDetail'},
                                          'ViewAction' => 'ViewSagePayError',
                }) if $hPaymentMethodInfo->{'UseSimulator'};
            }
            else {
                $Form->executeFormError({ 'Name'       => $Response->{'Status'},
                                          'Reason'     => $Response->{'StatusDetail'},
                                          'ViewAction' => 'ViewSagePayError',
                });
            }
        }

        # We send to the view the method used to process the payment ( iFrame or SagePAy server )

        $Servlet->vars('SagePayPaymentMode', $hPaymentMethodInfo->{'SagePayPaymentMode'} );
    }
    return $NextURL;
}

#========================================================================================
# function     ProcessSagePayCallback
#----------------------------------------------------------------------------------------
# syntax       ProcessSagePayCallback( $Servlet );
#----------------------------------------------------------------------------------------
# description  check the SagePay callback request parameters and send the response,
#               update PaymentLineItem
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# return       void
#========================================================================================
sub ProcessSagePayCallback {

    my ($Servlet) = @_;
    my $FormValues = $Servlet->params();
    my $Shop = $Servlet->object;

    #--- used for debugging
    my @LogInfo = ('SagePay callback for ', $Shop->pathString, ' (ObjectID ', $Shop->id, '), ');

    LogDebug( join('', @LogInfo) .', FormValues:', $FormValues );

    my %Request = (
        'VPSTxId'        => $FormValues->{'VPSTxId'},
        'VendorTxCode'   => $FormValues->{'VendorTxCode'},
        'Status'         => $FormValues->{'Status'},
        'TxAuthNo'       => $FormValues->{'TxAuthNo'},
        'AVSCV2'         => $FormValues->{'AVSCV2'},
        'AddressResult'  => $FormValues->{'AddressResult'},
        'PostCodeResult' => $FormValues->{'PostCodeResult'},
        'CV2Result'      => $FormValues->{'CV2Result'},
        'GiftAid'        => $FormValues->{'GiftAid'},
        '3DSecureStatus' => $FormValues->{'3DSecureStatus'},
        'CAVV'           => $FormValues->{'CAVV'},
        'AddressStatus'  => $FormValues->{'AddressStatus'},
        'PayerStatus'    => $FormValues->{'PayerStatus'},
        'CardType'       => $FormValues->{'CardType'},
        'Last4Digits'    => $FormValues->{'Last4Digits'},
        'DeclineCode'    => $FormValues->{'DeclineCode'},
        'ExpiryDate'     => $FormValues->{'ExpiryDate'},
        'FraudResponse'  => $FormValues->{'FraudResponse'},
        'BankAuthCode'   => $FormValues->{'BankAuthCode'},
        'ACSTransID'     => $FormValues->{'ACSTransID'},
        'DSTransID'      => $FormValues->{'DSTransID'},
        'SchemeTraceID'  => $FormValues->{'SchemeTraceID'},
        'VPSProtocol'    => $FormValues->{'VPSProtocol'},
        'TxType'         => $FormValues->{'TxType'},
        'StatusDetail'   => $FormValues->{'StatusDetail'},
        'VPSSignature'   => $FormValues->{'VPSSignature'},
    );
    LogPayment('SagePay', join('', @LogInfo) . "Request:\n", \%Request);

    my $Result = 'ERROR';
    my $Object = $Shop;
    my ($Msg, $Err);

    # --- check if PaymentLineItem GUID is transmitted
    if (!defined($Request{'VendorTxCode'})) {
        $Msg = 'Missing parameter VendorTxCode';
        $Err = 'MissingVendorTxCode';
        LogPayment('SagePay', join('', @LogInfo) . $Msg, '');
    }
    # --- check if PaymentLineItem GUID exists (within the system)
    elsif (!ExistsObjectByGUID($Request{'VendorTxCode'})) {
        $Msg = 'Parameter VendorTxCode not found.';
        $Err = 'VendorTxCodeNotFound';
        LogPayment('SagePay', join('', @LogInfo) . $Msg, {'VendorTxCode' => $Request{'VendorTxCode'}});
    }
    else {
        my $PaymentLineItem = LoadObjectByGUID( $Request{'VendorTxCode'} );

        $LogInfo[1] = $PaymentLineItem->pathString;
        $LogInfo[3] = $PaymentLineItem->id;

        $Object = $PaymentLineItem->container->parent;
        # --- check if the Site of the PaymentLineItem belongs to the shop
        if ( $PaymentLineItem->getSite->id != $Shop->id ) {
            $Msg = 'Transaction does\'t belong to requested shop.';
            $Err = 'ShopIDMisMatch';
            LogPayment('SagePay', join('', @LogInfo) . $Msg, {
                'PaymentLineItem-SiteID' => $PaymentLineItem->getSite->id,
                'ShopID'                 => $Shop->id
            });
        }
        # --- check the Signature
        elsif ( (my $CheckSum = _createSignature(\%Request, $PaymentLineItem)) ne $Request{'VPSSignature'} ) {
            $Msg = 'Signature check failed.';
            $Err = 'CheckSumMisMatch';
            LogPayment('SagePay', join('', @LogInfo) . $Msg, {
                'VPSSignature' => $Request{'VPSSignature'},
                'CheckSum'     => $CheckSum
            });
        }
        else {
            $Result = 'OK';
            if ($Request{'Status'} eq 'OK' ||
                $Request{'Status'} eq 'AUTHENTICATED' ||
                $Request{'Status'} eq 'REGISTERED') {
                my %Values = (
                    'TransStatus'       => $Request{'Status'},
                    'TransStatusDetail' => $Request{'StatusDetail'},
                    'AvsCode'           => $Request{'AVSCV2'},
                    'AddressResult'     => $Request{'AddressResult'},
                    'PostCodeResult'    => $Request{'PostCodeResult'},
                    'CV2Result'         => $Request{'CV2Result'},
                    'GiftAid'           => $Request{'GiftAid'},
                    'SecureStatus'      => $Request{'3DSecureStatus'},
                    'CAVV'              => $Request{'CAVV'},,
                    'CardType'          => $Request{'CardType'},
                    'Last4Digits'       => $Request{'Last4Digits'},
                );
                $PaymentLineItem->set(\%Values);

                # --- create order of necessary
                if ($Object->instanceOf('Basket')) {
                    $Object = DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object);
                    LogPayment('SagePay', join('', @LogInfo) . ", Order has been created.\n", {'OrderID' => $Object->id});
                }
                else {
                    LogPayment('SagePay', join('', @LogInfo) . ", Order already exists.\n", {'OrderID' => $Object->id});
                }
                if ($Request{'Status'} eq 'OK')   { $Object->set( {'PaidOn'   => GetCurrentDateTime()} ) }
                else                              { $Object->set( {'PendingOn'=> GetCurrentDateTime()} ) }
            }
            else {
                $PaymentLineItem->set({'NextURL' => ''});

                if ($Object->instanceOf('CustomerOrder')) {
                    my %Values = (
                        'TransStatus'       => $Request{'Status'},
                        'TransStatusDetail' => $Request{'StatusDetail'},
                    );
                    $PaymentLineItem->set(\%Values);
                    if ($Request{'Status'} eq 'NOTAUTHED' ||
                        $Request{'Status'} eq 'ABORT'     ||
                        $Request{'Status'} eq 'REJECTED'  ||
                        $Request{'Status'} eq 'ERROR')     {
                        $Object->set( {'CancelledOn'=>GetCurrentDateTime()} );
                    }
                }
            }
        }
    }
    # LogDebug ("ReponseParams", [$Object->pathString, $Result, $Msg, $Err] );
    _sendResponse( $Servlet, $Object, \%Request, $Result, $Msg, $Err);
}

#========================================================================================
sub _processTemplate {
    my ($PageTypeAlias, $Object, $ObjectHash, $LanguageID) = @_;

    my $PageType       = LoadPageTypeByAlias( $PageTypeAlias );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Object );
    my $Content        = ProcessPage($Object, $ObjectPageType, GetCodeByLanguageID($LanguageID), $ObjectHash);

    $Content =~ y/\cJ\cM//d;                # strip new lines
    $Content =~ s/<\!\s*--.*?-->//xgs;      # strip html comments

    return $Content;
}

#========================================================================================
sub _formatTemplateDescription {
    my ($TemplateDescription) = @_;

    if ( length($TemplateDescription) > REGPARAMS_DESCRIPTION_MAX_LENGTH ) {
        $TemplateDescription = substr( $TemplateDescription, 0, REGPARAMS_DESCRIPTION_SUBSTR ) . '...';
    }

    my $HtmlDescription = encode_entities($TemplateDescription);

    my $DescriptionSubtrLength = REGPARAMS_DESCRIPTION_SUBSTR;

    while ( length($HtmlDescription) > REGPARAMS_DESCRIPTION_MAX_LENGTH ) {
        $TemplateDescription = substr( $TemplateDescription, 0, $DescriptionSubtrLength ) . '...';
        $DescriptionSubtrLength -= 4;
        $HtmlDescription = encode_entities($TemplateDescription);
    }

    return $HtmlDescription;
}

#========================================================================================
sub _sendResponse {
    my ($Servlet, $Object, $hRequest, $CheckResult, $StatusDetail, $ErrorCode ) = @_ ;

    #--- used for debugging
    my $Shop = $Servlet->object;
    my $ShopPath = $Shop->pathString . ' (ObjectID ' . $Shop->id .')';

    # return url
    my %params = (
        'ViewAction' => 'ViewSagePayReturnIFrame',
        'Status'     => $CheckResult eq 'ERROR' ? 'ERROR' : $hRequest->{'Status'},
        'Detail'     => $CheckResult eq 'ERROR' ? $StatusDetail : $hRequest->{'StatusDetail'},
    );
    $params{'ErrorCode'} = $ErrorCode if ($ErrorCode);
    my $URL = BuildShopUrl($Object, \%params, {'UseSSL' => 1, 'UseObjectPath' => 1});

    my $Content  = "Status=$CheckResult\r\nRedirectURL=$URL";
    $Content .= "\r\nStatusDetail=$StatusDetail" if ($StatusDetail);

    my $Response = $Servlet->response;

    LogPayment('SagePay', "SagePay callback for shop $ShopPath, Response:\n", $Content);

    $Servlet->vars('ExitAfterEvent', 1);
    $Response->content($Content);
    $Response->code(RESPONSE_CODE);
}

#========================================================================================
sub _createSignature {
    my ($hRequest, $PaymentLineItem ) = @_ ;

    my $hPaymentLineItemInfo = $PaymentLineItem->get([qw( SecurityKey PaymentMethod )]);
    my $hPaymentMethodInfo = $hPaymentLineItemInfo->{'PaymentMethod'}->get([qw (SagePayVendor UseSimulator)]);

    my $String =  $hRequest->{'VPSTxId'};
    $String .= $hRequest->{'VendorTxCode'};
    $String .= $hRequest->{'Status'};
    $String .= $hRequest->{'TxAuthNo'}        if ($hRequest->{'TxAuthNo'});
    $String .= $hPaymentMethodInfo->{'UseSimulator'} ?
            SIMULATOR_VENDOR : lc($hPaymentMethodInfo->{'SagePayVendor'}); # VendorName
    $String .= $hRequest->{'AVSCV2'}          if ($hRequest->{'AVSCV2'});
    $String .= $hPaymentLineItemInfo->{'SecurityKey'};
    $String .= $hRequest->{'AddressResult'}   if ($hRequest->{'AddressResult'});
    $String .= $hRequest->{'PostCodeResult'}  if ($hRequest->{'PostCodeResult'});
    $String .= $hRequest->{'CV2Result'}       if ($hRequest->{'CV2Result'});
    $String .= $hRequest->{'GiftAid'};
    $String .= $hRequest->{'3DSecureStatus'};
    $String .= $hRequest->{'CAVV'}            if ($hRequest->{'CAVV'});
    $String .= $hRequest->{'AddressStatus'}   if ($hRequest->{'AddressStatus'});
    $String .= $hRequest->{'PayerStatus'}     if ($hRequest->{'PayerStatus'});
    $String .= $hRequest->{'CardType'};
    $String .= $hRequest->{'Last4Digits'};
    $String .= $hRequest->{'DeclineCode'}     if ($hRequest->{'DeclineCode'});
    $String .= $hRequest->{'ExpiryDate'}      if ($hRequest->{'ExpiryDate'});
    $String .= $hRequest->{'FraudResponse'}   if ($hRequest->{'FraudResponse'});
    $String .= $hRequest->{'BankAuthCode'}    if ($hRequest->{'BankAuthCode'});
    $String .= $hRequest->{'ACSTransID'}      if ($hRequest->{'ACSTransID'});
    $String .= $hRequest->{'DSTransID'}      if ($hRequest->{'DSTransID'});
    $String .= $hRequest->{'SchemeTraceID'}   if ($hRequest->{'SchemeTraceID'});

    return uc(md5_hex($String));
}

#========================================================================================
# function     _validateRegistrationParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       _validateRegistrationParameters( $hRegisterParameters );
#----------------------------------------------------------------------------------------
# description  Maps a country code to a language id, default is english
#----------------------------------------------------------------------------------------
# input        $hRegisterParameters | hash with payment registration parameters | ref.hash
#========================================================================================
sub _validateRegistrationParameters {
    my ($hRegisterParameters) = @_;

    foreach my $ParameterField (keys(%PARAMETERS_VALIDATION_LENGTH_LIMITS)) {
        if (isValid($hRegisterParameters->{$ParameterField}) &&
            length($hRegisterParameters->{$ParameterField}) > $PARAMETERS_VALIDATION_LENGTH_LIMITS{$ParameterField})
        {
            my $Original = $hRegisterParameters->{$ParameterField};
            $hRegisterParameters->{$ParameterField} = substr($Original, 0, $PARAMETERS_VALIDATION_LENGTH_LIMITS{$ParameterField});
            LogPayment('SagePay', $ParameterField . ' string truncated', {
                'original'  => $Original,
                'truncated' => $hRegisterParameters->{$ParameterField}
            });
        }
    }
}

1;
