#========================================================================================
# package      DE_EPAGES::Shop::API::Address
# state        public
# description  This module includes a function to update VATID
#========================================================================================
package DE_EPAGES::Shop::API::Address;
use base qw( Exporter );
use strict;

our @EXPORT_OK = qw (
    CheckStringVATID
    CheckVATID
    UpdateVATID
    IsTaxModelNet
    IsEUAddress
    CheckShopCountry
    GetTaxCountryIDForShop
    IsPrivateVATID
);

use Data::Dumper;
use DE_EPAGES::Core::API::VATIDValidation;
use DE_EPAGES::Address::API::Address qw(CompareZipCodes);
use DE_EPAGES::Core::API::Error qw( Error);
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Core::API::PerlTools qw(ElementOf );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString TestInteger );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Table::Country qw( GetInfoCountry ExistsCountryByCode2 );
use DE_EPAGES::Shop::API::Constants qw( $TAX_MODEL_GROSS_WITH_VATID $COUNTRIES_EXCLUDED_FROM_VIES_CHECK
                                        $COUNTRIES_WITH_SAME_TAX $EU_COUNTRY_AREA_EXCEPTIONS
                                        $NON_EU_COUNTRY_AREA_EXCEPTIONS );

#========================================================================================
# function     CheckStringVATID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $check = CheckStringVATID( $VATID );
# example      $check = CheckStringVATID( '118541153' );
#----------------------------------------------------------------------------------------
# description  returns 1 if vatid contains not only " " and "-"
#               returns 0 if vatid contain only " " and "-"
#----------------------------------------------------------------------------------------
# input        $VATID | VATID | string
# return       $check | valid | int
#========================================================================================
sub CheckStringVATID {
    my ($VATID) = @_;

    $VATID =~ s/ //g;
    $VATID =~ s/-//g;

    if ($VATID) {
        return 1;
    }

    return 0;
}

#========================================================================================
# function     CheckVATID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $check = CheckVATID( $VATID, $CountryID, $Shop );
#----------------------------------------------------------------------------------------
# description  returns 1 if country is not eu country
#               returns 1 if VATID is valid
#               returns -1 if CountryID and country VATID not same
#               returns 0 if VATID is not valid
#----------------------------------------------------------------------------------------
# input        $VATID | VATID | string
# input        $CountryID | country id | int
# input        $Shop | shop object | object
# return       $check | valid | int
#========================================================================================
sub CheckVATID {
    my ($VATID, $CountryID, $Shop) = @_;
    TestString('VATID', $VATID);

    $VATID =~ s/ //g;
    $VATID =~ s/-//g;

    unless ($CountryID) {
        my $Address = defined $Shop ? $Shop->get('Address') : undef;
        $CountryID = $Address->get('CountryID') if defined $Address;
    }
    return 1 unless $CountryID;
    return 1 unless GetInfoCountry($CountryID)->{'IsEU'};

    my $VATCountryCode = substr($VATID,0,2);
    my $VATNo = substr($VATID,2);
    return 1 if $VATCountryCode eq 'EU';

    my $CountryCode = GetInfoCountry($CountryID)->{'Code2'};
    if ($CountryCode eq 'GR') {
        # Greek VATIDs start with 'EL'
        $CountryCode = 'EL';
    }
    elsif ($CountryCode eq 'MC') {
        # VATIDs from Monaco start with 'FR' - AD-8725
        $CountryCode = 'FR';
    }
    return -1 if $VATCountryCode ne $CountryCode;

    my $hvatn = DE_EPAGES::Core::API::VATIDValidation->new();

    # Check if VATID is from country that doesn't have a complete VIES database.
    if (defined $COUNTRIES_EXCLUDED_FROM_VIES_CHECK->{$VATCountryCode}) {
        return 1 if $hvatn->local_check($VATNo, $VATCountryCode);
    }
    else {
        #check validity via soap call of the VIES database
        my $Check = $hvatn->check($VATNo, $VATCountryCode);

        my $ErrorCode = $hvatn->get_last_error_code();

        ##no critic qw(ProhibitMagicNumbers)
        if ($ErrorCode > 16) {
            #connection to VIES database failed
            if ($ErrorCode > 256) {
                #invalid response of the VIES database --> report error
                GetLog()->error('DE_EPAGES::Core::API::VATIDValidation: error connecting the VIES database: ' . $ErrorCode  . "\n" . $hvatn->get_last_error());
            }
            #no soap connection --> perform local check instead
            return 1 if $hvatn->local_check($VATNo, $VATCountryCode);
        }
        else {
            return 1 if $Check;
        }
        ##use critic qw(ProhibitMagicNumbers)
    }

    return 0;
}

#========================================================================================
# function     UpdateVATID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $VATID = UpdateVATID( $VATID, $CountryID, $Shop );
# example      $VATID = UpdateVATID( '118541153', 276, $Shop );
#----------------------------------------------------------------------------------------
# description  Adds the country code to the VATID if $CountryID is defined and the
#               country is a member of the European Union.
#----------------------------------------------------------------------------------------
# input        $VATID | VATID | string
# input        $CountryID | country id | int
# input        $Shop | shop object | object
# return       $VATID | VAT ID including country code, e.g. "DE118541153" | string
#========================================================================================
sub UpdateVATID {
    my ($VATID, $CountryID, $Shop) = @_;
    TestString('VATID', $VATID);

    $VATID = uc($VATID);
    $VATID =~ s/ //g;
    $VATID =~ s/-//g;

    unless ($CountryID) {
        my $Address = defined $Shop ? $Shop->get('Address') : undef;
        $CountryID = $Address->get('CountryID') if defined $Address;
    }
    return $VATID unless $CountryID;
    my $hCountry = GetInfoCountry($CountryID);
    return $VATID unless $hCountry->{'IsEU'};

    my $VATCountryCode = substr($VATID,0,2);
    return $VATID if $VATCountryCode eq 'EU';
    # in GR VATID starts with EL
    $VATCountryCode = $VATCountryCode eq 'GR' ? 0 : ($VATCountryCode eq 'EL' ? 'GR' : $VATCountryCode);
    return $VATID if ExistsCountryByCode2($VATCountryCode);
    my $CountryCode = $hCountry->{'Code2'};
    $CountryCode = 'EL' if $CountryCode eq 'GR';
    return $CountryCode . $VATID;
}

#========================================================================================
# function     IsTaxModelNet
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsNet = IsTaxModelNet( $Shop, $Address );
#----------------------------------------------------------------------------------------
# description  Checks the address on the basis of shop, countryid and vatid to be net.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $Address | address object | object
# return       $IsNet | true if address is non-eu-address
#                       or is eu-address and vatid is defined and valid | boolean
#========================================================================================
sub IsTaxModelNet {
    my ($Shop, $Address) = @_;
    TestObject('Shop', $Shop);
    TestObject('Address', $Address);

    my $AddressCountryID = $Address->get('CountryID');
    return 0 unless defined $AddressCountryID;
    my $AddressIsEU = IsEUAddress($Address);

    my $ShopAddress = $Shop->get('Address');
    my $ShopCountryID = GetTaxCountryIDForShop($Shop);
    my $ShopAddressIsEU = IsEUAddress($ShopAddress);

    # shop is non EU
    if( ! $ShopAddressIsEU ) {
        # switch to net if customer is from a different country
        # don't use the VATID, because we can't validate it

        # EPG-28940 Tax exception for Liechtenstein - it depends to Switzerland
        # test if we have exceptions
        # test if the key (ShopCountry) has a list with address countries for same tax handling
        my $aSameTaxCountries = $COUNTRIES_WITH_SAME_TAX->{$ShopCountryID} // [];

        if ( scalar(@$aSameTaxCountries) and ElementOf( $AddressCountryID, @$aSameTaxCountries) ) {
            return 0;
        }
        return $AddressCountryID == $ShopCountryID ? 0 : 1;
    }

    # customer non EU, we have net
    return 1 unless $AddressIsEU;

    # customer is EU, test the VATID
    my $VATID = $Address->get('VATID');

    # use gross
    return 0 unless defined $VATID;

    $VATID =~ s/ //g;
    $VATID =~ s/-//g;

    # check the vat id if we can use net
    if ($VATID && CheckVATID($VATID, $AddressCountryID, $Address->getSite()) == 1) {
        return 1 unless IsPrivateVATID($VATID);
    }

    # we use gross
    return 0;
}

#========================================================================================
# function     IsPrivateVATID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsPrivate = IsPrivateVATID( $VATID );
#----------------------------------------------------------------------------------------
# description  checks if the VATID is listed in the exception list $TAX_MODEL_GROSS_WITH_VATID
#               or if it is a usual (business) VATID
#----------------------------------------------------------------------------------------
# input        $VATID | VATID | string
# return       $IsPrivate | true if VATID is listed in the exception list
#                            $TAX_MODEL_GROSS_WITH_VATID | boolean
#========================================================================================
sub IsPrivateVATID {
    my ($VATID) = @_;
    TestString('VATID', $VATID);

    $VATID =~ s/ //g;
    $VATID =~ s/-//g;

    # use parts of the vat id for tests
    my $VATCountryCode = substr($VATID, 0, 2);
    my $VATNo          = substr($VATID, 2, 1);

    # search for exceptions for having tax model gross with a vat id => private VATID
    if (defined $TAX_MODEL_GROSS_WITH_VATID->{$VATCountryCode}) {
        return 1 if ElementOf(uc($VATNo), @{$TAX_MODEL_GROSS_WITH_VATID->{$VATCountryCode}});
    }

    # we have net (no entry in the exception list) => business VATID
    return 0;
}

#========================================================================================
# function     IsEUAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsEUAddress = IsEUAddress( $Address );
#----------------------------------------------------------------------------------------
# description  Checks the address on the basis of countryid and zip code exceptions to be a EU Address
#----------------------------------------------------------------------------------------
# input        $Address | address object | object
# return       $IsEUAddress | address is an EU address | int
#========================================================================================
sub IsEUAddress {
    my ($Address) = @_;
    TestObject('Address', $Address);

    my $AddressCountryID = $Address->get('CountryID');
    return 0 unless defined $AddressCountryID;
    my $hAddressCountryInfo = GetInfoCountry($AddressCountryID);

    my $ZipCode = $Address->get('Zipcode');

    # test if we have an non eu to eu exception
    if (not $hAddressCountryInfo->{'IsEU'}) {

        # we can only test with an zip code
        return 0 unless $ZipCode;

        # load exception list for zip codes stay in the EU
        # with a base country not in the EU
        my $ahNonEUToEUCountryExceptionList = $NON_EU_COUNTRY_AREA_EXCEPTIONS->{$AddressCountryID};

        # do we have entries in the exception list?
        return 0 unless defined $ahNonEUToEUCountryExceptionList;
        return 0 unless scalar @$ahNonEUToEUCountryExceptionList;

        # search for code and return 1 (EU) if code is in range
        foreach my $hZipCodeRange (@$ahNonEUToEUCountryExceptionList) {
            return 1 if (CompareZipCodes($hZipCodeRange->{'ZipcodeFrom'}, $ZipCode) != 1) and
                        (CompareZipCodes($hZipCodeRange->{'ZipcodeTo'}, $ZipCode) != -1);
        }

        # we have an no eu country
        return 0 ;
    }

    # test if we have areas in this EU country wich are handles as non EU areas regarding tax
    # e.g. Spain-> EU but Canarian Islands -> non EU
    # we can test this only with zip codes on the address
    # we have no zip informations-> so we have an EU country by CountryID
    return 1 unless $ZipCode;

    # load exception list
    my $ahCountryExceptionList = $EU_COUNTRY_AREA_EXCEPTIONS->{$AddressCountryID};

    # do we have entries in the exception list?
    return 1 unless defined $ahCountryExceptionList;
    return 1 unless scalar @$ahCountryExceptionList;

    # search for code and return 0 (non EU) if code is in range
    foreach my $hZipCodeRange (@$ahCountryExceptionList) {
        return 0 if (CompareZipCodes($hZipCodeRange->{'ZipcodeFrom'},$ZipCode) != 1) and
                    (CompareZipCodes($hZipCodeRange->{'ZipcodeTo'},$ZipCode) != -1);
    }

    # no exception was found. Address is in EU
    return 1;

}

#========================================================================================
# function     CheckShopCountry
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsCountrySupported = CheckShopCountry( $Shop, $CountryID );
# example      $IsCyprusSupported  = CheckShopCountry( $Shop, 196 );
#----------------------------------------------------------------------------------------
# description  Checks whether a $CountryID is served by the shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $CountryID | country id (ISO_3166-1_numeric) | int
# return       $CheckShopCountry | 1 (true) if country is supported by the shop or if country
#                                is undefined, 0 otherwise. | boolean
#========================================================================================
sub CheckShopCountry {
    my ($Shop, $CountryID) = @_;
    TestObject('Shop', $Shop);
    return 1 if not defined $CountryID;
    if ($CountryID eq '') {
        GetLog('BUGTRACKER::EPC320')->info("Given form value for CountryID is empty.\n".
            "User likely disabled javascript while in checkout address step. DebugData:\n".
             Dumper(_getRelatedShopData($Shop)));
        return 0;
    }
    TestInteger('CountryID', $CountryID);

    my $SupportedCountries = $Shop->get('ShopCountries');
    my @SupportedCountryIDs = map {$_->{'CountryID'}} @$SupportedCountries;

    return $CountryID ~~ @SupportedCountryIDs;
}

sub _getRelatedShopData {
    my ($Shop) = @_;

    my %DebugData = ('VisibleFields' => $Shop->get([qw (ShippingAddressSetVisibleFields
                                                 BillingAddressSetVisibleFields
                                                 RegistrationSetVisibleFields) ]),
                'DefaultShopCountry' => $Shop->defaultCountry());
    return %DebugData;
}

#========================================================================================
# function     GetTaxCountryIDForShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $CountryID = GetTaxCountryIDForShop( $Shop ));
#----------------------------------------------------------------------------------------
# description  Returns the CountryID for the given shop by the shop address.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# return       $CountryID | true country for shop | int
#========================================================================================
sub GetTaxCountryIDForShop {
    my ($Shop) = @_;
    TestObject('Shop',$Shop);

    # we use always the shop address to get the country. The Shop should have an address!
    my $ShopAddress = $Shop->get('Address');
    Warning('NO_SHOPADDRESS',{'ShopAlias' => $Shop->alias }) unless defined $ShopAddress;

    my $CountryID = defined $ShopAddress ?  $ShopAddress->get('CountryID') : undef ;
    return $CountryID if defined $CountryID;

    # fallback
    my $SystemLocaleID = LoadRootObject()->get('LocaleID');
    $CountryID = $Shop->siteLocale($SystemLocaleID)->get('Country')->{'CountryID'} if $SystemLocaleID;

    return $CountryID if defined $CountryID;

    Error('NO_COUNTRYID',{'ShopAlias' => $Shop->alias });

}

1;
