#========================================================================================
# package      DE_EPAGES::Shop::API::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Validates address data
#========================================================================================
package DE_EPAGES::Shop::API::AddressValidation;

use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Presentation::API::Form;
use DE_EPAGES::WebService::API::Util qw (TranslateAttributeName);

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Object, $LocaleID );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Object | some ePages business object | object
# input        $LocaleID | locale id string, e.g. 'de_DE' | string
# return       $self | self | DE_EPAGES::Shop::API::AddressValidation
#========================================================================================
sub new {
    my $class = shift;
    my ($Object, $LocaleID) = @_;
    my $self  = bless {
        'object' => $Object,
        'localeId' => $LocaleID,
    }, $class;

    if(not defined $LocaleID) {
        my $Shop = $Object->getSite();
        $self->{'localeId'} = $Shop->get('LocaleID');
    }

    return $self;
}



#========================================================================================
# function     validate
# state        public
#----------------------------------------------------------------------------------------
# syntax       validate( $hAddressData );
#----------------------------------------------------------------------------------------
# description  Validate the given address data and throws BadRequest error
#               if at least one of the attributes cannot be validated.
#----------------------------------------------------------------------------------------
# input        $hAddressData | address data | hash_ref
#========================================================================================
sub validate {
    my $self = shift;
    my ($hAddressData) = @_;

    my $Form = $self->_createForm();
    $Form->setlocale($self->{'localeId'});

    $self->_validate($Form, $hAddressData);

    my $aFormErrors = $Form->formErrors();
    my $aErrors = [
        map {
            {
                'attribute' => $self->_translateAttributeName($_->{'Name'}),
                'reason' => $_->{'Reason'},
            }
        } (@$aFormErrors)
    ];

    if(scalar @$aErrors > 0) {
        my $hFirstError = $aErrors->[-1];
        my $AttributeName = $hFirstError->{'attribute'};
        my $Reason = $hFirstError->{'reason'};
        Error('BadRequest', {
            'reason' => "Validation of $AttributeName failed: $Reason",
        });
    }
}



sub _validate {
    my $self = shift;
    my ($Form, $hAddressData) = @_;

    my $Object = $self->{'object'};
    my $AddressClass = $self->_LoadClassByAlias('Address');
    my $aAddressAttributes = $AddressClass->attributes();

    my ($Name, $Value, $ValueToValidate, $hFieldDefinition);
    foreach my $Attribute (@$aAddressAttributes) {

        $Name = $Attribute->get('Alias');
        if(exists $hAddressData->{$Name}) {
            $ValueToValidate = $hAddressData->{$Name};
            $hFieldDefinition = $self->_createFieldDefinition($Attribute);
            $Value = $Form->field( $Object, $ValueToValidate, $hFieldDefinition );
            $hAddressData->{$Name} = (defined $Value && $Value =~ /\S/ ) ? $Value : undef; # checks if defined and non-whitespace
        }

    }
}



# see Presentation::UI::Object::_mapScalarValue
sub _createFieldDefinition {
    my $self = shift;
    my ($Attribute) = @_;

    my $hAttributeValues = $Attribute->get(['Alias', 'Type', 'Length', 'IsHTML']);
    my $Type = $hAttributeValues->{'Type'};
    my $hFieldDefinition = {
        'Name' => $hAttributeValues->{'Alias'},
    };

    if($Type eq 'String') {
        $hFieldDefinition->{'Type'} = 'string';
        $hFieldDefinition->{'MaxValue'} = $hAttributeValues->{'Length'};
        $hFieldDefinition->{'IsHTML'} = $hAttributeValues->{'IsHTML'};
    }
    elsif($Type eq 'EMail') {
        $hFieldDefinition->{'Type'} = 'email_address';
    }
    else { # Integer, Float, DateTime, Money
        $hFieldDefinition->{'Type'} = 'reg_'. lc $Type;
    }

    return $hFieldDefinition;
}



sub _translateAttributeName {
    my $self = shift;
    my ($Ep7AttributeName) = @_;
    my $TranslatedName = TranslateAttributeName($Ep7AttributeName) // $Ep7AttributeName;
    return $TranslatedName;
}



sub _createForm {
    return DE_EPAGES::Presentation::API::Form->new();
}



sub _LoadClassByAlias {
    my $self = shift;
    my ($Alias) = @_;
    return LoadClassByAlias($Alias);
}

1;
