﻿#========================================================================================
# §package      DE_EPAGES::Shop::API::Object::Shop
# §state        public
# §base         DE_EPAGES::Object::API::Object::Site
# §description  object interface for Shop.
#========================================================================================
package DE_EPAGES::Shop::API::Object::Shop;
use base DE_EPAGES::Object::API::Object::Site;

use strict;
use Encode qw ( encode );
use HTTP::Request;
use LWP::UserAgent;
use JSON;
use utf8;
use feature 'switch';
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( GetLog LogError );
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::PerlTools qw ( Compare GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw ( TestInteger TestString TestObject CheckEMailAddrFormat );
use DE_EPAGES::Core::API::Url qw ( BuildParameterString );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Constants qw ( POSITION_START POSITION_STEP );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Object::API::SavedIncludesHandler;
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Object::API::Table::SiteLocaleMap qw (UpdateSiteLocaleMap);
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw( ImportAttributes );
use DE_EPAGES::Shop::API::Constants qw( TAX_MODEL_GROSS $NUMBER_FORMAT_INFIX_PATTERNS SHOP_JWT_LIFETIME );
use DE_EPAGES::Shop::API::Feature  qw( LoadFeatureByAlias );
use DE_EPAGES::Shop::API::JWT qw ( CreateShopJsonWebToken );
use DE_EPAGES::Shop::API::ShopChildOrderHandler qw (GetShopChildOrderList);
use DE_EPAGES::Shop::API::Table::ShopFeature qw(  DeleteShopFeature ExistsShopFeature GetInfoShopFeature
                                                  GetPKeyShopFeaturesByShopID InsertShopFeature UpdateShopFeature );
use DE_EPAGES::Shop::API::Table::ShopCurrency qw( DeleteShopCurrency ExistsShopCurrency GetInfoShopCurrency InsertShopCurrency
                                                  GetPKeyShopCurrenciesByShopID GetInfoShopCurrenciesByShopID UpdateShopCurrency);
use DE_EPAGES::Shop::API::Table::ShopCountry qw ( DeleteShopCountry ExistsShopCountry GetInfoShopCountry GetInfoShopCountriesByShopID
                                                  GetPKeyShopCountriesByShopID InsertShopCountry UpdateShopCountry );
use DE_EPAGES::Shop::API::Url qw ( BuildDefaultShopUrl );
## no critic qw( RequireCorrectImports )
use DE_EPAGES::Shop::API::XML::ShopDeleteHandler;
use DE_EPAGES::Shop::API::XML::ShopExportDriver;
use DE_EPAGES::Shop::API::XML::ShopImportHandler;
use DE_EPAGES::Shop::API::XML::FeatureDeleteHandler;
use DE_EPAGES::Shop::API::XML::FeatureExportDriver;
use DE_EPAGES::Shop::API::XML::FeatureImportHandler;
## use critic
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::WebInterface::API::ApplicationServer qw ( GetAppServer );


sub ErrorDefinition {
    return {
        'FeatureLimitExceeded' => 'Attempt use #NewValue of #MaxValue of feature #Feature in shop #Shop',
        'UNITY_CONFIG_ERROR' => 'No #Key set in the TBO --> Databases --> UnityWebServer',
        'UNITY_API_CALL_ERROR' => "Response from Unity: \'#Status, #Message\'.\nUnity Api #Method call failed to #ApiURL with content: \'#Content\'.\n",
        'UNITY_API_CALL_UNKNOWN_ACTION' => 'Action #Action is unknown'
    };
}

use constant STEP_SIZE => 10;

#========================================================================================
# §function     className
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $ClassName = $Shop->className;
#----------------------------------------------------------------------------------------
# §description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# §return       $ClassName | class name | String
#========================================================================================
sub className  { 'Shop' }

#========================================================================================
# §function     dbPackage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $DbPackage = $Shop->dbPackage;
#----------------------------------------------------------------------------------------
# §description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# §return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# §function     insert
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop = DE_EPAGES::Shop::API::Object::Shop->insert($hInfo);
# §example      my $Class = LoadClassByAlias('Shop');
#               my $Shop = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# §description  Insert this info as object, afterwards the hook 'OBJ_InsertShop' is triggered.
#----------------------------------------------------------------------------------------
# §input        $hInfo | attributes of new object | ref.hash
# §return       $Shop | new object | object
# §hook         OBJ_InsertShop | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopID        - shop id              - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShop', {'Object' => $self, 'ShopID' => $self->id });

    delete $hInfo->{$_} foreach qw ( Address TaxModel AddProductStyle ClosedByMerchant ClosedByProvider ShopTypeRef LastMerchantLogin IsTrialShop IsInternalTestShop InputTaxModel VATreleased AcceptPrivacyPolicy LoginRequired2Order LoginRequired2InstantlyOrder HasCreatedInitialShortURLs SuperMerchant ); ## no critic qw(ProhibitLongLines)

    $self->featureMaxValue('Languages', 1);
    $self->featureMaxValue('Currencies', 1);

    # get the system default locale
    my $System = LoadRootObject();
    my $SystemLocaleID = $System->get('LocaleID');
    my $SiteLocale = $System->siteLocale( $SystemLocaleID );

    # add shop language before locale
    my $LanguageID = $SiteLocale->get('Language')->{'LanguageID'};
    $self->addLanguage($LanguageID);

    # insert shop locale and set default
    $self->addLocale( $SystemLocaleID );
    $self->set({'LocaleID' => $SystemLocaleID });

    # set shop currency
    my $CurrencyID = $SiteLocale->get('Currency')->{'CurrencyID'};
    $self->addCurrency($CurrencyID);

    # set shop country
    my $CountryID = $SiteLocale->get('Country')->{'CountryID'};
    $self->addCountry($CountryID, STEP_SIZE, 1);

    # add an empty address
    my $Address = $self->insertChild( 'ShopAddress', {'Alias' => 'Address'} );
    $self->set({ 'Address' =>  $Address });

    return $self;
}

#========================================================================================
# §function     delete
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->delete;
#----------------------------------------------------------------------------------------
# §description  Deletes this object, therefore the hook 'OBJ_DeleteShop' is triggered.
#----------------------------------------------------------------------------------------
# §hook         OBJ_DeleteShop | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopID        - shop id              - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    # send DELETE request to Unity in case of Unity shops
    if ($self->get('IsUnityRegistered')) {
        eval { $self->sendUnityRequest('DELETE', '/shop'); };
        if (ExistsError()) {
            my $Error = GetError();
            # force delete shops that are not present in the Unity database anymore - UNITY-4756
            if ($Error->code eq 'UNITY_API_CALL_ERROR' && $Error->{'Vars'}->{'Status'} == 404) {
                Warning($Error)
            } else {
                Error($Error);
            }
        }
        # remove Unity flag from shop to avoid further calls to the Unity API (UNITY-5625)
        $self->set({'IsUnityRegistered' => 0});
    }

    TriggerHook('OBJ_DeleteShop', {'Object' => $self, 'ShopID' => $self->id });
    DeleteShopCountry($_) foreach @{GetPKeyShopCountriesByShopID($self->id)};
    DeleteShopCurrency($_) foreach @{GetPKeyShopCurrenciesByShopID($self->id)};
    LoadClassByAlias('ShopLanguage')->deleteObjects('ByShopID', [$self->id]);

    $self->set({
        'Address' => undef,
        'ShopTypeRef' => undef,
        'SuperMerchant' => undef,
    });
    $self->deleteChildren;
    DeleteShopFeature($_) foreach @{GetPKeyShopFeaturesByShopID($self->id)};

    $self->SUPER::delete;

    return;
}

#========================================================================================
# §function     set
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  This function triggers hooks 'OBJ_BeforeUpdateShop' and 'OBJ_AfterUpdateShop'.
#----------------------------------------------------------------------------------------
# §input        $hValues | attribute name => value hash | ref.hash
# §input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# §hook         OBJ_BeforeUpdateShop | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopID        - shop id              - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# §hook         OBJ_AfterUpdateShop | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopID        - shop id              - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShop', {'Object'=>$self, 'Values'=> $hValues, 'ShopID' => $self->id, 'LanguageID' => $LanguageID});

    # if close/open open all static pages are worthless
    # http result 304 for standard languageid and currencyid
    $self->clearPageCache if $self->_mustUpdatePageCache( $hValues );

    if (exists $hValues->{'Alias'}) {
        my $SavedIncludeHandler = DE_EPAGES::Object::API::SavedIncludesHandler->new();
        $SavedIncludeHandler->clearObjectCache($self);
    };

    $self->SUPER::set($hValues, $LanguageID);

    # send shop changes to Unity for Unity shops
    if ($self->get('IsUnityRegistered')) {
        $self->updateUnityShop($hValues, $LanguageID);
    }

    TriggerHook('OBJ_AfterUpdateShop',  {'Object'=>$self, 'Values'=> $hValues, 'ShopID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# §function     _mustUpdatePageCache
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $MustUpdate = $Object->_mustUpdatePageCache($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  Returns true if the page cache should be updated.
#----------------------------------------------------------------------------------------
# §input        $hValues | attribute name => value hash from <function set() | ref.hash
# §input        $LanguageID | (optional) language id for localized attributes | integer
# §return       $MustUpdate | true if the page cache should be updated | boolean
#----------------------------------------------------------------------------------------
sub _mustUpdatePageCache {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    my @Keys = qw( ClosedByMerchant ClosedByProvider IsTrialShop IsInternalTestShop
        LanguageID CurrencyID DomainName DefaultStyle );

    my @SetKeys = grep { exists $hValues->{$_} } @Keys;
    return 0 unless scalar @SetKeys;

    my @ChangedKeys;
    my $hOrgValues = $self->get(\@SetKeys, $LanguageID);
    my $Class = $self->class;
    foreach my $Key (@SetKeys) {
        my $Attribute = $Class->attribute($Key);
        die "can't compare array attribute ".$Attribute->alias if $Attribute->isArray;
        push @ChangedKeys, $Key if Compare($Attribute->type, $hOrgValues->{$Key}, $hValues->{$Key});
    }

    return scalar @ChangedKeys ? 1 : 0;
}

#========================================================================================
# §function     isTemporaryClosed
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $IsClosed = $Site->isTemporaryClosed;
#----------------------------------------------------------------------------------------
# §description  Returns true if the site is closed temporary. In this case all pages
#               should be delivered with the HTTP status 503 (Service Temporarily
#               Unavailable) to prevent search engines from caching the "shop closed"
#               message.
#----------------------------------------------------------------------------------------
# §return       $IsClosed | true if site is closed | boolean
#========================================================================================
sub isTemporaryClosed {
    my $self = shift;

    return 1 if $self->get('ClosedOrClosedTempByProvider'); #$self->get('ClosedByMerchant') or
    return $self->SUPER::isTemporaryClosed();
}

#========================================================================================
# §function     featureMaxValue
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $MaxValue = $Shop->featureMaxValue( $FeatureName );
# §syntax       $MaxValue = $Shop->featureMaxValue( $Feature );
# §syntax       $Shop->featureMaxValue( $Feature, $NewValue, $IsVisible );
# §example      if( $Shop->featureMaxValue( 'Products' ) > $CurrentValue ) { ... };
# §example      my $OldMaxValue = $Shop->featureMaxValue( $FeatureProducts );
#               $Shop->featureMaxValue( $FeatureProducts, $OldMaxValue + 10 );
#               my $NewMaxValue = $Shop->featureMaxValue( $FeatureProducts );
#----------------------------------------------------------------------------------------
# §description  Returns or sets the maximum value of a shop feature.
#               It is not an error to set the featureMaxValue to a value less than
#               <function featureCurrentValue>.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
# §input        $NewValue | maximum allowed value | int
# §input        $IsVisible | is visible at mbo | boolean
# §return       $MaxValue | maximum allowed value | int
#========================================================================================
sub featureMaxValue {
    my $self = shift;
    my ( $Feature, $NewValue, $IsVisible) = @_;

    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    TestObject( 'Feature', $Feature );

    if( defined $NewValue ) {
        TestInteger( 'NewValue', $NewValue );
        if (ExistsShopFeature( $self->id, $Feature->id )) {
            my $hInfo = GetInfoShopFeature( $self->id, $Feature->id );
            if ($hInfo->{'MaxValue'} != $NewValue or $hInfo->{'IsVisible'} != $IsVisible) {
                UpdateShopFeature( $self->id, $Feature->id, {
                    MaxValue => $NewValue,
                    IsVisible => $IsVisible ? 1 : 0
                });
                $self->clearSavedPages;
            }
        }
        else {
            InsertShopFeature( {
                ShopID => $self->id,
                FeatureID => $Feature->id,
                MaxValue => $NewValue,
                IsVisible => $IsVisible ? 1 : 0,
                IsDirty => 0,
                CurrentValue => 0
            } );
            $self->clearSavedPages;
        }
        return $NewValue;
    }
    else {
        return 0 unless ExistsShopFeature( $self->id, $Feature->id );
        return GetInfoShopFeature( $self->id, $Feature->id )->{MaxValue};
    }
}

#========================================================================================
# §function     featureCurrentValue
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $CurrentValue = $Shop->featureCurrentValue( $FeatureName );
# §syntax       $CurrentValue = $Shop->featureCurrentValue( $Feature );
# §syntax       $Shop->featureCurrentValue( $Feature, $NewValue );
# §syntax       $Shop->featureCurrentValue( $Feature, $NewValue, $AllowToExceed );
# §example      if( $Shop->featureCurrentValue( 'Products' ) < $MaxValue ) { ... };
# §example      my $CountProducts = CountShopProducts( $Shop );
#               $Shop->featureCurrentValue( $FeatureProducts, $CountProducts );
#----------------------------------------------------------------------------------------
# §description  Returns or sets the currently used value of a shop feature. The function
#               also checks if the new value is larger than the previous value and
#               larger than the limit of <function featureMaxValue> for this shop.
#               The error FeatureLimitExceeded is thrown if $NewValue > featureMaxValue
#               && $NewValue > featureCurrentValue.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
# §input        $NewValue | currently used value | int
# §input        $AllowToExceed | (optional, default: false) don't thow an error if the
#                   new value exceeds the maximum  allowed. Set this flag to true when
#                   objects are deleted | boolean
# §return       $CurrentValue | currently used value | int
#========================================================================================
sub featureCurrentValue {
    my $self = shift;
    my ( $Feature, $NewValue, $AllowToExceed ) = @_;
    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    TestObject( 'Feature', $Feature );

    if( defined $NewValue ) {
        my $CurrentValue = $self->featureCurrentValue($Feature);
        my $MaxValue = $self->featureMaxValue($Feature);
        my %Info = ( CurrentValue => $NewValue );
        if ($NewValue < 0) {
            $NewValue = 0;
            GetLog()->info(sprintf('Setting feature %s to negative value %d', $Feature->alias, $NewValue));
            %Info = ( CurrentValue => $NewValue, 'IsDirty' => 1 );
        }
        if( $NewValue > $CurrentValue && $NewValue > $MaxValue ) {
            if ($Feature->get('AllowToExceed')) {
                $Info{'IsDirty'}= 1;
            } else {
                my $Unit = $Feature->get('Unit');
                my $UnitName = defined $Unit ? $Unit->get('Abbreviation') : '(undef)';
                Error( 'FeatureLimitExceeded', {
                    Feature => $Feature->alias,
                    CurrentValue => $CurrentValue,
                    MaxValue => $MaxValue,
                    NewValue => $NewValue,
                    Shop => $self->alias,
                    Name => defined $DE_EPAGES::Object::API::Language::LANGUAGEID
                        ? $Feature->get('NameOrAlias')
                        : $Feature->get('Alias' ),
                    Unit => $UnitName
                }) unless $AllowToExceed;
            }
        }
        if (ExistsShopFeature( $self->id, $Feature->id )) {
            UpdateShopFeature( $self->id, $Feature->id, \%Info );
        } else {
            InsertShopFeature({ 'ShopID'=>$self->id, 'FeatureID'=>$Feature->id, 'MaxValue'=>0,
                IsDirty=>0, IsVisible=>0,
                %Info });
        }
        return $NewValue;
    }
    else {
        return 0 unless ExistsShopFeature( $self->id, $Feature->id );

        my %HookArgs = (
            'Object'    => $self,
            'FeatureID' => $Feature->id,
            'Name'      => $Feature->alias,
            'Value'     => undef,
        );
        TriggerHook('OBJ_GetShopFeatureCurrentValue', \%HookArgs);
        return $HookArgs{'Value'} if defined $HookArgs{'Value'};

        return GetInfoShopFeature( $self->id, $Feature->id )->{CurrentValue};
    }
}

#========================================================================================
# §function     featureTestValue
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->featureTestValue( $FeatureName );
# §syntax       $Shop->featureTestValue( $Feature );
#----------------------------------------------------------------------------------------
# §description  Throws error if feature is more used as possible.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
#========================================================================================
sub featureTestValue {
    my $self = shift;
    my ( $Feature ) = @_;
    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    TestObject( 'Feature', $Feature );

    my $CurrentValue = $self->featureCurrentValue($Feature);
    my $MaxValue = $self->featureMaxValue($Feature);
    return if $CurrentValue <= $MaxValue;
    Error( 'FeatureLimitExceeded', {
        Feature => $Feature->alias,
        CurrentValue => $CurrentValue,
        MaxValue => $MaxValue,
        NewValue => 0,
        Shop => $self->alias,
        Name => defined $DE_EPAGES::Object::API::Language::LANGUAGEID
            ? $Feature->get('NameOrAlias')
            : $Feature->get('Alias' ),
        Unit => $Feature->get('Unit')->get('Abbreviation')
    });
}

#========================================================================================
# §function     featureIsVisible
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $IsVisible = $Shop->featureIsVisible( $FeatureName );
# §syntax       $IsVisible = $Shop->featureIsVisible( $Feature );
# §syntax       $Shop->featureIsVisible( $Feature );
#----------------------------------------------------------------------------------------
# §description  Returns true shop feature is visible.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
# §return       $IsVisible | visible | boolean
#========================================================================================
sub featureIsVisible {
    my $self = shift;
    my ($Feature ) = @_;
    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    TestObject( 'Feature', $Feature );

    return 0 unless ExistsShopFeature( $self->id, $Feature->id );
    return GetInfoShopFeature( $self->id, $Feature->id )->{'IsVisible'};
}

#========================================================================================
# §function     featureIncrementValue
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $CurrentValue = $Shop->featureIncrementValue( $FeatureName, $IncValue );
# §syntax       $CurrentValue = $Shop->featureIncrementValue( $Feature, $IncValue );
# §example      $Shop->featureIncrementValue('Products', 1);  # on insert product
# §example      $Shop->featureIncrementValue('Products', -1); # on delete product
#----------------------------------------------------------------------------------------
# §description  Increments or decrements the currently value of a shop feature. The function
#               also checks if the new value is larger than the previous value and
#               larger than the limit of <function featureMaxValue> for this shop.
#               The error FeatureLimitExceeded is thrown if $NewValue > featureMaxValue
#               && $NewValue > featureCurrentValue.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
# §input        $IncValue | increment value (optional 1) | int
# §return       $CurrentValue | currently used value | int
#========================================================================================
sub featureIncrementValue {
    my $self = shift;
    my ($Feature, $IncrementValue) = @_;
    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    $IncrementValue = 1 unless defined $IncrementValue;
    return $self->featureCurrentValue( $Feature, $self->featureCurrentValue( $Feature ) + $IncrementValue );
}

#========================================================================================
# §function     featureIsDirty
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $IsDirty = $Shop->featureIsDirty( $Feature );
# §syntax       $IsDirty = $Shop->featureIsDirty( $FeatureName );
# §syntax       $Shop->featureIsDirty( $Feature, $IsDirty );
# §example      $Shop->featureIsDirty( 'GallerySize', 1 );
#----------------------------------------------------------------------------------------
# §description  Returns or sets the IsDirty flag for a feature.
#               If this flag is true, then the current value of the feature may not be
#               corrent and should be updated by the script resetDirtyShopFeatures.pl.
#----------------------------------------------------------------------------------------
# §input        $FeatureName | feature name | string
# §input        $Feature | feature object | DE_EPAGES::Shop::API::Object::Feature
# §input        $IsDirty | true if the feature current value is not up to date | int
# §return       $IsDirty | true if the feature current value is not up to date | int
#========================================================================================
sub featureIsDirty {
    my $self = shift;
    my ( $Feature, $IsDirty) = @_;
    $Feature = LoadFeatureByAlias( $Feature ) unless ref $Feature;
    TestObject( 'Feature', $Feature );

    if( defined $IsDirty ) {
        if (ExistsShopFeature( $self->id, $Feature->id )) {
            UpdateShopFeature( $self->id, $Feature->id, {
                IsDirty => $IsDirty ? 1 : 0
            });
        }
        else {
            InsertShopFeature( {
                ShopID => $self->id,
                FeatureID => $Feature->id,
                MaxValue => 0,
                IsVisible => 0,
                IsDirty => $IsDirty ? 1 : 0,
                CurrentValue => 0
            } );
        }
        return;
    }
    else {
        return 0 unless ExistsShopFeature( $self->id, $Feature->id );
        return GetInfoShopFeature( $self->id, $Feature->id )->{IsDirty};
    }
}

#========================================================================================
# §function     features
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aFeature = $Shop->features;
#----------------------------------------------------------------------------------------
# §description  Returns all features that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aFeature | list of feature objects | ref.array.object
#========================================================================================
sub features {
    my $self = shift;

    return [
        map { LoadObject( $_->{FeatureID} ) }
        @{ GetPKeyShopFeaturesByShopID( $self->id ) }
    ];
}

#========================================================================================
# §function     languages
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aLanguageID = $Shop->languages;
#----------------------------------------------------------------------------------------
# §description  Returns all languages that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aLanguageID | list of language identifier | ref.array.int
#========================================================================================
sub languages {
    my $self = shift;

    return [
        map { $_->get('LanguageID') }
        sort {$a->get('Position') <=> $b->get('Position')}
        @{LoadClassByAlias('ShopLanguage')->getObjects('ByShopID', [$self->id])}
    ];
}

#========================================================================================
# §function     localeByLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $LocaleID = $Shop->localeByLanguage($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Returns the first locale for a given language, sorted by position.
#----------------------------------------------------------------------------------------
# §input        $LanguageID | $LanguageID | int
# §return       $LocaleID | locale identifier | string
#========================================================================================
sub localeByLanguage {
    my $self = shift;
    my ($LanguageID) = @_;

    my @Locales = grep { $_->get('Language')->{'LanguageID'} == $LanguageID} @{$self->get('SiteLocales')};
    my $LocaleID;
    if(@Locales && scalar(@Locales) > 0) {
        $LocaleID = $Locales[0]->get('LocaleID');
    } else {
        $LocaleID    = LoadRootObject()->get('LocaleID');
        $LocaleID    = 'en_US' unless defined $LocaleID;
    }

    return $LocaleID;
}

#========================================================================================
# §function     localeForLanguageByShopLocaleIDAndLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $LocaleID = $Shop->localeForLanguageByShopLocaleIDAndLanguage($Language);
#----------------------------------------------------------------------------------------
# §description  Returns the locale for a ShopLocaleID and language
#               If we have "de_CH" as shop locale and add "Italian" as language
#               this function tries to find the locale ch_IT otherwise it_IT is returned
#----------------------------------------------------------------------------------------
# §input        $Language | $Language | object
# §return       $LocaleID | found locale identifier | string
#========================================================================================
sub localeForLanguageByShopLocaleIDAndLanguage {
    my $self = shift;
    my ($NewLanguage) = @_;

    TestObject('NewLanguage', $NewLanguage);

    my $ShopLocaleID = $self->get('LocaleID');

    # use the shop locale part and the default locale part from the language
    my @Locales = grep { $_->get('Language')->{'LanguageID'} == $NewLanguage->get('LanguageID')} @{LoadRootObject()->get('Locales')};

    # loop through locales and try to find a matching locale for the shop and the language
    #<LanguagePartFromLanguageToAdd>_<LocalePartFromShop>  e.g. if we add EN to an DE shop
    # we searching for en_DE. If found, we add this locale otherwise we use the default locale
    # from the language en_EN
    my (undef, $CountryPart) = split(/_/,$ShopLocaleID);

    my $NewLocaleID;
    foreach my $Locale (@Locales) {
        my $LocaleID = $Locale->get('LocaleID');
        # use the prefix from the existing locale and the suffix from the shop locale
        my ($LangPrefix, undef) = split(/_/,$LocaleID);
        my $SearchLocaleID = $LangPrefix."_".$CountryPart;
        next unless $LocaleID eq $SearchLocaleID;
        # use found locale
        $NewLocaleID = $SearchLocaleID;
    }

    # we have not found a locale, so we use the default locale from the language
    unless ($NewLocaleID) {
        $NewLocaleID = $NewLanguage->get('DefaultLocaleID');
    }

    return $NewLocaleID;
}

#========================================================================================
# §function     activeLocaleByLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $LocaleID = $Shop->activeLocaleByLanguage($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Returns the first ACTIVE locale for a given language, sorted by position.
#----------------------------------------------------------------------------------------
# §input        $LanguageID | language id | int
# §return       $LocaleID | locale identifier | string
#========================================================================================
sub activeLocaleByLanguage {
    my $self = shift;
    my ($LanguageID) = @_;

    my $ShopLanguage = shift @{LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$self->id, $LanguageID])};
    my $LocaleID;

    if (defined $ShopLanguage) {
        my @Locales = grep { $_->{'IsActive'} == 1; } @{$ShopLanguage->get('Locales')};
        my $hLocale = shift @Locales;
        $LocaleID = $hLocale->{LocaleID};
    } else {
        $LocaleID    = LoadRootObject()->get('LocaleID');
        $LocaleID    = 'en_US' unless defined $LocaleID;
    }
    return $LocaleID;
}

#========================================================================================
# §function     addLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->addLanguage($LanguageID);
# §example      $Shop->addLanguage(32);
#----------------------------------------------------------------------------------------
# §description  Adds a Language to the shop.
#               If the shop has only one storefront language (feature Langauges=1), then
#               the current language is removed before the new language is added. In this
#               case also the default locale is changed.
#----------------------------------------------------------------------------------------
# §input        $LanguageID | Language identifier | int
# §return       $Language | language object | object
#========================================================================================
sub addLanguage {
    my $self = shift;
    my ($LanguageID, $FromUI) = @_;
    TestInteger( 'LanguageID', $LanguageID );

    my $count = scalar @{$self->languages};

    return LoadClassByAlias('ShopLanguage')->insertObject({
        'Parent'      => $self->folder('Languages'),
        'Alias'       => GetCodeByLanguageID($LanguageID),
        'Site'        => $self,
        'Shop'        => $self,
        'LanguageID'  => $LanguageID,
        'Position'    => $count*STEP_SIZE+STEP_SIZE,
        'FromUI'      => $FromUI,
    });
}

#========================================================================================
# §function     existsLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Exists = $Shop->existsLanguage($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Asks if a Language exists for the shop.
#----------------------------------------------------------------------------------------
# §input        $LanguageID | Language identifier | int
# §return       $Exists | true if the language is enabled in the shop | boolean
#========================================================================================
sub existsLanguage {
    my $self = shift;
    my ($LanguageID) = @_;
    TestInteger( 'LanguageID', $LanguageID );

    my $aShopLanguages = LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$self->id, $LanguageID]);
    return scalar @$aShopLanguages ? 1 : 0;
}

#========================================================================================
# §function     removeLanguage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->removeLanguage($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Removes a Language from the shop.
#----------------------------------------------------------------------------------------
# §input        $LanguageID | Language identifier | int
#========================================================================================
sub removeLanguage {
    my $self = shift;
    my ($LanguageID) = @_;
    TestInteger( 'LanguageID', $LanguageID );

    LoadClassByAlias('ShopLanguage')->deleteObjects('ByShopIDLanguageID', [$self->id, $LanguageID]);
}

#========================================================================================
# §function     existsSiteLocale
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Exists = $Site->existsSiteLocale($LocaleID);
# §example      if( $Site->existsSiteLocale('en_GB') ) { ... }
#----------------------------------------------------------------------------------------
# §description  Returns true if the site supports the given locale.
#               Returns false if the shop does not support the languageof the locale,
#               because the number of languages per shop is restricted by a feature.
#----------------------------------------------------------------------------------------
# §input        $LocaleID | locale id, e.g. 'en_GB' | string
# §return       $Exists | true if the locale is supported by the site | boolean
#========================================================================================
sub existsSiteLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    TestString('LocaleID', $LocaleID);

    return 0 unless $self->SUPER::existsSiteLocale( $LocaleID );
    return $self->existsLanguage( GetInfoLocale($LocaleID)->{'LanguageID'} );
}

#========================================================================================
# §function     updateSiteLocale
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->updateSiteLocale($LocaleID, $Position);
#----------------------------------------------------------------------------------------
# §description  Update the position of the sitelocale in the shop.
#----------------------------------------------------------------------------------------
# §input        $LocaleID | Locale identifier | int
# §input        $Position | position in the list| int
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub updateSiteLocale {
    my $self = shift;
    my ($LocaleID, $Position) = @_;
    TestString('LocaleID', $LocaleID);
    TestInteger('Position', $Position);

    return UpdateSiteLocaleMap($self->id, $LocaleID, {'Position'  => $Position});
}

#========================================================================================
# §function     backofficeLocales
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aLocales = $Shop->backofficeLocales;
#----------------------------------------------------------------------------------------
# §description  Returns all locales that can be used in the back-office of the shop.
#----------------------------------------------------------------------------------------
# §return       $aLocales | list of SiteLocale objects | ref.array.object
#========================================================================================
sub backofficeLocales {
    my $self = shift;
    return [
        grep { $self->existsLanguage($_->get('Language')->{'LanguageID'}) }
        @{$self->SUPER::backofficeLocales}
    ];
}

#========================================================================================
# §function     regions
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aRegions = $Shop->regions;
#----------------------------------------------------------------------------------------
# §description  Returns all regions that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aRegions | list of region objects | ref.array.object
#========================================================================================
sub regions {
    my $self = shift;

    return LoadClassByAlias('Region')->getObjects('ByShopID', [$self->id]);
}

#========================================================================================
# §function     addRegion
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->addRegion($Alias);
# §example      $Shop->addRegion('EU-Countries');
#----------------------------------------------------------------------------------------
# §description  Adds a region to the shop.
#----------------------------------------------------------------------------------------
# §input        $Alias | the Alias for the Region | string
# §return       $Region | region object | DE_EPAGES::Shop::API::Object::Region
#========================================================================================
sub addRegion {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    my $Folder = $self->folder('Regions');
    my $Region = $Folder->insertChild('Region', {
        'Alias'     => $Alias,
    });

    return $Region;
}

#========================================================================================
# §function     existsRegion
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Exists = $Shop->existsRegion($Alias);
#----------------------------------------------------------------------------------------
# §description  Asks if a Region exists for the shop.
#----------------------------------------------------------------------------------------
# §input        $Alias | Region alias | string
# §return       $Exists | 1/0  the Region exists (yes/no) | boolean
#========================================================================================
sub existsRegion {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'RegionAlias', $Alias );

    my $Folder = $self->folder('Regions');
    return $Folder->existsChild($Alias) ? 1 : 0;

}

#========================================================================================
# §function     removeRegion
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->removeRegion($Alias);
#----------------------------------------------------------------------------------------
# §description  Removes a Region from the shop.
#----------------------------------------------------------------------------------------
# §input        $Alias | Region alias | string
#========================================================================================
sub removeRegion {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'RegionAlias', $Alias );

    return unless $self->existsRegion($Alias);

    my $Folder = $self->folder('Regions');
    $Folder->child($Alias)->delete if $Folder->existsChild($Alias);

    return;
}

#========================================================================================
# §function     currencies
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aCurrencyID = $Shop->currencies;
#----------------------------------------------------------------------------------------
# §description  Returns all currencies that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aCurrencyID | list of currency identifiers | ref.array.char(3)
#========================================================================================
sub currencies {
    my $self = shift;

    return [
        map { $_->{CurrencyID} }
        sort { $a->{Position} <=> $b->{Position} }
        @{ GetInfoShopCurrenciesByShopID( $self->id ) }
    ];
}

#========================================================================================
# §function     activeCurrencies
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aCurrencyID = $Shop->activeCurrencies;
#----------------------------------------------------------------------------------------
# §description  Returns all currencies that are usable in the storefront of the shop.
#----------------------------------------------------------------------------------------
# §return       $aCurrencyID | list of currency identifiers | ref.array.char(3)
#========================================================================================
sub activeCurrencies {
    my $self = shift;

    return [
        map { $_->{CurrencyID} }
        sort { $a->{Position} <=> $b->{Position} }
        grep { $_->{IsActivated} }
        @{ GetInfoShopCurrenciesByShopID( $self->id ) }
    ];
}

#========================================================================================
# §function     countries
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aCountryID = $Shop->countries;
#----------------------------------------------------------------------------------------
# §description  Returns all countries that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aCountryID | list of country identifiers | ref.array.cahr(3)
#========================================================================================
sub countries {
    my $self = shift;

    return [
        map { $_->{CountryID} }
        @{ GetPKeyShopCountriesByShopID( $self->id ) }
    ];
}

#========================================================================================
# §function     addCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->addCountry($CountryID,$Position,$IsDefault);
# §example      $Shop->addCountry(4,10,1);
#----------------------------------------------------------------------------------------
# §description  Adds a Country to the shop.
#----------------------------------------------------------------------------------------
# §input        $CountryID | Country identifier | int
# §input        $Position | Position in the list | int
# §input        $IsDefault | default country for shop | int
#========================================================================================
sub addCountry {
    my $self = shift;
    my ($CountryID, $Position,$IsDefault) = @_;
    TestInteger( 'CountryID', $CountryID);
    TestInteger( 'Position', $Position);

    InsertShopCountry( {
        ShopID => $self->id,
        CountryID => $CountryID,
        Position => $Position,
        IsDefault => $IsDefault,
    } );
}

#========================================================================================
# §function     existsCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Exists = $Shop->existsCountry($CountryID);
#----------------------------------------------------------------------------------------
# §description  Asks if a Country exists for the shop.
#----------------------------------------------------------------------------------------
# §input        $CountryID | Country identifier | int
# §return       $Exists | 1/0  the ShopCountry exists (yes/no) | boolean
#========================================================================================
sub existsCountry {
    my $self = shift;
    my ($CountryID) = @_;
    TestInteger( 'CountryID', $CountryID);
    return ExistsShopCountry($self->id, $CountryID);
}

#========================================================================================
# §function     updateCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->updateCountry($CountryID, $Position, $IsDefault);
#----------------------------------------------------------------------------------------
# §description  Update the position of a country in the shop or set a country default.
#----------------------------------------------------------------------------------------
# §input        $CountryID | Country identifier | int
# §input        $Position | position in the list| int
# §input        $IsDefault | true if a country is default | int
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub updateCountry {
    my $self = shift;
    my ($CountryID, $Position, $IsDefault) = @_;
    TestInteger( 'CountryID', $CountryID);

    return UpdateShopCountry($self->id, $CountryID, {'Position'  => $Position,
                                                     'IsDefault' => $IsDefault });
}

#========================================================================================
# §function     renumberCountries
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->renumberCountries();
#----------------------------------------------------------------------------------------
# §description  Update the positions of countries in the shop.
#========================================================================================
sub renumberCountries {
    my $self = shift;

    #save sort order for shopcountries
    my $Position = POSITION_START;
    my @ShopCountries = sort { $self->infoCountry($a)->{'Position'} <=> $self->infoCountry($b)->{'Position'} }
                        @{$self->countries};

    foreach my $CountryID (@ShopCountries) {
        my $IsDefault = $self->infoCountry($CountryID)->{'IsDefault'};
        $self->updateCountry($CountryID,$Position,$IsDefault);
        $Position += POSITION_STEP;
    }
    return;
}

#========================================================================================
# §function     removeCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->removeCountry($CountryID);
#----------------------------------------------------------------------------------------
# §description  Removes a Country from the shop.
#----------------------------------------------------------------------------------------
# §input        $CountryID | Country identifier | int
#========================================================================================
sub removeCountry {
    my $self = shift;
    my ($CountryID) = @_;
    TestInteger( 'CountryID', $CountryID );

    return unless ExistsShopCountry( $self->id, $CountryID);
    DeleteShopCountry( $self->id, $CountryID );

    # remove country from all regions
    foreach my $Region ( @{$self->regions} ) {
        $Region->removeCountry( $CountryID ) if $Region->existsCountry( $CountryID );
    }
}

#========================================================================================
# §function     infoCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->infoCountry($CountryID);
#----------------------------------------------------------------------------------------
# §description  Return information about a country of the shop.
#----------------------------------------------------------------------------------------
# §input        $CountryID | Country identifier | int
#----------------------------------------------------------------------------------------
# §return       $hShopCountry | with following keys:
#               <ul>
#                   <li>ShopID        - shop id              (part of primary key)- int
#                   <li>CountryID     - country identifier - iso num c (part of primary key)- int
#                   <li>Position      - position             (optional)- int
#                   <li>IsDefault      - is default          (optional)- int
#               </ul>| reference to hash
#========================================================================================
sub infoCountry {
    my $self = shift;
    my ($CountryID) = @_;
    TestInteger( 'CountryID', $CountryID );

    return GetInfoShopCountry($self->id, $CountryID);
}

#========================================================================================
# §function     defaultCountry
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->defaultCountry();
#----------------------------------------------------------------------------------------
# §description  Return id of default country of the shop, or undef if no default country is set
#----------------------------------------------------------------------------------------
# §return       $CountryID | CountryID | int
#========================================================================================
sub defaultCountry {
    my $self = shift;
    my $aShopcountries = GetInfoShopCountriesByShopID( $self->id );
    foreach my $ShopCountry (@{$aShopcountries}) {
        return $ShopCountry->{'CountryID'} if $ShopCountry->{'IsDefault'};
    }
    return;
}


#========================================================================================
# §function     addCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->addCurrency($CurrencyID);
# §example      $Shop->addCurrency('EUR');
#----------------------------------------------------------------------------------------
# §description  Adds a currency to the shop.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
#========================================================================================
sub addCurrency {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);

    my $aCurrencies = $self->currencies;
    my $count = scalar @$aCurrencies;

    #set default and active if no default exists
    my $IsActivated = scalar @$aCurrencies ? 0 : 1;
    Transaction( sub {
        InsertShopCurrency( {
            ShopID => $self->id,
            CurrencyID => $CurrencyID,
            IsActivated => $IsActivated,
            Position => $count*STEP_SIZE+STEP_SIZE,
        } );
        $self->set({'CurrencyID' => $CurrencyID }) if $IsActivated;
    });
    return;
}

#========================================================================================
# §function     existsCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Exists = $Shop->existsCurrency($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Asks if a currency exists in the shop.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
# §return       $Exists | 1/0  the ShopCurrency exists (yes/no) | boolean
#========================================================================================
sub existsCurrency {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);
    return ExistsShopCurrency($self->id, $CurrencyID);
}

#========================================================================================
# §function     updateCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->updateCurrency($CurrencyID, $Position);
#----------------------------------------------------------------------------------------
# §description  Update the position of the currency in the shop.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | int
# §input        $Position | position in the list| int
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub updateCurrency {
    my $self = shift;
    my ($CurrencyID, $Position) = @_;
    return UpdateShopCurrency($self->id, $CurrencyID, {'Position'  => $Position});
}

#========================================================================================
# §function     removeCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->removeCurrency($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Removes a currency from the shop.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub removeCurrency {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);

    return unless ExistsShopCurrency( $self->id, $CurrencyID);
    DeleteShopCurrency( $self->id, $CurrencyID );
}

#========================================================================================
# §function     activateCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->activateCurrency($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Sets a currency active in the storefront.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub activateCurrency {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);
    return UpdateShopCurrency($self->id, $CurrencyID,{'IsActivated' => 1});
}

#========================================================================================
# §function     deactivateCurrency
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $Finished = $Shop->deactivateCurrency($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Sets a currency inactive in the storefront.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
# §return       $Finished | Returns 1 if update is finished | int
#========================================================================================
sub deactivateCurrency {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);
    return UpdateShopCurrency($self->id, $CurrencyID,{'IsActivated' => 0});
}

#========================================================================================
# §function     isCurrencyActive
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $IsActivated = $Shop->isCurrencyActive($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Asks if the currency is active in the storefront.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | Currency identifier | char(3)
# §return       $IsActivated | currency is activated at storefront | bit
#========================================================================================
sub isCurrencyActive {
    my $self = shift;
    my ($CurrencyID) = @_;
    TestString( 'CurrencyID', $CurrencyID, 3, 3);
    return GetInfoShopCurrency($self->id, $CurrencyID)->{'IsActivated'};
}

#========================================================================================
# §function     importXML
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# §description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# §input        $Handler | xml import handler with support function addObject | object
# §input        $hElement | XML element structure, see XML::SAX | ref.hash
# §input        $Class | class object; the new object will be an instance of this class | object
# §input        $hAttributes | attributes hash | ref.hash
# §return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $self = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $Object = $self->SUPER::importXML($Handler, $hElement, $Class, $hAttributes );
    $Handler->pushID( 'Shop', $Object);
    $Handler->pushID( 'ShopID', $Object->id);
    return $Object;
}

#========================================================================================
# §function     importXMLEnd
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Object->importXMLEnd( $Handler, $hElement );
#----------------------------------------------------------------------------------------
# §description  import the object attributes with are childs.
#----------------------------------------------------------------------------------------
# §input        $Handler | xml import handler with support function addObject | object
# §input        $hElement | xml import element (undef on end document) | hash ref
#========================================================================================
sub importXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $self->SUPER::importXMLEnd( $Handler, $hElement );
    $Handler->popID( 'Shop' );
    $Handler->popID( 'ShopID' );
}

#========================================================================================
# §function     exportXMLElements
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->exportXMLElements($Driver, $allAttributes);
#----------------------------------------------------------------------------------------
# §description  Exports features of shop.
#----------------------------------------------------------------------------------------
# §input        $Driver | export driver | object
# §input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    if ($Driver->exportLevel == $self->class->get('ExportLevel')) {
        # do not export Features
        # DE_EPAGES::Object::API::XML::ObjectExportDriver::ShopFeaturesByShop($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ShopLanguagesByShop($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ShopLocalesByShop($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ShopCountriesByShop($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ShopCurrenciesByShop($Driver);
    }
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
}

#========================================================================================
# §function     exportXMLChildElements
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Object->exportXMLChildElements( $Driver, $hAttributes );
#----------------------------------------------------------------------------------------
# §description  Add xml elements for children (asks each with planXMLExport to add to driver).
#----------------------------------------------------------------------------------------
# §input        $Driver | xml export driver with support function addObject | object
# §input        $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLChildElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # do only export child objects if ExportRecursive
    if ($Driver->{'ExportRecursive'}) {
        if($Driver->exportLevel == 1){
            #sort by shopchild order
            my $aSortedChildObjects = GetShopChildOrderList();
            my $hUnsortedChildObjects = {};
            my @SortedExistingChildObjects;
            my @SortedExistingChildObjectAliases;

            foreach my $Object (@{$self->exportableChildren}){
                $hUnsortedChildObjects->{$Object->alias} = 1;
            }

            foreach my $Alias (@{$aSortedChildObjects}){
                if ($hUnsortedChildObjects->{$Alias}){
                    push(@SortedExistingChildObjectAliases, $Alias);
                    delete $hUnsortedChildObjects->{$Alias};
                }
            }

            my @MissingChildsInOrderList = keys %$hUnsortedChildObjects;

            push(@SortedExistingChildObjectAliases, @MissingChildsInOrderList);

            foreach my $Alias (@SortedExistingChildObjectAliases){
                push(@SortedExistingChildObjects, $self->child($Alias));
            }

            $_->planXMLExport($Driver) foreach @SortedExistingChildObjects;
        }
        else{
            $_->planXMLExport($Driver) foreach sort {
                $a->get('Position')     <=> $b->get('Position') ||
                $a->class->alias cmp $b->class->alias ||
                $a->alias        cmp $b->alias
                } @{$self->exportableChildren};
        }
    }
    return;
}

#========================================================================================
# §function     exportableChildren
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aChildren = $Object->exportableChildren
#----------------------------------------------------------------------------------------
# §description  Don't export the folder "Languages".
#----------------------------------------------------------------------------------------
# §return       $aChildren | children of the object | array ref
#========================================================================================
sub exportableChildren {
    my $self = shift;

    return [ grep { $_->alias ne 'Languages' } @{$self->children} ];
}

#========================================================================================
# §function     formatter
#----------------------------------------------------------------------------------------
# §syntax       $Formatter = $Shop->formatter($CurrencyID);
#----------------------------------------------------------------------------------------
# §description  Returns the money formatter for the currency.
#----------------------------------------------------------------------------------------
# §input        $CurrencyID | currency code | char(3)
# §return       $Formatter | money formatter | object
#========================================================================================
sub formatter {
    my $self = shift;
    my ($CurrencyID) = @_;
    $self->{'Formatter'} = {} unless exists $self->{'Formatter'};
    if (not exists $self->{'Formatter'}{$CurrencyID}) {
        $self->{'Formatter'}{$CurrencyID} = DE_EPAGES::Core::API::Object::NumberFormat->new(
            int_currencyid => $CurrencyID,
            decimal_digits => GetInfoCurrency($CurrencyID)->{'MinorUnit'},
        );
    }
    return $self->{'Formatter'}{$CurrencyID};
}

#========================================================================================
# §function     default
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hValues = DE_EPAGES::Object::API::Object::Site->default( $Parent, $hInfo );
# §example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# §description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# §input        $Parent | parent object of the new object | object
# §input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# §return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsStandalone' => 1,
        'TaxModel'         => TAX_MODEL_GROSS,
        'AddProductStyle'  => 0,
        'ClosedByMerchant' => 0,
        'ClosedByProvider' => 0,
        'IsTrialShop' => 0,
        'IsInternalTestShop' => 0,
        'InputTaxModel' => TAX_MODEL_GROSS,
        'VATreleased' => 0,
        'AcceptPrivacyPolicy' => 0,
        'LoginRequired2Order' => 0,
        'LoginRequired2InstantlyOrder' => 0,
        'HasCreatedInitialShortURLs' => 0,
    };
}

#========================================================================================
# §function     businessHours
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Calendar = $Shop->businessHours;
#----------------------------------------------------------------------------------------
# §description  Returns shop calendar (child BusinessHours), if not exists then the
#               calendar will be created.
#----------------------------------------------------------------------------------------
# §return       $Calendar | calendar object | object
#========================================================================================
sub businessHours {
    my $self = shift;
    return $self->child('BusinessHours') if $self->existsChild('BusinessHours');
    return $self->insertChild('Calendar', { Alias=>'BusinessHours'});
}

#========================================================================================
# §function     canInvalidatePageCache
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Object->canInvalidatePageCache;
#----------------------------------------------------------------------------------------
# §description  Return true if page cache can be updated. Default is false. If page cache
#               should set to invalid the object has to overwrite this method.
#----------------------------------------------------------------------------------------
# §return       $canInvalidate | true if page cache can be updated | boolean
#========================================================================================
sub canInvalidatePageCache {
    return 1;
}

#========================================================================================
# §function     get
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Value = $Shop->get($Name);
# §syntax       $hValues = $Shop->get($aNames);
#----------------------------------------------------------------------------------------
# §example      $Password = $Shop->get('TaxModel');
# §example      $Password = $Shop->get(['TaxModel','WebPath'])->{'WebPath'};
#----------------------------------------------------------------------------------------
# §description  Returns one or more attribute values.
#               Triggers hook OBJ_GetShop
#----------------------------------------------------------------------------------------
# §input        $Name    | attribute name | string
# §input        $aNames  | list of attribute names | array ref
# §return       $hValues | name => value pairs | hash ref
# §return       $Value   | attribute value | string (or other)
#========================================================================================
sub get {
    my $self = shift;
    my ($aNames, $LanguageID) = @_;

    return $self->triggerGetHook('OBJ_GetShop', $aNames, $LanguageID, sub {
        my ($_aNames, $_LanguageID) = @_;
        return $self->SUPER::get($_aNames, $_LanguageID)
    });
}

#========================================================================================
# §function     formatNumber
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $FormNo = $Shop->formatNumber($ClassName, $Number, $Parent);
#----------------------------------------------------------------------------------------
# §example      $FormNo = $Shop->formatNumber('Invoice', 1234, $Order);
#----------------------------------------------------------------------------------------
# §description  Returns the alias for this class.
#----------------------------------------------------------------------------------------
# §input        $ClassName | class name | string
# §input        $Number | number suffix of alias | integer
# §input        $Parent | parent object (optional) | DE_EPAGES::Object::API::Object::Object
# §return       $NextNo | new number alias suitable for given class name | string
#========================================================================================
sub formatNumber {
    my $self = shift;
    my ($ClassName, $Number, $Parent) = @_;

    my $hValues = $self->get('NumberFormats') // {};
    TestString('ClassName', $ClassName);

    my $hPatterns = $hValues->{$ClassName} // {};
    my %InfixPatternMap = %{ $NUMBER_FORMAT_INFIX_PATTERNS };

    $hPatterns->{'Infix'} //= '';
    my $ParentInfix = undef;
    foreach my $key ('CustomerNo','OrderNo'){
        if($hPatterns->{'Infix'} eq $key){
            TestObject('Parent',$Parent);
            $ParentInfix = $Parent->alias;
        }
    }

    my $String = '';
    $String .= $hPatterns->{'Prefix'} if $hPatterns->{'Prefix'};
    if($hPatterns->{'Infix'}) {
        if(exists($InfixPatternMap{$hPatterns->{'Infix'}})) {
            $String .= DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale' => $self->get('LocaleID'))
                ->format_datetime(GetCurrentDateTime(), 'date', $InfixPatternMap{$hPatterns->{'Infix'}}) if $hPatterns->{'Infix'};
        } else {
            $String .= $ParentInfix;
        }
    }
    $String .= $hPatterns->{'Separator'} if $hPatterns->{'Separator'};
    if(defined $Number){
        $hPatterns->{'Suffix'} //= '4'; #default number of digits
        $String .= sprintf '%0'.$hPatterns->{'Suffix'}.'s', $Number;
    }
    return $String;

}

#========================================================================================
# §function     useMobileView
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $UseMobileView = DE_EPAGES::Object::API::Object::Site->useMobileView();
#----------------------------------------------------------------------------------------
# §description  if the feature is enabled and the shop allows mobile view
#----------------------------------------------------------------------------------------
# §return       $UseMobileView | true if the mobile view should be used | boolean
#========================================================================================
sub useMobileView {
    my $self = shift;

    if ($self->featureMaxValue('MobileView') && $self->get('UseMobileSF')) {
        return 1;
    }
    return 0;
}

#========================================================================================
# §function     registerUnity
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->registerUnity;
#----------------------------------------------------------------------------------------
# §description  Register the shop at the Unity server.
#               Throws UNITY_API_CALL_ERROR if this failed.
#----------------------------------------------------------------------------------------
# §hook         API_BeforeCreateUnityShop | hook parameter keys :
#               <ul>
#                 <li>Object  - this object          - object
#                 <li>Content - content to send to the unity API - ref.hash
#               </ul>
#========================================================================================
sub registerUnity {
    my $self = shift;

    return unless $self->featureMaxValue('Unity');

    my %hRequest;

    TriggerHook('API_BeforeCreateUnityShop', {
        'Object'  => $self,
        'Content' => \%hRequest
    });

    eval {
        $self->sendUnityRequest('POST', '/shop', undef, \%hRequest);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        Error($Error);
    }
    else {
        $self->set({'IsUnityRegistered' => 1});
    }

    TriggerHook('API_AfterCreateUnityShop', {
        'Object'  => $self,
    });
}

#========================================================================================
# §function     isUnity
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $IsUnityShop = $Shop->isUnity();
#----------------------------------------------------------------------------------------
# §description  Asks if the shop has been registered as a unity shop.
#----------------------------------------------------------------------------------------
# §return       $IsUnityShop | shop is a unity one | bit
#========================================================================================
sub isUnity {
    my $self = shift;

    return $self->get('IsUnityRegistered');
}

#========================================================================================
# §function     updateUnityShop
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->updateUnityShop($hParameter);
#----------------------------------------------------------------------------------------
# §description  Performs a REST call to Unity in case the shop attributes have changed.
#               This function triggers the hook 'API_BeforePatchUnityShop'.
#----------------------------------------------------------------------------------------
# §input        $hParams | content for request | hash ref
# §input        $LanguageID | language id | integer
# §hook         API_BeforePatchUnityShop | hook parameter keys :
#               <ul>
#                 <li>Object  - this object          - object
#                 <li>Values  - new values for object - ref.hash
#                 <li>Content - content to patch to the unity API - ref.hash
#               </ul>
#========================================================================================
sub updateUnityShop {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    my %hContent;

    TriggerHook('API_BeforePatchUnityShop', {
        'Object'  => $self,
        'Values'  => $hValues,
        'Content' => \%hContent,
        'LanguageID' => $LanguageID
    });

    #return if nothing to set
    return unless scalar keys %hContent;

    # add the locale from the language id to the query params
    my $hQueryParams = {};

    # get locale from language id
    if (defined $LanguageID){
        my $ShopLanguage = shift @{LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$self->id, $LanguageID])};
        my $LocaleIDForNewShop = $self->localeForLanguageByShopLocaleIDAndLanguage($ShopLanguage);
        $hQueryParams->{'locale'} = $LocaleIDForNewShop;
    } else {
        # (uncomment if needed) we need or don't need the locale if we have a non localized request for all shops
        #$hQueryParams->{'locale'} = $self->get('LocaleID');
    }

    $self->sendUnityRequest('PATCH', '/shop', $hQueryParams, \%hContent);


    TriggerHook('API_AfterPatchUnityShop', {
        'Object'  => $self,
        'Values'  => $hValues,
        'LanguageID' => $LanguageID
    });

}

#========================================================================================
# §function     sendUnityRequest
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->sendUnityRequest($Method, $Route, $hQueryParams, $hContent, $aIgnoreCodes);
# §syntax       $Shop->sendUnityRequest('POST', '/shop/locale', {}, {'locale'=>'ru_RU'});
# §syntax       $Shop->sendUnityRequest('PATCH', '/shop', {'locale'=>'en_EN'}, {'name'=>'MyShop'});
# §syntax       $Shop->sendUnityRequest('DELETE', '/shop/locale', {'locale'=>'en_EN'}, {}, [400,404]);
#----------------------------------------------------------------------------------------
# §description  sends a request to the unity server with the given parameter
#               Throws UNITY_API_CALL_ERROR if the request was not successfull.
#----------------------------------------------------------------------------------------
# §input        $Method | request method  | string
# §input        $Route | route to object | string
# §input        $hQueryParams | query parameters for call (optional) | string
# §input        $hContent | payload for call (optional) | string
# §input        $aIgnoreCodes | ignore this response codes (do not produce errors) | ref.array
# §return       $Response | JSON decoded response if method is GET otherwise undef | string
#========================================================================================
sub sendUnityRequest {
    my $self = shift;
    my ($Method, $Route, $hQueryParams, $hContent, $aIgnoreCodes) = @_;
    TestString( 'Method', $Method );

    # encode content
    my $EncodedContent;
    $EncodedContent = encode('UTF-8',to_json($hContent)) if defined $hContent;

    # build the correct request URL by adding route and query params
    my $RequestURL = $self->_buildUnityRequestURL($Route, $hQueryParams);
    my $Request = HTTP::Request->new($Method => $RequestURL);

    #add current RequestID to the Unity request
    my $AS=GetAppServer();
    my $hHeader = {
                   'Authorization' => 'Bearer ' . CreateShopJsonWebToken(SHOP_JWT_LIFETIME, $self, 'provisioning'),
                   'Content-Type'  => 'application/json'
                  };

    if (defined $AS && defined $AS->currentRequest && defined $AS->currentRequest->headers->header('x-epages-requestid')) {
      $hHeader->{'X-epages-RequestId'} =  $AS->currentRequest->headers->header('x-epages-requestid');
    }

    # create a provisoining JWT and adds a request header
    $Request->header(%$hHeader);

    # adds the content and send request
    $Request->content($EncodedContent) if defined $EncodedContent;

    GetLog()->debug('Shop.sendUnityRequest  Request:'. $Request->as_string());
    my $Response = LWP::UserAgent->new->request($Request);
    GetLog()->debug('Shop.sendUnityRequest  Response:'. $Response->as_string());

    # error handling
    unless ($Response->is_success) {

        # hide credentials in error log
        $hContent = $self->_hideCredentials($hContent);
        $EncodedContent = encode('UTF-8',to_json($hContent));

        my $ErrorObject = {
            'Method'  => $Method,
            'ApiURL'  => $RequestURL,
            'Content' => $EncodedContent,
            'Status'  => $Response->code,
            'Message' => $Response->content
        };

        # test if we have response codes to ignore
        if ($Response->code ~~ @$aIgnoreCodes) {
            Warning('UNITY_API_CALL_ERROR', $ErrorObject);
            return;
        }

        Error('UNITY_API_CALL_ERROR', $ErrorObject);
    }

    # return decoded content if we have a GET request only
    return decode_json($Response->decoded_content) if (($Method eq 'GET') or ($Method eq 'POST'));

    return;

}

#========================================================================================
# §function     getCountries
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aCountries = $Shop->getCountries;
#----------------------------------------------------------------------------------------
# §description  Returns all countries that are assigned to the shop.
#----------------------------------------------------------------------------------------
# §return       $aCountries | list of countries | ref.array.hash
#========================================================================================
sub getCountries {
    my $self = shift;

    my $ahShopCountries = $self->get('ShopCountries');
    my @restructured = map { {
        'code2'            => $_->{'Code2'},
        'code3'            => $_->{'Code3'},
        'countryID'        => $_->{'CountryID'},
        'englishName'      => $_->{'English'},
        'countryIsEU'      => $_->{'IsEU'} + 0,
        'nativeName'       => $_->{'Native'},
        'countryIsDefault' => $_->{'IsDefault'} + 0,
    } }  grep {!($_->{'Code2'} eq "GX")} @{$ahShopCountries}; # remove United Kingdom fallback

    return \@restructured;
}

#========================================================================================
# §function     _hideCredentials
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hContent = $Shop->_hideCredentials($hContent);
#----------------------------------------------------------------------------------------
# §description  removes the oauth credentials from $hContent to prevent writing to logfiles
#----------------------------------------------------------------------------------------
# §input        $hContent | $hContent with credentials | ref.hash
# §return       $hContent | $hContent without credentials | ref.hash
#========================================================================================
sub _hideCredentials {
    my $self = shift;
    my ($hContent) = @_;

    return {} unless defined $hContent;

    if (ref($hContent) eq 'HASH'
        and defined $hContent->{'jrest'}
        and defined $hContent->{'jrest'}->{'oauth'}
        and defined $hContent->{'jrest'}->{'oauth'}->{'credentials'}) {
        $hContent->{'jrest'}->{'oauth'}->{'credentials'} = 'xxx';
    }

    return $hContent;
}

#========================================================================================
# §function     _buildUnityRequestURL
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->_buildUnityRequestURL($Route, $hQueryParams);
# §syntax       $Shop->_buildUnityRequestURL('/shop', {'locale'=> 'ru_RU'});
#----------------------------------------------------------------------------------------
# §description  inserts the $Route into the 'WebUrlUnityApiSSL and adds adds $hQueryParams
#               afterwards to the URL
#----------------------------------------------------------------------------------------
# §input        $Route | route to add to the url | string
# §input        $hQueryParams | locale id for call (optional) | string
# §return       $UnityApiUrl | Unity api URL including the /shop/locale or locale parameter route | string
#========================================================================================
sub _buildUnityRequestURL{
    my $self = shift;
    my ($Route, $hQueryParams) = @_;

    my $UnityUrl = $self->get('WebUrlUnityApiSSL');

    unless (defined $UnityUrl and $UnityUrl =~ m/\S/) {
        Error('UNITY_CONFIG_ERROR', {
            'Key' => 'UnityApiUrlScheme'
        });
    }

    if ($Route =~ /\/products/) {
        my $WebUrlUnitySSL = $self->get('WebUrlUnitySSL');
        my $System = LoadRootObject();
        my $UnityApiRoute = $System->get('UnityApiRoute');
        $UnityUrl = $WebUrlUnitySSL . $UnityApiRoute . '?';
    }
    my $ParamString = defined $hQueryParams ? BuildParameterString($hQueryParams) . '&' : '';
    $UnityUrl =~ s:\?:$Route\?$ParamString:;

    return $UnityUrl;
}

#========================================================================================
# §function     getLogoUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->getLogoUrl($Logo);
#----------------------------------------------------------------------------------------
# §description  retruns the absolute URL of the logo, used to hand it over to Unity
#----------------------------------------------------------------------------------------
# §input        $Logo | relative path of the logo, stored in the 'Logo' attribute
#                   of the shop | string
# §return       $LogoURL | absolute URL to the shop logo | string
#========================================================================================
sub getLogoUrl {
    my $self = shift;
    my ($Logo) = @_;

    return undef if not defined $Logo;

    my $WebPath = $self->get('WebPath');

    return $Logo if $Logo =~ $WebPath;

    return join '/', $WebPath, $Logo;
}

#========================================================================================
# §function     getFavIconUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->getFavIconUrl($FavIcon);
#----------------------------------------------------------------------------------------
# §description  retruns the absolute URL of the favicon, used to hand it over to Unity,
#               in case there is no shop specific favicon it will return the URL of the
#               system favicon
#----------------------------------------------------------------------------------------
# §input        $FavIcon | relative path of the favicon, stored in the 'FavIcon' attribute
#                   of the shop | string
# §return       $FavIconURL | absolute URL to the shop or system FavIcon | string
#========================================================================================
sub getFavIconUrl {
    my $self = shift;
    my ($FavIcon) = @_;

    if (defined $FavIcon) {
        my $WebPath = $self->get('WebPath');
        return $FavIcon if $FavIcon =~ $WebPath;
        return join '/', $WebPath, $FavIcon;
    } else {
        my $System = LoadRootObject();
        return join '/', $System->get('StoreRoot'), 'favicon.ico';
    }
}

#========================================================================================
# §function     getWebUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $WebUrl = $Object->getWebUrl($hParams);
#----------------------------------------------------------------------------------------
# §description  Construct the web url of a given object dependend of the following parameters:
#----------------------------------------------------------------------------------------
# §input        $hParams | attributes of new object
#                       <ul>
#                         <li>SSL - SSL web url - boolean</li>
#                         <li>Section - Storefront | Mobile | Admin - string</li>
#                       </ul> | hash ref
# §return       $WebUrl | web url of the object | string
#========================================================================================
sub getWebUrl {
    my $self = shift;
    my ($hParams) = @_;
    my $WebUrl;

    if ($self->get('IsUnityRegistered')) {
        $WebUrl = $self->getWebUrlUnity($hParams);
    } else {
        $WebUrl = $self->SUPER::getWebUrl($hParams);
    }

    return $WebUrl;
}

#========================================================================================
# §function     getShopUrl
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $ShopUrl = $Object->getShopUrl($hParams, $hOptions, $Servlet);
#----------------------------------------------------------------------------------------
# §description  Construct the shop url of a given object dependend of the following parameters:
#----------------------------------------------------------------------------------------
# §input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# §input        $hOptions | with following keys:
#               <ul>
#                   <li>Type          - one of 'sf', 'admin', 'mobile', 'sf/mobile',
#                   if type is 'sf/mobile' a .mobile will be generated if the servlet is defined and the client
#                   is a mobile device, otherwise a .sf url is generated; defaults to 'sf' - string
#                   <li>UseSSL        - use https as protocol if SSL enabled for shop / database (optional, defaults to 0) - boolean
#                   <li>UseObjectPath - use object path rather than objectid (optional, defaults to 0) - boolean
#                   <li>AddToken      - add security token or not (optional, defaults to 0) - boolean
#                   <li>Token         - the security token (required AddToken is true) - String
#                   <li>AddCookies    - add cookies to url or not (optional, defaults to 0) - boolean
#                   <li>Cookies       - cookie values (required if AddCookies is true) - hash.ref
#                   <li>NoUTF8        - do not use utf-8 for encoding get parameters (optional, defaults to 0) - boolean
#                   <li>IsMobileServlet - is the client a mobile device or not - boolean
#               </ul>  | ref.hash
# §input        $Shop | the current shop | object
# §return       $url | the url | string
#========================================================================================
sub getShopUrl {
    my $self = shift;
    my ($hParams, $hOptions, $Shop) = @_;
    my $ShopUrl;

    $ShopUrl = BuildDefaultShopUrl($self, $hParams, $hOptions, $Shop);

    return $ShopUrl;
}

#========================================================================================
# §function     importUneditableAttributesUnityTranslations
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->importUneditableAttributesUnityTranslations($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Imports the uneditable translations in the given language for a Unity
#               shop from the files $ENV{'EPAGES_SHARED'}.'/Stores/Store/ShopImport/
#               Translation.UneditableAttributesUnity.**.xml
#----------------------------------------------------------------------------------------
# §input        $LanguageID | languageID | integer
#========================================================================================
sub importUneditableAttributesUnityTranslations {
    my $self = shift;
    my ($LanguageID) = @_;

    $self->_importUnityTranslations('UneditableAttributes', $LanguageID);
}

#========================================================================================
# §function     importPrefilledAttributesUnityTranslations
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->importPrefilledAttributesUnityTranslations($LanguageID);
#----------------------------------------------------------------------------------------
# §description  Imports translations that should be prefilled in the given language for a
#               Unity shop from the files $ENV{'EPAGES_SHARED'}.'/Stores/Store/ShopImport/
#               Translation.PrefilledAttributesUnity.**.xml
#----------------------------------------------------------------------------------------
# §input        $LanguageID | languageID | integer
#========================================================================================
sub importPrefilledAttributesUnityTranslations {
    my $self = shift;
    my ($LanguageID) = @_;

    $self->_importUnityTranslations('PrefilledAttributes', $LanguageID);
}

#========================================================================================
# §function     _importUnityTranslations
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->_importUnityTranslations($FileNameExtension, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  Imports translations in the given language for a Unity
#               shop from the directory $ENV{'EPAGES_SHARED'}.'/Stores/Store/ShopImport/
#----------------------------------------------------------------------------------------
# §input        $FileNameExtension | file name extension:
#                   Translation.<Extension>Unity.xx.xml | string
# §input        $LanguageID | languageID | integer
#========================================================================================
sub _importUnityTranslations {
    my $self = shift;
    my ($FileNameExtension, $LanguageID) = @_;

    my $hInfoLanguage = GetInfoLanguage($LanguageID);
    my $LanguageIdentifier = $hInfoLanguage->{'Code2'};

    # dialect handling (e.g. American English -> en-us)
    my $DialectCode = $hInfoLanguage->{'Dialect'};
    if (defined $DialectCode) {
        $LanguageIdentifier = $LanguageIdentifier . '-' . $DialectCode;
    }

    my $Path = LoadRootObject()->get('PrivatePath').'/ShopImport/';
    ImportAttributes($Path . 'Translation.' . $FileNameExtension . 'Unity.'
        . $LanguageIdentifier . '.xml', $self->pathString);
}

#========================================================================================
# §function     getSiteEmailAddress
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $EMailAddress = $Site->getSiteEmailAddress();
#----------------------------------------------------------------------------------------
# §exmple       $EMailAddress = $Site->getSiteEmailAddress();
#----------------------------------------------------------------------------------------
# §description  returns the email of a Site's address or null if it
#               address is not defined
#----------------------------------------------------------------------------------------
# §return       $EmailAddress | email | string
#========================================================================================
sub getSiteEmailAddress {
    my $self = shift;

    # Shop has no address at all
    my $SiteAddress = $self->get('Address');
    return undef unless defined $SiteAddress;

    # shop email address is not valid
    my $Email = $SiteAddress->get('EMail');
    return CheckEMailAddrFormat($Email) ? undef : $Email;
}

1;
