#==============================================================================
# package      DE_EPAGES::ShopConfiguration::UI::Provider
# state        private
#==============================================================================
package DE_EPAGES::ShopConfiguration::UI::Provider;
use base 'DE_EPAGES::Presentation::UI::Object';

use strict;

use DE_EPAGES::Core::API::Constants               qw(PASSWORD_DUMMY);
use DE_EPAGES::Core::API::Error                   qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::File                    qw(ExistsFile);
use DE_EPAGES::Core::API::Password                qw(CryptPassword);
use DE_EPAGES::Core::API::PerlTools               qw(ElementOf GetCurrentDateTime);
use DE_EPAGES::Object::API::Domain                qw(TestWebServerScriptNamePart);
use DE_EPAGES::Object::API::Factory               qw(LoadObjectByPath LoadRootObject LoadClassByAlias Transaction);
use DE_EPAGES::Object::API::Language              qw(GetCurrentLanguageID);
use DE_EPAGES::Permission::API::Permission        qw(IsActionAllowed RoleByName PERMISSION_ALLOW);
use DE_EPAGES::Permission::API::Table::Permission qw(DeletePermission ExistsPermission InsertPermission);
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition      qw(GetSearchDefinition);
use DE_EPAGES::ShopConfiguration::API::Object::PagerDomainSearch;

sub _SetShopsClosed ($$$) {
    my ($Provider, $aShopRefIDs, $Closed) = @_;

    $_->set({IsClosed => $Closed }) foreach
        grep { !$_->get('IsDeleted') }
        @{ $Provider->loadObjects( $aShopRefIDs, 'ShopRef' ) };
    return;
}

#------------------------------------------------------------------------------
# TBO actions
#------------------------------------------------------------------------------

sub SaveNewProvider {
    my $self = shift;
    my ($Servlet) = @_;

    my $System = LoadRootObject();
    my $Params = $Servlet->form->form($System,'SaveProvider');
    if ($Params->{'Password'} !~ /\S/) {
        $Servlet->form->addFormError({
            'Reason'=> 'PASSWORD_EMPTY',
            'Form'  => 'Save',
            'Name'  => 'Password',
        }) ;
    } elsif ($Params->{'PasswordConf'} ne $Params->{'Password'}) {
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'Password',
        }) ;
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'PasswordConf',
        }) ;
    }

    # test if provider alias already exists
    if ( $System->folder('Providers')->findChild($Params->{'Alias'}) ){
        $Servlet->form->addFormError({
            'Reason'=> 'PROVIDER_EXISTS',
            'Form'  => 'Save',
            'Name'  => 'Alias',
        }) ;
    }
    my $NewUserPassword = eval { CryptPassword($Params->{'Password'}); };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'NativeDie' and $Error->shortMessage eq 'Wide character in crypt';
        $Servlet->form->addFormError({
            'Reason'=> 'PASSWORD_WRONG',
            'Form'  => 'Save',
            'Name'  => 'Password',
        }) ;
    }

    $Servlet->form->executeFormError;

    # split provider and user
    my %UserParams;
    $UserParams{$_} = $Params->{'User.'. $_} foreach map { s/^User\.//; $_ } grep { /^User\./ } keys %$Params;
    $UserParams{'Password'}   = $NewUserPassword;

    delete $Params->{$_} foreach ('PasswordConf', 'Password');
    delete $Params->{$_} foreach grep { /^User\./ } keys %$Params;
    $Params->{'LocaleID'} = $UserParams{'LocaleID'}; # used for both provider and user of provider

    my $Provider;

    Transaction( sub {
        $Provider = $System->folder('Providers')->insertChild( 'Provider', $Params);
        my $User  = $Provider->folder('Users')->insertChild( 'User', \%UserParams);
        $Provider->set({ 'User' => $User });

        # set permissions
        InsertPermission( {
            ObjectID => $Provider->id,
            TrusteeID => $User->id,
            MetaActionID => RoleByName($Provider->class, 'BusinessAdministrator' )->id,
            Allow => PERMISSION_ALLOW
        } );
        InsertPermission( {
            ObjectID => $User->id,
            TrusteeID => $User->id,
            MetaActionID => RoleByName($User->class, 'BusinessAdministrator' )->id,
            Allow => PERMISSION_ALLOW
        } );
        InsertPermission( {
            ObjectID => $Provider->id,
            TrusteeID => $User->id,
            MetaActionID => RoleByName($Provider->class, 'WebService' )->id,
            Allow => PERMISSION_ALLOW
        } );

    });

    return $Provider;
}

sub SaveProvider {
    my $self = shift;
    my ($Servlet) = @_;

    # test if Save button pressed
    return if ! $Servlet->params('Save');

    my $Provider = $Servlet->object;
    my $System = LoadRootObject();
    my $Params = $Servlet->form->form($System, 'SaveProvider');

    # test existing provider / user
    my $ChildProviderID = $System->folder('Providers')->findChild($Params->{'Alias'});
    if ( defined $ChildProviderID and $Provider->id != $ChildProviderID ){
        $Servlet->vars('PROVIDER_EXISTS',1);
        $Servlet->form->executeFormError({
            'Reason'=> 'PROVIDER_EXISTS',
            'Form'  => 'Save',
            'Name'  => 'Alias',
        }) ;
        return;
    }

    my $Users = $Provider->child('Users');
    my $ChildUserID = $Users->findChild($Params->{'User.Alias'});

    my $User = $Provider->get('User');
    if ( defined $ChildUserID and $User->id != $ChildUserID) {
        $Servlet->vars('USER_EXISTS',1);
        $Servlet->form->executeFormError({
            'Reason'=> 'USER_EXISTS',
            'Form'  => 'Save',
            'Name'  => 'Name',
        }) ;
        return;
    }

    # test password
    $Params->{'Password'}     = '' unless defined $Params->{'Password'}     || $Params->{'Password'}     eq PASSWORD_DUMMY;
    $Params->{'PasswordConf'} = '' unless defined $Params->{'PasswordConf'} || $Params->{'PasswordConf'} eq PASSWORD_DUMMY;
    if ($Params->{'PasswordConf'} ne $Params->{'Password'}) {
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'Password',
        }) ;
        $Servlet->form->addFormError({
            'Reason'=> 'PWD_NOT_SAME',
            'Form'  => 'Save',
            'Name'  => 'PasswordConf',
        }) ;
    }

    # split provider and user
    my %UserParams;
    $UserParams{$_} = $Params->{'User.'. $_} foreach map { s/^User\.//; $_ } grep { /^User\./ } keys %$Params;

    if ($Params->{'Password'} =~ /\S/) {
        eval { $UserParams{'Password'} = CryptPassword($Params->{'Password'}) };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless $Error->code eq 'NativeDie' and $Error->shortMessage eq 'Wide character in crypt';
            $Servlet->form->addFormError({
                'Reason'=> 'PASSWORD_WRONG',
                'Form'  => 'Save',
                'Name'  => 'Password',
            }) ;
        }
    }
    $Servlet->form->executeFormError;

    delete $Params->{$_} foreach ('PasswordConf', 'Password');
    delete $Params->{$_} foreach grep { /^User\./ } keys %$Params;
    $Params->{'LocaleID'} = $UserParams{'LocaleID'}; # used for both provider and user of provider

    # save provider and user
    Transaction( sub {
        $User->set(\%UserParams);
        $Provider->set($Params);
    });

    return;
}

sub DeleteProvider {
    my $self = shift;
    my (@Params) = @_;

    $self->SUPER::Delete( @Params );
    return;
}

sub AssignProviderDatabases {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;

    my @StoreIDs = $Servlet->form->value('StoreID');
    my $aStores = LoadRootObject()->loadObjects(\@StoreIDs, 'Store');
    $_->set({'Provider' => $Provider }) foreach @$aStores;
    return;
}

sub UnassignProviderDatabases {
    my $self = shift;
    my ($Servlet) = @_;

    my @StoreIDs = $Servlet->form->value('ListObjectID');
    my $aStores = LoadRootObject()->loadObjects(\@StoreIDs, 'Store');
    $_->set({'Provider' => undef }) foreach @$aStores;
    return;
}

#------------------------------------------------------------------------------
# PBO actions
#------------------------------------------------------------------------------

sub SaveDefaultStore {
    my $self = shift;
    my ($Servlet) = @_;

    $self->_CheckAnSaveDefaultStore($Servlet, 'DefaultStore');
    $self->_CheckAnSaveDefaultStore($Servlet, 'DefaultStoreUnity');

    return;
}

sub _CheckAnSaveDefaultStore {
    my $self = shift;
    my ($Servlet, $DefaultStoreAttributeName) = @_;

    my $StoreID = $Servlet->form->value($DefaultStoreAttributeName);
    my $Store = (defined $StoreID && $StoreID =~ /\S/) ? LoadRootObject()->loadObject($StoreID, 'Store') : undef;
    my $Provider = $Servlet->object;

    # security check
    if( defined $Store && $Store->get('Provider')->id != $Provider->id ) {
        die 'Store ' . $Store->alias . ' does not belong to provider ' .$Provider->alias;
    }

    $Provider->set({$DefaultStoreAttributeName => $Store });
    return;
}

sub SearchShops {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;

    my $ClassShopRef = LoadClassByAlias('ShopRef');
    my $SearchDefinition = GetSearchDefinition($ClassShopRef, 'ShopRef');

    my $Form = $Servlet->form;

    if( $Form->existsValue('Search_Domain') ) {
        my $Domain = $Form->value('Search_Domain');
        return $self->_SearchShopDomain( $Servlet, $Domain );
    }

    my %Parameters = ( 'Provider.ID' => $Provider->id );
    my $Alias = $Form->value('Search_Alias');
    $Parameters{'Alias'} = "\%$Alias\%" if defined $Alias && $Alias =~ /\S/;

    my $Comment = $Form->value('Search_Comment');
    if( defined $Comment && $Comment =~ /\S/ ) {
        $Parameters{'Comment'} = "\%$Comment\%";
        $Parameters{'CommentAttributeID'} = $ClassShopRef->attribute('Comment')->id;
    }

    my @Flags = qw( IsMarkedForDel IsClosed IsDeleted IsTrialShop IsInternalTestShop IsPreCreated);
    foreach my $FlagName ( @Flags ) {
        my $Value = $Form->value('Search_'.$FlagName);
        $Parameters{$FlagName} = $Value if defined $Value && $Value =~ /\S/;
    }

    # create reverse parameter (NotClosed, NotDeleted) for datetime flags
    foreach my $FlagName (qw( IsClosed IsDeleted IsMarkedForDel)) {
        if( exists $Parameters{$FlagName} && !$Parameters{$FlagName} ) {
            my $ReverseFlagName = $FlagName;
            $ReverseFlagName =~ s/^Is/Not/;
            delete $Parameters{$FlagName};
            $Parameters{$ReverseFlagName} = 1;
        }
    }

    my $StoreID = $Form->value('Search_StoreID');
    $Parameters{'Store.ID'} = $StoreID if $StoreID;

    my $ShopTypeID = $Form->value('Search_ShopTypeID');
    $Parameters{'ShopType.ID'} = $ShopTypeID if $ShopTypeID;

    my $ShopFeaturePackID = $Form->value('Search_ShopFeaturePackID');
    $Parameters{'ShopFeaturePack.ID'} = $ShopFeaturePackID if $ShopFeaturePackID;

    my %UriParameters;
    foreach my $Key ( @Flags, qw( Alias StoreID ShopTypeID ShopFeaturePackID)) {
        my $Value = $Form->value('Search_' . $Key);
        next unless defined $Value && $Value =~ /\S/;
        $UriParameters{'Search_' . $Key} = $Value;
    }

    my $Pager = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'Servlet' => $Servlet,
        'Object'  => $Provider,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => \%Parameters,
        'UriParameters' => \%UriParameters,
        'OrderBy' => 'Alias',
    });

    return $Pager->page;
}

sub _SearchShopDomain {
    my $self = shift;
    my ($Servlet, $Domain) = @_;

    my %UriParameters = (
        'Search_Domain' => $Domain,
    );

    my $Pager = DE_EPAGES::ShopConfiguration::API::Object::PagerDomainSearch->new({
        'DomainName' => $Domain,
        'Servlet' => $Servlet,
        'Object'  => $Servlet->object,
        'UriParameters' => \%UriParameters,
    });

    return $Pager->page;
}

sub ViewShops {
    my $self = shift;
    my (@Params) = @_;

    return $self->SearchShops( @Params );
}

sub CloseShops {
    my $self = shift;
    my ($Servlet) = @_;

    my @ShopRefIDs = $Servlet->form->value('ListObjectID');

    eval {
        _SetShopsClosed( $Servlet->object, \@ShopRefIDs, 1 );
    };
    if (ExistsError()) {
        _handleError(GetError(), $Servlet);
    }

    return;
}

sub OpenShops {
    my $self = shift;
    my ($Servlet) = @_;

    my @ShopRefIDs = $Servlet->form->value('ListObjectID');

    eval {
        _SetShopsClosed( $Servlet->object, \@ShopRefIDs, 0 );
    };
    if (ExistsError()) {
        _handleError(GetError(), $Servlet);
    }

    return;
}

sub MarkShopsForDelete {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my @ShopRefIDs = $Servlet->form->value('ListObjectID');

    eval{
        foreach my $ShopRef ( @{$Provider->loadObjects( \@ShopRefIDs, 'ShopRef' )} ){
            $ShopRef->set({'IsClosed' => 1, 'MarkedForDelOn' => GetCurrentDateTime()});
        }
    };
    if (ExistsError()) {
        _handleError(GetError(), $Servlet);
    }
}

sub UnMarkShopsForDelete {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my @ShopRefIDs = $Servlet->form->value('ListObjectID');

    foreach my $ShopRef ( @{$Provider->loadObjects( \@ShopRefIDs, 'ShopRef' )} ){
        $ShopRef->set({'MarkedForDelOn' => undef});
    }
}

sub DeleteShops {
    my $self = shift;
    my ($Servlet) = @_;

    # nothing to do
    return if ! $Servlet->params('ListObjectID');

    my $Provider = $Servlet->object;
    my $Form = $Servlet->form;
    my @ShopRefIDs = $Form->value('ListObjectID');
    my @ShopRefs = @{ $Provider->loadObject( \@ShopRefIDs, 'ShopRef' ) } ;
    foreach my $Index ( 0 .. scalar(@ShopRefs)-1 ) {
        my $Object = $ShopRefs[$Index];
        next if $Servlet->hasPermission($Object, 'Delete');

        $Form->addFormError( {
            Reason => 'PermissionDenied',
            Index => $Index,
            LoopName => 'ListObjectIDs',
            Form => 'ShopRefList',
            Name => 'ListObjectID'
        } );
    }
    $Form->executeFormError;

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template
        my @DeleteObjects = map { $_->tleHash } grep { ! $_->get('IsDeleted') } @ShopRefs;

        if (scalar(@DeleteObjects)) {
            $Servlet->vars('DeleteObjects',\@DeleteObjects);
            $Servlet->vars('DELETE',1);
        }else {
            # else reset delete
            $Servlet->vars('DELETE',0);
        }

        return;
    }
    else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    # now delete all selected shops
    eval{
        foreach my $ShopRef (@ShopRefs) {
            $ShopRef->deleteRemoteShop;
        }
    };
    if (ExistsError()) {
        _handleError(GetError(), $Servlet);
    }

    return;
}

sub CreateShop {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $Provider = $Servlet->object;
    my $System = LoadRootObject();

    my $Params = $Servlet->form->form($System,'CreateShop');

    my $ShopType  = $Provider->loadObject($Params->{'ShopTypeID'}, 'ShopType');
    my $Store     = $System->loadObject($Params->{'StoreID'}, 'Store');
    my $ShopAlias = $Params->{'Alias'};

    # test if the shop alias exists
    $Form->addFormError({
        Reason => 'ShopExists',
        Form => 'CreateShop',
        Name => 'Alias'
    }) if $Provider->existsShop( $ShopAlias );

    # test if all import files exist
    my $FileNotFound = 0;
    my $ShopImportDir = $System->get('PrivatePath').'/ShopImport';
    foreach my $FileName ( @{$ShopType->get('ShopImportFiles')} ) {
        $FileNotFound = 1 unless ExistsFile( "$ShopImportDir/$FileName" );
    }

    $Form->addFormError( {
        Reason => 'NoFileForShopType',
        Form => 'CreateShop',
        Name => 'ShopTypeID'
    } ) if $FileNotFound;
    $Form->executeFormError;

    my $WebServerScriptNamePart = $ShopAlias;
    $WebServerScriptNamePart = undef unless TestWebServerScriptNamePart($WebServerScriptNamePart);

    eval {
        $Provider->insertShop({
            ShopType => $ShopType,
            Store => $Store,
            Alias => $ShopAlias,
            ShopAlias => $ShopAlias,
            WebServerScriptNamePart => $WebServerScriptNamePart,
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        my $ErrorCode = $Error->code;

        if (ElementOf($ErrorCode,
                      qw(ShopRefAlias_Exists ShopRefAlias_ContainsIllegalCharacters
                         ShopRefAlias_IllegalStartEnd))) {
            $Form->mapFormError({
                Form   => 'CreateShop',
                Name   => 'Alias',
            }, $Error );
            $Form->executeFormError;
        }
        elsif ($ErrorCode eq 'WebServiceFault') {
            my $RemoteErrorCode = $Error->vars->{'FaultDetail'}{'ErrorDetails'}{'Code'};
            if( $RemoteErrorCode eq 'FeatureNotAvialable') {
                $Form->mapFormError({
                    Form   => 'CreateShop',
                    Name   => 'StoreID',
                    Reason => 'DatabaseNoFeature',
                    Feature => $Error->vars->{'FaultDetail'}{'ErrorDetails'}{'Vars'}{'Feature'},
                }, $Error );
                $Form->mapFormError({
                    Form   => 'CreateShop',
                    Name   => 'ShopTypeID',
                    Reason => 'DatabaseNoFeature',
                }, $Error );
                $Form->executeFormError;
            }
            elsif ( $RemoteErrorCode eq 'FeatureLimitExceeded') {
                $Form->mapFormError({
                    Form   => 'CreateShop',
                    Name   => 'ShopTypeID',
                    Reason => 'ImportFeatureLimitExceeded',
                    Feature => $Error->vars->{'FaultDetail'}{'ErrorDetails'}{'Vars'}{'Feature'},
                }, $Error );
                $Form->executeFormError;
            }
            else {
                Error($Error);
            }
        }
        elsif ($ErrorCode eq 'UNITY_CONFIG_ERROR') {
            $Servlet->vars('UnityError', 1);
            $Form->addFormError({
                'Reason' => 'UnityConfigError',
                'Name'   => 'UnityConfigError'
            });
            $Form->executeFormError();
        }
        elsif ($ErrorCode eq 'UNITY_API_CALL_ERROR') {
            $Servlet->vars('UnityError', $Error->vars()->{'Message'});
            $Form->addFormError({
                'Reason' => 'UnityApiUrlError',
                'Name'   => 'UnityApiUrlError'
            });
            $Form->executeFormError();
        }
        else {
            Error($Error);
        }
    }

    return;
}

sub SaveSupportStaffUser {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my $Form = $Servlet->form;

    #add new support user
    if ($Form->value('Alias') ne '') {
        # test existing support user
        my $Folder = $Provider->folder('Users');
        if ( $Folder->existsChild($Form->value('Alias')) ){
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->executeFormError({
                'Reason'=> 'OBJECT_EXISTS',
                'Form'  => 'SaveSupportStaffUser',
                'Name'  => 'Alias',
            }) ;
            return;
        }


        my $hValues = $Form->form($Provider,'CreateUser');
        # name should be like alias initially
        $hValues->{'Name'} = $Form->value('Alias');


        my $User = $Folder->insertChild( 'User', $hValues);
        my $Role = LoadObjectByPath('/Classes/Provider/Roles/ServiceAdminShopRead');

        InsertPermission({'ObjectID'=>$Provider->id,
                  'TrusteeID'=>$User->id,
                  'MetaActionID'=>$Role->id,
                  'Allow'=>PERMISSION_ALLOW });

        $Role = LoadObjectByPath('/Classes/User/Roles/ServiceAdminShopRead');
        InsertPermission({'ObjectID'=>$User->id,
                  'TrusteeID'=>$User->id,
                  'MetaActionID'=>$Role->id,
                  'Allow'=>PERMISSION_ALLOW });
    }

    return;
}

sub DeleteSupportStaffUser {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my $UserID = $Servlet->params('User');

    my $User = $Provider->loadObject($UserID, 'User');
    $User->delete;

    return;
}

sub UpdateUserPermission {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my $RoleAlias = $Servlet->params('RoleAlias');
    my $UserID = $Servlet->params('UserID');
    my $Selected = $Servlet->params('Selected');

    my $User = $Provider->loadObject($UserID, 'User');

    my %CheckIt;

    my @UserPermissions = qw(ServiceAdminShopRead ServiceAdminShopEdit ServiceAdminShopCreate);

    if ($RoleAlias eq 'ServiceAdminShopRead') {
        if (!$Selected) {
            _UpdateUserPermission($Provider, 'ServiceAdminShopDirectAccess', $User, $Selected);
            $CheckIt{'ServiceAdminShopDirectAccess'} = 0;
        }
    }

    if ($RoleAlias eq 'WebService' || $RoleAlias eq 'ServiceAdminShopDirectAccess') {
        _UpdateUserPermission($Provider, $RoleAlias, $User, $Selected);
        if ($Selected && $RoleAlias eq 'ServiceAdminShopDirectAccess') {
            _UpdateUserPermission($Provider, 'ServiceAdminShopRead', $User, $Selected);
            $CheckIt{'ServiceAdminShopRead'} = 1;
        }
    } else {
        if ($Selected) {
            foreach my $UserPermission (@UserPermissions) {
                _UpdateUserPermission($Provider, $UserPermission, $User, $Selected);
                $CheckIt{$UserPermission} = 1;
                last if $RoleAlias eq $UserPermission;
            }
        }
        else {
            foreach my $UserPermission (reverse(@UserPermissions)) {
                _UpdateUserPermission($Provider, $UserPermission, $User, $Selected);
                $CheckIt{$UserPermission} = 0;
                last if $RoleAlias eq $UserPermission;
            }
        }
    }

    $Servlet->writeContentOutput({
        'CheckIt' => \%CheckIt
    });

    return;
}

sub _UpdateUserPermission($$$$) {
    my ($Provider, $RoleAlias, $User, $Allow) = @_;

    my $Role = LoadObjectByPath('/Classes/Provider/Roles')->child($RoleAlias);

    if ($Allow && !IsActionAllowed($Provider, $User, $Role)) {
        InsertPermission({'ObjectID'=>$Provider->id,
                          'TrusteeID'=>$User->id,
                          'MetaActionID'=>$Role->id,
                          'Allow'=>PERMISSION_ALLOW });
    }
    elsif (!$Allow && IsActionAllowed($Provider, $User, $Role)) {
        if (ExistsPermission($Provider->id, $User->id, $Role->id)) {
            DeletePermission($Provider->id, $User->id, $Role->id);
        }
    }

    return;
}

#========================================================================================
# function     _handleError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _handleError(GetError(), $Servlet);
#----------------------------------------------------------------------------------------
# description  check if a given error is caused by a Unity API error and handle this as
#               form error, throw an error otherwise
#----------------------------------------------------------------------------------------
# input        $Error | catched error | object
# input        $Servlet | the current servlet | object
#========================================================================================
sub _handleError() {
    my ($Error, $Servlet) = @_;
    my $Form = $Servlet->form;

    my $RemoteErrorCode = $Error->vars->{'FaultDetail'}{'ErrorDetails'}{'Code'};
    if ($RemoteErrorCode eq 'UNITY_API_CALL_ERROR') {
        $Servlet->vars('ExistsUnityError', 1);
        $Form->addFormError({
            'Reason' => 'UnityApiError',
            'Name'   => 'UnityApiError'
        });
        $Form->executeFormError();
    }
    else {
        Error($Error);
    }
}

sub ViewShopSurvey {
    my $self = shift;
    my ($Servlet) = @_;

    # alternative language
    my $AltLanguageID = $self->_getAltLanguageID($Servlet);

    $Servlet->params('ViewLanguageID', GetCurrentLanguageID());
    if (defined $AltLanguageID) {
        $Servlet->params('AltLanguageID', $AltLanguageID);
    }

    return;
}

sub SaveShopSurvey {
    my $self = shift;
    my ($Servlet) = @_;

    my $Provider = $Servlet->object;
    my $Params = $Servlet->form->form($Provider, 'SaveShopSurvey');

    my $AltLanguageID = $self->_getAltLanguageID($Servlet);

    # test if we have an URL, InfoText if we switch the survey active
    # and build a correct error handling
    if ($Params->{'ShopSurveyIsActive'}){
        my $hValues = $Params->{'ShopSurveyTexts'};
        foreach my $hValue (@$hValues) {
            my $PostFix = $AltLanguageID eq $hValue->{'LanguageID'} ? 'Translated' : '';

            if($Params->{'ShopSurveyShowButtonBar'}){
                unless($hValue->{'ShopSurveyURL'}){
                    $Servlet->form->addFormError({
                        'Reason' => 'DataMissing',
                        'Form'   => 'SaveShopSurvey',
                        'Name'   => 'ShopSurveyURL'.$PostFix
                    });
                    $Servlet->params('DataMissing', 1);
                }
            }
            unless ($hValue->{'ShopSurveyIntroText'}){
                $Servlet->form->addFormError({
                    'Reason' => 'DataMissing',
                    'Form'   => 'SaveShopSurvey',
                    'Name'   => 'ShopSurveyIntroText'.$PostFix
                }) ;
            };
        }
    }

    $Servlet->form->executeFormError();

    # save now values and store data in Stores
    my $ShopSurveyIsActive = $Params->{'ShopSurveyIsActive'};
    $self->SUPER::Save($Servlet);

    # loop languages and test if we have set all needed values for all languages
    my $aLanguages = LoadRootObject()->get('Languages');
    my $ExistsIntroText=1;
    my $ExistsURL=1;

    if ((scalar @$aLanguages) and $ShopSurveyIsActive){
        # test InfoText
        foreach my $Language (@$aLanguages){
            $ExistsIntroText = 0 unless $Provider->get('ShopSurveyIntroText',$Language->{'LanguageID'});
            if($Params->{'ShopSurveyShowButtonBar'}){
                $ExistsURL = 0 unless $Provider->get('ShopSurveyURL',$Language->{'LanguageID'});
            }
        }
    }

    $Servlet->form->executeFormError({
                    'Reason' => 'DataMissing',
                    'Form'   => 'SaveShopSurvey',
                    'Name'   => 'TextOrURL'
                }) unless ($ExistsIntroText and $ExistsURL) ;

    $Provider->set({'ShopSurveyIsActive' => $ShopSurveyIsActive ? 1:0 });

    my $hValues;
    $hValues->{$_} =  $Params->{$_} foreach qw(ShopSurveyResetMerchant ShopSurveyIsActive ShopSurveyShopMinAge ShopSurveyShopMaxAge ShopSurveyShowButtonBar);
    my $hLangValues = $Params->{'ShopSurveyTexts'};

    # save values on Site for Provider
    my $System = LoadRootObject();
    my @Stores = grep { $_->get('Connection')->{'active'} } @{$System->get('Stores')};
    foreach my $Store (@Stores) {
        next if $Store->get('Alias') eq $System->get('StoreName');

        # set language independent attributes
        $Store->set($hValues,undef);

        # set language dependent attributes
        foreach my $hValue (@$hLangValues) {
            my $LanguageID =  $hValue->{'LanguageID'};
            delete $hValue->{'LanguageID'};
            $Store->set($hValue, $LanguageID);
        }
    }

    return;
}

# returns the alternate language id or undef if only one language is installed
sub _getAltLanguageID {
    my $self = shift;
    my ($Servlet) = @_;

    my $AltLanguageID = $Servlet->params('AltLanguageID');
    return $AltLanguageID if defined $AltLanguageID;

    my $aLanguages = LoadRootObject()->get('Languages');
    return undef if scalar @$aLanguages <= 1;

    return $aLanguages->[0]->{'LanguageID'} == GetCurrentLanguageID()
        ? $aLanguages->[1]->{'LanguageID'}
        : $aLanguages->[0]->{'LanguageID'};
}

1;
