#========================================================================================
# package      DE_EPAGES::ShopMigration::API::CustomerGroupsCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing customergroups
#========================================================================================
package DE_EPAGES::ShopMigration::API::CustomerGroupsCommon;
use base qw(Exporter);

use strict;

our @EXPORT_OK = qw(
    GetCustomerGroups
    SetCustomerGroups
    GetCustomerGroupData
    SetCustomerGroupData
    DeleteCustomerGroupsPrepShop
    CorrectCustomerGroupsPriceListsPrepShop
);

use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::ShopMigration::API::Constants qw(
    CUSTOMERGROUP_ATTRIBUTES
    CUSTOMERGROUP_LOCALIZEDATTRIBUTES
);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::PriceList::API::PriceList qw (CreateUnityPriceListForCustomerGroup);
use DE_EPAGES::PriceList::API::Constants qw(UNITY_PRICELIST_PREFIX);
##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::HTMLSanitizer qw(SanitizeHTMLForScribe);
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);

#========================================================================================
# function     GetCustomerGroups
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahCustomerGroups = GetCustomerGroups($Shop);
#----------------------------------------------------------------------------------------
# description  Get Customer groups from Shop
#----------------------------------------------------------------------------------------
# input        $Shop | BaseShop (epages6) | object
#----------------------------------------------------------------------------------------
# return       $ahCustomerGroups| see GetCustomerGroupData() | reference to array of hashes
#========================================================================================
sub GetCustomerGroups {
    my ($Shop) = @_;

    my @Items;
    my $CustomerGroups = $Shop->get('CustomerGroups');
    foreach my $CustomerGroup (sort {$a->alias cmp $b->alias} @$CustomerGroups) {
        my $hInfo = GetCustomerGroupData($Shop, $CustomerGroup);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetCustomerGroupData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hCustomerGroup = GetCustomerGroupData($Shop, $CustomerGroup);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single CustomerGroup
#               returns empty hash if customergroup is not exportable
#----------------------------------------------------------------------------------------
# input        $Shop | baseshop or prepshop | object
# input        $CustomerGroup | CustomerGroup | object
#----------------------------------------------------------------------------------------
# return       $hCustomerGroup | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                   - string</li>
#                           <li>IsNonMemberDefault      - boolean</li>
#                           <li>IsNewMemberDefault      - boolean</li>
#                       </ul>
#                   </li>
#                    <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2            - en|de|..</li>
#                               <ul>
#                                   <li>Name             - string</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetCustomerGroupData {
    my ($Shop, $CustomerGroup) = @_;

    my %Info = ();
    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    # get baseattributes
    my $Attributes = [ @{CUSTOMERGROUP_ATTRIBUTES()} ];
    $Info{'Attributes'} = $CustomerGroup->get($Attributes);

    # get localized attributes
    my $LocalizedAttributes = [ @{CUSTOMERGROUP_LOCALIZEDATTRIBUTES()} ];
    foreach my $hLanguage (@$ActiveLanguages) {
        my $hLocalInfo = $CustomerGroup->get($LocalizedAttributes, $hLanguage->{'LanguageID'});
        $Info{'LocalizedAttributes'}{$hLanguage->{'Code2'}} = $hLocalInfo;
    }

    return \%Info;
}

#========================================================================================
# function     SetCustomerGroups
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCustomerGroups($Shop, $ahCustomerGroups);
#----------------------------------------------------------------------------------------
# description  sets attributes on customergroup as specified in the $ahCustomerGroups
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahCustomerGroups  | see SetCustomerGroupData() | reference to array of hashes
#========================================================================================
sub SetCustomerGroups {
    my ($Shop, $ahCustomerGroups) = @_;

    foreach my $hCustomerGroup (@$ahCustomerGroups) {
        SetCustomerGroupData($Shop, $hCustomerGroup);
    }
}

#========================================================================================
# function     SetCustomerGroupData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetCustomerGroupData($Shop, $hCustomerGroup);
#----------------------------------------------------------------------------------------
# description  sets attributes on a customergroup as specified in the hash $hCustomerGroup
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hCustomerGroup    | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                    - string</li>
#                           <li>IsNonMemberDefault      - boolean</li>
#                           <li>IsNewMemberDefault      - boolean</li>
#                       </ul>
#                   </li>
#                    <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2            - en|de|..</li>
#                               <ul>
#                                   <li>Name             - string</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub SetCustomerGroupData {
    my ($Shop, $hCustomerGroup) = @_;

    my $CustomerGroupAlias = $hCustomerGroup->{Attributes}{Alias};
    my $Attributes = [ @{CUSTOMERGROUP_ATTRIBUTES()} ];
    my $LocalizedAttributes = [ @{CUSTOMERGROUP_LOCALIZEDATTRIBUTES()} ];

    if ($Shop->get('IsMigrationPrepShop')) {
        # rename epages6 "NewCustomer" group if it has not IsNonMemberDefault flag due to bulkprices handling in Unity
        if ($CustomerGroupAlias eq 'NewCustomer' && !$hCustomerGroup->{Attributes}{IsNonMemberDefault}) {
            $CustomerGroupAlias = 'CG_epages6_' . $CustomerGroupAlias;
            LogDebug(sprintf("Renaming epages6 CustomerGroup '%s' to '%s' due to bulkprices handling in prepshop!\n",
                $hCustomerGroup->{Attributes}{Alias}, $CustomerGroupAlias), 1);
        }
    } else {
       #set correct IsNonMemberDefault and IsNewMemberDefault flag to keep customers assignement
       my $CustomerGroups = $Shop->get('CustomerGroups');
       foreach my $CustomerGroup (@$CustomerGroups) {
           if ($CustomerGroupAlias eq 'CG_epages6_NewCustomer' && $CustomerGroup->alias eq 'NewCustomer') {
               if (!$CustomerGroup->get('IsNonMemberDefault')) {
                   $CustomerGroup->set({'Alias' => $CustomerGroupAlias});
                   last;
               }
           }
           if ($hCustomerGroup->{Attributes}{IsNonMemberDefault} && $CustomerGroup->get('IsNonMemberDefault')) {
               if ($CustomerGroup->alias ne 'NewCustomer') {
                   if ($Shop->folder('Groups')->existsChild('NewCustomer')) {
                       my $BaseNewCustomerGroup = $Shop->folder('Groups')->child('NewCustomer');
                       $BaseNewCustomerGroup->set({'Alias' => 'CG_epages6_NewCustomer'});
                   } else {
                       $CustomerGroup->set({'Alias' => $CustomerGroupAlias});
                   }
               } else {
                   $CustomerGroup->set({'IsNewMemberDefault' => 1});
                   LogDebug(sprintf("Reset IsNewMemberDefault flag for customer group '%s' due to bulkprices handling! \n",
                       $hCustomerGroup->{Attributes}{Alias}), 1);
               }
           }
       }
    }

    my $CustomerGroup = $Shop->folder('Groups')->existsChild($CustomerGroupAlias) ?
        $Shop->folder('Groups')->child($CustomerGroupAlias) :
        $Shop->folder('Groups')->insertChild('CustomerGroup', { Alias => $CustomerGroupAlias });

    foreach my $Type (keys %$hCustomerGroup) {
        if ($Type eq 'Attributes') {
            foreach my $Key (keys %{$hCustomerGroup->{Attributes}}) {
                die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$Attributes);
            }
            $CustomerGroup->set({ 'IsNonMemberDefault'    => $hCustomerGroup->{Attributes}{IsNonMemberDefault},
                                  'IsNewMemberDefault'    => $hCustomerGroup->{Attributes}{IsNewMemberDefault}
            });
        }
        elsif ($Type eq  'LocalizedAttributes') {
            foreach my $LanguageCode2 (keys %{$hCustomerGroup->{LocalizedAttributes}}) {
                foreach my $Key (keys %{$hCustomerGroup->{LocalizedAttributes}{$LanguageCode2}}) {
                    die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$LocalizedAttributes);
                    next unless $Key eq 'Name';
                    my $Value = $hCustomerGroup->{LocalizedAttributes}{$LanguageCode2}{$Key};
                    if (defined $Value && $Value !~ /\w/) {
                        Error('NAME_NOT_VALID', {
                            'CustomerGroup'      => $CustomerGroupAlias,
                            'LanguageCode2' => $LanguageCode2,
                            'Attribute'     => 'Name',
                            'Value'         => $Value,
                        });
                    }
                }
                my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
                $CustomerGroup->set($hCustomerGroup->{LocalizedAttributes}{$LanguageCode2}, $LanguageID);
            }
        } else {
            die "Invalid Type: $Type";
        }
    }

    return;
}

#========================================================================================
# function     DeleteCustomerGroupsPrepShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteCustomerGroupsPrepShop($Shop);
#----------------------------------------------------------------------------------------
# description  deletes customergroups from prepshop
#----------------------------------------------------------------------------------------
# input        $Shop | prep shop - shop object | object
#========================================================================================
sub DeleteCustomerGroupsPrepShop {
    my ($Shop) = @_;

    if ($Shop->get('IsMigrationPrepShop')) {
        my $aCustomerGroups = $Shop->get('CustomerGroups');
        foreach my $Group (@{$aCustomerGroups}) {
                $Group->delete;
            }
        }
    return;
}

#========================================================================================
# function     CorrectCustomerGroupsPriceListsPrepShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       CorrectCustomerGroupsPriceListsPrepShop($Shop);
#----------------------------------------------------------------------------------------
# description  Ensure CustomerGroup/PriceList relations for correct bulkprice handling in Now
#----------------------------------------------------------------------------------------
# input        $Shop | prep shop - shop object | object
#========================================================================================
sub CorrectCustomerGroupsPriceListsPrepShop {
    my ($Shop) = @_;

    my $CustomerGroups = $Shop->get('CustomerGroups');

    foreach my $CustomerGroup (@$CustomerGroups) {
        my $aCustomerGroupPriceLists = $CustomerGroup->get('PriceLists');
        if ($CustomerGroup->get('IsNonMemberDefault')) {
            # rename customergroup with IsNonMemberDefault flag to NewCustomer
            if ($CustomerGroup->alias ne 'NewCustomer') {
                my $Alias = $CustomerGroup->alias;
                $CustomerGroup->set({'Alias' => 'NewCustomer'});
                LogDebug(sprintf("Renaming customergroup '%s' to '%s' due to Unity bulkprice handling!\n",
                    $Alias, $CustomerGroup->alias), 1);

                if (defined $aCustomerGroupPriceLists && scalar(@{$aCustomerGroupPriceLists})) {
                    $Alias = @$aCustomerGroupPriceLists[0]->alias;
                    my $PriceListCurrencyID = @$aCustomerGroupPriceLists[0]->get('CurrencyID');
                    my $NewAlias = UNITY_PRICELIST_PREFIX . $PriceListCurrencyID;
                    @$aCustomerGroupPriceLists[0]->set({ 'Alias' => $NewAlias });
                    LogDebug(sprintf("Renaming pricelist alias '%s' to '%s' due to Unity bulkprice handling!\n",
                        $Alias, $NewAlias), 1);
                }else {
                    # create simplified pricelist in case it does not exists and feature PriceList is enabled
                    if ($Shop->featureMaxValue('PriceLists')) {
                        my $NewPriceList = CreateUnityPriceListForCustomerGroup($CustomerGroup);
                        $NewPriceList->set({ 'Alias' => UNITY_PRICELIST_PREFIX . $Shop->get('CurrencyID') });
                        # add customergroup to new simplified pricelist
                        $NewPriceList->addCustomerGroup($CustomerGroup);
                    }
                }
            } else {
                if (defined $aCustomerGroupPriceLists && scalar(@{$aCustomerGroupPriceLists})) {
                    my $Alias = @$aCustomerGroupPriceLists[0]->alias;
                    my $PriceListCurrencyID = @$aCustomerGroupPriceLists[0]->get('CurrencyID');
                    my $NewAlias = UNITY_PRICELIST_PREFIX . $PriceListCurrencyID;
                    @$aCustomerGroupPriceLists[0]->set({ 'Alias' => $NewAlias });
                    LogDebug(sprintf("Renaming pricelist alias '%s' to '%s' because it is the NewCustomer Group!\n",
                        $Alias, $NewAlias), 1);
                }
            }
        } else {
            # create pricelist for customergroup in case no pricelist is assigned and PriceLists feature is active
            if (!scalar(@{$aCustomerGroupPriceLists}) && $Shop->featureMaxValue('PriceLists')) {
                my $NewPriceList = CreateUnityPriceListForCustomerGroup($CustomerGroup);
                $NewPriceList->set({ 'Alias' => $CustomerGroup->alias });
                $NewPriceList->addCustomerGroup($CustomerGroup);
                LogDebug(sprintf("Create new pricelist '%s' and add customergroup '%s' to pricelist due to Unity bulkprice handling!\n",
                    $NewPriceList->alias, $CustomerGroup->alias), 1);
            }
        }
    }

    return;
}

1;
