#========================================================================================
# package      DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver
# state        public
#----------------------------------------------------------------------------------------
# description  Base class for jsonlines export drivers
#               This exports an array of hashes into a JSONlines file
#               (see also http://jsonlines.org/)
#========================================================================================
package DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver;

use strict;
use Encode qw(encode FB_DEFAULT WARN_ON_ERR);
use FileHandle;
use File::Basename qw(dirname);
use File::Spec::Functions;
use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::File qw(CreateDirectory SetAccessRights);
use DE_EPAGES::Core::API::Iterator qw(ArrayIterator);
use DE_EPAGES::Core::API::TestParameter qw(TestString);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::ShopMigration::API::JSON;

my %FILE_FORMAT = (
    'open_mode' => '>',
    'line_sep'  => "\n",
);

our %JSON_OPTS = (
    'canonical'       => 0,
    'utf8'            => 0,
    'latin1'          => 0,
    'ascii'           => 0,
    'pretty'          => 0,
    'indent'          => 0,
    'space_before'    => 0,
    'space_after'     => 0,
    'relaxed'         => 0,
    'allow_nonref'    => 0,
    'allow_unknown'   => 0,
    'convert_blessed' => 1,
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'JSON_FILE_OPEN_FAILED'   => 'JSON file (#FileName): can\'t open file (#Code)',
        'JSON_FILE_CLOSE_FAILED'  => 'JSON file (#FileName): can\'t close file (#Code)',
        'JSON_WRITE_LINE_FAILED'  => 'JSON file (#FileName), line #Line: can\'t write line (#Code)',
        'JSON_WRITE_LINE_WARNING' => 'JSON file (#FileName), line #Line: error writing line (#Message) (#Value)',
        'JSON_INVALID_CHARACTER'  => 'JSON file (#FileName), line #Line: character #CharacterCode does not map to encoding #Encoding (#Value)',
    };
}
#ERRORDEFINITION

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exporter = DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver->new(%options);
#----------------------------------------------------------------------------------------
# description  create a new jsonlines export driver
#----------------------------------------------------------------------------------------
# input        %options |  exporter options | hash
# return       $Exporter |  exporter | object
#========================================================================================
sub new {
    my $class = shift;
    my %params = @_;
    return bless \%params, $class;
}

#========================================================================================
# function     exportJSONLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exporter->exportJSONLines($DirName, $BaseFileName, $hFormat, $hJsonOptions, $ahItems);
#----------------------------------------------------------------------------------------
# description  exports data (array of hashes) into a file.
#               starts the document, calls function parse and closes the document
#----------------------------------------------------------------------------------------
# input        $DirName  | directory for json file | string (optional)
# input        $BaseFileName | filename for the json file | string (optional)
# input        $hFormat | export file format
#                   <ul>
#                     <li>open_mode - file open mode (optional, default: ">")</li>
#                     <li>line_sep - use this character to separate records (optional, default: "\n")</li>
#                   <ul>
# input        $hJsonOptions | parameters that will be passed through to JSON encode
#                   <ul>
#                     <li>canonical       - sorting keys in json (default: '0')</li>
#                   </ul>
#                    | ref.hash
# input        $ahItems | data for export | DE_EPAGES::Core::API::Iterator or ref.array.hash.string
# return       $Exporter |  exporter | object
#========================================================================================
sub exportJSONLines {
    my $self = shift;
    my ($DirName, $BaseFileName, $hFormat, $hJsonOptions, $ahItems) = @_;

    $self->dirName($DirName) if defined $DirName;
    $self->baseFileName($BaseFileName) if defined $BaseFileName;

    my $FileName = $self->fileName();
    TestString('FileName', $FileName);
    $hFormat //= {};
    $hJsonOptions //= {};
    $hFormat->{$_} = $FILE_FORMAT{$_} foreach grep {not exists $hFormat->{$_}} keys %FILE_FORMAT;
    $hJsonOptions->{$_} = $JSON_OPTS{$_} foreach grep {not exists $hJsonOptions->{$_}} keys %JSON_OPTS;
    $self->{'FileName'} = $FileName;
    $self->{'Writer'} = DE_EPAGES::ShopMigration::API::JSON->new($hJsonOptions);
    $self->{'LineNumber'} = 0;
    $self->{'Items'} = $ahItems // [];
    $self->{'Encoding'} = 'UTF-8';
    $self->{'LineSep'} = $hFormat->{'line_sep'};

    CreateDirectory(dirname($FileName));

    my $fh = FileHandle->new;
    $self->{'FileHandle'} = $fh;

    open($fh, $hFormat->{'open_mode'}, $FileName) ## no critic qw(RequireBriefOpen)
        or Error('JSON_FILE_OPEN_FAILED', { 'Mode' => $hFormat->{'open_mode'}, 'FileName' => $FileName, 'Code' => $! });
    eval {
        $fh->autoflush(1);
        $self->parse();
    };
    my $Error = ExistsError() ? GetError() : undef;
    close $fh or
        Error('JSON_FILE_CLOSE_FAILED', { 'Mode' => $hFormat->{'open_mode'}, 'FileName' => $FileName, 'Code' => $! });;
    Error($Error) if defined $Error;

    SetAccessRights($FileName);
    return $self;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Items = $self->items();
#----------------------------------------------------------------------------------------
# description  return the items to export
#----------------------------------------------------------------------------------------
# return       $Items | parameter of exportJSON | ref.array.hash.string or DE_EPAGES::Core::API::Iterator
#========================================================================================
sub items {return shift->{'Items'};}

#========================================================================================
# function     iterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Items = $self->iterator;
# example      my $Items = $Driver->iterator;
#               while ( my $Item = <$Items> ) {
#                   ProcessItem( $Item );
#               }
#----------------------------------------------------------------------------------------
# description  returns an iterator for items to export
#----------------------------------------------------------------------------------------
# return       $Items | item iterator | ref.array.array.string or DE_EPAGES::Core::API::Iterator
#========================================================================================
sub iterator {
    my $self = shift;

    my $Items = $self->items;

    return (ref($Items) eq 'ARRAY')
        ? ArrayIterator([ @$Items ]) # copy the array, because the iterator would destroy it
        : $Items;
}

#========================================================================================
# function     fileHandle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileHandle;
#----------------------------------------------------------------------------------------
# description  return the current fileHandle
#----------------------------------------------------------------------------------------
# return       $FileHandle | set by exportJSON | filehandle
#========================================================================================
sub fileHandle {return shift->{'FileHandle'};}

#========================================================================================
# function     dirName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->dirName;
# syntax       $self->dirName('/opt/eproot/Shared/Patches/');
#----------------------------------------------------------------------------------------
# description  returns or sets the base directory name to be used
#----------------------------------------------------------------------------------------
# input        $DirName | new directory name | string
# return       $DirName | directory name | string
#========================================================================================
sub dirName {
    my $self = shift;
    my ($DirName) = @_;
    if (@_ == 1) {
        $self->{DirName} = $DirName;
    }
    $self->{DirName} //= $self->defaultDirName();
    return $self->{DirName};
}

#========================================================================================
# function     baseFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->baseFileName;
# syntax       $self->fileName('filename.ext');
#----------------------------------------------------------------------------------------
# description  returns or sets the base filename to be used
#----------------------------------------------------------------------------------------
# input        $FileName | new file name | string
# return       $FileName | file name | string
#========================================================================================
sub baseFileName {
    my $self = shift;
    my ($BaseFileName) = @_;
    if (@_ == 1) {
        $self->{BaseFileName} = $BaseFileName ne '' ? $BaseFileName : undef;
    }
    $self->{BaseFileName} //= $self->defaultBaseFileName();
    return $self->{BaseFileName};
}

#========================================================================================
# function     defaultBaseFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->defaultBaseFileName;
#----------------------------------------------------------------------------------------
# description  returns the default json filename
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub defaultBaseFileName {
    return 'export.json';
}

#========================================================================================
# function     defaultDirName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->defaultDirName;
#----------------------------------------------------------------------------------------
# description  returns the default directory
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub defaultDirName {
    return '.';
}

#========================================================================================
# function     fileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileName;
#----------------------------------------------------------------------------------------
# description  return the full path to the filename
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub fileName {
    my $self = shift;
    return catfile($self->dirName(), $self->baseFileName());
}

#========================================================================================
# function     writer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Writer = $self->writer;
#----------------------------------------------------------------------------------------
# description  return the current json writer
#----------------------------------------------------------------------------------------
# return       $Writer
#========================================================================================
sub writer {return shift->{'Writer'};}

#========================================================================================
# function     encodeTo
# state        public
#----------------------------------------------------------------------------------------
# syntax       encodeTo();
# example      $Driver->encodeTo();
#----------------------------------------------------------------------------------------
# description  returns the target encoding (was set in <function exportJSON>)
#----------------------------------------------------------------------------------------
# return       $Encoding | target encoding (utf8,iso-8859-1,...) | string
#========================================================================================
sub encodeTo {return shift->{'Encoding'};}

#========================================================================================
# function     incLineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->incLineNumber;
#----------------------------------------------------------------------------------------
# description  return the incremented linenumber
#----------------------------------------------------------------------------------------
# return       $linenumber |
#========================================================================================
sub incLineNumber {return ++shift->{'LineNumber'};}

#========================================================================================
# function     lineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->lineNumber;
#----------------------------------------------------------------------------------------
# description  return the current linenumber
#----------------------------------------------------------------------------------------
# return       $linenumber |
#========================================================================================
sub lineNumber {return shift->{'LineNumber'};}

#========================================================================================
# function     writeLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->writeLine($aLineElements);
#----------------------------------------------------------------------------------------
# description  exports a line ( converts and writes to the filehandle )
#----------------------------------------------------------------------------------------
# input        $hValues | record/hash for one element/line | ref.hash.string
#========================================================================================
sub writeLine {
    my $self = shift;
    my ($hValues) = @_;

    my $value = $self->writer->encode($hValues);
    $value .= $self->{LineSep};

    my $Warning;
    local $SIG{__WARN__} = sub {
        my ($Message) = @_;

        return if defined $Warning; # remember only one warning per line

        $Message =~ s/ at \S+ line \d+\b.*//; # remove code line
        if ($Message =~ /"\\x\{([0-9A-Fa-f]+)\}"\ does\ not\ map\ to\ (\S+)/ix) {
            my $CharacterCode = $1;
            my $Character = chr(hex($CharacterCode));
            my $Encoding = $2;
            $Warning = DE_EPAGES::Core::API::Error->new(
                'Code' => 'JSON_INVALID_CHARACTER',
                'Vars' => {
                    'FileName'      => $self->fileName,
                    'CharacterCode' => $CharacterCode,
                    'Character'     => $Character,
                    'Encoding'      => $Encoding,
                    'Line'          => $self->lineNumber,
                    'Value'         => $value,
                },
            );
        }
        else {
            $Warning = DE_EPAGES::Core::API::Error->new(
                'Code' => 'JSON_WRITE_LINE_WARNING',
                'Vars' => {
                    'FileName' => $self->fileName,
                    'Message'  => $Message,
                    'Line'     => $self->lineNumber,
                    'Value'    => $value,
                },
            );
        }
    };

    $self->_print($value) or Error('JSON_WRITE_LINE_FAILED', {
        'FileName' => $self->fileName,
        'Code'     => $!,
        'Line'     => $self->lineNumber,
        'Value'    => $value
    });

    Warning($Warning) if defined $Warning;
    $self->incLineNumber;
    return;
}

# write to file using the selected encoding
sub _print {
    my $self = shift;
    my ($Text) = @_;

    # substitute invalid character with '?' instead of \x{0000}
    my $Bytes = encode($self->encodeTo, $Text, FB_DEFAULT | WARN_ON_ERR);
    return $self->fileHandle->print($Bytes);
}

#========================================================================================
# function     exportLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->exportLine();
#----------------------------------------------------------------------------------------
# description  exports one data line, get the line element from function topID('Item')
#========================================================================================
sub exportLine {
    my $self = shift;

    my $hValues = $self->topID('Item');
    $self->writeLine($hValues);
    return;
}

#========================================================================================
# function     writeLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->writeLines();
#----------------------------------------------------------------------------------------
# description  writes the content lines
#               each item of $self-><function iterator> is exported via <function exportLine>.
#========================================================================================
sub writeLines {
    my $self = shift;

    my $Items = $self->iterator;
    while (my $Item = <$Items>) {
        $self->pushID('Item', $Item);
        $self->exportLine();
        $self->popID('Item');
    }
    return;
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  write the data content lines
#========================================================================================
sub drive {
    my $self = shift;

    $self->writeLines();
    return;
}

#========================================================================================
# function     parse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->parse();
#----------------------------------------------------------------------------------------
# description  just calls function drive
#========================================================================================
sub parse {
    my $self = shift;

    $self->drive();
    return;
}

#========================================================================================
# function     pushID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->pushID( $Key, $Value );
# example      $self->pushID( 'CompanyID', $CompanyID );
#----------------------------------------------------------------------------------------
# description  adds a parameter to the parameter stack. Any previous parameter with the
#               same name will be hidden until $self->popID( $Key ) is called.
#               Use $self->topID( $Key ) to get the parameter value.
#               Use $self->popID( $Key ) to remove the parameter from the stack.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# input        $Value | parameter value
#========================================================================================
sub pushID {
    my $self = shift;
    my ($Key, $Value) = @_;

    $self->{'IDs'}->{$Key} = [] unless exists $self->{'IDs'}->{$Key};
    push @{$self->{'IDs'}->{$Key}}, $Value;
}

#========================================================================================
# function     popID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->popID( $Key );
# example      $CompanyID = $self->popID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  removes a parameter from the parameter stack and returns its value.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub popID {
    my $self = shift;
    my ($Key) = @_;

    return pop @{$self->{'IDs'}->{$Key}};
}

#========================================================================================
# function     topID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->topID( $Key );
# example      $CompanyID = $self->topID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  returns the current value of a parameter
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub topID {
    my $self = shift;
    my ($Key) = @_;

    return() unless exists $self->{'IDs'}->{$Key};
    return $self->{'IDs'}->{$Key}->[-1];
}

1;
