#========================================================================================
# package      DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Base class for json lines import drivers
#               This imports a JSONlines file
#               (see also http://jsonlines.org/)
#========================================================================================
package DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler;

use strict;
use FileHandle;
use File::Spec::Functions;

use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError HandleError);
use DE_EPAGES::Core::API::File qw(DetectFileEncoding);
use DE_EPAGES::Core::API::TestParameter qw(TestString);
use DE_EPAGES::ShopMigration::API::JSON;

my %FILE_FORMAT = (
    'line_sep' => "\n",
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'JSON_FILE_OPEN_FAILED'  => 'JSON file (#FileName): can\'t open file (#Code)',
        'JSON_FILE_CLOSE_FAILED' => 'JSON file (#FileName): can\'t close file (#Code)',
        'JSON_FILE_SEEK_FAILED'  => 'JSON file (#FileName): can\'t see to position #Position (#Code)',
        'JSON_READ_LINE_FAILED'  => 'JSON file (#FileName), line #Line: can\'t read line (#Code)',
        'JSON_FILE_NOT_TEXT'     => 'The file #FileName is not a text file.',
    };
}
#ERRORDEFINITION

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::ShopMigration::API::JSONLines::BaseImportHandler->new(%options);
#----------------------------------------------------------------------------------------
# description  create a new json import handler
#----------------------------------------------------------------------------------------
# input        %options |  importer options | hash
# return       $Handler |  importer | object
#========================================================================================
sub new {
    my $class = shift;
    my %params = @_;
    $params{$_} = $FILE_FORMAT{$_} foreach grep {not exists $params{$_}} keys %FILE_FORMAT;
    $params{'Converter'} = DE_EPAGES::ShopMigration::API::JSON->new(\%params);
    $params{'Encoding'} = 'UTF-8';
    return bless \%params, $class;
}

#========================================================================================
# function     importJSONLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package->importJSONLines($DirName, $BaseFileName, $hFormat));
#----------------------------------------------------------------------------------------
# description  imports data (2 dim. array) from a file.
#               opens the document, calls function setLinesCount, calls function parse
#               and closes the document
#----------------------------------------------------------------------------------------
# input        $DirName  | directory for json file | string (optional)
# input        $BaseFileName | filename for the json file | string (optional)
# input        $hOptions |  export file format
#                   <ul>
#                     <li>eol - end of line (optional, default: "\n")</li>
#                     <li>encoding - character encoding (optional, default: "iso-8859-1")</li>
#                     <li>quote_char - field delimiter (optional, default: '"')</li>
#                     <li>escape_char - escape character (optional, default: '"')</li>
#                     <li>sep_char - field separator (optional, default: ",")</li>
#                     <li>... other options</li>
#                   </ul> | ref.hash
# return       $Importer |  importer | object
#========================================================================================
sub importJSONLines {
    my $class = shift;
    my ($DirName, $BaseFileName, $hOptions) = @_;

    my $self = $class->new(defined $hOptions ? %$hOptions : ());
    $self->baseFileName($BaseFileName) if defined $BaseFileName;
    $self->dirName($DirName) if defined $DirName;
    my $FileName = $self->fileName();
    TestString('FileName', $FileName);
    Error('FileNotFound', { FileName => $FileName }) unless -f $FileName;
    Error('JSON_FILE_NOT_TEXT', { FileName => $FileName }) unless -T $FileName;

    my $DetectedEncoding = DetectFileEncoding($FileName);
    if (defined $DetectedEncoding) {
        $self->{'encoding'} = $DetectedEncoding;
    }

    $self->{'FileName'} = $FileName;
    $self->{'LineNumber'} = 0;
    $self->{'FileHandle'} = FileHandle->new;

    my $fh = $self->fileHandle;
    my $OpenMode = '<:encoding(' . $self->encodeFrom . ')';

    $fh->open($FileName, $OpenMode) or Error('JSON_FILE_OPEN_FAILED', {
        'Mode'     => $OpenMode,
        'FileName' => $FileName,
        'Code'     => $!
    });

    # count the records
    $self->setLinesCount;

    # revert to start of file
    $fh->seek(0, 0) or Error('JSON_FILE_SEEK_FAILED', {
        'FileName' => $FileName,
        'Position' => 0,
        'Code'     => $!
    });

    # import data
    eval {
        $self->parse();
    };
    HandleError(
        OnError => sub {
            my ($Error) = @_;
            $Error->addVars({ 'FileName' => $FileName });
            $fh->close;
        },
    );

    $fh->close or Error('FILE_CLOSE_FAILED', {
        'Mode'     => $OpenMode,
        'FileName' => $FileName,
        'Code'     => $!
    });

    return $self;
}

#========================================================================================
# function     setLinesCount
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setLinesCount;
#----------------------------------------------------------------------------------------
# description  counts the lines of current fileHandle by reading the file
#               save the result to get it via function linesCount
#========================================================================================
sub setLinesCount {
    my $self = shift;

    my $fh = $self->fileHandle;
    $fh->seek(0, 0); # revert to start of file

    # ignore encoding warnings
    local $SIG{__WARN__} = sub {;};

    # try to read the header line
    my $converter = $self->converter;
    my $hFields = {};

    # count the data lines
    my $LinesCount = 0;
    while ($hFields) {
        $hFields = eval {
            $converter->getline($fh)
        };
        if (ExistsError()) {
            my $Error = GetError();

            if ($Error->code eq 'GET_LINE_FAILED') {
                Error('JSON_READ_LINE_FAILED', {
                    'FileName' => $self->fileName,
                    'Code'     => $Error->vars->{'Message'},
                    'Line'     => $LinesCount + 1,
                });
            }
            elsif ($Error->code eq 'NON_PRINTABLE_CHARS') {
                Error('JSON_READ_LINE_FAILED', {
                    'FileName' => $self->fileName,
                    'Code'     => 'contains characters ' . $Error->vars->{'Chars'},
                    'Line'     => $LinesCount + 1
                });
            }
            else {
                Error($Error);
            }
        }

        last unless defined $hFields;
        $LinesCount++;
    }

    $self->{'LinesCount'} = $LinesCount;
    return;
}

#========================================================================================
# function     linesCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->linesCount;
#----------------------------------------------------------------------------------------
# description  return the count of lines of current fileHandle
#----------------------------------------------------------------------------------------
# return       $LinesCount | set by setLinesCount | integer
#========================================================================================
sub linesCount {return shift->{'LinesCount'};}

#========================================================================================
# function     fileHandle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileHandle;
#----------------------------------------------------------------------------------------
# description  return the current fileHandle
#----------------------------------------------------------------------------------------
# return       $FileHandle | setted by exportJSON | filehandle
#========================================================================================
sub fileHandle {return shift->{'FileHandle'};}

#========================================================================================
# function     dirName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->dirName;
# syntax       $self->dirName('/opt/eproot/Shared/Patches/');
#----------------------------------------------------------------------------------------
# description  returns or sets the base directory name to be used
#----------------------------------------------------------------------------------------
# input        $DirName | new directory name | string
# return       $DirName | directory name | string
#========================================================================================
sub dirName {
    my $self = shift;
    my ($DirName) = @_;
    if (@_ == 1) {
        $self->{DirName} = $DirName;
    }
    $self->{DirName} //= $self->defaultDirName();
    return $self->{DirName};
}

#========================================================================================
# function     baseFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->baseFileName;
# syntax       $self->fileName('filename.ext');
#----------------------------------------------------------------------------------------
# description  returns or sets the base filename to be used
#----------------------------------------------------------------------------------------
# input        $FileName | new file name | string
# return       $FileName | file name | string
#========================================================================================
sub baseFileName {
    my $self = shift;
    my ($BaseFileName) = @_;
    if (@_ == 1) {
        $self->{BaseFileName} = $BaseFileName;
    }
    $self->{BaseFileName} //= $self->defaultBaseFileName();
    return $self->{BaseFileName};
}

#========================================================================================
# function     defaultBaseFileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->defaultBaseFileName;
#----------------------------------------------------------------------------------------
# description  returns the default json filename
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub defaultBaseFileName {
    return 'export.json';
}

#========================================================================================
# function     defaultDirName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->defaultDirName;
#----------------------------------------------------------------------------------------
# description  returns the default directory
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub defaultDirName {
    return '.';
}

#========================================================================================
# function     fileName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->fileName;
#----------------------------------------------------------------------------------------
# description  return the full path to the filename
#----------------------------------------------------------------------------------------
# return       $FileName | file name | string
#========================================================================================
sub fileName {
    my $self = shift;
    return catfile($self->dirName(), $self->baseFileName());
}

#========================================================================================
# function     converter
# state        public
#----------------------------------------------------------------------------------------
# syntax       $converter = $self->converter;
#----------------------------------------------------------------------------------------
# description  return the current json converter
#----------------------------------------------------------------------------------------
# return       $converter
#========================================================================================
sub converter {return shift->{'Converter'};}

#========================================================================================
# function     encodeFrom
# state        public
#----------------------------------------------------------------------------------------
# syntax       encodeFrom();
# example      $Driver->encodeFrom();
#----------------------------------------------------------------------------------------
# description  returns the target encoding (was set in <function exportJSON>)
#----------------------------------------------------------------------------------------
# return       $Encoding | target encoding (utf8,iso-8859-1,...) | string
#========================================================================================
sub encodeFrom {return shift->{'Encoding'};}

#========================================================================================
# function     incLineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->incLineNumber;
#----------------------------------------------------------------------------------------
# description  increments the line number and returns the new number
#----------------------------------------------------------------------------------------
# return       $linenumber | current line number | integer
#========================================================================================
sub incLineNumber {return ++shift->{'LineNumber'};}

#========================================================================================
# function     lineNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       $linenumber = $self->lineNumber;
#----------------------------------------------------------------------------------------
# description  return the current linenumber
#----------------------------------------------------------------------------------------
# return       $linenumber | current line number | integer
#========================================================================================
sub lineNumber {return shift->{'LineNumber'};}

#========================================================================================
# function     readLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aLineElements = $self->readLine;
#----------------------------------------------------------------------------------------
# description  imports a line ( reads and converts from the filehandle )
#----------------------------------------------------------------------------------------
# return       $hFields | hash with values for that line | ref.hash.string
#========================================================================================
sub readLine {
    my $self = shift;

    local $SIG{__WARN__} = sub {
        my ($Message) = @_;
        $Message =~ s/ at \S+ line \d+\b.*//; # remove code line
        Error('JSON_READ_LINE_FAILED', {
            'FileName' => $self->fileName,
            'Code'     => $Message,
            'Line'     => $self->lineNumber,
        });
    };

    my $hFields = eval {$self->converter->getline($self->fileHandle)};
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'GET_LINE_FAILED') {
            Error('JSON_READ_LINE_FAILED', {
                'FileName' => $self->fileName,
                'Code'     => $Error->vars->{'Message'},
                'Line'     => $self->lineNumber
            })
        };
        Error($Error);
    }

    return undef unless defined $hFields;

    $self->incLineNumber;
    return $hFields;
}

#========================================================================================
# function     importLine
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importLine();
#----------------------------------------------------------------------------------------
# description  imports one data line, get the line element from function topID('Item')
#               this is only a dummy funtion here.
#========================================================================================
sub importLine {
    my $self = shift;

    # add item to database
    return;
}

#========================================================================================
# function     readLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->readLines();
#----------------------------------------------------------------------------------------
# description  reads the content lines via function readLine,
#               calls for each line function importLine
#========================================================================================
sub readLines {
    my $self = shift;
    while (my $hFields = $self->readLine) {
        $self->pushID('Item', $hFields);
        $self->importLine();
        $self->popID('Item');
    }
    return;
}

#========================================================================================
# function     drive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->drive();
#----------------------------------------------------------------------------------------
# description  read the data content lines
#========================================================================================
sub drive {
    my $self = shift;

    $self->readLines();
    return;
}

#========================================================================================
# function     parse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->parse();
#----------------------------------------------------------------------------------------
# description  just calls function drive
#========================================================================================
sub parse {
    my $self = shift;

    $self->drive();
    return;
}

#========================================================================================
# function     pushID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->pushID( $Key, $Value );
# example      $self->pushID( 'CompanyID', $CompanyID );
#----------------------------------------------------------------------------------------
# description  adds a parameter to the parameter stack. Any previous parameter with the
#               same name will be hidden until $self->popID( $Key ) is called.
#               Use $self->topID( $Key ) to get the parameter value.
#               Use $self->popID( $Key ) to remove the parameter from the stack.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# input        $Value | parameter value
#========================================================================================
sub pushID {
    my $self = shift;
    my ($Key, $Value) = @_;

    $self->{'IDs'}->{$Key} = [] unless exists $self->{'IDs'}->{$Key};
    push @{$self->{'IDs'}->{$Key}}, $Value;
}

#========================================================================================
# function     popID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->popID( $Key );
# example      $CompanyID = $self->popID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  removes a parameter from the parameter stack and returns its value.
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub popID {
    my $self = shift;
    my ($Key) = @_;

    return pop @{$self->{'IDs'}->{$Key}};
}

#========================================================================================
# function     topID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $self->topID( $Key );
# example      $CompanyID = $self->topID( 'CompanyID' );
#----------------------------------------------------------------------------------------
# description  returns the current value of a parameter
#----------------------------------------------------------------------------------------
# input        $Key | parameter name | string
# return       $Value | parameter value
#========================================================================================
sub topID {
    my $self = shift;
    my ($Key) = @_;

    return() unless exists $self->{'IDs'}->{$Key};
    return $self->{'IDs'}->{$Key}->[-1];
}

1;
