#========================================================================================
# package      DE_EPAGES::ShopMigration::API::ResetShopHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the functions to reset a base shop,
#               this means to remove obsolete data while migrating to now.
#========================================================================================
package DE_EPAGES::ShopMigration::API::ResetShopHandler;
use base Exporter;
use strict;
use File::Spec::Functions;

use DE_EPAGES::Cartridge::API::Table::Cartridge qw(GetAllInfoCartridges);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Core::API::File qw(ExistsFile DeleteFile);
use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::TestParameter qw(TestObject);
use DE_EPAGES::Design::API::Cartridge; ##no critic (RequireCorrectImports)
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);
use DE_EPAGES::GoogleAdWords::API::Table::GoogleAdWordsCode; ##no critic (RequireCorrectImports) # qw (OnDeleteShop)
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias LoadObject ExistsObject Transaction ResetFactoryCaches DeleteObject LoadRootObject);
use DE_EPAGES::Object::API::Table::BooleanAttribute qw(DeleteBooleanAttribute GetPKeyBooleanAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::DateTimeAttribute qw(DeleteDateTimeAttribute GetPKeyDateTimeAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::FloatAttribute qw(DeleteFloatAttribute GetPKeyFloatAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::IntegerAttribute qw(DeleteIntegerAttribute GetPKeyIntegerAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::MoneyAttribute qw(DeleteMoneyAttribute GetPKeyMoneyAttributesByAttributeID);
use DE_EPAGES::Object::API::Table::StringAttribute qw(DeleteStringAttribute GetPKeyStringAttributesByAttributeID);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ProductRecommendation::API::Table::ProductRecommendationUser;                                              ##no critic (RequireCorrectImports) # qw (OnDeleteShop)
use DE_EPAGES::CrossSelling::API::Table::AutoCrossSelling qw(GetAllInfoAutoCrossSellingsByShopID DeleteAutoCrossSelling); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::Cartridge;
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);
use DE_EPAGES::ShopMigration::API::ProductsCommon qw(GetResetProductsIterator);
use DE_EPAGES::ShoppingList::API::Constants qw(UNITY_SHOPPINGLIST_ALIAS); ##no critic (RequireCorrectImports)
use Time::HiRes qw (time);

our @EXPORT_OK = qw(
    ResetBaseShop
    UnsetLegacyAddressValues
);

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'ResetShop_Unknown_AttributeType' => 'Attribute (#AttributePath) has unknown type (#AttributeType) or package (#AttributePackage).',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     ResetBaseShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       ResetBaseShop($Shop);
#----------------------------------------------------------------------------------------
# description  Removes all obsolete data of a base shop, when it is transferred to now:
#               - Imports files of Data/ResetBaseShop.d
#               - calls Reset<Cartridge> function for each installed cartridge, if defined
#----------------------------------------------------------------------------------------
# input        $Shop        | shop object  | object
#========================================================================================
sub ResetBaseShop {
    my ($Shop) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ResetShopHandler = __PACKAGE__->new($Shop);
    $ResetShopHandler->resetShopDataPerCartridge();
    # necessary because we do direct SQL Updates
    ResetFactoryCaches();
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Shop::API::Object::Shop->new( $Shop );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# input        $Shop | $Shop object | object
# return       $self | self | DE_EPAGES::ShopMigration::API::ResetShopHandler
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop) = @_;

    my $Cartridge = DE_EPAGES::ShopMigration::API::Cartridge->new;
    my $XMLDirectory = catdir($ENV{EPAGES_CARTRIDGES}, $Cartridge->cartridgeDirectory, 'Data', 'ResetBaseShop.d');

    my $self = bless {
        'Shop'         => $Shop,
        'XMLDirectory' => $XMLDirectory,
    }, $class;
    return $self;
}

#========================================================================================
# function     resetShopDataPerCartridge
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ResetShopHandler->resetShopDataPerCartridge();
#----------------------------------------------------------------------------------------
# description  calls
#               - function resetCartridgeData foreach installed Cartridge
#========================================================================================
sub resetShopDataPerCartridge {
    my $self = shift;

    GetLog->debug('resetShopDataPerCartridge for shop ' . $self->shopAlias);
    # get all installed Catridges of DE_EPAGES
    my @CartridgePackages = map {$_->{'Package'}} grep {$_->{'InstallState'} == 2} @{GetAllInfoCartridges()};
    foreach my $Package (reverse @CartridgePackages) {
        my ($Company, $Cartridge) = split(/::/, $Package);
        next unless $Company eq 'DE_EPAGES';
        $self->resetCartridgeData($Cartridge);
    }
    return;
}

#========================================================================================
# function     resetCartridgeData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ResetShopHandler->resetCartridgeData($Cartridge);
#----------------------------------------------------------------------------------------
# description  calls
#               - function importResetBaseShopCartridgeImportFile
#               - function Reset<Cartridge> if exists
#----------------------------------------------------------------------------------------
# input        $Cartridge  | cartridge name | string
#========================================================================================
sub resetCartridgeData {
    my $self = shift;
    my ($Cartridge) = @_;

    #import xml file from the directory ShopMigration/Data/ResetBaseShop.d if exists for cartridge
    $self->importResetBaseShopCartridgeImportFile($Cartridge);

    #call special subs for this cartridge
    my $ResetSubName = 'Reset' . $Cartridge;
    my $Sub = $self->can($ResetSubName);
    if ($Sub) {
        GetLog->debug("call $ResetSubName for shop " . $self->shopAlias);
        $Sub->($self->shop);
    }
    return;
}

#========================================================================================
# function     importResetBaseShopCartridgeImportFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ResetShopHandler->importResetBaseShopCartridgeImportFile($Cartridge);
#----------------------------------------------------------------------------------------
# description  imports <Cartridge>.xml from $ResetShopHandler->xmlDirectory
#----------------------------------------------------------------------------------------
# input        $Cartridge  | cartridge name | string
#========================================================================================
sub importResetBaseShopCartridgeImportFile {
    my $self = shift;
    my ($Cartridge) = @_;

    my $FileName = catfile($self->xmlDirectory, $Cartridge . '.xml');
    my $Shop = $self->shop;
    if (ExistsFile($FileName)) {
        GetLog->debug("import $FileName for shop " . $self->shopAlias);
        ImportObjects($FileName, $Shop->pathString);
    }
    return;
}

#========================================================================================
# function     ResetShop
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetShop( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Shop:
#               - delete all admin users except supermerchant
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetShop {
    my ($Shop) = @_;

    my $aMerchants = $Shop->child('Groups')->child('Merchants')->directMembers;
    foreach my $Merchant (@$aMerchants) {
        if ($Merchant->get('IsSuperMerchant')) {
            #delete all widgets (except SalesVolumeUnity)
            my $WidgetFolder = $Merchant->folder('MBOWidgets');
            foreach my $Widget (@{$WidgetFolder->children}) {
                $Widget->delete() unless $Widget->alias eq 'SalesVolumeUnity';
            }
        } else {
            #delete all admin users except supermerchant
            $Merchant->delete;
        }
    }
    return;
}

#========================================================================================
# function     ResetShopMailType
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetShopMailType( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ShopMailType:
#               - delete obsolete shopmailtype NewPassword
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetShopMailType {
    my ($Shop) = @_;

    # delete obsolete shopmailtype NewPassword
    _deleteShopMailType($Shop, 'NewPassword');
    return;
}

#========================================================================================
# function     ResetCrossSelling
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetCrossSelling( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge CrossSelling:
#               - remove objects of class ProductComparisons and Accessories of this shop
#               - remove AutoCrossSellings of this shop
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetCrossSelling {
    my ($Shop) = @_;

    #remove ProductComparisons and Accessories of this shop
    foreach my $ClassAlias (qw(ProductComparison Accessory)) {
        _deleteShopObjectsbyClassAlias($Shop, $ClassAlias);
    }
    #remove AutoCrossSellings of this shop
    my $Iterator = GetAllInfoAutoCrossSellingsByShopID($Shop->id);
    while (my $hAutoCrossSelling = <$Iterator>) {
        DeleteAutoCrossSelling($hAutoCrossSelling);
    }
    return;
}

#========================================================================================
# function     ResetDesign
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetDesign( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Design:
#               - delete all styles which are not default
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetDesign {
    my ($Shop) = @_;

    my $DefaultStyle= $Shop->get('DefaultStyle');
    my $aStyles = $Shop->get('Styles');
    foreach my $Style (@$aStyles) {
        #delete all styles which are not default
        $Style->delete unless $Style->id == $DefaultStyle->id;
    }
    #rename style, if it is MyDesign which is default for now shops
    $DefaultStyle->set({'Alias' => 'MyDesign_Base' }) if($DefaultStyle->alias eq 'MyDesign');

    #import MyDesign Style and set it to new default
    my $Cartridge = DE_EPAGES::Design::API::Cartridge->new;
    my $XMLDirectory = catdir($ENV{EPAGES_CARTRIDGES}, $Cartridge->cartridgeDirectory, 'Database', 'XML');
    my $FileName = catfile($XMLDirectory, 'DefaultShop.xml');
    ImportObjects($FileName, $Shop->pathString);
    return;
}

#========================================================================================
# function     ResetProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetProduct( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Product:
#               - delete BusinessHours
#               - delete obsolete shopmailtypes
#                 AppointmentAccepted, AppointmentCancelled, AppointmentCreated,
#                 DownloadActivated, ProductAvailibityNotification, ProductInquiry
#               - set default NextProductNo if it is not set
#               - transform product attributes to LocalizedString, PreDefLocalizedString
#               - delete product attributes of type PreDefMultiString, PreDefMultiLocalizedString, PreDefCustomerString, File, LocalizedFile
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetProduct {
    my ($Shop) = @_;

    # delete BusinessHours
    $Shop->businessHours->removeBookingPeriods;

    # delete obsolete shopmailtypes AppointmentAccepted, AppointmentCancelled, AppointmentCreated, DownloadActivated, ProductAvailibityNotification, ProductInquiry
    foreach my $MailTypeAlias (qw(AppointmentAccepted AppointmentCancelled AppointmentCreated DownloadActivated ProductAvailibityNotification ProductInquiry)) {
        _deleteShopMailType($Shop, $MailTypeAlias);
    }

    #set default NextProductNo if it is not set
    $Shop->set({ 'NextProductNo' => 1001 }) unless defined $Shop->get('NextProductNo');

    #transform product attributes to LocalizedString, PreDefLocalizedString
    #delete product attributes of type PreDefMultiString, PreDefMultiLocalizedString, PreDefCustomerString, File, LocalizedFile
    _transformProductAttributes($Shop);

    # delete appointments, bundle, customizable and download products
    my $Iterator = GetResetProductsIterator($Shop);
    while (my $ObjectID = <$Iterator>) { DeleteObject($ObjectID); }

    # also delete appointments folder
    $Shop->child('Appointments')->delete if $Shop->existsChild('Appointments');

    return;
}

#========================================================================================
# function     ResetCatalog
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetCatalog( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Catalog:
#               - delete objects with class CategorySpecialOffer
#               - reset startpage attributes (IntroIsVisible, IntroContent)
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetCatalog {
    my ($Shop) = @_;

    # delete objects with class CategorySpecialOffer
    _deleteShopObjectsbyClassAlias($Shop, 'CategorySpecialOffer');

    #reset startpage attributes:
    my $StartPageClass = LoadClassByAlias('StartPage');
    my $aStartPageIDs = $StartPageClass->findObjects('BySiteIDClassID', [ $Shop->id, $StartPageClass->id ]);
    foreach my $StartPageID (@$aStartPageIDs) {
        next unless ExistsObject($StartPageID);
        my $StartPage = LoadObject($StartPageID);
        $StartPage->set({ 'IntroIsVisible' => 0, 'IntroContent' => undef });
    }
    return;
}

#========================================================================================
# function     ResetCustomer
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetCustomer( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Customer:
#               - delete obsolete shopmailtypes
#                 UserAddressChanged, UserByMerchantWelcome, UserEmailChanged
#               - delete secondary user addresses
#               - delete customer groups if prepshop does not have feature
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetCustomer {
    my ($Shop) = @_;

    # delete obsolete shopmailtypes UserAddressChanged, UserByMerchantWelcome, UserEmailChanged
    foreach my $MailTypeAlias (qw(UserAddressChanged UserByMerchantWelcome UserEmailChanged)) {
        _deleteShopMailType($Shop, $MailTypeAlias);
    }
    #delete secondary user addresses
    _deleteUnassignedUserAddresses($Shop);

    # unset legacy address attributes
    UnsetLegacyAddressValues($Shop);

    #delete customer groups if prepshop does not have 'CustomerGroups' feature
    my $PrepShop = $Shop->get('MigrationPrepShop');
    unless ($PrepShop->featureMaxValue('CustomerGroups')) {
        # delete customergroups in base shop
        LoadClassByAlias('CustomerGroup')->deleteObjects('BySiteID', [ $Shop->id ]);
    }

    # unset Comment on Customers
    _resetCommentOnCustomers($Shop);

    return;
}

#========================================================================================
# function     ResetProductRating
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetProductRating( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ProductRating:
#               - delete ProductRatings
#               - delete obsolete shopmailtypes
#                 NewProductRating, ProductRatingInvitation
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetProductRating {
    my ($Shop) = @_;

    # delete ProductRatings
    _deleteShopObjectsbyClassAlias($Shop, 'ProductRating');

    # delete obsolete shopmailtypes NewProductRating, ProductRatingInvitation
    foreach my $MailTypeAlias (qw(NewProductRating ProductRatingInvitation)) {
        _deleteShopMailType($Shop, $MailTypeAlias);
    }
    return;
}

#========================================================================================
# function     ResetProductRecommendation
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetProductRecommendation( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ProductRecommendation:
#               - delete ProductRecommendations
#               - delete obsolete shopmailtype ProductRecommendation
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetProductRecommendation {
    my ($Shop) = @_;

    # delete ProductRecommendations
    DE_EPAGES::ProductRecommendation::API::Table::ProductRecommendationUser::OnDeleteShop({ 'ShopID' => $Shop->id });

    # delete obsolete shopmailtype ProductRecommendation
    _deleteShopMailType($Shop, 'ProductRecommendation');
    return;
}

#========================================================================================
# function     ResetGoogleAdWords
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetGoogleAdWords( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge GoogleAdWords:
#               - delete GoogleAdWordsCode
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetGoogleAdWords {
    my ($Shop) = @_;

    # delete GoogleAdWordsCode
    DE_EPAGES::GoogleAdWords::API::Table::GoogleAdWordsCode::OnDeleteShop({ 'ShopID' => $Shop->id });
    return;
}

#========================================================================================
# function     ResetSafeBuy
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetSafeBuy( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge SafeBuy:
#               - delete SafeBuyShield
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetSafeBuy {
    my ($Shop) = @_;
    # delete SafeBuyShield
    my $SafeBuyShield = $Shop->get('SafeBuyShield');
    if ($SafeBuyShield) {
        $Shop->set({ 'SafeBuyShield' => undef });
        DeleteFile($Shop->get('PublicPath') . '/' . $SafeBuyShield);
    }
    return;
}

#========================================================================================
# function     ResetEkomi
# state        private
#----------------------------------------------------------------------------------------
# syntax       ResetEkomi( $Shop );
#----------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Ekomi:
#               - delete obsolete shopmailtype EkomiRating
#----------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#----------------------------------------------------------------------------------------
sub ResetEkomi {
    my ($Shop) = @_;

    # delete obsolete shopmailtype EkomiRating
    _deleteShopMailType($Shop, 'EkomiRating');
    return;
}

#===============================================================================================
# function     ResetShoppingList
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetShoppingList( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ShoppingList:
#               - clean up obsolete shoppinglists, rename last one
#               each user should have only one shopping list with alias UNITY_SHOPPINGLIST_ALIAS
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetShoppingList {
    my ($Shop) = @_;

    my $Class = LoadClassByAlias('ShoppingList');
    my $aShoppingListIDs = $Class->findObjects('ByShopID', [ $Shop->id ]);
    #clean up obsolete shoppinglists, rename last one
    foreach my $ShoppingListID (@$aShoppingListIDs) {
        next unless ExistsObject($ShoppingListID);
        my $ShoppingList = LoadObject($ShoppingListID);
        my $User = $ShoppingList->get('User');
        my $aUserShoppingLists = [ sort {$b->get('ChangeDate') <=> $a->get('ChangeDate')} @{$Class->getObjects('ByUserID', [ $User->id ])} ];

        my $First = 1;
        #each user should have only one shopping list with alias UNITY_SHOPPINGLIST_ALIAS
        foreach my $UserShoppingList (@$aUserShoppingLists) {
            if ($First == 1) {
                $UserShoppingList->set({ 'Alias' => UNITY_SHOPPINGLIST_ALIAS })
            } else {
                $UserShoppingList->delete;
            }
            $First++;
        }
    }
    return;
}

#===============================================================================================
# function     ResetShippingOption
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetShippingOption( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ShippingOption:
#               - delete ShippingOptions
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetShippingOption {
    my ($Shop) = @_;

    # delete shippingoptions
    _deleteShopObjectsbyClassAlias($Shop, 'ShippingOption');
    return;
}

#===============================================================================================
# function     ResetShipping
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetShipping( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Shipping:
#               - delete customergroups from shippingmethods
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetShipping {
    my ($Shop) = @_;

    #delete customergroups from shippingmethods
    my $aShippingMethods = $Shop->get('ShippingMethods');
    foreach my $ShippingMethod (@$aShippingMethods) {
        #only shippingmethods with assigned CustomerGroups
        next unless defined $ShippingMethod->get('CustomerGroups');
        $ShippingMethod->set({'CustomerGroups' => undef});
    }
    return;
}


#===============================================================================================
# function     ResetPriceList
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetPriceList( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge PriceList:
#               - delete pricelists
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetPriceList {
    my ($Shop) = @_;
    # delete pricelists
    LoadClassByAlias('PriceList')->deleteObjects('BySiteID', [ $Shop->id ]);
    return;
}

#===============================================================================================
# function     ResetCoupon
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetCoupon( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Coupon:
#               - delete CouponCampaigns
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetCoupon {
    my ($Shop) = @_;

    # delete couponcampaigns
    _deleteShopObjectsbyClassAlias($Shop, 'CouponCampaign');
    return;
}

#===============================================================================================
# function     ResetShopNewsletter
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetShopNewsletter( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ShopNewsletter:
#               - delete NewsletterCampaigns
#               - delete obsolete shopmailtypes:
#                 SubscribeConfirmation, SubscribeConfirmationRequest,
#                 UnsubscribeConfirmation, UnsubscribeConfirmationRequest
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetShopNewsletter {
    my ($Shop) = @_;

    # delete newslettercampaigns
    _deleteShopObjectsbyClassAlias($Shop, 'NewsletterCampaign');

    # delete obsolete shopmailtypes SubscribeConfirmation, SubscribeConfirmationRequest, UnsubscribeConfirmation, UnsubscribeConfirmationRequest
    foreach my $MailTypeAlias (qw(SubscribeConfirmation SubscribeConfirmationRequest UnsubscribeConfirmation UnsubscribeConfirmationRequest)) {
        _deleteShopMailType($Shop, $MailTypeAlias);
    }
    return;
}

#===============================================================================================
# function     ResetCreditCheck
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetCreditCheck( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge CreditCheck:
#               - delete folders CreditCheckProviderConfigs, CreditCheckAgencyConfigs
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetCreditCheck {
    my ($Shop) = @_;
    $Shop->child('CreditCheckProviderConfigs')->delete if $Shop->existsChild('CreditCheckProviderConfigs');
    $Shop->child('CreditCheckAgencyConfigs')->delete if $Shop->existsChild('CreditCheckAgencyConfigs');
    return;
}

#===============================================================================================
# function     ResetSearch
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetSearch( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Search:
#               - delete SearchTerms
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetSearch {
    my ($Shop) = @_;

    # delete searchterms
    _deleteShopObjectsbyClassAlias($Shop, 'SearchTerm');
    return;
}

#===============================================================================================
# function     ResetMediaGallery
# state        private
#-----------------------------------------------------------------------------------------------
# syntax       ResetMediaGallery( $Shop );
#-----------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge MediaGallery:
#               - delete child MediaGallery
#-----------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------
sub ResetMediaGallery {
    my ($Shop) = @_;

    # delete MediaGallery
    if ($Shop->existsChild('MediaGallery')) {
        $Shop->child('MediaGallery')->delete;
        #this will be recrated by DE_EPAGES/MediaGallery/API/Attributes/Shop.pm:16 but is now empty
    }
    return;
}

#=================================================================================================================
# function     ResetContent
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetContent( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Content:
#               - delete objects with class Link, Article or SiteMap
#               - legal pages will be deleted by DE_EPAGES/Content/Hooks/Shop.pm:61 when IsUnityRegistered is set
#               - reset ReadPermissionFor attribute to Everyone
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetContent {
    my ($Shop) = @_;

    #delete objects with class Link, Article or SiteMap
    foreach my $ClassAlias (qw(Link Article SiteMap)) {
        _deleteShopObjectsbyClassAlias($Shop, $ClassAlias);
    }
    #delete legal pages (Imprint TermsAndConditions PrivacyPolicy CustomerInformation ShippingAndPaymentInformation)
    #this will be deleted by DE_EPAGES/Content/Hooks/Shop.pm:61 when IsUnityRegistered is set

    my @Categories = ($Shop->get('Categories'), @{$Shop->get('Categories')->get('RecursiveSubCategories')});
    my $EveryoneID = LoadRootObject()->child('Groups')->child('Everyone')->id();
    foreach my $Category (@Categories) {
        $Category->set({'ReadPermissionFor' => $EveryoneID});
    }
    return;
}

#=================================================================================================================
# function     ResetImageGallery
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetImageGallery( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge ImageGallery:
#               - delete objects with class ImageGallery
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetImageGallery {
    my ($Shop) = @_;
    # delete ImageGallery
    _deleteShopObjectsbyClassAlias($Shop, 'ImageGallery');
    return;
}

#=================================================================================================================
# function     ResetForum
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetForum( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Forum:
#               - delete objects with class Forum, GuestBook or Blog
#               - delete obsolete shopmailtype NewPost
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetForum {
    my ($Shop) = @_;

    #delete objects with class Forum, GuestBook or Blog
    foreach my $ClassAlias (qw(Forum GuestBook Blog)) {
        _deleteShopObjectsbyClassAlias($Shop, $ClassAlias);
    }
    # delete obsolete shopmailtype NewPost
    _deleteShopMailType($Shop, 'NewPost');
    return;
}

#=================================================================================================================
# function     ResetCustomForm
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetCustomForm( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge CustomForm:
#               - delete CustomForms
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetCustomForm {
    my ($Shop) = @_;

    # delete CustomForms
    _deleteShopObjectsbyClassAlias($Shop, 'CustomForm');
    return;
}

#=================================================================================================================
# function     ResetEtracker
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetEtracker( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Etracker:
#               - set EtrackerURLExtension to undef at all portalconfigs
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetEtracker {
    my ($Shop) = @_;
    #set EtrackerURLExtension to undef at all portalconfigs
    foreach my $PortalConfig (@{$Shop->get('PortalConfigs')}) {
        $PortalConfig->set({ 'EtrackerURLExtension' => undef });
    }
    return;
}

#=================================================================================================================
# function     ResetPayment
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPayment( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Payment:
#               - delete InterestRateMatrices of shop
#               - set IsHirePurchaseEnabled=0 foreach paymentmethod
#               - remove customergroups from paymentmethod
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPayment {
    my ($Shop) = @_;

    # delete InterestRateMatrices of shop
    _deleteShopObjectsbyClassAlias($Shop, 'InterestRateMatrix');

    foreach my $PaymentMethod (@{$Shop->get('PaymentMethods')}) {
        # set IsHirePurchaseEnabled=0 foreach paymentmethod
        $PaymentMethod->set({ 'IsHirePurchaseEnabled' => 0 });
        # remove customergroups from paymentmethods
        next unless defined $PaymentMethod->get('CustomerGroups');
        $PaymentMethod->set({'CustomerGroups' => undef});
    }
    return;
}

#=================================================================================================================
# function     ResetPaymentTypes
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPaymentTypes( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge PaymentTypes:
#               - delete payments of type FixPrice,FreeLimit,DirectDebitViaFax,DirectCreditCard
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPaymentTypes {
    my ($Shop) = @_;

    # delete payments of type FixPrice,FreeLimit,DirectDebitViaFax,DirectCreditCard
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(FixPrice FreeLimit DirectDebitViaFax DirectCreditCard) ]);
    return;
}

#=================================================================================================================
# function     ResetAmazonPayments
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetAmazonPayments( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge AmazonPayments:
#               - delete obsolete shopmailtype AmazonPaymentsPaymentDeclined
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetAmazonPayments {
    my ($Shop) = @_;

    # delete obsolete shopmailtype AmazonPaymentsPaymentDeclined
    _deleteShopMailType($Shop, 'AmazonPaymentsPaymentDeclined');
    return;
}

#=================================================================================================================
# function     ResetMoneybookers
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetMoneybookers( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Moneybookers:
#               - delete payments of type MoneybookersCreditCard
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetMoneybookers {
    my ($Shop) = @_;

    # delete payments of type MoneybookersCreditCard
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(MoneybookersCreditCard) ]);
    return;
}

#=================================================================================================================
# function     ResetOgone
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetOgone( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Ogone:
#               - delete payments of type Acceptance
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetOgone {
    my ($Shop) = @_;

    # delete payments of type Acceptance
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(Acceptance) ]);
    return;
}

#=================================================================================================================
# function     ResetPayPal
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPayPal( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge PayPal:
#               - delete payments of type PayPal, PayPalIntegralEvolution
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPayPal {
    my ($Shop) = @_;

    # delete payments of type PayPal, PayPalIntegralEvolution
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(PayPal PayPalIntegralEvolution) ]);
    return;
}

#=================================================================================================================
# function     ResetPayPalPro
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPayPalPro( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge PayPalPro:
#               - delete payments of type PayPalPro
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPayPalPro {
    my ($Shop) = @_;

    # delete payments of type PayPalPro
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(PayPalPro) ]);
    return;
}

#=================================================================================================================
# function     ResetWorldPay
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetWorldPay( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge WorldPay:
#               - delete payments of type WorldPay
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetWorldPay {
    my ($Shop) = @_;

    # delete payments of type WorldPay
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(WorldPay) ]);
    return;
}

#=================================================================================================================
# function     ResetDocdataPay
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetDocdataPay( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge DocdataPay:
#               - delete payments of type DocdataPay
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetDocdataPay {
    my ($Shop) = @_;

    # delete payments of type DocdataPay
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(DocdataPay) ]);
    return;
}

#=================================================================================================================
# function     ResetKlarna
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetKlarna( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Klarna:
#               - delete payments of type KlarnaFinancing KlarnaHirePurchase KlarnaInvoice
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetKlarna {
    my ($Shop) = @_;

    # delete payments of type KlarnaFinancing, KlarnaHirePurchase, KlarnaInvoice
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(KlarnaFinancing KlarnaHirePurchase KlarnaInvoice) ]);
    return;
}

#=================================================================================================================
# function     ResetCyberMUT
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetCyberMUT( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge CyberMUT:
#               - delete payments of type CyberMUT
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetCyberMUT {
    my ($Shop) = @_;

    # delete payments of type CyberMUT
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(CyberMUT) ]);
    return;
}

#=================================================================================================================
# function     ResetSPSBillingOnline
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetSPSBillingOnline( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge SPSBillingOnline:
#               - delete payments of type SPSBillingOnline
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetSPSBillingOnline {
    my ($Shop) = @_;

    # delete payments of type SPSBillingOnline
    _deleteShopPaymentMethodsByPaymentTypeAliases($Shop, [ qw(SPSBillingOnline) ]);
    return;

}

#=================================================================================================================
# function     ResetDHL
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetDHL( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge DHL:
#               - delete shipping methods with shippingfulfillmenttype DHL
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetDHL {
    my ($Shop) = @_;

    # delete shipping methods with shippingfulfillmenttype DHL
    _deleteShopShippingMethodsByFullfillmentType($Shop, 'DHL');
    return;
}

#=================================================================================================================
# function     ResetSDA
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetSDA( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge SDA:
#               - delete shipping methods with shippingfulfillmenttype SDA
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetSDA {
    my ($Shop) = @_;

    # delete shipping methods with shippingfulfillmenttype SDA
    _deleteShopShippingMethodsByFullfillmentType($Shop, 'SDA');
    return;
}

#=================================================================================================================
# function     ResetSEUR
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetSEUR( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge SEUR:
#               - delete shipping methods with shippingfulfillmenttype SEUR
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetSEUR {
    my ($Shop) = @_;

    # delete shipping methods with shippingfulfillmenttype SEUR
    _deleteShopShippingMethodsByFullfillmentType($Shop, 'SEUR');
    return;
}

#=================================================================================================================
# function     ResetEnvialia
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetEnvialia( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Envialia:
#               - delete shipping methods with shippingfulfillmenttype Envialia
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetEnvialia {
    my ($Shop) = @_;

    # delete shipping methods with shippingfulfillmenttype Envialia
    _deleteShopShippingMethodsByFullfillmentType($Shop, 'Envialia');
    return;
}

#=================================================================================================================
# function     ResetKelkoo
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetKelkoo( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Kelkoo:
#               - delete portalconfig of type Kelkoo
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetKelkoo {
    my ($Shop) = @_;

    # delete portalconfig of type Kelkoo
    _deleteShopPortalConfigsByPortalAlias($Shop, 'KelkooConfig');
    return;
}

#=================================================================================================================
# function     ResetPangora
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPangora( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Pangora:
#               - delete portalconfig of type Pangora
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPangora {
    my ($Shop) = @_;

    # delete portalconfig of type Pangora
    _deleteShopPortalConfigsByPortalAlias($Shop, 'PangoraConfig');
    return;
}

#=================================================================================================================
# function     ResetPreissuchmaschine
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPreissuchmaschine( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Preissuchmaschine:
#               - delete portalconfig of type Preissuchmaschine
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPreissuchmaschine {
    my ($Shop) = @_;

    # delete portalconfig of type Preissuchmaschine
    _deleteShopPortalConfigsByPortalAlias($Shop, 'PreissuchmaschineConfig');
    return;
}

#=================================================================================================================
# function     ResetPriceMinister
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetPriceMinister( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge PriceMinister:
#               - delete portalconfig of type PriceMinister
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetPriceMinister {
    my ($Shop) = @_;

    # delete portalconfig of type PriceMinister
    _deleteShopPortalConfigsByPortalAlias($Shop, 'PriceMinisterConfig');
    return;

}

#=================================================================================================================
# function     ResetShopzilla
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetShopzilla( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Shopzilla:
#               - delete portalconfig of type Shopzilla
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetShopzilla {
    my ($Shop) = @_;

    # delete portalconfig of type Shopzilla
    _deleteShopPortalConfigsByPortalAlias($Shop, 'ShopzillaConfig');
    return;

}

#=================================================================================================================
# function     ResetToppreise
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetToppreise( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Toppreise:
#               - delete portalconfig of type Toppreise
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetToppreise {
    my ($Shop) = @_;

    # delete portalconfig of type Toppreise
    _deleteShopPortalConfigsByPortalAlias($Shop, 'ToppreiseConfig');
    return;

}

#=================================================================================================================
# function     ResetTwenga
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetTwenga( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Twenga:
#               - delete portalconfig of type Twenga
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetTwenga {
    my ($Shop) = @_;

    # delete portalconfig of type Twenga
    _deleteShopPortalConfigsByPortalAlias($Shop, 'TwengaConfig');
    return;

}

#=================================================================================================================
# function     ResetEbay
# state        private
#-----------------------------------------------------------------------------------------------------------------
# syntax       ResetEbay( $Shop );
#-----------------------------------------------------------------------------------------------------------------
# description  Reset base shop data for Cartridge Ebay:
#               - delete EbayUserTemplates
#               - transform EbayDefaultAuction
#-----------------------------------------------------------------------------------------------------------------
# input        $Shop | base shop object | object
#-----------------------------------------------------------------------------------------------------------------
sub ResetEbay {
    my ($Shop) = @_;

    # delete EbayUserTemplates
    my $aEbayShopTemplates = $Shop->get('EbayUserTemplates');
    foreach my $EbayShopTemplate (@$aEbayShopTemplates) {
        $EbayShopTemplate->delete;
    }
    # transform EbayDefaultAuction
    my $DefaultAuction = $Shop->get('EbayDefaultAuction');
    if (defined $DefaultAuction) {
        my $hResetInfo = {
            'IsBoldTitle'        => 0,
            'IsHighlight'        => 0,
            'IsBordered'         => 0,
            'IsFeatured'         => 0,
            'IsHomePageFeatured' => 0,
            'GalleryUrl'         => undef,
            'PictureUrl'         => undef
        };
        my $Gallery = $DefaultAuction->get('Gallery');
        $hResetInfo->{'Gallery'} = 'None' if $Gallery eq 'Featured';
        $DefaultAuction->set($hResetInfo);
    }
    return;
}

#helper functions
sub shop {
    my $self = shift;
    return $self->{'Shop'};
}

sub shopAlias {
    my $self = shift;
    return $self->{'Shop'}->alias;
}

sub xmlDirectory {
    my $self = shift;
    return $self->{'XMLDirectory'};

}

# delete shopmailtypes with given $MailTypeAlias
sub _deleteShopMailType {
    my ($Shop, $MailTypeAlias) = @_;

    #delete shopmailtypes with given $MailTypeAlias
    $Shop->child('MailTypes')->child($MailTypeAlias)->delete if $Shop->child('MailTypes')->existsChild($MailTypeAlias);
    return;
}

sub _deleteShopShippingMethodsByFullfillmentType {
    my ($Shop, $ShippingFullfillmentTypeAlias) = @_;

    # delete Shippingmethods with ShippingFullfillmentTypeAlias
    my $aShippingMethods = $Shop->get('ShippingMethods');
    foreach (grep {defined $_->get('SelectedShippingFulfillmentMethod')} @{$aShippingMethods}) {
        if ($_->get('SelectedShippingFulfillmentMethod')->get('ShippingFulfillmentType')->alias eq $ShippingFullfillmentTypeAlias) {
            $_->delete
        }
    }
    return;
}

sub _deleteShopPaymentMethodsByPaymentTypeAliases {
    my ($Shop, $aObsoletePaymentTypeAliases) = @_;

    # delete payments of paymenttypes with alias in $aObsoletePaymentTypeAliases
    my $aShopPaymentMethods = $Shop->get('PaymentMethods');
    foreach (grep {ElementOf($_->get('PaymentType')->alias, @$aObsoletePaymentTypeAliases)} @{$aShopPaymentMethods}) {
        $_->delete
    }
    return;
}

sub _deleteShopPortalConfigsByPortalAlias {
    my ($Shop, $PortalAlias) = @_;

    # delete portalconfig of type $PortalAlias
    my $aShopPortals = $Shop->get('PortalConfigs');
    foreach my $PortalConfig (@$aShopPortals) {
        $PortalConfig->delete if $PortalConfig->instanceOf($PortalAlias);
    }
    return;
}

sub _deleteShopObjectsbyClassAlias {
    my ($Shop, $ClassAlias) = @_;
    my $Class = LoadClassByAlias($ClassAlias);
    if ($Class->baseClass->alias eq 'BaseObject') {
        $Class->deleteObjects('BySiteID', [ $Shop->id ]);
    } else {
        $Class->deleteObjects('BySiteIDClassID', [ $Shop->id, $Class->id ]);
    }
    return;
}

sub _getUnassignedUserAddresses {
    my ($Shop) = @_;

    my $dbi = GetCurrentDBHandle();
    my $UserClass = LoadClassByAlias('User');
    my $BillingAddressAttributeID = $UserClass->attribute('BillingAddress')->id;
    my $ShippingAddressAttributeID = $UserClass->attribute('ShippingAddress')->id;

    #get user addresses which are not assigned as billing address and not as shipping address
    my @Parameter = ($Shop->id, $UserClass->id, $BillingAddressAttributeID, $ShippingAddressAttributeID);

    my $dbResult = $dbi->execute(<<END_SQL, \@Parameter, [ 'Integer' ]); #, [qw(AddressID)]);
SELECT addressid FROM address a
                          INNER JOIN object ao ON a.addressid=ao.objectid AND ao.siteid=?
                          INNER JOIN object pao ON pao.objectid = ao.parentid AND pao.alias='Addresses'
                          INNER JOIN object uo ON uo.objectid=pao.parentid AND uo.classid=?
WHERE NOT EXISTS (SELECT 1 FROM referenceattribute WHERE referencedobjectid=a.addressid AND attributeid=?)
  AND NOT EXISTS (SELECT 1 FROM referenceattribute WHERE referencedobjectid=a.addressid AND attributeid=?)
END_SQL
    return [ map {$_->[0]} @$dbResult ];
}

sub _deleteUnassignedUserAddresses {
    my ($Shop) = @_;

    my $aAddressIDs = _getUnassignedUserAddresses($Shop);
    foreach my $AddressID (@$aAddressIDs) {
        next unless ExistsObject($AddressID);
        my $Address = LoadObject($AddressID);
        $Address->delete;
    }
    return;
}

sub _resetCommentOnCustomers {
    my ($Shop) = @_;

    my $dbi = GetCurrentDBHandle();
    my $CustomerClass = LoadClassByAlias('Customer');
    my $CommentAttributeID = $CustomerClass->attribute('Comment')->id;
    my $ShopCustomerClassID = $Shop->get('CustomerClass')->id;
    my $SiteID = $Shop->id;
    my @Parameter = ($CommentAttributeID, $ShopCustomerClassID, $SiteID);
    my $dbResult = $dbi->execute(<<END_SQL, \@Parameter, [ 'Integer' ]); #, [qw(AddressID)]);
    SELECT objectid FROM stringattribute s
    WHERE attributeid=? AND objectid IN (
        SELECT objectid FROM object WHERE classid=? AND siteid=?
    )
END_SQL
    my $aCustomers = [ map {$_->[0]} @$dbResult ];
    foreach my $CustomerID (@{$aCustomers}) {
        my $Customer = LoadObject($CustomerID);
        $Customer->set({Comment => undef});
    }
    return;
}

sub _transformProductAttributes {
    my ($Shop) = @_;

    #transform product attributes to LocalizedString, PreDefLocalizedString
    my $ahLanguages = GetActiveShopLanguages($Shop);
    my @SupportedDefaultTypes = qw(String Integer Float Boolean Money DateTime Date Time);
    my @ProductTypes = ($Shop->get('ProductClass'), @{$Shop->get('ProductTypes')->children});
    foreach my $ProductType (@ProductTypes) {
        my $aAttributes = $ProductType->get('UserDefinedAttributes');
        foreach my $Attribute (@$aAttributes) {
            next if $Attribute->get('AttributeClass')->id != $ProductType->id;
            my $Type = $Attribute->get('Type');
            next if $Type eq 'LocalizedString' or $Type eq 'PreDefLocalizedString';
            my $Package = $Attribute->get('Package');
            if (ElementOf($Type, @SupportedDefaultTypes) and $Package eq 'DE_EPAGES::Object::API::Attributes::DefaultAttribute') {
                _convertDefaultToLocalizedStringAttribute($Attribute, $Type, $ahLanguages);
            } elsif ($Type eq 'PreDefString' and $Package eq 'DE_EPAGES::Object::API::Attributes::PreDefined') {
                _convertPreDefStringToPreDefLocalizedStringAttribute($Attribute, $ahLanguages);
            } elsif ($Type eq 'PreDefMultiString'
                or $Type eq 'PreDefMultiLocalizedString'
                or $Type eq 'PreDefCustomerString'
                or $Type eq 'File'
                or $Type eq 'LocalizedFile'
            ) {
                #delete product attributes of type PreDefMultiString, PreDefMultiLocalizedString, PreDefCustomerString, File, LocalizedFile
                $Attribute->delete;
            } else {
                Error('ResetShop_Unknown_AttributeType', {
                    'AttributePath'    => $Attribute->pathString,
                    'AttributeType'    => $Type,
                    'AttributePackage' => $Package
                });
            }
        }
    }
    return;
}

sub _convertDefaultToLocalizedStringAttribute {
    my ($Attribute, $Type, $ahLanguages) = @_;

    my $dbi = GetCurrentDBHandle();
    Transaction(
        sub {
            $Attribute->set({ 'Type' => 'LocalizedString', 'Package' => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute', 'IsCacheable' => 0 });
            if ($Type eq 'String') { ##no critic qw(ProhibitCascadingIfElse)
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, value
FROM stringattribute
where attributeid=?
END_SQL
                }
                DeleteStringAttribute($_) foreach @{GetPKeyStringAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Boolean') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, convert(value , CHAR(255))
FROM booleanattribute
where attributeid=?
END_SQL
                }
                DeleteBooleanAttribute($_) foreach @{GetPKeyBooleanAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Float') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, convert(value , CHAR(255))
FROM floatattribute
where attributeid=?
END_SQL
                }
                DeleteFloatAttribute($_) foreach @{GetPKeyFloatAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Integer') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, convert(value , CHAR(255))
FROM integerattribute
where attributeid=?
END_SQL
                }
                DeleteIntegerAttribute($_) foreach @{GetPKeyIntegerAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Money') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, convert(value , CHAR(255))
FROM moneyattribute
where attributeid=?
END_SQL
                }
                DeleteMoneyAttribute($_) foreach @{GetPKeyMoneyAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'DateTime') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, convert(value , CHAR(255))
FROM datetimeattribute
where attributeid=?
END_SQL
                }
                DeleteDateTimeAttribute($_) foreach @{GetPKeyDateTimeAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Date') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, DATE_FORMAT(value,GET_FORMAT(DATE,'ISO'))
FROM datetimeattribute
where attributeid=?
END_SQL
                }
                DeleteDateTimeAttribute($_) foreach @{GetPKeyDateTimeAttributesByAttributeID($Attribute->id)};
            }
            elsif ($Type eq 'Time') {
                #copy value foreach language to LocalizedString:
                foreach my $hLanguage (@$ahLanguages) {
                    $dbi->do(<<'END_SQL', [ $hLanguage->{'LanguageID'}, $Attribute->id ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, attributeid, ?, DATE_FORMAT(value,GET_FORMAT(TIME,'ISO'))
FROM datetimeattribute
where attributeid=?
END_SQL
                }
                DeleteDateTimeAttribute($_) foreach @{GetPKeyDateTimeAttributesByAttributeID($Attribute->id)};
            }
        }
    );
    return;
}

sub _convertPreDefStringToPreDefLocalizedStringAttribute {
    my ($Attribute, $ahLanguages) = @_;

    my $ValueStringAttributeID = LoadClassByAlias('PreDefAttribute')->attribute('ValueString')->id;
    my $ValueLocStringAttributeID = LoadClassByAlias('PreDefAttribute')->attribute('ValueLocString')->id;
    my $dbi = GetCurrentDBHandle();

    Transaction(
        sub {
            $Attribute->set({ 'Type' => 'PreDefLocalizedString' });
            if ($Attribute->existsChild('PreDefAttributes') && scalar @{$Attribute->child('PreDefAttributes')->children}) {
                foreach my $PreDefAttributeID (@{$Attribute->child('PreDefAttributes')->findChildren}) {
                    #copy value foreach language to LocalizedString:
                    foreach my $hLanguage (@$ahLanguages) {
                        $dbi->do(<<'END_SQL', [ $ValueLocStringAttributeID, $hLanguage->{'LanguageID'}, $PreDefAttributeID, $ValueStringAttributeID ]);
INSERT INTO localizedstringattribute (objectid, attributeid, languageid, value)
SELECT objectid, ?, ?, value
FROM stringattribute
where objectid=? and attributeid=?
END_SQL
                    }
                    DeleteStringAttribute($PreDefAttributeID, $ValueStringAttributeID);
                }
            }
        }
    );
    return;
}

#========================================================================================
# function     UnsetLegacyAddressValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       UnsetLegacyAddressValues($Shop);
#----------------------------------------------------------------------------------------
# description  unsets legacy fields on user/customer addresses that are no longer part
#               of UnityRegistrationSet (but where in RegistrationsSet)
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub UnsetLegacyAddressValues {
    my ($Shop) = @_;

    my $Counter = 0;
    my $ClassAddress = LoadClassByAlias('Address');
    my $aCustomers = $Shop->folder('Customers')->children;

    my $StartTime = time;
    GetLog->debug(sprintf('_unsetLegacyAddressValues started for shop %s', $Shop->alias));
    foreach my $Customer (@$aCustomers) {
        foreach my $Address (@{$Customer->folder('Addresses')->children}) {
            next unless $Address->instanceOf($ClassAddress);
            _unsetLegacyValuesForAddress($Address);
            $Counter++;
        }
        foreach my $User (@{$Customer->get('Users')}) {
            foreach my $Address (@{$User->folder('Addresses')->children}) {
                next unless $Address->instanceOf($ClassAddress);
                _unsetLegacyValuesForAddress($Address);
                $Counter++;
            }
        }
    }
    GetLog->debug(sprintf('updated %s addresses for shop %s took %.3f seconds',
        $Counter, $Shop->alias, time() - $StartTime));
}

sub _unsetLegacyValuesForAddress {
    my ($Address) = @_;

    $Address->set({
        AddressExtension  => undef,
        BankAccountHolder => undef,
        BankAccountNo     => undef,
        BankCode          => undef,
        BankName          => undef,
        CodePorte         => undef,
        Department        => undef,
        DisplayName       => undef,
        EMailBusiness     => undef,
        EMailPrivate      => undef,
        Fax               => undef,
        Gender            => undef,
        JobTitle          => undef,
        MiddleName        => undef,
        PhoneBusiness     => undef,
        PhonePrivate      => undef,
        Title             => undef,
        URL               => undef,
    });
    return;
}

1;
