#==============================================================================
# package      DE_EPAGES::ShopNewsletter::API::Newsletter
# state        public
# description  Returns template snippets for the newsletter.
#==============================================================================
package DE_EPAGES::ShopNewsletter::API::Newsletter;
use base Exporter;

use strict;
use DE_EPAGES::Core::API::Html qw (RemoveHtmlTags);
use DE_EPAGES::Core::API::Unicode qw(Unicode2Filename);
use DE_EPAGES::Core::API::Url qw(StringIsURL);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByAlias);
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage );
use DE_EPAGES::ShopNewsletter::DAL::Newsletter qw ( DAL_GetCampaignSubscribedCount DAL_GetCampaignUnsubscribedCount );

our @EXPORT_OK = qw(
    AddUnsubscribe
    InsertCouponSnippet
    AddProductsAndCatalogs
    AddShopLogo
    GetShopLogo
    GetSubscribeCount
    GetUnsubscribeCount
);

#========================================================================================
# function     AddUnsubscribe
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UnsubscribeCode = AddUnsubscribe( $Newsletter, $LanguageID, $IsHtml );
#----------------------------------------------------------------------------------------
# description  Returns the template snippet for unsubscribing the newsletter.
#----------------------------------------------------------------------------------------
# input        $Newsletter | newsletter object | object
# input        $LanguageID |language id | integer
# input        $IsHtml | is html allowed | boolean
#----------------------------------------------------------------------------------------
# return       $UnsubscribeCode | unsubscribe code | string
#========================================================================================
sub AddUnsubscribe {
    my ($Newsletter, $LanguageID, $IsHtml) = @_;

    my $Shop = $Newsletter->getSite();
    my $UnsubscribeCode;
    # update content
    my $shopHash = $Shop->tleHash;
    $shopHash->{'LanguageID'} = $LanguageID;
    my $PageType       = LoadPageTypeByAlias( 'InsertUnsubscribeSnippet' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Shop );
    $UnsubscribeCode .= ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($LanguageID), $shopHash);
    $UnsubscribeCode = _RemoveHtmlTags($UnsubscribeCode) unless $IsHtml;

    return $UnsubscribeCode;
}

#========================================================================================
# function     GetSubscribeCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SubscribeCount = GetSubscribeCount($NewsletterID);
#----------------------------------------------------------------------------------------
# description  Returns the amount of subscribers to the given newsletter.
#----------------------------------------------------------------------------------------
# input        $NewsletterID | ID of the newsletter | integer
#----------------------------------------------------------------------------------------
# return       $SubscribeCount | Amount of subscribers | integer
#========================================================================================
sub GetSubscribeCount {
    my ($NewsletterID) = @_;
    return DAL_GetCampaignSubscribedCount($NewsletterID);
}

#========================================================================================
# function     GetUnsubscribeCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UnsubscribeCount = GetUnsubscribeCount($NewsletterID);
#----------------------------------------------------------------------------------------
# description  Returns the amount of Unsubscribers to the given newsletter.
#----------------------------------------------------------------------------------------
# input        $NewsletterID | ID of the newsletter | integer
#----------------------------------------------------------------------------------------
# return       $UnsubscribeCount | Amount of unsubscribers | integer
#========================================================================================
sub GetUnsubscribeCount {
    my ($NewsletterID) = @_;
    return DAL_GetCampaignUnsubscribedCount($NewsletterID);
}

#========================================================================================
# function     InsertCouponSnippet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CouponSnippet = InsertCouponSnippet( $Newsletter, $LanguageID, $IsHtml );
#----------------------------------------------------------------------------------------
# description  Returns the template snippet for a Coupon.
#----------------------------------------------------------------------------------------
# input        $Newsletter | newsletter object | object
# input        $LanguageID |language id | integer
# input        $IsHtml | is html allowed | boolean
#----------------------------------------------------------------------------------------
# return       $CouponSnippet | coupon code | string
#========================================================================================
sub InsertCouponSnippet {
    my ($Newsletter, $LanguageID, $IsHtml) = @_;

    my $CouponCampaign = $Newsletter->get('CouponCampaign');
    return unless $CouponCampaign;

    #TODO use Coupon from form
    my $CouponSnippet;

    my $couponHash = $CouponCampaign->tleHash;
    $couponHash->{'LanguageID'} = $LanguageID;

    my $PageType       = LoadPageTypeByAlias( 'InsertCouponSnippet' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $CouponCampaign );

    $CouponSnippet .= ProcessPage($CouponCampaign, $ObjectPageType, GetCodeByLanguageID($LanguageID), $couponHash);
    $CouponSnippet = _RemoveHtmlTags($CouponSnippet) unless $IsHtml;

    return $CouponSnippet;
}

#========================================================================================
# function     AddProductsAndCatalogs
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Promotion = AddProductsAndCatalogs( $Newsletter, $LanguageID, $IsHtml,
#                               $aProducts, $aCategories );
#----------------------------------------------------------------------------------------
# description  Returns the template with the selected catalog and product objects.
#----------------------------------------------------------------------------------------
# input        $Newsletter | newsletter object | object
# input        $LanguageID |language id | integer
# input        $IsHtml | is html allowed | boolean
# input        $aProducts | list of products | ref.array
# input        $aCategories | list of categories | ref.array
# input        $ForceSSL | force SSL as server protocol | boolean
#----------------------------------------------------------------------------------------
# return       $Promotion | promotion code | string
#========================================================================================
sub AddProductsAndCatalogs {
    my ($Newsletter, $LanguageID, $IsHtml, $aProducts, $aCategories, $ForceSSL) = @_;

    my $Shop = $Newsletter->getSite();

    my $Promotion = '';
    my $ProtocolServerName;

    if ($ForceSSL) {
        $ProtocolServerName = $Shop->get('ProtocolAndServerSSL');
    }
    else {
        $ProtocolServerName = ($Shop->get('HasSSLCertificate') and $Shop->get('LoginRequired'))
            ? $Shop->get('ProtocolAndServerSSL')
            : $Shop->get('ProtocolAndServer');
    }

    foreach my $Product (@$aProducts) {
        my $productHash = $Product->tleHash;
        $productHash->{'LanguageID'} = $LanguageID;
        if ($productHash->{'ImageSmall'} && $IsHtml) {
            my $ProductURL = StringIsURL($productHash->{'ImageSmall'}) ?
                             $productHash->{'ImageSmall'} :
                             $ProtocolServerName . $Product->get('WebPath') . '/' . $productHash->{'ImageSmall'};
            $productHash->{'ImageSmall'} = $ProductURL;
        }
        my $PageType       = LoadPageTypeByAlias( 'InsertProductPromotion' );
        my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Product );
        $Promotion .= ProcessPage($Product, $ObjectPageType, GetCodeByLanguageID($LanguageID), $productHash);
    }

    foreach my $Category (@$aCategories) {
        my $categoryHash = $Category->tleHash;
        $categoryHash->{'LanguageID'} = $LanguageID;
        if ($categoryHash->{'ImageMedium'} && $IsHtml) {
            my $CategoryURL = StringIsURL($categoryHash->{'ImageMedium'}) ?
                              $categoryHash->{'ImageMedium'} :
                              $ProtocolServerName . $Category->get('WebPath') . '/' . $categoryHash->{'ImageMedium'};
            $categoryHash->{'ImageMedium'} = $CategoryURL;
        }

        my $PageType       = LoadPageTypeByAlias( 'InsertCategoryPromotion' );
        my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Category );

        $Promotion .= ProcessPage($Category, $ObjectPageType, GetCodeByLanguageID($LanguageID), $categoryHash);
    }
    $Promotion = _RemoveHtmlTags($Promotion) unless $IsHtml;
    $Promotion = _RemoveEmptyLines($Promotion) if $IsHtml ;

    return $Promotion;
}

#========================================================================================
# function     AddShopLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LogoCode = AddShopLogo( $Newsletter, $LanguageID, $IsHtml );
#----------------------------------------------------------------------------------------
# description  Returns the template snippet for the shop logo.
#----------------------------------------------------------------------------------------
# input        $Newsletter | newsletter object | object
# input        $LanguageID |language id | integer
# input        $IsHtml | is html allowed | boolean
#----------------------------------------------------------------------------------------
# return       $LogoCode | logo code | string
#========================================================================================
sub AddShopLogo {
    my ($Newsletter, $LanguageID, $IsHtml) = @_;

    my $Shop = $Newsletter->getSite();
    my $hResult = GetShopLogo($Newsletter);
    return unless defined $hResult;

    my $shopHash = $Shop->tleHash;
    $shopHash->{'LanguageID'} = $LanguageID;
    $shopHash->{'URL'} = $hResult->{'URL'};
    $shopHash->{'LogoSrc'} = $hResult->{'LogoSrc'};

    my $LogoCode;

    # update content
    my $PageType       = LoadPageTypeByAlias( 'InsertShopLogo' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $Shop );
    $LogoCode .= ProcessPage($Shop, $ObjectPageType, GetCodeByLanguageID($LanguageID), $shopHash);
    $LogoCode = _RemoveHtmlTags($LogoCode) unless $IsHtml;

    return $LogoCode;
}

#========================================================================================
# function     GetShopLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShopLogo = GetShopLogo( $Shop, $Newsletter );
#----------------------------------------------------------------------------------------
# description  Returns the url and source of the shop logo.
#----------------------------------------------------------------------------------------
# input        $Newsletter | newsletter object | object
#----------------------------------------------------------------------------------------
# return       $hResult | logo url, source
#               <ul>
#                   <li>URL - logo url - string</li>
#                   <li>LogoSrc - logo source - string</li>
#               </ul> | ref.hash
#========================================================================================
sub GetShopLogo {
    my ($Newsletter) = @_;

    my $Shop = $Newsletter->getSite();
    my $Logo = $Shop->get('Logo');
    return unless defined $Logo;

    my $hResult = {};
    # insert URL
    my $DefaultStyle = $Shop->get('DefaultStyle');
    my $NavBarElementsFolder = $DefaultStyle->folder('NavBarElements');
    my $NavBarElementLogo = $NavBarElementsFolder->existsChild('Logo') ? $NavBarElementsFolder->child('Logo') : undef;
    my $URL = defined $NavBarElementLogo ? $NavBarElementLogo->get('URL') : undef;
    if (defined $URL and $URL !~ /^http/) {
        $URL = $Shop->get('WebUrl').'/'.$URL;
    }
    $hResult->{'URL'} = $URL if defined $URL;
    # attach shop logo
    my $attachmentAlias = $Shop->alias . '_' . $Logo;
    $attachmentAlias = Unicode2Filename($attachmentAlias);
    my $ProtocolServerName = ($Shop->get('HasSSLCertificate') and  $Shop->get('LoginRequired')) ?
                             $Shop->get('ProtocolAndServerSSL') : $Shop->get('ProtocolAndServer');
    my $LogoURL = $ProtocolServerName . $Shop->get('WebPath') . '/' . $Logo;

    #$hResult->{'LogoSrc'} = 'cid:' . $attachmentAlias;
    $hResult->{'LogoSrc'} = $LogoURL;

    return $hResult
}

sub _RemoveHtmlTags {
    my ($Code) = @_;
    #
    $Code =~ s/(<a\shref=")(.*?)(".*?<\/a>)/$2/igx;
    $Code =~ s/<hr\s*\/?>/<br\/>------------------------------------------------------------------------------------------------------------------<br\/>/ig;
    return RemoveHtmlTags($Code);
}

sub _RemoveEmptyLines {
    my ($Code) = @_;

    my $NewCode;
    foreach my $Line (split(/\n/,$Code)) {
       $NewCode .= $Line ."\n" if $Line =~/\S/;
    }
    return $NewCode;
}

1;
