#========================================================================================
# §package      DE_EPAGES::ShopNewsletter::DAL::SQL::Newsletter
# §description  Generic SQL database interface for tables:
#               <ul>
#                   <li>newsletteraddressee
#               </ul>
# §state        private
#========================================================================================
package DE_EPAGES::ShopNewsletter::DAL::SQL::Newsletter;
use base qw ( Exporter );

use utf8;
use strict;
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );

our @EXPORT_OK = qw (
    GetCampaignSubscribedCount
    GetCampaignUnsubscribedCount
);

#========================================================================================
# §function     GetIsSubscribed
#----------------------------------------------------------------------------------------
# §syntax       $isEmailSubscribed = $Package->GetIsSubscribed($dbi, $newslettercampaignid, $email);
#----------------------------------------------------------------------------------------
# §example      $isEmailSubscribed = $Package->GetIsSubscribed($dbi, 42237, "ozscheyge@epages.com");
#----------------------------------------------------------------------------------------
# §description  Returns whether a email address is subscribed (and confirmed) to a newsletter.
#----------------------------------------------------------------------------------------
# §input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# §input        $newslettercampaignid | ID of the newsletter campaign | int
# §input        $email | Email address to check | string
# §return       $isSubscribed | 1 if email is subscribed, 0 if not | int/bool
#========================================================================================
sub GetIsSubscribed {
    shift;
    my ($dbi, $newslettercampaignid, $email) = @_;
    my $aaDBResult = $dbi->execute('SELECT unsubscribed FROM newsletteraddressee WHERE newslettercampaignid=? AND email=?',
                                   [$newslettercampaignid, $email], ['Integer']);
    if (scalar @$aaDBResult) {
        return 1 if $aaDBResult->[0]->[0] == 0;
    }
    return 0;
}

#========================================================================================
# §function     _GetCampaignAdresseeCountBySubscribeStatus
#----------------------------------------------------------------------------------------
# §syntax       $SubscriberCount = _GetCampaignAdresseeCountBySubscribeStatus($Newslettercampaignid, $Subscribed);
#----------------------------------------------------------------------------------------
# §example      $SubscriberCount = _GetCampaignSubscribedCount(42237, 1);
#----------------------------------------------------------------------------------------
# §description  Returns the count of sub/unsubscribers to a newslettercampaign.
#----------------------------------------------------------------------------------------
# §input        $Newslettercampaignid | ID of the newsletter campaign | int
# §input        $Subscribed | 0 for subscribers; 1 for unsubscribers | int
# §return       $SubscriberCount | Number of sub/unsubscribers to the newsletter campaign | int
#========================================================================================
sub _GetCampaignAdresseeCountBySubscribeStatus {
    my ($NewsletterCampaignID, $Subscribed) = @_;
    my $dbi = GetCurrentDBHandle();
    my $aaDBResult = $dbi->execute( ##no critic qw(ProhibitImplicitNewlines)
        'SELECT COUNT(newsletteraddresseeid)
        FROM newsletteraddressee
        WHERE(
            newslettercampaignid=? AND
            unsubscribed=?)',
        [$NewsletterCampaignID, $Subscribed],
        ['Integer']);
    return $aaDBResult->[0]->[0];
}

#========================================================================================
# §function     GetCampaignSubscribedCount
#----------------------------------------------------------------------------------------
# §syntax       $SubscriberCount = GetCampaignSubscribedCount($NewsletterCampaignID);
#----------------------------------------------------------------------------------------
# §example      $SubscriberCount = GetCampaignSubscribedCount(42237);
#----------------------------------------------------------------------------------------
# §description  Returns the count of unsubscribers to a newslettercampaign.
#----------------------------------------------------------------------------------------
# §input        $NewsletterCampaignID | ID of the newsletter campaign | int
# §return       $unsubscriberCount | count of unsubscribers to the newsletter campaign | int
#========================================================================================
sub GetCampaignSubscribedCount {
    my ($NewsletterCampaignID) = @_;
    return  _GetCampaignAdresseeCountBySubscribeStatus($NewsletterCampaignID, 0);
}

#========================================================================================
# §function     GetCampaignUnsubscribedCount
#----------------------------------------------------------------------------------------
# §syntax       $unsubscriberCount = GetCampaignUnsubscribedCount($NewsletterCampaignID);
#----------------------------------------------------------------------------------------
# §example      $unsubscriberCount = GetCampaignUnsubscribedCount(42237);
#----------------------------------------------------------------------------------------
# §description  Returns the count of unsubscribers to a newslettercampaign.
#----------------------------------------------------------------------------------------
# §input        $NewsletterCampaignID | ID of the newsletter campaign | int
# §return       $unsubscriberCount | count of unsubscribers to the newsletter campaign | int
#========================================================================================
sub GetCampaignUnsubscribedCount {
    my ($NewsletterCampaignID) = @_;
    return _GetCampaignAdresseeCountBySubscribeStatus($NewsletterCampaignID, 1);
}
1;
