#==============================================================================
# package      DE_EPAGES::ShopTransfer::API::Transform
# state        public
# description  Creates the 'import' directory from the 'export' directory.
#==============================================================================
package DE_EPAGES::ShopTransfer::API::Transform;
use base Exporter;

use strict;

use Config;
use File::Basename;
use File::Find;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::File qw ( ExistsDirectory CreateDirectory DeleteDirectory
                                    CopyFile ExistsFile MoveFile
                                    GetFileNames RenameDirectory );
use DE_EPAGES::Core::API::PerlTools qw (UsePackage GenerateGUID);
use DE_EPAGES::Core::API::SystemCommand qw (PackTar UnpackTar);
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw (GenerateSplittedGUIDPath);
use DE_EPAGES::ShopTransfer::API::BCP qw (BCPInIterator BCPOutMapper);

sub new {
    my $class = shift;

    my $DBIPackage = GetCurrentDBHandle()->driverName =~ /mysql/i
        ? 'DE_EPAGES::ShopTransfer::API::DBImySQL'
        : 'DE_EPAGES::ShopTransfer::API::DBISybase';
    UsePackage($DBIPackage);

    return bless {
        'dbh' => $DBIPackage->new,
    }, $class;
}

sub getDBIInstance {
    my $self = shift;
    return $self->{'dbh'};
}
#========================================================================================
# function     getSourceFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FileName = $trans->getSourceFile($StorePath, $SiteAlias, $Table, $hOptions);
#----------------------------------------------------------------------------------------
# description  Returns existing file name (may be bcp file was changed in patch).
#----------------------------------------------------------------------------------------
# input        $StorePath | bcp path to store sometimes $hOptions 'storePath' or 'sourcePath'| string
# input        $SiteAlias | SYSTEM or shop alias | string
# input        $Table | table name | string
# input        $hOptions | options, keys: versions, storePath, db_name, bcpParameter, encoding | ref.hash
# return       $FileName | file name | string
#========================================================================================
sub getSourceFile {
    my $self = shift;
    my ($StorePath, $SiteAlias, $Table, $hOptions) = @_;

    my $FileName = undef;
    foreach my $Version (@{$hOptions->{'versions'}}) {
        $FileName = join '/', $StorePath,$Version,$SiteAlias,$Table.'.bcp';
        return $FileName if ExistsFile($FileName);
    }
    return join '/', $StorePath,'export',$SiteAlias,$Table.'.bcp';
}

#========================================================================================
# function     loadSystem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSystemMapping = $trans->loadSystem($Table, $hOptions, $IsNew);
#----------------------------------------------------------------------------------------
# description  Loads System Mappings (IDs to UniqueExternalPresentation).
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $hOptions | options, keys: versions, storePath, sourcePath, encoding | ref.hash
# input        $IsNew | load new or old system | string
# return       $hSystemMapping | mapping keys: internal identifier values: external identifier| ref.hash
#========================================================================================
sub loadSystem {
    my $self = shift;
    my ($Table, $hOptions, $IsNew) = @_;
    my $StorePath = $hOptions->{$IsNew ? 'storePath' : 'sourcePath'};

    my $FileName = $self->getSourceFile($StorePath, 'SYSTEM', $Table, $hOptions);
    return {} unless ExistsFile($FileName); # ignore tables that where not exported

    my $hObjectMapping = $hOptions->{'SystemMappings'}{'object'};
    my %SystemMapping;
    my @UnknownRows; # remember rows that can't be resolved in the first run
    my %Iterators = (
        'object' => sub {
            my ($ID, $ClassID, $ParentID, $Alias) = @_;
            if ($ParentID eq '' or exists $SystemMapping{$ParentID}) {
                my $string = $ParentID ne ''
                    ? ( $ParentID != 1 ? $SystemMapping{$ParentID}.'/'.$Alias : '/'.$Alias )
                    : '/';
                # remove trailing and leading whitespaces
                $string =~ s/^\s+//;
                $string =~ s/\s+$//;

                $SystemMapping{$ID} = $string;
                    #print "$ID: " . $SystemMapping{$ID} . "\n";
            } else {
                push @UnknownRows, [@_];
            }
        },
        'language' => sub { # code3 is unique
            $SystemMapping{$_[0]} = $_[3];
        },
        'searchfrom' => sub { # tablename is unique
            $SystemMapping{$_[0]} = $_[3];
        },
        'locale' => sub { # pkey
        },
        'stylegroupmap' => sub { # (stylegroupid, styletemplateid) is unique
            $SystemMapping{$_[0]} = $IsNew ? $_[1].'.'.$_[2] : $hObjectMapping->{$_[1]}.'.'.$hObjectMapping->{$_[2]};
        },
        'template' => sub { # (pagetypeid, name) is unique
            $SystemMapping{$_[0]} = $IsNew ? $_[1].'.'.$_[2] : $hObjectMapping->{$_[1]}.'.'.$_[2];
        },
    );

    my $cIterator = $Iterators{$Table};
    if (not defined $cIterator) {
        my ($FirstUniqueIndex) = $self->getDBIInstance()->getUniqueIndexes($Table);
        die "No mapping for table '$Table'" unless defined $FirstUniqueIndex;

        my $PosFirstUniqueIndex = $self->getDBIInstance()->getColumnPosition($Table, $FirstUniqueIndex);
        my $PosPrimaryKey = $self->getDBIInstance()->getColumnPosition($Table, $self->getDBIInstance()->getPKeyOfTable($Table));
        $cIterator = sub {
            $SystemMapping{$_[$PosPrimaryKey]} = $_[$PosFirstUniqueIndex];
        };
    }

    BCPInIterator($FileName, $cIterator, $hOptions->{'encoding'});

    # For tables with self-references some rows need a second run.
    my @Unknown = @UnknownRows;
    @UnknownRows = ();
    $cIterator->(@$_) foreach @Unknown;
    die "unknown rows at table '$Table'" if scalar @UnknownRows;

    return \%SystemMapping;
}

#========================================================================================
# function     createSystemMapping
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hSystemMapping = $trans->createSystemMapping($Table, $hOptions);
#----------------------------------------------------------------------------------------
# description  Loads System Mappings (IDs to IDs).
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $hOptions | options, keys: versions, storePath, sourcePath, encoding | ref.hash
# return       $hSystemMapping | mapping keys: old ids values: new ids | ref.hash.integer
#========================================================================================
sub createSystemMapping {
    my $self = shift;
    my ($Table, $hOptions) = @_;

    my $hOldSystem = $self->loadSystem($Table, $hOptions, 0);
    my $hNewSystem = $self->loadSystem($Table, $hOptions, 1);

    # turn values into keys
    my %NewSystem = reverse %$hNewSystem;
    my %Mapping;
    $Mapping{$_} = $NewSystem{$hOldSystem->{$_}} foreach keys %$hOldSystem;

    my @undefinedMappings = grep { ! defined $Mapping{$_} } keys %Mapping;
    my @oldValuesFromUndefinedMappings = map { $hOldSystem->{$_} } @undefinedMappings;
    #my @Warnings = sort grep { ! /^\/AnonymousUsers/ } @oldValuesFromUndefinedMappings;
    my @Warnings = sort @oldValuesFromUndefinedMappings;

    print "System row at $Table not found '$_' - maybe harmless.\n" foreach @Warnings;
    delete $Mapping{$_} foreach @undefinedMappings;

    #return undef unless grep { $_ ne $Mapping{$_} } keys %Mapping;
    return \%Mapping;
}

#========================================================================================
# function     getCountOfRows
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Count = $trans->getCountOfRows($Table, $ShopAlias, $hOptions)
#----------------------------------------------------------------------------------------
# description  Counts rows of source bcp file.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $ShopAlias | shop alias | string
# input        $hOptions | options, keys: versions, sourcePath, encoding | ref.hash
# return       $Count | count of rows in table | integer
#========================================================================================
sub getCountOfRows {
    my $self = shift;
    my ($Table, $ShopAlias, $hOptions) = @_;

    my $FileName = $self->getSourceFile($hOptions->{'sourcePath'}, $ShopAlias, $Table, $hOptions);
    my $Counter =0;
    BCPInIterator($FileName, sub { $Counter++; }, $hOptions->{'encoding'});
    return $Counter;
}

#========================================================================================
# function     createTableMapping
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hMapping = $trans->createTableMapping($Table, $ShopAlias, $FirstID, $hSystemMapping, $hOptions)
#----------------------------------------------------------------------------------------
# description  Adds shop mapping to system mapping.
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $ShopAlias | shop alias | string
# input        $FirstID | first identifier | integer
# input        $hSystemMapping | mapping keys: old ids values: new ids | ref.hash.integer
# input        $hOptions | options, keys: versions, sourcePath, encoding | ref.hash
# return       $hMapping | mapping of ids | integer
#========================================================================================
sub createTableMapping {
    my $self = shift;
    my ($Table, $ShopAlias, $FirstID, $hSystemMapping, $hOptions) = @_;

    my %Mapping = defined $hSystemMapping ? %$hSystemMapping : ();
    # add shop mapping
    my $PosPrimaryKey = $self->getDBIInstance()->getColumnPosition($Table, $self->getDBIInstance()->getPKeyOfTable($Table));
    my $FileName = $self->getSourceFile($hOptions->{'sourcePath'}, $ShopAlias, $Table, $hOptions);

    BCPInIterator($FileName, sub { $Mapping{$_[$PosPrimaryKey]} = $FirstID++; }, $hOptions->{'encoding'});
    return \%Mapping;
}

sub testTableType {
    my $self = shift;
    my ($Table, $ShopAlias, $hOptions, $hNonExistentLocales) = @_;

    if( $Table eq 'object') {
        my @dynTables = @{$self->getDBIInstance()->dynamicIDTables};
        print 'Dynamic Tables: ' . join(', ', @dynTables) . "\n";
    }
    print "Processing table $Table...\n";
    if ( $self->getDBIInstance()->isColumnAutoIncrement( $Table ) ) {
        print "$Table is somewhat auto increment table!\n";
    }
}

#========================================================================================
# function     transferTable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hGUIDMapping = $trans->transferTable($Table, $ShopAlias, $hOptions, $hNonExistentLocales)
#----------------------------------------------------------------------------------------
# description  Replaces old identifier of shop bcp file with new identifier.
#               Returns a hash with mappings from old GUID subdirectories to new ones
#               (Used for private files)
#----------------------------------------------------------------------------------------
# input        $Table | table name | string
# input        $ShopAlias | shop alias | string
# input        $NewShopAlias | new shop alias | string
# input        $hOptions | options, keys: versions, sourcePath,storePath,encoding | ref.hash
# input        $hNonExistentLocales | hash with localeids of non-existing locales as key | ref.hash
# return       $hGUIDMapping | GUID mapping from old to new GUIDs | ref.hash
#========================================================================================
sub transferTable {
    my $self = shift;
    my ($Table, $ShopAlias, $NewShopAlias, $hOptions, $hNonExistentLocales) = @_;

    my $dbi = $self->getDBIInstance();
    my @References = $dbi->getReferenceColumns($Table);
    my %Definitions = ();

    my $hAllColumns = $dbi->getAllColumns();
    my $GetColumnPosition = sub {
        my ($DatabaseTable, $Column) = @_;
        my $hColumn = $hAllColumns->{$DatabaseTable}->{$Column} or die "Column $DatabaseTable.$Column doesn't exist.";
        return $hColumn->{'Position'};
    };

    # object.classid has no reference to class.classid
    if ($Table eq 'object') {
        $Definitions{$GetColumnPosition->($Table, 'classid')} = 'object';
        unless (defined $hOptions->{'SystemMappings'}{'object'}) {
            $hOptions->{'SystemMappings'}{'object'} = $self->createSystemMapping('object', $hOptions)
        }
        if (defined $hOptions->{'SystemMappings'}{'object'}){
            $hOptions->{'ShopMappings'}{'object'} = { %{$hOptions->{'SystemMappings'}{'object'}} } unless exists $hOptions->{'ShopMappings'}{'object'};
        }
    }

    foreach my $RefColumn (@References) {
        my ($MasterTable, $MasterColumn) = $dbi->getMasterColumn($Table, $RefColumn);

        $hOptions->{'SystemMappings'}{$MasterTable} = $self->createSystemMapping($MasterTable, $hOptions)
            unless exists $hOptions->{'SystemMappings'}{$MasterTable} or ($MasterTable eq $Table and $RefColumn eq $MasterColumn);

        $Definitions{$GetColumnPosition->($Table, $RefColumn)} = $MasterTable;

        unless (exists $hOptions->{'ShopMappings'}{$MasterTable}) {
            $hOptions->{'ShopMappings'}{$MasterTable} = { %{$hOptions->{'SystemMappings'}{$MasterTable}} }
        }
    }

    #
    # If table has an auto increment column, pre-create new ids for all the rows which will be needed
    # for the new datasets; the insert takes place later, but we need the next free id right now for
    # further process!
    #
    if ( $dbi->isColumnAutoIncrement( $Table ) ) {
        # get count of new rows
        my $CountOfRows = $self->getCountOfRows( $Table, $ShopAlias, $hOptions);
        # reserve block of new numbers
        my $FirstID = $dbi->reserveIdentifier($Table, $CountOfRows);
        # map old numbers to new
        $hOptions->{'ShopMappings'}{$Table} = $self->createTableMapping($Table, $ShopAlias, $FirstID, $hOptions->{'SystemMappings'}{$Table}, $hOptions);
        $Definitions{$GetColumnPosition->($Table, $dbi->getPKeyOfTable($Table))} = $Table;
    }
    my $SourceFile = $self->getSourceFile($hOptions->{'sourcePath'}, $ShopAlias, $Table, $hOptions);
    return unless ExistsFile($SourceFile);

    my @Positions = keys %Definitions; # (0 => object, 1 => 'object')
    my $Counter = 0;
    my %GUIDMapping; # holds a mapping from old to new GUIDs of objects
    my $TargetFile = "$hOptions->{'storePath'}/import/$NewShopAlias/$Table.bcp";

    BCPOutMapper( $SourceFile, $TargetFile, sub {
        my @Values = @_;
        $Counter++;

        my $ShopTypeRefPosition = $GetColumnPosition->('shop', 'shoptyperefid');

        foreach my $ColumnIndex (@Positions) {
            my $TableName = $Definitions{$ColumnIndex};
            next if $TableName eq 'locale'; # don't transform localeid

            my $Value = $Values[$ColumnIndex];
            if ($Value eq '' or exists $hOptions->{'ShopMappings'}{$TableName}{$Value}) {
                $Values[$ColumnIndex] = $hOptions->{'ShopMappings'}{$TableName}{$Value};

                # change shop type
                if ($hOptions->{'shoptype'} && $Table eq 'shop' && $ColumnIndex == $ShopTypeRefPosition) {
                    my $ShopTypeRefID = $self->_getShopTypeRefID($hOptions);
                    if ($ShopTypeRefID) {
                        $Value = $ShopTypeRefID;
                        print "  Resetting shop type: $Value\n";
                    }
                    else {
                        printf "unknown ShopType %s\n", $hOptions->{'shoptype'};
                    }
                }
            }
            else {
                Error( 'NO_MAPPING', {
                    'ColumnNo' => $ColumnIndex,
                    'Line' => $Counter,
                    'TableName' => $TableName,
                    'Value' => $Value,
                    'SourceFile' => $SourceFile,
                    'TargetFile' => $TargetFile,
                });
            }
        }

        if ($Table eq 'localizedstringattribute') {
            # remove attributes for locales that do not exist
            foreach my $Locale (keys %$hNonExistentLocales){
                my $Position = $GetColumnPosition->('localizedstringattribute', 'languageid');
                return if $Values[$Position] == $hNonExistentLocales->{$Locale};
            }
        }
        elsif ($Table eq 'sitelocalemap') {
            foreach my $Locale (keys %$hNonExistentLocales){
                my $Position = $GetColumnPosition->('sitelocalemap', 'localeid');
                return if $Values[$Position] eq $Locale;
            }
        }
        elsif ($Table eq 'object') {
            if ($hOptions->{'newguid'}) {
               my $NewGUID = GenerateGUID();
               my $Position = $GetColumnPosition->('object', 'guid');
               $GUIDMapping{$Values[$Position]} = $NewGUID; # store mapping from old to new GUID
               $Values[$Position] = $NewGUID;
            }
            my $PositionObjectID = $GetColumnPosition->('object', 'objectid');
            my $PositionSiteID = $GetColumnPosition->('object', 'siteid');
            if ($Values[$PositionObjectID] == $Values[$PositionSiteID]) {
                my $PositionAlias = $GetColumnPosition->('object', 'alias');
                $Values[$PositionAlias] = $NewShopAlias;
            }
        }
        elsif ($Table eq 'style') {
            # set CSS styles to invalid in order to force a rebuild
            my $Position = $GetColumnPosition->('style', 'isinvalid');
            $Values[$Position] = 1 unless $ShopAlias eq $NewShopAlias;
        }
        else {
            # nothing todo
        }

        return @Values;
    }, $hOptions->{'encoding'} );

    return \%GUIDMapping;
}


#========================================================================================
# function     transferFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $trans->transferFiles( $hGUIDMapping, $ShopAlias, $hOptions )
#----------------------------------------------------------------------------------------
# description  Copies the files public.tar and private.tar to the "import" directory.
#               Renames all GUID-based subdirectries in shop directory to new GUIDs
#----------------------------------------------------------------------------------------
# input        $hGUIDMapping | GUID mapping from old to new GUIDs | ref.hash
# input        $ShopAlias | shop alias | string
# input        $NewShopAlias | new shop alias | string
# input        $hOptions | options, keys: versions, sourcePath, encoding | ref.hash
#========================================================================================
sub transferFiles {
    my $self = shift;
    my ($hGUIDMapping, $ShopAlias, $NewShopAlias, $hOptions) = @_;

    foreach my $FileType (qw( private public )) {
        my $SourceTar = "$hOptions->{'sourcePath'}/export/$ShopAlias/$FileType.tar";
        my $TargetTar = "$hOptions->{'storePath'}/import/$NewShopAlias/$FileType.tar";
        if( ExistsFile( $SourceTar ) ) {
            if( $hOptions->{'newguid'} ) {
                $self->_renameGUIDSubdirectories($SourceTar, $TargetTar, $hGUIDMapping, $ShopAlias, $NewShopAlias, $hOptions);
            }
            else {
                if ($ShopAlias eq $NewShopAlias) {
                    CopyFile( $SourceTar, $TargetTar );
                } else {
                    # Unpack the *.tar archive, rename the top-level directory to the new shop alias, and compress it again.
                    my $TransformDirectory = "$hOptions->{'storePath'}/transform";
                    $self->_uncompressSourceToDirectory($SourceTar, $TransformDirectory, $hOptions);
                    RenameDirectory( "$TransformDirectory/$ShopAlias", "$TransformDirectory/$NewShopAlias", { create_directory => 1 } );
                    $self->_compressTargetInDirectory($TargetTar, $NewShopAlias, $TransformDirectory, $hOptions);
                    DeleteDirectory( $TransformDirectory, { 'recursive' => 1 } );
                }
            }
        }
    }

    return;
}

sub _uncompressSourceToDirectory {
    my $self = shift;
    my ($SourceTar, $TransformDirectory, $hOptions) = @_;

    # Unpack the *.tar archive to a temporary transform directory
    UnpackTar($SourceTar, $TransformDirectory, {'Print' => $hOptions->{'verbose'}});
    return;
}

sub _compressTargetInDirectory {
    my $self = shift;
    my ($TargetTar, $NewShopAlias, $TransformDirectory, $hOptions) = @_;

    $self->_deleteEmptySubDirs( "$TransformDirectory/$NewShopAlias" );
    PackTar($TargetTar, $TransformDirectory, [$NewShopAlias], {'Print' => $hOptions->{'verbose'}});
    return;
}

sub _renameGUIDSubdirectories {
    my $self = shift;
    my ($SourceTar, $TargetTar, $hGUIDMapping, $ShopAlias, $NewShopAlias, $hOptions) = @_;

    # Unpack the private.tar archive to a temporary transform directory to rename the GUID paths afterwards.
    my $TransformDirectory = "$hOptions->{'storePath'}/transform";
    $self->_uncompressSourceToDirectory($SourceTar, $TransformDirectory, $hOptions);
    RenameDirectory( "$TransformDirectory/$ShopAlias", "$TransformDirectory/$NewShopAlias", { create_directory => 1 } );

    # Iterate all GUID paths and migrate the files in it to the new paths.
    foreach my $OldGUID (keys %$hGUIDMapping) {
        my $OldPath = join( '/', $TransformDirectory, $NewShopAlias, GenerateSplittedGUIDPath( $OldGUID ) );
        my $NewPath = join( '/', $TransformDirectory, $NewShopAlias, GenerateSplittedGUIDPath( $hGUIDMapping->{$OldGUID} ));
        # If old GUID-based directory (downloadable product) exists, migrate this
        #  directory path to the new GUID to ensure proper file access after the transfer.
        if (ExistsDirectory( $OldPath )) {
            CreateDirectory( $NewPath, { 'recursive' => 1 });
            my $aFiles = GetFileNames( $OldPath, { 'recursive' => 1 } );
            foreach my $File (@$aFiles) {
                MoveFile( $File, join('/', $NewPath, basename($File)), { 'create_directory' => 1});
                printf "Copying file $File from $OldPath to $NewPath\n" if $hOptions->{'verbose'};
            }
        }
    }

    # Repack
    $self->_compressTargetInDirectory($TargetTar, $NewShopAlias, $TransformDirectory, $hOptions);
    DeleteDirectory( $TransformDirectory, { 'recursive' => 1 } );

    return;
}

sub _deleteEmptySubDirs {
    my $self = shift;
    my ($Directory) = @_;

    # count all entries in sub directories
    my %Directories;
    find({
        wanted => sub {
            $Directories{$_} = 0 if -d;
            $Directories{dirname($_)}++;
        },
        no_chdir => 1,
    }, $Directory);

    # EPG-30738: don't delete the parent directory, even if all sub dirs are empty
    delete $Directories{$Directory};

    # delete empty sub directories until no empty directories are left
    while( my @EmptyDirs = grep { 0 == $Directories{$_} } keys %Directories ) {
        foreach my $Dir ( @EmptyDirs ) {
            delete $Directories{$Dir};
            $Directories{dirname($Dir)}--;
            DeleteDirectory( $Dir );
        }
    }

    return;
}

#========================================================================================
# function     getVisibleLocales
# state        public
#----------------------------------------------------------------------------------------
# syntax       %VisibleLocales = $trans->getVisibleLocales($hOptions)
#----------------------------------------------------------------------------------------
# description  Gets visible locales of bcp file.
#----------------------------------------------------------------------------------------
# input        $hOptions | options, keys: versions, sourcePath, encoding | ref.hash
# return       %VisibleLocales | visible locales in system | ref.hash
#========================================================================================
sub getLocales {
    my $self = shift;
    my ($Path, $hOptions) = @_;
    my $Table = 'locale';
    my $FileName = $self->getSourceFile($Path, 'SYSTEM', $Table, $hOptions);
    my %VisibleLocales;

    unless (ExistsFile($FileName)) {
        Warning ("Can't import system $FileName");
    }
    else {
        BCPInIterator($FileName, sub {
            my @Parameter = @_;
            $VisibleLocales{$Parameter[0]} = $Parameter[2] if $Parameter[1];
        }, $hOptions->{'encoding'});
    }
    return %VisibleLocales;
}

sub getVisibleLocales {
    my $self = shift;
    my ($hOptions) = @_;
    my %SourceLocales = $self->getLocales($hOptions->{'sourcePath'}, $hOptions);
    my %TargetLocales = $self->getLocales($hOptions->{'storePath'}, $hOptions);
    delete $SourceLocales{$_} foreach grep {defined $TargetLocales{$_}} keys %SourceLocales;
    return undef unless scalar keys %SourceLocales;
    return \%SourceLocales;
}

sub _getShopTypeRefID {
    my $self = shift;
    my ($hOptions) = @_;
    my (@Objects, $NewShopTypeRefID);
    BCPInIterator($self->getSourceFile($hOptions->{'storePath'}, 'SYSTEM', 'object', $hOptions), sub {
        my @Parameter = @_;
        push @Objects, $Parameter[0] if $Parameter[3] eq $hOptions->{'shoptype'};
    }, $hOptions->{'encoding'});
    BCPInIterator($self->getSourceFile($hOptions->{'storePath'}, 'SYSTEM', 'shoptyperef', $hOptions), sub {
        my @Parameter = @_;
        foreach my $ObjectID (@Objects) {
            $NewShopTypeRefID = $ObjectID if $Parameter[0] eq $ObjectID;
        }
    }, $hOptions->{'encoding'});
    return $NewShopTypeRefID;
}

1;
