#========================================================================================
# package      DE_EPAGES::TaxUS::API::Object::TaxMatrixUS
# state        public
# base         DE_EPAGES::Tax::API::Object::TaxMatrix
# description  object interface for TaxMatrixUS.
#========================================================================================
package DE_EPAGES::TaxUS::API::Object::TaxMatrixUS;
use base DE_EPAGES::Tax::API::Object::TaxMatrix;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestFloat TestString TestObject TestInteger );
use DE_EPAGES::Object::API::Factory qw ( LoadObject Transaction );
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Shop::API::Constants qw( TAX_MODEL_GROSS );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Tax::API::Table::TaxRate qw ( ExistsTaxRate GetInfoTaxRatesByTaxAreaID InsertTaxRate GetInfoTaxRate );
use DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS qw( GetInfoZip2TaxUSByZipCodeCityState );    ## no critic qw(RequireCorrectImports)

use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Object::API::Object::Object;
use constant PERCENT => 100.00;

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrixUS = DE_EPAGES::TaxUS::API::Object::TaxMatrixUS->insert($hInfo);
# example      my $Class = LoadClassByAlias('TaxMatrixUS');
#               my $TaxMatrixUS = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertTaxMatrixUS' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $TaxMatrixUS | new object | object
# hook         OBJ_InsertTaxMatrixUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixUSID - tax matrixus object id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertTaxMatrixUS', {'Object' => $self, 'TaxMatrixID' => $self->id});
    delete $hInfo->{$_} foreach qw ( LineItemClass );    ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrixUS->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteTaxMatrixUS' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteTaxMatrixUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixUSID - tax matrixus object id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteTaxMatrixUS', {'Object' => $self, 'TaxMatrixUSID' => $self->id});
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrixUS->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateTaxMatrixUS' and 'OBJ_AfterUpdateTaxMatrixUS'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateTaxMatrixUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixUSID - tax matrixus object id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateTaxMatrixUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMatrixUSID - tax matrixus object id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateTaxMatrixUS', {'Object' => $self, 'Values' => $hValues, 'TaxMatrixUSID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateTaxMatrixUS', {'Object' => $self, 'Values' => $hValues, 'TaxMatrixUSID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     clone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Clone = $Object->clone( $hInfo );
# syntax       $Clone = $Object->clone( $hInfo, $Recursive );
#----------------------------------------------------------------------------------------
# description  Inserts a new object into the database as copy from old object. This
#               function copies also tax rates of the matrix, which are no objects.
#----------------------------------------------------------------------------------------
# input        $hInfo | hash with mandatory attribute values.
#                       <ul>
#                         <li>ClassID - class id - int</li>
#                         <li>Alias - alias - varchar(255)</li>
#                         <li>ParentID - parent objectid - int</li>
#                         <li>Inherit - (optional) inherit permissions from parent object - boolean</li>
#                       </ul> | hash ref
# input        $Recursive | clone child object too (optional, default is false) | boolean
# return       $Clone | the new object | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;
    $Recursive = 0 unless defined $Recursive;

    my $Clone;
    Transaction(
        sub {
            $Clone = $self->DE_EPAGES::Object::API::Object::Object::clone($hInfo, $Recursive);

            ### clone tax rates us ###
            # map id to alias
            my %SelfTaxClassesByID;
            $SelfTaxClassesByID{$_->id} = $_->alias foreach @{$self->get('TaxClasses')};
            my %SelfTaxAreasByID;
            $SelfTaxAreasByID{$_->id} = $_->alias foreach @{$self->get('TaxAreas')};

            # map aliases to object
            my %CloneTaxClassesByAlias;
            $CloneTaxClassesByAlias{$_->alias} = $_->id foreach @{$Clone->get('TaxClasses')};
            my %CloneTaxAreasByAlias;
            $CloneTaxAreasByAlias{$_->alias} = $_->id foreach @{$Clone->get('TaxAreas')};

            # set defaults
            $Clone->set({'DefaultTaxClass' => $Clone->child($self->defaultTaxClass->alias)});
            $Clone->set({'DefaultTaxArea'  => $Clone->child($self->defaultTaxArea->alias)});
            $Clone->set({'FallbackTaxArea'  => undef});

            # insert tax rates us
            foreach my $TaxAreaID (keys %SelfTaxAreasByID) {
                my $CloneTaxAreaID = $CloneTaxAreasByAlias{$SelfTaxAreasByID{$TaxAreaID}};
                my $aTaxRates      = GetInfoTaxRatesByTaxAreaID($TaxAreaID);
                my %TaxRates;
                $TaxRates{$_->{'TaxClassUSID'}} = $_->{'Rate'} foreach @$aTaxRates;
                foreach my $TaxClassID (keys %SelfTaxClassesByID) {
                    my $CloneTaxClassID = $CloneTaxClassesByAlias{$SelfTaxClassesByID{$TaxClassID}};
                    next unless defined $TaxRates{$TaxClassID};
                    InsertTaxRate({'TaxAreaUSID' => $CloneTaxAreaID, 'TaxClassUSID' => $CloneTaxClassID, 'Rate' => $TaxRates{$TaxClassID}});
                }

                # set shop address to default tax area us
                my $TaxAreaUS = LoadObject($CloneTaxAreaID);
                if ($TaxAreaUS->get('IsDefault')) {
                    $TaxAreaUS->set({'PresenceAddress' => $TaxAreaUS->getSite()->get('Address')});
                }
            }
        });

    return $Clone;
}

# we do not convert gross2net because we have always TaxModel Net
sub gross2net {
    my $self = shift;
    my ($CurrencyID, $Price, $hParams) = @_;

    TestString('CurrencyID', $CurrencyID, 3, 3);
    TestFloat('Price', $Price);
    TestInteger('Precision', $hParams->{'Precision'}) if defined $hParams->{'Precision'};

    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
    my $Precision = $hParams->{'Precision'} // (GetInfoCurrency($CurrencyID)->{'MinorUnit'} + 1);
    return $Formatter->round($Price, $Precision);
}

# we do not convert net2gross because we have always TaxModel Net
sub net2gross {
    my $self = shift;
    my ($CurrencyID, $Price, $hParams, $ExtraPrecision) = @_;

    TestString('CurrencyID', $CurrencyID, 3, 3);
    TestFloat('Price', $Price);
    TestInteger('Precision', $hParams->{'Precision'}) if defined $hParams->{'Precision'};

    my $MinorUnit = GetInfoCurrency($CurrencyID)->{'MinorUnit'};
    if ($hParams->{'TaxModel'} == TAX_MODEL_GROSS) {

        # we do not convert to gross because we have always net for us customers
        # we can not return undef here because the editing function for orders allow to switch to gross
        # an we run into "ProductNoPrice" error if we return undef
        # so we di not convert and return the net price as a workaround
        #return undef;
        $MinorUnit++ if $ExtraPrecision;
    }
    else {
        $MinorUnit++ if $ExtraPrecision;
    }

    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new();
    my $Precision = $hParams->{'Precision'} // $MinorUnit;

    return $Formatter->round($Price, $Precision);
}

#========================================================================================
# function     defaultTaxArea
#----------------------------------------------------------------------------------------
# syntax       $TaxArea = $TaxMatrixUS->defaultTaxAreaUS;
#----------------------------------------------------------------------------------------
# description  Gets the default tax area of the matrix
#----------------------------------------------------------------------------------------
# return       $AreaUS | tax area us | object
#========================================================================================
sub defaultTaxArea {
    my $self = shift;

    my $DefaultTaxAreaUS = $self->get('DefaultTaxArea');
    Error(
        'NO_DEFAULT_TAX_AREA', {
            ObjectID => $self->id,
            Alias    => $self->alias
        }) unless defined $DefaultTaxAreaUS;

    return $DefaultTaxAreaUS;
}

#========================================================================================
# function     defaultTaxClass
#----------------------------------------------------------------------------------------
# syntax       $TaxClass = $TaxMatrixUS->defaultTaxClass;
#----------------------------------------------------------------------------------------
# description  Gets the default tax class of the matrix
#----------------------------------------------------------------------------------------
# return       $TaxClassUS | tax class us | object
#========================================================================================
sub defaultTaxClass {
    my $self = shift;

    my $DefaultTaxClassUS = $self->get('DefaultTaxClass');
    Error(
        'NO_DEFAULT_TAX_CLASS', {
            ObjectID => $self->id,
            Alias    => $self->alias
        }) unless defined $DefaultTaxClassUS;

    return $DefaultTaxClassUS;
}

sub fallbackTaxArea {
    my $self = shift;
    return $self->defaultTaxArea;
}

#========================================================================================
# function     taxRate
#----------------------------------------------------------------------------------------
# syntax       $TaxRate = $TaxMatrix->taxRate($Area, $Class);
#----------------------------------------------------------------------------------------
# description  Tax rate of matrix defined by tax area and class.
#----------------------------------------------------------------------------------------
# input        $Area | tax area | object
# input        $Class | tax class | object
# return       $TaxRate | percentage of tax (eg 0.16 for 16 %) | float
#========================================================================================
sub taxRate {
    my $self = shift;
    my ($Area, $Class, $Container) = @_;
    TestObject('TaxArea',  $Area);
    TestObject('TaxClass', $Class);

    # test if we have an entry in taxrate for this combination
    return GetInfoTaxRate($Area->id, $Class->id)->{'Rate'} if ExistsTaxRate($Area->id, $Class->id);

    # we have no address informations on TaxArea
    return 0 if $Area->isOriginBased == -1;

    # we use the tax rate by address
    if ($Area->isOriginBased) {
        my $Rate = $self->taxRateByAreaAddress($Area);
        return $Rate;
    }
    else {

        # destination based and we need a container for calculating taxes
        return 0 unless defined $Container;
        return $self->taxRateByBasketAddress($Area, $Container->get('BasketOrOrder'));
    }

    return 0;

}

sub taxRateByAreaAddress {
    my $self = shift;
    my ($Area) = @_;

    my $hInfo = $self->taxInfoByAreaAddress($Area);
    return $hInfo->{'SalesTaxRate'} / PERCENT;
}

sub taxInfoByAreaAddress {
    my $self = shift;
    my ($Area) = @_;

    my $PresenceAddress = $Area->get('PresenceAddress');

    # we have no address
    return {} unless defined $PresenceAddress;

    my $hParams = $PresenceAddress->get(['Zipcode', 'City', 'State']);
    $hParams->{'ZipCode'} = $hParams->{'Zipcode'};

    # get rate by address state,city and zip
    my $hInfo = GetInfoZip2TaxUSByZipCodeCityState($hParams);

    return $hInfo;
}

sub taxRateByBasketAddress {
    my $self = shift;
    my ($Area, $Basket) = @_;

    # BASKET: do we have already a basket address?
    my $TaxAddress;
    $TaxAddress = $Basket->get('ShippingAddress') // $Basket->get('BillingAddress');

    # USER: do we have already a user address?
    $TaxAddress = $Basket->get('User')->get('ShippingAddress') // $Basket->get('User')->get('BillingAddress') unless $TaxAddress;

    return 0 unless defined $TaxAddress;

    # get taxrate by address data
    my $hParams = $TaxAddress->get(['Zipcode', 'City', 'State']);

    $hParams->{'ZipCode'} = $hParams->{'Zipcode'};
    return 0 unless defined $hParams->{'ZipCode'};
    return 0 unless defined $hParams->{'City'};
    return 0 unless defined $hParams->{'State'};

    # get rate by address state,city and zip
    my $hInfo = GetInfoZip2TaxUSByZipCodeCityState($hParams);
    return $hInfo->{'SalesTaxRate'} / PERCENT;

}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::TaxUS::API::Object::TaxMatrixUS->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {%{$class->SUPER::default($Parent, $hInfo)},};
}

1;
