#========================================================================================
# package      DE_EPAGES::Tax::API::Object::TaxMethodTaxUS
# state        public
# base         DE_EPAGES::Tax::API::Object::TaxMethod
# description  object interface for TaxMethodTaxUS.
#========================================================================================
package DE_EPAGES::TaxUS::API::Object::TaxMethodTaxUS;
use base DE_EPAGES::Tax::API::Object::TaxMethod;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (Compare ElementOf);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::TaxUS::API::Constants qw (NO_US_PAYMENT_TAX);
use DE_EPAGES::TaxUSSystemDB::API::Zip2TaxUS qw(GetInfoZip2TaxUSByZipCodeCityState);    ## no critic qw(RequireCorrectImports)

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $TaxMethodTaxUS->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return undef; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMethodTaxUS = DE_EPAGES::Tax::API::Object::TaxMethodTaxUS->insert($hInfo);
# example      my $Class = LoadClassByAlias('TaxMethodTaxUS');
#               my $TaxMethodTaxUS = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertTaxMethodTaxUS' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $TaxMethodTaxUS | new object | object
# hook         OBJ_InsertTaxMethodTaxUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMethodID   - tax method id    - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertTaxMethodTaxUS', {'Object' => $self, 'TaxMethodID' => $self->id});
    return $self;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TaxMethodTaxUS->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateTaxMethodTaxUS' and 'OBJ_AfterUpdateTaxMethodTaxUS'.
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateTaxMethodTaxUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMethodTaxUSID   - tax method id    - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateTaxMethodTaxUS | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TaxMethodTaxUSID   - tax method id    - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateTaxMethodTaxUS', {'Object' => $self, 'Values' => $hValues, 'TaxMethodTaxUSID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateTaxMethodTaxUS', {'Object' => $self, 'Values' => $hValues, 'TaxMethodTaxUSID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     isActive
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $TaxMethodTaxUS->isActive;
#----------------------------------------------------------------------------------------
# description  Returns true if the TaxMethod is activated on the shop.
#----------------------------------------------------------------------------------------
# return       $isActive | 1 if active | boolean
#========================================================================================
sub isActive {
    my $self = shift;
    my $Shop = $self->getSite;

    return 1 if (defined $Shop->get('TaxMethod')) && $Shop->get('TaxMethod')->instanceOf('TaxMethodTaxUS');
    return 0;
}

#========================================================================================
# function     taxMatrix
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxMatrix = $TaxMethodTaxUS->taxMatrix;
#----------------------------------------------------------------------------------------
# description  Returns the TaxMatrixUS from Shop
#----------------------------------------------------------------------------------------
# return       $TaxMatrix | TaxMatrix | object
#========================================================================================
sub taxMatrix {
    my $self = shift;
    my $Shop = $self->getSite;

    return $Shop->child('TaxMatrixUS');
}

#========================================================================================
# function     isDefaultTaxType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShowSelection = $TaxMethodTaxUS->isDefaultTaxType;
#----------------------------------------------------------------------------------------
# description  True if tax method is now the default tyx type for the shop.
#----------------------------------------------------------------------------------------
# return       $IsDefaultTaxType | true or false  | boolean
#========================================================================================
sub isDefaultTaxType { return 1;}

#========================================================================================
# function     showTaxModelSelectionForCustomer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShowTaxModel = $TaxMethodTaxUS->showTaxModelSelectionForCustomer;
#----------------------------------------------------------------------------------------
# description  Returns true if the TaxModel selection can be visible in MBO.
#----------------------------------------------------------------------------------------
# return       $ShowTaxModel | 1 if active | boolean
#========================================================================================
sub showTaxModelSelectionForCustomer { return 0; }

#========================================================================================
# function     showTaxAreaSelectionForBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopTaxArea = $TaxMethodTaxUS->showTaxAreaSelectionForBasket;
#----------------------------------------------------------------------------------------
# description  Returns true if the TaxArea selection can be visible in MBO.
#----------------------------------------------------------------------------------------
# return       $ShopTaxArea | 1 if active | boolean
#========================================================================================
sub showTaxAreaSelectionForBasket { return 0; }

#========================================================================================
# function     showInAutomaticTaxCalculation
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Show = $TaxMethod->showInAutomaticTaxCalculation;
#----------------------------------------------------------------------------------------
# description  True if tax method selection can be shown in selection.
#----------------------------------------------------------------------------------------
# return       $Show | true or false  | boolean
#========================================================================================
sub showInAutomaticTaxCalculation {
    my $self = shift;
    my ($hInfo) = @_;

    my $Shop        = $self->getSite;
    my $Address     = $Shop->get('Address');
    my $ShopCountry = defined($Address) ? $Address->get('Country') : undef;

    return 1 if ( (defined $Address) && (defined $ShopCountry) && ($ShopCountry->{'Code2'} eq 'US') );
    return 0;
}

#========================================================================================
# function     disableShippingSplit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsDisabled = $TaxMethodTaxUS->disableShippingSplit;
#----------------------------------------------------------------------------------------
# description  Returns true if shipping splitting is disabled.
#----------------------------------------------------------------------------------------
# return       $IsDisabled | 1 if active | boolean
#========================================================================================
sub disableShippingSplit { return 1; }

#========================================================================================
# function     disablePaymentSplit
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsDisabled = $TaxMethodTaxUS->disablePaymentSplit;
#----------------------------------------------------------------------------------------
# description  Returns true if payment splitting is disabled.
#----------------------------------------------------------------------------------------
# return       $IsDisabled | 1 if active | boolean
#========================================================================================
sub disablePaymentSplit { return 1; }

#========================================================================================
# function     isShippingTaxable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsTaxable = $TaxMethodTaxUS->isShippingTaxable;
#----------------------------------------------------------------------------------------
# description  Returns true if shipping is taxable.
#----------------------------------------------------------------------------------------
# return       $IsTaxable | 1 if active | boolean
#========================================================================================
sub isShippingTaxable {
    my $self = shift;
    my ($Container) = @_;

    my $TaxArea = $Container->get('TaxArea');

    return $self->SUPER::isShippingTaxable() unless defined $TaxArea;
    return $self->SUPER::isShippingTaxable() unless $TaxArea->instanceOf('TaxAreaUS');

    # we do not recognize origin/destination based shop/customer addresses because
    # a whole state is shippingtaxable or not
    my $hInfo = $Container->get('TaxMatrix')->taxInfoByAreaAddress($TaxArea);
    return 0 unless exists $hInfo->{'ShippingTaxable'};
    return $hInfo->{'ShippingTaxable'};
}

#========================================================================================
# function     isPaymentTaxable
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsTaxable = $TaxMethodTaxUS->isPaymentTaxable;
#----------------------------------------------------------------------------------------
# description  Returns true if payment is taxable.
#----------------------------------------------------------------------------------------
# return       $IsTaxable | 1 if active | boolean
#========================================================================================
sub isPaymentTaxable {
    my $self = shift;
    my ($Container) = @_;

    my $TaxArea = $Container->get('TaxArea');
    return $self->SUPER::isPaymentTaxable() unless defined $TaxArea;
    return $self->SUPER::isPaymentTaxable() unless $TaxArea->instanceOf('TaxAreaUS');

    my $PresenceAddress = $TaxArea->get('PresenceAddress');
    return $self->SUPER::isPaymentTaxable() unless defined $PresenceAddress;

    # use list from constants
    return 0 if ElementOf($PresenceAddress->get('State'), NO_US_PAYMENT_TAX);
    return 1;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsUsable = $TaxMethodTaxUS->usableAtStorefront;
#----------------------------------------------------------------------------------------
# description  Returns true is method is usable.
#----------------------------------------------------------------------------------------
# return       $IsUsable | 1 if active | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hInfo) = @_;

    my $Shop        = $self->getSite;
    my $AddressValid = _TestShopAddress($Shop);

    return ( $AddressValid && $self->SUPER::usableAtStorefront($hInfo));
}

sub _TestShopAddress {
   my ($Shop) = @_;

    my $Address     = $Shop->get('Address');
    return 0 unless defined $Address;
    my $ShopCountry = $Address->get('Country');
    return 0 unless defined $ShopCountry;
    return 0 unless $ShopCountry->{'Code2'} eq 'US';

    # test if we have a valid zip2Tax shop address
    my $hAddressParams = $Address->get(['Zipcode', 'City', 'State']);
    $hAddressParams->{'ZipCode'} = $hAddressParams->{'Zipcode'};

    my $hInfoZip2Tax = GetInfoZip2TaxUSByZipCodeCityState($hAddressParams);
    my $AddressValid = exists $hInfoZip2Tax->{'ZipCode'} ? 1:0;

    return $AddressValid;
}

#========================================================================================
# function     updateContainerTaxArea
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsUpdated = $TaxMethodTaxUS->updateContainerTaxArea;
#----------------------------------------------------------------------------------------
# description  Updates the container taxarea by using the customer/user address.
#----------------------------------------------------------------------------------------
# input        $Container | LineitemContainer | object
#----------------------------------------------------------------------------------------
# return       $IsUpdated | 1 if active | boolean
#========================================================================================
sub updateContainerTaxArea {
    my $self = shift;
    my ($Container) = @_;
    return unless defined $Container;

    my $NewTaxArea = $self->taxAreaByAddress($Container);
    my $OldTaxArea = $Container->get('TaxArea');

    if (Compare('TaxArea', $OldTaxArea, $NewTaxArea)) {
        $Container->set({'TaxArea' => $NewTaxArea});
        $Container->recalculate;
        return 1;
    }

    return;
}

#========================================================================================
# function     taxAreaByAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TaxArea = $TaxMethodTaxUS->taxAreaByAddress;
#----------------------------------------------------------------------------------------
# description  Returns the TaxArea by the customer/user address.
#               Use non merchant state TaxArea if we have no informations about the customer.
#----------------------------------------------------------------------------------------
# input        $Container | LineitemContainer | object
#----------------------------------------------------------------------------------------
# return       $TaxArea | TaxArea | obejct
#========================================================================================
sub taxAreaByAddress {
    my $self        = shift;
    my ($Container) = @_;
    my $Basket      = $Container->get('BasketOrOrder');

    my $TaxAddress;

    # BASKET: do we have already a basket address?
    $TaxAddress = $Basket->get('ShippingAddress') // $Basket->get('BillingAddress');

    # USER: do we have already a user address?
    $TaxAddress = $Basket->get('User')->get('ShippingAddress') // $Basket->get('User')->get('BillingAddress') unless $TaxAddress;

    # test if we have a TaxArea for customers address
    # we need to test only the state here (use the merchants state, physical presences or none)
    # find area by address state
    my $RestOfWorldTaxArea;
    foreach my $TaxArea (@{$self->taxMatrix->get('TaxAreas')}) {
        my $AreaAddress = $TaxArea->get('PresenceAddress');
        if (not defined $AreaAddress) {
            $RestOfWorldTaxArea = $TaxArea;
            next;
        }
        else {
            return $TaxArea
            if (defined $TaxAddress)
            and ($AreaAddress->get('State') eq $TaxAddress->get('State'))
            and ($AreaAddress->get('CountryID') eq $TaxAddress->get('CountryID'));
        }
    }

    # return rest of world if we have no match here
    return $RestOfWorldTaxArea;
}

#========================================================================================
# function     IsMethodDirectlyUsable
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsUsable = $Package->isMethodDirectlyUsable($Shop);
#----------------------------------------------------------------------------------------
# description  Returns true if TaxMethod directly usable in the
#               shop without further configuration.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# return       $IsUsable | Method can be used | boolean
#========================================================================================
sub IsMethodDirectlyUsable {
    my $Package = shift;
    my ($Shop) = @_;

    return _TestShopAddress($Shop);
}

1;
