#========================================================================================
# package      DE_EPAGES::Test::Mock::Mockify
# state        public
#----------------------------------------------------------------------------------------
# description  frame work to create stable mocks/stubs
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::Test::Mock::Mockify;
#               my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
#                   'DE_EPAGES::Test::API::ModuleTwo',
#                   $aParameterList
#               );
#               my $MethodName = 'doWebservice';
#               my $ReturnValue = 'TestReturnValue';
#               my $aParameterTypes = [{'string' => 'aValue'}];
#               $Mockify->addMockWithReturnValueAndParameterCheck($MethodName, $ReturnValue, $aParameterTypes);
#               my $MockedModuleTwo = $Mockify->getMockObject();
#               my $ModuleOne = DE_EPAGES::Test::API::ModuleOne->new($MockedModuleTwo);
#               is( $ModuleOne->doSomeThingWithModuleTwo('aValue'), $ReturnValue, "$SubTestName - test if all is fine" );
#========================================================================================
package DE_EPAGES::Test::Mock::Mockify;
use base qw ( Exporter );

use DE_EPAGES::Core::API::Error qw ( Error ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid UsePackage IsA );
use DE_EPAGES::Test::Mock::Mockify::MethodCallCounter;
use DE_EPAGES::Core::API::TestParameter qw (
    TestArray
    TestHash
    TestInteger
    TestObject
    TestString
);
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );
use Test::MockObject::Extends;
use Data::Dumper;
use Data::Compare;

use strict;

our @EXPORT_OK = qw (
    GetParametersFromMockifyCall
    WasCalled
    GetCallCount
);

##no critic qw(ProhibitCascadingIfElse)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $FakeClassPath, $FakeParams );
#----------------------------------------------------------------------------------------
# description  build the module which controlls the mocked module
#----------------------------------------------------------------------------------------
# input        $FakeModulePath | path to a existing module | String
# input        $aFakeParams | parameter list for this module | refarray
# return       $self | self | DE_EPAGES::Test::Mock::Mockify
#========================================================================================
sub new {
    my $class = shift;
    my ( $FakeModulePath, $aFakeParams ) = @_;

    UsePackage( $FakeModulePath );
    my $FakeClass = $FakeModulePath->new( @{$aFakeParams} );
    my $self  = bless {}, $class;
    $self->{'__MockedModulePath'} = $FakeModulePath;
    $self->{'__MockedModule'} = Test::MockObject::Extends->new( $FakeClass );
    $self->_initMockedModule();


    return $self;
}
#========================================================================================
# function     _initMockedModule
# state        private
#----------------------------------------------------------------------------------------
# syntax       _initMockedModule( );
#----------------------------------------------------------------------------------------
# description  init mockify
#========================================================================================
sub _initMockedModule {
    my $self = shift;

    $self->{'__MockedModule'}->{'__MethodCallCounter'} = DE_EPAGES::Test::Mock::Mockify::MethodCallCounter->new();
    $self->{'__MockedModule'}->{'__isMockified'} = 1;
    $self->_addGetParameterFromMockifyCall();

    return;
}
#========================================================================================
# function     GetParametersFromMockifyCall
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetParametersFromMockifyCall( );
#----------------------------------------------------------------------------------------
# description  After mocking a method with Mockify framework and using the method,
#               you can use this function to retrive you the Parameters that were
#               given to the mocked method.
#               If the Mocked Method was called mutiple times you can assess the parameters
#               from a specific call.
#               If Position is not defined it tocks the parameters from the first call.
#               The position acts like an Array so first Element you will get with 0, the second with 1, and so on.
#----------------------------------------------------------------------------------------
# input        $MockifiedMockedObject | MockifiedMockedObject which was build with Mockify | object
# input        $MethodName | name of method | string
# input        $Position |  | Input1Type_boolean_integer_String_object_ref_hash_refarray
# return       Array with the parameters that were given to the mocked method | array
#========================================================================================
sub GetParametersFromMockifyCall {
    my ( $MockifiedMockedObject, $MethodName, $Position ) = @_;

    _TestMockifyObject( $MockifiedMockedObject );
    TestExistsMethod( $MockifiedMockedObject, '__getParametersFromMockifyCall' );
    if( not isValid( $Position ) ){
        $Position = 0;
    } else {
         TestInteger( 'Position', $Position );
    }
    return $MockifiedMockedObject->__getParametersFromMockifyCall( $MethodName, $Position );
}
#========================================================================================
# function     WasCalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       WasCalled( $MockifiedMockedObject, $MethodName );
#----------------------------------------------------------------------------------------
# description  returns true if the Method was called
#----------------------------------------------------------------------------------------
# input        $MockifiedMockedObject | MockifiedMockedObject which was build with Mockify | object
# input        $MethodName | name of method | string
# return       $WasCalled | was called | boolean
#========================================================================================
sub WasCalled {
    my ( $MockifiedMockedObject, $MethodName ) = @_;

    my $WasCalled;
    my $AmountOfCalles = GetCallCount( $MockifiedMockedObject, $MethodName );
    if($AmountOfCalles > 0){
        $WasCalled = 1;
    }else{
        $WasCalled = 0;
    }

    return $WasCalled;
}
#========================================================================================
# function     GetCallCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCallCount( $MockifiedMockedObject, $MethodName );
#----------------------------------------------------------------------------------------
# description  Return a number with the amount of calls, this mockified method was called.
#               Dies if the requested method was not Mockified.
#----------------------------------------------------------------------------------------
# input        $MockifiedMockedObject | MockifiedMockedObject which was build with Mockify | object
# input        $MethodName | name of method | string
# return       Amount Of Calls | integer
#========================================================================================
sub GetCallCount {
    my ( $MockifiedMockedObject, $MethodName ) = @_;

    _TestMockifyObject( $MockifiedMockedObject );
    return $MockifiedMockedObject->{'__MethodCallCounter'}->getAmountOfCalls( $MethodName );
}
#========================================================================================
# function     getMockObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       getMockObject( );
#----------------------------------------------------------------------------------------
# description  return the Mocked Module
#----------------------------------------------------------------------------------------
# return       $MockedModule | MockedModule | object
#========================================================================================
sub getMockObject {
    my $self = shift;
   return $self->{'__MockedModule'};
}
#========================================================================================
# function     mock
# state        public
#----------------------------------------------------------------------------------------
# syntax       mock( $MethodName, $ReturnValue, $aParameterTypes );
#----------------------------------------------------------------------------------------
# description  Based on the type and amout of parameters this is a shortcut for:
#               addMock                                 | mock('name', sub {})
#               addMockWithReturnValuemock              | mock('name', 'returnValue')
#               addMockWithReturnValueAndParameterCheck | mock('name', 'returnValue', [{'string'=>'jajaGenau'}])
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $ReturnValue | value which will be returned by the mocked method | integer String object refhash refarray
# input        $aParameterTypes | differnd parameter types | refarray
#========================================================================================
sub mock { ## no critic ( RequireCommentForExportedFunctions )
    my $self = shift;
    my @Parameters = @_;
    my $ParameterAmount = scalar @Parameters;
    if($ParameterAmount == 2){
        my ( $MethodName, $ReturnValueOrFunctionPointer ) = @Parameters;
        if( ref($ReturnValueOrFunctionPointer) eq 'CODE' ){
            $self->addMock($MethodName, $ReturnValueOrFunctionPointer);
        }else{
            $self->addMockWithReturnValue($MethodName, $ReturnValueOrFunctionPointer);
        }
    }
    if($ParameterAmount == 3){
        my ( $MethodName, $ReturnValue, $aParameterTypes ) = @_;
        $self->addMockWithReturnValueAndParameterCheck($MethodName, $ReturnValue, $aParameterTypes);
    }
    return;
}
#========================================================================================
# function     addMock
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMock( $MethodName, $rSub );
#----------------------------------------------------------------------------------------
# description  overrides a method in the MockedModule.
#               Throws Error if method not exists in the MockedModule.
#
#               $MockObject->addMock('myMethod', sub {
#                   # Your implementation
#                   }
#               );
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $rSub | function pointer which will be used for overriding | refSub
#========================================================================================
sub addMock {
    my $self = shift;
    my ( $MethodName, $rSub ) = @_;

    $self->_testExistsMethod( $self->{'__MockedModule'}, $MethodName );
    $self->{'__MockedModule'}->{'__MethodCallCounter'}->addMethod( $MethodName );
    $self->{'__MockedModule'}->mock( $MethodName, sub {
        $self->{'__MockedModule'}->{'__MethodCallCounter'}->increment( $MethodName );
        return $rSub->( @_ );
    } );

    return;
}
#========================================================================================
# function     addMockWithReturnValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMockWithReturnValue( $MethodName, $ReturnValue );
#----------------------------------------------------------------------------------------
# description  add a Method to the mocked module which always returns the $ReturnValue.
#               The added Method will throw an Error if it was called with a Parameter.
#
#               $Mockify->addMockWithReturnValue('myMethod','the return value');
#
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $ReturnValue | value which will be returned by the mocked method | integer String object refhash refarray
#========================================================================================
sub addMockWithReturnValue {
    my $self = shift;
    my ( $MethodName, $ReturnValue ) = @_;

    $self->addMock($MethodName, sub {
        my $MockedSelf = shift;
        $self->_unexpectedParameterCheck( $MethodName, \@_ );
        return $ReturnValue;
    } );

    return;
}
#========================================================================================
# function     addMethodSpy
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMethodSpy( $MethodName );
#----------------------------------------------------------------------------------------
# description  Give option to observe a Method while keeping the original functionality.
#----------------------------------------------------------------------------------------
# input        $MethodName | $MethodName | string
#========================================================================================
sub addMethodSpy {
    my $self = shift;
    my ( $MethodName ) = @_;

    my $PointerOriginalMethod = \&{$self->{'__MockedModulePath'}.'::'.$MethodName};
    $self->addMock( $MethodName, sub {
        $PointerOriginalMethod->( @_ );
    } );

    return;
}
#========================================================================================
# function     addMethodSpyWithParameterCheck
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMethodSpyWithParameterCheck( $MethodName, $aParameterTypes );
#----------------------------------------------------------------------------------------
# description  Give option to observe a Method while keeping the original functionality.
#               but also check the parameters
#               check, based on the $aParameterTypes if it was called with the correct parameters
#
#               my $aParameterTypes = ['string',{'string' => 'ABCD'}];
#               $Mockify->addMockWithReturnValueAndParameterCheck('myMethod','the return value',$aParameterTypes);
#               my $MyFakeObject = $MockObject->getMockObject();
#               ok( $MyModuleObject->myMethod('Hello','ABCD') ),
#
#               possible parameters types are:
#                   ['string', 'int', 'hashref', 'arrayref', 'object', 'undef', 'any']
#               or with more detail:
#                   [{'string'=>'abcdef'}, {'int' => 123}, {'hashref' => {'key'=>'value'}}, {'arrayref'=>['one', 'two']}, {'object'=> 'PAth::to:Obejct}]
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $aParameterTypes | differnd parameter types | refarray
#========================================================================================
sub addMethodSpyWithParameterCheck {
    my $self = shift;
    my ( $MethodName, $aParameterTypes ) = @_;

    $self->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
    my $PointerOriginalMethod = \&{$self->{'__MockedModulePath'}.'::'.$MethodName};
     $self->addMock( $MethodName, sub {
            my $MockedSelf = shift;
            my @MockedParameters = @_;
            $self->_storeParameters( $MethodName, $MockedSelf, \@MockedParameters );
            $self->_testParameterTypes( $MethodName , $aParameterTypes, \@MockedParameters );
            $self->_testParameterAmount( $MethodName , $aParameterTypes, \@MockedParameters );
            $PointerOriginalMethod->($MockedSelf, @MockedParameters);
    } );
    return;
}
#========================================================================================
# function     addMockWithReturnValueAndParameterCheck
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMockWithReturnValueAndParameterCheck( $MethodName, $ReturnValue, $aParameterTypes );
#----------------------------------------------------------------------------------------
# description  will add a Method to the mocked module
#               will return the $ReturnValue
#               check, based on the $aParameterTypes if it was called with the correct parameters
#
#               my $aParameterTypes = ['string',{'string' => 'ABCD'}];
#               $Mockify->addMockWithReturnValueAndParameterCheck('myMethod','the return value',$aParameterTypes);
#               my $MyFakeObject = $MockObject->getMockObject();
#               ok( $MyModuleObject->myMethod('Hello','ABCD') ),
#
#               possible parameters types are:
#                   ['string', 'int', 'hashref', 'arrayref', 'object', 'undef', 'any']
#               or with more detail:
#                   [{'string'=>'abcdef'}, {'int' => 123}, {'hashref' => {'key'=>'value'}}, {'arrayref'=>['one', 'two']}, {'object'=> 'PAth::to:Obejct}]
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $ReturnValue | value which will be returned by the mocked method | integer String object refhash refarray
# input        $aParameterTypes | differnd parameter types | refarray
#========================================================================================
sub addMockWithReturnValueAndParameterCheck {
    my $self = shift;
    my ( $MethodName, $ReturnValue, $aParameterTypes ) = @_;

    $self->_checkParameterTypesForMethod( $MethodName , $aParameterTypes );
    $self->addMock(
        $MethodName,
        sub{
            my $MockedSelf = shift;
            my @Parameters = @_;
            $self->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
            $self->_testParameterTypes( $MethodName , $aParameterTypes, \@Parameters );
            $self->_testParameterAmount( $MethodName , $aParameterTypes, \@Parameters );
            return $ReturnValue;
        }
    );

    return;
}
#========================================================================================
# function     addMockWithConditionalReturn
# state        public
#----------------------------------------------------------------------------------------
# syntax       addMockWithConditionalReturn( $MethodName, $ReturnValue, $aParameterMatchers );
#----------------------------------------------------------------------------------------
# description  will add a Method to the mocked module
#               will return the $ReturnValue if it was called with the matching parameters.
#               can be called multiple times to implement different return values for different parameters.
#
#               my $aParameterMatchers = ['string',{'string' => 'ABCD'}];
#               my $aParameterMatchers2 = ['string',{'string' => 'EFGH'}];
#               $Mockify->addMockWithConditionalReturn('myMethod','the return value',$aParameterMatchers);
#               $Mockify->addMockWithConditionalReturn('myMethod','the second return value',$aParameterMatchers2);
#               my $MyFakeObject = $MockObject->getMockObject();
#               $MyModuleObject->myMethod('Hello','ABCD'); #will return 'the return value'
#               $MyModuleObject->myMethod('Hello','EFGH'); #will return 'the second return value'
#
#               possible parameters types are:
#                   ['string', 'int', 'hashref', 'arrayref', 'object', 'undef', 'any']
#               or with more detail:
#                   [{'string'=>'abcdef'}, {'int' => 123}, {'hashref' => {'key'=>'value'}}, {'arrayref'=>['one', 'two']}, {'object'=> 'PAth::to:Obejct}]
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $ReturnValue | value which will be returned by the mocked method if the parameters match the matchers | integer String object refhash refarray
# input        $aParameterMatchers | differnd parameter types | refarray
#========================================================================================
sub addMockWithConditionalReturn {
    my $self = shift;
    my ( $MethodName, $ReturnValue, $aParameterMatchers ) = @_;

    $self->_checkParameterTypesForMethod( $MethodName , $aParameterMatchers );
    $self->_saveMatchers( $MethodName , $ReturnValue, $aParameterMatchers );
    $self->addMock(
        $MethodName,
        sub{
            my $MockedSelf = shift;
            my @Parameters = @_;
            $self->_storeParameters( $MethodName, $MockedSelf, \@Parameters );
            return $self->_getConditionalReturnValue( $MethodName , \@Parameters );
        }
    );

    return;
}

sub _saveMatchers {
    my $self = shift;
    my ( $MethodName, $ReturnValue, $aParameterMatchers ) = @_;

    my $aMatchers = $self->{$MethodName.'_Matchers'};
    $aMatchers //= [];

    my $hMatcherEntry = {
        'Matchers' => $aParameterMatchers,
        'ReturnValue' => $ReturnValue
    };

    foreach my $hExistingMatcherEntry (@{$aMatchers}){
        my $Compare = Data::Compare->new();
        if( $Compare->Cmp($aParameterMatchers, $hExistingMatcherEntry->{'Matchers'}) ){
            my $DumpedValue = Dumper( $aParameterMatchers );
            Error( 'MatcherAlreadyExists', {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'Matcher' => $DumpedValue
            } );
        }
    }

    push(@{$aMatchers}, $hMatcherEntry);
    $self->{$MethodName.'_Matchers'} = $aMatchers;

    return;
}

sub _getConditionalReturnValue {
    my $self = shift;
    my ( $MethodName , $aMockedParameters ) = @_;

    my $aMatchers = $self->{$MethodName.'_Matchers'};
    foreach my $hMatcherEntry (@{$aMatchers}){
        my $ReturnValue = eval {
            $self->_testParameterTypes( $MethodName , $hMatcherEntry->{'Matchers'}, $aMockedParameters );
            $self->_testParameterAmount( $MethodName , $hMatcherEntry->{'Matchers'}, $aMockedParameters );
            return $hMatcherEntry->{'ReturnValue'};
        };
        if (not ExistsError()) {
            return $ReturnValue;
        }
    }

    Error( 'NoMatchFound', {
        'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)"
    } );
}

#========================================================================================
# function     _storeParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       _storeParameters( $MethodName, $MockedSelf, $aMockedParameters );
#----------------------------------------------------------------------------------------
# description  store the parameters of a call, also if this method will be called multiple times
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $MockedSelf | the self of the object which will be build | object
# input        $aMockedParameters | pointer to the parameter array | refarray
#========================================================================================
sub _storeParameters {
    my $self = shift;
    my ( $MethodName, $MockedSelf, $aMockedParameters ) = @_;

    push( @{$MockedSelf->{$MethodName.'_MockifyParams'}}, $aMockedParameters );

    return;
}

#========================================================================================
# function     _testParameterTypes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testParameterTypes( $MethodName , $aExpectedParameterTypes, $aActualInputParameters );
#----------------------------------------------------------------------------------------
# description  test the types of the parameters
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $aExpectedParameterTypes | expected types of parameter | refarray
# input        $aActualInputParameters | actuall input paramaters | refarray
#========================================================================================
sub _testParameterTypes {
    my $self = shift;
    my ( $MethodName, $aExpectedParameterTypes, $aActualInputParameters ) = @_;

    my $MockedParametersSize = scalar @{$aActualInputParameters};
    for ( my $i = 0; $i < $MockedParametersSize; $i++ ) { ## no critic ( ProhibitCStyleForLoops )
        my $TypeTestResult = $self->_testParameterType("Parameter[$i]", $aActualInputParameters->[$i], $aExpectedParameterTypes->[$i], $MethodName );
        if ( ! $TypeTestResult ){
             Error( 'UnknownParametertype', {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'UnknownParameterType' => $self->_getParameterType( $aExpectedParameterTypes->[$i] ),
                'ParameterNumber'=> $i,
            } );
        }
    }

    return;
}

#========================================================================================
# function     _testParameterType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testParameterType( $ParameterName, $Value, $TestParameter, $MethodName );
#----------------------------------------------------------------------------------------
# description  test the type of a parameter
#----------------------------------------------------------------------------------------
# input        $ParameterName | name of parameter | String
# input        $Value | value of parameter | boolean integer string object refhash refarray
# input        $TestParameter | the type of the value and evtually the actuall value | int string object hashref arrayref undef any
# input        $MethodName | name of method | String
# return       0 or 1 | boolean
#========================================================================================
sub _testParameterType {
    my $self = shift;
    my ( $ParameterName, $Value, $TestParameter, $MethodName ) = @_;

    my $TestParameterType = $self->_getParameterType( $TestParameter );
    if( $TestParameterType eq  'string' ) {
        $self->_testExpectedString( $ParameterName,$Value, $TestParameter ,$MethodName );
    }
    elsif( $TestParameterType eq  'int' ) {
        $self->_testExpectedInt( $ParameterName,$Value, $TestParameter, $MethodName );
    }
    elsif( $TestParameterType eq  'hashref' ) {
        $self->_testExpectedHashRef( $ParameterName,$Value, $TestParameter, $MethodName );
    }
    elsif( $TestParameterType eq  'arrayref' ) {
        $self->_testExpectedArrayRef( $ParameterName,$Value, $TestParameter, $MethodName );
    }
    elsif( $TestParameterType eq  'object' ) {
        $self->_testExpectedObject( $ParameterName,$Value, $TestParameter,$MethodName );
    }
    elsif( $TestParameterType eq  'undef' ) {
        $self->_testUndefind( $ParameterName,$Value,$MethodName );
    }
    elsif( $TestParameterType eq  'sub' ) {
        $self->_testFunctionPointer( $ParameterName,$Value,$MethodName );
    }
    elsif( $TestParameterType eq  'any' ) {
        return 1;
    }
    else {
       return 0;
    }

    return 1;
}

#========================================================================================
# function     _addGetParameterFromMockedCall
# state        private
#----------------------------------------------------------------------------------------
# syntax       _addGetParameterFromMockedCall( );
#----------------------------------------------------------------------------------------
# description  add the helper method getParametersFromMockedCall to the MockedModule
#========================================================================================
sub _addGetParameterFromMockifyCall {
    my $self = shift;

    $self->{'__MockedModule'}->mock('__getParametersFromMockifyCall',sub{
                my $MockedSelf = shift;
                my ( $MethodName, $Position ) = @_;
                my $aParametersFromAllCalls = $MockedSelf->{$MethodName.'_MockifyParams'};
                if( ref $aParametersFromAllCalls ne 'ARRAY' ){
                    Error( "$MethodName was not called" );
                }
                if( scalar @{$aParametersFromAllCalls} < $Position ) {
                     Error( "$MethodName was not called ".( $Position+1 ).' times',{
                             'Method' => "$MethodName(???)",
                             'Postion' => $Position,
                         } );
                }
                else {
                     my $ParameterFromMockifyCall = $MockedSelf->{$MethodName.'_MockifyParams'}[$Position];
                     return $ParameterFromMockifyCall;
                }
            } # END of sub ref
       );

    return;
}

#========================================================================================
# function     _getParameterType
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getParameterType( $TestParameter );
#----------------------------------------------------------------------------------------
# description  extracts the parameter type from $TestParameter
#----------------------------------------------------------------------------------------
# input        $TestParameter | input test parameter type | String refhash
# return       $TestParameterType | parameter type | String
#========================================================================================
sub _getParameterType {
    my $self = shift;
    my ( $TestParameter ) = @_;

    my $TestParameterType = undef;
    if( $self->_isHash( $TestParameter ) ){
        my @Keys = keys %{$TestParameter};
        $TestParameterType = $Keys[0];
    } else {
        $TestParameterType = $TestParameter;
    }

    return $TestParameterType;
}

#========================================================================================
# function     _isHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHash( $TestValue );
#----------------------------------------------------------------------------------------
# description  checks if $TestValue is a Hash
#----------------------------------------------------------------------------------------
# input        $TestValue | value which will be checked | refhash
# return       True or false | Boolean
#========================================================================================
sub _isHash {
    my $self = shift;
    my ( $TestValue ) = @_;
    if( ref( $TestValue ) eq 'HASH' ){
        return 1;
    }else{
        return 0;
    }
}

#========================================================================================
# function     _testParameterAmount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testParameterAmount( $MethodName , $aExpectedParameterTypes, $aActualInputParameters );
#----------------------------------------------------------------------------------------
# description  throws an error if the amount of parameters is not as expected
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $aExpectedParameterTypes | expected parameter types | refarray
# input        $aActualInputParameters | actual input parameters | refarray
#========================================================================================
sub _testParameterAmount {
    my $self = shift;
    my ( $MethodName , $aExpectedParameterTypes, $aActualInputParameters ) = @_;

    my $AmountExpectedParameterTypes = scalar @{$aExpectedParameterTypes};
    my $AmountActualInputParameters = scalar @{$aActualInputParameters};

    if( $AmountActualInputParameters < $AmountExpectedParameterTypes ){
        Error( 'WrongAmountOfParameters', {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'ExpectedAmount' => $AmountExpectedParameterTypes,
                'ActualAmount' => $AmountActualInputParameters,
            } );
    }

    return;
}

#========================================================================================
# function     _testString
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testString( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not a string
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | String
# input        $MethodName | name of method | String
#========================================================================================
sub _testString {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    return if ( $Value eq '' ); #empty string is ok
    eval {
        TestString( $Name, $Value );
    };
    if ( ExistsError() ) {
        Error( "$Name is not a String", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        } );
    }
    return;
}

#========================================================================================
# function     _testExpectedString
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExpectedString( $Name, $Value, $TestParameterType, $MethodName );
#----------------------------------------------------------------------------------------
# description  tests if Value is an String and as expected
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | String
# input        $TestParameterType | value which will be checked | String refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testExpectedString {
    my $self = shift;
    my ( $Name, $Value, $TestParameterType, $MethodName ) = @_;

    $self->_testString( $Name, $Value, $MethodName );
    if( $self->_isHash( $TestParameterType ) ){
        my @Values = values %{$TestParameterType};
        my $ExpectedValue = $Values[0];
        if( $Value ne $ExpectedValue ){
            Error( "$Name unexpected value", {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'Value' => $Value,
                'ExpectedValue' => $ExpectedValue,
            } );
        }
    }

    return;
}

#========================================================================================
# function     _testInteger
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testInteger( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not an integer
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | integer
# input        $MethodName | name of method | String
#========================================================================================
sub _testInteger {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    eval {
        TestInteger( $Name, $Value );
    };
    if ( ExistsError() ) {
        Error( "$Name is not a Integer", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        } );
    }

    return;
}

#========================================================================================
# function     _testExpectedInt
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExpectedInt( $Name, $Value, $TestParameterType, $MethodName );
#----------------------------------------------------------------------------------------
# description  tests if Value is an Integer and as expected
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | Integer
# input        $TestParameterType | value which will be checked | Integer refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testExpectedInt {
    my $self = shift;
    my ( $Name, $Value, $TestParameterType, $MethodName ) = @_;

    $self->_testInteger( $Name, $Value, $MethodName );
    if( $self->_isHash( $TestParameterType ) ){
        my @Values = values %{$TestParameterType};
        my $ExpectedValue = $Values[0];
        if( $Value != $ExpectedValue ){
            Error( "$Name unexpected value", {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'Value' => $Value,
                'ExpectedValue' => $ExpectedValue,
            } );
        }
    }

    return;
}

#========================================================================================
# function     _testUndefind
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testUndefind( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not undefined
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | integer String object refhash refarray
# input        $MethodName | name of method | String
#========================================================================================
sub _testUndefind {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    if ( isValid( $Value ) ) {
        Error( "$Name is not undefined", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        });
    }

    return;
}

#========================================================================================
# function     _testFunctionPointer
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testFunctionPointer( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not an function pointer
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | integer String object refhash refarray
# input        $MethodName | name of method | String
#========================================================================================
sub _testFunctionPointer {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    if(ref($Value) ne 'CODE') {
        Error( "$Name is not FunctionPointer", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        });
    }

    return;
}


#========================================================================================
# function     _testHashRef
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testHashRef( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not a hash reference
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testHashRef {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    eval {
        TestHash( $Name, $Value );
    };
    if ( ExistsError() ) {
        Error( "$Name is not a HashRef", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        });
    }

    return;
}

#========================================================================================
# function     _testExpectedHashRef
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExpectedHashRef( $Name, $Value, $TestParameterType, $MethodName );
#----------------------------------------------------------------------------------------
# description  tests if Value is an hash reference and as expected
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | refhash
# input        $TestParameterType | value which is expected | refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testExpectedHashRef {
    my $self = shift;
    my ( $Name, $Value, $TestParameterType, $MethodName ) = @_;

    $self->_testHashRef( $Name, $Value, $MethodName );
    if( $self->_isHash( $TestParameterType ) ){
        my @Values = values %{$TestParameterType};
        my $ExpectedValue = $Values[0];
        my $Compare = Data::Compare->new();
        if( not $Compare->Cmp($Value,$ExpectedValue) ){
        my $DumpedValue = Dumper( $Value );
        my $DumpedExpected = Dumper( $ExpectedValue );
            Error( "$Name unexpected value", {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'got value' => $DumpedValue,
                'expected value' => $DumpedExpected,
            } );
        }
    }

    return;
}

#========================================================================================
# function     _testArrayRef
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testArrayRef( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not an array reference
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | refarray
# input        $MethodName | name of method | String
#========================================================================================
sub _testArrayRef {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    eval {
        TestArray( $Name, $Value );
    };
    if ( ExistsError() ) {
        Error( "$Name is not a ArrayRef", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        } );
    }

    return;
}

#========================================================================================
# function     _testExpectedArrayRef
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExpectedArrayRef( $Name, $Value, $TestParameterType, $MethodName );
#----------------------------------------------------------------------------------------
# description  tests if Value is an array reference and as expected
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | refarray
# input        $TestParameterType | value which will be checked | refarray refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testExpectedArrayRef {
    my $self = shift;
    my ( $Name, $Value, $TestParameterType, $MethodName ) = @_;

    $self->_testArrayRef( $Name, $Value, $MethodName );
    if( $self->_isHash( $TestParameterType ) ){
        my @Values = values %{$TestParameterType};
        my $ExpectedValue = $Values[0];
        my $Compare = Data::Compare->new();
        if( not $Compare->Cmp($Value,$ExpectedValue) ){
        my $DumpedValue = Dumper( $Value );
        my $DumpedExpected = Dumper( $ExpectedValue );
            Error( "$Name unexpected value", {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'got value' => $DumpedValue,
                'expected value' => $DumpedExpected,
            } );
        }
    }

    return;
}
#========================================================================================
# function     _testObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testObject( $Name, $Value, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if $Value is not an object
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | integer String object refhash refarray
# input        $MethodName | name of method | String
#========================================================================================
sub _testObject {
    my $self = shift;
    my ( $Name, $Value, $MethodName ) = @_;

    eval {
        TestObject( $Name, $Value );
    };
    if ( ExistsError() ) {
        Error( "$Name is not a Object", {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
            'Value' => $Value
        } );
    }

    return;
}

#========================================================================================
# function     _testExpectedObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExpectedObject( $Name, $Value, $TestParameterType, $MethodName );
#----------------------------------------------------------------------------------------
# description  tests if Value is an object reference and as expected
#----------------------------------------------------------------------------------------
# input        $Name | name of variable | String
# input        $Value | value which will be checked | Object
# input        $TestParameterType | value which will be checked | Object refhash
# input        $MethodName | name of method | String
#========================================================================================
sub _testExpectedObject {
    my $self = shift;
    my ( $Name, $Value, $TestParameterType, $MethodName ) = @_;

    $self->_testObject( $Name, $Value, $MethodName );
    if( $self->_isHash( $TestParameterType ) ){
        my @Values = values %{$TestParameterType};
        my $ExpectedValue = $Values[0];
        if( !IsA( $Value, $ExpectedValue ) ){
            Error( "$Name unexpected value", {
                'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
                'Value' => $Value,
                'ExpectedObjectType' => $ExpectedValue,
            } );
        }
    }

    return;
}

#========================================================================================
# function     _checkParameterTypesForMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _checkParameterTypesForMethod( $MethodName, $aParameterTypes );
#----------------------------------------------------------------------------------------
# description  check if $aParameterTypes is defined or is not an Array
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# input        $aParameterTypes | differnd parameter types | refarray
#========================================================================================
sub _checkParameterTypesForMethod {
    my $self = shift;
    my ( $MethodName, $aParameterTypes ) = @_;

    if ( not ( defined $aParameterTypes ) or ref( $aParameterTypes ) ne 'ARRAY' ){
        Error( 'ParameterTypesNotProvided', {
            'Method' => $self->{'__MockedModulePath'}."->$MethodName(???)",
        } );
    }

    return;
}
#========================================================================================
# function     _TestMockifyObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TestMockifyObject( $MockifiedMockedObject );
#----------------------------------------------------------------------------------------
# description  tests if a Object is aMockified object
#----------------------------------------------------------------------------------------
# input        $MockifiedMockedObject | a mockified object | object
# return       is a MockifiedObject | boolean
#========================================================================================
sub _TestMockifyObject {
    my ( $MockifiedMockedObject ) = @_;

    my $ObjectPath = ref( $MockifiedMockedObject );
    if( not isValid( $ObjectPath ) ){
        Error( 'Object is not defined' );
    }
    if ( $MockifiedMockedObject->{'__isMockified'} != 1){
        Error( "The Object: '$ObjectPath' is not mockified" );
    }

    return;
}
#========================================================================================
# function     _testExistsMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       _testExistsMethod( $ModulePath, $MethodName );
#----------------------------------------------------------------------------------------
# description  throws an error if the Module dont have the expected Method
#----------------------------------------------------------------------------------------
# input        $ModulePath | path to a module | String
# input        $MethodName | name of method | String
#========================================================================================
sub _testExistsMethod {
    my $self = shift;
    my ( $ModulePath, $MethodName ) = @_;

    if( not $ModulePath->can( $MethodName ) ){
        Error( $self->{'__MockedModulePath'}." don't have a method like: $MethodName" );
    }

    return;
}
#========================================================================================
# function     _unexpectedParameterCheck
# state        private
#----------------------------------------------------------------------------------------
# syntax       _unexpectedParameterCheck( $MethodName, $aParameterListe );
#----------------------------------------------------------------------------------------
# description  throws Error if there is an unexpected Parameter
#----------------------------------------------------------------------------------------
# input        $MethodName | name of method | String
# return       $aParameterListe | Parameter from a Call (without self) | refarray
#========================================================================================
sub _unexpectedParameterCheck {  ## no critic ( RequireCommentForExportedFunctions )
    my $self = shift;
    my ( $MethodName, $aParameterListe ) = @_;
    my $ParameterListSize = scalar @{$aParameterListe};
    if ( $ParameterListSize > 0 ){
        Error('UnexpectedParameter',{
            'Method' => "$self->{'__MockedModulePath'}->$MethodName(???)",
            'ParameterList' => "(@_)",
            'AmountOfUnexpectedParameters' => $ParameterListSize,
        } );
    }
}
1;
