#=============================================================================
# package      DE_EPAGES::Twenga::API::TwengaRequest
# state        private
#-----------------------------------------------------------------------------
# description  This package provides the Twenga request functions.
#=============================================================================

## no critic( RequireCorrectImports RequireArgUnpacking RequireExistsError )

package DE_EPAGES::Twenga::API::TwengaRequest;

use base qw( Exporter );

use strict;
our @EXPORT_OK = qw (GetRedirectUrl GetTrackingScript GetSiteExists CallTwenga);

use LWP::UserAgent;

use DE_EPAGES::Twenga::API::Impl;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Twenga::API::Twenga qw ( TwengaLog );
use DE_EPAGES::Twenga::API::Mail qw ( SendTwengaErrorMail );
use DE_EPAGES::Twenga::API::Constants qw(
    ERR_TWENGA_ACCOUNT_EXPIRED
    ERR_TWENGA_INVALID_HASHKEY
    ERR_TWENGA_UNDEF_USERNAME
    ERR_TWENGA_UNDEF_PASSWORD
);

sub _get_credentials {
    my( $PortalConfig ) = @_;
    my @keys_src = qw/PortalMerchantID TwengaMerchantUsername TwengaMerchantPassword/;
    my @keys_dst = qw/merchant_hashkey merchant_login         merchant_password/;
    my $hCredentials = $PortalConfig->get( \@keys_src );
    my %ret; @ret{ @keys_dst } = @$hCredentials{ @keys_src };
    return \%ret;
}

sub _do_req {
    my( $auth, $noun, $verb, $parms, $PortalConfig ) = @_;
    # One Perl process may serve several providers. Hence the key must not
    # persist between requests: next time around, it may be on behalf of
    # another provider.
    $auth->{partner_auth_key} = LoadRootObject()->get('TwengaPartnerAuthKey');

    my $Twenga = Twenga->new( $auth );
    my $Request = $Twenga->make_req( $noun, $verb, $parms );
    TwengaLog "$noun/$verb", $parms;
    TwengaLog Request => $Request->as_string;

    my $ua = LWP::UserAgent->new;
    my $Response = $ua->request( $Request );
    TwengaLog Response => $Response->as_string;

    if( !$Response->is_success ) {
        return {
            error => 'HTTP_ERROR',
            errormessage => $Response->status_line
        };
    }

    my $hResult = $Twenga->data( $Response );
    TwengaLog Result => $hResult;

    _HandleError( $PortalConfig, $hResult ) if $hResult->{error};

    return $hResult;
}

#=============================================================================
# function     GetRedirectUrl
# state        public
#-----------------------------------------------------------------------------
# syntax       $hResult = GetRedirectUrl( $PortalConfig, $hParams );
# example      $Servlet->redirect( GetRedirectUrl( $PortalConfig, $hParams )->{content} );
#-----------------------------------------------------------------------------
# description  execute twenga site redirect request
#-----------------------------------------------------------------------------
# input        $PortalConfig | the portal config object | object
# input        $hParams | params for the call | hash
# return       $hResult | result hash ref. The redirection URL is the value
#                   of the key 'content' | ref.hash
#=============================================================================
sub GetRedirectUrl{
    my( $PortalConfig, $hParams ) = @_;
    return _do_req {}, Site => GetSubscriptionLink => $hParams, $PortalConfig;
}

#=============================================================================
# function     GetSiteExists
# state        public
#-----------------------------------------------------------------------------
# syntax       $hResult = GetSiteExists( $PortalConfig, $hAuth );
#-----------------------------------------------------------------------------
# description  execute twenga site exists request
#-----------------------------------------------------------------------------
# input        $PortalConfig | the portal config object | object
# input        $hAuth | authethication data (keys: merchant_login, merchant_password, merchant_hashkey)| ref.hash
# return       $hResult | result hash ref | ref.hash
#=============================================================================
sub GetSiteExists{
    my( $PortalConfig, $hAuth ) = @_;
    return _do_req $hAuth, Site => Exist => {}, $PortalConfig;
}

#=============================================================================
# function     GetTrackingScript
# state        public
#-----------------------------------------------------------------------------
# syntax       $Script = GetTrackingScript( $PortalConfig, $hParams );
#-----------------------------------------------------------------------------
# description  returns the Javascript code for twenga tracking
#-----------------------------------------------------------------------------
# input        $PortalConfig | the portal config object | object
# input        $hParams | params for the call | ref.hash
# return       $Script | tracking script | string
#=============================================================================
sub GetTrackingScript{
    my ( $PortalConfig, $hParams ) = @_;
    my $auth = _get_credentials $PortalConfig;
    my $rslt = _do_req $auth, Site => GetTrackingScript => $hParams, $PortalConfig;
    return '' if $@ or $rslt->{error};
    _applyJavascriptPatch( \$rslt->{content} );
    return $rslt->{content};
}

# Replace some JS to prevent HTTP requests being issued from an HTTPS page.
# See EPG-17836 and my comments there for more info. -- MiLu 20.05.11
sub _applyJavascriptPatch {
    my ($sref) = @_;
    my $patch = <<'EOS';
function twengaSend() {
       if (clientTwenga) {
               var data = '{';
               for(var i=0; i < this.aOrder.length; i++) {
                       if(i > 0) {
                               data += ', "item'+i+'"';
                       } else {
                               data += '"order"';
                       }
                       data += ': '+this.aOrder[i];
               }
               data += '}';
               var data = 'obj='+r(data, true);
        // HTTPS au lieu de HTTP
               document.write('<img src="https://tracker.twenga.de/rts_log.php?'+data+'" border="0" width="1" height="1" title="" alt="" />');
       }
}
EOS

    #In the following regexes the # is used as regex-operator.
    $$sref =~ s#\bhttp://#https://#g; # replaces http with https in the whole string
    $$sref =~ s#var track = new twengaTrack\(\);\K#\n$patch\n#; # inserts $patch after 'var track = new twengaTrack();'
    return;
}

#=============================================================================
# function     CallTwenga
# state        public
#-----------------------------------------------------------------------------
# syntax       $hResult = CallTwenga( $PortalConfig, $noun, $verb, $parms );
#-----------------------------------------------------------------------------
# description  make call to Twenga API
#-----------------------------------------------------------------------------
# input        $PortalConfig | the portal config object | object
# input        $noun | noun for the call | string
# input        $verb | verb for the call | string
# input        $parms | parameters for the call | hash
# return       $hResult | result hash ref | ref.hash
#=============================================================================
sub CallTwenga {
    my ( $PortalConfig, $noun, $verb, $parms ) = @_;
    my $auth = _get_credentials $PortalConfig;
    return _do_req $auth, $noun, $verb, $parms, $PortalConfig;
}

#=============================================================================
# function     _HandleError
# state        private
#-----------------------------------------------------------------------------
# syntax       _HandleError($PortalConfig, $hErr);
#-----------------------------------------------------------------------------
# description  handles a communication error with the Twenga server
#-----------------------------------------------------------------------------
# input        $PortalConfig | the portal config object | object
# input        $hErr | error hash ref (keys: error, errormessage) | ref.hash
#=============================================================================
sub _HandleError{
    my ( $PortalConfig, $hErr ) = @_;
    TwengaLog( Error => $hErr );

    my $err = $hErr->{error};
    my $msg = $hErr->{errormessage};

    my $fmt = qq(ERROR = $err : $msg PortalMerchantID=%s);
    if ( $err == ERR_TWENGA_ACCOUNT_EXPIRED ) {
        TwengaLog( AccountExpired => 'notifying merchant' );
        my $System = LoadRootObject();
        my $MailType = $System->folder('MailTypes')->child('TwengaMerchantExpired');
        eval { SendTwengaErrorMail( $MailType, $PortalConfig ) }; # ignore failure
        GetLog->warn( sprintf $fmt, $PortalConfig->get('PortalMerchantID'));
        # clear values
        $PortalConfig->set({ PortalMerchantID => undef, ExportInterval => 0 });
    }
    elsif ( $err == ERR_TWENGA_INVALID_HASHKEY ) {
        GetLog->warn( sprintf $fmt, $PortalConfig->get('PortalMerchantID'));
    }
    else {
        # I'm not sure why we don't log the merchant ID here. -- Milu 18.05.11
        GetLog->warn( sprintf $fmt, 'N/A' );
    }

    return;
}

1;
