#========================================================================================
# package      DE_EPAGES::USPS::API::ShippingOrder
# state        public
# description  Description
#========================================================================================
package DE_EPAGES::USPS::API::ShippingOrder;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Order::API::Order qw ( GetLabelImagePath );
use DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel;
use DE_EPAGES::USPS::API::Constants qw (%LABEL_STATUS);
use DE_EPAGES::USPS::API::Tools qw ( trimErrorDescription );

use MIME::Base64 qw ( decode_base64 );
use DE_EPAGES::Core::API::File qw ( WriteFile );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabelManager->new( $UspsWebService );
#----------------------------------------------------------------------------------------
# description  Constuctor, it will create its own TrackingDeliveryConfirmationLabelManager
#               If not handed over
#----------------------------------------------------------------------------------------
# input        $ShippingFulfillmentMethod | ShippingFulfillmentMethodUSPS  object | object
# input        $UspsWebService | TrackingDeliveryConfirmationLabelManager object | object
# return       $self | self | object
#========================================================================================
sub new {
    my $class = shift;
    my ($ShippingFulfillmentMethod, $UspsWebService) = @_;

    my $self  = bless({}, $class);
    if (isValid($UspsWebService)){
        $self->{'UspsWebService'} = $UspsWebService;
    }else{
        $self->{'UspsWebService'} = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);
    }
    return $self;
}

#========================================================================================
# function     trigger
# state        public
#----------------------------------------------------------------------------------------
# syntax       trigger( $Transaction );
#----------------------------------------------------------------------------------------
# description  trigger the Transaction.
#----------------------------------------------------------------------------------------
# input        $Transaction | a Shippingorder Transaction | object
#========================================================================================
sub trigger {
    my $self = shift;
    my ($Transaction) = @_;

    my $aShippingLabel = $Transaction->get('ShippingLabels');

    foreach my $ShippingLabel (@{$aShippingLabel}){
        $self->_triggerShippingLabel($ShippingLabel);
        $self->_refreshWebService($ShippingLabel);
    }
    $self->_setTransactionStatus($Transaction);
    $self->_setTransactionID($Transaction);

    return undef;
}

#========================================================================================
# function     _triggerShippingLabel
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_triggerShippingLabel( $ShippingLabel );
#----------------------------------------------------------------------------------------
# description  trigger a shippingLabel and set the Issue state
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _triggerShippingLabel {
    my $self = shift;
    my ($ShippingLabel) = @_;

    eval {
        $self->_prepareShippingLabelWebService($ShippingLabel);
        $self->_generateShippingLabelOrError();
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);
        Error($Error) if ( $Error->code eq 'NativDie' );
        LogError($Error);
    } else {
        $self->_reportAndUpdateShippingLabelOnSuccess($ShippingLabel);
    }

    return undef;
}

#========================================================================================
# function     _prepareShippingLabelWebService
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_prepareShippingLabelWebService( $ShippingLabel );
#----------------------------------------------------------------------------------------
# description  prepare data for the Tracking Delivery Confirmation Label USPS web service
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _prepareShippingLabelWebService {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $WebService = $self->{'UspsWebService'};
    $WebService->feedRequest($ShippingLabel);

    return;
}

#========================================================================================
# function     _generateShippingLabelOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_generateShippingLabelOrError();
#----------------------------------------------------------------------------------------
# description  call web service if Tracking Delivery Confirmation Label input request
#               is valid and get response. Throw an error if there's any problem with the
#               request or response.
#========================================================================================
sub _generateShippingLabelOrError {
    my $self = shift;

    my $WebService = $self->{'UspsWebService'};
    $WebService->callWebService();
    $WebService->isValidResponseOrError();

    return;
}

#========================================================================================
# function     _reportAndUpdateShippingLabelOnError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_reportAndUpdateShippingLabelOnError($Error, $ShippingLabel);
#----------------------------------------------------------------------------------------
# description  report error and update shipping label information accordingly
#----------------------------------------------------------------------------------------
# input        $Error | Error Object | object
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _reportAndUpdateShippingLabelOnError {
    my $self = shift;
    my ($Error, $ShippingLabel) = @_;

    $ShippingLabel->set({
        'Issued' => '0',
        'LabelStatus' => $self->_obtainShippingLabelErrorStatus($Error->code),
        'ShippingLabelErrorDescription' => trimErrorDescription($Error->shortMessage()),
    });

    LogShippingFulfillment('USPS', 'Error at Shipping Label dispatch phase: ',
         {
             'ShippingLabel' => $ShippingLabel->alias,
             'ShippingLabelID' => $ShippingLabel->id,
             'ErrorCode' => $Error->code,
             'LabelStatus' => $ShippingLabel->get('LabelStatus'),
             'ShippingLabelErrorDescription' => $ShippingLabel->get('ShippingLabelErrorDescription'),
         }
    );

    return;
}

#========================================================================================
# function     _obtainShippingLabelErrorStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LabelStatus = $self->_obtainShippingLabelErrorStatus( $errorCode );
#----------------------------------------------------------------------------------------
# description  provide appropriate label error status value from incoming error type
#----------------------------------------------------------------------------------------
# input        $errorCode | error code | string
# return       $LabelStatus | integer
#========================================================================================
sub _obtainShippingLabelErrorStatus {
    my $self = shift;
    my ($errorCode) = @_;

    if ($errorCode =~ /INVALID_REQUEST/) {
        return $LABEL_STATUS{'INVALID_REQUEST'};
    }
    elsif ($errorCode =~ /INVALID_RESPONSE/) {
        return $LABEL_STATUS{'INVALID_RESPONSE'};
    }
    elsif ($errorCode eq 'ERROR_RESPONSE') {
        return $LABEL_STATUS{'ERROR_RESPONSE'};
    }
    elsif ($errorCode eq 'CONNECTION_FAILED') {
        return $LABEL_STATUS{'CONNECTION_FAILED'};
    }
    else {
        return $LABEL_STATUS{'UNKNOWN_ERROR'};
    }
    return undef;
}

#========================================================================================
# function     _reportAndUpdateShippingLabelOnSuccess
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_reportAndUpdateShippingLabelOnSuccess($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  report success and update shipping label information accordingly
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _reportAndUpdateShippingLabelOnSuccess {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $WebService = $self->{'UspsWebService'};

    $ShippingLabel->set({
        'Issued' => '1',
        'LabelStatus' => '1',
        'ShippingLabelErrorDescription' => undef,
        'TrackingID' => $WebService->getResponseDeliveryConfirmationNumberOrError(),
        'TrackingLink' => $ShippingLabel->generateTrackingLink(),
    });

    $self->_saveShippingLabelFile($ShippingLabel);

    LogShippingFulfillment('USPS', 'Shipping Label dispatched successfully: ',
         {
             'ShippingLabel' => $ShippingLabel->alias,
             'ShippingLabelID' => $ShippingLabel->id,
             'DeliveryConfirmationNumber' => $ShippingLabel->get('TrackingID'),
         }
    );

    return undef;
}

#========================================================================================
# function     _saveShippingLabelFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_saveShippingLabelFile($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  save into web server provided shipping label file
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | a shippinglabel | object
#========================================================================================
sub _saveShippingLabelFile {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $WebService = $self->{'UspsWebService'};
    my $LabelImageCode = $WebService->getResponseLabelBase64CodeOrError();
    my $LabelImageFile = decode_base64($LabelImageCode);

    my $LabelImagePath = GetLabelImagePath($ShippingLabel);
    $ShippingLabel->set({ 'LabelDownloadURL' => $LabelImagePath });

    WriteFile( $LabelImagePath, \$LabelImageFile, { binary => 1, create_directory => 1 } );

    return;
}

#========================================================================================
# function     _refreshWebService
# state        private
#----------------------------------------------------------------------------------------
# syntax       _refreshWebService();
#----------------------------------------------------------------------------------------
# input        $ShippingFulfillmentMethod | ShippingFulfillmentMethodUSPS  object | object
#----------------------------------------------------------------------------------------
# description  clear request and response information for the next service call. It only
#               keeps the API caller object.
#========================================================================================
sub _refreshWebService {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $CurrentWebService = $self->{'UspsWebService'};
    return unless ( isValid($CurrentWebService) );

    my $ShippingFulfillmentMethod = $ShippingLabel->get('ShippingFulfillmentMethod');
    my $FreshWebService = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);

    my $Caller = $CurrentWebService->getCaller();
    $FreshWebService->setCaller($Caller) if ( isValid($Caller) );
    $self->{'UspsWebService'} = $FreshWebService;

    return undef;
}

#========================================================================================
# function     _setTransactionStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setTransactionStatus( $Transaction );
#----------------------------------------------------------------------------------------
# description  validate a Transaction to three different states
#               1. all shipping labels could be triggert
#               2. some shipping labels could be triggert
#               3. no shipping labels could be triggert
#----------------------------------------------------------------------------------------
# input        $Transaction | a Shippingorder Transaction | object
#========================================================================================
sub _setTransactionStatus {
    my $self = shift;
    my ($Transaction) = @_;

    $Transaction->set({'TransStatus'=>'AllShippingLabelsProceeded'});
    my $CountFaultyShippinglabels = 0;
    my $aShippingLabel = $Transaction->get('ShippingLabels');

    foreach my $ShippingLabel (@{$aShippingLabel}){
        if ( $ShippingLabel->get('Issued') != 1){
            $Transaction->set({'TransStatus'=>'SomeShippingLabelsProceeded'});
            $CountFaultyShippinglabels++;
        }
    }
    if ($CountFaultyShippinglabels == scalar @{$aShippingLabel} ){
        $Transaction->set({'TransStatus'=>'NoShippingLabelsProceeded'});
    }
    $Transaction->issueShippingOrder();

    return undef;
}

#========================================================================================
# function     _setTransactionID
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setTransactionID( $Transaction );
#----------------------------------------------------------------------------------------
# description  Obtain transaction ID and set TransID attribute with it
#----------------------------------------------------------------------------------------
# input        $Transaction | a Shippingorder Transaction | object
#========================================================================================
sub _setTransactionID {
    my $self = shift;
    my ($Transaction) = @_;

    my $TransactionStatus = $Transaction->get('TransStatus');
    if ( isValid($TransactionStatus) && ($TransactionStatus ne 'NoShippingLabelsProceeded') ) {
        my $TransactionAlias = $Transaction->get('NameOrAlias');
        $Transaction->set({ 'TransID' => $TransactionAlias });
    }

    return;
}

1;
