#========================================================================================
# package      DE_EPAGES::USPS::API::WebServices::Base::XMLrequest
# state        public
#----------------------------------------------------------------------------------------
# description  It holds common request operations
#========================================================================================
package DE_EPAGES::USPS::API::WebServices::Base::XMLrequest;

use strict;

use XML::LibXML;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::TestParameter qw (TestString TestFloat TestInteger);
use DE_EPAGES::Core::API::Error qw ( Error );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'INVALID_REQUEST_NO_API_FILE'               => 'No API XML file has been provided to generate USPS Web Service XML object',
        'INVALID_REQUEST_NOT_VALID_NODE'            => '#nodename node is not valid or not well configured',
        'INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES' => '#nodename node attributes are not well configured',
        'INVALID_REQUEST_VALUE_NOT_ALLOWED'         => '#textvalue is not allowed for #nodename node',
    };
}

use constant FIRST_ARRAY_ELEMENT => 0;
use constant TEXT_NODE_NAME => '#text'; # '#text' is the name assigned to a leaf node of type text.

#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XMLrequest = DE_EPAGES::USPS::API::WebServices::Base::XMLrequest->new($APIrequest);
#----------------------------------------------------------------------------------------
# description  Creates a new XMLrequest object .
#----------------------------------------------------------------------------------------
# input        $APIrequest | API request hash structure | hash ref
# return       $XMLrequest | XMLrequest object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($APIrequestFile) = @_;

    if ( !isValid($APIrequestFile) ) {
        Error('INVALID_REQUEST_NO_API_FILE');
    }
    my $parser      = XML::LibXML->new();
    my $APIdocument = $parser->load_xml( location => $APIrequestFile );
    my $APIrootNode = $APIdocument->documentElement();
    my $XMLdocument = XML::LibXML::Document->new( '1.0', 'UTF-8' );
    my $XMLrootNode = $XMLdocument->createElement( $APIrootNode->nodeName() );
    $XMLdocument->setDocumentElement($XMLrootNode);

    my $self = bless {
        'APIrequestModel' => $APIdocument,
        'XMLrequest' => $XMLdocument,
    }, $class;

    $self->initializeRequest();
    return $self;
}

#========================================================================================
# function     initializeRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->initializeRequest();
#----------------------------------------------------------------------------------------
# description  It initializes XML DOM node tree from API DOM.
#========================================================================================
sub initializeRequest {
    my $self = shift;

    my $document    = $self->{'XMLrequest'};
    my $XMLrootNode = $document->documentElement();

    my $APIrootNode = $self->_getAPIrootNode();
    $self->_createNodeAttributes( $XMLrootNode, $APIrootNode );

    my @APInodes = $APIrootNode->nonBlankChildNodes();
    foreach my $APInode (@APInodes) {
        $self->_setNode( $XMLrootNode, $APInode );
    }
}

#========================================================================================
# function     _getAPIrootNode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $APIrootNode = $self->_getAPIrootNode();
#----------------------------------------------------------------------------------------
# description  It gets API (request) root node element from XMLrequest object
#----------------------------------------------------------------------------------------
# return       $APIrootNode | API Root element node object | object
#========================================================================================
sub _getAPIrootNode {
    my $self = shift;

    my $APIdocument = $self->{'APIrequestModel'};
    return $APIdocument->documentElement();
}

#========================================================================================
# function     _getXMLrootNode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $XMLrootNode = $self->_getXMLrootNode();
#----------------------------------------------------------------------------------------
# description  It gets XML (request) root node element from XMLrequest object
#----------------------------------------------------------------------------------------
# return       $XMLrootNode | XML Root element node object | object
#========================================================================================
sub _getXMLrootNode {
    my $self = shift;

    my $XMLdocument = $self->{'XMLrequest'};
    return $XMLdocument->documentElement();
}

#========================================================================================
# function     _setNode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setNode($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It reproduces (recursively) XML nodes from API nodes and sets default
#               values from API DOM into XML DOM.
#----------------------------------------------------------------------------------------
# input        $XMLnode | XML node object | object
# input        $APInode | API node object | object
#========================================================================================
sub _setNode {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;

    my $XMLsubNode = XML::LibXML::Element->new( $APInode->nodeName() );
    $self->_createNodeAttributes( $XMLsubNode, $APInode );
    if ( $self->_isGroupNode($APInode) ) {
        foreach my $APIsubnode ( $APInode->nonBlankChildNodes() ) {
            $self->_setNode( $XMLsubNode, $APIsubnode );
        }
    }
    else {
        $XMLsubNode->appendText( $self->_getDefaultValueOrEmpty($APInode) );
    }
    $XMLnode->appendChild($XMLsubNode);

    return;
}

#========================================================================================
# function     _createNodeAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createNodeAttributes($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It sets attributes for the XML node by getting them from API node.
#----------------------------------------------------------------------------------------
# input        $XMLnode | processed XML node object | object
# input        $APInode | corresponding API node object | object
#========================================================================================
sub _createNodeAttributes {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;
    if ( $APInode->hasAttribute('attributes') ) {
        my @attributes = split / /, $APInode->getAttribute('attributes');
        foreach my $attribute (@attributes) {
            $XMLnode->setAttribute( $attribute, '' );
        }
    }
}

#========================================================================================
# function     _isGroupNode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isGroup = $self->_isGroupNode($APInode);
#----------------------------------------------------------------------------------------
# description  It checks if API node is of group type and it really holds subnodes
#----------------------------------------------------------------------------------------
# input        $APInode | corresponding API node object | object
# return       $isGroup | boolean
#========================================================================================
sub _isGroupNode {
    my $self = shift;
    my ($APInode) = @_;

    if ( $APInode->hasAttribute('type') && ( $APInode->getAttribute('type') eq 'group' ) && $APInode->hasChildNodes() )
    {
        return 1;
    }
    return 0;
}

#========================================================================================
# function     _getDefaultValueOrEmpty
# state        private
#----------------------------------------------------------------------------------------
# syntax       $defaultValue = $self->_getDefaultValueOrEmpty($APInode);
#----------------------------------------------------------------------------------------
# description  It gets text content from "DefaultValue" tag for the provided API node.
#               It returns an empty string if not found.
#----------------------------------------------------------------------------------------
# input        $APInode | default value string | string
# return       $defaultValue | default value string | string
#========================================================================================
sub _getDefaultValueOrEmpty {
    my $self = shift;
    my ($APInode) = @_;

    my $NodeValue = '';
    my @defaultValueNodes = $APInode->getChildrenByTagName('DefaultValue');
    if ( scalar (@defaultValueNodes) == 1 ) {
        $NodeValue = $defaultValueNodes[FIRST_ARRAY_ELEMENT]->textContent();
    }
    return $NodeValue;
}

#========================================================================================
# function     toString
# state        public
#----------------------------------------------------------------------------------------
# syntax       $XMLstring = $self->toString();
#----------------------------------------------------------------------------------------
# description  return string representing XML DOM hold in the object.
#----------------------------------------------------------------------------------------
# return       $XMLstring | XML DOM string | string
#========================================================================================
sub toString {
    my $self = shift;

    my $XMLrequest = $self->{'XMLrequest'};
    return $XMLrequest->toString();
}

#========================================================================================
# function     isValidRequestOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isValidRequestOrError();
#----------------------------------------------------------------------------------------
# description  It checks if internal XML (DOM) object request is valid by reviewing
#               API (DOM) structure. It throws an error if XML is not valid.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub isValidRequestOrError {
    my $self = shift;

    my $XMLrootNode = $self->_getXMLrootNode();
    my $APIrootNode = $self->_getAPIrootNode();

    return $self->_isValidNodeOrError( $XMLrootNode, $APIrootNode );
}

#========================================================================================
# function     _isValidNodeOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidNodeOrError($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It checks if XML node is compliant with API structure. It throws an error
#               if XML appears as not valid.
#----------------------------------------------------------------------------------------
# input        $XMLnode | XML element node object | object
# input        $APInode | API element node object | object
# return       $isValid | boolean
#========================================================================================
sub _isValidNodeOrError {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;

    Error( 'INVALID_REQUEST_NOT_VALID_NODE', { 'nodename' => $XMLnode->nodeName() } )
      unless ( $XMLnode->nodeName() eq $APInode->nodeName() );
    Error( 'INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES', { 'nodename' => $XMLnode->nodeName() } )
      unless ( $self->_areValidAttributes( $XMLnode, $APInode ) );

    if ( $self->_isGroupNode($APInode) ) {
        my @APIsubnodes = $APInode->nonBlankChildNodes();
        foreach my $APIsubnode (@APIsubnodes) {
            my @XMLsubnodes = $XMLnode->getChildrenByTagName( $APIsubnode->nodeName() );
            if ( not ( scalar (@XMLsubnodes) == 1 ) or not $self->_isValidNodeOrError( $XMLsubnodes[FIRST_ARRAY_ELEMENT], $APIsubnode ) ) {
                return 0;
            }
        }
        return 1;
    }
    else {
        return $self->_isValidLeafNodeOrError( $XMLnode, $APInode );
    }
}

#========================================================================================
# function     _isValidLeafNodeOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidLeafNodeOrError($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It checks if XML node is a leaf node and if it's compliant with API
#               structure. It throws an error if XML appears as not valid.
#----------------------------------------------------------------------------------------
# input        $XMLnode | XML element node object | object
# input        $APInode | API element node object | object
# return       $isValid | boolean
#========================================================================================
sub _isValidLeafNodeOrError {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;

    Error( 'INVALID_REQUEST_NOT_VALID_NODE', { 'nodename' => $XMLnode->nodeName() } )
      unless ( $self->_isTextNodeLeaf($XMLnode) );

    if ( $self->_matchesEmpty( $XMLnode, $APInode ) ) {
        return 1;
    }

    my $XMLtextContent = $XMLnode->textContent();
    my $APInodeType    = $APInode->getAttribute('type');

    if ($APInodeType eq 'enumeration') {
        return $self->_isValidEnumOrError( $APInode, $XMLtextContent );
    }
    elsif ($APInodeType eq 'string') {
        return $self->_isValidStringOrError( $APInode, $XMLtextContent );
    }
    elsif ($APInodeType eq 'boolean') {
        return $self->_isValidBooleanOrError( $APInode, $XMLtextContent );
    }
    elsif ($APInodeType eq 'decimal') {
        return $self->_isValidNonNegativeDecimalOrError( $APInode, $XMLtextContent );
    }
    elsif ($APInodeType eq 'integer') {
        return $self->_isValidNonNegativeIntegerOrError( $APInode, $XMLtextContent );
    }
    elsif ($APInodeType eq 'date') {
        return $self->_isValidDateOrError( $APInode, $XMLtextContent );
    }
    else {
        Error( 'INVALID_REQUEST_NOT_VALID_NODE', { 'nodename' => $APInode->nodeName()}  );
    }
}

#========================================================================================
# function     _isTextNodeLeaf
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isTextNodeLeaf = $self->_isTextNodeLeaf($XMLnode);
#----------------------------------------------------------------------------------------
# description  It checks if XML node is a leaf node of type text.
#----------------------------------------------------------------------------------------
# input        $XMLnode | Element node object | object
# return       $isTextNodeLeaf | boolean
#========================================================================================
sub _isTextNodeLeaf {
    my $self = shift;
    my ($XMLnode) = @_;

    if ( !$XMLnode->hasChildNodes() ) {
        return 1;
    }

    my @XMLsubnodes = $XMLnode->childNodes();
    if ( scalar (@XMLsubnodes) > 1 ) {
        return 0;
    } elsif ( ( $XMLsubnodes[FIRST_ARRAY_ELEMENT]->nodeName() eq TEXT_NODE_NAME ) ) {
        return 1;
    } else {
        return 0;
    }
}

#========================================================================================
# function     _areValidAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $areValid = $self->_areValidAttributes($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It checks if XML node holds nodes specified in API XML.
#----------------------------------------------------------------------------------------
# input        $XMLnode | XML node object | object
# input        $APInode | API node object | object
# return       $areValid | boolean
#========================================================================================
sub _areValidAttributes {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;

    my @XMLattributes = $XMLnode->getAttributes();
    my @APIattributes = split / /, $APInode->getAttribute('attributes') // '';

    if ( @XMLattributes == @APIattributes ) {
        foreach my $attribute (@APIattributes) {
            return 0 unless ( $XMLnode->hasAttribute($attribute) );
            return 0 if ( not isValid( $XMLnode->getAttribute($attribute) ) );
        }
    }
    else {
        return 0;
    }
    return 1;
}

#========================================================================================
# function     _matchesEmpty
# state        private
#----------------------------------------------------------------------------------------
# syntax       $matchesEmpty = $self->_matchesEmpty($XMLnode, $APInode);
#----------------------------------------------------------------------------------------
# description  It checks if XML node is empty and it can be empty (from API perspective).
#----------------------------------------------------------------------------------------
# input        $XMLnode | Element node object | object
# input        $APInode | corresponding API hash | hash ref
# return       $matchesEmpty | boolean
#========================================================================================
sub _matchesEmpty {
    my $self = shift;
    my ( $XMLnode, $APInode ) = @_;

    my $XMLtextContent = $XMLnode->textContent();
    return 1 if ( !isValid($XMLtextContent) && ( $APInode->getAttribute('canBeEmpty') ) );

    return 0;
}

#========================================================================================
# function     _isValidEnumOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidEnumOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type Enum. Enumerated items are provided
#               by an array. It throws an error if value is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidEnumOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    my $APInodeName = $APInode->nodeName();
    my $aEnumValues = $self->getAPInodeEnumValues($APInode);
    Error( 'INVALID_REQUEST_VALUE_NOT_ALLOWED', { 'nodename' => $APInodeName, 'textvalue' => $textValue } )
      unless ( $textValue ~~ $aEnumValues );
    return 1;
}

#========================================================================================
# function     getAPInodeEnumValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aEnumValues = $self->getAPInodeEnumValues($APInode);
#----------------------------------------------------------------------------------------
# description  It returns an array with values from "EnumValue" child tags.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# return       $aEnumValues | array ref
#========================================================================================
sub getAPInodeEnumValues {
    my $self = shift;
    my ($APInode) = @_;

    my @enumNodes = $APInode->getChildrenByTagName('EnumValue');
    my @enumValues = map { $_->textContent() } @enumNodes;
    return \@enumValues;
}

#========================================================================================
# function     _isValidStringOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidStringOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type string. It throws an error if value
#               is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidStringOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    return 1 if ( $APInode->getAttribute('canBeEmpty') && ( length ($textValue) == 0 ) );
    TestString( $APInode->nodeName(), $textValue );
    return 1;
}

#========================================================================================
# function     _isValidBooleanOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidBooleanOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type string boolean (true / false).
#               It throws an error if value is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidBooleanOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    my $APInodeName = $APInode->nodeName();
    TestString( $APInodeName, $textValue );
    Error( 'INVALID_REQUEST_VALUE_NOT_ALLOWED', { 'nodename' => $APInodeName, 'textvalue' => $textValue } )
      unless ( ( $textValue eq 'true' || $textValue eq 'false' ) );
    return 1;
}

#========================================================================================
# function     _isValidNonNegativeDecimalOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidNonNegativeDecimalOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type decimal. It throws an error if value
#               is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidNonNegativeDecimalOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    return 1 if ( $APInode->getAttribute('canBeEmpty') && ( length ($textValue) == 0 ) );
    TestFloat( $APInode->nodeName(), $textValue, 0 );
    return 1;
}

#========================================================================================
# function     _isValidNonNegativeIntegerOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidNonNegativeIntegerOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type integer. It throws an error if value
#               is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidNonNegativeIntegerOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    return 1 if ( $APInode->getAttribute('canBeEmpty') && ( length ($textValue) == 0 ) );
    TestInteger( $APInode->nodeName(), $textValue, 0 );
    return 1;
}

#========================================================================================
# function     _isValidDateOrError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->_isValidDateOrError($APInode, $textValue);
#----------------------------------------------------------------------------------------
# description  It checks if XML node leaf is of type string date with a valid format.
#               It throws an error if value is not conformant.
#----------------------------------------------------------------------------------------
# input        $APInode | API node object | object
# input        $textValue | value of XML node | string
# return       $isValid | boolean
#========================================================================================
sub _isValidDateOrError {
    my $self = shift;
    my ( $APInode, $textValue ) = @_;

    return 1 if ( $APInode->getAttribute('canBeEmpty') && ( !isValid($textValue) ) );
    #string date valid formats we check are "mm/dd/yyyy" OR "mm/dd/yy"
    if ( $textValue =~ /^[01][0-9]\/[0123][0-9]\/\d{2}(\d{2})?$/ ) {
        return 1;
    } else {
        Error( 'INVALID_REQUEST_VALUE_NOT_ALLOWED', { 'nodename' => $APInode->nodeName(), 'textvalue' => $textValue } )
    }
}

#========================================================================================
# function     setTextNodeWithinXMLrequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setTextNodeWithinXMLrequest($nodeName, $textValue);
#----------------------------------------------------------------------------------------
# description  It replaces the current text value of an XML DOM node.
#----------------------------------------------------------------------------------------
# input        $nodeName | node name to be found into XML DOM | string
# input        $textValue | replacement text value | string
# input        $XMLnode | XML node object | object
#========================================================================================
sub setTextNodeWithinXMLrequest {
    my $self = shift;
    my ( $nodeName, $textValue, $XMLnode ) = @_;

    $textValue //= '';

    if ( !isValid($XMLnode) ) {
        my $Request = $self->{'XMLrequest'};
        $XMLnode = $Request->documentElement();
    }

    my @XMLnodes = $XMLnode->getChildrenByTagName($nodeName);
    if ( scalar (@XMLnodes) == 0 ) {
        # no subnode holds the node name we're looking for, so lets check at next level down
        foreach my $XMLsubNode ( $XMLnode->nonBlankChildNodes() ) {
            $self->setTextNodeWithinXMLrequest( $nodeName, $textValue, $XMLsubNode )
              unless ( $self->_isTextNodeLeaf($XMLsubNode) );
        }
    }
    else {
        my $XMLtextNodeLeaf = $XMLnodes[FIRST_ARRAY_ELEMENT];
        return unless ( $self->_isTextNodeLeaf($XMLtextNodeLeaf) );
        my $XMLreplacementNode = XML::LibXML::Element->new($nodeName);
        $XMLreplacementNode->appendText($textValue);
        $XMLtextNodeLeaf->replaceNode($XMLreplacementNode);
    }
    return;
}

#========================================================================================
# function     setNodeAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setNodeAttribute( $attributeName, $attributeValue, $XMLnode );
#----------------------------------------------------------------------------------------
# description  It sets a value into an attribute for the specified node.
#----------------------------------------------------------------------------------------
# input        $attributeName | attribute name for the provided node | string
# input        $attributeValue | attribute text value | string
# input        $XMLnode | XML node object | object
#========================================================================================
sub setNodeAttribute {
    my $self = shift;
    my ( $attributeName, $attributeValue, $XMLnode ) = @_;

    $XMLnode->setAttribute( $attributeName, $attributeValue );

}

#========================================================================================
# function     getAPIrequestNode
# state        private
#----------------------------------------------------------------------------------------
# syntax       $foundNode = $self->getAPIrequestNode($APIattributeName, $APInode);
#----------------------------------------------------------------------------------------
# description  It provides the API node provided by name.
#----------------------------------------------------------------------------------------
# input        $APInodeName | API node name | string
# input        $APInode | API node object to look into | object
# return       $foundNode | API node object | object
#========================================================================================
sub getAPIrequestNode {
    my $self = shift;
    my ( $APInodeName, $APInode ) = @_;

    if ( !isValid($APInode) ) {
        $APInode = $self->_getAPIrootNode();
    }
    my $foundNode;
    foreach my $APIsubnode ( $APInode->nonBlankChildNodes() ) {
        if ( $APIsubnode->nodeName() eq $APInodeName ) {
            $foundNode = $APIsubnode;
        } else {
            if ( $self->_isGroupNode($APIsubnode) ) {
                $foundNode = $self->getAPIrequestNode( $APInodeName, $APIsubnode );
            }
        }
        if ( isValid($foundNode) ) {
            last;
        }
    }
    return $foundNode;
}

#========================================================================================
# function     getXMLrequestNode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $foundNode = $self->getXMLrequestNode($nodeName, $XMLnode);
#----------------------------------------------------------------------------------------
# description  It provides the XML node provided by name.
#----------------------------------------------------------------------------------------
# input        $nodeName | XML node name | string
# input        $XMLnode | XML node object to look into | object
# return       $foundNode | XML node object | object
#========================================================================================
sub getXMLrequestNode {
    my $self = shift;
    my ( $nodeName, $XMLnode ) = @_;

    if ( !isValid($XMLnode) ) {
        $XMLnode = $self->_getXMLrootNode();
        if ( $XMLnode->nodeName() eq $nodeName ) {
            return $XMLnode;
        }
    }
    my @XMLsubNodes = $XMLnode->getChildrenByTagName($nodeName);
    if ( not scalar (@XMLsubNodes) > 0 ) {
        foreach my $XMLsubNode ( $XMLnode->nonBlankChildNodes() ) {
            if ( !$self->_isTextNodeLeaf($XMLsubNode) ) {
                return $self->getXMLrequestNode( $nodeName, $XMLsubNode );
            }
        }
    }
    else {
        return $XMLsubNodes[FIRST_ARRAY_ELEMENT];
    }
    return;
}

#========================================================================================
# function     getTextContentByXMLnodeName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $textContent = $self->getTextContentByXMLnodeName($nodeName, $XMLnode);
#----------------------------------------------------------------------------------------
# description  It provides the text value of an XML node by name.
#----------------------------------------------------------------------------------------
# input        $nodeName | XML node name | string
# input        $XMLnode | XML node object | object
# return       $textContent | text node value | string
#========================================================================================
sub getTextContentByXMLnodeName {
    my $self = shift;
    my ( $nodeName, $XMLnode ) = @_;

    my $foundNode = $self->getXMLrequestNode( $nodeName, $XMLnode );
    if ( isValid($foundNode) ) {
        return $foundNode->textContent();
    }
    return;
}

1;
