#========================================================================================
# package      DE_EPAGES::USPS::API::WebServices::Extensions::TrackingDeliveryConfirmationLabelInternalConditionsValidator
# state        public
#----------------------------------------------------------------------------------------
# description  It contains validation methods required to test conformance on USPS
#               Tracking/Delivery Confirmation Label requests.
#========================================================================================
package DE_EPAGES::USPS::API::WebServices::Extensions::TrackingDeliveryConfirmationLabelInternalConditionsValidator;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use Date::Calc qw ( Today Decode_Date_US Date_to_Days );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'INVALID_REQUEST_NO_CONDITIONS_MET' => 'following condition not met: #condition',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     areInternalConditionsMetOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->areInternalConditionsMetOrError();
#----------------------------------------------------------------------------------------
# description  It checks validity of the request's internal conditions (the ones between
#               different request parameters). It returns 1 (true) or it throws an error.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub areInternalConditionsMetOrError {
    my $self = shift;

    $self->isLabelDateWithinRangeEmptyOrError();
    $self->isAddressServiceAndImageParameterDependencyValidOrError();
    $self->areParcelDimensionsAndContainerAndSizeDependenciesValidOrError();

    return 1;
}

#========================================================================================
# function     isLabelDateWithinRangeEmptyOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isLabelDateWithinRangeEmptyOrError();
#----------------------------------------------------------------------------------------
# description  It checks validity of LabelDate. It must be between 0 to 3 days from today
#               or empty. It throws an error otherwise.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub isLabelDateWithinRangeEmptyOrError {
    my $self = shift;
    #LabelDate date (date package will be mailed) must be between 0 to 3 days from today
    my $labelDate = $self->getRequestNodeContentOrError('LabelDate');
    if ( isValid($labelDate) ) {
        my $deltaDays = Date_to_Days( Decode_Date_US($labelDate) ) - Date_to_Days( Today() );
        if ( $deltaDays < 0 || $deltaDays > 3 ) {
            Error( 'INVALID_REQUEST_NO_CONDITIONS_MET', { 'condition' => 'label date out of rank' } )
        }
    }
    return;
}

#========================================================================================
# function     isAddressServiceAndImageParameterDependencyValidOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->isAddressServiceAndImageParameterDependencyValidOrError();
#----------------------------------------------------------------------------------------
# description  AddressServiceRequested is not available with the "Barcode Only" label option.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub isAddressServiceAndImageParameterDependencyValidOrError {
    my $self = shift;

    my $addressServiceRequested = $self->getRequestNodeContentOrError('AddressServiceRequested');
    my $imageParamenter         = $self->getRequestNodeContentOrError('ImageParameter');
    if ( $addressServiceRequested eq 'true' && $imageParamenter eq 'BARCODE ONLY' ) {
        Error( 'INVALID_REQUEST_NO_CONDITIONS_MET',
          { 'condition' => 'can not request address service with "barcode only" image label option' } )
    }
}

#========================================================================================
# function     areParcelDimensionsAndContainerAndSizeDependenciesValidOrError
# state        public
#----------------------------------------------------------------------------------------
# syntax       $isValid = $self->areParcelDimensionsAndContainerAndSizeDependenciesValidOrError();
#----------------------------------------------------------------------------------------
# description  parcel dimension mandatory requirements change on Container and Size values.
#----------------------------------------------------------------------------------------
# return       $isValid | boolean
#========================================================================================
sub areParcelDimensionsAndContainerAndSizeDependenciesValidOrError {
    my $self = shift;

    my $Container = $self->getRequestNodeContentOrError('Container');
    my $Size      = $self->getRequestNodeContentOrError('Size');
    my $Girth     = $self->getRequestNodeContentOrError('Girth');
    my $Width     = $self->getRequestNodeContentOrError('Width');
    my $Length    = $self->getRequestNodeContentOrError('Length');
    my $Height    = $self->getRequestNodeContentOrError('Height');

    if ( $Size eq 'LARGE' ) {
        if ($Container eq 'RECTANGULAR') {
            Error( 'INVALID_REQUEST_NO_CONDITIONS_MET',
                { 'condition' => 'width, length & height values required for current container and size.' }
            ) unless ( isValid($Width) && isValid($Length) && isValid($Height) );
        }
        elsif ($Container eq 'NONRECTANGULAR') {
            Error( 'INVALID_REQUEST_NO_CONDITIONS_MET',
                { 'condition' => 'girth, width, length & height values required for current container and size.' }
            ) unless ( isValid($Girth) && isValid($Width) && isValid($Length) && isValid($Height) );
        }
        else {
            Error( 'INVALID_REQUEST_NO_CONDITIONS_MET',
                { 'condition' => 'When size is large, container must be Rectangular or NonRectangular.' }
            );
        }
    }

    return 1;
}

1;
