#========================================================================================
# package      DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver
# base         DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a csv driver to export User defined Portals.
#----------------------------------------------------------------------------------------
#  example      use DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver;
#
#               my $Driver = DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->exportCSV;
#========================================================================================
package DE_EPAGES::UniversalPortal::API::CSV::UniversalProductsExportDriver;
use base DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver;
use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );

sub ErrorDefinition {
    return { 'WrongAttributeType' =>
          'Missing attribute type for attribute #ObjectID.', };
}

##no critic qw(ProhibitCascadingIfElse ProhibitQuotedWordLists)

my %QUOTECHARACTER = (
    'quotes'  => '\'',
    'dquotes' => '"',
);

my %ROWSEPERATOR = (
    'newl'    => "\n",
    'newlret' => "\n\r",
    'ret'     => "\r",
);

my %COLUMNSEPERATOR = (
    'comma'     => ',',
    'semicolon' => ';',
    'pipe'      => '|',
    'tabulator' => "\t",
);

sub exportCSV {
    my $self = shift;
    my ( $FileName, $hFileFormat, $aColumnNames, $ViewObject, $aObjects ) = @_;
    my $PortalSite = $ViewObject->get('PortalSite');

    $hFileFormat                   //= {};
    $hFileFormat->{'encoding'}     //= 'utf8';
    $hFileFormat->{'sep_char'}     //= $COLUMNSEPERATOR{ $PortalSite->get('ListColumnSeparator') };
    $hFileFormat->{'eol'}          //= $ROWSEPERATOR{ $PortalSite->get('ListRowSeparator') };
    $hFileFormat->{'quote_char'}   //= $QUOTECHARACTER{ $PortalSite->get('ListQuoteCharacter') };
    $hFileFormat->{'always_quote'} //= $PortalSite->get('AlwaysQuote');

    my @MAP_Universal_ColumnNames = ();
    if ( !$ViewObject->existsChild('UniversalPortalAttributeSets')
         || scalar( @{ $ViewObject->get('UniversalPortalAttributeSets') } ) == 0 ){
        return;
    }
    $self->{'UniversalPortalAttributeSet'} =
      $ViewObject->get('UniversalPortalAttributeSets');
    foreach my $Attribute ( @{ $self->{'UniversalPortalAttributeSet'} } ) {
        push( @MAP_Universal_ColumnNames, $Attribute->alias );
    }
    $self->SUPER::exportCSV( $FileName, $hFileFormat,
        \@MAP_Universal_ColumnNames, $ViewObject, $aObjects );
    return;
}

#========================================================================================
# function     getExportValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getExportValues($Object);
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  Overwrites the method in DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver.
#               helper method of <function exportLine> to convert column names to methods
#               names which are stored in UniversalAttributes. This exporter and calls the
#               method to get the values of the exported object. Futher it truncates it
#               to the user defined length if the field is set.
#----------------------------------------------------------------------------------------
# return       $aValues | list of values | ref.array.string
#========================================================================================
sub getExportValues {
    my $self      = shift;
    my ($Product) = @_;
    my @Values    = ();
    foreach my $Attribute ( @{ $self->UniversalPortalAttributeSet() } ) {
        my $Value = $self->getAttributeValue( $Product, $Attribute ) // '';
        my $maxLength = $Attribute->get('MaxLength');
        if ( defined($maxLength) && length($Value) > $maxLength ) {
            $Value = substr( $Value, 0, $maxLength );
        }
        push( @Values, $Value );
    }
    return \@Values;
}

#========================================================================================
# function     getAttributeValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAttributeValue( $Product, $Attribute );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
# input        $Attribute  | an UniversalPortalAttribute or a ProductAttribute
#               reference | ref.hash
#----------------------------------------------------------------------------------------
# description  The method returns a specific value from a product or a user defined
#               "ProductAttribute" as string.
#----------------------------------------------------------------------------------------
# return       $ProductValue | Return a product value or a  "ProductAttribute"
#               as string.| String
#========================================================================================
sub getAttributeValue {
    my $self = shift;
    my ( $Product, $Attribute ) = @_;
    if ( defined( $Attribute->get('UniversalPortalAttribute') ) ) {
        my $AttributeAlias = $Attribute->get('UniversalPortalAttribute')->alias;
        my $MethodName     = $AttributeAlias;
        $MethodName =~ s/\W/_/g;
        if ( $self->can($MethodName) ) {
            return $self->$MethodName($Product);
        }
        elsif ( ElementOf($AttributeAlias, qw(Height Width Length)) ) {
            return $self->MeasureWithUnit( $Product, $AttributeAlias );
        } else {
            return $Product->getWithParent( $AttributeAlias,
                                            $self->languageID );
        }
    }
    elsif ( defined( $Attribute->get('ProductAttribute') ) ) {
        my $AttributeAlias = $Attribute->get('ProductAttribute')->alias;
        my $Type           = $Attribute->get('ProductAttribute')->get('Type');
        my $UserDefinedAttributValue = '';

        if (ElementOf($Type, (
                          'String',          #Text
                          'LocalizedString', #Language-dependent text
                          'Boolean',         #Yes/No
                          'Integer',         #Integer
                          'Float'            # Decimal Number
                      ))) {
            $UserDefinedAttributValue =
                $Product->getWithParent( $AttributeAlias, $self->languageID );
        }
        elsif ($Type eq 'Money') {
            my $Money =
              $Product->getWithParent( $AttributeAlias, $self->languageID );
            if ( defined( $Money ) ) {
                $UserDefinedAttributValue =
                  $self->portalSite->formatMoney( $Money );
            }
        }
        elsif ($Type eq 'Time') {
            my $Time =
              $Product->getWithParent( $AttributeAlias, $self->languageID );
            if ( defined($Time) ) {
                $UserDefinedAttributValue =
                  $self->portalSite->formatDateTime( $Time, 'time' );
            }
        }
        elsif ($Type eq 'Date') {
            my $Date =
              $Product->getWithParent( $AttributeAlias, $self->languageID );
            if ( defined($Date) ) {
                $UserDefinedAttributValue =
                  $self->portalSite->formatDateTime( $Date, 'date' );
            }
        }
        elsif ($Type eq 'DateTime') {
            my $DateTime =
              $Product->getWithParent( $AttributeAlias, $self->languageID );
            if ( defined($DateTime) ) {
                $UserDefinedAttributValue =
                  $self->portalSite->formatDateTime( $DateTime,
                    'datetime' );
            }
        }
        elsif ($Type eq 'PreDefString') {
            my $StringObject = $Product->get($AttributeAlias);
            if ( defined($StringObject) ) {
                $UserDefinedAttributValue =
                  $StringObject->get( 'ValueOrAlias', $self->languageID );
            }
        }
        elsif ($Type eq 'PreDefLocalizedString')
        {    # Language-dependenttextwithvalueselection
            my $PreDefCustomerString =
              $Product->getWithParent( $AttributeAlias, $self->languageID );
            if ( defined($PreDefCustomerString) ) {
                $UserDefinedAttributValue =
                  $PreDefCustomerString->get( 'ValueOrAlias',
                    $self->languageID );
            }
        }
        elsif (ElementOf($Type, (
                             'PreDefMultiString',         # options
                             'PreDefMultiLocalizedString' # language dependent options
                         ))) {
            my $aPreDefMultiStrings =
              $Product->getWithParent( $AttributeAlias, $self->languageID )
              // [];
            $UserDefinedAttributValue = join( '/',
                map { $_->get( 'ValueOrAlias', $self->languageID ) }
                  @{$aPreDefMultiStrings} );
        }
        return $UserDefinedAttributValue;
    }
    else {
        Error( 'WrongAttributeType', { ObjectID => $Attribute->id } );
    }
}

#========================================================================================
# function     isProductValidPortalExt1
# state        public
#----------------------------------------------------------------------------------------
# syntax       isProductValidPortalExt1( $Input1 );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  Overwrites the method in DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver
#               The method checks whether the current Value is mandatory. If it is so the
#               whole line will not printed to the CSV-file.
#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    # check mandatory attributes
    foreach my $Attribute ( @{ $self->UniversalPortalAttributeSet } ) {
        next unless $Attribute->get('IsMandatory');
        my $Value = $self->getAttributeValue( $Product, $Attribute );

        Error(
            'AttributeIsMandatory',
            {
                'ProductID'     => $Product->id,
                'Product'       => $Product,
                'AttributeName' => $Attribute->alias,
            }
        ) unless ( isValid($Value) );
    }
    return;
}

#========================================================================================
# function     isProductValidPortal
# state        public
#----------------------------------------------------------------------------------------
# syntax       isProductValidPortal( $Input1 );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  Overwrites the method in DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver
#               and calls isProductValidPortalExt1
#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    $self->isProductValidPortalExt1($Product);

    return;
}

#========================================================================================
# function     UniversalPortalAttributeSet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $UniversalPortalAttributeSet = $self->UniversalPortalAttributeSet();
#----------------------------------------------------------------------------------------
# description  Returns the UniversalPortalAttributeSet.
#----------------------------------------------------------------------------------------
# return       $UniversalPortalAttributeSet  | UniversalPortalAttributeSet | object
#========================================================================================
sub UniversalPortalAttributeSet {
    return shift->{'UniversalPortalAttributeSet'};
}

#========================================================================================
# function     RefPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       RefPrice( $Product );
#----------------------------------------------------------------------------------------
# description  Returns String with all data regarding reference price if there is no
#                refrence price returns an empty String
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# return       $Delivery | reference price | String
#========================================================================================

sub RefPrice {
    my $self            = shift;
    my ($Product)       = @_;
    my $ReferencePrice  = $self->formatedReferencePrice( $Product );
    return ( $ReferencePrice eq '0' ) ? '' : $ReferencePrice ;
}

#========================================================================================
# function     DeliveryPeriod
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeliveryPeriod( $Product );
#----------------------------------------------------------------------------------------
# description  Returns the product delivery period
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# return       $Delivery | delivery period | String
#========================================================================================

sub DeliveryPeriod {
    my $self = shift;
    my ($Product) = @_;

    return $self->deliveryPeriod($Product);
}

#========================================================================================
# function     MainCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       MainCategory( $Product );
#----------------------------------------------------------------------------------------
# description  Export column "shop_cat" (shop category) which shall be the complete path of categories in the
# shop. It must be seperated by other symbols than the columns of the export file! (e.g. " / ")
#----------------------------------------------------------------------------------------
# input        $Product | a produc | ref
#----------------------------------------------------------------------------------------
# return       $CategoryNamesString | product category | String
#========================================================================================
sub MainCategory {
    my $self = shift;
    my ($Product) = @_;

    # do not use getWithParent to get the category
    my $CategoryNamesString =
      $self->_getCategoryString( $Product->get('Categories') );

    return $self->_getCategoryString(
        $Product->get('SuperProduct')->get('Categories') )
      if !isValid($CategoryNamesString) && $Product->get('SuperProduct');
    return $CategoryNamesString;
}

#creates the category string using breadcrumbs
sub _getCategoryString {
    my $self = shift;
    my ($aCategories) = @_;
    my $Category;
    foreach my $TempCategory ( @{$aCategories} ) {
        $Category = $TempCategory;
        last
          unless ( $TempCategory->instanceOf('StartPage')
            || $TempCategory->instanceOf('CategorySpecialOffer') );
    }

    my @CategoryNames;

    # As long as category is not Home page (has Alias named Categories)
    while ( defined($Category) && ( $Category->get('Alias') ne 'Categories' ) )
    {
        push @CategoryNames, $Category->get( 'NameOrAlias', $self->languageID );
        $Category = $Category->parent;
    }
    return join( '/', reverse(@CategoryNames) );
}

#========================================================================================
# function     Function
# state        public_private
#----------------------------------------------------------------------------------------
# syntax       Function( $Product );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# return       $Alias | Alias of product | String
#========================================================================================
sub Alias {
    my $self = shift;
    my ($Product) = @_;

    return $Product->alias;
}

#========================================================================================
# function     WebUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WebUrl = WebUrl($Product)
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# description  returns the WebUrl
#----------------------------------------------------------------------------------------
# return       $WebUrl | WebUrl | String
#========================================================================================
sub WebUrl {
    my $self = shift;
    my ($Product) = @_;

    return $self->productUrl($Product);
}

#========================================================================================
# function     ImageLarge
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImageLarge( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# description  returns the URL to the large image
#----------------------------------------------------------------------------------------
# return       $ImageURL | URL to the large image | String
#========================================================================================
sub ImageLarge {
    my $self = shift;
    my ($Product) = @_;

    return $self->imageUrl( $Product,
        LoadClassByAlias('Product')->attribute('ImageLarge') );
}

#========================================================================================
# function     ImageMedium
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImageMedium( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# description  returns the URL to the medium image
#----------------------------------------------------------------------------------------
# return       $ImageURL | URL to the medium image | String
#========================================================================================
sub ImageMedium {
    my $self = shift;
    my ($Product) = @_;

    return $self->imageUrl( $Product,
        LoadClassByAlias('Product')->attribute('ImageMedium') );
}

#========================================================================================
# function     ImageSmall
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImageSmall( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# description  returns the URL to the small image
#----------------------------------------------------------------------------------------
# return       $ImageURL | URL to the small image | String
#========================================================================================
sub ImageSmall {
    my $self = shift;
    my ($Product) = @_;

    return $self->imageUrl( $Product,
        LoadClassByAlias('Product')->attribute('ImageSmall') );
}

#========================================================================================
# function     ProductImage
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProductImage( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# description  returns the URL to the medium image.
#               Unity only has one image attribute, this was mapped to ImageMedium
#----------------------------------------------------------------------------------------
# return       $ImageURL | URL to the medium image | String
#========================================================================================
sub ProductImage {
    my $self = shift;
    my ($Product) = @_;

    return $self->ImageMedium($Product);
}

#========================================================================================
# function     UPCEAN
# state        public
#----------------------------------------------------------------------------------------
# syntax       UPCEAN( $Product );
#----------------------------------------------------------------------------------------
# description  returns the UPCEAN
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# return       $UPCEAN | returns the UPCEAN | String
#========================================================================================
sub UPCEAN {
    my $self      = shift;
    my ($Product) = @_;
    my $UPCEAN    = $Product->getWithParent('UPCEAN') // '';

# "UPCEAN" (international / european article number). Max. 13 characters of text.
    $UPCEAN = substr( $UPCEAN, 0, 13 );  ## no critic qw( ProhibitMagicNumbers )

    return $UPCEAN;
}

#========================================================================================
# function     ListPrice
# state        public
#----------------------------------------------------------------------------------------
# syntax       ListPrice( $Product );
#----------------------------------------------------------------------------------------
# description  returns the price
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref
#----------------------------------------------------------------------------------------
# return       $Price | returns the price | String
#========================================================================================
sub ListPrice {
    my $self      = shift;
    my ($Product) = @_;
    my $Price     = '';
    eval { $Price = $self->productPrice($Product); };
    if ( ExistsError() ) {
        my $Error = GetError();

        # ignore ProductNoPrice errors
        Error($Error) unless ( $Error->code eq 'ProductNoPrice' );
    }
    return $Price;
}

#========================================================================================
# function     Weight
# state        public
#----------------------------------------------------------------------------------------
# syntax       Weight( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  retruns the Weight additional Unit
#----------------------------------------------------------------------------------------
# return       $Weight | the Weight of product with Unit | String
#========================================================================================
sub Weight {
    my $self       = shift;
    my ($Product)  = @_;
    my $Weight     = $Product->getWithParent( 'Weight', $self->languageID );
    my $WeightUnit = $Product->getWithParent( 'WeightUnit', $self->languageID );
    my $Unit       = '';
    if ( defined($WeightUnit) ) {
        $Unit = $WeightUnit->get('Abbreviation');
        $Unit = $WeightUnit->alias unless defined($Unit);
    }
    return defined($Weight) ? $Weight . $Unit : '';
}

#========================================================================================
# function     Description
# state        public
#----------------------------------------------------------------------------------------
# syntax       Description( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  retruns the description and remove HTML-Tags and new lines
#----------------------------------------------------------------------------------------
# return       $DescriptionValueString | The description without HTML and new lines | String
#========================================================================================
sub Description {
    my $self = shift;
    my ($Product) = @_;

    my $DescriptionAttribute = $self->descriptionAttribute;
    if ( defined $DescriptionAttribute ) {
        my $Alias = $DescriptionAttribute->alias;
        my $Value = $Product->getWithParent( $Alias, $self->languageID );
        if ( defined $Value ) {
            return $self->removeHtmlTags($Value);
        }
    }
    return undef;
}

#========================================================================================
# function     MeasureWithUnit
# state        public
#----------------------------------------------------------------------------------------
# syntax       MeasureWithUnit( $Product,$Measure );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
# input        $measure | name of the current measure | String
#----------------------------------------------------------------------------------------
# description  retruns the measure additional Unit
#----------------------------------------------------------------------------------------
# return       $Width | the Width of product with Unit | String
#========================================================================================
sub MeasureWithUnit {
    my $self = shift;
    my ( $Product, $Measure ) = @_;
    my $MeasureWithUnit =
      $Product->getWithParent( $Measure, $self->languageID );
    my $Unit = 'mm';
    return defined($MeasureWithUnit) ? $MeasureWithUnit . $Unit : '';
}

#========================================================================================
# function     DeliveryCost
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeliveryCost( $Product );
#----------------------------------------------------------------------------------------
# input        $Product | a product | ref.hash
#----------------------------------------------------------------------------------------
# description  Retruns the shipping price if it set or a empthy string
#----------------------------------------------------------------------------------------
# return       $ShippingPrice | shipping price or empthy string | String
#========================================================================================
sub DeliveryCost {
    my $self          = shift;
    my ($Product)     = @_;
    my $ShippingPrice = '';
    eval { $ShippingPrice = $self->shippingPrice($Product); };

    if ( ExistsError() ) {
        my $Error = GetError();

        # ignore ProductNoShipping errors
        Error($Error) unless ( $Error->code eq 'ProductNoShipping' );
    }
    return $ShippingPrice;
}

1;
