#========================================================================================
# package      DE_EPAGES::WebService::API::JSONPatch
# state        public
#----------------------------------------------------------------------------------------
# description  Handles JSON patch objects.
#               Inherit from this class to create your custom patch handler.
#               (for further details see RFC 6902: https://tools.ietf.org/html/rfc6902)
#========================================================================================
package DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw (Error);

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::WebService::API::JSONPatch
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     patch
# state        public
#----------------------------------------------------------------------------------------
# syntax       patch( $hPatchOperation );
#----------------------------------------------------------------------------------------
# description  Executes a patch operation
#----------------------------------------------------------------------------------------
# input        $hPatchOperation | patch operation regarding RFC 6902 | ref_hash
# return       Patch result | refhash
#========================================================================================
sub patch {
    my $self = shift;
    my ($hPatchOperation) = @_;

    my $Operation = $hPatchOperation->{'op'};
    my $From = $hPatchOperation->{'from'};
    my $Path = $hPatchOperation->{'path'};
    my $hValue = $hPatchOperation->{'value'};

    if ($Operation eq 'test') {
        return $self->_test($Path, $hValue);
    }
    elsif ($Operation eq 'remove') {
        return $self->_remove($Path, $hValue);
    }
    elsif ($Operation eq 'add') {
        my $hParams = {};
        $self->_add($Path, $hValue, $hParams) ;
        # copy status to patch operation
        $hPatchOperation->{'status'} = $hParams->{'status'};
        return;
    }
    elsif ($Operation eq 'replace') {
        return $self->_replace($Path, $hValue);
    }
    elsif ($Operation eq 'move') {
        return $self->_move($From, $Path, $hValue);
    }
    elsif ($Operation eq 'copy') {
        return $self->_copy($From, $Path, $hValue);
    }
    else {
        Error("PATCH operation '$Operation' is not supported!");
    }
    return;
}

sub _test {
    my $self = shift;
    my ($Path, $hValue) = @_;
    Error('PATCH operation \'test\' is not implemented!');
    return;
}

sub _remove {
    my $self = shift;
    my ($Path) = @_;
    Error('PATCH operation \'remove\' is not implemented!');
    return;
}

sub _add {
    my $self = shift;
    my ($Path, $hValue) = @_;
    Error('PATCH operation \'add\' is not implemented!');
    return;
}

sub _replace {
    my $self = shift;
    my ($Path, $hValue) = @_;
    Error('PATCH operation \'replace\' is not implemented!');
    return;
}

sub _move {
    my $self = shift;
    my ($From, $Path, $hValue) = @_;
    Error('PATCH operation \'move\' is not implemented!');
    return;
}

sub _copy {
    my $self = shift;
    my ($From, $Path, $hValue) = @_;
    Error('PATCH operation \'copy\' is not implemented!');
    return;
}

1;
